/*
 * bstd.h: bstd specific defines
 *
 * Copyright (C) 2002 Intel Corporation (yu.tang@intel.com)
 *
 * (C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef BLOB_ARCH_BSTD_H
#define BLOB_ARCH_BSTD_H

/* Use STUART */
#define USE_SERIAL1	1
#define TERMINAL_SPEED	baud_115200

#define BLOB_ABS_BASE_ADDR	(0x80008000)

/* where do various parts live in RAM */
#define BLOB_RAM_BASE		(0x80200000)
#define KERNEL_RAM_BASE		(0x80800000)
#define PARAM_RAM_BASE		(0x80400000)
#define RAMDISK_RAM_BASE	(0x80600000)

/* define kernel_xip_base */
#define KERNEL_XIP_BASE		(0x10040000)

/* and where do they live in flash */
#define BLOB_FLASH_BASE		(0x80000)
#define BLOB_FLASH_LEN		(256 * 1024)
#define PARAM_FLASH_BASE	(BLOB_FLASH_BASE + BLOB_FLASH_LEN)
#define PARAM_FLASH_LEN		(0) /* no parameters */
#define KERNEL_FLASH_BASE	(0x600000)
#define KERNEL_FLASH_LEN	(2 * 1024 * 1024)
#define LOAD_RAMDISK		1 /* load ramdisk into ram */
#define RAMDISK_FLASH_BASE	(KERNEL_FLASH_BASE + KERNEL_FLASH_LEN)
#define RAMDISK_FLASH_LEN	((1 * 1024 * 1024) / 2)

#define FTM_KERNEL_FLASH_BASE	(0xe300000)
#define FTM_KERNEL_FLASH_LEN	(3 * 1024 * 1024)

#define RECOVERY_KERNEL_FLASH_BASE	(0x280000)
#define RECOVERY_KERNEL_FLASH_LEN	(2 * 1024 * 1024)

/* blob flash memory map base*/
#define BLBO_NTIM_MEM_BASE (0x00000000)
#define BLBO_OBM_MEM_BASE (0x00020000)
#define BLBO_BLOB_MEM_BASE (0x00080000)
#define BLBO_SEC_MEM_BASE (0x00100000)
#define BLBO_NVM_MEM_BASE (0x00140000)
#define BLBO_LOGO_MEM_BASE (0x00200000)
#define BLBO_PANIC_MEM_BASE (0x00240000)
#define BLBO_RKERNEL_MEM_BASE (0x00280000)
#define BLBO_RRAMDISK_MEM_BASE (0x00480000)
#define BLBO_KERNEL_MEM_BASE (0x00600000)
#define BLBO_RAMDISK_MEM_BASE (0x00800000)
#define BLBO_SYSTEM_MEM_BASE (0x00900000)
#define BLBO_USRDATA_MEM_BASE (0x06900000)
#define BLBO_LOCAL_MEM_BASE (0x09900000)
#define BLBO_CACHE_MEM_BASE (0x0A800000)
#define BLBO_OPL_MEM_BASE (0x0B000000)
#define BLBO_FTM_MEM_BASE (0x0E300000)
#define BLBO_FOTA_MEM_BASE (0x0F480000)
#define BLBO_BBT_MEM_BASE (0x0FA80000)

/* blob flash end*/

/* enable MMU */
#define MMU_ENABLE
#ifdef MMU_ENABLE
#define DDR_UNCACHABLE_SIZE     (0x000100000)
#define DDR_UNCACHABLE_BASE     (RAMDISK_RAM_BASE - DDR_UNCACHABLE_SIZE)
#endif

#undef NOISE_WORKAROUND_ENABLE
//#define NOISE_WORKAROUND_ENABLE

#undef SIM_WORKAROUND_ENABLE
//#define SIM_WORKAROUND_ENABLE

//#undef MACH_TYPE_ENABLE
#define MACH_TYPE_ENABLE

//#undef HW_VERSION_ENABLE
#define HW_VERSION_ENABLE

/* the position of the kernel boot parameters */
#define BOOT_PARAMS		(0x80000100)

/* the size (in kbytes) to which the compressed ramdisk expands */
#define RAMDISK_SIZE		(8 * 1024)

/* TFTP download RAM base */
#define TFTP_RAM_START 		(0x80800000)

/* SMC91c111 */
#define SMC_BASE	(0x30000300)
#define GPIO_BASE	(0x40E00000)
#define MFP_BASE	(0x40E10000)
#define OST_BASE	(0x40A00000)
#define CLK_BASE	(0x41340000)
#define KPC_BASE	(0x41500000)
#define RTC_BASE        (0x40900000)
#define LCDC_BASE	(0x44000000)
#define MLCD_BASE	(0x46000000)
#define TSIC_BASE  	(0x41c00000)
#define UDC_BASE	(0x40600000)
#define USBH_BASE	(0x4C000000)
#define DMAC_BASE	(0x40000000)
#define QCI_BASE        (0x50000000)
#define I2C_BASE        (0x40301680)
#define MMC1_BASE	(0x41100000)
#define MMC2_BASE	(0x42000000)
#define SPMU_BASE       (0x40F50000)
#define BPMU_BASE       (0x40F40000)
#define MEMC_BASE       (0x48100000)
#define PML_BASE        (0x4600ff00)
#define INTC_BASE       (0x4A000000)
#define	PI2C_BASE	(0x40F500C0)

#define RECOVER_KERNEL_BOOTUP_REASON (1<<22)
#endif 
