/*
 * frodo.h: 2d3D, Inc. SA-1110 Development Board
 *
 * Copyright (C) 2001  Abraham van der Merwe <abraham@2d3d.co.za>
 *
 * $Id: frodo.h,v 1.11 2002/10/25 17:24:14 erikm Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: frodo.h,v 1.11 2002/10/25 17:24:14 erikm Exp $"

#ifndef BLOB_ARCH_FRODO_H
#define BLOB_ARCH_FRODO_H

/* boot CPU speed */
#define CPU_SPEED		(CPU_CORE_SPEED_206mhz)

/*
 * Serial port 1 is used for the Serial port w/ flow control.
 */

/* serial port */
#define USE_SERIAL2
#define TERMINAL_SPEED baud_9600

/* GPIO for the LED */
#define LED_GPIO			(0x00000000) /* no led (for now) */

/* the base address were BLOB is loaded by the first stage loader */
#define BLOB_ABS_BASE_ADDR	(0xc0200400)

/* where do various parts live in RAM */
#define BLOB_RAM_BASE		(0xc0100000)
#define KERNEL_RAM_BASE		(0xc0008000)
#define PARAM_RAM_BASE		(0xc0110000)
#define RAMDISK_RAM_BASE	(0xc0400000)

/* and where do they live in flash */
#define BLOB_FLASH_BASE		0x00000000
#define BLOB_FLASH_LEN		(256 * 1024)
#define PARAM_FLASH_BASE	(BLOB_FLASH_BASE + BLOB_FLASH_LEN)
#define PARAM_FLASH_LEN		(256 * 1024)
#define KERNEL_FLASH_BASE	(PARAM_FLASH_BASE + PARAM_FLASH_LEN)
#define KERNEL_FLASH_LEN	(1024 * 1024)
#define LOAD_RAMDISK		1 /* load ramdisk into ram */
#define RAMDISK_FLASH_BASE	(KERNEL_FLASH_BASE + KERNEL_FLASH_LEN)
#define RAMDISK_FLASH_LEN	(4 * 1024 * 1024)

/* the position of the kernel boot parameters */
#define BOOT_PARAMS			(0xc0000100)

/* the size (in kbytes) to which the compressed ramdisk expands */
#define RAMDISK_SIZE		(8 * 1024)

/* Memory configuration */
#ifdef BLOB_NEED_MEMCONFIG

#define SMCNFG_VALUE	0x00000000
#define MDCNFG_VALUE	0x0000a265

#ifdef MDREFR_DONT_BELIEVE_IN_MAGIC
#define MDREFR_VALUE	0x00700061
#endif	/* #ifdef MDREFR_DONT_BELIEVE_IN_MAGIC */

#define MDCAS00_VALUE	0x5555557f
#define MDCAS01_VALUE	0x55555555
#define MDCAS02_VALUE	0x55555555
#define MSC0_VALUE		0x00004f70	/* nCS[1:0] 32-bit, flash */
#define MSC1_VALUE		0x4f710000	/* nCS[3:2] 32-bit, daughter card */
#define MSC2_VALUE		0x0000ef75	/* nCS[5:4] 16-bit, CPLDs, etc. */

/*
 * These are currently unused:
 *
 *      MECR_VALUE
 *      MDCAS20_VALUE
 *      MDCAS21_VALUE
 *      MDCAS22_VALUE
 */
#define MECR_VALUE		0x994a994a
#define MDCAS20_VALUE	0xaaaaaa7f
#define MDCAS21_VALUE	0xaaaaaaaa
#define MDCAS22_VALUE	0xaaaaaaaa

#endif	/* #ifdef BLOB_NEED_MEMCONFIG */

#endif	/* #ifndef BLOB_ARCH_FRODO_H */
