/*
 * shannon.h: SHANNON specific defines
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * $Id: shannon.h,v 1.13 2002/04/30 23:30:13 timriker Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: shannon.h,v 1.13 2002/04/30 23:30:13 timriker Exp $"

#ifndef BLOB_ARCH_SHANNON_H
#define BLOB_ARCH_SHANNON_H

/* boot CPU speed */
#define CPU_SPEED		CPU_CORE_SPEED_191mhz

/* serial port */
#define USE_SERIAL3
#define TERMINAL_SPEED baud_115200

/* GPIO for the LED */
#define LED_GPIO		(0x00040000) /* GPIO 18, codec (ucb1300) reset */

/* to connect this up, you'll need an LED and a resistor (100ohm - 1k will probably do)
 *
 *             o o o  =\ =o o o o            o o    ___________ \
 *           33| |  \   \ |//|               | |  1////-33 \ \ \ |/--\o
 *           --+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-- |||| --+-+-+-+-+-+-+
 *           | : : : : : : : : : : : : : : : : : | |||| | : : : : : : :
 *           --+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-- |||| --+-+-+-+-+-+-+
 * <-- front 34o o o o o|o o   J601  o|||||||| o  2|||| 34o J600o|o|o|o       back -->
 *             |       /   /\       /\o|o|o|o|     ||||   |      | | |
 *                         ||       ||
 *                         ||       ||
 *                       GPIO18     GND
 *
 * This is supposed to be ascii art of the daughter board rotated 90 degrees clockwise.
 * An LED and resistor should be connected between the two vias indicated by GPIO18 and
 * GND. These vias are on either side of the J601 silk label. The circuit should be connected
 * as shown below (other interpretations sholud work as well).
 *
 *
 *                        |\ |
 *    GPIO18 o---/\/\/\---| >|---------o GND 
 *                        |/ |
 *
 *                 R1      D1
 *
 *
 * A little excessive description maybe, but I don't want anyone blowing up their tux :)
 *
 */
/* Shannon memory layout is a bit precious because we want to be able
 * to load blob at the correct location for both 8 and 32MB
 * SODIMMs. We assume that the following amount of memory is at least
 * available in both situations:
 *
 * 1MB @ 0xc0000000
 * 1MB @ 0xc8000000
 * 4MB @ 0xd0000000
 * 4MB @ 0xd8000000
 *
 */

/* the base address were blob-rest is loaded by the first stage loader */
#define BLOB_ABS_BASE_ADDR	(0xc8000400)

/* where do various parts live in RAM */
#define BLOB_RAM_BASE		(0xc0120000)
#define KERNEL_RAM_BASE		(0xc0008000)
#define PARAM_RAM_BASE		(0xc8040000)
#define RAMDISK_RAM_BASE	(0xd0000000)

/* and where do they live in flash */
#define BLOB_FLASH_BASE		(0x00000000)
#define BLOB_FLASH_LEN		(64 * 1024)
#define PARAM_FLASH_BASE	(BLOB_FLASH_BASE + BLOB_FLASH_LEN)
#define PARAM_FLASH_LEN		(64 * 1024)
#define KERNEL_FLASH_BASE	(PARAM_FLASH_BASE + PARAM_FLASH_LEN)
#define KERNEL_FLASH_LEN	((1024 - 128) * 1024) /* RAM_LEN if load_kernel */
#define LOAD_RAMDISK		0 /* leave ramdisk in flash */

#if defined(CONFIG_CRAMFS_SUPPORT) || defined(CONFIG_JFFS2_SUPPORT)
#define RAMDISK_FLASH_BASE	KERNEL_FLASH_BASE
#define RAMDISK_FLASH_LEN	(4 * 1024 * 1024 - BLOB_FLASH_LEN - PARAM_FLASH_LEN)
#else
#define RAMDISK_FLASH_BASE	(KERNEL_FLASH_BASE + KERNEL_FLASH_LEN)
#define RAMDISK_FLASH_LEN	(4 * 1024 * 1024 - BLOB_FLASH_LEN - PARAM_FLASH_LEN - KERNEL_FLASH_LEN)
#endif

#define PARAM_START PARAM_FLASH_BASE
#define PARAM_LEN PARAM_FLASH_LEN

/* the position of the kernel boot parameters */
#define BOOT_PARAMS		(0xc0000100)

/* the size (in kbytes) to which the compressed ramdisk expands */
#define RAMDISK_SIZE		(8 * 1024)

#endif
