/*
 * led.h: header file for memory.c
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: led.h,v 1.6 2003/09/04 18:03:09 russd Exp $"

#ifndef BLOB_LED_H
#define BLOB_LED_H




void led_init(void);
int led_on(void);
int led_off(void);
void led_toggle(void);
void led_lock(void);
void led_unlock(void);

typedef void (*led_on_func_t)(void);
typedef void (*led_off_func_t)(void);
typedef void (*led_toggle_func_t)(void);
typedef void (*led_lock_func_t)(void);
typedef void (*led_unlock_func_t)(void);

typedef struct {
    led_on_func_t led_on;
    led_off_func_t led_off;
    led_toggle_func_t led_toggle;
    led_lock_func_t led_lock;
    led_unlock_func_t led_unlock;
} led_driver_t;

/* implemented LED drivers */
extern led_driver_t s3c2500_gpio_led_driver;
extern led_driver_t sa11x0_gpio_led_driver;
extern led_driver_t mmap_led_driver;

/* FIXME: This still needs to be implemented */
/* extern led_driver_t pxa_gpio_led_driver; */

/* defined in arch specific file */
extern led_driver_t *led_driver;

#endif
