/*
 * spd.h: defines for PC SDRAM serial presence detect
 *
 * Copyright (C) 2001  Hewlett-Packard Company
 * Written by Christopher Hoover <ch@hpl.hp.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * PC SDRAM Serial Presence Detect (SPD) Specification, Revision 1.2B
 *
 * http://developer.intel.com/technology/memory/pc133sdram/spec/Spdsd12b.pdf
 */

#ifndef _SPD_H_
#define _SPD_H_

#define SPD_I2C_ADDR(a) (0x50 | (a))

#define SPD_SPD_MANUF_BYTES		0
#define SPD_SPD_TOTAL_BYTES		1
#define SPD_MEMORY_TYPE			2
#  define SPD_MEMORY_TYPE_EDO 2
#  define SPD_MEMORY_TYPE_SDRAM 4
#define	SPD_ROW_BITS			3
#define SPD_COL_BITS			4
#define SPD_MODULE_ROWS			5
#define SPD_DATA_WIDTH			6
#define SPD_DATA_WIDTH_L		6
#define SPD_DATA_WIDTH_H		7
#define SPD_VOLTAGE_INTERFACE		8
#define SPD_CYCLE_TIME			9
#define SPD_ACCESS_TIME			10
#define SPD_MODULE_TYPE			11
#define SPD_REFRESH_RATE_TYPE		12
#define SPD_PRIMARY_SDRAM_WIDTH		13
#define SPD_MIN_CLOCK_B2B_RAND_COL	15
#define SPD_BURST_LENGTHS		16
#define SPD_NUMBER_OF_BANKS_SDRAM	17
#define SPD_CAS_LATENCY			18
#define SPD_CS_LATENCY			19
#define SPD_WE_LATENCY			20
#define SPD_MODULE_ATTRIBS		21
#define SPD_DEVICE_ATTRIBS		22
#define SPD_CYCLE_TIME_CL_X_1		23
#define SPD_ACCESS_TIME_CL_X_1		24
#define SPD_CYCLE_TIME_CL_X_2		25
#define SPD_ACCESS_TIME_CL_X_2		26
#define SPD_TRP				27
#define SPD_TRRD			28
#define SPD_TRCD			29
#define SPD_TRAS			30
#define SPD_ROW_DENSITY			31
#define SPD_CMD_ADDR_SETUP_TIME		32
#define SPD_CMD_ADDR_HOLD_TIME		33
#define SPD_DATA_SETUP_TIME		34
#define SPD_DATA_HOLD_TIME		35
#define SPD_DATA_REVISION		62
#define SPD_CHECKSUM			63
#define SPD_MANUF_JDEC_ID		64
#define SPD_MANUF_LOCATION		72
#define SPD_MANUF_PART_NUMBER		73
#define SPD_REVISION_CODE		91
#define SPD_MANUF_DATE			93
#define SPD_SERIAL_NUMBER		95
#define SPD_MANUF_SPECIFIC_DATA		99
#define SPD_INTEL_SPEC_FREQ		126
#define SPD_INTEL_SPEC_CAS_LATENCY	127

#endif

