/*
 * bez.c: Benzar specific stuff
 *
 * Modified from src/blob/lubbock.c
 *
 *(C) Copyright 2007 Marvell International Ltd.  
 * All Rights Reserved 
 * Copyright (C) 2004  Intel Corporation (stanley.cai@intel.com)
 * Copyright (C) 2003  Intel Corporation (yu.tang@intel.com)
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>
#include <blob/flash.h>
#include <blob/init.h>
#include <blob/reboot.h>
#include <blob/serial.h>
#include <blob/time.h>
#include <blob/command.h>
#include <blob/util.h>
#include <net/ether.h>
#include <net/net.h>
#include <blob/strata.h>
#include <blob/command.h>

#ifndef CONFIG_USBDNET_SUPPORT
#include <net/ether.h>
#endif

#define	FLASH_BASE 	0x10000000
int BLOB_BOOT_DELAY_TIME = 0;
static struct flash_chip flash_chips[2];

#if defined(BEACON)

enum mach_version {
    MACH_VERSION_W00 = 0x0000A000,      /* wingboard */
    MACH_VERSION_P00 = 0x0000E000,      /* P0 */
    MACH_VERSION_P01 = 0x0000E001,      /* P0.1 */
    MACH_VERSION_P02 = 0x0000E002,      /* P0.2 */
    MACH_VERSION_P10 = 0x0000E010,      /* P1 */
    MACH_VERSION_P11 = 0x0000E011,      /* P1.1 */
    MACH_VERSION_P15 = 0x0000E050,      /* P15 */
    MACH_VERSION_P20 = 0x0000E020,      /* P2 */
    MACH_VERSION_P21 = 0x0000E021,      /* P2.1 */
    MACH_VERSION_P30 = 0x0000E030,      /* P3 */
    MACH_VERSION_P40 = 0x0000E040,      /* P4 */
    MACH_VERSION_M10 = 0x0000F010,      /* MA 1.0 */
};

extern unsigned int get_hw_version_id(void);

char mtdparts[] ="mtdparts=pxa3xx-onenand:0x100000@0x0(bootldr)ro,\
0x40000@0x100000(security),\
0x800000@0x140000(cpImage)ro,\
0xc0000@0x940000(NVM),0x40000@0xa00000(logo)ro,\
0x300000@0xa40000(boot)ro,0xc800000@0xd40000(system),\
0x1000000@0xd540000(local),\
0x600000@0xe540000(fota),0x8c0000@0xeb40000(cache),\
0x600000@0xf400000(ftm),0x80000@0xfa00000(panic),\
0x80000@0xfa80000(misc),0x3000000@0xfe00000(opl),\
0xc780000@0x12e00000(userdata),0x80000@0x1f580000(BBT)ro;";

char mtdparts_recovery[] ="mtdparts=pxa3xx-onenand:0x100000@0x0(bootldr)ro,\
0x40000@0x100000(security),\
0x800000@0x140000(cpImage)ro,\
0xc0000@0x940000(NVM),0x40000@0xa00000(logo)ro,\
0x300000@0xa40000(boot)ro,0xc800000@0xd40000(system),\
0x1000000@0xd540000(local),\
0x600000@0xe540000(fota),0x8c0000@0xeb40000(cache),\
0x600000@0xf400000(ftm),0x80000@0xfa00000(panic),\
0x80000@0xfa80000(misc),0x300000@0xfb00000(recovery)ro,\
0x3000000@0xfe00000(opl),\
0xC780000@0x12e00000(userdata),0x80000@0x1f580000(BBT)ro;";

char mtdparts_kernel_flash[] ="mtdparts=pxa3xx-onenand:0x20000@0x0(NTIM),\
0x1f560000@0x20000(BIG_PARTITION),0x80000@0x1f580000(BBT)ro;";

#else

char mtdparts[] ="mtdparts=pxa3xx-onenand:0x100000@0x0(bootldr)ro,\
0x40000@0x100000(security),\
0x800000@0x140000(cpImage)ro,\
0xc0000@0x940000(NVM),0x40000@0xa00000(logo)ro,\
0x300000@0xa40000(boot)ro,0x8000000@0xd40000(system),\
0x3000000@0x8d40000(userdata),0x2000000@0xbd40000(opl),\
0x800000@0xdd40000(local),0x600000@0xe540000(fota),\
0x8c0000@0xeb40000(cache),\
0x600000@0xf400000(ftm),0x80000@0xfa00000(panic),\
0x80000@0xfa80000(BBT)ro;";
#endif

#ifdef MMU_ENABLE
struct mem_area
{
	unsigned long phy_start;
	unsigned long vir_start;
	unsigned long size;
	unsigned char attr;
};

static struct mem_area io_map[]={
	{0x00000000, 0x00000000,    0x10000000,	0x2}, /* ROM, 256M */
	{0x10000000, 0x10000000,    0x04000000,	0x2}, /* ROM, 64M */
	{0x14000000, 0x14000000,    0x04000000,	0x2}, /* LAN91c111, 64M*/
	{0x18000000, 0x18000000,    0x04000000,	0x2}, /* SMEMC reserved ,64M */
	{0x1c000000, 0x1c000000,    0x04000000,	0x2}, /* SMEMC reserved,64M */
	{0x20000000, 0x20000000,    0x04000000,	0x2}, /* PCMCIA - CF IO space,64M */
	{0x24000000, 0x24000000,    0x04000000,	0x2}, /* SMEMC reserved,64M */
	{0x28000000, 0x28000000,    0x04000000,	0x2}, /* PCMCIA - CF Attr space,64M */
	{0x2c000000, 0x2c000000,    0x04000000,	0x2}, /* PCMCIA - CF common space,64M */
	{0x30000000, 0x30000000,    0x10000000,	0x2}, /* Static chip select 1, 256M */
	{0x40000000, 0x40000000,    0x04000000, 0x2}, /* Periperial device  */
	{0x43000000, 0x43000000,    0x00100000, 0x2}, /* CADDO */
	{0x44000000, 0x44000000,    0x00100000, 0x2}, /* LCD */
	{0x46000000, 0x46000000,    0x00100000, 0x2}, /* Mini LCD & */
	{0x48000000, 0x48000000,    0x04000000, 0x2}, /* Memory Ctl*/
	{0x48100000, 0x48100000,    0x00100000, 0x2}, /* Dynamic Mem Ctl */
	{0x4a000000, 0x4a000000,    0x00100000, 0x2}, /* Static Mem Ctl */
	{0x4c000000, 0x4c000000,    0x00100000, 0x2}, /* USB host */
	{0x50000000, 0x50000000,    0x00100000, 0x2}, /* Camera */
	{0x54000000, 0x54000000,    0x00200000, 0x2}, /* 2D Graphics & USB2.0 */
	{0x55500000, 0x55500000,    0x00100000, 0x2}, /* USB2.0 PHY */
	{0x58000000, 0x58000000,    0x00100000, 0x2}, /* Internal SRAM ctl */	
	{0x5C000000, 0x5C000000,    0x00100000, 0xe}, /* SRAM */
	{0x80000000, 0x80000000,    KERNEL_RAM_BASE - 0x80000000 - DDR_UNCACHABLE_SIZE,
	   		0xe}, /* SDRAM part 0: cachable */
	{DDR_UNCACHABLE_BASE, DDR_UNCACHABLE_BASE, 
		DDR_UNCACHABLE_SIZE, 0x2}, /* SDRAM part 1: uncachable*/
#if defined(BEACON)
	{0x80800000, 0x80800000,     0x0f800000, 0xe}, /* SDRAM part 2: cachable */
	{0xc0000000, 0xc0000000,     0x10000000, 0xe}, /* SDRAM part 2: cachable */
#else
	{0x80800000, 0x80800000,     0x07800000, 0xe}, /* SDRAM part 2: cachable */
	{0x88000000, 0x88000000,     0x08000000, 0xe}, /* 2nd SDRAM bank: cachable */
	{0xc0000000, 0xc0000000,     0x08000000, 0xe}, /* SDRAM part 2: cachable */
	{0xc8000000, 0xc8000000,     0x08000000, 0xe}, /* 2nd SDRAM bank: cachable */
#endif
};

// Large Page, 1M per entry 
#define MAX_PE	4096

static unsigned long mmu_table[MAX_PE] __attribute__ ( (aligned(0x4000)));

static void enable_mmu()
{
	unsigned long i, phy_addr, vir_addr, phy_end;

	/* init MMU table */
	for (i=0; i < MAX_PE; i++) {
		mmu_table[i] = 0;
	}

	for (i=0; i < sizeof(io_map)/sizeof(struct mem_area); i++) {
		phy_addr = io_map[i].phy_start;
		phy_end = phy_addr + io_map[i].size;
		vir_addr = io_map[i].vir_start;

		while( phy_addr < phy_end ) {
			mmu_table[vir_addr>>20] = phy_addr | io_map[i].attr;
			phy_addr += 0x100000;
			vir_addr += 0x100000;
		}
	}

        asm(" \
	MRC p15, 0, R2, c1, c0 \n\
	ORR R2, R2, #0x1000 \n\
	ORR R2, R2, #0x800 \n\
	MCR p15, 0, R2, c1, c0, 0 \n\
 \
	mrc p15, 0, R2, c2, c0, 0 \n\
	mov R2, R2  \n\
	sub PC, PC, #4 \n\
 \
	ldr r0, =mmu_table \n\
	mcr p15, 0, r0, c2, c0 \n\
	mrc p15, 0, R0, c2, c0, 0 \n\
	mov R0, R0 \n\
	sub PC, PC, #4 \n\
 \
	mvn r0, #0 \n\
	mcr p15, 0, r0, c3, c0, 0 \n\
	mrc p15, 0, R0, c2, c0, 0 \n\
	mov R0, R0 \n\
	sub PC, PC, #4 \n\
 \
	MCR p15, 0, r0, c7, c10, 4 \n\
	MRC p15, 0, r0, c1, c0, 0 \n\
	ORR r0, r0, #4 \n\
	MCR p15, 0, r0, c1, c0, 0 \n\
 \
	mrc p15, 0, R0, c2, c0, 0 \n\
	mov R0, R0 \n\
	sub PC, PC, #4 \n\
 \
	MRC p15,0,R0,c1,c0,0 \n\
	ORR R0, R0, #0x1 \n\
	MCR p15,0,R0,c1,c0,0 \n\
 \
	mrc p15, 0, R0, c2, c0, 0 \n\
	mov R0, R0 \n\
	sub PC, PC, #4 \n\
        "
        );
}
#endif
#if defined(BEACON)
int is_beacon_15(void)
{
    int retval = 0;

    switch (get_hw_version_id()){
        case MACH_VERSION_W00:
        case MACH_VERSION_P15:
        case MACH_VERSION_P20:
        case MACH_VERSION_P21:
        case MACH_VERSION_P30:
        case MACH_VERSION_P40:
        case MACH_VERSION_M10:
			retval = 1;
            break;
        default:
			retval = 0;
            break;
    }
    return retval;
}
#endif
/**********************************************************************
 * set gpio for nand flash
 */

static int DfcSetGPIO()
{
    /* INIT GPIO42 -- LOCK KEY */
	*(unsigned long *)0x40E10624=0x00001840;

#ifdef	BLOB_MFP_INIT

	*(unsigned long *)0x40E10290=0x00001841;
	*(unsigned long *)0x40E1028C=0x00001841;
	*(unsigned long *)0x40E10288=0x00001841;
	*(unsigned long *)0x40E10284=0x00001841;
	*(unsigned long *)0x40E10280=0x00001841;
	*(unsigned long *)0x40E1027C=0x00001841;
	*(unsigned long *)0x40E10278=0x00001841;
	*(unsigned long *)0x40E10274=0x00001841;
	*(unsigned long *)0x40E10270=0x00001801;
	*(unsigned long *)0x40E1026C=0x00001801;
	*(unsigned long *)0x40E10268=0x00001801;
	*(unsigned long *)0x40E10264=0x00001801;
	*(unsigned long *)0x40E10248=0x00001801;
	*(unsigned long *)0x40E10244=0x00001801;
	*(unsigned long *)0x40E10240=0x00001801;
	*(unsigned long *)0x40E1023C=0x00001801;

	/* nCS0 */
	*(unsigned long *)0x40E1022C=0x00001901;
	/* nCS1 */
	*(unsigned long *)0x40E10230=0x00001903;
	/* nWE */
	*(unsigned long *)0x40E10234=0x00001901;
	/* nRE OE */
	*(unsigned long *)0x40E10238=0x00001901;
	/* nCLE */
	*(unsigned long *)0x40E1020C=0x00001901;
	/* nADV1_ALE */
	*(unsigned long *)0x40E10218=0x00001901;

	/* nADV2_ALE */
	/* Onenand is on nCS2. nADV2 should be configured as nCS2 */
	*(unsigned long *)0x40E10224=0x00001903;
	/* INT_RnB */
	*(unsigned long *)0x40E10228=0x00001901;
	/* DF_SCLK_E */
	*(unsigned long *)0x40E10214=0x00001801;

	/* nLUA */
	*(unsigned long *)0x40E10254=0x00001901;
	/* nLLA */
	*(unsigned long *)0x40E10258=0x00001901;
	/* nBE0 */
	*(unsigned long *)0x40E1021C=0x00001801;
	/* nBE1 */
	*(unsigned long *)0x40E10220=0x00001801;

	/* RDY */
	*(unsigned long *)0x40e10200=0x00001801;
	/* ADDR */
	*(unsigned long *)0x40e1024c=0x00001803;
	*(unsigned long *)0x40e10250=0x00001800;
	*(unsigned long *)0x40e1025c=0x00001800;
	*(unsigned long *)0x40e10260=0x00001803;

	/* nXCVREN */
	*(unsigned long *)0x40e10204=0x00001800;
#endif
}

int UartSetGPIO()
{
#if defined(USE_SERIAL3)
	*(unsigned int *)GPIO77_MFPR=0xa0c5;
	*(unsigned int *)GPIO78_MFPR=0xa045;
	CKENA |= CKENA_23_STUART;
#elif defined(USE_SERIAL1)
	*(unsigned int*)(0x40E10000 + 0x02c0) = 0x801;
	*(unsigned int*)(0x40E10000 + 0x02c4) = 0x801;
	*(unsigned int*)(0x40E10000 + 0x02cc) = 0x801;
	*(unsigned int*)(0x40E10000 + 0x02d0) = 0x801;
	CKENA |= CKENA_22_FFUART;
#endif

#if defined(BEACON) && defined(BLOB_DEBUG)
#ifdef HW_VERSION_ENABLE
#if defined(T8200)
	*(unsigned long *)0x40E10428 = 0x0000000; //gpio109

	*(unsigned long *)0x40E0010C |= 0x00002000; //gpdr3 
	*(unsigned long *)0x40E0040C |= 0x00002000; //gsdr3 
	*(unsigned long *)0x40E00118 |= 0x00002000; //gpsr3 
#else
	switch (get_hw_version_id()){
		case MACH_VERSION_W00:
		case MACH_VERSION_P21:
		case MACH_VERSION_P30:
		case MACH_VERSION_P40:
			break;
		case MACH_VERSION_P15:
            *(unsigned long *)0x40E10454 = 0x00000000; //gpio112
            *(unsigned long *)0x40E00118 |= 0x00010000; //gpsr3
            *(unsigned long *)0x40E0010c |= 0x00010000; //gpdr3
            *(unsigned long *)0x40E0040c |= 0x00010000; //gsdr3

			*(unsigned long *)0x40E10450 = 0x00000000; //gpio121
			*(unsigned long *)0x40E0010C &= ~(0x2000000); //gpdr3
			*(unsigned long *)0x40E0042C &= ~(0x2000000); //gcdr3

			break;
        case MACH_VERSION_P20:
        case MACH_VERSION_M10:
            *(unsigned long *)0x40E10454 = 0x00000000; //gpio112
            *(unsigned long *)0x40E00118 |= 0x00010000; //gpsr3
            *(unsigned long *)0x40E0010c |= 0x00010000; //gpdr3
            *(unsigned long *)0x40E0040c |= 0x00010000; //gsdr3

			*(unsigned long *)0x40E102BC = 0x00000000; //gpio47
			*(unsigned long *)0x40E00010 &= ~(0x8000); //gpdr1
			*(unsigned long *)0x40E00424 &= ~(0x8000); //gcdr1

			break;
		default:
			*(unsigned long *)0x40E00118=0x00002000;//set gpio 109
			*(unsigned long *)0x40E0010c=0x00002000;
			break;
	}
#endif

#endif
#else

#ifdef HW_VERSION_ENABLE
	switch (get_hw_version_id()){
		case MACH_VERSION_W00:
		case MACH_VERSION_P21:
		case MACH_VERSION_P30:
		case MACH_VERSION_P40:
			break;
		case MACH_VERSION_P15:
            *(unsigned long *)0x40E00118 &= ~(0x00010000); //gpsr3
            *(unsigned long *)0x40E0010c &= ~(0x00010000); //gpdr3
            *(unsigned long *)0x40E0040c &= ~(0x00010000); //gsdr3
			break;
        case MACH_VERSION_P20:
        case MACH_VERSION_M10:
            *(unsigned long *)0x40E00118 &= ~(0x00010000); //gpsr3
            *(unsigned long *)0x40E0010c &= ~(0x00010000); //gpdr3
            *(unsigned long *)0x40E0040c &= ~(0x00010000); //gsdr3
			break;
		default:
			*(unsigned long *)0x40E00118 &= ~(0x00002000);//set gpio 109
			*(unsigned long *)0x40E0010c &= ~(0x00002000);
			break;
	}

#endif

#endif

}

static int get_cpuid(void)
{
	 unsigned int    cpuid;
         /* read CPU ID */
         __asm__ (
                 "mrc p15, 0, %0, c0, c0, 0\n"
                 : "=r" (cpuid)
         );

	 return cpuid;
}

void bez_init_hardware(void)
{
#ifdef CONFIG_NETWORK_SUPPORT
	unsigned char serverip[4]={192,168,1,100};
	unsigned char clientip[4]={192,168,1,101};

	/* set default ip */
	memcpy(serveripaddress, serverip, sizeof(serverip));
	memcpy(clientipaddress, clientip, sizeof(clientip));
#endif

	*(unsigned int*)(0x40E10000 + 0x022c) = 0x801;
	*(unsigned int*)(0x40E10000 + 0x0230) = 0x803;
	*(unsigned int*)(0x40E10000 + 0x0418) = 0x8c0;
	
	//CSADRCFG3 = 0x003e080b;
	//MSC1 = 0x7ffc0000 | (MSC1 & 0xffff);

	MSC0 = 0x72297229;
	MSC1 = 0x003c0000 | (MSC1 & 0xffff);
	MECR = 0x00000002;
	SXCNFG = 0x00040004;	
	CSADRCFG0 = 0x00320919;
	CSADRCFG1 = 0x00320919;	
	CSADRCFG2 = 0x00320919;
	CSADRCFG3 = 0x0012c80b;
	OSCC = 0xE502E9E5;

#ifndef CONFIG_NAND_SUPPORT
	/* Set Static memory controller register for Sibley */
	//*(u32*)(MFPR3_ADDR) = 0x1;
	CSADRCFG2 = 0x00320919;
	MSC1 = (MSC1 & 0xFFFF0000) | 0x00007ff8;
	DfcSetGPIO();
#else
	/* we must set nand flash gpio here, 
	 * for ethernet/NAND/Sibley will use these pins.
	 */
	DfcSetGPIO();
#endif

#if 0 
#if 0
	printf("AO: Assign both SRAM banks to COMM.\n");
	*(unsigned int*)(0x40e101f4) |= 0xe00;
#endif
    /* pxa930_acipc.c requires fixes for PXA935 to prevent ACIPC interrupt enabling; otherwise due to HW issue
	 * the interrupts cannot be cleared, and once COMM sends the first one Linux "hangs" */
	printf("Release Comm side.\n");
	*(unsigned int*)(0x40F5001C) = 0x11;
#endif

	/* select serial driver */
	serial_driver = &mhn_serial_driver;
	reboot_driver = &mhn_reboot_driver;
        timer_driver = &intelarm_timer_driver;

#ifdef CONFIG_NETWORK_SUPPORT
#ifdef CONFIG_USBDNET_SUPPORT
	ether_driver = &bvdusb_ether_driver;
#elif CONFIG_USBDNET2_SUPPORT
#ifdef CONFIG_CPU_TAVOR_PV
 	ether_driver = &mvu2o_ether_driver;
#else
	ether_driver = &mhnu2d_ether_driver;
#endif
#else
	ether_driver = &smc91x_ether_driver;
#endif
#endif


	/* initialize flash driver */
	//init_flash_driver();

	/* turn on MMU, for better performace */
#ifdef MMU_ENABLE
	enable_mmu();
#endif
	msleep(100);
}


__initlist(bez_init_hardware, INIT_LEVEL_DRIVER_SELECTION);

void init_product(void)
{
	return;
}

#ifdef MMU_ENABLE
static void disable_mmu()
{
	asm(
	" @ memory to be sure we hit the same cache.\n\t"
	"bic     r2, pc, #0x1f\n\t"
	"add     r3, r2, #0x10000        @ 64 kb is quite enough...\n\t"
	"1:\n\t"
	"ldr     r0, [r2], #32\n\t"
	"teq     r2, r3\n\t"
	"bne     1b\n\t"
	"mcr     p15, 0, r0, c7, c10, 4  @ drain WB\n\t"
	"mcr     p15, 0, r0, c7, c7, 0   @ flush I & D caches\n\t"
	"@ disabling MMU and caches\n\t"
	"mrc     p15, 0, r0, c1, c0, 0   @ read control reg\n\t"
	"bic     r0, r0, #0x05           @ clear DC, MMU\n\t"
	"bic     r0, r0, #0x1000         @ clear Icache\n\t"
	"mcr     p15, 0, r0, c1, c0, 0\n\t");
}
#endif

void bez_deinit_hardware(void)
{
#ifdef MMU_ENABLE
	/* we disable MMU to make sure the kernel binary can be flushed into memory */
	disable_mmu();
#endif
}

__exitlist(bez_deinit_hardware, INIT_LEVEL_DRIVER_SELECTION);


#ifndef CONFIG_NAND_SUPPORT
static void init_bez_nor_flash_driver(void)
{
        int i = 0;
        flash_descriptors = NULL;
        flash_driver = NULL;

        struct flash_chip *chip = &flash_chips;

        chip->base = FLASH_BASE;
        chip->buswidth = 2;
        flash_probe(chip);
}

__initlist(init_bez_nor_flash_driver, INIT_LEVEL_DRIVER_SELECTION);

/*********************************************************************
 * cmd_flash_write
 *
 * AUTOR:	SELETZ
 * REVISED:
 *
 * Command wrapper for low-level flash write access
 *
 */
static int cmd_flash_write( int argc, char *argv[] )
{
	int ret = 0;
	u32	src	= 0L;
	u32	dest	= 0L;
	u32	len	= 0L;
	struct strata_flash_descriptor *chip=NULL;

//	printf("%s(): enter into cmd_flash_write\n", __func__);
	
	if ( argc < 4 ) {
		ret = -1;
		goto DONE;
	}

	ret = strtou32( argv[1], &src );
	if ( ret < 0 ) {
		ret = -1;
		goto DONE;
	}

	ret = strtou32( argv[2], &dest );
	if ( ret < 0 )	{
		ret = -1;
		goto DONE;
	}

	ret = strtou32( argv[3], &len );
	if ( ret < 0 ) {
		ret = -1;
		goto DONE;
	}

	if (dest < 0x10000000) {
		printf("You cannot burn flash below 0x10000000.\n");
		return -1;
	}

	chip = &flash_chips[0];

//	printf("%s(): chip: 0x%x, dest: 0x%x, src: 0x%x, len: 0x%x\n", __func__, chip, dest, (u8*)src, len);
	ret = flash_write( chip, dest, (u8*) src, len); 
//	printf("%s(): flash_write return %d\n", __func__, ret);

DONE:
	return ret;
}


static char flashwritehelp[] = "fwrite srcadr destadr size(bytes)\n"
"flash a memory region\n";
__commandlist( cmd_flash_write, "fwrite", flashwritehelp );

/*********************************************************************
 * cmd_flash_erase
 *
 * AUTOR:	SELETZ
 * REVISED:
 *
 * Command wrapper for low-level flash erasing
 *
 */
static int cmd_flash_erase( int argc, char *argv[] )
{
	int ret = 0;
	u32	dest	= 0L;
	u32	len	= 0L;
	struct strata_flash_descriptor *chip=NULL;

	if ( argc < 3 ) {
		ret = -1;
		goto DONE;
	}

	ret = strtou32( argv[1], &dest );
	if ( ret < 0 ) {
		ret = -1;
		goto DONE;
	}	

	ret = strtou32( argv[2], &len );
	if ( ret < 0 ){
		ret = -1;
		goto DONE;
	}

	chip = &flash_chips[0];
	ret = flash_erase(chip, dest, len);

DONE:
	return ret;
}


static char flasherasehelp[] = "ferase adr size(bytes)\n"
"erase a flash region\n";
__commandlist( cmd_flash_erase, "ferase", flasherasehelp );

#endif

#if 0
/*********************************************************************
 * cmd_clock_info
 *
 * AUTOR:	yu tang, rusty geldmacher
 * REVISED:
 *
 * prints out clock information. taken from Yu's kernel patch
 *
 */
#define BASE_CLK 13000000
static int cmd_clock_info(int argc, char *argv[])
{
	unsigned long acsr_val, clkcfg_val, mcc_val, memclkcfg_val;
	unsigned int XL, M, XN, HSS, DMSFS, SFLFS, SMCFS, DF_CLKDIV, EMPI_CLKDIV;
	unsigned int r_clk, t_clk, s_clk, ddr_clk, sram_clk, smem_clk, df_clk, empi_clk;
  
	acsr_val = ACSR;
	memclkcfg_val = MEMCLKCFG;
	asm("mrc p14, 0, %0, c6, c0, 0" : "=r" (clkcfg_val) );
  
	XL  =  acsr_val & 0x1f;
	XN  = (acsr_val >> 8) & 0x7;
	HSS = (acsr_val >> 14) & 0x3;
	DMSFS = (acsr_val >> 12) & 0x3;
	SFLFS = (acsr_val >> 18) & 0x3;
	SMCFS = (acsr_val >> 23) & 0x7;
	
	EMPI_CLKDIV = memclkcfg_val & 0x7;
	DF_CLKDIV = (memclkcfg_val >> 16) & 0x7;

	r_clk = XL * BASE_CLK;
	t_clk = XL * BASE_CLK * XN;

	switch (HSS) {
	case 0:
		s_clk = 104000000;
		break;
	case 1:
		s_clk = 156000000;
		break;
	case 2:
		s_clk = 208000000;
		break;
	case 3:
		s_clk = 312000000;
	}

	switch (DMSFS) {
	case 0:
	  ddr_clk = 13000000;
	  break;
	case 1:
	  ddr_clk = 78000000;
	  break;
	case 2:
	  ddr_clk = 104000000;
	  break;
	case 3:
	  ddr_clk = 130000000;
	}

	switch (SFLFS) {
	case 0:
	  sram_clk = 104000000;
	  break;
	case 1:
	  sram_clk = 156000000;
	  break;
	case 2:
	  sram_clk = 208000000;
	  break;
	case 3:
	  sram_clk = 312000000;
	}

	switch (SMCFS) {
	case 0:
	  smem_clk = 78000000;
	  break;
	case 1:
	  smem_clk = 89100000;
	  break;
	case 2:
	  smem_clk = 104000000;
	  break;
	case 3:
	  smem_clk = 124800000;
	  break;
	case 4:
	  smem_clk = 156000000;
	  break;
	case 5:
	  smem_clk = 208000000;
	  break;
	case 6:
	  smem_clk = 78000000;
	  break;
	case 7:
	  smem_clk = 78000000;
	  break;

	}
   
	switch (EMPI_CLKDIV) {
	case 1:
	  empi_clk = smem_clk;
	  break;
	case 2:
	  empi_clk = smem_clk / 2;
	  break;
	case 3:
	  empi_clk = smem_clk / 4;
	  break;
	default:
	  empi_clk = 0;
	}

	switch (DF_CLKDIV) {
	case 2:
	  df_clk = smem_clk / 2;
	  break;
	case 3:
	  df_clk = smem_clk / 4;
	  break;
	default:
	  df_clk = 0;
	}

	printf("Run Mode clock: %d.%02dMHz (*%d)\n",
	 	r_clk / 1000000, (r_clk % 1000000) / 10000, XL );
	printf("Turbo Mode clock: %d.%02dMHz (*%d, %sactive)\n",
		t_clk / 1000000, (t_clk % 1000000) / 10000, XN,
		(clkcfg_val & 0x1) ? "" : "in" );
	printf("DDR controller clock: %d.%02dMHz\n",
		ddr_clk / 1000000, (ddr_clk % 1000000) / 10000);
	printf("Static memory controller clock: %d.%02dMHz\n",
		smem_clk / 1000000, (smem_clk % 1000000) / 10000);
	printf("Data Flash controller clock (DF_SCLK): %d.%02dMHz\n",
		df_clk / 1000000, (df_clk % 1000000) / 10000);
	printf("EMPI clock (EMPI_SCLK): %d.%02dMHz\n",
		empi_clk / 1000000, (empi_clk % 1000000) / 10000);
	printf("Internal SRAM clock: %d.%02dMHz\n",
		sram_clk / 1000000, (sram_clk % 1000000) / 10000);
	printf("High Speed I/O Bus clock: %d.%02dMHz \n",
		s_clk / 1000000, (s_clk % 1000000) / 10000 );

	return 0;
}

static char clockinfohelp[] = "clockinfo\n"
"print out processor clock information\n";
__commandlist(cmd_clock_info, "clockinfo", clockinfohelp);
#endif


// 64MB
#define TEST_MAX 0x4000000
// 1MB
//#define TEST_MAX 0x100000
// 8MB
//#define TEST_MAX 0x800000

//#define DDR_PHYS 0x80000000
#define DDR_PHYS 0x80800000

static int cmd_memtest_byte(int argc, char *argv[])
{
  unsigned char *addr;
  unsigned char c;
  int i;

  unsigned long ddr_phys_add, test_max;

  if ( argc == 3 ){
    strtou32( argv[1], &ddr_phys_add);
    strtou32( argv[2], &test_max);
    addr = ddr_phys_add;
  }
  else {
  // base address of DDR
    ddr_phys_add = DDR_PHYS;
    test_max = TEST_MAX;
    addr = (unsigned char *)DDR_PHYS;
  }

  printf("This test will take 4-5 minutes!\nTesting DDR memory from physical 0x%08x through 0x%08x (%d MB)\n",
	 addr, addr + test_max, test_max >> 20);

  printf("Phase 1: Memory byte test... ");

  // test writing bytes
  for (i = 0; i < test_max; i++) {
    addr[i] = (unsigned char) 0xff;
  }


  // test reading back with known values
  for (i = 0; i < test_max; i++) {
    c = addr[i];
    if (c != (unsigned char) 0xff) {
      printf("failed at byte %d, wrong value is 0x%08x\n", i, c);
      return -1;
    }
  }

  // test writing bytes
  for (i = 0; i < test_max; i++) {
    addr[i] = (unsigned char) 0x00;
  }

  printf("passed\n");
  printf("Phase 2: Memory byte test... ");

  // test reading back with known values
  for (i = 0; i < test_max; i++) {
    c = addr[i];
    if (c != (unsigned char) 0x00) {
      printf("failed at byte %d, wrong value is 0x%08x\n", i, c);
      return -1;
    }
  }
  
  printf("passed\n");
  
  return 0;
}

static char memtestbytehelp[] = "1: memtest_byte\n"
"2: memtest_byte [start memory addres] [test length]\n"
"DDR test with bytes\n";
__commandlist(cmd_memtest_byte, "memtest_byte", memtestbytehelp);

static int cmd_memtest_word(int argc, char *argv[])
{
  unsigned long *addr;
  unsigned long w;
  unsigned long pattern[] = {0xaaaaaaaa, 0x55555555};
  unsigned long ddr_phys_add, test_max;
  int i;

  if ( argc == 3 ){
    strtou32( argv[1], &ddr_phys_add);
    strtou32( argv[2], &test_max);
    addr = ddr_phys_add;
  }
  else {
  // base address of DDR
    ddr_phys_add = DDR_PHYS;
    test_max = TEST_MAX;
    addr = (unsigned char *)DDR_PHYS;
  }

    printf("Memory word test... 0x%x, 0x%x \n",pattern[0], pattern[1]);
  printf("This test will take 4-5 minutes!\nTesting DDR memory from physical 0x%08x through 0x%08x (%d MB)\n",
	 ddr_phys_add, ddr_phys_add + test_max, test_max >> 20);

  printf("Memory word test... ");

  // test writing words
  i = 0;
  for (addr = (unsigned long *)ddr_phys_add; addr < (unsigned long *)(ddr_phys_add + test_max); addr++) {
    *addr = pattern[i];
    i = (i + 1) % 2;
  }

  // test reading back with known values
  i = 0;
  for (addr = (unsigned long *)ddr_phys_add; addr < (unsigned long *)(ddr_phys_add + test_max); addr++) {
    w = *addr;
    if (w != pattern[i]) {
      printf("failed at address 0x%08x\n", addr);
      return -1;
    }
    i = (i + 1) % 2;
  }

  printf("passed\n");
  
  return 0;
}

static char memtestwordhelp[] = "1: memtest_word\n"
"2: memtest_word [start memory addres] [test length]\n"
"DDR test with words\n";
__commandlist(cmd_memtest_word, "memtest_word", memtestwordhelp);
