/*
 * frodo.c: FRODO specific stuff
 *
 * Copyright (C) 2001  Abraham van der Merwe <abraham@2d3d.co.za>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: frodo.c,v 1.8 2003/09/04 17:39:00 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/flash.h>
#include <blob/init.h>
#include <blob/reboot.h>
#include <blob/serial.h>
#include <blob/led.h>
#include <blob/time.h>




/* flash descriptor for FRODO flash */
/* 2x Intel 28F128J3A strataflash (32MB) */
static const flash_descriptor_t frodo_flash_descriptors[] =
{
	{
		size: 2 * 128 * 1024,
		num: 128,
		lockable: 1
	},
	{
		/* NULL block */
	},
};




static void init_frodo_flash_driver(void)
{
	flash_descriptors = frodo_flash_descriptors;
	flash_driver = &intel32_flash_driver;
}

__initlist(init_frodo_flash_driver, INIT_LEVEL_DRIVER_SELECTION);




#define RS232_ENABLE	0x00001000
#define IDERESET		0x80
#define IDELED			0x40

static void frodo_init_hardware(void)
{
	u32 *bcr = (u32 *) 0x12000000;
	u16 *ide = (u16 *) 0x40008000;

	/* enable RS232 tranceiver */
	*bcr = RS232_ENABLE;

	/* switch off IDE led */
	*ide &= ~IDELED;

	/* reset the drive */
	*ide |= IDERESET;
	*ide &= ~IDERESET;

	/* select drivers */
	reboot_driver = &sa11x0_reboot_driver;
	serial_driver = &sa11x0_serial_driver;
	led_driver = &sa11x0_gpio_led_driver;
	timer_driver = &intelarm_timer_driver;
}


__initlist(frodo_init_hardware, INIT_LEVEL_DRIVER_SELECTION);
