/*
 * idr.c: Vercel UD-1 (IDR) specific stuff
 *
 * Copyright (C) 2001 Tim Riker <Tim@Rikers.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: idr.c,v 1.15 2003/09/04 17:39:00 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/main.h>
#include <blob/flash.h>
#include <blob/init.h>
#include <blob/reboot.h>
#include <blob/serial.h>
#include <blob/arch.h>
#include <blob/proc/sa1111.h>
#include <blob/led.h>
#include <blob/time.h>

/* flash descriptor for IDR flash */
/* 1x Intel 28F128J3A strataflash (16MB) */
static const flash_descriptor_t idr_flash_descriptors[] =
{
	{
		size: 128 * 1024,
		num: 128,
		lockable: 1
	},
	{
		/* NULL block */
	},
};

static void sa1111_init()
{
	/* switch on clock for sa1111 */
	GAFR |= GPIO_32_768kHz;
	GPDR |= GPIO_32_768kHz;
	TUCR = TUCR_3_6864MHz;

	/* Now, set up the PLL and RCLK in the SA-1111: */
	SBI_SKCR = SKCR_PLL_BYPASS | SKCR_RDYEN | SKCR_OE_EN;
	msleep(100);
	SBI_SKCR = SKCR_PLL_BYPASS | SKCR_RCLKEN | SKCR_RDYEN | SKCR_OE_EN;

	SerialOutputString( "SA1111 ID: " );
	SerialOutputHex( SBI_SKID );
	SerialOutputString( "\n" );

	/*
	 * SA-1111 Register Access Bus should now be available. Clocks for
	 * any other SA-1111 functional blocks must be enabled separately
	 * using the SKPCR.
	 */
	SKPCR |= SKPCR_DCLKEN | SKPCR_PWMCLKEN;

	/*
	 * If the system is going to use the SA-1111 DMA engines, set up
	 * the memory bus request/grant pins. Also configure the shared
	 * memory controller on the SA-1111 (SA-1111 Developer's Manual,
	 * section 3.2.3) and power up the DMA bus clock:
	 */
	GAFR |= (GPIO_MBGNT | GPIO_MBREQ);
	GPDR |= GPIO_MBGNT;
	GPDR &= ~GPIO_MBREQ;
	TUCR |= TUCR_MR;

	SBI_SMCR = (SMCR_DTIM | SMCR_MBGE |
		FInsrt(FExtr(MDCNFG, MDCNFG_SA1110_DRAC0), SMCR_DRAC) |
		((FExtr(MDCNFG, MDCNFG_SA1110_TDL0)==3) ? SMCR_CLAT : 0));

	/*
	 * flash_enable on GPIO_A2
	 * since we are planning jffs2, flash_enable should always be set. but
	 * there is no accounting for taste. Tim Riker <Tim@Rikers.org>
	 */
	PA_DDR &= ~GPIO_GPIO2;
	PA_DWR |= GPIO_GPIO2;

	/*
	 * for debug, we turn the backlight on and off here
	 */
	PA_DDR &= ~GPIO_GPIO3;
#if 0
	/* reboots the machine, if no battery is connected */
	PA_DWR |= GPIO_GPIO3;
	msleep(1000);
	PA_DWR &= ~GPIO_GPIO3;
#endif
}

/* we still are not getting ID on serial, perhaps erik knows why */
__initlist(sa1111_init,		INIT_LEVEL_OTHER_HARDWARE + 1);


static void init_idr_flash_driver(void)
{
	flash_descriptors = idr_flash_descriptors;
	flash_driver = &intel16_flash_driver;
}

__initlist(init_idr_flash_driver, INIT_LEVEL_DRIVER_SELECTION);


static void idr_init_hardware(void)
{
	/* select drivers */
	reboot_driver = &sa11x0_reboot_driver;
	serial_driver = &sa11x0_serial_driver;
	led_driver = &sa11x0_gpio_led_driver;
	timer_driver = &intelarm_timer_driver;

	/* tweak blob config */
	blob_status.boot_delay = 1;
}

__initlist(idr_init_hardware, INIT_LEVEL_DRIVER_SELECTION);
