/*
 * lart.c: LART specific stuff
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: lart.c,v 1.11 2003/09/04 17:39:00 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>
#include <blob/flash.h>
#include <blob/init.h>
#include <blob/partition.h>
#include <blob/reboot.h>
#include <blob/serial.h>
#include <blob/led.h>
#include <blob/time.h>




/* flash descriptor for LART flash */
/* 2x Intel 28F160F3B fast boot block flash (4MB) */
static const flash_descriptor_t lart_flash_descriptors[] =
{
	{ 
		size: 2 * 8 * 1024,
		num: 8,
	},
	{
		size: 2 * 64 * 1024,
		num: 31,
	},
	{
		/* NULL block */
	},
};




/* default partition table for LART */
static const blob_partition_t lart_default_partition_table[] = {
	{
		/* start of table */
		magic:	BLOB_DEFAULT_PART_TABLE_MAGIC,
		next:	sizeof(blob_partition_t),
		offset:	0x00000000,	/* absolute base address */
		size:	4 * 1024 * 1024 /* total size of the flash */
	},
	{
		/* blob itself */
		magic:		BLOB_PART_VALID_MAGIC,
		next:		sizeof(blob_partition_t),
		offset:		BLOB_FLASH_BASE,
		size:		BLOB_FLASH_LEN,
		name:		"blob",
		mem_base:	BLOB_RAM_BASE
	},
	{
		/* parameter block */
		magic:	BLOB_PART_VALID_MAGIC,
		next:	sizeof(blob_partition_t),
		offset:	PARAM_FLASH_BASE,
		size:	PARAM_FLASH_LEN,
		name:	"param",
		flags:	BLOB_PART_FLAG_PTABLE
	},
	{
		/* kernel */
		magic:		BLOB_PART_VALID_MAGIC,
		next:		sizeof(blob_partition_t),
		offset: 	KERNEL_FLASH_BASE,
		size:		KERNEL_FLASH_LEN,
		name:		"kernel",
		flags:		BLOB_PART_FLAG_LOAD | BLOB_PART_FLAG_EXEC,
		mem_base:	KERNEL_RAM_BASE,
		entry_point:	KERNEL_RAM_BASE
	},
	{
		/* ramdisk */
		magic:		BLOB_PART_VALID_MAGIC,
		next:		sizeof(blob_partition_t),
		offset:		RAMDISK_FLASH_BASE,
		size:		RAMDISK_FLASH_LEN,
		name:		"ramdisk",
		flags:		BLOB_PART_FLAG_LOAD,
		mem_base:	RAMDISK_RAM_BASE
	},
	{
		/* last entry */
		magic:	BLOB_PART_LAST_MAGIC
	}
};




static void lart_set_partition_table(void)
{
	/* the default partition table */
	default_partition_table = lart_default_partition_table;
	
	/* we don't know where the flash partition table will live, so
	 * put the pointer at 0x0000000 and let the partition table
	 * parser figure out.
	 */
	flash_partition_table = (blob_partition_t*)0x00000000;
}

__initlist(lart_set_partition_table, INIT_LEVEL_OTHER_STUFF);
/* FIXME: need to think if this is the correct init level, we might
 * want to have this before the param list stuff starts running */



static void lart_select_drivers(void)
{
	reboot_driver = &sa11x0_reboot_driver;
	serial_driver = &sa11x0_serial_driver;
	led_driver = &sa11x0_gpio_led_driver;
	timer_driver = &intelarm_timer_driver;

	/* we could do funky detection over here, because the LART can
	 * have both internal and external flash with different
	 * properties. for the time being we just ignore that fact.
	 *   -- Erik
	 */

	flash_descriptors = lart_flash_descriptors;
	flash_driver = &intel32_flash_driver;
}

__initlist(lart_select_drivers, INIT_LEVEL_DRIVER_SELECTION);
