/*
 * ledasm-s3c2500.S: simple LED control code
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 * Copyright (C) 2003  Russ Dill (Russ.Dill@asu.edu)
 *
 * $Id: ledasm-s3c2500.S,v 1.1 2003/09/04 18:03:09 russd Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/*
 * Documentation:
 */

.ident "$Id: ledasm-s3c2500.S,v 1.1 2003/09/04 18:03:09 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>


.text



.globl ledinit
	/* initialise LED GPIO and turn LED on.
	 * clobbers r0 and r1
	 */
ledinit:

#if defined (NO_GPIO_LED_SUPPORT)
	mov	pc, lr	
#endif

	mov	r0, #ASIC_BASE
	orr	r0, r0, #IOPB
	ldr     r1, [r0, #IOPMODE1]
	bic	r1, r1, #LED_GPIO
	str	r1, [r0, #IOPMODE1]	/* LED GPIO is output */
	ldr	r1, [r0, #IOPDATA1]
	bic	r1, r1, #LED_GPIO
	str	r1, [r0, #IOPDATA1]	/* turn LED on */
	mov	pc, lr




.globl led_on
	/* turn LED on. clobbers r0 and r1 */
led_on:

#if defined (NO_GPIO_LED_SUPPORT)
	mov	pc, lr
#endif

	mov	r0, #ASIC_BASE
	orr	r0, r0, #IOPB
	ldr	r1, [r0, #IOPDATA1]
	bic	r1, r1, #LED_GPIO
	str	r1, [r0, #IOPDATA1]	/* turn LED on */
	mov	pc, lr




.globl led_off
	/* turn LED off. clobbers r0 - r2 */
led_off:

#if defined (NO_GPIO_LED_SUPPORT)
	mov	pc, lr
#endif

	mov	r0, #ASIC_BASE
	orr	r0, r0, #IOPB
	ldr	r1, [r0, #IOPDATA1]
	orr	r1, r1, #LED_GPIO
	str	r1, [r0, #IOPDATA1]	/* turn LED off */
	mov	pc, lr



	/* busy wait loop. clobbers r0 */
wait_loop:
	mov	r0, #0x40000
wait_loop1:
	subs	r0, r0, #1
	bne	wait_loop1
	mov	pc, lr


blinky:
	/* blink LED. clobbers r0 and r1 */
	mov	r2, lr
	bl	led_on
	bl	wait_loop
	bl	led_off
	bl	wait_loop
	mov	pc, r2
	

.globl led_blink
	/* blinks LED r0 times */
	/* clobbers r0 - r4 */
led_blink:

#if defined (NO_GPIO_LED_SUPPORT)
	mov	pc, lr
#endif

	mov	r4, lr
	mov	r3, r0

	/* switch LED off if it wasn't already off */
	bl	led_off
	bl	wait_loop
	
blink0:
	bl	blinky
	subs	r3, r3, #1
	bne	blink0
	mov	pc, r4
