/*
 * usb_set_para.c: set USB parameters
 *
 * Client and server IP are used for TFTP downloads.
 *
 * Copyright (C) 2003 Stefan Eletzhofer <stefan.eletzhofer@eletztrick.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/**********************************************************************
 * Includes
 */

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#if defined( CONFIG_USBDNET_SUPPORT ) || defined( CONFIG_USBDNET2_SUPPORT ) 

#include <blob/errno.h>
#include <blob/debug.h>
#include <blob/types.h>
#include <blob/util.h>
#include <blob/command.h>

#include <net/net.h>

/**********************************************************************
 * defines
 */
#undef SETIP_DEBUG
#ifdef SETIP_DEBUG
static int dbg = 1;
#	define DBG( x, args... )	if ( dbg>=x ) printf( args )
#else
#	define DBG( x, args... )
#endif

extern u16 usb_vendor_id;
extern u16 usb_product_id;

/**********************************************************************
 * command
 */
static int setusbid_cmd( int argc, char *argv[] )
{
	u16 *id;
	u32 n;
	char *ptr, *num;
//	int i, fini;

	if ( argc < 3 ) {
		printf("Vendor  ID : 0x%x \n", usb_vendor_id);
		printf("Product ID : 0x%x \n", usb_product_id);
		return 0;
	}

	switch ( argv[1][0] ) {
		case 'v':
		case 'V':
			id = &usb_vendor_id;
			break;
		case 'p':
		case 'P':
			id = &usb_product_id;
			break;
		default:
			return -EINVAL;
			break;
	}

	num = ptr = argv[2];

	strtou32( num, &n );
	*id = (u16)n;

	printf( "Set id: 0x%x\n", *id);

	return 0;
}
char setusbid_help[] = "setusbid command. Set USB VID/PID.\n"
"usage: setusbid {vid|pid} id\n" ;;

__commandlist(setusbid_cmd, "setusbid", setusbid_help);

#endif

