/*
 * start-s3c2500.S: Samsung s3c2500 specific start code
 *
 * Copyright (C) 1999 2000 2001 2002 Erik Mouw (J.A.K.Mouw@its.tudelft.nl) and
 *                     Jan-Derk Bakker (J.D.Bakker@its.tudelft.nl)
 * Copyright (C) 2003 Russ Dill (Russ.Dill@asu.edu)
 *
 * $Id: start-s3c2500.S,v 1.1 2003/09/04 18:03:09 russd Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/*
 * Documentation:
 */

.ident "$Id: start-s3c2500.S,v 1.1 2003/09/04 18:03:09 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>

#define F_BIT	(1 << 6)
#define	I_BIT	(1 << 7)

.text

cpu_speed:	.word	CPU_SPEED
bus_speed:	.word	BUS_SPEED

/* We start by implementing *all* exception vectors */


/* Reset vector: this initialises the machine */
.globl reset
reset:
	/* Disable interrupts */
	mrs	r0, cpsr
	orr	r0, r0, #I_BIT | F_BIT
	msr	cpsr, r0

	/* Enable "custom" clocks */
	mov	r0, #ASIC_BASE
	orr	r0, r0, #SYSCFG_BASEB

	mov	r1, #CPLLREN | SPLLREN
	str	r1, [r0]

	/* Set CPU clock */
	ldr	r1, cpu_speed
	str	r1, [r0, #CPLL]

	/* Set bus clock */
	ldr	r1, bus_speed
	str	r1, [r0, #SPLL]

	/* init LED */
	bl	ledinit

	/* setup memory */
	bl	memsetup

	/* turn off the LED. if it stays off it is an indication that
	 * we didn't make it into the C code */
	bl	led_off

	/* everything is said and done over here, call normal_boot in
	 * the generic startup code to continue the boot procedure */
	bl	normal_boot

	/* oops, normal_boot returns, something went wrong. signal an
	 * error to the user */
	mov	r6, #2
	b	endless_blink




/* we could choose to handle all exceptions in a nice way, but the
 * best is to treat them as errors because blob should not contain
 * errors
 */

/* Undefined instruction exception */
.globl undefined_instruction
undefined_instruction:
	mov	r6, #3
	b	endless_blink




/* SWI */
.globl software_interrupt
software_interrupt:
	/* NOTE: This is NOT an error! If you think that blob should return
	 * from software interrupts, you're plain WRONG. The source of the
	 * problem is in the kernel: you should *disable* CONFIG_ANGELBOOT
	 * simply because blob is not angel. -- Erik
	 */
	mov	r6, #4
	b	endless_blink

	
	

/* prefetch exception. shouldn't happen though we usually run with
 * i-cache enabled */
.globl prefetch_abort
prefetch_abort:
	mov	r6, #5
	b	endless_blink




/* data abort */
.globl data_abort
data_abort:
	mov	r6, #6
	b	endless_blink




/* we *should* never reach this */
.globl not_used
not_used:
	mov	r6, #7
	b	endless_blink




/* interrupt. we could handle this differently later if some kind of
 * driver in blob wants to be interrupt driven. for the time being we
 * treat it as an error.
 */
.globl irq
irq:
	mov	r6, #8
	b	endless_blink




/* FIQ. same as IRQ */
.globl fiq
fiq:
	mov	r6, #9
	b	endless_blink




/* endless loop that blinks the LED. r6 contains the number of blinks */
endless_blink:
	bl	wait
	mov	r0, r6
	bl	led_blink
	b	endless_blink
		
wait:
	/* busy wait loop*/
	mov	r5, #0x100000
wait0:
	subs	r5, r5, #1
	bne	wait0
	mov	pc, lr
