/*
 * start.S: blob start code
 *
 * Copyright (C) 1999 2000 2001 2002 Erik Mouw (J.A.K.Mouw@its.tudelft.nl) and
 *                     Jan-Derk Bakker (J.D.Bakker@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/*
 * This is the blob start code. The SA-1100 jumps to address 0x00000000
 * after a reset. We put a single branch code at this position which jumps
 * to a safe region to do the actual setup. All other vectors just point
 * to an endless loop for the moment.
 *
 * Documentation:
 * [1] Intel Corporation, "Intel StrongARM SA-1100 Microprocessor
 *     Developer's Manual", April 1999
 * [2] S. Furber, "ARM System Architecture", Addison Wesley Longman
 *     Ltd., Essex, England, 1996.
 * [3] Intel Corporation, "Intel StrongARM SA-1110 Microprocessor
 *     Advanced Developer's manual", December 1999
 */

.ident "$Id: start.S,v 1.12 2003/08/06 22:55:41 abz Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>


.text

/* Jump vector table as in table 3.1 in [1] */
.globl _start
_start:
	b	reset
	b	undefined_instruction
	b	software_interrupt
	b	prefetch_abort
	b	data_abort
	b	not_used
	b	irq
	b	fiq

/* some defines to make life easier */
BLOB_START:	.word	BLOB_ABS_BASE_ADDR
piggy_start:	.word	__piggy_start
piggy_end:	.word	__piggy_end
.align

.globl normal_boot
normal_boot:
	/* check the first 1MB of BLOB_START in increments of 4k */
	mov		r7, #0x1000
	mov		r6, r7, lsl #6	/* 4KB << 5 = 256KB */
	ldr		r5, BLOB_START

mem_test_loop:
	mov		r0, r5
	bl		testram
	teq		r0, #1
	beq		endless_loop	/* oops, something went wrong :( */

	add		r5, r5, r7
	subs	r6, r6, r7
	bne		mem_test_loop

relocate:
	/* relocate the second stage loader */
	adr	r1, _start

	ldr	r0, piggy_start
	add	r0, r0, r1
	ldr	r2, piggy_end
	add	r2, r2, r1

	ldr	r1, BLOB_START

	/* r0 = source address
	 * r1 = target address
	 * r2 = source end address
	 */
copy_loop:
	ldmia	r0!, {r3-r10}
	stmia	r1!, {r3-r10}
	cmp	r0, r2
	ble	copy_loop

	/* blob is copied to ram, so jump to it */
	ldr	r0, BLOB_START
	mov	pc, r0

endless_loop:
	/* either the second stage loader returned, or we got here from
	 * mem_test_loop. both is bad, and we really want to toggle a
	 * GPIO line or so. for the time being we just do an endless
	 * loop. FIXME! -- erik 
	 */
	b	endless_loop

