/*
 * start.s: Littletno/Monahans start code using XLLI
 *
 * Copyright (C) 2005 Intel Corporation (stanley.cai@intel.com)
 *
 * (C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

        .include  "xlli_monahans_defs_gnu.inc"                @ Monahans specific include file
        .include  "xlli_zylonite_defs_gnu.inc"                @ Zylonite specific include file

#define MFP_I2C_SCL 0x40e102c4
#define MFP_I2C_SDA 0x40e102c8

.text

/*
 * We start by implementing *all* exception vectors
 *
 * Reset vector: this initialises the machine
 * note here that this not yet taken sleep wakeup into account -- lets just
 * get something to work first.
 */
.macro blink, count
	mov  r3, \count
	b    endless_blink
.endm

.macro cpwait reg
	mrc  p15,0,\reg,c2,c0,0
	mov  \reg,\reg
	sub  pc,pc,#4
.endm

.macro wait, count
	mov		r0, \count
0:
	subs		r0, r0, #1
	bne		0b
.endm

.globl reset
reset:
2:
	/* setup SP pointer */
	/* put it at the end of iSRAM */
	ldr		sp, =0x5c040000

        /* set CPU to SVC32 mode */
        mrs             r0, cpsr
        bic             r0, r0, #0x1f
        orr             r0, r0, #0x13
        msr             cpsr, r0
	
        /* turn everything off (caches off, MMU off, etc.) */
        mov             r0, #0x78
        mcr             p15, 0, r0, c1, c0, 0

	/* PSR, is the processor in S2/S3 state*/
	@ldr	r1, =0x40f50004
	@ldr	r0, [r1]
	@tst     r0, #1
	@bne     init_volt

	/* Initialize GPIO */
	bl 	xlli_GPIO_init
	bl	xlli_MFPR_init

	bl  skip_init_volt

	/* Hack to enable the LCD backlight */
	@bl	init_i2c

	/* LCD backlight full intensity */
	@mov	r4, #0x68	/* Device address */
	@mov	r5, #0x3c	/* Register offset */
	@mov	r6, #0x7f	/* Data */
	@bl	writei2c

	/* First write operation need twice */
	@mov	r4, #0x68	/* Device address */
	@mov	r5, #0x3c	/* Register offset */
	@mov	r6, #0x7f	/* Data */
	@bl	writei2c
	
	/* LCD power 23mA */
	@mov	r4, #0x68	/* Device address */
	@bl	writei2c

init_volt:
	/* init I2C */
	bl      init_i2c

	/* VCC_CORE = 1375mv VCC_SRAM = 1400mv*/
	mov     r4, #0x68 /* Device address */
	mov     r5, #0x10 /* (OVER1) Register offset */
	mov     r6, #0x05 /* Data */
	bl      writei2c  /* I2C warm up in case the first write would fail */

	mov     r4, #0x68 /* Device address */
	mov     r5, #0x24 /* (ADTV2) Register offset */

	/* If is MHN LV A1, we raise core voltage to
	 * 1500mv to workaround NAND DB error.
	 */
	mrc     p15, 0, r2, c0, c0, 0
	mov	r3, #0xFF
	orr	r3, r3, #0xFF00
	and     r2, r2, r3
	
	mov	r8, #0x91
	orr	r8, r8, #0x6800
	cmp	r2, r8

	movne   r6, #0x1A /* Not MHN LV A1, 1375mv */
	moveq	r6, #0x1F /* MHN LV A1, 1500mv */
        bl      writei2c

	mov     r4, #0x68 /* Device address */
	mov     r5, #0x2A /* (SDTV2) Register offset */
	mov     r6, #0x1B /* Data */
	bl      writei2c

	mov     r4, #0x68 /* Device address */
	mov     r5, #0x20 /* (VCC1) Register offset */
	mov     r6, #0x33 /* Data */
	bl      writei2c

	mov     r4, #0x68 /* Device address */
	mov     r5, #0x0B /* Sysctl register for Watchdog */
	mov     r6, #0x79 /* Data */
	bl      writei2c

	mov     r4, #0x68 /* Device address */
	mov     r5, #0x15 /* LDO10 for TECH_3P */
	mov     r6, #0x02 /* Data */
	bl      writei2c

	mov     r4, #0x68 /* Device address */
	mov     r5, #0x16 /* LDO10 for TECH_3P */
	mov     r6, #0x02 /* Data */
	bl      writei2c

	mov     r4, #0x68 /* Device address */
	mov     r5, #0x11 /* OVER2 */
	mov     r6, #0xFB /* Enable LDO13, 12, 11, 10, 9, 7 & 6 */
	bl      writei2c

	/* Initialize LEDs */
	bl	ledsetup
	bl	led1set
	bl 	led2set

skip_init_volt:
        /* Initialize Interrupt Controller */
        /* Step 1 - Enable CP6 permission */
        mrc     p15, 0, r1, c15, c1, 0  @ read CPAR
        orr     r1, r1, #0x40
        mcr     p15, 0, r1, c15, c1, 0
        cpwait  r1
	
        bl      xlli_intr_init

init_clock:
	/* 
         * turn off most of D0CKEN_A and D0CKEN_B clocks by default, except 
         * SMC, DMC, SRAM, I2C, GPIO,interrupt controller
         */
	ldr     r4, =xlli_CLKREGS_PHYSICAL_BASE
	ldr     r0, =0x00000700
#ifdef CONFIG_CPU_MONAHANS_LV
        /* 
         * According to the DM,  for CKENA, Write 0b1 to reserve bit.
         */
        orr     r0, r0, #0x82000000
        orr     r0, r0, #0x00090000
        orr     r0, r0, #0x00000003
#endif
	str     r0, [r4, #xlli_D0CKEN_A_offset]
	ldr     r0, [r4, #xlli_D0CKEN_A_offset]

	ldr     r0, =0xe00000d0
#ifdef CONFIG_CPU_MONAHANS_LV
        /* 
         * According to the DM, for CKENB, Write 0b1 to reserve bit.
         */
        orr     r0, r0, #0xFF000000
        orr     r0, r0, #0x00FC0000
        orr     r0, r0, #0x0000FE00
        orr     r0, r0, #0x0000002D
	str     r0, [r4, #xlli_D0CKEN_B_offset] 
	ldr     r0, [r4, #xlli_D0CKEN_B_offset]
#endif


	/* change frequency routine */
	ldr	r4, =xlli_CLKREGS_PHYSICAL_BASE
	@  .... ..10 1... 11.. 1011 .010 ...0 0100
	@ldr	r0, =0x00033108		@ 104MHz (Run)	
	ldr	r0, =0x01073110		@ 208MHz (Run)	
	@ldr	r0, =0x010b7210		@ 416MHz (Turbo)
	@ldr	r0, =0x028fb218		@ 624MHz (Turbo)
	@ldr	r0, =0x028fb21f		@ 806MHz (Turbo)
	@ldr	r0, =0x028fb218		@ 624MHz (Turbo)
	str	r0, [r4, #xlli_ACCR_offset]

	bl	xlli_freq_change


@	sub      pc, pc, #8
skip_init_clock:
	@ get processor ID
	mrc     p15, 0, r2, c0, c0, 0
	mov	r3, #0xFF
	orr	r3, r3, #0xFF00
	and     r2, r2, r3

	mov     r1, r2
	mvn     r3, #0xf
	and     r3, r3, r1
	mov     r4, #0x6400
	orr     r4, r4, #0x20
	cmp     r3, r4
	beq     init_mem        @ Monahans P Ax, MOBM v2, need setup DDR

mhnPBx:
	mvn     r3, #0xf
	and     r3, r3, r1
	mov     r4, #0x6800
	orr     r4, r4, #0x20
	cmp     r3, r4
	bne     mhnL            @ Not Monahans P B step
	add     r4, r4, #6
	cmp     r1, r4
	bhi     init_smem       @ Monahans P > B2, MOBM v3, needn't setup DDR
	b       init_mem        @ Monahans P <= B2, MOBM v2, need setup DDR

mhnL:
	mov     r4, #0x6800
	orr     r4, r4, #0x80
	cmp     r1, r4
	beq     init_mem        @ Monahans L A0, MMOBM v2, need setup DDR
	b       init_smem       @ Monahans L > A0, LV MOBM v3, needn't setup DDR

init_mem:
	/* setup GPIO8(SW1) as GPIO input for xlli_Dmem_initP1 */
	ldr	r0, =xlli_MFPR_PHYSICAL_BASE
	ldr	r1, =0x0000A040
	str     r1, [r0, #0x298]
	ldr	r1, [r0, #0x298]

	ldr	r0, =xlli_MFPR_PHYSICAL_BASE
	ldr	r1, =0x0000A041
	str     r1, [r0, #0x49c]

	ldr     r0, =xlli_GPIOREGS_PHYSICAL_BASE
	ldr     r1, [r0, #xlli_GPDR0_offset]
	bic     r1, r1, #0x100
	str     r1, [r0, #xlli_GPDR0_offset]

	/* Initialize Memory Controller */
        ldr     r4,     =xlli_DMEMREGS_PHYSICAL_BASE

        @
        @ It seems that the Boot ROM may be touching the DMEMC, enabling DDR.
        @ I will test to explcitly disable it before re-configuring it.
        @
        ldr    r0, [r4, #xlli_MDCNFG_offset]
        ldr    r1, =0x8000077F                      @ clear bit 30, DMC_EN
        and    r1, r1, r0


        str    r1, [r4,#xlli_MDCNFG_offset]
        ldr    r0, [r4, #xlli_MDCNFG_offset]
        ldr    r1, =0xC000077C                      @ clear bits 0,1.  Disable both partitions
        and    r1, r1, r0
        str    r1, [r4,#xlli_MDCNFG_offset]
        ldr    r0, [r4, #xlli_MDCNFG_offset]
        ldr    r1, =0x00000000                      @ clear all bits at this point.
        and    r1, r1, r0
        str    r1, [r4,#xlli_MDCNFG_offset]
        ldr    r0, [r4, #xlli_MDCNFG_offset]       @ final read-back

        @@@ end disable code @@@

        bl      xlli_Dmem_initP1         @   init DDR-SDRAM (Part 1)

        ldr     r0,  =xlli_DMEMREGS_PHYSICAL_BASE
        bl      xlli_Rcomp
	
        ldr     r0,  =xlli_DMEMREGS_PHYSICAL_BASE
        ldr     r1,  =xlli_SDRAM_PHYSICAL_BASE
        bl      xlli_Dmem_initP2         @ init DDR-SDRAM (Part 2)

init_smem:
        ldr     r0,  =0x4a000000
        bl      xlli_Smem_init

	@ load PSR register
	ldr	r1, =0x40f50004
	ldr	r0, [r1]
	tst     r0, #1

	bne     s2_reset

	bl	led1clear
	bl	led2clear
	bl 	normal_boot

s2_reset:
os_resume:	
	ldr	r1, =0x80000000
	ldr	pc, [r1]

	/* should never be here */
	blink	#1

.globl undefined_instruction
undefined_instruction:
	blink	#2

.globl software_interrupt
software_interrupt:
	blink	#3

.globl prefetch_abort
prefetch_abort:
	blink	#4

.globl data_abort
data_abort:
	blink	#5

.globl not_used
not_used:
	blink	#6

.globl irq
irq:
	blink	#7

.globl fiq
fiq:
	blink	#8

/* The functions below is platform-dependent. */
led1set:
	ldr		r2, =0x2
	ldr		r1, =0x40E00018 @ GPSR0
	str		r2, [r1]
	
	mov		pc, lr

led1clear:
	mov		pc, lr

led2set:
	mov		pc, lr

led2clear:
	mov		pc, lr

ledsetup:
	mov		pc, lr

ledflash:
	mov		pc, lr 

endless_blink:
	wait		#0x2000000
	b		endless_blink

init_i2c:
	/* setup MFPRs for I2C */
	ldr		r0, =0xa841
	ldr		r1, =MFP_I2C_SCL
	str		r0, [r1]
	ldr		r1, =MFP_I2C_SDA
	str		r0, [r1]

	/* enable i2c clock */
	ldr             r1, =0x41340010  /* D0CKEN_B */
	ldr             r0, [r1]
	orr             r0, r0, #0x10      /* enable i2c clock */
	str             r0, [r1]
 
	/* Initialize I2C Unit */	
	ldr		r1, =0x40301680 /* I2C register start address */
	mov		r0, #0x0
	str		r0, [r1, #0x20] /* ISAR */
	mov		r0, #0x4000     /* reset I2C */
	str		r0, [r1, #0x10] /* ICR */
	mov		r0, #0x60 	/* set IUE and SCLE */
	str		r0, [r1, #0x10]
	mov		pc, lr

writei2c:
	ldr		r3, =0x40301680 /* I2C register start address */

	/* Set slave I2C address */
	str		r4, [r3, #0x8] 	/* IDBR */

	/* send a start condition */
	ldr             r0, [r3, #0x10]
	@mov		r0, #0x9	/* START & TB */
	orr		r0, r0, #0x9    /* START & TB */
	ldr             r2, =0xffffeffd 
	and             r0, r0, r2      /* ~(STOP | ALDIE) */
	str		r0, [r3, #0x10] /* ICR */

	/* wait until tx buffer empty */
	mov		r8, #0x80000 	/* time out */
1:
	ldr		r2, [r3, #0x18] /* ISR */
	subs		r8, r8, #1
	moveq		pc, lr
	ands		r2, r2, #0x40	/* ITE in ISR */
	beq		1b
	str		r2, [r3, #0x18] /* clear status */

	/* send first byte(register address) */
	str		r5, [r3, #0x8] 	/* IDBR */
	ldr             r0, [r3, #0x10] /* read ICR */
@	mov		r0, r0, #0x69	/* IUE, SCLE, START & TB */
	ldr             r2, =0x1008
	orr		r0, r0, r2	/* ALDIE, TB */
	ldr             r2, =0xfffffffe
	and		r0, r0, r2	/* ~START */
	ldr             r2, =0xfffffffd 
	and             r0, r0, r2          /* ~STOP */
	str		r0, [r3, #0x10]
	
	/* wait until tx buffer empty */
	mov		r8, #0x80000 	/* time out */
2:
	ldr		r2, [r3, #0x18] /* ISR */
	subs		r8, r8, #1
	moveq		pc, lr
	ands		r2, r2, #0x40	/* ITE in ISR */
	beq		2b
	str		r2, [r3, #0x18] /* clear status */

	/* send second byte (register value) */
	str		r6, [r3, #0x8] 	/* IDBR */
	ldr             r0, [r3, #0x10] /* read ICR */
@	mov		r0, r0, #0x69	/* IUE, SCLE, START & TB */
	ldr             r2, =0x1008
	orr		r0, r0, r2	/* ALDIE, TB */
	ldr             r2, =0xfffffffe
	and		r0, r0, r2	/* ~START */
	orr             r0, r0, #0x2        /* STOP */
	str		r0, [r3, #0x10]

	/* wait until tx buffer empty */
	mov		r8, #0x80000 	/* time out */
3:
	ldr		r2, [r3, #0x18] /* ISR */
	subs		r8, r8, #1
	moveq		pc, lr
	ands		r2, r2, #0x40	/* ITE in ISR */
	beq		3b
	str		r2, [r3, #0x18] /* clear status */

	mov		pc, lr
	
