/*
 * start.s: Zylonite/Monahans-P start code using XLLI
 *
 * Copyright (C) 2005 Intel Corporation (stanley.cai@intel.com)
 *
 * (C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

        .include  "xlli_monahans_defs_gnu.inc"                @ Monahans specific include file
        .include  "xlli_zylonite_defs_gnu.inc"                @ Zylonite specific include file

#if   defined(CONFIG_CPU_MONAHANS_P) || defined(CONFIG_CPU_MONAHANS_PL)
#define MFP_I2C_SCL 0x40e10414
#define MFP_I2C_SDA 0x40e10418
#define SCL_GPIO    32
#define SDA_GPIO    33
#elif defined(CONFIG_CPU_MONAHANS_L) || defined(CONFIG_CPU_MONAHANS_LV)
#define MFP_I2C_SCL 0x40e102c4
#define MFP_I2C_SDA 0x40e102c8
#define SCL_GPIO    21 
#define SDA_GPIO    22
#else
#error "please define MFP pins for I2C!"
#endif
.text

/*
 * We start by implementing *all* exception vectors
 *
 * Reset vector: this initialises the machine
 * note here that this not yet taken sleep wakeup into account -- lets just
 * get something to work first.
 */
.macro blink, count
	mov  r3, \count
	b    endless_blink
.endm

.macro cpwait reg
	mrc  p15,0,\reg,c2,c0,0
	mov  \reg,\reg
	sub  pc,pc,#4
.endm

.macro wait, count
	mov		r0, \count
0:
	subs		r0, r0, #1
	bne		0b
.endm

.globl reset
reset:
2:
	/* setup SP pointer */
	/* put it at the end of iSRAM */
	ldr		sp, =0x5c040000

        /* set CPU to SVC32 mode */
        mrs             r0, cpsr
        bic             r0, r0, #0x1f
        orr             r0, r0, #0x13
        msr             cpsr, r0
	
        /* turn everything off (caches off, MMU off, etc.) */
        mov             r0, #0x78
        mcr             p15, 0, r0, c1, c0, 0


chk_gpio_reset:
	/* check GPIO reset */
	ldr	r1, =0x40f40004
	ldr	r0, [r1]
	tst	r0, #8
	beq	control_i2c

	/* clear GPIO flag */
	/*mov	r2, #8*/
	/*str	r2, [r1]*/

control_i2c:
        /* control I2C */
	ldr		r0, =0xa840
	ldr		r1, =MFP_I2C_SCL
        str		r0, [r1]
	ldr		r1, =MFP_I2C_SDA
	str		r0, [r1]        /* configure SCL/SDA as gpio pin */
	ldr		r4, =0x40e00000
	mov		r5, #(1<<(SCL_GPIO%32)) /* corresponding gpio bit */
	ldr		r0, [r4, #(0x0c + 4*(SCL_GPIO/32))]	/* GPDR for SCL */
	orr		r0, r0, r5
	str		r0, [r4, #(0x0c + 4*(SCL_GPIO/32))] /* set SCL as output */
	mov		r5, #(1<<(SDA_GPIO%32)) /* corresponding gpio bit */
	ldr		r0, [r4, #(0x0c + 4*(SDA_GPIO/32))]	/* GPDR for SDA */
	bic		r0, r0, r5
	str		r0, [r4, #(0x0c + 4*(SDA_GPIO/32))] /* set SDA as input*/


loop_scl:
	mov		r2, #0x1000
loop1:
	subs		r2, r2, #1
	bne		loop1
	mov		r5, #(1<<(SCL_GPIO%32)) /* corresponding gpio bit */
	str		r5, [r4, #(0x18 + 4*(SCL_GPIO/32))]	/* GPSR: set SCL high */
	mov		r2, #0x1000
loop2:
	subs		r2, r2, #1
	bne		loop2
	str		r5, [r4, #(0x24 + 4*(SCL_GPIO/32))]	/* GPCR: set SCL low */
        mov		r2, #0x1000
loop3:
	subs		r2, r2, #1
	bne		loop3
	ldr		r0, [r4, #(0 + 4*(SDA_GPIO/32))]  /* GPLR for SDA */
        tst 		r0, #(1<<(SDA_GPIO%32)) /* Check SDA low or high */
	beq		loop_scl /* do until SDA is high */
	
	mov 		r5, #(1<<(SDA_GPIO%32))
	ldr		r0, [r4, #(0x0c + 4*(SDA_GPIO/32))] /* GPDR for SDA */
	orr		r0, r0, r5
        str		r0, [r4, #(0x0c + 4*(SDA_GPIO/32))] /* GPDR: set SDA as output */
	mov		r2, #0x1000
loop4: 
	subs		r2, r2, #1
	bne		loop4
	str		r5, [r4, #(0x24 + 4*(SDA_GPIO/32))] /* GPCR: set SDA as low */
	mov		r2, #0x1000
loop5:
	subs		r2, r2, #1
	bne		loop5
	str		r5, [r4, #(0x18 + 4*(SDA_GPIO/32))] /* GPSR: set SDA as high */
	mov 		r2, #0x1000
loop6:
	subs		r2, r2, #1
	bne		loop6
	mov		r5, #(1<<(SDA_GPIO%32)) /* select SDA */
	str		r5, [r4, #(0x18 + 4*(SDA_GPIO/32))] /* GPSR: set SDA as high */
	mov		r2, #0x1000
loop7:
	subs		r2, r2, #1
	bne		loop7

	/* PSR, is the processor in S2/S3 state*/
	ldr	r1, =0x40f50004
	ldr	r0, [r1]
	tst     r0, #1
	bne     init_volt

	/* Initialize GPIO */
	bl 	xlli_GPIO_init
	bl	xlli_MFPR_init

init_volt:
	/* init I2C */
	bl      init_i2c

        /* get processor id */
        mrc     p15, 0, r2, c0, c0, 0
	mov	r3, #0x69000000
	orr	r3, #0x00050000
	orr     r3, #0x00006800
        orr     r3, #0x00000091
        cmp     r2, r3
        beq     core_1550    /* For MHLV A1, set Vcc_core as 1550mv to avoid NAND failure */
	bne     core_1375

core_1550:
	/* VCC_CORE = 1550mv VCC_SRAM = 1400mv*/
        mov     r4, #0x92 /* Device address */
        mov     r5, #0x15 /* Register offset */
        mov     r6, #0x9c /* Data */
        bl      writei2c  /* I2C warm up in case the first write would fail */

        mov     r4, #0x92 /* Device address */
        mov     r5, #0x15 /* Register offset */
        mov     r6, #0x9c /* Data */
        bl      writei2c

	b       sram_1400

core_1375:
	/* VCC_CORE = 1375mv VCC_SRAM = 1400mv*/
        mov     r4, #0x92 /* Device address */
        mov     r5, #0x15 /* Register offset */
        mov     r6, #0x95 /* Data */
        bl      writei2c  /* I2C warm up in case the first write would fail */

        mov     r4, #0x92 /* Device address */
        mov     r5, #0x15 /* Register offset */
        mov     r6, #0x95 /* Data */
        bl      writei2c

sram_1400:
        mov     r4, #0x92 /* Device address */
        mov     r5, #0x12 /* Register offset */
        mov     r6, #0x30 /* Data */
        bl      writei2c

	/* Initialize LEDs */
	bl	ledsetup
	bl	led1set
	bl 	led2set

        /* Initialize Interrupt Controller */
        /* Step 1 - Enable CP6 permission */
        mrc     p15, 0, r1, c15, c1, 0  @ read CPAR
        orr     r1, r1, #0x40
        mcr     p15, 0, r1, c15, c1, 0
        cpwait  r1
	
        bl      xlli_intr_init

init_clock:
	/* 
         * turn off most of D0CKEN_A and D0CKEN_B clocks by default, except 
         * SMC, DMC, SRAM, I2C, GPIO,interrupt controller
         */
	ldr     r4, =xlli_CLKREGS_PHYSICAL_BASE
	ldr     r0, =0x00000700
#ifdef CONFIG_CPU_MONAHANS_LV
        /* 
         * According to the DM,  for CKENA, Write 0b1 to reserve bit.
         */
        orr     r0, r0, #0x82000000
        orr     r0, r0, #0x00090000
        orr     r0, r0, #0x00000001
#endif
	str     r0, [r4, #xlli_D0CKEN_A_offset]
	ldr     r0, [r4, #xlli_D0CKEN_A_offset]

	ldr     r0, =0xe00000d0
#ifdef CONFIG_CPU_MONAHANS_LV
        /* 
         * According to the DM, for CKENB, Write 0b1 to reserve bit.
         */
        orr     r0, r0, #0xFF000000
        orr     r0, r0, #0x00FC0000
        orr     r0, r0, #0x0000FE00
        orr     r0, r0, #0x0000002C
	str     r0, [r4, #xlli_D0CKEN_B_offset] 
	ldr     r0, [r4, #xlli_D0CKEN_B_offset]
#endif

	/* change frequency routine */
	ldr	r4, =xlli_CLKREGS_PHYSICAL_BASE
	@  .... ..10 1... 11.. 1011 .010 ...0 0100
	@ldr	r0, =0x00033108		@ 104MHz (Run)	
	@ldr	r0, =0x01073110		@ 208MHz (Run)	
	@ldr	r0, =0x010b7210		@ 416MHz (Turbo)
	@ldr	r0, =0x028fb218		@ 624MHz (Turbo)
	@ldr	r0, =0x028fb21f		@ 806MHz (Turbo)
	ldr	r0, =0x028fb218		@ 624MHz (Turbo)
	str	r0, [r4, #xlli_ACCR_offset]
        @orr     r2,  r2,  #2
        @mcr     p14, 0, r2, c6, c0, 0

	bl	xlli_freq_change

@#ifdef CONFIG_CPU_MONAHANS_LV
        /* get processor id */
        mrc     p15, 0, r2, c0, c0, 0
	mov	r3, #0x69000000
	orr	r3, #0x00050000
	orr     r3, #0x00006800
        orr     r3, #0x00000025
        cmp     r2, r3
        ble     init_mem    /* MhnP B1 and below */

	mov     r3, #0x6800
        orr     r3, #0x0080
        cmp     r2, r3
	beq	init_mem    /* MhnL A0 */

        b	init_smem
@#endif

init_mem:
	/* setup GPIO8(SW1) as GPIO input for xlli_Dmem_initP1 */
	ldr	r0, =xlli_MFPR_PHYSICAL_BASE
	ldr	r1, =0x0000A040
	str     r1, [r0, #0x298]
	ldr	r1, [r0, #0x298]

	ldr	r0, =xlli_MFPR_PHYSICAL_BASE
	ldr	r1, =0x0000A041
	str     r1, [r0, #0x49c]

	ldr     r0, =xlli_GPIOREGS_PHYSICAL_BASE
	ldr     r1, [r0, #xlli_GPDR0_offset]
	bic     r1, r1, #0x100
	str     r1, [r0, #xlli_GPDR0_offset]

	/* Initialize Memory Controller */
        ldr     r4,     =xlli_DMEMREGS_PHYSICAL_BASE

        @
        @ It seems that the Boot ROM may be touching the DMEMC, enabling DDR.
        @ I will test to explcitly disable it before re-configuring it.
        @
        ldr    r0, [r4, #xlli_MDCNFG_offset]
        ldr    r1, =0x8000077F                      @ clear bit 30, DMC_EN
        and    r1, r1, r0


        str    r1, [r4,#xlli_MDCNFG_offset]
        ldr    r0, [r4, #xlli_MDCNFG_offset]
        ldr    r1, =0xC000077C                      @ clear bits 0,1.  Disable both partitions
        and    r1, r1, r0
        str    r1, [r4,#xlli_MDCNFG_offset]
        ldr    r0, [r4, #xlli_MDCNFG_offset]
        ldr    r1, =0x00000000                      @ clear all bits at this point.
        and    r1, r1, r0
        str    r1, [r4,#xlli_MDCNFG_offset]
        ldr    r0, [r4, #xlli_MDCNFG_offset]       @ final read-back

        @@@ end disable code @@@

        bl      xlli_Dmem_initP1         @   init DDR-SDRAM (Part 1)

        ldr     r0,  =xlli_DMEMREGS_PHYSICAL_BASE
        bl      xlli_Rcomp
	
        ldr     r0,  =xlli_DMEMREGS_PHYSICAL_BASE
        ldr     r1,  =xlli_SDRAM_PHYSICAL_BASE
        bl      xlli_Dmem_initP2         @ init DDR-SDRAM (Part 2)

init_smem:
        ldr     r0,  =0x4a000000
        bl      xlli_Smem_init

	@ load PSR register
	ldr	r1, =0x40f50004
	ldr	r0, [r1]
	tst     r0, #1

	bne     s2_reset

	bl	led1clear
	bl	led2clear
	bl 	normal_boot

s2_reset:
os_resume:	
	ldr	r1, =0x80000000
	ldr	pc, [r1]

	/* should never be here */
	blink	#1

.globl undefined_instruction
undefined_instruction:
	blink	#2

.globl software_interrupt
software_interrupt:
	blink	#3

.globl prefetch_abort
prefetch_abort:
	blink	#4

.globl data_abort
data_abort:
	blink	#5

.globl not_used
not_used:
	blink	#6

.globl irq
irq:
	blink	#7

.globl fiq
fiq:
	blink	#8

/* The functions below is platform-dependent. */
led1set:
	ldr		r2, =0x2
	ldr		r1, =0x40E00018 @ GPSR0
	str		r2, [r1]
	
	mov		pc, lr

led1clear:
	ldr		r2, =0x2
	ldr		r1, =0x40E00024 @ GPCR0
	str		r2, [r1]
	
	mov		pc, lr

led2set:
	ldr		r2, =0x10
	ldr		r1, =0x40E00018 @ GPSR 0
	str		r2, [r1]
	
	mov		pc, lr

led2clear:
	ldr		r2, =0x10
	ldr		r1, =0x40E00024 @ GPCR 0
	str		r2, [r1]
	
	mov		pc, lr

ledsetup:
	ldr		r2, =0x12
	ldr		r1, =0x40E0000C @ GPDR 0
	str		r2, [r1]
	mov		r3, lr

	bl		led1clear
	bl		led2clear
	mov		pc, r3

ledflash:
	mov		r2, lr
	bl		led1set
	wait		#0x400000
	bl		led1clear
	wait		#0x400000
	mov		pc, r2

endless_blink:
	bl		led1clear
	mov		r4, r3
0:
	bl		ledflash
	subs		r4, r4, #1
	bne		0b

	wait		#0x2000000
	b		endless_blink

init_i2c:
	/* setup MFPRs for I2C */
	ldr		r0, =0xa841
	ldr		r1, =MFP_I2C_SCL
	str		r0, [r1]
	ldr		r1, =MFP_I2C_SDA
	str		r0, [r1]

	/* enable i2c clock */
	ldr             r1, =0x41340010  /* D0CKEN_B */
	ldr             r0, [r1]
	orr             r0, r0, #0x10      /* enable i2c clock */
	str             r0, [r1]
 
	/* Initialize I2C Unit */	
	ldr		r1, =0x40301680 /* I2C register start address */
	mov		r0, #0x0
	str		r0, [r1, #0x20] /* ISAR */
	mov		r0, #0x4000     /* reset I2C */
	str		r0, [r1, #0x10] /* ICR */
	mov		r0, #0x60 	/* set IUE and SCLE */
	str		r0, [r1, #0x10]
	mov		pc, lr

writei2c:
	ldr		r3, =0x40301680 /* I2C register start address */

	/* Set slave I2C address */
	str		r4, [r3, #0x8] 	/* IDBR */

	/* send a start condition */
	ldr             r0, [r3, #0x10]
	@mov		r0, #0x9	/* START & TB */
	orr		r0, r0, #0x9    /* START & TB */
	ldr             r2, =0xffffeffd 
	and             r0, r0, r2      /* ~(STOP | ALDIE) */
	str		r0, [r3, #0x10] /* ICR */

	/* wait until tx buffer empty */
	mov		r8, #0x80000 	/* time out */
1:
	ldr		r2, [r3, #0x18] /* ISR */
	subs		r8, r8, #1
	moveq		pc, lr
	ands		r2, r2, #0x40	/* ITE in ISR */
	beq		1b
	str		r2, [r3, #0x18] /* clear status */

	/* send first byte(register address) */
	str		r5, [r3, #0x8] 	/* IDBR */
	ldr             r0, [r3, #0x10] /* read ICR */
@	mov		r0, r0, #0x69	/* IUE, SCLE, START & TB */
	ldr             r2, =0x1008
	orr		r0, r0, r2	/* ALDIE, TB */
	ldr             r2, =0xfffffffe
	and		r0, r0, r2	/* ~START */
	ldr             r2, =0xfffffffd 
	and             r0, r0, r2          /* ~STOP */
	str		r0, [r3, #0x10]
	
	/* wait until tx buffer empty */
	mov		r8, #0x80000 	/* time out */
2:
	ldr		r2, [r3, #0x18] /* ISR */
	subs		r8, r8, #1
	moveq		pc, lr
	ands		r2, r2, #0x40	/* ITE in ISR */
	beq		2b
	str		r2, [r3, #0x18] /* clear status */

	/* send second byte (register value) */
	str		r6, [r3, #0x8] 	/* IDBR */
	ldr             r0, [r3, #0x10] /* read ICR */
@	mov		r0, r0, #0x69	/* IUE, SCLE, START & TB */
	ldr             r2, =0x1008
	orr		r0, r0, r2	/* ALDIE, TB */
	ldr             r2, =0xfffffffe
	and		r0, r0, r2	/* ~START */
	orr             r0, r0, #0x2        /* STOP */
	str		r0, [r3, #0x10]

	/* wait until tx buffer empty */
	mov		r8, #0x80000 	/* time out */
3:
	ldr		r2, [r3, #0x18] /* ISR */
	subs		r8, r8, #1
	moveq		pc, lr
	ands		r2, r2, #0x40	/* ITE in ISR */
	beq		3b
	str		r2, [r3, #0x18] /* clear status */

	mov		pc, lr
	
