/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * (C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/**
**  FILENAME:       xllp_dfc_plat.h
**
**  PURPOSE: Implementation for Xllp DFC module
**           
**					 
**            
**                  
**
******************************************************************************/

#ifndef __XLLP_DFC_PLATFORM_H
#define __XLLP_DFC_PLATFORM_H

#include "xllp_defs.h"
// The Data Flash Controller Flash timing structure
// For NAND flash used on Zylonite board(Samsung K9K1216Q0C),
// user should use value at end of each row of following member
// ,bracketed.
typedef struct {
	XLLP_UINT32_T   tCH;  		// Enable signal hold time. [10]
	XLLP_UINT32_T   tCS;  		// Enable signal setup time.[0]
	XLLP_UINT32_T   tWH;  		// ND_nWE high duration.[20]
	XLLP_UINT32_T   tWP;  		// ND_nWE pulse time.[40]
	XLLP_UINT32_T   tRH;  		// ND_nRE high duration.[20]
	XLLP_UINT32_T   tRP;  		// ND_nRE pulse width.[40]
	XLLP_UINT32_T   tR;   		// ND_nWE high to ND_nRE low for read.[11123]
	XLLP_UINT32_T   tWHR; 		// ND_nWE high to ND_nRE low delay for status read.[110]
	XLLP_UINT32_T   tAR;  		// ND_ALE low to ND_nRE low delay.[10]
} XLLP_DFC_FLASH_TIMING_T;


// The Data Flash Controller Flash specification structure
// For NAND flash used on Zylonite board(Samsung K9K1216Q0C),
// user should use value at end of each row of following member
// ,bracketed.
typedef struct {
	XLLP_DFC_FLASH_TIMING_T	timing;		// NAND Flash timing
	XLLP_UINT32_T   enableBusArbiter; // Data flash bus arbiter enable (ND_ARB_EN).[1]
	XLLP_UINT32_T   pagePerBlock;  		// Pages per block (PG_PER_BLK). [32]
	XLLP_UINT32_T   rowAddrStart;  		//  Row address start position (RA_START).[0]
	XLLP_UINT32_T   byteCntReadID; 		// returned ID bytes(RD_ID_CNT).[4]
	XLLP_UINT32_T   dfcMode;   				// NAND, CARBONDALE, PIXLEY... (ND_MODE)[0]
	XLLP_UINT32_T   ncsx; 						// Chip select don't care bit (NCSX).[0]
	XLLP_UINT32_T   pageSize;  	   		// Page size in bytes (PAGE_SZ).[512]
	XLLP_UINT32_T   widthFlash; 			// Width of Flash memory (DWIDTH_M).[16]
	XLLP_UINT32_T   widthDFC; 	      // Width of flash controller(DWIDTH_C).[16]
	XLLP_UINT32_T   numBlocks;        // Number of physical blocks in Flash.[4096]
	XLLP_UINT32_T   chipID;		// NAND chip ID
	XLLP_UINT32_T	readProgCycles;   // Read Program address cycles
	XLLP_UINT32_T	unlockAddrCycles; // Block Unlock address cycles

	// command codes
	XLLP_UINT32_T   read1;			// Read. [0x0000]
	XLLP_UINT32_T   read2;      // unused, DFC don't support yet.[0x0050]
	XLLP_UINT32_T   program;    // two cycle command;[0x1080]
	XLLP_UINT32_T   readStatus; // [0x0070]
	XLLP_UINT32_T   readID;			// [0x0090]
	XLLP_UINT32_T   erase;		  // two cycle command.[0xD060]
	XLLP_UINT32_T   reset;		  // [0x00FF]
	XLLP_UINT32_T   lock;				// lock whole flash.[0x002A]
	XLLP_UINT32_T   unlock_low;		// set lower boundary of unlocking area [0x23]
	XLLP_UINT32_T   unlock_high;	// set upper boundary of unlocking area [0x24]
	XLLP_UINT32_T   lockStatus;	// read block lock status.[0x007A]
	XLLP_STATUS_T (* Addr2NDCB1)(XLLP_UINT16_T cmd, XLLP_UINT32_T addr, XLLP_UINT32_T *pNdcb1);
	XLLP_STATUS_T (* NDBBR2Addr)(XLLP_UINT16_T cmd, XLLP_UINT32_T ndbbr, XLLP_UINT32_T *pAddr);
} XLLP_DFC_FLASH_SPECS_T, *P_XLLP_DFC_FLASH_SPECS_T;

typedef enum {
	XLLP_DFC_FLASH_NULL = 0 ,
	XLLP_DFC_FLASH_Samsung512MbX16 = 1,
	XLLP_DFC_FLASH_Samsung2GbX8 = 2,
	XLLP_DFC_FLASH_Micron1GbX8 = 3,
	XLLP_DFC_FLASH_Micron1GbX16= 4,
	XLLP_DFC_FLASH_STM1GbX16= 5,
	XLLP_DFC_FLASH_STM2GbX16= 6,
	XLLP_DFC_FLASH_STM70nm1GbX16= 7,
	XLLP_DFC_FLASH_Toshiba2GbX16= 8,
	XLLP_DFC_FLASH_Micron2GbX16 = 9,
	XLLP_DFC_FLASH_Hynix2GbX16= 10,

	XLLP_DFC_FLASH_END,
}XLLP_DFC_FLASH_TYPE;

XLLP_STATUS_T XllpFlashDfcGetFlashInfo(XLLP_DFC_FLASH_TYPE type, XLLP_DFC_FLASH_SPECS_T **ppFlashInfo);

#define CACHE_READ
#endif
