//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this sample source code is subject to the terms of the Microsoft
// license agreement under which you licensed this sample source code. If
// you did not accept the terms of the license agreement, you are not
// authorized to use this sample source code. For the terms of the license,
// please see the license agreement between you and Microsoft or, if applicable,
// see the LICENSE.RTF on your install media or the root of your tools installation.
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES OR INDEMNITIES.
//
//
// (C) Copyright 2006 Marvell International Ltd.
// All Rights Reserved
//
#if (BSTD)
#include "xllp_pi2c.h"
#include "xllp_ost.h"
#include "xllp_pm.h"
#include "xllp_dvm.h"
#include "xllp_defs.h"

#define PMIC_ADDR 0x34

#define OVER2_ADDR 0x12
#define LDO2_EN (1 << 2)
#define LDO3_EN (1 << 3)
#define LDO4_EN (1 << 4)

#define OVSR2_ADDR 0x13
#define LDO2_OK (1 << 2)
#define LDO3_OK (1 << 3)
#define LDO4_OK (1 << 4)

#define BCCR_ADDR 0x89
#define BUC_EN (1 << 2)
#define NBUB (1 << 7)

#define ADTV2_ADDR 0x24
#define SDTV2_ADDR 0x2A
#define VCC1_ADDR 0x20
#define VOL_CHANGE_EN 0x33

#define ISRA_ADDR 0x88
#define L34VCR_ADDR 0x3A

void SetBflt(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu, XLLP_UINT32_T mv)
{
    XLLP_UINT8_T buf[2];
    XLLP_UINT32_T VccBflt[5] = {2800, 3000, 3200, 3400, 3600};
    int i = 0;
    
    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

    while (pSpmu->pmer);

    do
    {
        pSpmu->pmer = 0x0F;
    } while (pSpmu->pmer != 0x0F);

    XllpOstDelayMicroSeconds(pOst, 100);

    while (pSpmu->pvcr & XLLP_PVCR_VCSA);

    buf[0] = BCCR_ADDR;
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);

    for (i = 0; i < 5; i++)
    {
        if (mv == VccBflt[i])
        {
            break;
        }
    }

    buf[1] &= 0xC7;
    buf[1] |= (i + 1) << 3;
    
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);

    pSpmu->pmer = 0x0;
}
void EnBBatCharger(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu)
{
    XLLP_UINT8_T buf[2];
    
    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

    while (pSpmu->pmer);

    do
    {
        pSpmu->pmer = 0x0F;
    } while (pSpmu->pmer != 0x0F);

    XllpOstDelayMicroSeconds(pOst, 100);

    while (pSpmu->pvcr & XLLP_PVCR_VCSA);

    buf[0] = BCCR_ADDR;
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);

    buf[1] |= BUC_EN;
    
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);

    pSpmu->pmer = 0x0;
}

void DisBuBatt(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu)
{
    XLLP_UINT8_T buf[2];
    
    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

    while (pSpmu->pmer);

    do
    {
        pSpmu->pmer = 0x0F;
    } while (pSpmu->pmer != 0x0F);

    XllpOstDelayMicroSeconds(pOst, 100);

    while (pSpmu->pvcr & XLLP_PVCR_VCSA);

    buf[0] = BCCR_ADDR;
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);

    buf[1] |= NBUB;
    
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);

    pSpmu->pmer = 0x0;
}

void PowerOnUsb(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu, XLLP_BOOL_T isOn)
{
    XLLP_UINT8_T buf[2];
    
    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

    while (pSpmu->pmer);

    do
    {
        pSpmu->pmer = 0x0F;
    } while (pSpmu->pmer != 0x0F);

    XllpOstDelayMicroSeconds(pOst, 100);

    while (pSpmu->pvcr & XLLP_PVCR_VCSA);

    buf[0] = OVER2_ADDR;
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);

    if (isOn == XLLP_TRUE)
        buf[1] |= LDO2_EN;
    else
        buf[1] &= ~LDO2_EN;
    
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);

    if (isOn == XLLP_TRUE)
    {
        do
        {
            buf[0] = OVSR2_ADDR;
            XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
            XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);
        } while (!(buf[1] & LDO2_OK));
    }

    pSpmu->pmer = 0x0;
}

void PowerOnCodec(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu, XLLP_BOOL_T isOn)
{
    XLLP_UINT8_T buf[2];
    buf[0] = OVER2_ADDR;
    
    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

    while (pSpmu->pmer);

    do
    {
        pSpmu->pmer = 0x0F;
    } while (pSpmu->pmer != 0x0F);

    XllpOstDelayMicroSeconds(pOst, 100);

    while (pSpmu->pvcr & XLLP_PVCR_VCSA);

    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);

    if (isOn == XLLP_TRUE)
        buf[1] |= LDO3_EN;
    else
        buf[1] &= ~LDO3_EN;
    
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);

    if (isOn == XLLP_TRUE)
    {
        do
        {
            buf[0] = OVSR2_ADDR;
            XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
            XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);
        } while (!(buf[1] & LDO3_OK));
    }

    pSpmu->pmer = 0x0;
}

void PowerOnCI(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu, XLLP_BOOL_T isOn)
{
    XLLP_UINT8_T buf[2];
    buf[0] = OVER2_ADDR;
    
    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

    while (pSpmu->pmer);

    do
    {
        pSpmu->pmer = 0x0F;
    } while (pSpmu->pmer != 0x0F);

    XllpOstDelayMicroSeconds(pOst, 100);

    while (pSpmu->pvcr & XLLP_PVCR_VCSA);

    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);

    if (isOn == XLLP_TRUE)
        buf[1] |= LDO4_EN;
    else
        buf[1] &= ~LDO4_EN;
    
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);

    if (isOn == XLLP_TRUE)
    {
        do
        {
            buf[0] = OVSR2_ADDR;
            XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
            XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);
        } while (!(buf[1] & LDO4_OK));
    }

    pSpmu->pmer = 0x0;
}

void close_watchdog(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu, XLLP_BOOL_T isOn)
{

    XLLP_UINT8_T buf[2];
    buf[0] = OVER2_ADDR;
    
    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

    buf[0] = 0x0b;
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);

    XllpOstDelayMicroSeconds(pOst, 10);
    printf("The init value is: %x \n", buf[1]);
    buf[1] = 0x59;//buf[1]&0xdf;
    printf("The value is set to %x \n", buf[1]);
    
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);
    XllpOstDelayMicroSeconds(pOst, 10);
    
    buf[0] = 0x0b;
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);

    XllpOstDelayMicroSeconds(pOst, 10);
    printf("The init value is: %x \n", buf[1]);

}

void PowerOnLCD(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu, XLLP_BOOL_T isOn)
{
    XLLP_UINT8_T buf[2];
    buf[0] = OVER2_ADDR;
    
    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

	/*
    while (pSpmu->pmer);

    do
    {
        pSpmu->pmer = 0x0F;
    } while (pSpmu->pmer != 0x0F);

    XllpOstDelayMicroSeconds(pOst, 100);

    while (pSpmu->pvcr & XLLP_PVCR_VCSA);
	*/

	// set voltage
    buf[0] = L34VCR_ADDR;
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);

    XllpOstDelayMicroSeconds(pOst, 10);
	printf("The init value is: %x \n", buf[1]);
    buf[1] = 0xFC;
	printf("The value is set to %x \n", buf[1]);
    
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);
    XllpOstDelayMicroSeconds(pOst, 10);

    buf[0] = L34VCR_ADDR;
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);
    XllpOstDelayMicroSeconds(pOst, 10);

	printf("To verify the value:%x \n", buf[1]);
	
	// to enable
	printf("Prepare to enable \n");
    buf[0] = OVER2_ADDR;
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);
	printf("OVER2 VALUE init :%x \n", buf[1]);

    if (isOn == XLLP_TRUE)
        buf[1] |= LDO4_EN;
    else
        buf[1] &= ~LDO4_EN;
    
    XllpOstDelayMicroSeconds(pOst, 10);
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);
    XllpOstDelayMicroSeconds(pOst, 10);

    buf[0] = OVER2_ADDR;
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);
	printf("OVER2 is set to :%x \n", buf[1]);

    if (isOn == XLLP_TRUE)
    {
        do
        {
            buf[0] = OVSR2_ADDR;
            XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
            XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);
        } while (!(buf[1] & LDO4_OK));
    }

    pSpmu->pmer = 0x0;
  
}

void PowerOnCamera(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu, XLLP_BOOL_T isOn)
{
    XLLP_UINT8_T buf[2];
    
    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

    while (pSpmu->pmer);

    do
    {
        pSpmu->pmer = 0x0F;
    } while (pSpmu->pmer != 0x0F);

    XllpOstDelayMicroSeconds(pOst, 100);

    while (pSpmu->pvcr & XLLP_PVCR_VCSA);

    buf[0] = OVER2_ADDR;
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);

    if (isOn == XLLP_TRUE)
        buf[1] |= LDO2_EN;
    else
        buf[1] &= ~LDO2_EN;
    
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);

    if (isOn == XLLP_TRUE)
    {
        do
        {
            buf[0] = OVSR2_ADDR;
            XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
            XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);
        } while (!(buf[1] & LDO2_OK));
    }

    printf("camera OVSR2_ADDR is set to :0x%x \n", buf[1]);
    pSpmu->pmer = 0x0;
}

void PowerOnMMC(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu, XLLP_BOOL_T isOn)
{
    XLLP_UINT8_T buf[2];
    
    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

    while (pSpmu->pmer);

    do
    {
        pSpmu->pmer = 0x0F;
    } while (pSpmu->pmer != 0x0F);

    XllpOstDelayMicroSeconds(pOst, 100);

    while (pSpmu->pvcr & XLLP_PVCR_VCSA);
/*   
    buf[0] = L34VCR_ADDR;
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);
    
    XllpOstDelayMicroSeconds(pOst, 10);
	printf("The init value is: %x \n", buf[1]);
    buf[1] = 0x0c;
	printf("The value is set to %x \n", buf[1]);
    
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);
    XllpOstDelayMicroSeconds(pOst, 10);
*/
    buf[0] = OVER2_ADDR;
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);

    if (isOn == XLLP_TRUE)
        buf[1] |= LDO3_EN;
    else
        buf[1] &= ~LDO3_EN;
    
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);

    if (isOn == XLLP_TRUE)
    {
        do
        {
            buf[0] = OVSR2_ADDR;
            XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
            XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);
        } while (!(buf[1] & LDO3_OK));
    }

    printf("MMC OVSR2_ADDR is set to :0x%x \n", buf[1]);
    pSpmu->pmer = 0x0;
    SetBflt( pI2C, pOst, pSpmu, 3000);
}


XLLP_UINT32_T GetPmicWakeupReason(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu)
{
    XLLP_UINT8_T buf[2];
    buf[0] = ISRA_ADDR;
    
    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

    while (pSpmu->pmer);

    do
    {
        pSpmu->pmer = 0x0F;
    } while (pSpmu->pmer != 0x0F);

    XllpOstDelayMicroSeconds(pOst, 100);

    while (pSpmu->pvcr & XLLP_PVCR_VCSA);
    
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);

    pSpmu->pmer = 0x0;

    return buf[1];
}

XLLP_UINT32_T GetVccCore(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu)
{
    XLLP_UINT8_T buf[2];
    XLLP_UINT32_T vcc[32] = {725, 750, 775, 800, 825, 850, 875, 900,
                             925, 950, 975, 1000, 1025, 1050, 1075, 1100,
                             1125, 1150, 1175, 1200, 1225, 1250, 1275, 1300,
                             1325, 1350, 1375, 1400, 1425, 1450, 1475, 1500};
    
    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

    while (pSpmu->pmer);

    do
    {
        pSpmu->pmer = 0x0F;
    } while (pSpmu->pmer != 0x0F);

    XllpOstDelayMicroSeconds(pOst, 100);

    while (pSpmu->pvcr & XLLP_PVCR_VCSA);

    buf[0] = ADTV2_ADDR;
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);

    pSpmu->pmer = 0x0;

    return vcc[buf[1]];
}

void SetVccCore(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu, XLLP_UINT32_T mv)
{
    XLLP_UINT8_T buf[2];
    XLLP_UINT32_T vcc[32] = {725, 750, 775, 800, 825, 850, 875, 900,
                             925, 950, 975, 1000, 1025, 1050, 1075, 1100,
                             1125, 1150, 1175, 1200, 1225, 1250, 1275, 1300,
                             1325, 1350, 1375, 1400, 1425, 1450, 1475, 1500};
    int i = 0;
    
    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

    while (pSpmu->pmer);

    do
    {
        pSpmu->pmer = 0x0F;
    } while (pSpmu->pmer != 0x0F);

    // why not query VCSA directly?
    // is there any difference? 
    // maybe access the timmer reg and pvcr reg is diffrerent.

    XllpOstDelayMicroSeconds(pOst, 100);

    while (pSpmu->pvcr & XLLP_PVCR_VCSA);

    buf[0] = ADTV2_ADDR;
    for (i = 0; i < 32; i++)
    {
        if (mv == vcc[i])
            buf[1] = i;
    }

    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);

    pSpmu->pmer = 0x0;
}

XLLP_UINT32_T GetVccSram(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu)
{
    XLLP_UINT8_T buf[2];
    XLLP_UINT32_T vcc[32] = {0, 0, 0, 0, 0, 850, 875, 900,
                             925, 950, 975, 1000, 1025, 1050, 1075, 1100,
                             1125, 1150, 1175, 1200, 1225, 1250, 1275, 1300,
                             1325, 1350, 1375, 1400, 1425, 1450, 1475, 1500};

    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

    while (pSpmu->pmer);

    do
    {
        pSpmu->pmer = 0x0F;
    } while (pSpmu->pmer != 0x0F);

    XllpOstDelayMicroSeconds(pOst, 100);

    while (pSpmu->pvcr & XLLP_PVCR_VCSA);

    buf[0] = SDTV2_ADDR;    
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_TRUE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);

    pSpmu->pmer = 0x0;

    return vcc[buf[1]];
}

void SetVccSram(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu, XLLP_UINT32_T mv)
{
    XLLP_UINT8_T buf[2];
    XLLP_UINT32_T vcc[32] = {0, 0, 0, 0, 0, 850, 875, 900,
                             925, 950, 975, 1000, 1025, 1050, 1075, 1100,
                             1125, 1150, 1175, 1200, 1225, 1250, 1275, 1300,
                             1325, 1350, 1375, 1400, 1425, 1450, 1475, 1500};
    int i = 0;
    
    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

    while (pSpmu->pmer);

    do
    {
        pSpmu->pmer = 0x0F;
    } while (pSpmu->pmer != 0x0F);

    XllpOstDelayMicroSeconds(pOst, 100);

    while (pSpmu->pvcr & XLLP_PVCR_VCSA);

    buf[0] = SDTV2_ADDR;
    for (i = 0; i < 32; i++)
    {
        if (mv == vcc[i])
            buf[1] = i;
    }
    
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);

    pSpmu->pmer = 0x0;
}

void EnVolChange(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu)
{
    XLLP_UINT8_T buf[2];
    
    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

    while (pSpmu->pmer);

    do
    {
        pSpmu->pmer = 0x0F;
    } while (pSpmu->pmer != 0x0F);

    XllpOstDelayMicroSeconds(pOst, 100);

    while (pSpmu->pvcr & XLLP_PVCR_VCSA);

    buf[0] = VCC1_ADDR;
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, &buf[0], 1, XLLP_FALSE);
    XllpPI2CRead(pI2C, pOst, PMIC_ADDR, &buf[1], 1, XLLP_TRUE);

    buf[1] |= VOL_CHANGE_EN;
    
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);

    pSpmu->pmer = 0x0;
}

void SetVccCombo(P_XLLP_PI2C_T pI2C, P_XLLP_OST_T pOst, P_XLLP_PM_SPMU_T pSpmu, XLLP_UINT32_T Coremv, XLLP_UINT32_T SRAMmv)
{
    XLLP_UINT8_T buf[2];
    XLLP_UINT32_T vcc[32] = {0, 0, 0, 0, 0, 850, 875, 900,
                             925, 950, 975, 1000, 1025, 1050, 1075, 1100,
                             1125, 1150, 1175, 1200, 1225, 1250, 1275, 1300,
                             1325, 1350, 1375, 1400, 1425, 1450, 1475, 1500};
    int i = 0;
    
    XllpPI2cInit(pI2C, XLLP_PI2C_SLAVE_ID);

    while (pSpmu->pmer);

    do
    {
        pSpmu->pmer = 0x0F;
    } while (pSpmu->pmer != 0x0F);

    XllpOstDelayMicroSeconds(pOst, 100);

    while (pSpmu->pvcr & XLLP_PVCR_VCSA);

    buf[0] = SDTV2_ADDR;
    for (i = 0; i < 32; i++)
    {
        if (SRAMmv < vcc[i])
            buf[1] = i;
    }
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);

    buf[0] = ADTV2_ADDR;
    for (i = 0; i < 32; i++)
    {
        if (Coremv == vcc[i])
            buf[1] = i;
    }
    XllpPI2CWrite(pI2C, pOst, PMIC_ADDR, buf, 2, XLLP_TRUE);

    pSpmu->pmer = 0x0;
}
#endif

