/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#include "xllp_pccardsocket.h"
#include "xllp_true_ide_plat.h"

#include    "xllp_serialization.h"
#include    "xllp_mfp.h"

static XLLP_UINT32_T XllpPccardMfpList[]={
								XLLP_MFP_NPIOR_GPIO_OFFSET,
								XLLP_MFP_NPIOW_GPIO_OFFSET,
								XLLP_MFP_NPWAIT_GPIO_OFFSET,
								XLLP_MFP_NIOIS16_GPIO_OFFSET,
								XLLP_MFP_CFCD_GPIO_OFFSET,
								XLLP_MFP_CFNIRQ_GPIO_OFFSET,
								XLLP_MFP_PIN_EOLIST_MARKER
								};//end of XllpPccardMfpList
								
static XLLP_MFP_ALT_FN_T XllpPccardAfList []={
								XLLP_MFP_NPIOR_GPIO_AF,
								XLLP_MFP_NPIOW_GPIO_AF,
								XLLP_MFP_NPWAIT_GPIO_AF,
								XLLP_MFP_NIOIS16_GPIO_AF,
								XLLP_MFP_CFCD_GPIO_AF,
								XLLP_MFP_CFNIRQ_GPIO_AF
								};//end of XllpPccardAfList

static XLLP_MFP_DRIVE_STRENGTH_T XllpPccardDsList[]={
								XLLP_MFP_DEFAULT_DS,
								XLLP_MFP_DEFAULT_DS, 
								XLLP_MFP_DEFAULT_DS, 
								XLLP_MFP_DEFAULT_DS,
								XLLP_MFP_DEFAULT_DS, 
								XLLP_MFP_DEFAULT_DS
								};//end of XllpPccardDsList

static XLLP_MFP_LPM_OUTPUT_T XllpPccardLpmOutputList[]={
								XLLP_MFP_LPMO_DRIVE_HIGH,       //  nPIOR
								XLLP_MFP_LPMO_DRIVE_HIGH,       //  nPIOW
								XLLP_MFP_LPMO_PULL_HIGH,       //  nPWAIT
								XLLP_MFP_LPMO_PULL_HIGH,       //  nIOIS16
								XLLP_MFP_LPMO_PULL_HIGH,       //  CFDC
								XLLP_MFP_LPMO_PULL_HIGH		//  CFnIRQ
								};//end of xllp_keypad_lpm_output_list


/******************************************************************************

  Function Name: XllpPCCardConfigureGPIOs

  Description: Configures BULVERDE's GPIOs for the PC Card interface

  Global Registers Modified: 

  Input Arguments:

  Output Arguments:

  Return Value: 


*******************************************************************************/
XLLP_STATUS_T XllpPCCardConfigureGPIOs(XLLP_PCCARDSOCKET_T *pstrSocketHandle)
{
	XLLP_STATUS_T status = XLLP_STATUS_SUCCESS;
	status = XllpMfpResourceManager_List(pstrSocketHandle->FPRmDb,
                                      XllpPccardMfpList,
                                      XLLP_MFP_RM_ID_XLLP_PCCARD,
                                      XLLP_SET);
  	if (status != XLLP_STATUS_SUCCESS){
    		return status;
 	 }

	XllpLock(XLLP_RESOURCE_MFPR);

  	status = XllpMfpSetAfDs_List(pstrSocketHandle->PBase,
                                XllpPccardMfpList,
                                XllpPccardAfList,
                                XllpPccardDsList);
 	 if (status != XLLP_STATUS_SUCCESS){
	 	 XllpUnlock(XLLP_RESOURCE_MFPR);
    		return status;
  	}

 	 status = XllpMfpConfigureLpmOutputLevel_List (pstrSocketHandle->PBase,
               					 XllpPccardMfpList,
					                XllpPccardLpmOutputList);
	
	if ( XLLP_STATUS_SUCCESS != status) {
        	XllpUnlock(XLLP_RESOURCE_MFPR);
		return status ;	
	}
	//pull up XLLP_MFP_NPWAIT_GPIO_OFFSET
	XllpMfpConfigurePullUpDown(pstrSocketHandle->PBase, 
									XLLP_MFP_NPWAIT_GPIO_OFFSET,
									XLLP_MFP_PULL_UP);
	XllpMfpActivatePullUpDown(pstrSocketHandle->PBase,
 									XLLP_MFP_NPWAIT_GPIO_OFFSET,
									XLLP_ON);

 	//pull up XLLP_MFP_NIOIS16_GPIO_OFFSET
	XllpMfpConfigurePullUpDown(pstrSocketHandle->PBase, 
									XLLP_MFP_NIOIS16_GPIO_OFFSET,
									XLLP_MFP_PULL_UP);
	XllpMfpActivatePullUpDown(pstrSocketHandle->PBase,
									XLLP_MFP_NIOIS16_GPIO_OFFSET, 
									XLLP_ON);
					
	//pull up XLLP_MFP_CFCD_GPIO_OFFSET
	XllpMfpConfigurePullUpDown(pstrSocketHandle->PBase, 
									XLLP_MFP_CFCD_GPIO_OFFSET,
									XLLP_MFP_PULL_UP);
	XllpMfpActivatePullUpDown(pstrSocketHandle->PBase,
									XLLP_MFP_CFCD_GPIO_OFFSET, 
									XLLP_ON);

	//pull up XLLP_MFP_CFNIRQ_GPIO_OFFSET
	XllpMfpConfigurePullUpDown(pstrSocketHandle->PBase, 
									XLLP_MFP_CFNIRQ_GPIO_OFFSET,
									XLLP_MFP_PULL_UP);
 	XllpMfpActivatePullUpDown(pstrSocketHandle->PBase,
									XLLP_MFP_CFCD_GPIO_OFFSET, 
									XLLP_ON);

	XllpGpioSetDirection(pstrSocketHandle->GpioRegsHandle, 
                       XLLP_GPIO_CFCD, 
                       XLLP_GPIO_DIRECTION_IN);

	XllpGpioSetDirection(pstrSocketHandle->GpioRegsHandle, 
                       XLLP_GPIO_NIRQ, 
                       XLLP_GPIO_DIRECTION_IN);
	
	XllpGpioSetFallingEdgeDetectEnable(pstrSocketHandle->GpioRegsHandle,
							XLLP_GPIO_CFCD, 
							XLLP_ON);
	XllpGpioSetRisingEdgeDetectEnable(pstrSocketHandle->GpioRegsHandle,
							XLLP_GPIO_CFCD, 
							XLLP_ON);
	XllpGpioSetFallingEdgeDetectEnable(pstrSocketHandle->GpioRegsHandle,
							XLLP_GPIO_NIRQ, 
							XLLP_ON);
	
	XllpUnlock(XLLP_RESOURCE_MFPR);
	return XLLP_STATUS_SUCCESS;

} //end XllpPCCardConfigureGPIOs()


/******************************************************************************

  Function Name: XllpPCCardHWSetup

  Description: call XllpPCCardConfigureGPIOs() to set GPIO


  Global Registers Modified: 
	NONE

  Input Arguments:
	XLLP_PCCARDSOCKET_T *pstrSocketHandle: Pointer to the global PC Card handle

  Output Arguments:
	NONE

  Return Value:


*******************************************************************************/
XLLP_STATUS_T XllpPCCardHWSetup(XLLP_PCCARDSOCKET_T *pstrSocketHandle)
{
	XLLP_STATUS_T re = XLLP_STATUS_SUCCESS;
	XLLP_CF_SOCKET_MODE_T mode= XLLP_CF_IO_MEM_MOD;
	mode = xllp_get_cf_socket_mode(pstrSocketHandle->hd_switch_addr);

	if ( XLLP_CF_IO_MEM_MOD != mode ) {
		return XLLP_STATUS_INVALID_STATE;
	}
	

	re =	XllpPCCardConfigureGPIOs(pstrSocketHandle);
	pstrSocketHandle->pStaticMemCtl->casdrcfg_p = 0x0032080c;
	return re;
} //end XllpPCCardHWSetup()




/******************************************************************************

  Function Name: XllpPCCardGetSocketState

  Description: 

  Global Registers Modified: 

  Input Arguments:

  Output Arguments:

  Return Value: 


*******************************************************************************/
XLLP_STATUS_T XllpPCCardGetSocketState(XLLP_PCCARDSOCKET_T *pstrSocketHandle)
{

	XLLP_STATUS_T ReturnValue = XLLP_STATUS_SUCCESS;
	XLLP_LEVEL_T ReturnLevel = XLLP_LO;
	//
	//Check the CD status bit[5] of the Socket 0 Status Register. If set, it indicates
	//that a card is either not present or is not properly inserted in the socket. If clear, 
	//it indicates that a card is present in the socket.
	//
	XllpGpioGetLevel(pstrSocketHandle->GpioRegsHandle, XLLP_GPIO_CFCD, &ReturnLevel);
	if(ReturnLevel == XLLP_HI)
	{
		pstrSocketHandle->pstrPCCardSocketState->blSocket0CDState = XLLP_FALSE;
	}
	else
	{
		pstrSocketHandle->pstrPCCardSocketState->blSocket0CDState = XLLP_TRUE;
	}
	//
	//BVD1 is assert high as BVD1 is not supported
	//
	pstrSocketHandle->pstrPCCardSocketState->blSocket0BVD1State = XLLP_TRUE;
		
	//
	//BVD2 is assert high as BVD1 is not supported
	//
	pstrSocketHandle->pstrPCCardSocketState->blSocket0BVD2State = XLLP_TRUE;
	//
	//Check the IREQ status bit[10] of the Socket 0 Status Register. If set, then it
	//indicates that the card is READY (i.e. no interrupt request is pending). If clear,
	//it indicates that the card is BUSY, pending an interrupt request.
	//
	XllpGpioGetLevel(pstrSocketHandle->GpioRegsHandle, XLLP_GPIO_NIRQ, &ReturnLevel);
	if(ReturnLevel == XLLP_HI)
	{
		pstrSocketHandle->pstrPCCardSocketState->blSocket0IREQState = XLLP_TRUE;
	}
	else
	{
		pstrSocketHandle->pstrPCCardSocketState->blSocket0IREQState = XLLP_FALSE;
	}

	return ReturnValue;

} //end XllpPCCardGetSocketState()

/******************************************************************************

  Function Name: XllpPCCardGetVoltageSetting

  Description: 

  Global Registers Modified: 

  Input Arguments:

  Output Arguments:

  Return Value: 


*******************************************************************************/
XLLP_STATUS_T XllpPCCardGetVoltageSetting(XLLP_PCCARDSOCKET_T *pstrSocketHandle,
                                             XLLP_UINT32_T *puiCardVoltage)
{
	*puiCardVoltage = XLLP_PCCARD_3_30VOLTS ;
	return XLLP_STATUS_SUCCESS;
} //end XllpPCCardGetVoltageSetting()



/******************************************************************************

  Function Name: XllpPCCardSetExpMemTiming

  Description: 

  Global Registers Modified: 

  Input Arguments:

  Output Arguments:

  Return Value: 


*******************************************************************************/
XLLP_STATUS_T XllpPCCardSetExpMemTiming(XLLP_PCCARDSOCKET_T *pstrSocketHandle )
{

    XLLP_STATUS_T ReturnValue = XLLP_STATUS_SUCCESS;

	//
    //Check the validity of the input arguments to the function
	//
    if(pstrSocketHandle == NULL)
    {
        ReturnValue = XLLP_STATUS_PCCARD_FAILURE;  
    }

	if(ReturnValue != XLLP_STATUS_PCCARD_FAILURE)
	{
		pstrSocketHandle->pStaticMemCtl->mcatt0 = pstrSocketHandle->trCardTimingHandle->mcatt0;
		pstrSocketHandle->pStaticMemCtl->mcmem0 = pstrSocketHandle->trCardTimingHandle->mcmem0;
		pstrSocketHandle->pStaticMemCtl->mcio0 = pstrSocketHandle->trCardTimingHandle->mcio0;
	}

	return ReturnValue;

} //end XllpPCCardSetExpMemTiming()



/******************************************************************************

  Function Name: XllpPCCardGetExpMemTiming

  Description: 

  Global Registers Modified: 

  Input Arguments:

  Output Arguments:

  Return Value: 


*******************************************************************************/
XLLP_STATUS_T XllpPCCardGetExpMemTiming(XLLP_PCCARDSOCKET_T *pstrSocketHandle,
                                        XLLP_CARDTIMING_T *pstrCardTimingHandle)
{
    XLLP_STATUS_T ReturnValue = XLLP_STATUS_SUCCESS;

	//
    //Check the validity of the input arguments to the function
	//
    if((pstrSocketHandle == NULL) ||
       (pstrCardTimingHandle == NULL))
    {
        ReturnValue = XLLP_STATUS_PCCARD_FAILURE;  
    }

	if(ReturnValue != XLLP_STATUS_PCCARD_FAILURE)
	{
		pstrCardTimingHandle->mcatt0 = pstrSocketHandle->pStaticMemCtl->mcatt0;
		pstrCardTimingHandle->mcmem0 = pstrSocketHandle->pStaticMemCtl->mcmem0;
		pstrCardTimingHandle->mcio0 = pstrSocketHandle->pStaticMemCtl->mcio0;
		
	}

	return ReturnValue;

} //end XllpPCCardGetExpMemTiming()






