//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this sample source code is subject to the terms of the Microsoft
// license agreement under which you licensed this sample source code. If
// you did not accept the terms of the license agreement, you are not
// authorized to use this sample source code. For the terms of the license,
// please see the license agreement between you and Microsoft or, if applicable,
// see the LICENSE.RTF on your install media or the root of your tools installation.
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES OR INDEMNITIES.
//
//
// (C) Copyright 2006 Marvell International Ltd.
// All Rights Reserved
//
/* 
** INTEL CONFIDENTIAL
** Copyright 2000-2006 Intel Corporation All Rights Reserved.
**
** The source code contained or described herein and all documents
** related to the source code (Material) are owned by Intel Corporation
** or its suppliers or licensors.  Title to the Material remains with
** Intel Corporation or its suppliers and licensors. The Material contains
** trade secrets and proprietary and confidential information of Intel
** or its suppliers and licensors. The Material is protected by worldwide
** copyright and trade secret laws and treaty provisions. No part of the
** Material may be used, copied, reproduced, modified, published, uploaded,
** posted, transmitted, distributed, or disclosed in any way without Intel�s
** prior express written permission.
**
** No license under any patent, copyright, trade secret or other intellectual
** property right is granted to or conferred upon you by disclosure or
** delivery of the Materials, either expressly, by implication, inducement,
** estoppel or otherwise. Any license under such intellectual property rights
** must be express and approved by Intel in writing.
*/
/**
**  FILENAME:       xllp_pi2c.c
**
**  PURPOSE:        Xllp pi2c file
**
**
******************************************************************************/
#if (BSTD)
#include "xllp_pi2c.h"
#include "xllp_defs.h"
#include "xllp_pi2c_proc.h"
    
/* 
 * Initialization to use PWRI2C bus
 *
 * PARAMETERS:
 *	P_XLLP_PI2C_T PI2C_regs structure for i2c regs
 *	P_XLLP_CLKMGR_T clkaddr - address of clkmanager
 *      XLLP_UINT32_T dev_id - Default slave device id for PXA
 *
 * RETURNS: XLLP_STATUS_WRONG_PARAMETER
 *			XLLP_STATUS_SUCCESS			
 */ 
XLLP_STATUS_T XllpPI2cInit(P_XLLP_PI2C_T PI2C_regs, XLLP_UINT32_T dev_id) 
{
	
#ifdef XLLP_DEBUG_PARAM_CHECK
	if (NULL == PI2C_regs)
		return XLLP_STATUS_WRONG_PARAMETER;
	
#endif	
	PI2C_regs->XLLP_ICR = 0;
	/* Setup PI2C slave address */ 
	PI2C_regs->XLLP_ISAR = dev_id;
	PI2C_regs->XLLP_ICR = XLLP_ICR_SCLEA | XLLP_ICR_FM;
	PI2C_regs->XLLP_ICR |= XLLP_ICR_UIE;
	return (XLLP_STATUS_SUCCESS);
}//end XllpPi2cinit


/* 
 * Wait for Receive empty status
 *
 * RETURNS: XLLP_STATUS_SUCCESS success
 *          XLLP_STATUS_TIME_OUT time out
 */ 
XLLP_STATUS_T XllpPI2cRxFull(P_XLLP_PI2C_T PI2C_regs, XLLP_OST_T * pOSTRegs, XLLP_INT32_T timeout) 
{
	XLLP_UINT32_T temp;
    XLLP_UINT32_T    ticks;
    XLLP_UINT32_T microseconds = timeout * 200;
    XLLP_UINT32_T    start_time;
    start_time = pOSTRegs->oscr0;
    ticks = microseconds * XLLP_OST_TICKS_US;  // approx. 3 ticks per microsecond. 

	while (1)
		{
            XLLP_UINT32_T time = pOSTRegs->oscr0 - start_time;
		temp = PI2C_regs->XLLP_ISR;
		if ((temp & XLLP_ISR_IRF) == XLLP_ISR_IRF)
			{
			PI2C_regs->XLLP_ISR = temp | XLLP_ISR_IRF;
			return XLLP_STATUS_SUCCESS;
			}
		
                if (time > ticks)
                        return XLLP_STATUS_TIME_OUT;
		    // delay 0.2 ms here
		    //XllpOstDelayMicroSeconds(pOSTRegs, 200);
		}
	return XLLP_STATUS_TIME_OUT;
}


/* Wait for transmit empty status
 *
 * RETURNS:
 *	XLLP_STATUS_SUCCESS
 *	XLLP_STATUS_TIME_OUT
 */ 
XLLP_STATUS_T XllpPI2cTxEmpty(P_XLLP_PI2C_T PI2C_regs, XLLP_OST_T * pOSTRegs, XLLP_INT32_T timeout) 
{
	XLLP_UINT32_T temp;
    XLLP_UINT32_T    ticks;
    XLLP_UINT32_T microseconds = timeout * 200;
    XLLP_UINT32_T    start_time;
    start_time = pOSTRegs->oscr0;
    ticks = microseconds * XLLP_OST_TICKS_US;  // approx. 3 ticks per microsecond. 

	while (1)
		{
                XLLP_UINT32_T time = pOSTRegs->oscr0 - start_time;
		temp = PI2C_regs->XLLP_ISR;
		if ((temp & XLLP_ISR_ITE) == XLLP_ISR_ITE)
			{
			PI2C_regs->XLLP_ISR = temp | XLLP_ISR_ITE;
			if ((temp & XLLP_ISR_ALD) == XLLP_ISR_ALD)
				{
				PI2C_regs->XLLP_ISR |= XLLP_ISR_ALD;
				}
			return XLLP_STATUS_SUCCESS;
			}
		//XllpOstDelayMicroSeconds(pOSTRegs, 200);
                if (time > ticks)
                     return XLLP_STATUS_TIME_OUT;
		}
	return XLLP_STATUS_TIME_OUT;
}//end XllpPI2cIxEmpty

XLLP_STATUS_T XllpPI2CWrite(P_XLLP_PI2C_T PI2C_regs, 
                            XLLP_OST_T * pOSTRegs,
            			    XLLP_UINT8_T slaveAddr,
            			    const XLLP_UINT8_T * bytesBuf,
            			    XLLP_UINT32_T bytesCount, 
            			    XLLP_BOOL_T bSendStop) 
{
	XLLP_UINT32_T reg;
	
#ifdef XLLP_DEBUG_PARAM_CHECK
	if (NULL == PI2C_regs || NULL == pOSTRegs)
		return XLLP_STATUS_WRONG_PARAMETER;
	
#endif
	PI2C_regs->XLLP_IDBR = (slaveAddr << 1) & ~XLLP_IDBR_MODE;
	reg = PI2C_regs->XLLP_ICR;
	reg |= (XLLP_ICR_START | XLLP_ICR_TB);
	reg &= ~(XLLP_ICR_STOP | XLLP_ICR_ALDIE);
	PI2C_regs->XLLP_ICR = reg;
	if (XllpPI2cTxEmpty(PI2C_regs, pOSTRegs, 20) == XLLP_STATUS_TIME_OUT)
		{
		return XLLP_STATUS_TIME_OUT;
		}
	
	    // Send all the bytes
	    while (bytesCount--)
		{
		PI2C_regs->XLLP_IDBR = (XLLP_UINT32_T) (*bytesBuf++);
		reg = PI2C_regs->XLLP_ICR;
		reg &= ~XLLP_ICR_START;
		reg |= (XLLP_ICR_ALDIE | XLLP_ICR_TB);
		if ((bytesCount == 0) && bSendStop)
			reg |= XLLP_ICR_STOP;
		
		else
			reg &= ~XLLP_ICR_STOP;
		PI2C_regs->XLLP_ICR = reg;
		if (XllpPI2cTxEmpty(PI2C_regs, pOSTRegs, 250) == XLLP_STATUS_TIME_OUT)
			{
			return XLLP_STATUS_TIME_OUT;
			}
		}
	
	    // Clear the STOP bit always
	    PI2C_regs->XLLP_ICR &= ~XLLP_ICR_STOP;
	return XLLP_STATUS_SUCCESS;
}//end XllpPI2CWrite

XLLP_STATUS_T XllpPI2CRead(P_XLLP_PI2C_T PI2C_regs, 
                           XLLP_OST_T * pOSTRegs,
            			   XLLP_UINT8_T slaveAddr, 
            			   XLLP_UINT8_T * bytesBuf,
            			   XLLP_UINT32_T bytesCount, 
            			   XLLP_BOOL_T bSendStop) 
{
	XLLP_UINT32_T reg;
	
#ifdef XLLP_DEBUG_PARAM_CHECK
	    if (NULL == PI2C_regs || NULL == pOSTRegs)
		return XLLP_STATUS_WRONG_PARAMETER;
	
#endif
	PI2C_regs->XLLP_IDBR = (slaveAddr << 1) | XLLP_IDBR_MODE;
	reg = PI2C_regs->XLLP_ICR;
	reg |= (XLLP_ICR_START | XLLP_ICR_TB);
	reg &= ~(XLLP_ICR_STOP | XLLP_ICR_ALDIE);
	PI2C_regs->XLLP_ICR = reg;
	if (XllpPI2cTxEmpty(PI2C_regs, pOSTRegs, 20) == XLLP_STATUS_TIME_OUT)
		{
		return XLLP_STATUS_TIME_OUT;
		}
	while (bytesCount--)
		{
		reg = PI2C_regs->XLLP_ICR;
		reg &= ~XLLP_ICR_START;
		reg |= XLLP_ICR_ALDIE | XLLP_ICR_TB;
		if (bytesCount == 0)
			{
			reg |= XLLP_ICR_ACKNACK;
			if (bSendStop)
				reg |= XLLP_ICR_STOP;
			
			else
				reg &= ~XLLP_ICR_STOP;
		} else
			{
			reg &= ~XLLP_ICR_ACKNACK;
			}
		PI2C_regs->XLLP_ICR = reg;
		if (XllpPI2cRxFull(PI2C_regs, pOSTRegs, 60) == XLLP_STATUS_TIME_OUT)
			{
			return XLLP_STATUS_TIME_OUT;
			}
		reg = PI2C_regs->XLLP_IDBR & 0xFF;
		*bytesBuf++ = (XLLP_UINT8_T) reg;
		}
	PI2C_regs->XLLP_ICR &= ~(XLLP_ICR_STOP | XLLP_ICR_ACKNACK);
	return XLLP_STATUS_SUCCESS;
}//end XllpPI2CRead

XLLP_PI2C_SPEED_T XllpPI2CGetSpeed(P_XLLP_PI2C_T PI2C_regs) 
{
	
#ifdef XLLP_DEBUG_PARAM_CHECK
	if (NULL == PI2C_regs)
		return XLLP_STATUS_WRONG_PARAMETER;
	
#endif
	return ((PI2C_regs->
		XLLP_ICR & XLLP_ICR_FM) ? XLLP_PI2C_FAST_SPEED :
		XLLP_PI2C_NORMAL_SPEED);
}//end XllpGetSpeed

XLLP_PI2C_SPEED_T XllpPI2CSetSpeed(P_XLLP_PI2C_T PI2C_regs, XLLP_PI2C_SPEED_T speed) 
{
	
#ifdef XLLP_DEBUG_PARAM_CHECK
	    if (NULL == PI2C_regs)
		return XLLP_STATUS_WRONG_PARAMETER;
	
#endif
	XLLP_UINT32_T tem;
	XLLP_PI2C_SPEED_T pre_speed = XllpPI2CGetSpeed(PI2C_regs);
	switch (speed) {
	case XLLP_PI2C_FAST_SPEED:
		PI2C_regs->XLLP_ICR |= XLLP_ICR_FM;
	
	//Be sure the configureation takes effect in case of split transaction problem
		tem = PI2C_regs->XLLP_ICR;
		return pre_speed;
		break;
	default:
		PI2C_regs->XLLP_ICR &= (~XLLP_ICR_FM);
		
	//Be sure the configuration takes effect in case of split transaction problem
		tem = PI2C_regs->XLLP_ICR;
		return pre_speed;
	}
}//end XllpSetSpeed
#endif

