/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/** FILENAME:	xllp_pm_arava.c
**
** PURPOSE:	contain APIs for arava PMIC.
**
******************************************************************************/

#include "xllp_pm_arava.h"
#include "xllp_serialization.h"
#include "xllp_camera_os_depend.h"

#define ARAVA_READ(reg, pval)	XllpPmAravaRead(reg, pval)
#define ARAVA_WRITE(reg, val)	XllpPmAravaWrite(reg, val)
#define MICCO_READ(reg, pval)	XllpPmMiccoRead(reg, pval)
#define MICCO_WRITE(reg, val)	XllpPmMiccoWrite(reg, val)

#define VBASE	1100
#define VSTEP	50
#define VMAX	2650

#define VBUCK2BASE	850
#define VBUCK2STEP	25
#define VBUCK2MAX	1625

#define	MICCO_BUCK1_BASE	725
#define	MICCO_BUCK1_STEP	25
#define	MICCO_BUCK1_MAX		1500

#define	MICCO_VSRAM_BASE	725
#define	MICCO_VSRAM_STEP	25
#define	MICCO_VSRAM_MAX		1500

/* Make sure that Power I2C has been initialized before invoke this function */	
XLLP_STATUS_T XllpPmAravaInit(void)
{
	return ARAVA_WRITE(XLLP_ARAVA_SYSCTRL_A, 0xE8);	
}

XLLP_STATUS_T XllpPmAravaWrite(XLLP_UINT8_T reg, XLLP_UINT8_T val)
{
	XLLP_UINT8_T buffer[2];
	int status;
	buffer[0] = reg;
	buffer[1] = val;

	XllpLock(XLLP_RESOURCE_I2C);
	status = OS_I2CMasterWriteData(XLLP_ARAVA_ADDRESS, buffer, 2);
	XllpUnlock(XLLP_RESOURCE_I2C);
	return status;	
}

XLLP_STATUS_T XllpPmAravaRead(XLLP_UINT8_T reg, XLLP_UINT8_T *pval)
{
	XLLP_UINT8_T buffer[1];
	int status;
	buffer[0] = reg;

	XllpLock(XLLP_RESOURCE_I2C);
	status = OS_I2CMasterWriteData( XLLP_ARAVA_ADDRESS, buffer, 1);
	if (!status) {
		status = OS_I2CMasterReadData( XLLP_ARAVA_ADDRESS, buffer, 1);
		*pval = buffer[0];
	}
	XllpUnlock(XLLP_RESOURCE_I2C);
	return status;
}

XLLP_STATUS_T XllpPmAravaSetVccSram(XLLP_UINT32_T mv)
{
	XLLP_STATUS_T status;
	XLLP_UINT8_T val;

	if ((mv < VBASE) || (mv > VMAX))
		return XLLP_STATUS_WRONG_PARAMETER;
	status = ARAVA_READ(XLLP_ARAVA_LDO1416, &val);
	if (status != XLLP_STATUS_SUCCESS)
		return status;
	val &= ~0xf8;
	val |= ((mv - VBASE)/VSTEP) << 0x3;
	status = ARAVA_WRITE(XLLP_ARAVA_LDO1416, val);
	return status;

}

XLLP_STATUS_T XllpPmAravaSetVccCore(XLLP_UINT32_T mv)
{
	XLLP_STATUS_T status;
	XLLP_UINT8_T val;

	if ((mv < VBUCK2BASE) || (mv > VBUCK2MAX))
		return XLLP_STATUS_WRONG_PARAMETER;
	status = ARAVA_READ(XLLP_ARAVA_BUCK2DVC1, &val);
	if (status != XLLP_STATUS_SUCCESS)
		return status;
	val &= ~0x1f;
	val |= (mv - VBUCK2BASE)/VBUCK2STEP | 0x80;
	status = ARAVA_WRITE(XLLP_ARAVA_BUCK2DVC1, val);
	return status;

}

XLLP_STATUS_T XllpPmAravaGetVccSram(XLLP_UINT32_T *pmv)
{
	XLLP_STATUS_T status;
	XLLP_UINT8_T val;

	status = ARAVA_READ(XLLP_ARAVA_LDO1416, &val);
	if (status != XLLP_STATUS_SUCCESS)
		return status;
	val = val >> 0x3;
	*pmv = val * VSTEP + VBASE;
	return XLLP_STATUS_SUCCESS;

}

XLLP_STATUS_T XllpPmAravaGetVccCore(XLLP_UINT32_T *pmv)
{
	XLLP_STATUS_T status;
	XLLP_UINT8_T val;

	status = ARAVA_READ(XLLP_ARAVA_BUCK2DVC1, &val);
	if (status != XLLP_STATUS_SUCCESS)
		return status;
	val &= 0x1f;
	*pmv = val * VBUCK2STEP + VBUCK2BASE;
	return XLLP_STATUS_SUCCESS;
}

#define	XLLP_MICCO_ADDRESS	0x34

XLLP_STATUS_T XllpPmMiccoWrite(XLLP_UINT8_T reg, XLLP_UINT8_T val)
{
	XLLP_UINT8_T buffer[2];
	int status;
	buffer[0] = reg;
	buffer[1] = val;

	XllpLock(XLLP_RESOURCE_I2C);
	status = OS_I2CMasterWriteData(XLLP_MICCO_ADDRESS, buffer, 2);
	XllpUnlock(XLLP_RESOURCE_I2C);
	return status;	
}

XLLP_STATUS_T XllpPmMiccoRead(XLLP_UINT8_T reg, XLLP_UINT8_T *pval)
{
	XLLP_UINT8_T buffer[1];
	int status;
	buffer[0] = reg;

	XllpLock(XLLP_RESOURCE_I2C);
	status = OS_I2CMasterWriteData(XLLP_MICCO_ADDRESS, buffer, 1);
	if (!status) {
		status = OS_I2CMasterReadData(XLLP_MICCO_ADDRESS, buffer, 1);
		*pval = buffer[0];
	}
	XllpUnlock(XLLP_RESOURCE_I2C);
	return status;
}

XLLP_STATUS_T XllpPmMiccoSetVccSram(XLLP_UINT32_T mv)
{
	XLLP_STATUS_T status;
	XLLP_UINT8_T val;

	if ((mv < MICCO_VSRAM_BASE) || (mv > MICCO_VSRAM_MAX))
		return XLLP_STATUS_WRONG_PARAMETER;

	status = MICCO_READ(0x2A, &val);
	if (status != XLLP_STATUS_SUCCESS)
		return status;
	val &= 0xe0;
	val = val | ((mv - MICCO_VSRAM_BASE) / MICCO_VSRAM_STEP);
	status = MICCO_WRITE(0x2A, val);
	if (status != XLLP_STATUS_SUCCESS)
		return status;
	status = MICCO_WRITE(0x20, 0x33);
	return status;
}

XLLP_STATUS_T XllpPmMiccoSetVccCore(XLLP_UINT32_T mv)
{
	XLLP_STATUS_T status;
	XLLP_UINT8_T val;

	if ((mv < MICCO_BUCK1_BASE) || (mv > MICCO_BUCK1_MAX))
		return XLLP_STATUS_WRONG_PARAMETER;

	status = MICCO_READ(0x24, &val);
	if (status != XLLP_STATUS_SUCCESS)
		return status;
	val &= 0xe0;
	val = val | ((mv - MICCO_BUCK1_BASE)/MICCO_BUCK1_STEP);
	status = MICCO_WRITE(0x24, val);
	if (status != XLLP_STATUS_SUCCESS)
		return status;
	status = MICCO_WRITE(0x20, 0x33);
	return status;
}

XLLP_STATUS_T XllpPmMiccoGetVccSram(XLLP_UINT32_T *pmv)
{
	XLLP_STATUS_T status;
	XLLP_UINT8_T val;

	status = MICCO_READ(0x2A, &val);
	if (status != XLLP_STATUS_SUCCESS)
		return status;
	val &= 0x1f;
	*pmv = val * MICCO_VSRAM_STEP + MICCO_VSRAM_BASE;
	return XLLP_STATUS_SUCCESS;

}

XLLP_STATUS_T XllpPmMiccoGetVccCore(XLLP_UINT32_T *pmv)
{
	XLLP_STATUS_T status;
	XLLP_UINT8_T val;

	status = MICCO_READ(0x24, &val);
	if (status != XLLP_STATUS_SUCCESS)
		return status;
	val &= 0x1f;
	*pmv = val * MICCO_BUCK1_STEP + MICCO_BUCK1_BASE;
	return XLLP_STATUS_SUCCESS;
}

#define	XLLP_MAX7320_ADDRESS	0x50

XLLP_STATUS_T xllp_max7320_write(XLLP_UINT8_T reg, XLLP_UINT8_T val)
{
	XLLP_UINT8_T buffer[2];
	int status;
	buffer[0] = reg;
	buffer[1] = val;

	XllpLock(XLLP_RESOURCE_I2C);
	status = OS_I2CMasterWriteData(XLLP_MAX7320_ADDRESS, buffer, 2);
	XllpUnlock(XLLP_RESOURCE_I2C);
	return status;	
	
}

XLLP_STATUS_T xllp_max7320_read(XLLP_UINT8_T reg, XLLP_UINT8_T *pval)
{
	XLLP_UINT8_T buffer[1];
	int status;
	buffer[0] = reg;

	XllpLock(XLLP_RESOURCE_I2C);
	status = OS_I2CMasterReadData(XLLP_MAX7320_ADDRESS, buffer, 1);
	*pval = buffer[0];
	XllpUnlock(XLLP_RESOURCE_I2C);
	return status;
}
