/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/**
**  FILENAME:       xllp_udc_plat.c
**
**  PURPOSE:   		This files contains the platform-specific driver for USB Device controller 
**                  (UDC).
**
**
******************************************************************************/

/*
*******************************************************************************
*   HEADER FILES
*******************************************************************************
*/                                                                      

#include "xllp_defs.h"
#include "xllp_udc.h"


void XllpUdcPlatformInit(P_XLLP_UDC_T pUdcHandle)
{
	// Configure to use internal transceiver in a non-OTG way.
	pUdcHandle->pRegsBase->up2ocr = XLLP_UDC_UP2OCR_HXOE | XLLP_UDC_UP2OCR_DPPUE;
}

void XllpUdcPlatformUninit(P_XLLP_UDC_T pUdcHandle)
{
	pUdcHandle->pRegsBase->up2ocr = 0;
}

void XllpUdcBusDevSoftConnect (P_XLLP_UDC_T pUdcHandle, XLLP_BOOL_T connect)
{
    // FIXME: need to check whether the pull-up control has effects
    // 		  when port2 is used as non-OTG client.
    //		  Got conflicts information from developer's manual.
    if (connect == XLLP_TRUE)
    {
        // Pull up the D+ pin at the internal transceiver.
        pUdcHandle->pRegsBase->up2ocr &= ~(XLLP_UDC_UP2OCR_DPPDE
        		|XLLP_UDC_UP2OCR_DMPUE
        		|XLLP_UDC_UP2OCR_DMPDE);        
        pUdcHandle->pRegsBase->up2ocr |= XLLP_UDC_UP2OCR_DPPUE;
    }
    else
    {
        // Float the D+,D- pin at the internal transceiver.
        pUdcHandle->pRegsBase->up2ocr &= ~(XLLP_UDC_UP2OCR_DPPUE
        		|XLLP_UDC_UP2OCR_DPPDE
	       		|XLLP_UDC_UP2OCR_DMPUE
        		|XLLP_UDC_UP2OCR_DMPDE);        

    }
}
