/*++
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
--*/
/******************************************************************************
**
**  FILENAME:       xllp_usbotg_zylonite.c
**  
**  PURPOSE: Implements USB OTG Zylonite platform specific primitives
**  
**  
******************************************************************************/

#include "xllp_usbotg.h"
#include "xllp_usbotg_zylonite.h"

static XLLP_UINT32_T xllp_otg_mfp_list[] = {
	XLLP_MFP_USB_OTG_SR_OFFSET,
	XLLP_MFP_OTG_EN_OFFSET,
	XLLP_MFP_OTG_ID_OFFSET,
	XLLP_MFP_PIN_EOLIST_MARKER};
static XLLP_MFP_ALT_FN_T xllp_otg_af_list[] = {
	XLLP_MFP_USB_OTG_SR_AF,
	XLLP_MFP_OTG_EN_AF,
	XLLP_MFP_OTG_ID_AF};
static XLLP_MFP_DRIVE_STRENGTH_T xllp_otg_ds_list[] = {
	XLLP_MFP_DEFAULT_DS,
	XLLP_MFP_DEFAULT_DS,
	XLLP_MFP_DEFAULT_DS};
//static XLLP_MFP_LPM_OUTPUT xllp_otg_lpm_output_list[] = {

/***************************************************************************** 
 * FUNCTION:	OtgDetectDefaultFunction
 * 
 * DESCRIPTION:	Internal function to detect IDON pin
 *
 * PARAMETERS:
 *		P_XLLP_OTG_T pOtgHandle - a pointer to USB OTG's handle
 *          
 * RETURNS: 
 *		XLLP_OTG_A_DEVICE
 *		XLLP_OTG_B_DEVICE			
 *
 *****************************************************************************/
/* When I debug, the following code don't work. The status bit only is set 
 * after enable system interrupt, then a interrupt come out */
static XLLP_OTG_FUNCTION OtgDetectDefaultFunction (P_XLLP_OTG_T pOtgHandle)
{
	int i=100;

	/* Set IDON bit in UP2OCR */
	pOtgHandle->pUdcRegs->up2ocr |= XLLP_OTG_UP2OCR_IDON;
	pOtgHandle->pUdcRegs->udcotgicr |= \
			(XLLP_OTG_UDCOTGISR_IRIDR |XLLP_OTG_UDCOTGISR_IRIDF);

	while(i--) {
		/* Read the UDCOTGISR to see whether there is 
		 * a rising edge happened on OTGID */
		if(pOtgHandle->pUdcRegs->udcotgisr & XLLP_OTG_UDCOTGISR_IRIDR) {
			/* Clear ID interrupt */
			pOtgHandle->pUdcRegs->udcotgisr = 0x3; 
			return XLLP_OTG_B_DEVICE;
		}
	
		/* Read the UDCOTGISR to see whether there is 
		 * a falling edge happened on OTGID */
		if(pOtgHandle->pUdcRegs->udcotgisr & XLLP_OTG_UDCOTGISR_IRIDF) {
			/* Clear ID interrupt */
			pOtgHandle->pUdcRegs->udcotgisr = 0x3; 
			return XLLP_OTG_A_DEVICE;
		}
		/* Delay 1 ms to wait for OTG ID stable */
		XllpOstDelayMilliSeconds(pOtgHandle->pOstRegs, 1);
	}
	return XLLP_OTG_B_DEVICE;
}

/***************************************************************************** 
 * FUNCTION:	OtgInitHandle
 * 
 * DESCRIPTION:	Internal function called when initialize fields in handle
 *
 * PARAMETERS:
 *		P_XLLP_OTG_T pOtgHandle - a pointer to USB OTG's handle
 *          
 * RETURNS: 
 *		XLLP_OTG_INVALID_PARAMETER
 *		XLLP_OTG_WRONG_STATE
 *
 *****************************************************************************/
static void OtgInitHandle(P_XLLP_OTG_T pOtgHandle, XLLP_OTG_FUNCTION function)
{
	pOtgHandle->a_set_b_hnp_en = 0;
	pOtgHandle->b_hnp_enabled = 0;
	pOtgHandle->a_hnp_supported = 0;
	pOtgHandle->a_alt_hnp_supported = 0;
	
	/* Initialize timer identifiers */
	pOtgHandle->a_wait_vrise_tmr = -1;
	pOtgHandle->a_wait_bcon_tmr = -1;
	pOtgHandle->a_aidl_bdis_tmr = -1;
	pOtgHandle->b_ase0_brst_tmr = -1;
	pOtgHandle->b_srp_fail_tmr = -1;
}

/***************************************************************************** 
 * FUNCTION:	OtgTimerCallbackAWaitVrise
 * 
 * DESCRIPTION:	Internal function called when timer a_wait_vrise_tmr expires
 *
 * PARAMETERS:
 *		P_XLLP_OTG_T pOtgHandle - a pointer to USB OTG's handle
 *          
 * RETURNS: 
 *		XLLP_OTG_INVALID_PARAMETER
 *		XLLP_OTG_WRONG_STATE
 *
 *****************************************************************************/
#if 0
static void OtgTimerCallbackAWaitVrise (unsigned long data)
{
	P_XLLP_OTG_T pOtgHandle = (P_XLLP_OTG_T)data;

#ifdef XLLP_DEBUG_PARAM_CHECK
	if (NULL == pOtgHandle) 
		return ;
#endif
	pOtgHandle = (P_XLLP_OTG_T)data;
	if (XLLP_OTG_A_WAIT_VRISE != pOtgHandle->state) {
		return ;
	}
	/* Turn off Vbus and return to idle state */
	XllpOtgVbusEnable(pOtgHandle, XLLP_FALSE);
	pOtgHandle->state = XLLP_OTG_A_IDLE;
	XllpOtgTimerExpired(pOtgHandle->a_wait_vrise_tmr);
	pOtgHandle->a_wait_vrise_tmr = -1;
}
#endif
/***************************************************************************** 
 * FUNCTION:	XllpOtgInit
 * 
 * DESCRIPTION:	Initialization
 *
 * PARAMETERS:
 *		P_XLLP_OTG_T pOtgHandle - a pointer to USB OTG's handle
 *          
 * RETURNS: 
 *		XLLP_OTG_SUCCESS
 *		XLLP_OTG_UDC_DISABLED
 *		XLLP_OTG_I2C_ERROR
 *
 *****************************************************************************/
XLLP_OTG_ERROR_T XllpOtgInit (P_XLLP_OTG_T pOtgHandle)
{
	XLLP_OTG_ERROR_T  error;
	XLLP_OTG_FUNCTION function;
	XLLP_UINT8_T value;
	
#ifdef XLLP_DEBUG_PARAM_CHECK
	if ( (NULL == pOtgHandle) || (NULL == pOtgHandle->pUdcRegs) ||\
			 (NULL == pOtgHandle->pMfpRegs) ||\
			 (NULL == pOtgHandle->pPI2cRegs) )
		return XLLP_OTG_INVALID_PARAMETER;
#endif
	if ( pOtgHandle->pUdcRegs->udccr & XLLP_UDCCR_UDE) {
		pOtgHandle->pUdcRegs->udccr = XLLP_UDCCR_UDE | XLLP_UDCCR_OEN;
	} else
		return XLLP_OTG_UDC_DISABLED;
		
	/* Configure MFP */
	error = XllpOtgConfigMfp(pOtgHandle);
	if (XLLP_OTG_SUCCESS != error) {
		return error;
	}
	
	/* Read the IDON pin to detect whether there is a mini-A plug 
	 * is alreadly inserted in receptacle.*/
	function = OtgDetectDefaultFunction(pOtgHandle);
	pOtgHandle->default_function = function;
	
	OtgInitHandle(pOtgHandle, function);

	/* Enable Session Valid bit in misc control B register of arava */
	error = AravaRead(XLLP_ARAVA_MISCB, &value);
	if (XLLP_STATUS_SUCCESS != error) {
		return XLLP_OTG_I2C_ERROR;
	}
	value |= MISCB_SESSION_VALID_ENABLE;
	AravaWrite(XLLP_ARAVA_MISCB, value);

	if (XLLP_OTG_A_DEVICE == function) {
	//	XLLP_INT32_T id;
		
		XllpOtgConfigPort2(pOtgHandle->pUdcRegs, 
				XLLP_USB_INT_OTG_HOST_DP);
		
		/* Mask some interrupts we don't care, */
		AravaRead(XLLP_ARAVA_IRQ_MASK_B, &value);
		value |= (EVENT_B_SESSION_VALID | EVENT_B_SRP_DETECT |\
				 EVENT_B_VBUS_4P0 );
		value &= ~EVENT_B_VBUS_4P4;
		AravaWrite(XLLP_ARAVA_IRQ_MASK_B, value);
		
		XllpOtgVbusEnable(pOtgHandle, XLLP_TRUE);
#if 0		
		id = XllpOtgRegisterTimer (pOtgHandle, T_A_WAIT_VRISE, 
				OtgTimerCallbackAWaitVrise);
		if(id < 0) {
			return XLLP_OTG_TIMER_FAILED;
		}
		pOtgHandle->a_wait_vrise_tmr = id;
#endif		
//		pOtgHandle->state = XLLP_OTG_A_WAIT_VRISE;
		pOtgHandle->state = XLLP_OTG_A_WAIT_BCON;
	} else if (XLLP_OTG_B_DEVICE == function) {
		XllpOtgConfigPort2(pOtgHandle->pUdcRegs, 
				XLLP_USB_INT_OTG_CLIENT_DP);
//		pOtgHandle->state = XLLP_OTG_B_IDLE;
		pOtgHandle->state = XLLP_OTG_B_PERIPHERAL;
		
		/* Mask some interrupts we don't care, we only care 
		 * session valid interrupt*/
		AravaRead(XLLP_ARAVA_IRQ_MASK_B, &value);
		value |= (EVENT_B_SRP_DETECT | EVENT_B_VBUS_4P0 |\
				 EVENT_B_VBUS_4P4);
		value &= ~EVENT_B_SESSION_VALID;
		AravaWrite(XLLP_ARAVA_IRQ_MASK_B, value);
		XllpOtgVbusEnable(pOtgHandle, XLLP_FALSE);
	}
#if 0
	/* Enable both rising and falling interrupt */
	AravaRead(XLLP_ARAVA_MISCB, &value);
	value &= ~MISCB_USB_INT_RISING;
	AravaWrite(XLLP_ARAVA_MISCB, value);
#endif	
	return XLLP_OTG_SUCCESS;
}

/***************************************************************************** 
 * FUNCTION:	XllpOtgResiterControl
 * 
 * DESCRIPTION:	Control USB OTG pullup and pulldown register
 *
 * PARAMETERS:
 *		P_XLLP_OTG_T pOtgHandle - a pointer to USB OTG's handle
 *		XLLP_OTG_RESISTER_T resister - register which will be disabled/enabled
 *		XLLP_BOOT_T enable - enable or disable
 * RETURNS: 
 *		XLLP_OTG_SUCCESS
 *		XLLP_OTG_INVALID_PARAMETER
 *		XLLP_OTG_WRONG_RESISTER
 *
 *****************************************************************************/
XLLP_OTG_ERROR_T XllpOtgResisterControl (P_XLLP_OTG_T pOtgHandle,
			XLLP_OTG_RESISTER_T resister, 
			XLLP_BOOL_T enable)
{
#ifdef XLLP_DEBUG_PARAM_CHECK
	if ( (NULL == pOtgHandle) || (NULL == pOtgHandle->pUdcRegs) ||\
			 (NULL == pOtgHandle->pMfpRegs) ||\
			 (NULL == pOtgHandle->pPI2cRegs) )
		return XLLP_OTG_INVALID_PARAMETER;

	if (resister > 5 || resister < 2) {
		return XLLP_OTG_WRONG_RESISTER;
	}
#endif
	
	if (XLLP_TRUE == enable) {
		pOtgHandle->pUdcRegs->up2ocr |= (1u << resister);
	} else {
		pOtgHandle->pUdcRegs->up2ocr &= ~(1u << resister);
	}
	
	return XLLP_OTG_SUCCESS;
}

/***************************************************************************** 
 * FUNCTION:	XllpOtgVbusEnable
 * 
 * DESCRIPTION:	Enable or disable USB Vbus 
 *
 * PARAMETERS:
 *		P_XLLP_OTG_T pOtgHandle - a pointer to USB OTG's handle
 *		XLLP_BOOT_T enable - enable or disable USB Vbus power
 * RETURNS: 
 *		XLLP_OTG_SUCCESS
 *		XLLP_OTG_INVALID_PARAMETER
 *		XLLP_OTG_WRONG_RESISTER
 *
 *****************************************************************************/
XLLP_OTG_ERROR_T XllpOtgVbusEnable (P_XLLP_OTG_T pOtgHandle, XLLP_BOOL_T enable)
{
	XLLP_INT8_T value;
	XLLP_STATUS_T status = XLLP_STATUS_SUCCESS;

#ifdef XLLP_DEBUG_PARAM_CHECK
	if (NULL == pOtgHandle->pPI2cRegs)
		return XLLP_OTG_INVALID_PARAMETER;
#endif	
	if ( XLLP_TRUE == enable) {
		/*XLLP_ARAVA_USBPUMP: USB pump, Set En_USBVEP, En_USBVE, USBVE*/
		status = AravaWrite(XLLP_ARAVA_USBPUMP, (USBPUMP_EN_USBVEP|\
				USBPUMP_EN_USBVE|USBPUMP_USBVE)); 
		if (XLLP_STATUS_SUCCESS != status) {
			return XLLP_OTG_I2C_ERROR;
		}
	} else {
		AravaRead(XLLP_ARAVA_USBPUMP, &value);
		value &= ~USBPUMP_USBVE;
		AravaWrite(XLLP_ARAVA_USBPUMP, value);
	}
	return XLLP_OTG_SUCCESS;
}

/***************************************************************************** 
 * FUNCTION:	XllpOtgConfigMfp
 * 
 * DESCRIPTION:	Configure MFP used in USB OTG module on Zylonite 
 *
 * PARAMETERS:
 *		P_XLLP_OTG_T pOtgHandle - a pointer to USB OTG's handle
 * RETURNS: 
 *		XLLP_OTG_SUCCESS
 *		XLLP_OTG_INVALID_PARAMETER
 *		XLLP_OTG_MFP_FAILED
 *
 *****************************************************************************/
XLLP_OTG_ERROR_T XllpOtgConfigMfp (P_XLLP_OTG_T pOtgHandle)
{
	XLLP_STATUS_T status = XLLP_STATUS_SUCCESS;
	
#ifdef XLLP_DEBUG_PARAM_CHECK
	if ((NULL == pOtgHandle) || (NULL == pOtgHandle->pMfpRegs))
		return XLLP_OTG_INVALID_PARAMETER;
#endif		
	
	status = XllpMfpResourceManager_List(pOtgHandle->pMFPRmDb, \
			xllp_otg_mfp_list, 
			XLLP_MFP_RM_ID_XLLP_USBOTG, 
			XLLP_SET);
	if(XLLP_STATUS_SUCCESS != status) {
		return XLLP_OTG_MFP_FAILED;
	}

	status = XllpMfpSetAfDs_List(pOtgHandle->pMfpRegs,
				xllp_otg_mfp_list,
				xllp_otg_af_list,
				xllp_otg_ds_list);

	if(XLLP_STATUS_SUCCESS != status) {
		return XLLP_OTG_MFP_FAILED;
	}

#if 0	
	status = XllpMfpConfigureLpmOutputLevel_List (
			(P_XLLP_VUINT32_T)camera_context->mfp_reg_base,
			xllp_otg_mfp_list,
			xllp_otg_lpm_output_list);

	if ( XLLP_STATUS_SUCCESS != status) {
		return XLLP_OTG_MFP_FAILED;    
	}
#endif
    	status = XllpMfpActivatePullUpDown_List (
			pOtgHandle->pMfpRegs,
               		xllp_otg_mfp_list,
               		XLLP_OFF);

	if ( XLLP_STATUS_SUCCESS != status) {
       		return XLLP_OTG_MFP_FAILED ;    
    	}

	return XLLP_OTG_SUCCESS;
}


/***************************************************************************** 
 * FUNCTION:	OtgTimerCallbackBSrpFail * 
 * DESCRIPTION:	Internal function called when timer b_srp_fail_tmr expires
 *
 * PARAMETERS:
 *		P_XLLP_OTG_T pOtgHandle - a pointer to USB OTG's handle
 *          
 * RETURNS: 
 *		XLLP_OTG_INVALID_PARAMETER
 *		XLLP_OTG_WRONG_STATE
 *
 *****************************************************************************/
static void OtgTimerCallbackBSrpFail (unsigned long data)
{
	P_XLLP_OTG_T pOtgHandle = (P_XLLP_OTG_T)data;

#ifdef XLLP_DEBUG_PARAM_CHECK
	if (NULL == pOtgHandle) 
		return ;
#endif
	if (XLLP_OTG_B_IDLE != pOtgHandle->state) {
		return ;
	}
	XllpOtgTimerExpired(pOtgHandle->b_ase0_brst_tmr);
	pOtgHandle->b_ase0_brst_tmr = -1;
	/* Call a function to told upper layer software this failure */
}

/***************************************************************************** 
 * FUNCTION:	XllpOtgVbusSRP
 * 
 * DESCRIPTION:	Called to start SRP 
 *
 * PARAMETERS:
 *		P_XLLP_OTG_T pOtgHandle - a pointer to USB OTG's handle
 * RETURNS: 
 *		XLLP_OTG_SUCCESS
 *		XLLP_OTG_INVALID_PARAMETER
 *
 *****************************************************************************/
XLLP_OTG_ERROR_T XllpOtgVbusSRP (P_XLLP_OTG_T pOtgHandle)
{
	XLLP_INT8_T value;
	XLLP_INT32_T up2ocr;
	XLLP_STATUS_T status = XLLP_STATUS_SUCCESS;	

#ifdef XLLP_DEBUG_PARAM_CHECK
	if (( NULL == pOtgHandle) || (NULL == pOtgHandle->pPI2cRegs))
		return XLLP_OTG_INVALID_PARAMETER;
#endif	

	pOtgHandle->state = XLLP_OTG_B_SRP_INIT;
	
	/* Enable Arava to start Vbus SRP */
	AravaRead(XLLP_ARAVA_USBPUMP, &value);
	value |= (USBPUMP_EN_USBVEP | USBPUMP_USBVEP); //Set En_USBVEP USBVEP
	AravaWrite(XLLP_ARAVA_USBPUMP, value);

	/* Enable D+ pull-up resister to enable Data line SRP */
	up2ocr = pOtgHandle->pUdcRegs->up2ocr;
	up2ocr &= ~XLLP_OTG_UP2OCR_DPPDE;
	up2ocr |= XLLP_OTG_UP2OCR_DPPUE;
	pOtgHandle->pUdcRegs->up2ocr = up2ocr;
	XllpOtgRegisterTimer(pOtgHandle, T_B_SRP_FAIL, \
			OtgTimerCallbackBSrpFail);
	XllpOstDelayMilliSeconds(pOtgHandle->pOstRegs, T_B_DATA_PLS);
	
	/* Remove D+ pull-up resister to stop data line SRP */
	up2ocr = pOtgHandle->pUdcRegs->up2ocr;
	up2ocr &= ~XLLP_OTG_UP2OCR_DPPUE;
	up2ocr |= XLLP_OTG_UP2OCR_DPPDE;
	pOtgHandle->pUdcRegs->up2ocr = up2ocr;
	
	/*  */
	XllpOstDelayMilliSeconds(pOtgHandle->pOstRegs, T_B_SRP_INIT - T_B_DATA_PLS);
	
	/* Stop Vbus SRP after Tb_srp_init */
	AravaRead(XLLP_ARAVA_USBPUMP, &value);
	value &= ~USBPUMP_USBVEP; //Set En_USBVEP USBVEP
	AravaWrite(XLLP_ARAVA_USBPUMP, value);
	
	return status;
}

/***************************************************************************** 
 * FUNCTION:	XllpOtgVbusInterrupt
 * 
 * DESCRIPTION:	Called when Vbus interrupt happend 
 *
 * PARAMETERS:
 *		P_XLLP_OTG_T pOtgHandle - a pointer to USB OTG's handle
 *		XLLP_UINT8_T eventb - value of register event b in arava
 * RETURNS: 
 *		XLLP_OTG_SUCCESS
 *		XLLP_OTG_INVALID_PARAMETER
 *
 *****************************************************************************/
XLLP_OTG_ERROR_T XllpOtgVbusInterrupt (P_XLLP_OTG_T pOtgHandle, XLLP_UINT8_T eventb)
{
	XLLP_INT8_T value;

#ifdef XLLP_DEBUG_PARAM_CHECK
	if (( NULL == pOtgHandle) || (NULL == pOtgHandle->pPI2cRegs))
		return XLLP_OTG_INVALID_PARAMETER;
#endif
	if (eventb & EVENT_B_SESSION_VALID) {
		AravaRead(XLLP_ARAVA_USBPUMP, &value);
		if (USBPUMP_SESSION_VALID & value) {
			if ((XLLP_OTG_B_DEVICE == pOtgHandle->default_function)
					&&(XLLP_OTG_B_IDLE==pOtgHandle->state)){
				pOtgHandle->state = XLLP_OTG_B_PERIPHERAL;
			}
		} else {/* Session invalid */
			if (XLLP_OTG_B_DEVICE == pOtgHandle->default_function) {
				pOtgHandle->state = XLLP_OTG_B_IDLE;
			}
		}
	}
	
	if (eventb & EVENT_B_SRP_DETECT) {
		if ((XLLP_OTG_A_DEVICE == pOtgHandle->default_function) &&\
				(XLLP_OTG_A_IDLE == pOtgHandle->state)) {
			AravaRead(XLLP_ARAVA_USBPUMP, &value);	
			if (value & USBPUMP_SRP_DETECT) {
				pOtgHandle->state = XLLP_OTG_A_WAIT_VRISE;
				XllpOtgVbusEnable(pOtgHandle, XLLP_TRUE);				
			}
		}
	}
	
	/* Voltage falled below or rised above Vbus valid threshold */
	if (eventb & EVENT_B_VBUS_4P4) {
		XLLP_OTG_STATE state = pOtgHandle->state;
		if ((XLLP_OTG_A_HOST == state) ||\
				(XLLP_OTG_A_PERIPHERAL == state)||\
				(XLLP_OTG_A_WAIT_BCON == state)||\
				(XLLP_OTG_A_SUSPEND == state)) {
			pOtgHandle->state = XLLP_OTG_A_VBUS_ERR;
			XllpOtgVbusEnable(pOtgHandle, XLLP_FALSE);
			pOtgHandle->state = XLLP_OTG_A_WAIT_VFALL;
		}
	}	

	return XLLP_OTG_SUCCESS;
}

/***************************************************************************** 
 * FUNCTION:	XllpOtgInterruptHandle 
 * 
 * DESCRIPTION:	This function is called when USB OTG interrupt happened
 *
 * PARAMETERS:
 * P_XLLP_OTG_T pOtgHandle - a pointer to USB OTG's handle
 *          
 * RETURNS: 
 *		XLLP_OTG_SUCCESS
 *		XLLP_OTG_INVALID_PARAMETER
 *		XLLP_OTG_TIMER_FAILED
 *
 *****************************************************************************/
XLLP_OTG_ERROR_T XllpOtgInterruptHandle (P_XLLP_OTG_T pOtgHandle)
{
	XLLP_INT8_T value;
	XLLP_UINT32_T otgisr;

#ifdef XLLP_DEBUG_PARAM_CHECK
	if ( NULL == pOtgHandle )
		return XLLP_OTG_INVALID_PARAMETER;
#endif
//	XllpOstDelayMilliSeconds(pOtgHandle->pOstRegs, 10);
	otgisr = pOtgHandle->pUdcRegs->udcotgisr;
	/* OTG SET FEATURE received */
	if (otgisr & XLLP_OTG_SETFEATURE) {
		if (pOtgHandle->pUdcRegs->udccr & XLLP_UDCCR_BHNP)
			pOtgHandle->b_hnp_enabled = 1;

		if (pOtgHandle->pUdcRegs->udccr & XLLP_UDCCR_AHNP)
			pOtgHandle->a_hnp_supported = 1;

		if (pOtgHandle->pUdcRegs->udccr & XLLP_UDCCR_AALTHNP)
			pOtgHandle->a_alt_hnp_supported = 1;

		pOtgHandle->pUdcRegs->udcotgisr = XLLP_OTG_SETFEATURE;
	}
		
	/* mini-A is plugged in */
	if (otgisr & XLLP_OTG_UDCOTGISR_IRIDF) {
		if (XLLP_OTG_B_DEVICE == pOtgHandle->default_function) {
//			XLLP_INT32_T id;

			OtgInitHandle(pOtgHandle, XLLP_OTG_A_DEVICE);
			pOtgHandle->default_function = XLLP_OTG_A_DEVICE;
			XllpOtgConfigPort2(pOtgHandle->pUdcRegs, 
					XLLP_USB_INT_OTG_HOST_DP);
			
			/* Mask some interrupts we don't care, */
			AravaRead(XLLP_ARAVA_IRQ_MASK_B, &value);
			value |= (EVENT_B_SESSION_VALID | EVENT_B_SRP_DETECT |\
					 EVENT_B_VBUS_4P0 );
			value &= ~EVENT_B_VBUS_4P4;
			AravaWrite(XLLP_ARAVA_IRQ_MASK_B, value);
			
			/* Enable USB PUMP */
			XllpOtgVbusEnable(pOtgHandle, XLLP_TRUE) ;
#if 0		
			/* Wait for vbus to rising */
			id = XllpOtgRegisterTimer (pOtgHandle, T_A_WAIT_VRISE, 
					OtgTimerCallbackAWaitVrise);
			if(id < 0) {
				return XLLP_OTG_TIMER_FAILED;
			}
			pOtgHandle->a_wait_vrise_tmr = id;
#endif		
//			pOtgHandle->state = XLLP_OTG_A_WAIT_VRISE;
			pOtgHandle->state = XLLP_OTG_A_WAIT_BCON;

			/* The following two line code is need to work around
			 * USB ID pin sighting */
			pOtgHandle->pUdcRegs->udcotgisr = \
					(XLLP_OTG_UDCOTGISR_IRIDR |\
					XLLP_OTG_UDCOTGISR_IRIDF);
			return XLLP_OTG_SUCCESS;
		}
	}
	/* mini-A is plugged out */
	if (otgisr & XLLP_OTG_UDCOTGISR_IRIDR) {
		if (XLLP_OTG_A_DEVICE == pOtgHandle->default_function){
			/* Disable usb pump */
			XllpOtgVbusEnable(pOtgHandle, XLLP_FALSE) ;

			OtgInitHandle(pOtgHandle, XLLP_OTG_B_DEVICE);
			pOtgHandle->default_function = XLLP_OTG_B_DEVICE;
//			pOtgHandle->state = XLLP_OTG_B_IDLE;
			pOtgHandle->state = XLLP_OTG_B_PERIPHERAL;
			XllpOtgConfigPort2(pOtgHandle->pUdcRegs, 
					XLLP_USB_INT_OTG_CLIENT_DP);
			
			/* Mask some interrupts we don't care, 
			 * we only care session valid interrupt*/
			AravaRead(XLLP_ARAVA_IRQ_MASK_B, &value);
			value |= (EVENT_B_SRP_DETECT | EVENT_B_VBUS_4P0 | EVENT_B_VBUS_4P4);
			value &= ~EVENT_B_SESSION_VALID;
			AravaWrite(XLLP_ARAVA_IRQ_MASK_B, value);
		}
	}
	pOtgHandle->pUdcRegs->udcotgisr = (XLLP_OTG_UDCOTGISR_IRIDR |\
					XLLP_OTG_UDCOTGISR_IRIDF);

	return XLLP_OTG_SUCCESS;
}

