/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#include "xllp_ac97acodec.h"
#include "xllp_wm9713.h"
#include "xllp_acodec.h"

static int CurrentMastInputGain = 15;
static int CurrentHeadsetVol = 48;

static const XLLP_UINT8_T xllp_9713_unsave [] =
{
    0x00,
    0x28,
    0x54,
    0x68,
    0x6a,
    0x6c,
    0x6e,
    0x74,
    0x7a,
    0x7c,
    0x7e    
};


XLLP_ACODEC_ERROR_T XllpWm9713SetMasterVol(XLLP_ACODEC_CONTEXT_T *pDeviceContext, XLLP_UINT16_T GainInDb)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    XLLP_UINT8_T leftVolume, rightVolume;
    
    /*take the unit of GainInDb as 1 db*/
    leftVolume = (XLLP_UINT8_T)(GainInDb * 10 / 15);
    if (leftVolume > XLLP_AC97_WM9713_MAX_VOLUME)
    {
        leftVolume = XLLP_AC97_WM9713_MAX_VOLUME;
    }
    rightVolume = leftVolume;
    
    if ((0 == leftVolume) && (0 == rightVolume))
    {
        //mute the output
        value = XLLP_AC97_WM9713_MVR_MM;
    }
    else
    {
        leftVolume = XLLP_AC97_WM9713_MAX_VOLUME - leftVolume;    //translate into attenuation value
        rightVolume = XLLP_AC97_WM9713_MAX_VOLUME - rightVolume;
        value = leftVolume << XLLP_AC97_WM9713_MVR_ML_SHIFT;
        value |= rightVolume;
    }

    status = XllpAc97ACodecWrite (pDeviceContext, SPEAKER_VOLUME, value);     
    return status;    
}


XLLP_ACODEC_ERROR_T XllpWm9713SetMasterInputGain(XLLP_ACODEC_CONTEXT_T *pDeviceContext, XLLP_UINT16_T GainInDb)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT8_T leftVolume, rightVolume, vol;
    XLLP_UINT16_T value;
    
    /*take the unit of GainInDb as 1 db*/
    leftVolume = (XLLP_UINT8_T)(GainInDb * 10 / 15);
    if (leftVolume > XLLP_AC97_WM9713_MAX_ADCGAIN)
    {
        leftVolume = XLLP_AC97_WM9713_MAX_ADCGAIN;
    }
    vol = leftVolume;
    rightVolume = leftVolume;
    if ((0 == leftVolume) && (0 == rightVolume))
    {
        //mute the record
        value = XLLP_AC97_WM9713_RGR_RM;
    }
    else
    {
        value = leftVolume << XLLP_AC97_WM9713_RGR_GL_SHIFT;
        value |= rightVolume;
    }
      status = XllpAc97ACodecWrite (pDeviceContext, REC_PGA_VOL, value);     
	 if (status == XLLP_ACODEC_SUCCESS) {
   		CurrentMastInputGain = vol * 15/10 ;
   	}
    return status;    
}


XLLP_ACODEC_ERROR_T XllpWm9713GetInSampleRate(XLLP_ACODEC_CONTEXT_T *pDeviceContext, XLLP_UINT16_T * RateInHz)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    status = XllpAc97ACodecRead(pDeviceContext, AUDIO_ADC_RATE, &value);             				 
    
    if (XLLP_ACODEC_SUCCESS != status)
    {
        return status;
    }
    
    switch (value)    
    {
        case XLLP_AC97_WM9713_DR_8000:
            *RateInHz = 8000;
            break;
        case XLLP_AC97_WM9713_DR_11025:
            *RateInHz = 11025;
            break;
         case XLLP_AC97_WM9713_DR_12000:
            *RateInHz = 12000;
            break;
        case XLLP_AC97_WM9713_DR_16000:
            *RateInHz = 16000;
            break;
        case XLLP_AC97_WM9713_DR_22050:
            *RateInHz = 22050;
            break;
        case XLLP_AC97_WM9713_DR_24000:
            *RateInHz = 24000;
            break;
        case XLLP_AC97_WM9713_DR_32000:
            *RateInHz = 32000;
            break;
        case XLLP_AC97_WM9713_DR_44100:
            *RateInHz = 44100;
            break;
        default:
            *RateInHz = 48000;
            break;
    }
    
    return status;
}

XLLP_ACODEC_ERROR_T XllpWm9713GetOutSampleRate (XLLP_ACODEC_CONTEXT_T *pDeviceContext, XLLP_UINT16_T * RateInHz)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    status = XllpAc97ACodecRead(pDeviceContext, AUDIO_DAC_RATE, &value);             				 
    
    if (XLLP_ACODEC_SUCCESS != status)
    {
        return status;
    }
    
    switch (value)    
    {
        case XLLP_AC97_WM9713_DR_8000:
            *RateInHz = 8000;
            break;
        case XLLP_AC97_WM9713_DR_11025:
            *RateInHz = 11025;
            break;
        case XLLP_AC97_WM9713_DR_12000:
            *RateInHz = 12000;
            break;
        case XLLP_AC97_WM9713_DR_16000:
            *RateInHz = 16000;
            break;
        case XLLP_AC97_WM9713_DR_22050:
            *RateInHz = 22050;
            break;
        case XLLP_AC97_WM9713_DR_32000:
            *RateInHz = 32000;
            break;
         case XLLP_AC97_WM9713_DR_24000:
            *RateInHz = 24000;
            break;
        case XLLP_AC97_WM9713_DR_44100:
            *RateInHz = 44100;
            break;
        default:
            *RateInHz = 48000;
            break;
    }
    
    return status;
}

XLLP_ACODEC_ERROR_T XllpWm9713SetInSampleRate(XLLP_ACODEC_CONTEXT_T *pDeviceContext, XLLP_UINT16_T RateInHz)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    switch (RateInHz)
    {
        case 8000:
            value = XLLP_AC97_WM9713_DR_8000;
            break;
        case 11025:
            value = XLLP_AC97_WM9713_DR_11025;
            break;
        case 12000:
        	value = XLLP_AC97_WM9713_DR_12000;
        	break;
        case 16000:
            value = XLLP_AC97_WM9713_DR_16000;
            break;
        case 22050:
            value = XLLP_AC97_WM9713_DR_22050;
            break;
        case 24000:
        	value = XLLP_AC97_WM9713_DR_24000;
        	break;
        case 32000:
            value = XLLP_AC97_WM9713_DR_32000;
            break;
        case 44100:
            value = XLLP_AC97_WM9713_DR_44100;
            break;
        case 48000:
            value = XLLP_AC97_WM9713_DR_48000;
            break;
        default:
            return XLLP_ACODEC_SAMPLERATE_NOT_SUPPORTED;        
    }
    //enable VRA mode
    status = XllpAc97ACodecWrite(pDeviceContext, XLLP_AC97_CR_E_AUDIO_CTRL_STAT, XLLP_AC97_WM9713_EASCR_VRA);     
    if (XLLP_ACODEC_SUCCESS != status)
    {
        return status;
    }
    
    status = XllpAc97ACodecWrite(pDeviceContext, XLLP_AC97_CR_E_ASR_PCM_LR_ADC_RT, value);             				 
    return status;
}

XLLP_ACODEC_ERROR_T XllpWm9713SetOutSampleRate (XLLP_ACODEC_CONTEXT_T *pDeviceContext, XLLP_UINT16_T RateInHz)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    switch (RateInHz)
    {
        case 8000:
            value = XLLP_AC97_WM9713_DR_8000;
            break;
        case 11025:
            value = XLLP_AC97_WM9713_DR_11025;
            break;
        case 12000:
        	value = XLLP_AC97_WM9713_DR_12000;
        	break;
        case 16000:
            value = XLLP_AC97_WM9713_DR_16000;
            break;
        case 22050:
            value = XLLP_AC97_WM9713_DR_22050;
            break;
        case 24000:
        	value = XLLP_AC97_WM9713_DR_24000;
        	break;
        case 32000:
            value = XLLP_AC97_WM9713_DR_32000;
            break;
        case 44100:
            value = XLLP_AC97_WM9713_DR_44100;
            break;
        case 48000:
            value = XLLP_AC97_WM9713_DR_48000;
            break;
        default:
            return XLLP_ACODEC_SAMPLERATE_NOT_SUPPORTED;        
    }
    //enable VRA mode
    status = XllpAc97ACodecWrite(pDeviceContext, EXTENDED_AUD_STAT_CTRL, XLLP_AC97_WM9713_EASCR_VRA);     
    if (XLLP_ACODEC_SUCCESS != status)
    {
        return status;
    }
    
    status = XllpAc97ACodecWrite(pDeviceContext, AUDIO_DAC_RATE, value);             				 
    return status;
}

//this is platform specific
//do later: not to enable recording route and playback route in this function, leave it to driver to call other function
XLLP_ACODEC_ERROR_T XllpWm9713SpecificInit (XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
	//this assumes that the aclink is initialized
	//wait some time and then do a warm reset to enabled the ACLINK, required for wm9713 (not wm9712 or ucb1400)
    
    //pay attention: whether the workaround is still needed?
    P_XLLP_AC97ACODEC_T pAc97Reg = (P_XLLP_AC97ACODEC_T)(pDeviceContext->pCtrlReg); 
    
	pAc97Reg->gcr |= XLLP_AC97_GCR_WARM_RESET_MSK;

	XllpOstDelayMilliSeconds(pDeviceContext->pOSTRegs, 5);

	//power on all the necessary unit
	XllpAc97ACodecWrite(pDeviceContext,POWERDOWN_CTRL_STAT, 0x000);	/*26*/
	//open left headphone mixer
	//open right headphone mixer
	//open right/left dac
	//open right/left adc
	//open temperature sensor
	//enable reference generator
	XllpAc97ACodecWrite(pDeviceContext,POWER_DOWN_1, 0xda00);		/*3c */
	//open microphone bias
	//open HPL output PGA
	//open HPR output PGA
	//open mic PGA MA
	//open mic pre-amp MPA
	//if here we enable SPKL and SPKR PGA, then Touch screen will doesn't work
	XllpAc97ACodecWrite(pDeviceContext,POWER_DOWN_2,0xb9f5);   /*3e */	
	
	//recording route and microphone input
    //microphone selection, now fixed to MIC1 input and mic bias output
    //MIC1 only, MICBIAS enable
	XllpAc97ACodecWrite (pDeviceContext, MIC_BIAS, 0xC440);	/*0x22h*/

	//mic pga setting to mixer (side tone)
	//comment the below code to make MICA/B play back volume gain + 0db
	//XllpAc97ACodecWrite (pDeviceContext, MIC_PGA_VOLUME, 0x0000);	/*0x0eh*/

	//recording side tone and ADC boost, now fixed to default (14h)
	//recording volume  0dB
	XllpAc97ACodecWrite(pDeviceContext, REC_PGA_VOL, 0x0); /*12*/

	//hifi playback route and output mixer	
	//by default, fixed to enable headphone only

	//comment the below code to make SPEAKER default MUTE
	XllpAc97ACodecWrite (pDeviceContext, SPEAKER_VOLUME, 0x0);		/*02*/

	//comment the below code to make OUT3_OUT4 default MUTE
	//XllpAc97ACodecWrite (pDeviceContext, OUT3_OUT4_VOLUME, 0x8000);    /*06*/

	//remove all  the mute bit  volume gain + 0db 
	XllpAc97ACodecWrite(pDeviceContext, HEADPHONE_VOLUME, 0x0);	/*04*/	
	
	//DAC route
	//open DAC to headphone mixer path
	//left DAC gain +0db
	//right DAC gain +0db
	XllpAc97ACodecWrite(pDeviceContext, DAC_PGA_VOL_ROUTE,0x0808);	/*0c*/

	//out3 configure, invert to HPMIXR
	//XllpAc97ACodecWrite(pDeviceContext,DAC_3D_CTRL_INV_MUX_SEL, 0x8000);	/*1e*/

 	//output control
	 //select HPMIXR HPMIXL out
	 //other out are all VIM
	XllpAc97ACodecWrite(pDeviceContext,OUTPUT_PGA_MUX, 0x9BA8);		/*1c*/
	
	//set sample rates
	//enable variable rate conversion
	XllpAc97ACodecWrite(pDeviceContext, EXTENDED_AUD_STAT_CTRL , 0x1); /*2a*/
	// DAC 44kHZ
	XllpAc97ACodecWrite(pDeviceContext,AUDIO_DAC_RATE,0xac44); /*2c*/
	// ADC 16KHZ
	XllpAc97ACodecWrite(pDeviceContext,AUDIO_ADC_RATE,0x3E80); /*32*/

	//clock scheme, use external clock, it is 24MHZ from MCLK_A
    
    //the input Gain set to Max
	XllpWm9713SetMasterInputGain(pDeviceContext,15);
	
	XllpWm9713EarSpeakerEnable(pDeviceContext);
	
    return XLLP_ACODEC_SUCCESS;
}

static XLLP_ACODEC_ERROR_T  XllpWm9713HiFiNearInEnable(XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
	XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
   	 XLLP_UINT16_T value;
    
	//turn DAC on, we don't change 26h setting. insteadly we use 3ch and 3eh to control specific power domain 3CH  bit 7 and 6
	//power on left DAC and right DAC 
	status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
 	value &= ~(XLLP_AC97_9713_PWR_DACL | XLLP_AC97_9713_PWR_DACR |XLLP_AC97_9713_PWR_MBIAS);
	status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);

	//DAC route
	//left DAC gain +0db
	//right DAC gain +0db
	XllpAc97ACodecWrite(pDeviceContext, DAC_PGA_VOL_ROUTE, 0xe808);	/*0c*/
    
    return status;



}

XLLP_ACODEC_ERROR_T	XllpWm9713HiFiStreamPathEnable (XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
//turn DAC on, we don't change 26h setting. insteadly we use 3ch and 3eh to control specific power domain 3CH  bit 7 and 6
	//power on left DAC and right DAC 
	status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
 	value &= ~(XLLP_AC97_9713_PWR_DACL | XLLP_AC97_9713_PWR_DACR |XLLP_AC97_9713_PWR_MBIAS);
	status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);

//DAC route
	//open DAC to headphone mixer path
	//left DAC gain +0db
	//right DAC gain +0db
	XllpAc97ACodecWrite(pDeviceContext, DAC_PGA_VOL_ROUTE, 0x6808);	/*0c*/
    
    return status;
}

static XLLP_ACODEC_ERROR_T XllpWm9713CloseHifiNearIn(XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    //mute DAC
    XllpAc97ACodecWrite(pDeviceContext, DAC_PGA_VOL_ROUTE, 0xe808);	/*0c*/
    //power DAC off, we don't change 26h setting. insteadly we use 3ch and 3eh to control specific power domain 3CH  bit 7 and 6
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
    value |= (XLLP_AC97_9713_PWR_DACL | XLLP_AC97_9713_PWR_DACR);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);
    
    return status;
}
XLLP_ACODEC_ERROR_T	XllpWm9713HiFiStreamPathDisable (XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    //mute DAC
    XllpAc97ACodecWrite(pDeviceContext, DAC_PGA_VOL_ROUTE, 0xe808);	/*0c*/
    //power DAC off, we don't change 26h setting. insteadly we use 3ch and 3eh to control specific power domain 3CH  bit 7 and 6

    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
    value |= (XLLP_AC97_9713_PWR_DACL | XLLP_AC97_9713_PWR_DACR);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);
    
    return status;
}
static XLLP_ACODEC_ERROR_T XllpWm9713OpenHifiNearOut(XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
	XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    //turn ADC and MA pga on, we don't change 26h setting. insteadly we use 3ch and 3eh to control specific power domain 3CH  bit 5 and 4, 3eh bit 3 and 1 
   	//open right/left ADC 
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
    value &= ~(XLLP_AC97_9713_PWR_ADCL | XLLP_AC97_9713_PWR_ADCR |XLLP_AC97_9713_PWR_MBIAS);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);

    //select MICA into ADCL and ADCR
    status = XllpAc97ACodecWrite(pDeviceContext, REC_ROUTE_MUX_SEL, 0xc600);
    return status;
}

static XLLP_ACODEC_ERROR_T XllpWm9713CloseHifiNearOut(XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
	XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    //turn ADC and MA pga off, we don't change 26h setting. insteadly we use 3ch and 3eh to control specific power domain 3CH  bit 5 and 4, 3eh bit 3 and 1 
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
    value |= (XLLP_AC97_9713_PWR_ADCL | XLLP_AC97_9713_PWR_ADCR);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);
    return status;
}

XLLP_ACODEC_ERROR_T	XllpWm9713RecordingPathEnable (XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    //turn ADC and MA pga on, we don't change 26h setting. insteadly we use 3ch and 3eh to control specific power domain 3CH  bit 5 and 4, 3eh bit 3 and 1 
   	//open right/left ADC 
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
    value &= ~(XLLP_AC97_9713_PWR_ADCL | XLLP_AC97_9713_PWR_ADCR |XLLP_AC97_9713_PWR_MBIAS);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);

    //open mic PGA MA and mic pre-amp MPA
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_2, &value);
    value &= ~(XLLP_AC97_9713_PWR_MA | XLLP_AC97_9713_PWR_MPA);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_2, value);

    //select MICA into ADCL and ADCR
    status = XllpAc97ACodecWrite(pDeviceContext, REC_ROUTE_MUX_SEL, 0x600);
    return status;
}

XLLP_ACODEC_ERROR_T	XllpWm9713RecordingPathDisable (XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    //turn ADC and MA pga off, we don't change 26h setting. insteadly we use 3ch and 3eh to control specific power domain 3CH  bit 5 and 4, 3eh bit 3 and 1 

    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
    value |= (XLLP_AC97_9713_PWR_ADCL | XLLP_AC97_9713_PWR_ADCR);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);
    
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_2, &value);
    value |= (XLLP_AC97_9713_PWR_MA | XLLP_AC97_9713_PWR_MPA);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_2, value);
    
    return status;
}

XLLP_ACODEC_ERROR_T	XllpWm9713HeadsetEnable (XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    //mixer route 0ch:15, this bit control belongs to hifipath, because on Zylonite all playback goes
    //to headset mixer
    
    //output control: 1ch:7-6 5-4 04h:15, 7
    //select HPMIXR HPMIXL out
    status = XllpAc97ACodecRead(pDeviceContext, OUTPUT_PGA_MUX, &value);
    value &= ~(0xf0);
    value |= 0xa0;
    status = XllpAc97ACodecWrite(pDeviceContext, OUTPUT_PGA_MUX, value);

    //remove mute bit of HPL/HPR
    status = XllpAc97ACodecRead(pDeviceContext, HEADPHONE_VOLUME, &value);
    value &= ~(0x8080);
    status = XllpAc97ACodecWrite(pDeviceContext, HEADPHONE_VOLUME, value);
    
    //power 3ch: 3-2, 3eh 10-9
    //power on left/right headpone mix
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
    value &= ~(XLLP_AC97_9713_PWR_HPLX | XLLP_AC97_9713_PWR_HPRX);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);

    //power on HPL/HPR output PGA
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_2, &value);
    value &= ~(XLLP_AC97_9713_PWR_HPL | XLLP_AC97_9713_PWR_HPR);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_2, value);
    
    return status;
}

XLLP_ACODEC_ERROR_T	XllpWm9713HeadsetDisable (XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    //mixer route 0ch:15, this bit control belongs to hifipath, because on Zylonite all playback goes
    //to headset mixer
    
    //output control: 1ch:7-6 5-4 04h:15, 7
    status = XllpAc97ACodecRead(pDeviceContext, OUTPUT_PGA_MUX, &value);
    value &= ~(0xf0);
    status = XllpAc97ACodecWrite(pDeviceContext, OUTPUT_PGA_MUX, value);
    status = XllpAc97ACodecRead(pDeviceContext, HEADPHONE_VOLUME, &value);
    value |= 0x8080;
    status = XllpAc97ACodecWrite(pDeviceContext, HEADPHONE_VOLUME, value);
    
    //power 3ch: 3-2, 3eh 10-9
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
    value |= (XLLP_AC97_9713_PWR_HPLX | XLLP_AC97_9713_PWR_HPRX);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);
    
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_2, &value);
    value |= (XLLP_AC97_9713_PWR_HPL | XLLP_AC97_9713_PWR_HPR);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_2, value);
    
    return status;
}

XLLP_ACODEC_ERROR_T	XllpWm9713EarSpeakerEnable (XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;

//enable the below out putline
//HPMIXR -> INV1 ->OUT
//HPMIXL -> HPL

    //mixer route 0ch:15, this bit control belongs to hifipath, because on Zylonite all playback goes
    //to headset mixer
    
    //out3 route: 1eh: 15-13
    status = XllpAc97ACodecRead(pDeviceContext, DAC_3D_CTRL_INV_MUX_SEL, &value);
    value &= ~(0xe000);
    value |= 0x8000;
    status = XllpAc97ACodecWrite(pDeviceContext, DAC_3D_CTRL_INV_MUX_SEL, value);
    
    //output control: 1ch:7-6 3-2, 06h: 7, 04h: 15
    status = XllpAc97ACodecRead(pDeviceContext, OUTPUT_PGA_MUX, &value);
    value &= ~(0xcc);
    value |= 0x88;
    status = XllpAc97ACodecWrite(pDeviceContext, OUTPUT_PGA_MUX, value);
    
    status = XllpAc97ACodecRead(pDeviceContext, OUT3_OUT4_VOLUME, &value);
    value &= ~(0x80);
    status = XllpAc97ACodecWrite(pDeviceContext, OUT3_OUT4_VOLUME, value);
    
    status = XllpAc97ACodecRead(pDeviceContext, HEADPHONE_VOLUME, &value);
    value &= ~(0x8000);
    status = XllpAc97ACodecWrite(pDeviceContext, HEADPHONE_VOLUME, value);
    
    //power on left/right mixer
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
    value &= ~(XLLP_AC97_9713_PWR_HPLX | XLLP_AC97_9713_PWR_HPRX);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);

    //power on HPL output PGA 
    //power on OUT3 output PGA
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_2, &value);
    value &= ~(XLLP_AC97_9713_PWR_HPL | XLLP_AC97_9713_PWR_OUT3);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_2, value);
    
    return status;
}

XLLP_ACODEC_ERROR_T	XllpWm9713EarSpeakerDisable (XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    //mixer route 0ch:15, this bit control belongs to hifipath, because on Zylonite all playback goes
    //to headset mixer
    
    //out3 route: 1eh: 15-13
    status = XllpAc97ACodecRead(pDeviceContext, DAC_3D_CTRL_INV_MUX_SEL, &value);
    value &= ~(0xe000);
    status = XllpAc97ACodecWrite(pDeviceContext, DAC_3D_CTRL_INV_MUX_SEL, value);
    
    //output control: 1ch:7-6 3-2, 06h: 7, 04h: 15
    status = XllpAc97ACodecRead(pDeviceContext, OUTPUT_PGA_MUX, &value);
    value &= ~(0xcc);
    status = XllpAc97ACodecWrite(pDeviceContext, OUTPUT_PGA_MUX, value);
    
    status = XllpAc97ACodecRead(pDeviceContext, OUT3_OUT4_VOLUME, &value);
    value |= (0x80);
    status = XllpAc97ACodecWrite(pDeviceContext, OUT3_OUT4_VOLUME, value);
    
    status = XllpAc97ACodecRead(pDeviceContext, HEADPHONE_VOLUME, &value);
    value |= (0x8000);
    status = XllpAc97ACodecWrite(pDeviceContext, HEADPHONE_VOLUME, value);
    
    //power
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
    value |= (XLLP_AC97_9713_PWR_HPLX | XLLP_AC97_9713_PWR_HPRX);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);
    
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_2, &value);
    value |= (XLLP_AC97_9713_PWR_HPL | XLLP_AC97_9713_PWR_OUT3);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_2, value);
    
    return status;
}

XLLP_ACODEC_ERROR_T	XllpWm9713LouderSpeakerEnable (XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    //mixer route 0ch:15, this bit control belongs to hifipath, because on Zylonite all playback goes
    //to headset mixer
    
    
    //output control: 1ch 13-11, 10-8, 02h: 15, 7
    //HPMIXL->SPKL
    //HPMIXR->SPKR
    status = XllpAc97ACodecRead(pDeviceContext, OUTPUT_PGA_MUX, &value);
    value &= ~(0x3f00);
    value |= 0x1200;
    status = XllpAc97ACodecWrite(pDeviceContext, OUTPUT_PGA_MUX, value);

    //remove mute bit of SPKR/SPKL
    status = XllpAc97ACodecRead(pDeviceContext, SPEAKER_VOLUME, &value);
    value &= ~(0x8080);
    status = XllpAc97ACodecWrite(pDeviceContext, SPEAKER_VOLUME, value);
    
    //power on HPMIXR/HPMIXL
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
    value &= ~(XLLP_AC97_9713_PWR_HPLX | XLLP_AC97_9713_PWR_HPRX);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);

    //power on SPKL/SPKR output PGA
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_2, &value);
    value &= ~(XLLP_AC97_9713_PWR_SPKL | XLLP_AC97_9713_PWR_SPKR);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_2, value);
    
    return status;
}

XLLP_ACODEC_ERROR_T	XllpWm9713LouderSpeakerDisable (XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    //mixer route 0ch:15, this bit control belongs to hifipath, because on Zylonite all playback goes
    //to headset mixer
    
    
    //output control: 1ch 13-11, 10-8, 02h: 15, 7
    status = XllpAc97ACodecRead(pDeviceContext, OUTPUT_PGA_MUX, &value);
    value &= ~(0x3f00);
    status = XllpAc97ACodecWrite(pDeviceContext, OUTPUT_PGA_MUX, value);
    
    status = XllpAc97ACodecRead(pDeviceContext, SPEAKER_VOLUME, &value);
    value |= (0x8080);
    status = XllpAc97ACodecWrite(pDeviceContext, SPEAKER_VOLUME, value);
    
    //power
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
    value |= (XLLP_AC97_9713_PWR_HPLX | XLLP_AC97_9713_PWR_HPRX);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);
    
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_2, &value);
    value |= (XLLP_AC97_9713_PWR_SPKL | XLLP_AC97_9713_PWR_SPKR);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_2, value);
    
    return status;
}

XLLP_ACODEC_ERROR_T	XllpWm9713MicEnable (XLLP_ACODEC_CONTEXT_T *pDeviceContext, XLLP_UINT8_T micType)
{
    //mic input route: 22h: 13-12
    if (micType == HANDSET_MIC_MASK)
    {//mic1
    	XllpAc97ACodecWrite (pDeviceContext, MIC_BIAS, 0x4040);	/*0x22h*/                
    }
    else
    {//mic2a
    	XllpAc97ACodecWrite (pDeviceContext, MIC_BIAS, 0x5040);	/*0x22h*/
    }
    return XLLP_ACODEC_SUCCESS;
}

XLLP_ACODEC_ERROR_T XllpWm9713MicDisable (XLLP_ACODEC_CONTEXT_T *pDeviceContext, XLLP_UINT8_T micType)
{
   return XLLP_ACODEC_SUCCESS;
}

 XLLP_ACODEC_ERROR_T XllpWm9713SetHeadsetVol(struct _CONTEXT_T *pDeviceContext, XLLP_UINT16_T GainInDb)
 {
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    XLLP_UINT8_T leftVolume, rightVolume, vol;
    
    /*take the unit of GainInDb as 1 db*/
    leftVolume = (XLLP_UINT8_T)(GainInDb * 10 / 15);
    if (leftVolume > XLLP_AC97_WM9713_MAX_VOLUME)
    {
        leftVolume = XLLP_AC97_WM9713_MAX_VOLUME;
    }
    rightVolume = leftVolume;
    vol = leftVolume;
    if ((0 == leftVolume) && (0 == rightVolume))
    {
        //mute the HPL HPR
        value = XLLP_AC97_WM9713_MVR_MM | (1<<7);
    }
    else
    {
    	
        leftVolume = XLLP_AC97_WM9713_MAX_VOLUME - leftVolume;    //translate into attenuation value
        rightVolume = XLLP_AC97_WM9713_MAX_VOLUME - rightVolume;
        value = leftVolume << XLLP_AC97_WM9713_MVR_ML_SHIFT;
        value |= rightVolume;
    }
       status = XllpAc97ACodecWrite (pDeviceContext, HEADPHONE_VOLUME, value);    
        if (status == XLLP_ACODEC_SUCCESS) {
   		CurrentHeadsetVol = vol * 15/10;
   	}
    return status;    
 }


XLLP_ACODEC_ERROR_T XllpWm9713GetVol(XLLP_ACODEC_CONTEXT_T *pDeviceContext, VOL_PORT_TYPE_T port,  XLLP_UINT16_T *GainInDb)
{

	switch(port){
		case XLLP_MIC1:
		case XLLP_MIC2:
			*GainInDb = CurrentMastInputGain;
			break;
		case XLLP_HEADSET:
			*GainInDb = CurrentHeadsetVol;
			break;
		default:
			return XLLP_ACODEC_FEATURE_NO_SUPPORTED;
	}
	return XLLP_ACODEC_SUCCESS;
}

XLLP_ACODEC_ERROR_T XllpWm9713SetVol(XLLP_ACODEC_CONTEXT_T *pDeviceContext, VOL_PORT_TYPE_T port,  XLLP_UINT16_T GainInDb)
{
	XLLP_ACODEC_ERROR_T status = XLLP_ACODEC_SUCCESS;
	//the DB value is 1.5 step,
	//if the GainInDb == 1 DB will set as 0
	//So set DB as 2 at this situation 
	if(GainInDb == 1){
		GainInDb = 2;
		}

	switch(port){
		case XLLP_MIC1:
		case XLLP_MIC2:
			status = XllpWm9713SetMasterInputGain(pDeviceContext,GainInDb);
			break;
		case XLLP_HEADSET:
			status = XllpWm9713SetHeadsetVol(pDeviceContext,GainInDb);
			break;
		default:
			status = XLLP_ACODEC_FEATURE_NO_SUPPORTED;
	}
	return status;
}

static XLLP_ACODEC_ERROR_T  XllpWm9713SaveContext (XLLP_ACODEC_CONTEXT_T *pDeviceContext, XLLP_9713_CONTEXT_SAVE_T *pCodecContext)
{
    XLLP_UINT16_T value;
    XLLP_UINT16_T ii, jj, kk;

    jj = kk = 0;
    for (ii = 0; ii < 128; ii += 2)    
    {
        if (ii == xllp_9713_unsave[jj])
        {
            jj++;
            continue;
        }
        XllpAc97ACodecRead(pDeviceContext, ii, &value);             				 
        pCodecContext->wm9713RegisterContext [kk++] = value;
    }
    return XLLP_ACODEC_SUCCESS;
}

static XLLP_ACODEC_ERROR_T  XllpWm9713RestoreContext (XLLP_ACODEC_CONTEXT_T *pDeviceContext, XLLP_9713_CONTEXT_SAVE_T *pCodecContext)
{//check later: need special order to restore codec register?
    XLLP_UINT16_T value;
    XLLP_UINT16_T ii, jj, kk;

    jj = kk = 0;
    for (ii = 0; ii < 128; ii += 2)    
    {
        if (ii == xllp_9713_unsave[jj])
        {
            jj++;
            continue;
        }
        value = pCodecContext->wm9713RegisterContext [kk++];
        XllpAc97ACodecWrite(pDeviceContext, ii, value);             				 
    }
    return XLLP_ACODEC_SUCCESS;
}


//After audio_codec boot up, it should be in sleep mode to save power.
//So firstly, application should wake up audio codec
//first_time used to show if it is the first time to wake up audio codec
//if it is the first time, codec registers need not to be restored.
//static int first_time = 1;
//static ACODEC_STATE_T wm9713_codec_state =XLLP_CODEC_SLEEP;

//---------------------------------------------------------------------------------------------------------------
// Function: XllpWm9713Sleep
// Purpose: sleep the codec 
// Parameter:
// Returns: XLLP_ACODEC_ERROR_T
//operation: 1. make codec go to sleep
//note: this function should substitutes XllpWm9713SaveContext()
//---------------------------------------------------------------------------------------------------------------
 XLLP_ACODEC_ERROR_T XllpWm9713Sleep(XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{		
		XLLP_UINT16_T value;
	    XllpWm9713SaveContext(pDeviceContext, (XLLP_9713_CONTEXT_SAVE_T *)(pDeviceContext->pSaveMemory));
		//make the codec into sleep mode 
		XllpAc97ACodecRead(pDeviceContext, POWERDOWN_CTRL_STAT, &value);
 		value |= 0x1000 ;
		XllpAc97ACodecWrite(pDeviceContext, POWERDOWN_CTRL_STAT, value);
    		return XLLP_ACODEC_SUCCESS;
}
//
//---------------------------------------------------------------------------------------------------------------
// Function: XllpWm9713Wakeup
// Purpose:  wake up the codec and restore it to the original state
// Parameter:
// Returns:  XLLP_ACODEC_ERROR_T
//operation: 1. warm reset to enabled the ACLINK
//                2. restore the state
//note: this function should substitutes XllpWm9713RestoreContext()
//---------------------------------------------------------------------------------------------------------------
 XLLP_ACODEC_ERROR_T XllpWm9713Wakeup(XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{	
	P_XLLP_AC97ACODEC_T pAc97Reg = (P_XLLP_AC97ACODEC_T)(pDeviceContext->pCtrlReg); 
	//warm reset AClink to wake up the codec
	pAc97Reg->gcr |= XLLP_AC97_GCR_WARM_RESET_MSK;
	XllpOstDelayMilliSeconds(pDeviceContext->pOSTRegs, 5);
	//USE POWER_DOWN_1(2) to manage power
	//XllpAc97ACodecWrite(pDeviceContext, POWERDOWN_CTRL_STAT, 0x0);
	XllpWm9713RestoreContext (pDeviceContext, (XLLP_9713_CONTEXT_SAVE_T *)(pDeviceContext->pSaveMemory));
	return XLLP_ACODEC_SUCCESS;
}

#if 0
//---------------------------------------------------------------------------------------------------------------
// Function: XllpWm9713GetState 
// Purpose: get the codec state
// Parameter: pState return the codec state
// 
//---------------------------------------------------------------------------------------------------------------
XLLP_ACODEC_ERROR_T XllpWm9713GetState(XLLP_ACODEC_CONTEXT_T *pDeviceContext, ACODEC_STATE_T *pState)
{
	*pState = wm9713_codec_state;
	return  XLLP_ACODEC_SUCCESS;

}
#endif 

//---------------------------------------------------------------------------------------------------------------
// Function: XllpWm9713SpecificDeInit 
// Purpose: close all the power units and sleep codec
// Parameter:
// Returns:  0 do not support the control
//              >0 support the control
//operation: 1. save the state
//                2. make codec go to sleep
//note: this function should substitutes XllpWm9713SaveContext()
//---------------------------------------------------------------------------------------------------------------
XLLP_ACODEC_ERROR_T XllpWm9713SpecificDeInit (XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{//do later: shut down all power
	//u16 value=0;
	XLLP_UINT16_T value = 0;
	
	//close the power of all units
	XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, 0xffff);
	XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_2, 0xffff);
	XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
 	value &= ~(XLLP_AC97_9713_PWR_MBIAS);
	XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);

	return XLLP_ACODEC_SUCCESS;
}

static XLLP_ACODEC_ERROR_T SupportScenario(XLLP_UINT16_T *RoutMap)
{
	if(RoutMap[XLLP_INDEX_BT_HIFI_NEAR_IN_CONTROL]) 
		return XLLP_ACODEC_ROUTE_NO_SUPPORTED;

	if(RoutMap[XLLP_INDEX_VOICE_NEAR_IN_CONTROL]) 
		return XLLP_ACODEC_ROUTE_NO_SUPPORTED;

	if(RoutMap[XLLP_INDEX_BT_VOICE_NEAR_IN_CONTROL]) 
		return XLLP_ACODEC_ROUTE_NO_SUPPORTED;

	if(RoutMap[XLLP_INDEX_BT_MIC_IN_CONTROL]) 
		return XLLP_ACODEC_ROUTE_NO_SUPPORTED;

	if(RoutMap[XLLP_INDEX_MIC2_IN_CONTROL]) 
		return XLLP_ACODEC_ROUTE_NO_SUPPORTED;

	if(RoutMap[XLLP_INDEX_FAR_IN_CONTROL]) 
		return XLLP_ACODEC_ROUTE_NO_SUPPORTED;

	if(RoutMap[XLLP_INDEX_MIDI_IN_CONTROL]) 
		return XLLP_ACODEC_ROUTE_NO_SUPPORTED;

	if(RoutMap[XLLP_INDEX_FM_IN_CONTROL]) 
		return XLLP_ACODEC_ROUTE_NO_SUPPORTED;

	if(RoutMap[XLLP_INDEX_HIFI_NEAR_IN_CONTROL] & 
	~(XLLP_STEREO_HEAD_SET_OUT_ROUTE|XLLP_HIFI_NEAR_OUT_ROUTE) ) 
		return XLLP_ACODEC_ROUTE_NO_SUPPORTED;

	if(RoutMap[XLLP_INDEX_MIC1_IN_CONTROL]& 
	~(XLLP_STEREO_HEAD_SET_OUT_ROUTE|XLLP_HIFI_NEAR_OUT_ROUTE)) 
		return XLLP_ACODEC_ROUTE_NO_SUPPORTED;
	return XLLP_ACODEC_SUCCESS;
}

//---------------------------------------------------------------------------------------------------------------
// Function: XllpWm9713SetRoute
// Purpose:  set the codec route
// Parameter: 
// 			RoutMap__ the route map want to set
//			CurrentMap__ the current rout map
// explain:   RoutMap and CurrentMap
//               This function use the state of input devices state and ouput devices state to
//               to show the codec rout map. More information please refer to spec 
//               [linux-2.6.9-zylonite-alpha2] [ALSA] Subsystem High Level Design.doc
//               Every bit of the RoutMap/CurrentMap show one input/output device state.
//               1 means open; 0 means close. The bit NO is the device index. 
// Returns: XLLP_ACODEC_SUCCESS 
//              XLLP_ACODEC_ROUTE_NO_SUPPORTED
//Operation: Function will first scan the supported route map to find that if RoutMap is be supported. If the codec does not support the
//                route return XLLP_ACODEC_ROUTE_NO_SUPPORTED; If the route can be supported. set the route according the RoutMap,
//                CurrentMap can be used to speed the route set process. If some route is the same as before,  then this route need not set again.
//                
//---------------------------------------------------------------------------------------------------------------
XLLP_ACODEC_ERROR_T XllpWm9713SetRoute(XLLP_ACODEC_CONTEXT_T *pDeviceContext, XLLP_UINT16_T *RoutMap, XLLP_UINT16_T* CurrentMap)
{	
	XLLP_UINT16_T HifiNearInChangeState, HifiNearIn , CurrentOutState, OutState;
	XLLP_UINT16_T Mic1ChangeState, Mic1In, OutChangeState;
	//u16 val=0;
	XLLP_UINT16_T val = 0;
      if(XLLP_ACODEC_SUCCESS != SupportScenario(RoutMap)) {
		return(XLLP_ACODEC_ROUTE_NO_SUPPORTED) ;
      }
	HifiNearIn = RoutMap[XLLP_INDEX_HIFI_NEAR_IN_CONTROL];
	Mic1In = RoutMap[XLLP_INDEX_MIC1_IN_CONTROL];
	Mic1ChangeState =RoutMap[XLLP_INDEX_MIC1_IN_CONTROL]
			 ^ CurrentMap[XLLP_INDEX_MIC1_IN_CONTROL];
	HifiNearInChangeState = RoutMap[XLLP_INDEX_HIFI_NEAR_IN_CONTROL]
			 ^ CurrentMap[XLLP_INDEX_HIFI_NEAR_IN_CONTROL];

	//manage input device
	if (Mic1ChangeState ) {
		if(CurrentMap[XLLP_INDEX_MIC1_IN_CONTROL] && !RoutMap[XLLP_INDEX_MIC1_IN_CONTROL] ){
			//Mic change from on to off
			// close the Mic1
			//close mic PGA MA and mic pre-amp MPA
			XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_2, &val);
    			val |= (XLLP_AC97_9713_PWR_MA | XLLP_AC97_9713_PWR_MPA);
   			XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_2, val);
		}else if(!CurrentMap[XLLP_INDEX_MIC1_IN_CONTROL] && RoutMap[XLLP_INDEX_MIC1_IN_CONTROL])
		{
			//change from off to on
			//open Mic1
			XllpWm9713MicEnable (pDeviceContext, HANDSET_MIC_MASK);
			//open mic PGA MA and mic pre-amp MPA
    			XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_2, &val);
    			val &= ~(XLLP_AC97_9713_PWR_MA | XLLP_AC97_9713_PWR_MPA);
    			XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_2, val);
		}
	}//end MicChangeState
	if(HifiNearInChangeState){
		if(CurrentMap[XLLP_INDEX_HIFI_NEAR_IN_CONTROL] && !RoutMap[XLLP_INDEX_HIFI_NEAR_IN_CONTROL]){
			//from on to off
			XllpWm9713CloseHifiNearIn(pDeviceContext);
		}else if(!CurrentMap[XLLP_INDEX_HIFI_NEAR_IN_CONTROL] && RoutMap[XLLP_INDEX_HIFI_NEAR_IN_CONTROL] )
		{
			//from off to on
			XllpWm9713HiFiNearInEnable(pDeviceContext);
		}
	}//end HifiNearInChangeState	

	//manage ouput device
	OutState = RoutMap[XLLP_INDEX_HIFI_NEAR_IN_CONTROL] |RoutMap[XLLP_INDEX_MIC1_IN_CONTROL];
	CurrentOutState = CurrentMap[XLLP_INDEX_HIFI_NEAR_IN_CONTROL] |CurrentMap[XLLP_INDEX_MIC1_IN_CONTROL];
	OutChangeState = OutState ^ CurrentOutState;

	if(OutChangeState & XLLP_STEREO_HEAD_SET_OUT_ROUTE) {
		if(CurrentOutState & XLLP_STEREO_HEAD_SET_OUT_ROUTE ){
			//change from open to close
			XllpWm9713HeadsetDisable(pDeviceContext);
		}else {
			//change from close to open
			XllpWm9713HeadsetEnable(pDeviceContext);
		}
		
	}

	if(OutChangeState & XLLP_HIFI_NEAR_OUT_ROUTE) {
		if(CurrentOutState & XLLP_HIFI_NEAR_OUT_ROUTE ){
			//change from open to close
			XllpWm9713CloseHifiNearOut(pDeviceContext);
			
		}else {
			//change from close to open
			XllpWm9713OpenHifiNearOut (pDeviceContext);
		}
		
	}
	//manage path
	if(Mic1In & XLLP_HIFI_NEAR_OUT_ROUTE){
		//Mic1 to hifi_near
		XLLP_UINT16_T tem=0;
		XllpAc97ACodecRead(pDeviceContext, REC_ROUTE_MUX_SEL, &tem);
		tem &= 0xffc0;
    		XllpAc97ACodecWrite(pDeviceContext, REC_ROUTE_MUX_SEL, tem);
	}else{
	}

	if(Mic1In & XLLP_STEREO_HEAD_SET_OUT_ROUTE){
		//Open Mic1 to head set
		XllpAc97ACodecRead(pDeviceContext,REC_ROUTE_MUX_SEL,&val);
        	val &= 0x3fff;
        	XllpAc97ACodecWrite(pDeviceContext,REC_ROUTE_MUX_SEL,val);
	} else{
		//Close Mic1 to head set 
		XllpAc97ACodecRead(pDeviceContext,REC_ROUTE_MUX_SEL,&val);
        	val |= 0xc000;
       	 XllpAc97ACodecWrite(pDeviceContext,REC_ROUTE_MUX_SEL,val);
	}

	if(HifiNearIn & XLLP_STEREO_HEAD_SET_OUT_ROUTE ){
		//open hifi_near to head set
		XllpAc97ACodecWrite(pDeviceContext, DAC_PGA_VOL_ROUTE, 0x6808);
	} else {
		//close hifi_near to head set
		XllpAc97ACodecWrite(pDeviceContext, DAC_PGA_VOL_ROUTE, 0xe808);
		}

	#if 0
	if(Mic2In & XLLP_HIFI_NEAR_OUT_ROUTE){
		//Mic1 to hifi_near
		XLLP_UINT16_T tem=0;
		XllpAc97ACodecRead(pDeviceContext, REC_ROUTE_MUX_SEL, &tem);
		tem &= 0xffc0;
		tem |= 0x9;
    		XllpAc97ACodecWrite(pDeviceContext, REC_ROUTE_MUX_SEL, tem);
	}else{
	#endif
	return XLLP_ACODEC_SUCCESS; 
}


XLLP_ACODEC_ERROR_T XLLPACodecSetPenDownInterupt(XLLP_ACODEC_CONTEXT_T *pDeviceContext, XLLP_BOOL_T enable)
{//disable/enable pen down interrupt in the codec. This function is not implemented for Wm9713
    //because the pen down detection could not be disabled in codec
    return XLLP_ACODEC_SUCCESS;    
}

XLLP_ACODEC_ERROR_T XLLPACodecEnableTouch(XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{//enable touch functionality in the codec
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    //power setting
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
    value &= ~(XLLP_AC97_9713_PWR_PADCPD);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);
    
    //basic touch setting    
    status = XllpAc97ACodecWrite(pDeviceContext, DIGITIZER_3_WM13, 0xc008);
    status = XllpAc97ACodecWrite(pDeviceContext, DIGITIZER_2_WM13, 0x6);
    
    
    //9713 powerdown virtual gpio setting (polarity, sticky, wakeup)
    //9713 gpio 2(pin45) route to IRQ
    // Notes: Can use defaults for IRQ polarity, PENDOWN polarity in IRQ,
    //  sticky for PENDOWN in IRQ and wakeup for PENDOWN.    
    status = XllpAc97ACodecRead(pDeviceContext, GPIO_PIN_CFG, &value);
    value &= ~(0x4);
    status = XllpAc97ACodecWrite(pDeviceContext, GPIO_PIN_CFG, value);

    status = XllpAc97ACodecRead(pDeviceContext, GPIO_PIN_SHARING, &value);
    value &= ~(0x4);
    status = XllpAc97ACodecWrite(pDeviceContext, GPIO_PIN_SHARING, value);

    status = XllpAc97ACodecRead(pDeviceContext, GPIO_PIN_WAKEUP, &value);
    value |= (0x2000);
    status = XllpAc97ACodecWrite(pDeviceContext, GPIO_PIN_WAKEUP, value);
      
    status = XllpAc97ACodecRead(pDeviceContext, GPIO_PIN_STICKY, &value);
    value |= (0x2000);
    status = XllpAc97ACodecWrite(pDeviceContext, GPIO_PIN_STICKY, value);
    
    return status;    
}

XLLP_ACODEC_ERROR_T XLLPACodecDisableTouch(XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{//disable touch functionality in the codec
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    
    //power setting
    status = XllpAc97ACodecRead(pDeviceContext, POWER_DOWN_1, &value);
    value |= (XLLP_AC97_9713_PWR_PADCPD);
    status = XllpAc97ACodecWrite(pDeviceContext, POWER_DOWN_1, value);
    
    return status;
}

XLLP_ACODEC_ERROR_T XLLPAcodecGetADCSample(XLLP_ACODEC_CONTEXT_T *pDeviceContext, XLLP_UINT16_T *pSampleData, XLLP_UINT16_T ADCType, XLLP_BOOL_T *pPenDown)
{
    XLLP_ACODEC_ERROR_T	status = XLLP_ACODEC_SUCCESS;
    XLLP_UINT16_T value;
    XLLP_UINT32_T wait;
    
    if (ADCType == XLLP_TOUCH_SAMPLE_X)
    {
        value = 0x202;
    }
    else
    {//Y sample
        value = 0x204;
    }

    status = XllpAc97ACodecWrite(pDeviceContext, DIGITIZER_1_WM13, value);
    
    wait = 0;
    do 
    {
        status = XllpAc97ACodecRead(pDeviceContext, DIGITIZER_1_WM13, &value);
        if ( !(value & 0x200 ) )
        {
            break;
        }
    }while ( 100 > wait++ );
    
    status = XllpAc97ACodecRead(pDeviceContext, DIGITIZER_READ_BACK, &value);
    if (value & 0x8000)
    {//means pen down
        *pPenDown = XLLP_TRUE;
    }
    else
    {
        *pPenDown = XLLP_FALSE;
    }
    *pSampleData = value & 0xfff;
    
    return status;
}

//XLLPAcodecIsPenDow is not implemented here because driver could detect pen down by GPIO status
//XLLPACodecADCSampleEnable is not implemented here because it is not needed for 9713

//do later: Some functions implemented here are platform dependent, need to separate to another file later
