//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this sample source code is subject to the terms of the Microsoft
// license agreement under which you licensed this sample source code. If
// you did not accept the terms of the license agreement, you are not
// authorized to use this sample source code. For the terms of the license,
// please see the license agreement between you and Microsoft or, if applicable,
// see the LICENSE.RTF on your install media or the root of your tools installation.
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES OR INDEMNITIES.
//
//
// (C) Copyright 2006 Marvell International Ltd.
// All Rights Reserved
//
/* 
** INTEL CONFIDENTIAL
** Copyright 2000-2006 Intel Corporation All Rights Reserved.
**
** The source code contained or described herein and all documents
** related to the source code (Material) are owned by Intel Corporation
** or its suppliers or licensors.  Title to the Material remains with
** Intel Corporation or its suppliers and licensors. The Material contains
** trade secrets and proprietary and confidential information of Intel
** or its suppliers and licensors. The Material is protected by worldwide
** copyright and trade secret laws and treaty provisions. No part of the
** Material may be used, copied, reproduced, modified, published, uploaded,
** posted, transmitted, distributed, or disclosed in any way without Intel�s
** prior express written permission.
**
** No license under any patent, copyright, trade secret or other intellectual
** property right is granted to or conferred upon you by disclosure or
** delivery of the Materials, either expressly, by implication, inducement,
** estoppel or otherwise. Any license under such intellectual property rights
** must be express and approved by Intel in writing.
*/
/**
**  FILENAME:       xllp_pi2c_proc.h
**  
**  PURPOSE: contains all PI2C processor specific macros, typedefs, and 
**           prototypes.  Declares no storage.
**  
**  
******************************************************************************/
#ifndef __XLLP_PI2C_PROC_H
#define __XLLP_PI2C_PROC_H

#include "xllp_gpio.h"
#include "xllp_mfp.h"
#include "xllp_defs.h"
#include "xllp_ost.h"

#ifdef __cplusplus
extern "C"
{
#endif
/**************************
 * PI2C Bus Interface Unit *
 **************************/
// note offset at 0x40F5_00C0 for power i2c

typedef struct 
{
    	XLLP_VUINT32_T XLLP_IBMR;           		/* Bus monitor register */
	XLLP_VUINT32_T XLLP_IDBR;			/* Data buffer Register */
  	XLLP_VUINT32_T XLLP_ICR;			/* Global Control Register */
  	XLLP_VUINT32_T XLLP_ISR;			/* Status Register*/
  	XLLP_VUINT32_T XLLP_ISAR;			/* Slave address register */
} XLLP_PI2C_T, *P_XLLP_PI2C_T;

/*  bus monitor register */
#define	XLLP_IBMR_SDA		XLLP_BIT_0  /* reflects the status of SDA pin */
#define	XLLP_IBMR_SCL		XLLP_BIT_1 /* reflects the status of SCL pin */

/* data buffer regiter mask */
#define	XLLP_IDBR_ADDR		0xFF;  /*buffer for I2C bus send/receive data */
#define	XLLP_IDBR_MODE		XLLP_BIT_0	
/* Control Register */
#define	XLLP_ICR_START		XLLP_BIT_0 /* 1:send a Start condition to the I2C when in master mode */
#define	XLLP_ICR_STOP		XLLP_BIT_1 /* 1:send a Stop condition after next data byte transferred on I2C bus in master mode */
#define	XLLP_ICR_ACKNACK	XLLP_BIT_2  /* Ack/Nack control: 1:Nack, 0:Ack (negative or positive pulse) */
#define	XLLP_ICR_TB			XLLP_BIT_3  /* 1:send/receive byte, 0:cleared by I2C unit when done */
#define	XLLP_ICR_MA			XLLP_BIT_4  /* 1:I2C sends STOP w/out data permission, 0:ICR bit used only */
#define	XLLP_ICR_SCLEA		XLLP_BIT_5  /* I2C clock output: 1:Enabled, 0:Disabled. ICCR configured before ! */
#define	XLLP_ICR_UIE		XLLP_BIT_6 /* I2C unit: 1:Enabled, 0:Disabled */
#define	XLLP_ICR_GCD		XLLP_BIT_7  /* General Call: 1:Disabled, 0:Enabled */
#define	XLLP_ICR_ITEIE		XLLP_BIT_8  /* 1: IDBR Transmit Empty Interrupt Enable */
#define	XLLP_ICR_DRFIE		XLLP_BIT_9  /* 1: IDBR Receive Full Interrupt Enable */
#define	XLLP_ICR_BEIE		XLLP_BIT_10  /* 1: Bus Error Interrupt Enable */
#define	XLLP_ICR_SSDIE		XLLP_BIT_11 /* 1: Slave Stop Detected Interrupt Enable */
#define	XLLP_ICR_ALDIE		XLLP_BIT_12  /* 1: Arbitration Loss Detected Interrupt Enable */
#define	XLLP_ICR_SADIE		XLLP_BIT_13  /* 1: Slave Address Detected Interrupt Enable */
#define	XLLP_ICR_UR			XLLP_BIT_14  /* 1: I2C unit reset */
#define XLLP_ICR_FM   		XLLP_BIT_15 /* 1: Fast mode - 400 KBit/sec. operation. Default is 100 KBit/sec */

/* Status Register */
#define	XLLP_ISR_RWM		XLLP_BIT_0  /* 1: I2C in master receive = slave transmit mode */
#define	XLLP_ISR_ACKNACK	XLLP_BIT_1  /* 1: I2C received/sent a Nack, 0: Ack */
#define	XLLP_ISR_UB			XLLP_BIT_2  /* 1: Processor's I2C unit busy */
#define	XLLP_ISR_IBB		XLLP_BIT_3  /* 1: I2C bus busy. Processor's I2C unit not involved */
#define	XLLP_ISR_SSD		XLLP_BIT_4  /* 1: Slave Stop detected (when in slave mode: receive or transmit) */
#define	XLLP_ISR_ALD		XLLP_BIT_5  /* 1: Arbitration Loss Detected */
#define	XLLP_ISR_ITE		XLLP_BIT_6  /* 1: Transfer finished on I2C bus. If enabled in ICR, interrupt signaled */
#define	XLLP_ISR_IRF		XLLP_BIT_7  /* 1: IDBR received new byte from I2C bus. If ICR, interrupt signaled */
#define	XLLP_ISR_GCAD		XLLP_BIT_8  /* 1: I2C unit received a General Call address */
#define	XLLP_ISR_SAD		XLLP_BIT_9  /* 1: I2C unit detected a 7-bit address matching the general call or ISAR */
#define	XLLP_ISR_BED		XLLP_BIT_10  /* Bit set by unit when a Bus Error detected */

/*  slave address mask */
#define	XLLP_ISAR_ADDR		0x7F;	/*  7-bit(6:0) add to which I2C unit responses to in slave/receive mode*/

#define XLLP_ICR_INIT_VALUE (XLLP_ICR_UIE|XLLP_ICR_SCLEA)

#define XLLP_I2C_NO_STOP     0    /* Don't issue stop bit */
#define XLLP_I2C_STOP        1    /* Issue stop bit */

/* Processor PI2C Device ID */
#define XLLP_PI2C_SLAVE_ID 0

/*Speed mode*/
typedef enum {
	XLLP_PI2C_FAST_SPEED = 1,
	XLLP_PI2C_NORMAL_SPEED
} XLLP_PI2C_SPEED_T;

#ifdef __cplusplus
}
#endif
#endif /* XLLP_PI2C_PROC_H */
