/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/**
**   xllp_tsi.c
**
**   Valid for Monahans touch screen interface
**
*****************************************************************************/
#ifndef XLLP_TSI_H
#define XLLP_TSI_H

#include "xllp_defs.h"
#include "xllp_ost.h"
typedef struct
{
    XLLP_VUINT32_T XLLP_ADCD;
    XLLP_VUINT32_T XLLP_ADCS;
    XLLP_VUINT32_T XLLP_ADCE;
    XLLP_VUINT32_T XLLP_ADCP;
} XLLP_TSI_REGS_T;

typedef enum
{
	XLLP_TSI_EVENT_NONE = 0,
	XLLP_TSI_EVENT_TOUCH = 1,
	XLLP_TSI_EVENT_LIFT = 2
}XLLP_TSI_EVENT_T;

typedef enum
{
	XLLP_TSI_PEN_UNKNOW =0 ,
	XLLP_TSI_PEN_DOWN = 1,
	XLLP_TSI_PEN_UP = 2
}XLLP_TSI_PEN_STATE_T;

//ADCD
#define XLLP_TSI_SD              (0x1 << 31)
#define XLLP_TSI_VAL             (0x1 << 30)
#define XLLP_TSI_SV                (0x1<<29)
#define MASK_XLLP_TSI_ADC_DATA1  (0xfff << 15)
#define MASK_XLLP_TSI_ADC_MUX0   (0x7 << 12)
#define MASK_XLLP_TSI_ADC_DATA0  (0xfff)

//ADCS
#define XLLP_TSI_RUN             (0x1 << 31)
#define XLLP_TSI_CC              (0x1 << 30)
#define XLLP_TSI_SD_EN           (0x1 << 29)
#define XLLP_TSI_VAL_EN          (0x1 << 28)
#define XLLP_TSI_XYM             (0x1 << 27)
#define XLLP_TSI_PRED            (0x5 << 20)  //this value too long may cause 
				//unwanted pen down interrupt after ADC conversion
#define XLLP_TSI_RESOU_6BIT      (0x1 << 19)
#define XLLP_TSI_ADC_MUX (0x7)<<16
#define XLLP_TSI_ADC_DELAY       (0xff)

//ADCE
#define XLLP_TSI_BIAS_EN      (0x1<<0)
#define XLLP_TSI_P1        (0x1<<7) 
#define XLLP_TSI_P2        (0x1<<8)
#ifdef __cplusplus
extern "C"{
#endif

//-----------------------------------------------------------------------------------------
// Function: InitializeTSI
// Purpose:  Initialize the TSI 
// 1.	clear all the bits of register ADCD
// 2.	enable pen down event detection 
// 3.	enable valid conversion event detection
// 4.	disable consecutive X and Y measurements
// 5.	set PRED and ADC_DELAY time
// 6.	set resolution  value
//Input Parameters:
// XLLP_TSI_REGS *pTSIregs __ the TSI register base address
//Output Parameters:
//Returns:
//--------------------------------------------------------------------------------------------
extern void XllpInitializeTSI(XLLP_TSI_REGS_T *pTSIregs);

//-----------------------------------------------------------------------------------------
// Function: PowerOnTSI
// Purpose:  Power on /off the ADC of TSI 
// Input Parameters: 
// XLLP_TSI_REGS *pTSIregs __   TSI register base address
// XLLP_BOOL_T enable __  XLLP_TRUE: power on the ADC;
//  XLLP_FALSE: power off the ADC  
// Output Parameters:
// Returns:
//--------------------------------------------------------------------------------------------
extern void XllpPowerOnTSI(XLLP_TSI_REGS_T *pTSIregs, XLLP_BOOL_T enable);

//-----------------------------------------------------------------------------------------
// Function: PowerOnTSI
// Purpose:  This function is used to decide if the pen down event has happened. 
// If the pen down event happends the function will clear SD bit of register ADCD. 
// Input Parameters:
// XLLP_TSI_REGS *pTSIregs __   TSI register base address
// Output Parameters:
//  XLLP_BOOL_T * pPenDown  __ XLLP_TRUE: pen down event happened.
//                                               XLLP_FALSE: pen down event did not happen. 

// Returns:
//XLLP_STATUS_UNSUPPORTED __ the pen down event detection is not enabled.
//XLLP_STATUS_SUCCESS

//--------------------------------------------------------------------------------------------
extern XLLP_STATUS_T XllpGetPenEvent(XLLP_TSI_REGS_T *pTSIregs, XLLP_TSI_EVENT_T *pPenEvent);

//-----------------------------------------------------------------------------------------
// Function:ReadTSI 
//
// Purpose: 
// The function can get the position value(x , y) from TSI. 
// The operation process is that fist, it will disable the pen 
// down event detection and start a sample routine, 
// then poll the x and y value, after these value has been read, 
// pen down event detection will be enabled again.
// If the pen down event happends the function will clear SD bit of register ADCD. 
//
// Input Parameters:
// XLLP_TSI_REGS *pTSIregs __   TSI register base address
// Output Parameters:
// XLLP_UINT16_T *x __ return x value; if something is wrong, 0xffff will be set to the value
// XLLP_UINT16_T *y __ return y value; if something is wrong, 0xffff will be set to the value


// Returns:
//XLLP_STATUS_TIME_OUT 
//XLLP_STATUS_SUCCESS: 

//--------------------------------------------------------------------------------------------
extern XLLP_STATUS_T XllpReadTSI(XLLP_TSI_REGS_T *pTSIregs,
				P_XLLP_OST_T pOstregs, 
				XLLP_UINT16_T *x, 
				XLLP_UINT16_T *y,
				XLLP_TSI_PEN_STATE_T *pPenState
				);

#ifdef __cplusplus
}
#endif

#endif
