/*++
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
--*/


/**
**  FILENAME:   xllp_u2d.h
**
**  PURPOSE:   	This files is the header file for USB2.0 Device controller (U2D)
**
**
******************************************************************************/



#ifndef XLLP_U2D_HEADER
#define XLLP_U2D_HEADER

#include "xllp_dmac.h"
#include "xllp_udc.h"
#include "xllp_gpio_proc.h"
#include "xllp_mfp_proc.h"

#define XLLP_U2DCR_UDE (1<<0)
#define XLLP_U2DCR_UDA	(1<<1)
#define XLLP_U2DCR_UDR (1<<2)
#define XLLP_U2DCR_EMCE (1<<3)
#define XLLP_U2DCR_DWRE (1<<16)
#define XLLP_U2DCR_SMAC (1<<17)
#define XLLP_U2DCR_HS	(1<<18)
#define XLLP_U2DCR_CC	(1<<19)
#define XLLP_U2DCR_ADD	(1<<20)
#define XLLP_U2DCR_ABP	(1<<21)

// Active interface and Alternate Interface
#define XLLP_U2DCR_AAISN_SHIFT  4               // Shift and a mask for the Alternate Interface
#define XLLP_U2DCR_AAISN_MASK   ( 0xFU << XLLP_U2DCR_AAISN_SHIFT )  // Settings  (0-7)
#define XLLP_U2DCR_AIN_SHIFT    8               // Shift and a mask for the Interface
#define XLLP_U2DCR_AIN_MASK     ( 0xFU << XLLP_U2DCR_AIN_SHIFT )    // Number    (0-7)
#define XLLP_U2DCR_ACN_SHIFT    12              // Shift and a mask for the Configuration
#define XLLP_U2DCR_ACN_MASK     ( 0xFU << XLLP_U2DCR_ACN_SHIFT )    // Number    (0-3)


// UDC Endpoint 0 Control/Status Register (UDCCSR0)
#define XLLP_U2DCCSR0_OPC		( 0x1U << 0 )	// OUT packet to endpoint zero received
#define XLLP_U2DCCSR0_IPR		( 0x1U << 1 )	// Packet has been written to endpoint zero FIFO
#define XLLP_U2DCCSR0_FTF		( 0x1U << 2 )	// Flush the Tx FIFO
#define XLLP_U2DCCSR0_SST		( 0x1U << 4 )	// UDC sent stall handshake
#define XLLP_U2DCCSR0_FST		( 0x1U << 5 )	// Force the UDC to issue a stall handshake
#define XLLP_U2DCCSR0_RNE		( 0x1U << 6 )	// There is unread data in the Rx FIFO
#define XLLP_U2DCCSR0_SA			( 0x1U << 7 )	// Current packet in FIFO is part of UDC setup command
#define XLLP_U2DCCSR0_IPA		( 0x1U<<8)      // In Packet Adjusted

// UDC Endpoint Control/Status Registers A-X 
#define XLLP_U2DCCSR_FS			( 0x1U << 0 )	// FIFO needs service
#define XLLP_U2DCCSR_PC			( 0x1U << 1 )	// Packet Complete
#define XLLP_U2DCCSR_EFE			( 0x1U << 2 )	// Endpoint FIFO error
#define XLLP_U2DCCSR_DME			( 0x1U << 3 )	// DMA Enable
#define XLLP_U2DCCSR_SST			( 0x1U << 4 )	// Sent STALL
#define XLLP_U2DCCSR_FST			( 0x1U << 5 )	// Force STALL
#define XLLP_U2DCCSR_BNE		    ( 0x1U << 6 )	// Buffer not empty/full
#define XLLP_U2DCCSR_SP			( 0x1U << 7 )	// Short Packet
#define XLLP_U2DCCSR_FEF			( 0x1U << 8 )	// Flash Endpoint FIFO
#define XLLP_U2DCCSR_DPE			( 0x1U << 9 )	// Data Packet Error
#define XLLP_U2DCCSR_BF_BE			( 0x1U << 10 )
#define XLLP_U2DCCSR_RW_BITS_MASK	(XLLP_UDC_UDCCSR_DME|XLLP_UDC_UDCCSR_FST)

#define XLLP_U2DDMA_CMD_XFRDIS		(0x1U<<31)
#define XLLP_U2DDMA_CMD_STARTIRQEN	(0x1U<<22)
#define XLLP_U2DDMA_CMD_ENDIRQEN	(0x1U<<21)
#define XLLP_U2DDMA_CMD_PACKCOMP	(0x1U<<13)

#define XLLP_U2DDAM_CMD_LEN_MASK	(0x3FF)

/// Enumerate USB2.0 Endpoints
typedef enum XLLP_U2D_EP_E
{
    U2D_ENDPOINT_0 = 0,
    U2D_ENDPOINT_A,
    U2D_ENDPOINT_B,
    U2D_ENDPOINT_C,
    U2D_ENDPOINT_D,
    U2D_ENDPOINT_E,
    U2D_ENDPOINT_F,
    U2D_ENDPOINT_G,
    U2D_ENDPOINT_INVALID,
    XLLP_U2D_USB_MAX_EP_NUM=U2D_ENDPOINT_INVALID
} XLLP_U2D_EP_T;


/// U2DICR Register bit index. U2DICR and U2DISR bit location is the same. 
typedef enum XLLP_U2D_INTERRUPTS_E
{
    U2D_INT_ENDPOINT_0  = 0,     
    U2D_INT_ENDPOINT_A,
    U2D_INT_ENDPOINT_B,
    U2D_INT_ENDPOINT_C,
    U2D_INT_ENDPOINT_D,
    U2D_INT_ENDPOINT_E,
    U2D_INT_ENDPOINT_F,
    U2D_INT_ENDPOINT_G,

    U2D_INT_RESET		 = 26,		///< Reset
    U2D_INT_SUSPEND     = 27,		///< Suspend
    U2D_INT_RESUME      = 28,		///< Resume
    U2D_INT_USOF      	 = 29,		///< Micro Start of Frame(HS Only)
    U2D_INT_SOF         = 30,		///< Start of Frame
    U2D_INT_CONFIG      = 31 		///< Configuration Change
} XLLP_U2D_INTERRUPTS_T;


/// U2D interrutp type include U2D DMA
typedef enum XLLP_U2D_EP_INTERRUPT_TYPE_E
{
    U2D_SHORT_PACKET_COMPL_INT = 1,  	///< Short Packet Complete Interrupt
    U2D_PACKET_COMPL_INT = 1<<1,  	///< Complete Interrupt
    U2D_FIFO_ERROR_INT   = 1<<2,	///< FIFO Error Interrupt

	U2D_DMA_STOP		 = 1<<29,   ///< U2D DMA Stop interrupt enable
    U2D_DMA_EOR			 = 1<<28,	///< U2D DMA End of Receive Interrupt enable
    U2D_DMA_EORJMP	     = 1<<27,	///< Jump to the Next Descriptor
    U2D_DMA_EORSTOP  	 = 1<<26,	///< Stope the channel on an EOR
    U2D_DMA_RAS			 = 1<<23,   ///< Request After Channel Stopped Intterupt enable
	U2D_DMA_ALL			 = (U2D_DMA_STOP|U2D_DMA_EOR|U2D_DMA_EORJMP|U2D_DMA_EORSTOP|U2D_DMA_RAS),

	U2D_INTR_ALL		 = (U2D_DMA_ALL|U2D_SHORT_PACKET_COMPL_INT|U2D_PACKET_COMPL_INT|U2D_FIFO_ERROR_INT)
    
} XLLP_U2D_EP_INTERRUPT_TYPE_T;

#if 0 //only for create document
/// Enumerate U2D Transfers Types. The same as USB1.1 Controller
typedef enum XLLP_U2D_EP_TYPE_E
{
    CNTRL   = 0x00,
    ISO     = 0x01,
    BULK    = 0x02,
    INTERPT = 0x03
} XLLP_UDC_EP_TYPE_T;

/// Enumerate UDC Endpoint Direction. The same as USB1.1 Controller
typedef enum XLLP_UDC_EP_DIR_E
{
    XLLP_USB_OUT = 0x00,
    XLLP_USB_IN  = 0x01,
} XLLP_UDC_EP_DIR_T;
/// Enumerate UDC Endpoint Max. Packet Size. The same as USB1.1 Controller
typedef enum XLLP_UDC_EP_MPS_E
{
    MPS_CNTRL_8  = 8,
    MPS_CNTRL_16 = 16,
    MPS_BULK_8  = 8,
    MPS_BULK_16 = 16,
    MPS_BULK_32 = 32,
    MPS_BULK_64 = 64,
    MPS_INT_8   = 8,
    MPS_INT_16  = 16, 
    MPS_INT_32  = 32,
    MPS_INT_64  = 64,
    MPS_ISO_256 = 256,
    MPS_ISO_512 = 512,
    MPS_ISO_1023 = 1023, 
    MPS_ISO_LSB_256 = 0x00, 
    MPS_ISO_MSB_256 = 0x01,
    MPS_ISO_LSB_512 = 0x00, 
    MPS_ISO_MSB_512 = 0x02,
    MPS_ISO_LSB_1023 = 0xff,
    MPS_ISO_MSB_1023 = 0x03 
} XLLP_UDC_EP_MPS_T;

/// Enumerate UDC Endpoint Interrupts. The same as USB1.1 Controller


typedef enum XLLP_UDC_XFER_RESULT_E
{
    UDC_XFER_CONTINUE	= 0,
    UDC_XFER_FINISH		= 1,
	UDC_XFER_FINISH_AND_MORE = 3,

	UDC_XFER_UNSUPPORT_EP0 =0x1000
} XLLP_UDC_XFER_RESULT_T;


typedef enum XLLP_UDC_XFER_RESULT_E
{
    UDC_XFER_CONTINUE	= 0,
    UDC_XFER_FINISH		= 1,
	UDC_XFER_FINISH_AND_MORE = 3,

	UDC_XFER_UNSUPPORT_EP0 =0x1000
} XLLP_UDC_XFER_RESULT_T;

/**
 * U2D DMA Descriptor. The same with general DMA Discriptor
 **/
typedef struct 
{
    XLLP_VUINT32_T   ddadr;  ///< descriptor address reg
    XLLP_VUINT32_T   dsadr;  ///< source address register
    XLLP_VUINT32_T   dtadr;  ///< target address register
    XLLP_VUINT32_T   dcmd;   ///< command address register
}XLLP_DMAC_DESCRIPTOR_T, *P_XLLP_DMAC_DESCRIPTOR_T;

#endif

/// U2D.0 Registers
typedef struct XLLP_U2D_REGISTERS_S {
    XLLP_VUINT32_T u2dctrl;           					///< U2D Control register
    XLLP_VUINT32_T u2dicr;          					///< U2D Interrupt Control Register
    XLLP_VUINT32_T reserverd0;         
    XLLP_VUINT32_T u2disr;         						///< U2D Interrrupt Status Register 0
    XLLP_VUINT32_T reserverd1;    
    XLLP_VUINT32_T u2dfnr;          					///< U2D Frame Number Register
    XLLP_VUINT32_T reserved2[58];   					///< Reserved 0x40600018-0x406000FF 

	
    XLLP_VUINT32_T u2dcsr[XLLP_U2D_USB_MAX_EP_NUM];     ///< U2D Control/Status registers 0, A-G 
    
    XLLP_VUINT32_T reserved3[56];   					///< Reserved 0x40600120-0x406001FF 
    
    XLLP_VUINT32_T u2dbcr[XLLP_U2D_USB_MAX_EP_NUM];     ///< U2D Byte Count Registers 0, A-G
    
    XLLP_VUINT32_T reserved4[56];  						///< Reserved 0x40600120-0x406002FF 

	XLLP_VUINT32_T u2ddr[1];       						///< U2D Data Registers 0

	XLLP_VUINT32_T reserved5[63];   					///< Reserved 0x40600304-0x406003FF,
    
    XLLP_VUINT32_T u2dcr[XLLP_U2D_USB_MAX_EP_NUM];      ///< U2D Config Register 0 , A-G
                                   						///< @remarks u2dcr[0] is Reserved

	XLLP_VUINT32_T reserved6[56];   					///< Reserved 0x40600420-0x406004FF 

	XLLP_VUINT32_T u2dsca;								///< U2D Setup Command 
	XLLP_VUINT32_T u2dne[XLLP_U2D_USB_MAX_EP_NUM];		///< U2D Endpoint information register 0, A-G

	XLLP_VUINT32_T reserved7[695];   					///< Reserved 0x40600524-0x40600FFF 

	//U2D DMA Register
	XLLP_VUINT32_T u2ddmacsr[XLLP_U2D_USB_MAX_EP_NUM];	///< U2D DMA Control/Status register 0, A-G
	XLLP_VUINT32_T reserved8[24];   					///< Reserved 0x40601020-0x4060107F 

	XLLP_VUINT32_T u2ddmacr;							///< U2DDMA Control register
	XLLP_VUINT32_T reserved9[27];   					    ///< Reserved 0x40601084-0x406010DF
	
	XLLP_VUINT32_T u2dmaint;							///< U2D DMA interrupt register		
	XLLP_VUINT32_T reserved10[3];   					    ///< Reserved 0x406010F4-0x406010FF
	
	XLLP_VUINT32_T u2ddmabr[XLLP_U2D_USB_MAX_EP_NUM];	///< U2D DMA Branch register 0, A-G
	XLLP_VUINT32_T reserved11[56];   					///< Reserved 0x40601120-0x406011FF
	XLLP_DMAC_DESCRIPTOR_T ddg[XLLP_U2D_USB_MAX_EP_NUM];///< U2D DMA Descriptor Group for channel 0-7
	
                                   						
} XLLP_U2D_REGISTERS_T, *P_XLLP_U2D_REGISTERS_T;

#define U2D_OFFSET(x) ((XLLP_UINT32_T)&(((P_XLLP_U2D_REGISTERS_T)0)->x))

//#define LINUX_DEBUG

#ifdef WINCE_DEBUG
void EdbgOutputDebugString(char *, ...);
#define U2D_DBG(x) EdbgOutputDebugString x
#endif

#ifdef LINUX_DEBUG
#define U2D_DBG(x) printk x
#endif

#ifndef U2D_DBG
#define U2D_DBG(x)
#endif

/// U2D Endpoint Configure 
typedef struct XLLP_U2D_EP_CONFIG_S
{
/**
USB Endpoint Type:
11 C Interrupt
10 C Bulk
01 C Isochronous
00 C Not used
*/

	XLLP_UDC_EP_TYPE_T ET;

/**	
USB Endpoint Direction:
The Endpoint Direction bit determines whether the endpoint is a an In
endpoint, which sends data from the U2D to the USB host controller, or an
Out endpoint, which receives data from the USB host controller.
0 C Out
1 C In
*/
	 XLLP_UDC_EP_DIR_T	ED;

/**
Endpoint Number:
Endpoint number is used to define the endpoint number. Each Endpoint A -
G can be programmed to respond to USB endpoint numbers 1 - 15.
More than one endpoint can be programmed to the same endpoint number,
but the endpoints must not be active in the same configuration. For
example, if Endpoint A and Endpoint B can both be assigned endpoint
number 1, but with Endpoint A in Configuration 1, and Endpoint B in
Configuration 2.
1 - 15 C USB Endpoint Number
*/
	XLLP_UINT32_T 	EN;
/**
Configuration Number:
Configuration number must be set before the U2D is enabled. Refer to
Figure 11-11 for more details on the Configuration register programming
sequence. Each Endpoint A - G can be assigned to Configuration 1-15.
Each Endpoint A - G can be assigned to only one configuration/interface/
alternate interface setting combination, and will be active only when the
USB host controller configures the U2D for that configuration, interface, and
alternate interface setting.
1 - 15 C USB Configuration Number
*/
	XLLP_UINT32_T 	CN;
/**
Interface Number:
Interface number must be set before the U2D is enabled. Refer to
Figure 11-16 for more details on the Configuration register programming
sequence.Each Endpoint A - G can be assigned to Interface 0 - 15. Each
Endpoint A - G can be assigned to only one configuration/interface/
alternate interface setting combination, and will be active only when the
USB host controller configures the U2D for that configuration, interface, and
alternate interface setting.
0- 15 C USB Interface Number
*/
	XLLP_UINT32_T	InterfaceNumber;
/**
Alternate Interface Number:
AIN alternate interface setting numbers for Endpoints A - G must be set
before the UCD is enabled. Refer to Figure 11-16 for more details on the
Configuration register programming sequence. Each Endpoint A - G can be
assigned to alternate interface setting 0 - 15. Each Endpoint A - G can be
assigned to only one configuration/interface/alternate interface setting
combination, and will be active only when the USB host controller
configures the U2D for that configuration, interface, and alternate interface
setting.
0 - 15 C USB Interface Alternate Interface Setting Number
*/
	XLLP_UINT32_T		  AISN;
/**
Maximum Packet Size:
Valid values are defined by endpoint type:
For interrupt endpoint: 1 - 64 (1-1024 for high speed)
For bulk endpoint: 8, 16, 32, 64 (512 for high speed)
For isochronous endpoint: 1 - 1023 (1-1024 for high speed)
*/
	XLLP_UINT32_T		   MPS;
/**
High Bandwidth Commands per Micro-frame:
For high-speed isochronous and interrupt In endpoints only this field
represents how many accesses per micro-frame is issued. Values of 0 or 1
represent a normal bandwidth of a single access per micro-frame. Values of
2 or 3 represent a high bandwidth endpoint with the corresponding number
of accesses each micro-frame. This field is ignored during low speed mode,
for Out endpoints, and for bulk and control endpoints.
*/
	XLLP_UINT32_T			HBW;
/**
Size of the FIFO for endpoint data. Must be set larger than the endpoints
max packet size, but is not restricted to an integer of max packet size.
Endpoints with small max packet sizes may require a FIFO depth several
times their max packet size in order to not be limited by system latency,
while endpoints with large max packet sizes may tolerate a FIFO depth not
even twice their max packet size and not be limited by system latency.
*/
	XLLP_UINT32_T			 bufferSize;     
	
//U2D DMA Descriptor
	XLLP_DMAC_DESCRIPTOR_T * vDmaDescriptor; ///< U2D DMA Descritpor virtual address. It must be 16 byte alignment
	XLLP_UINT32_T			 pDmaDescriptor; ///< U2D DMA Descritpor phyical address. It must be 16 byte alignment
	XLLP_UINT32_T			 nDescriptors;	 ///< Descriptor numbers in buffer vDmaDescriptor
}
XLLP_U2D_EP_CONFIG, *P_XLLP_U2D_EP_CONFIG;

///U2D Transfer data struct
typedef struct XLLP_U2D_TRANSFER_S
{								
	XLLP_U2D_EP_T udcEndpointNum;			///<!End point Number 	
    XLLP_UINT32_T physicalAddress;			///<!Physical address. Physcial continue. 
	XLLP_UINT32_T len;						///<!Buffer Size
	XLLP_UDC_EP_DIR_T direct;				///<!Transfer direct IN or OUT
	XLLP_BOOL_T isZeroPackage;
				///<!Add zero package after transfer. Only valid for IN transfer.
	XLLP_BOOL_T EnableEndIrq;				///<!Enalbe/Disable End Interrupt at last descriptor
	
}XLLP_U2D_TRANSFER_T, *P_XLLP_U2D_TRANSFER;


/// U2D.0 Handle. 
/// @remarks, please don't change any field directly, please use macor or function to access field
typedef struct XLLP_U2D_S 
{
    P_XLLP_U2D_REGISTERS_T  pRegsBase;          ///< Pointer to U2D.0's registers
    P_XLLP_VUINT32_T pMfpRegBase; 				///< Pointer to the MFP base registers
	P_XLLP_GPIO_T pGPIORegs;					///< Pointer to the GPIO base registers
	P_XLLP_MFP_RM_DB_ID_T pMfpRmDb;				///< Pointer to the resource base
    
	XLLP_U2D_EP_CONFIG ConfigTable[XLLP_U2D_USB_MAX_EP_NUM];  ///< Endpoints Config
	XLLP_U2D_TRANSFER_T u2dEpXferTable[XLLP_U2D_USB_MAX_EP_NUM];  ///< Used to support the transfers
    XLLP_UINT8_T    usbConfigNumActive;         ///< Active Usb Configuration Number: (1-3)
    XLLP_UINT8_T    usbInterfaceNumChanged;      ///< Changed Usb Interface Number: (1-7)
    XLLP_UINT8_T    usbIntAltSettingsNumSelected; ///< Selected Usb Interface Alternate Settings Number: (1-7)
} XLLP_U2D_T, *P_XLLP_U2D_T;

#ifdef __cplusplus
extern "C"

{
#endif 

XLLP_STATUS_T XllpU2DInitHandle(P_XLLP_U2D_T pU2DHandle, P_XLLP_U2D_REGISTERS_T pU2DBase, P_XLLP_VUINT32_T pMfpRegBase,
								P_XLLP_GPIO_T pGPIORegs,P_XLLP_MFP_RM_DB_ID_T pMfRmDb);
void XllpU2DPlatformInit(P_XLLP_U2D_T pU2DHandle);

void XllpU2DPlatformUninit(P_XLLP_U2D_T pU2DHandle );

XLLP_STATUS_T XllpU2DClearAllConfigure(P_XLLP_U2D_T pU2DHandle);

XLLP_STATUS_T XllpU2DEnable (P_XLLP_U2D_T pU2DHandle);

XLLP_STATUS_T XllpU2DDisable (P_XLLP_U2D_T pU2DHandle);


/*!
	Software simulate disconnect/connect USB Calbe

@param[in]   pU2DHandle	  U2DHandle, which is initilizated in XllpU2DInitHandle
@param[in]	 connect	 	- TRUE: Connect. 
							- FALSE Disconnect						

@return
	- XLLP_STATUS_SUCCESS	Success
	- Other value			Fail
*/
XLLP_STATUS_T XllpU2DBusDevSoftConnect(P_XLLP_U2D_T pU2DHandle, XLLP_BOOL_T connect);




XLLP_STATUS_T XllpU2DEnableInterrupt (P_XLLP_U2D_T pU2DHandle, 
                             XLLP_U2D_INTERRUPTS_T udcEndpointNum,
                             XLLP_U2D_EP_INTERRUPT_TYPE_T interruptType);


XLLP_STATUS_T XllpU2DDisableInterrupt  (P_XLLP_U2D_T pU2DHandle, 
                             XLLP_U2D_INTERRUPTS_T udcEndpointNum,
                             XLLP_U2D_EP_INTERRUPT_TYPE_T interruptType);


XLLP_STATUS_T XllpU2DConfigChangeCallback (P_XLLP_U2D_T pU2DHandle);



XLLP_STATUS_T XllpU2DForceEndpointStall (P_XLLP_U2D_T pU2DHandle,
                                XLLP_U2D_EP_T udcEndpointNum);


XLLP_STATUS_T XllpU2DFlushEndpointFifo (P_XLLP_U2D_T pU2DHandle,
                                XLLP_U2D_EP_T udcEndpointNum);


XLLP_STATUS_T XllpU2DStartTransfer (P_XLLP_U2D_T pU2DHandle,
				  P_XLLP_U2D_TRANSFER pTransfer);


XLLP_STATUS_T XllpU2DGetTransferStatus (P_XLLP_U2D_T pU2DHandle,
									    XLLP_U2D_EP_T udcEndpointNum,
										XLLP_UINT32_T * lenth
										);

XLLP_INT32_T XllpU2DWriteDwordFifo(P_XLLP_U2D_T pU2DHandle, XLLP_U2D_EP_T udcEndpointNum, P_XLLP_UINT8_T src, XLLP_INT32_T len);

XLLP_INT32_T XllpU2DReadDwordFifo(P_XLLP_U2D_T pU2DHandle, XLLP_U2D_EP_T udcEndpointNum, P_XLLP_UINT8_T src, XLLP_INT32_T len);

XLLP_STATUS_T XllpU2DEndpointSetConfig(P_XLLP_U2D_T pU2DHandle, XLLP_U2D_EP_T EP, P_XLLP_U2D_EP_CONFIG pConfig);

XLLP_STATUS_T XllpU2DEndpointSetInfo(P_XLLP_U2D_T pU2DHandle, XLLP_U2D_EP_T EP, P_XLLP_U2D_EP_CONFIG pConfig);

/*!

    Call this function when system entery sleep status.
    
@param[in]    pU2DHandle			U2DHandle, which is initilizated in XllpU2DInitHandle
@return
	- XLLP_STATUS_SUCCESS	Success
	- Other value			Fail	
*/
XLLP_STATUS_T XllpU2DSuspend(P_XLLP_U2D_T pU2DHandle);

/*!

    Call this function when system wakeup from sleep status.
    
@param[in]    pU2DHandle			U2DHandle, which is initilizated in XllpU2DInitHandle
@return
	- XLLP_STATUS_SUCCESS	Success
	- Other value			Fail	
*/
XLLP_STATUS_T XllpU2DResume(P_XLLP_U2D_T pU2DHandle);
	
#ifdef __cplusplus
}
#endif 


#endif

