/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/****************************************************************************
**  FILENAME:       xllp_usbotg.h
**  
**  PURPOSE: contains all USB OTG specific macros, typedefs, and prototypes.
**           Declares no storage.
**  
**  
******************************************************************************/
#ifndef __XLLP_OTG_H__
#define __XLLP_OTG_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "xllp_defs.h"
#include "xllp_gpio.h"
#include "xllp_mfp.h"
#include "xllp_ost.h"
#include "xllp_udc.h"
#include "xllp_i2c.h"


/* Enumberation XLLP_OTG_FUNCTION */
typedef enum _XLLP_OTG_FUNCTION {
	XLLP_OTG_B_DEVICE = 0,
	XLLP_OTG_A_DEVICE,
}XLLP_OTG_FUNCTION;

/* Enumeration XLLP_OTG_STATE */
typedef enum _XLLP_OTG_STATE {
	XLLP_OTG_UNDEFINED = 0,
	
	/* B device state */
	XLLP_OTG_B_IDLE,
	XLLP_OTG_B_SRP_INIT,
	XLLP_OTG_B_PERIPHERAL,
	XLLP_OTG_B_WAIT_ACON,
	XLLP_OTG_B_HOST,

	/* A device state */
	XLLP_OTG_A_IDLE,
	XLLP_OTG_A_WAIT_VRISE,
	XLLP_OTG_A_WAIT_BCON,
	XLLP_OTG_A_HOST,
	XLLP_OTG_A_SUSPEND,
	XLLP_OTG_A_PERIPHERAL,
	XLLP_OTG_A_WAIT_VFALL,
	XLLP_OTG_A_VBUS_ERR,

}XLLP_OTG_STATE;

/* Enumeration XLLP_OTG_MODE_T */
typedef enum XLLP_OTG_MODE_E {
	XLLP_USB_OTG_OFF = 0,
	XLLP_USB_NON_OTG_CLIENT_SEP,
	XLLP_USB_NON_OTG_HOST_SEP,
	XLLP_USB_EXT_OTG_CLIENT_SEP,
	XLLP_USB_EXT_OTG_HOST_SEP,
	XLLP_USB_NON_OTG_HOST_SEP_CLIENT_DP,
	XLLP_USB_NON_OTG_CLIENT_SEP_HOST_DP,
	XLLP_USB_INT_OTG_HOST_DP,
	XLLP_USB_INT_OTG_CLIENT_DP,
	XLLP_USB_NON_OTG_CLIENT_DP,
	XLLP_USB_NON_OTG_HOST_DP 
} XLLP_OTG_MODE_T;
//The definition is described in function XllpOtgConfigPort2.

/* Enumeration XLLP_OTG_RESISTER_T */
typedef enum _XLLP_OTG_RESISTER_T {
	XLLP_OTG_DP_PULLDOWN=2,
	XLLP_OTG_DM_PULLDOWN,
	XLLP_OTG_DP_PULLUP,
	XLLP_OTG_DM_PULLUP
} XLLP_OTG_RESISTER_T;

/* UDC Handle Structure */
typedef struct XLLP_OTG_S {
	// public
P_XLLP_UDC_REGISTERS_T  pUdcRegs;       // Pointer to UDC's registers
P_XLLP_VUINT32_T	pMfpRegs;	// Pointer to the MFP register address
P_XLLP_MFP_RM_DB_ID_T   pMFPRmDb;      	// MFP Resource Manager database
P_XLLP_I2C_T		pPI2cRegs;	// Pointer to power i2c registers
P_XLLP_OST_T		pOstRegs;	// Pointer to OST regsiters

XLLP_OTG_FUNCTION	default_function;
XLLP_OTG_STATE		state;
XLLP_INT32_T		a_set_b_hnp_en;		// A-Device set b_hnp_en
XLLP_INT32_T		b_hnp_enabled;		// Set feature command with hnp enable received.
XLLP_INT32_T		a_hnp_supported;
XLLP_INT32_T		a_alt_hnp_supported;

XLLP_INT32_T		b_bus_required;		// B-Device Require Bus

//Timer identifiers
XLLP_INT32_T		a_wait_vrise_tmr;	//Identifier of timer
XLLP_INT32_T		a_wait_bcon_tmr;
XLLP_INT32_T		a_aidl_bdis_tmr;
XLLP_INT32_T		b_ase0_brst_tmr;
XLLP_INT32_T		b_srp_fail_tmr;
} XLLP_OTG_T, *P_XLLP_OTG_T;


/* Enumeration XLLP_OTG_ERROR_T */
typedef enum _XLLP_OTG_ERROR_T {
	XLLP_OTG_SUCCESS = 0,
	XLLP_OTG_INVALID_PARAMETER,
	XLLP_OTG_UDC_DISABLED,
	XLLP_OTG_WRONG_STATE,
	XLLP_OTG_WRONG_RESISTER,
	XLLP_OTG_I2C_ERROR,
	XLLP_OTG_MFP_FAILED,
	XLLP_OTG_TIMER_FAILED
}XLLP_OTG_ERROR_T;

/* bits in UDCCR */
#define XLLP_UDCCR_UDE			( 1u << 0 )
#define XLLP_UDCCR_BHNP			( 1u << 28 )
#define XLLP_UDCCR_AHNP			( 1u << 29 )
#define XLLP_UDCCR_AALTHNP		( 1u << 30 )
#define XLLP_UDCCR_OEN			( 1u << 31 )

/* fields  and bits for UP2OCR*/
#define XLLP_OTG_UP2OCR_CPVEN		( 1u << 0 )
#define XLLP_OTG_UP2OCR_CPVEP		( 1u << 1 )
#define XLLP_OTG_UP2OCR_DPPDE		( 1u << 2 )
#define XLLP_OTG_UP2OCR_DMPDE		( 1u << 3 )
#define XLLP_OTG_UP2OCR_DPPUE		( 1u << 4 )
#define XLLP_OTG_UP2OCR_DMPUE		( 1u << 5 )
#define XLLP_OTG_UP2OCR_DPPUBE		( 1u << 6 )
#define XLLP_OTG_UP2OCR_DMPUBE		( 1u << 7 )
#define XLLP_OTG_UP2OCR_EXSP		( 1u << 8 )
#define XLLP_OTG_UP2OCR_EXSUS		( 1u << 9 )
#define XLLP_OTG_UP2OCR_IDON		( 1u << 10)
#define XLLP_OTG_UP2OCR_HXS		( 1u << 16)
#define XLLP_OTG_UP2OCR_HXOE		( 1u << 17)
#define XLLP_OTG_UP2OCR_SEOS_SHIFT	( 24)

/* fields and bits for UDCOTGISR */
#define XLLP_OTG_UDCOTGISR_IRIDF		( 1u << 0 )
#define XLLP_OTG_UDCOTGISR_IRIDR		( 1u << 1 )
#define XLLP_OTG_SETFEATURE			( 1u << 24)

/* Timer's interval, unit ms*/
#define	T_A_WAIT_VRISE		100
#define T_A_WAIT_BCON		2000
#define T_A_AIDL_BDIS		1000
/* In order to debug, we use a longer timer */
//#define T_B_ASE0_BRST		4
#define T_B_ASE0_BRST		400
#define T_B_SRP_FAIL		30000
#define T_B_DATA_PLS		10
#define T_B_SRP_INIT		100


/* Primitives these are not platform specific */
XLLP_OTG_ERROR_T XllpOtgConfigPort2 (P_XLLP_UDC_REGISTERS_T  pUdcRegs,	
			XLLP_OTG_MODE_T mode);

XLLP_OTG_ERROR_T XllpOtgRequireBus (P_XLLP_OTG_T pOtgHandle, 
			XLLP_BOOL_T bRequire, XLLP_BOOL_T bSRP);

XLLP_OTG_ERROR_T XllpOtgSetBHNPEnable (P_XLLP_OTG_T pOtgHandle, 
			XLLP_BOOL_T bSet);

XLLP_OTG_ERROR_T XllpOtgSuspendDevice (P_XLLP_OTG_T pOtgHandle);

XLLP_OTG_ERROR_T XllpOtgRespondSuspend (P_XLLP_OTG_T pOtgHandle);

XLLP_OTG_ERROR_T XllpOtgRespondConnect (P_XLLP_OTG_T pOtgHandle);

XLLP_OTG_ERROR_T XllpOtgRespondDisconnect (P_XLLP_OTG_T pOtgHandle);

XLLP_OTG_ERROR_T XllpOtgSuspend (P_XLLP_OTG_T pOtgHandle);

XLLP_OTG_ERROR_T XllpOtgResume (P_XLLP_OTG_T pOtgHandle);



/* Primitives these are platform sepcific */
XLLP_OTG_ERROR_T XllpOtgInit (P_XLLP_OTG_T pOtgHandle);

XLLP_OTG_ERROR_T XllpOtgResisterControl (P_XLLP_OTG_T pOtgHandle,
			XLLP_OTG_RESISTER_T resister, 
			XLLP_BOOL_T enable);

/* Don't call it directly */
XLLP_OTG_ERROR_T XllpOtgInterruptHandle (P_XLLP_OTG_T pOtgHandle);

XLLP_OTG_ERROR_T XllpOtgVbusInterrupt (P_XLLP_OTG_T pOtgHandle, XLLP_UINT8_T);

XLLP_OTG_ERROR_T XllpOtgVbusEnable (P_XLLP_OTG_T pOtgHandle, XLLP_BOOL_T bEnable);

XLLP_OTG_ERROR_T XllpOtgConfigMfp (P_XLLP_OTG_T pOtgHandle);

XLLP_OTG_ERROR_T XllpOtgVbusSRP (P_XLLP_OTG_T pOtgHandle);


/* OS Wrapper */
XLLP_INT32_T XllpOtgRegisterTimer (P_XLLP_OTG_T pOtgHandle, 
			XLLP_INT32_T interval, 
			void (*timer_callback)(unsigned long));

XLLP_OTG_ERROR_T XllpOtgCancelTimer (XLLP_INT32_T timer_id);
XLLP_OTG_ERROR_T XllpOtgTimerExpired (XLLP_INT32_T id);
XLLP_OTG_ERROR_T XllpOtgStateChange (P_XLLP_OTG_T pOtgHandle);

#ifdef __cplusplus
}
#endif

#endif // __XLLP_OTG_H__

