/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/**  FILENAME:       xllp_clkmgr.c
**
**  CORE STEPPING:  
**
**  PURPOSE: contains clock enable/disable function.
**           
**                  
**
*/
#include "xllp_clkmgr.h"

/**
This function enable/disable special peripheral clock in D0(Run) mode.

@param
	pReg  	    Virtual Base Address of Clock Module 
	Module  	Module name. It is enumeration of all Monahanss peripheral.
	enable		- true : Enable peripheral module clock
				- false: Disable peripheral module clock
@return
	Old peripheral module clock status(disable/enable) before call this function.

*/

XLLP_BOOL_T XllpClockEnable(P_XLLP_CLKMGR_T  pReg, XLLP_CLK_MODULE module, XLLP_BOOL_T enable)
{
	
	XLLP_BOOL_T ret;
    XLLP_UINT32_T dump;

	if(module<32)
	{
		ret = (XLLP_BOOL_T)(pReg->d0cken_a & (0x1u << module));
		
		if(enable)
		{
			pReg->d0cken_a |= (0x1u << module);
			if( XLLP_CLK_AC97 == module){
				//fix me: make out clock for ac97 codec be 24.576MHZ. 
				pReg->ac97_div = 1625<<12 | 128;
			}

		}else
		{
			pReg->d0cken_a &= ~(0x1u << module);
		}
	}
	else
	{
		ret = (XLLP_BOOL_T) (pReg->d0cken_b & (0x1u << (module-32) ));

		if(enable)
		{
			pReg->d0cken_b |= (0x1u << (module-32) );
		
		}else
		{
			pReg->d0cken_b &= ~(0x1u << (module-32) );
		}
	}
	dump = pReg->d0cken_b;  //dump read to make sure write register successfully
	return ret;

}

/**
This function save state before system enter sleep

@param
    pVirtualAddress     Virtual address of clock module.
    pSaveBuff           buff of saving state. 

@return
    XLLP_STATUS_SUCCESS             -   Success
    XLLP_STATUS_WRONG_PARAMETER     -   fail, input address is zero
*/
XLLP_STATUS_T XllpClockSave(P_XLLP_CLKMGR_T  pVirtualAddress, P_XLLP_CLKMGR_T pSaveBuff)
{
    if(!pVirtualAddress)
        return XLLP_STATUS_WRONG_PARAMETER;

    if(!pSaveBuff)
        return XLLP_STATUS_WRONG_PARAMETER;

    *pSaveBuff=*pVirtualAddress;
    
    return XLLP_STATUS_SUCCESS;
}

/**
This function restore state after system wakeup

@param
    pVirtualAddress     Virtual address of clock module.
    pSaveBuff           buff of saving state. 

@return
    XLLP_STATUS_SUCCESS             -   Success
    XLLP_STATUS_WRONG_PARAMETER     -   fail, input address is zero

@remark
    This function will NOT issue frequency change. 
    Only restore d0cken_a d0cken_b ac97_div and aicsr control bit. Status bit will be keeped. 
    OS Driver must restore working frequency. 
*/               
XLLP_STATUS_T XllpClockRestore(P_XLLP_CLKMGR_T  pVirtualAddress, P_XLLP_CLKMGR_T pSaveBuff)
{
    XLLP_UINT32_T aicsr;
    if(!pVirtualAddress)
        return XLLP_STATUS_WRONG_PARAMETER;

    if(!pSaveBuff)
        return XLLP_STATUS_WRONG_PARAMETER;

    aicsr = pSaveBuff->aicsr;
    aicsr &=~ (XLLP_AICSR_PCIS|XLLP_AICSR_TCIS|XLLP_AICSR_FCIS);

    pVirtualAddress->d0cken_a = pSaveBuff->d0cken_a;
    pVirtualAddress->d0cken_b = pSaveBuff->d0cken_b;
    pVirtualAddress->ac97_div = pSaveBuff->ac97_div;
    pVirtualAddress->aicsr    = aicsr;

    aicsr=pVirtualAddress->aicsr ;  //dump read to make sure write register successfully

    return XLLP_STATUS_SUCCESS;
}

