/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/**
**  FILENAME:       xllp_dfc_proc.c
**
**  PURPOSE: Implementation for Xllp DFC module
**           
**					 
**            
**                  
**
******************************************************************************/
#include "xllp_dfc_proc.h"
#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif
#include <blob/util.h>

#ifdef  CONFIG_CPU_MONAHANS_LV
#define DFC_WORKAROUND 1
#endif

#define MIN(x, y)   ((x < y) ? x : y)
#ifdef DFC_WORKAROUND
#define DFC_WORKAROUND_ENTER() \
	do {								\
		XLLP_UINT32_T tmp;					\
		*(volatile P_XLLP_UINT32_T)(0x4134000C) &= ~(1 << 4);	\
		tmp = *(volatile P_XLLP_UINT32_T)(0x4134000C);		\
	} while(0)
#define DFC_WORKAROUND_EXIT() \
	do {								\
		XLLP_UINT32_T tmp;					\
		*(volatile P_XLLP_UINT32_T)(0x4134000C) |= (1 << 4);	\
		tmp = *(volatile P_XLLP_UINT32_T)(0x4134000C);		\
	} while(0)
#else
#define DFC_WORKAROUND_ENTER() 
#define DFC_WORKAROUND_EXIT() 
#endif

static XLLP_UINT32_T dfc_mfp_list[] = {
	XLLP_MFP_DF_INT_RnB_OFFSET,
	XLLP_MFP_DF_nRE_OFFSET,
	XLLP_MFP_DF_nWE_OFFSET,
	XLLP_MFP_ND_CLE_OFFSET,
#if defined(CONFIG_CPU_MONAHANS_P) || defined(CONFIG_CPU_MONAHANS_PL) ||\
	defined(CONFIG_CPU_TAVOR_P) || defined(CONFIG_CPU_TAVOR_PV)
	XLLP_MFP_DF_nADV1_OFFSET,
	XLLP_MFP_DF_nADV2_OFFSET,
#elif defined(CONFIG_CPU_MONAHANS_L) || defined(CONFIG_CPU_MONAHANS_LV)
	XLLP_MFP_DF_ALE_OFFSET,
#else
#error "please define DFC ALE!"
#endif
	XLLP_MFP_DF_IO_0_OFFSET,
	XLLP_MFP_DF_IO_1_OFFSET,
	XLLP_MFP_DF_IO_2_OFFSET,
	XLLP_MFP_DF_IO_3_OFFSET,
	XLLP_MFP_DF_IO_4_OFFSET,
	XLLP_MFP_DF_IO_5_OFFSET,
	XLLP_MFP_DF_IO_6_OFFSET,
	XLLP_MFP_DF_IO_7_OFFSET,
	XLLP_MFP_DF_IO_8_OFFSET,
	XLLP_MFP_DF_IO_9_OFFSET,
	XLLP_MFP_DF_IO_10_OFFSET,
	XLLP_MFP_DF_IO_11_OFFSET,
	XLLP_MFP_DF_IO_12_OFFSET,
	XLLP_MFP_DF_IO_13_OFFSET,
	XLLP_MFP_DF_IO_14_OFFSET,
	XLLP_MFP_DF_IO_15_OFFSET,
	XLLP_MFP_DF_NCS0_OFFSET,
	XLLP_MFP_DF_NCS1_OFFSET,
	XLLP_MFP_PIN_EOLIST_MARKER
};
	
static XLLP_MFP_ALT_FN_T dfc_af_list[] = {
	XLLP_MFP_DF_INT_RnB_AF,
	XLLP_MFP_DF_nRE_AF,
	XLLP_MFP_DF_nWE_AF,
	XLLP_MFP_ND_CLE_AF,
#if defined(CONFIG_CPU_MONAHANS_P) || defined(CONFIG_CPU_MONAHANS_PL) ||\
	defined(CONFIG_CPU_TAVOR_P) || defined(CONFIG_CPU_TAVOR_PV)
	XLLP_MFP_DF_nADV1_AF,
	XLLP_MFP_DF_nADV2_AF,
#elif defined(CONFIG_CPU_MONAHANS_L) || defined(CONFIG_CPU_MONAHANS_LV)
	XLLP_MFP_DF_ALE_AF,
#else
#error "please define DFC ALE!"
#endif	
	XLLP_MFP_DF_IO_0_AF,
	XLLP_MFP_DF_IO_1_AF,
	XLLP_MFP_DF_IO_2_AF,
	XLLP_MFP_DF_IO_3_AF,
	XLLP_MFP_DF_IO_4_AF,
	XLLP_MFP_DF_IO_5_AF,
	XLLP_MFP_DF_IO_6_AF,
	XLLP_MFP_DF_IO_7_AF,
	XLLP_MFP_DF_IO_8_AF,
	XLLP_MFP_DF_IO_9_AF,
	XLLP_MFP_DF_IO_10_AF,
	XLLP_MFP_DF_IO_11_AF,
	XLLP_MFP_DF_IO_12_AF,
	XLLP_MFP_DF_IO_13_AF,
	XLLP_MFP_DF_IO_14_AF,
	XLLP_MFP_DF_IO_15_AF,
	XLLP_MFP_DF_NCS0_AF,
	XLLP_MFP_DF_NCS1_AF,
};
	
static XLLP_MFP_DRIVE_STRENGTH_T dfc_ds_list[] = {
	XLLP_MFP_ND_NON_DATA_DS,
	XLLP_MFP_ND_NON_DATA_DS,
	XLLP_MFP_ND_NON_DATA_DS,
	XLLP_MFP_ND_NON_DATA_DS,
#if defined(CONFIG_CPU_MONAHANS_P) || defined(CONFIG_CPU_MONAHANS_PL)||\
	defined(CONFIG_CPU_TAVOR_P) || defined(CONFIG_CPU_TAVOR_PV)
	XLLP_MFP_ND_NON_DATA_DS,
	XLLP_MFP_ND_NON_DATA_DS,
#elif defined(CONFIG_CPU_MONAHANS_L) || defined(CONFIG_CPU_MONAHANS_LV)
	XLLP_MFP_ND_NON_DATA_DS,
#else
#error "please define DFC ALE!"
#endif		
	XLLP_MFP_ND_DATA_DS,
	XLLP_MFP_ND_DATA_DS,
	XLLP_MFP_ND_DATA_DS,
	XLLP_MFP_ND_DATA_DS,
	XLLP_MFP_ND_DATA_DS,
	XLLP_MFP_ND_DATA_DS,
	XLLP_MFP_ND_DATA_DS,
	XLLP_MFP_ND_DATA_DS,
	XLLP_MFP_ND_DATA_DS,
	XLLP_MFP_ND_DATA_DS,
	XLLP_MFP_ND_DATA_DS,
	XLLP_MFP_ND_DATA_DS,
	XLLP_MFP_ND_DATA_DS,
	XLLP_MFP_ND_DATA_DS,
	XLLP_MFP_ND_DATA_DS,
	XLLP_MFP_ND_DATA_DS,
	XLLP_MFP_ND_NON_DATA_DS,
	XLLP_MFP_ND_NON_DATA_DS,
};
	
/////////////////////////////////////////////////////////////////////////
//
//  The Low Power Mode output table
//
// The following table provides this Low Power Mode signal out behavior 
//  by the DFC MFPs:
//      RnB:  Float ("Z"). (It is input pin)
//      nWE/nRE/nCSX: Pull High (They are output pins, active low.)
//      ALE/CLE: Pull Low(They are output pins, active high)
//      IO<0..15>:Pull High(They are input/output, not care if active high/low)
                          
static XLLP_MFP_LPM_OUTPUT_T dfc_lpm_list []={
		XLLP_MFP_LPMO_FLOAT,      //  RnB
		XLLP_MFP_LPMO_PULL_HIGH,  //  nOE
		XLLP_MFP_LPMO_PULL_HIGH,  //  nWE
		XLLP_MFP_LPMO_PULL_LOW,   //  CLE
#if defined(CONFIG_CPU_MONAHANS_P) || defined(CONFIG_CPU_MONAHANS_PL)||\
	defined(CONFIG_CPU_TAVOR_P) || defined(CONFIG_CPU_TAVOR_PV)
		XLLP_MFP_LPMO_PULL_LOW,   //  ADV1
		XLLP_MFP_LPMO_PULL_LOW,   //  ADV2
#elif defined(CONFIG_CPU_MONAHANS_L) || defined(CONFIG_CPU_MONAHANS_LV)
		XLLP_MFP_LPMO_PULL_LOW,   //  ALE
#else
#error "please define DFC ALE!"
#endif	
		XLLP_MFP_LPMO_PULL_HIGH,  //  IO_0
		XLLP_MFP_LPMO_PULL_HIGH,  //  IO_1
		XLLP_MFP_LPMO_PULL_HIGH,  //  IO_2
		XLLP_MFP_LPMO_PULL_HIGH,  //  IO_3
		XLLP_MFP_LPMO_PULL_HIGH,  //  IO_4
		XLLP_MFP_LPMO_PULL_HIGH,  //  IO_5
		XLLP_MFP_LPMO_PULL_HIGH,  //  IO_6
		XLLP_MFP_LPMO_PULL_HIGH,  //  IO_7
		XLLP_MFP_LPMO_PULL_HIGH,  //  IO_8
		XLLP_MFP_LPMO_PULL_HIGH,  //  IO_9
		XLLP_MFP_LPMO_PULL_HIGH,  //  IO_10
		XLLP_MFP_LPMO_PULL_HIGH,  //  IO_11
		XLLP_MFP_LPMO_PULL_HIGH,  //  IO_12
		XLLP_MFP_LPMO_PULL_HIGH,  //  IO_13
		XLLP_MFP_LPMO_PULL_HIGH,  //  IO_14
		XLLP_MFP_LPMO_PULL_HIGH,  //  IO_15
		XLLP_MFP_LPMO_PULL_HIGH,  //  NCS0
		XLLP_MFP_LPMO_PULL_HIGH,  //  NCS1
};//end of xllp_keypad_lpm_output_list
	 
// The low level primitives

/******************************************************************************
	XllpFlashDfcSetGPIO

	Description:
	  This function sets appropriate values including alternate functions, low 
	  power mode behavior for corresponding MFPRs. It is called once to initialize 
	  hardware to make DFC signal effective. 
  Input Parameters:
	  pContext -Pointer to DFC context structure
  Output Parameters:
    None
  Returns:
    XLLP_STATUS_SUCCESS if MFPRs are set correctly
    XLLP_STATUS_NO_RESOURCES if the MFP ownership is already assigned to another
    device.
******************************************************************************/
XLLP_STATUS_T XllpFlashDfcSetGPIO(P_XLLP_DFC_CONTEXT_T pContext)
{
	XLLP_STATUS_T status;

	status = XllpMfpResourceManager_List(pContext->pMFPRmDb,
			&dfc_mfp_list[0],
			XLLP_MFP_RM_ID_XLLP_FLASH,
			XLLP_SET);
	if (status != XLLP_STATUS_SUCCESS){
		return status;
	}

	status = XllpMfpSetAfDs_List(pContext->pMFPBase,
			&dfc_mfp_list[0],
			&dfc_af_list[0],
			&dfc_ds_list[0]);
	if (status != XLLP_STATUS_SUCCESS){
		return status;
	}


	status = XllpMfpConfigureLpmOutputLevel_List(
			pContext->pMFPBase,
			&dfc_mfp_list[0],
			&dfc_lpm_list[0]);

	if (status != XLLP_STATUS_SUCCESS){
		return status;
	}


	status = XllpMfpActivatePullUpDown_List(pContext->pMFPBase,
			&dfc_mfp_list[0],
			XLLP_OFF);
	if (status != XLLP_STATUS_SUCCESS){
		return status;
	} 


	return XLLP_STATUS_SUCCESS;
}



/******************************************************************************
	XllpFlashDfcSetTiming
	
	Description:
		This function sets flash timing property in DFC timing register accoringt 
		to input timing value embodied in context structure. It is called once 
		during the hardware initialization.
  Input Parameters:
		pContext-Pointer to DFC context structure
  Output Parameters:
		None
  Returns:
		None  
*******************************************************************************/
void XllpFlashDfcSetTiming(P_XLLP_DFC_CONTEXT_T pContext)
{
	XLLP_DFC_FLASH_TIMING_T	 timing = pContext->pFlashInfo->timing;

	//float 	clock_ns = 1000/XLLP_DFC_CLOCK;
#define clock_ns XLLP_DFC_CLOCK/1000

	XLLP_UINT32_T  r0 = 0;
	XLLP_UINT32_T  r1 = 0;

	//	  num of clock cycles = time (ns) / one clock sycle (ns) + 1
	//    - integer division will truncate the result, so add a 1 in all cases
	//    - subtract the extra 1 cycle added to all register timing values
	//
	timing.tCH = MIN(((XLLP_INT32_T) (timing.tCH * clock_ns) + 1), 
			XLLP_TIMING_MAX_tCH);
	timing.tCS = MIN(((XLLP_INT32_T) (timing.tCS * clock_ns) + 1), 
			XLLP_TIMING_MAX_tCS);
	timing.tWH = MIN(((XLLP_INT32_T) (timing.tWH * clock_ns) + 1), 
			XLLP_TIMING_MAX_tWH);
	timing.tWP = MIN(((XLLP_INT32_T) (timing.tWP * clock_ns) + 1), 
			XLLP_TIMING_MAX_tWP);
	timing.tRH = MIN(((XLLP_INT32_T) (timing.tRH * clock_ns) + 1), 
			XLLP_TIMING_MAX_tRH);
	timing.tRP = MIN(((XLLP_INT32_T) (timing.tRP * clock_ns) + 1), 
			XLLP_TIMING_MAX_tRP);

	r0 = (timing.tCH << XLLP_DFC_TIMING_tCH) |
		(timing.tCS << XLLP_DFC_TIMING_tCS) |
		(timing.tWH << XLLP_DFC_TIMING_tWH) |
		(timing.tWP << XLLP_DFC_TIMING_tWP) |
		(timing.tRH << XLLP_DFC_TIMING_tRH) |
		(timing.tRP << XLLP_DFC_TIMING_tRP);
	pContext->pDFCReg->ndtr0cs0 = r0;

	timing.tR   = MIN(((XLLP_INT32_T) (timing.tR   * clock_ns) + 1), 
			XLLP_TIMING_MAX_tR);
	timing.tWHR = MIN(((XLLP_INT32_T) (timing.tWHR * clock_ns) + 1), 
			XLLP_TIMING_MAX_tWHR);
	timing.tAR  = MIN(((XLLP_INT32_T) (timing.tAR  * clock_ns) + 1), 
			XLLP_TIMING_MAX_tAR);

	r1 = (timing.tR   << XLLP_DFC_TIMING_tR)   |
		(timing.tWHR << XLLP_DFC_TIMING_tWHR) |
		(timing.tAR  << XLLP_DFC_TIMING_tAR);

	pContext->pDFCReg->ndtr1cs0 = r1;
	return;
}

/******************************************************************************
	XllpFlashDfcSetDMA

	Description:
		Enables or Disables DMA in line with setting in DFC mode of context 
		structure. DMA mode of DFC. Performs a read-modify-write operation that 
		only changes the driven DMA_EN bit field In DMA mode, all commands and 
		data are transferred by DMA.  DMA can be enable/disable on the fly.
  Input Parameters:
		pContext -Pointer to DFC context structure
  Output Parameters:
		None
  Returns:
		None  
*******************************************************************************/
void 
XllpFlashDfcSetDMA(P_XLLP_DFC_CONTEXT_T pContext) 
{
	if (pContext->pDFCMode->enableDMA)
		pContext->pDFCReg->ndcr |= XLLP_DFC_CTRL_DMA_EN;
	else
		pContext->pDFCReg->ndcr &= ~XLLP_DFC_CTRL_DMA_EN;
	
	return;
}


/******************************************************************************
	XllpFlashDfcSetECC

	Description:
		This function enables or disables hardware ECC capability of DFC in line 
		with setting in DFC mode of context structure. 
  Input Parameters:
		pContext -Pointer to DFC context structure
  Output Parameters:
		None
  Returns:
		None  
*******************************************************************************/
void 
XllpFlashDfcSetECC(P_XLLP_DFC_CONTEXT_T pContext) 
{
	if (pContext->pDFCMode->enableECC)
		pContext->pDFCReg->ndcr |= XLLP_DFC_CTRL_ECC_EN;
	else
		pContext->pDFCReg->ndcr &= ~XLLP_DFC_CTRL_ECC_EN;
		
	return;
}

/******************************************************************************
	XllpFlashDfcSetSpare

	Description:
		This function enables or disables accesses to spare area of NAND Flash 
		through DFC in line with setting in DFC mode of context structure. 
  Input Parameters:
		pContext -Pointer to DFC context structure
  Output Parameters:
		None
  Returns:
		None  
*******************************************************************************/
void 
XllpFlashDfcSetSpare(P_XLLP_DFC_CONTEXT_T pContext) 
{
	if (pContext->pDFCMode->enableSpareArea)
		pContext->pDFCReg->ndcr |= XLLP_DFC_CTRL_SPARE_EN;
	else
		pContext->pDFCReg->ndcr &= ~XLLP_DFC_CTRL_SPARE_EN;
	
	return;
}

static int 
getTimebase (void)
{
    return 3;
}

static XLLP_UINT32_T 
getDelta (P_XLLP_OST_T pRegs, XLLP_UINT32_T start)
{
    XLLP_UINT32_T stop = pRegs->oscr0;
    if (stop < start)
        return (stop + (start^0xffffffff) + 1);
    return (stop - start);
}


static XLLP_STATUS_T 
XllpFlashDfcWaitEvent(P_XLLP_DFC_CONTEXT_T pContext, 
	XLLP_UINT32_T event, P_XLLP_UINT32_T eventOut, XLLP_UINT32_T timeout, 
	XLLP_BOOL_T enableInt)
{
	P_XLLP_OST_T pOSTReg = pContext->pOSTReg;
	XLLP_UINT32_T ndsr;
	XLLP_UINT32_T to = getTimebase() * timeout;
	XLLP_STATUS_T status;
	XLLP_UINT32_T start = pOSTReg->oscr0;
//	unsigned int reg;

	if (enableInt)
		XllpFlashDfcEnableInt(pContext, event);
		
	while (1) {
		XllpFlashDfcReadDFCStatus(pContext, &ndsr);
		if (ndsr & event) {
			// event happened
			*eventOut = ndsr & event;
			XllpFlashDfcClearInt(pContext, *eventOut);
			status = XLLP_STATUS_SUCCESS;
			break;
		}
		else if (getDelta(pOSTReg, start) > to) {
			status = XLLP_STATUS_TIME_OUT;
			break;
		}
	}

	if (enableInt)
		XllpFlashDfcDisableInt(pContext, event);
	return status;
}


/******************************************************************************
	XllpFlashDfcGetPattern

	Description:
		This function is used to retrieve buffer size setting for a transaction 
		based on cmd. 
  Input Parameters:
	  pContext - Pointer to DFC context structure
	  cmd
	    Specifies type of command to be sent to NAND flash .The LSB of this 
		  parameter defines the first command code for 2-cycles command. The 
		  MSB defines the second command code for 2-cycles command. If MSB is 
		  set to zero, this indicates that one cycle command 
	Output Parameters:
	  pDataSize
	    It is used to retrieve  length of data transferred to/from DFC, 
		  which includes padding bytes
	  pPaddingSize
	    It is used to retrieve how many padding bytes there should be 
		  in buffer of dataSize.
	Returns:
		XLLP_STATUS_ SUCCESS 
		  If size setting is returned successfully
		XLLP_STATUS_UNSUPPORTED 
		  If page size specified in flash spec of context structure is not 512 or 
		  2048;If specified command index is not read1/program/erase/reset/readID/
		  readStatus. 
*******************************************************************************/
XLLP_STATUS_T 
XllpFlashDfcGetPattern(P_XLLP_DFC_CONTEXT_T pContext, 
		XLLP_UINT16_T cmd, P_XLLP_DFC_DATA_SIZE_T pDataSize, 
		P_XLLP_DFC_PADDING_SIZE_T pPaddingSize) 
{
	P_XLLP_DFC_MODE_T	pDFCMode = pContext->pDFCMode;
	P_XLLP_DFC_FLASH_SPECS_T pFlashInfo = pContext->pFlashInfo;
	XLLP_UINT32_T pageSize = pContext->pFlashInfo->pageSize; // 512 or 2048
	
	if (cmd == pFlashInfo->read1 ||
		cmd == pFlashInfo->program) {
		if (512 == pageSize) {
			/* add for DMA */
			if (pDFCMode->enableDMA) {
				*pDataSize = XLLP_DFC_DATA_SIZE_544;
				if (pDFCMode->enableECC) 
					*pPaddingSize = XLLP_DFC_PADDING_SIZE_24;	
				else
					*pPaddingSize = XLLP_DFC_PADDING_SIZE_16;
			}
			else if (!pDFCMode->enableSpareArea) {
				*pDataSize = 	XLLP_DFC_DATA_SIZE_512;
				*pPaddingSize = XLLP_DFC_PADDING_SIZE_0;
			} else {
				
				if (pDFCMode->enableECC) 
					*pDataSize = XLLP_DFC_DATA_SIZE_520;	
				else
					*pDataSize = XLLP_DFC_DATA_SIZE_528;
				
				*pPaddingSize = XLLP_DFC_PADDING_SIZE_0;
				/*
				if (pDFCMode->enableECC) 
					*pPaddingSize = XLLP_DFC_PADDING_SIZE_24;	
				else
					*pPaddingSize = XLLP_DFC_PADDING_SIZE_16;
				*/
			}
		}else if (2048 == pageSize) {
			if (!pDFCMode->enableSpareArea) {
				*pDataSize = 	XLLP_DFC_DATA_SIZE_2048;
				*pPaddingSize = XLLP_DFC_PADDING_SIZE_0;
			} else {
				
				if (pDFCMode->enableECC) 
					*pDataSize = XLLP_DFC_DATA_SIZE_2088;	
				else
					*pDataSize = XLLP_DFC_DATA_SIZE_2112;
				
				*pPaddingSize = XLLP_DFC_PADDING_SIZE_0;
			}
		}else 
			return XLLP_STATUS_UNSUPPORTED;
	}else if (cmd == pFlashInfo->readID) {
		*pDataSize = XLLP_DFC_DATA_SIZE_ID;
		*pPaddingSize = XLLP_DFC_PADDING_SIZE_0;
	}else if(cmd == pFlashInfo->readStatus) {
		*pDataSize = XLLP_DFC_DATA_SIZE_STATUS;
		*pPaddingSize = XLLP_DFC_PADDING_SIZE_0;
	}else if (cmd == pFlashInfo->erase ||
		cmd == pFlashInfo->reset) {
		*pDataSize = XLLP_DFC_DATA_SIZE_UNUSED;
		*pPaddingSize = XLLP_DFC_PADDING_SIZE_UNUSED;
	}else
		return XLLP_STATUS_UNSUPPORTED;
	return XLLP_STATUS_SUCCESS;
}		
		

/******************************************************************************
	XllpFlashDfcSendCommand

	Description:
		This function configures DFC to send command through DFC to NAND flash  
  Input Parameters:
	  pContext 
		  Pointer to DFC context structure
	  cmd
		  Specifies type of command to be sent to NAND flash .The LSB of this 
		  parameter defines the first command code for 2-cycles command. The 
		  MSB defines the second command code for 2-cycles command. If MSB is 
		  set to zero, this indicates that one cycle command 
	  addr
		  Address sent out to the flash device withthis command. For page read/
		  program commands , 4-cycles address is sent. For erase command only 
		  3-cycles address is sent. If it is equal to 0xFFFFFFFF, the address 
		  should not be used. 
	  numPages
	    It specifies the number of pages of data to be transferred for 
		  a program or read commands. Unused for any other commands than 
		  read/program.

	Output Parameters:
		None
	Returns:
		XLLP_STATUS_ SUCCESS 
		  If size setting is returned successfully
		XLLP_STATUS_UNSUPPORTED 
		  If specified command index is not read1/program/erase/reset/readID/
		  readStatus. 
*******************************************************************************/
XLLP_STATUS_T
XllpFlashDfcSendCommand(P_XLLP_DFC_CONTEXT_T pContext, 
		XLLP_UINT16_T cmd, XLLP_UINT32_T addr, XLLP_UINT8_T numPages)
{
	P_XLLP_DFC_FLASH_SPECS_T pFlashInfo = pContext->pFlashInfo;
	P_XLLP_DFC_MODE_T	pDFCMode = pContext->pDFCMode;
	P_XLLP_DFC_REG_T pDFCReg = pContext->pDFCReg;
	XLLP_STATUS_T status;
	XLLP_UINT8_T cmd2;
	XLLP_UINT32_T eventOut;
	XLLP_UINT32_T ndcb0=0, ndcb1=0, ndcb2=0;

	DFC_WORKAROUND_ENTER();
		
	// It is a must to set ND_RUN firstly, then write command buffer
	// If conversely,it does not work
	pDFCReg->ndsr = XLLP_DFC_NDSR_CLEAR;
	 
	
	// Set ND_RUN
	pDFCReg->ndcr |= XLLP_DFC_CTRL_ND_RUN;
	
	DFC_WORKAROUND_EXIT();

	// Wait for write command request
	status = XllpFlashDfcWaitEvent(pContext, XLLP_DFC_STATUS_WRCMDREQ, 
		&eventOut, NAND_CMD_TIMEOUT, XLLP_FALSE);
	
	if (status != XLLP_STATUS_SUCCESS)
	{
		return status; //Timeout
	}

	cmd2 = (cmd>>8) & 0xFF;
	ndcb0 = cmd | (pDFCMode->chipSelect<<24) | ((cmd2?1:0)<<19);
	
	if (cmd == pFlashInfo->read1) {
		if (0xFFFFFFFF != addr) {
			ndcb0 |= (pFlashInfo->readProgCycles<<16);
			status = pFlashInfo->Addr2NDCB1(cmd, addr, &ndcb1);
			if (status != XLLP_STATUS_SUCCESS)
				return status;
			ndcb2 = (numPages - 1) << 8;
		}
		/* If has A27, need to set addr 5 */ 
		if (addr >= 0x8000000)
			ndcb2 |= 0x1;
	}else if (cmd == pFlashInfo->program) {
		ndcb0 |= (1<<21) | (1<<25);
		ndcb0 |= (pFlashInfo->readProgCycles<<16);
		status = pFlashInfo->Addr2NDCB1(cmd, addr, &ndcb1);
		if (status != XLLP_STATUS_SUCCESS)
			return status;
		ndcb2 = (numPages-1) << 8;
		/* If has A27, need to set addr 5 */ 
		if (addr >= 0x8000000)
			ndcb2 |= 0x1;
	}else if (cmd == pFlashInfo->erase) {
		ndcb0 |= (2<<21) | (1<<25);
		ndcb0 |= 3<<16;
		status = pFlashInfo->Addr2NDCB1(cmd, addr, &ndcb1);
		if (status != XLLP_STATUS_SUCCESS)
			return status;
	}else if (cmd == pFlashInfo->readID) {
		ndcb0 |= ((1 << 16) | (3<<21));
	}else if(cmd == pFlashInfo->readStatus) {
		ndcb0 |= ((1 << 16) | (4<<21));
	}else if(cmd == pFlashInfo->reset) {
		ndcb0 |= (5<<21);
	}else if (cmd == pFlashInfo->lock) {
		ndcb0 |= (5<<21);
#ifdef CACHE_READ
	}else if(cmd == 0x34) {
		ndcb0 |= (4<<21);
#endif
	}else if (cmd == pFlashInfo->unlock_low || cmd == pFlashInfo->unlock_high) {
		ndcb0 |= (3<<21);
		ndcb0 |= ((pFlashInfo->unlockAddrCycles)<<16);
		status = pFlashInfo->Addr2NDCB1(cmd, addr, &ndcb1);
		if (status != XLLP_STATUS_SUCCESS)
			return status;
	}else
		return XLLP_STATUS_UNSUPPORTED;

	DFC_WORKAROUND_ENTER();
	// Write to DFC command register
	pDFCReg->ndcb0 = ndcb0;
	pDFCReg->ndcb0 = ndcb1;
	pDFCReg->ndcb0 = ndcb2;
	
	DFC_WORKAROUND_EXIT();
	return XLLP_STATUS_SUCCESS;
}

/******************************************************************************
	XllpFlashDfcRun

	Description:
		This function sets ND_RUN bit of NDCR, that starts to execute the command 
		in the command buffer.
  Input Parameters:
		pContext--Pointer to DFC context structure
  Output Parameters:
		None
  Returns:
		None  
*******************************************************************************/
void
XllpFlashDfcRun(P_XLLP_DFC_CONTEXT_T pContext) 
{
	pContext->pDFCReg->ndcr |= XLLP_DFC_CTRL_ND_RUN;
	return;
}

/******************************************************************************
	XllpFlashDfcStop

	Description:
		This function clears ND_RUN bit of NDCR. 
  Input Parameters:
		pContext--Pointer to DFC context structure
  Output Parameters:
		None
  Returns:
		None  
*******************************************************************************/
void
XllpFlashDfcStop(P_XLLP_DFC_CONTEXT_T pContext) 
{
	pContext->pDFCReg->ndcr &= ~XLLP_DFC_CTRL_ND_RUN;
	return;
}

/******************************************************************************
	XllpFlashDfcClearECC

	Description:
		This function initializes the ECC logic. That can be used during 
		initialization and after existing any low power modes. 
  Input Parameters:
		pContext--Pointer to DFC context structure
  Output Parameters:
		None
  Returns:
		None  
*******************************************************************************/
/*
void 
XllpFlashDfcClearECC(P_XLLP_DFC_CONTEXT_T pContext)
{
	pContext->pDFCReg->ndcr |= XLLP_DFC_CTRL_CLR_ECC;
	return;
}
*/

/******************************************************************************
	XllpFlashDfcClearPGCNT

	Description:
		This function clears the page counter register. 		
  Input Parameters:
		pContext--Pointer to DFC context structure
  Output Parameters:
		None
  Returns:
		None  
*******************************************************************************/
/*
void 
XllpFlashDfcClearPGCNT(P_XLLP_DFC_CONTEXT_T pContext)
{
	pContext->pDFCReg->ndcr |= XLLP_DFC_CTRL_CLR_PG_CNT;
	return;
}
*/

XLLP_STATUS_T XllpFlashDfcCmdDmaDescriptor(P_XLLP_DFC_CONTEXT_T pContext, 
		XLLP_UINT16_T cmd, XLLP_UINT32_T addr, XLLP_UINT8_T numPages,
		P_XLLP_UINT32_T buf, XLLP_UINT32_T bufPhy,
		XLLP_UINT32_T nextDescPhy, XLLP_UINT32_T dmaIntEn,
		P_XLLP_DMAC_DESCRIPTOR_T pDescriptor)
{
	P_XLLP_DFC_FLASH_SPECS_T pFlashInfo = pContext->pFlashInfo;
	P_XLLP_DFC_MODE_T	pDFCMode = pContext->pDFCMode;
	P_XLLP_DFC_REG_T pDFCReg = pContext->pDFCReg;
	XLLP_STATUS_T status;
	XLLP_UINT8_T cmd2;
	XLLP_UINT32_T eventOut;
	XLLP_UINT32_T ndcb0=0, ndcb1=0, ndcb2=0;
	
	// It is a must to set ND_RUN firstly, then write command buffer
	// If conversely,it does not work
	pDFCReg->ndsr = XLLP_DFC_NDSR_CLEAR;
	 
	
	// Set ND_RUN
	pDFCReg->ndcr |= XLLP_DFC_CTRL_ND_RUN;

	// Wait for write command request
	status = XllpFlashDfcWaitEvent(pContext, XLLP_DFC_STATUS_WRCMDREQ, 
		&eventOut, NAND_CMD_TIMEOUT, XLLP_FALSE);
	
	if (status != XLLP_STATUS_SUCCESS)
	{
		return status; //Timeout
	}
	
	cmd2 = (cmd>>8) & 0xFF;
	ndcb0 = cmd | (pDFCMode->chipSelect<<24) | ((cmd2?1:0)<<19);
	
	if (cmd == pFlashInfo->read1) {
		if (0xFFFFFFFF != addr) {
			ndcb0 |= 4<<16;
			status = pFlashInfo->Addr2NDCB1(cmd, addr, &ndcb1);
			if (status != XLLP_STATUS_SUCCESS)
				return status;
			ndcb2 = (numPages-1) << 8;
		}
	}else if (cmd == pFlashInfo->program) {
		ndcb0 |= (1<<21) | (1<<25);
		ndcb0 |= (4<<16);

		status = pFlashInfo->Addr2NDCB1(cmd, addr, &ndcb1);
		if (status != XLLP_STATUS_SUCCESS)
			return status;
		ndcb2 = (numPages-1) << 8;
	}else if (cmd == pFlashInfo->erase) {
		ndcb0 |= (2<<21) | (1<<25);
		ndcb0 |= 3<<16;

		status = pFlashInfo->Addr2NDCB1(cmd, addr, &ndcb1);
		if (status != XLLP_STATUS_SUCCESS)
			return status;
	}else if (cmd == pFlashInfo->readID) {
		ndcb0 |= (3<<21);
	}else if(cmd == pFlashInfo->readStatus) {
		ndcb0 |= (4<<21);
	}else if(cmd == pFlashInfo->reset) {
		ndcb0 |= (5<<21);
	}else if (cmd == pFlashInfo->lock) {
		ndcb0 |= (5<<21);
	}else if (cmd == pFlashInfo->unlock_low || cmd == pFlashInfo->unlock_high) {
		ndcb0 |= (3<<21);
		ndcb0 |= ((pFlashInfo->unlockAddrCycles)<<16);	
		status = pFlashInfo->Addr2NDCB1(cmd, addr, &ndcb1);
		if (status != XLLP_STATUS_SUCCESS)
			return status;
	} else
		return XLLP_STATUS_UNSUPPORTED;
	
	*((P_XLLP_UINT32_T)buf) = ndcb0;
	*((P_XLLP_UINT32_T)buf + 1) = ndcb1;
	*((P_XLLP_UINT32_T)buf + 2) = ndcb2;

	dmaIntEn &= (XLLP_DMAC_DCMD_START_IRQ_EN | XLLP_DMAC_DCMD_END_IRQ_EN);
		
	pDescriptor->ddadr = nextDescPhy;
	pDescriptor->dsadr = bufPhy;
	pDescriptor->dtadr = XLLP_DFC_NDCB0;
	pDescriptor->dcmd = XLLP_DMAC_DCMD_INC_SRC_ADDR
			| XLLP_DMAC_DCMD_FLOW_TRG 
			| dmaIntEn 
			| (XLLP_DMAC_WIDTH_32 << XLLP_BIT_FIELD_14)
			| (XLLP_DMAC_BURSTSIZE_16 << XLLP_BIT_FIELD_16)
			| 12;
	return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T XllpFlashDfcDataDmaDescriptor(P_XLLP_DFC_CONTEXT_T pContext, 
		XLLP_UINT16_T cmd, XLLP_UINT32_T bufPhy, 
		XLLP_UINT32_T nextDescPhy, XLLP_UINT32_T dmaIntEn,
		P_XLLP_DMAC_DESCRIPTOR_T pDescriptor)
{
	XLLP_DFC_DATA_SIZE_T datasize;
	XLLP_DFC_PADDING_SIZE_T paddingsize;
	P_XLLP_DFC_FLASH_SPECS_T pFlashInfo = pContext->pFlashInfo;

	XllpFlashDfcGetPattern(pContext, cmd, &datasize, &paddingsize);

	pDescriptor->ddadr = nextDescPhy;
	dmaIntEn &= (XLLP_DMAC_DCMD_START_IRQ_EN | XLLP_DMAC_DCMD_END_IRQ_EN);
	
	if (cmd == pFlashInfo->program) {
		pDescriptor->dsadr = bufPhy;
		pDescriptor->dtadr = XLLP_DFC_NDDB;
		pDescriptor->dcmd = XLLP_DMAC_DCMD_INC_SRC_ADDR
				| XLLP_DMAC_DCMD_FLOW_TRG 
				| dmaIntEn 
				| (XLLP_DMAC_WIDTH_32 << XLLP_BIT_FIELD_14)
				| (XLLP_DMAC_BURSTSIZE_32 << XLLP_BIT_FIELD_16)
				| datasize;
	} else if (cmd == pFlashInfo->read1 || cmd == pFlashInfo->readID || cmd == pFlashInfo->readStatus) {
		pDescriptor->dsadr = XLLP_DFC_NDDB;		
		pDescriptor->dtadr = bufPhy;
		pDescriptor->dcmd = XLLP_DMAC_DCMD_INC_TRG_ADDR
				| XLLP_DMAC_DCMD_FLOW_SRC
				| dmaIntEn 
				| (XLLP_DMAC_WIDTH_32 << XLLP_BIT_FIELD_14)
				| (XLLP_DMAC_BURSTSIZE_32 << XLLP_BIT_FIELD_16)
				| datasize;

	}
	else
		return XLLP_STATUS_WRONG_PARAMETER;
	return XLLP_STATUS_SUCCESS;
}

void XllpFlashDfcStartCmdDma(P_XLLP_DFC_CONTEXT_T pContext, P_XLLP_DMAC_DESCRIPTOR_T pDesc)
{
	XllpDmacCfgChannelDescTransfer(pContext->pDmacRegsBase, pDesc, pContext->pDma->cmdDmaChannel, XLLP_DMAC_NAND_CMD);
	XllpDmacStartTransfer(pContext->pDmacRegsBase, pContext->pDma->cmdDmaChannel);
}

void XllpFlashDfcStartDataDma(P_XLLP_DFC_CONTEXT_T pContext, P_XLLP_DMAC_DESCRIPTOR_T pDesc)
{
	XllpDmacCfgChannelDescTransfer(pContext->pDmacRegsBase, pDesc, pContext->pDma->dataDmaChannel, XLLP_DMAC_NAND_DATA);
	XllpDmacStartTransfer(pContext->pDmacRegsBase, pContext->pDma->dataDmaChannel);
}

void XllpFlashDfcStopCmdDma(P_XLLP_DFC_CONTEXT_T pContext) 
{
	XllpDmacStopTransfer(pContext->pDmacRegsBase, pContext->pDma->cmdDmaChannel);
}

void XllpFlashDfcStopDataDma(P_XLLP_DFC_CONTEXT_T pContext) 
{
	XllpDmacStopTransfer(pContext->pDmacRegsBase, pContext->pDma->dataDmaChannel);
}


/******************************************************************************
	XllpFlashDfcReadFIFOPartial

	Description:
		This function reads data from data buffer of DFC.Bytes can be any less than 
		or equal to dataSize, the left is ignored by ReadFIFO though they will be 
		read from NDDB to clear data buffer. 		
  Input Parameters:
	  pContext
	    Pointer to DFC context structure
	  bytes
	    Indicating how much data should be read into buffer. 
	  dataSize
	    Specifing length of data transferred to/from DFC, which includes 
		  padding bytes
	Output Parameters:
	  pBuffer
		  Pointer to the data buffer where data should be placed.
	Returns:
		None  
*******************************************************************************/
void 
XllpFlashDfcReadFIFOPartial(P_XLLP_DFC_CONTEXT_T pContext, 
		P_XLLP_UINT8_T pBuffer, XLLP_UINT32_T bytes,
		XLLP_DFC_DATA_SIZE_T dataSize)
{
	P_XLLP_DFC_REG_T pDFCReg = pContext->pDFCReg;
	XLLP_UINT32_T data;
	XLLP_UINT32_T i=0;
	XLLP_UINT32_T bytesMulti;
	XLLP_UINT32_T bytesRemain;
	P_XLLP_UINT32_T temp;
	
	
	if (1 == dataSize) {
		data = (pDFCReg->nddb) & 0xFF;
		*pBuffer = (XLLP_UINT8_T)data;
	} else if (2 == dataSize) {
		P_XLLP_UINT16_T temp = (P_XLLP_UINT16_T)pBuffer;
		data = (pDFCReg->nddb) & 0xFFFF;
		*temp = (XLLP_UINT16_T)data;
	} else {
		bytesMulti = (bytes & 0xFFFFFFFC);
		bytesRemain = bytes & 0x03;
		
		while(i< ((XLLP_UINT32_T)dataSize) ) {
			if (i < bytesMulti) {
				temp = (P_XLLP_UINT32_T)pBuffer;
				*temp = pDFCReg->nddb;
			}else if (i == bytesMulti && bytesRemain){
				XLLP_UINT32_T j = 0;
				data = pDFCReg->nddb;
				while (j < bytesRemain) {
				  *pBuffer = (XLLP_UINT8_T)((data>>(8*j)) & 0xFF);	
				  pBuffer++;
				  j++;
				}
			}else {
				data = pDFCReg->nddb;
			}
			i += 4;
			pBuffer += 4;
		}
	}
	return;
}
			
/******************************************************************************
	XllpFlashDfcWriteFIFOPartial

	Description:
		Write to data buffer of DFC from a buffer.Bytes can be same as dataSize, 
		also can be dataSize-paddingSize, but cant be random value, the left will 
		be automatically padded by WriteFIFO.
  Input Parameters:
	  pContext
	    Pointer to DFC context structure
	  bytes
	    Indicating how much data should be read into buffer. 
	  dataSize
	    Specifing length of data transferred to/from DFC, which includes 
		  padding bytes
		pBuffer
		  Pointer to the data buffer where data will be taken from to be written to 
		  DFC data buffer
	Output Parameters:
	  None
	Returns:
		None  
*******************************************************************************/
void 
XllpFlashDfcWriteFIFOPartial(P_XLLP_DFC_CONTEXT_T pContext, 
		P_XLLP_UINT8_T pBuffer, XLLP_UINT32_T bytes, 
		XLLP_DFC_DATA_SIZE_T dataSize)
{
	P_XLLP_DFC_REG_T pDFCReg = pContext->pDFCReg;
	//XLLP_UINT32_T data;
	XLLP_UINT32_T i=0;
	
	
	XLLP_UINT32_T bytesMulti = (bytes & 0xFFFFFFFC);
	XLLP_UINT32_T bytesRemain = bytes & 0x03;
	P_XLLP_UINT32_T temp;
	// caller guarantee pBuffer contains appropriate data thereby
	// it is impossible for bytes not to be a multiple of 4 byte
	while(i< ((XLLP_UINT32_T)dataSize) ) {
		if (i < bytesMulti) {
			temp = (P_XLLP_UINT32_T)pBuffer;
			pDFCReg->nddb = *temp; 
		}
		else if (i == bytesMulti && bytesRemain){
				XLLP_UINT32_T j = 0, data = 0xFFFFFFFF;
				while (j < bytesRemain) {
				  data &= (XLLP_UINT8_T)(*pBuffer) << j;	
				  pBuffer++;
				  j++;
				}
			pDFCReg->nddb = data; 
		}
		else {
			pDFCReg->nddb = 0xFFFFFFFF;
		}
		i += 4;
		pBuffer += 4;
	}
	
	return;
}			

/******************************************************************************
	XllpFlashDfcReadFIFO
	Description:
		This function reads data from data buffer of DFC.Bytes can be any less than 
		or equal to dataSize, the left is ignored by ReadFIFO though they will be 
		read from NDDB to clear data buffer. 		
  Input Parameters:
	  pContext
	    Pointer to DFC context structure
	  bytes
	    Indicating how much data should be read into buffer. 
	  dataSize
	    Specifing length of data transferred to/from DFC, which includes 
		  padding bytes
	Output Parameters:
	  pBuffer
		  Pointer to the data buffer where data should be placed.
	Returns:
		None  
*******************************************************************************/

void 
XllpFlashDfcReadFIFO(P_XLLP_DFC_CONTEXT_T pContext, 
		P_XLLP_UINT8_T pBuffer, XLLP_UINT32_T bytes)
{
	P_XLLP_DFC_REG_T pDFCReg = pContext->pDFCReg;
	XLLP_UINT32_T data;
	XLLP_UINT32_T i=0;
	
	XLLP_UINT32_T bytesMulti = (bytes & 0xFFFFFFFC);
	XLLP_UINT32_T bytesRemain = bytes & 0x03;
	P_XLLP_UINT32_T temp;

	while (i < bytesMulti) {
	    temp = (P_XLLP_UINT32_T)pBuffer;
	    *temp = pDFCReg->nddb;
	    i += 4;
	    pBuffer += 4;
	}

	if (bytesRemain)
	{
		data = pDFCReg->nddb;
		for ( i = 0; i<bytesRemain; i++) {
			*pBuffer++ = (XLLP_UINT8_T)((data>>(8*i)) & 0xFF);
		}
	}

	return;
}
			
/******************************************************************************
	XllpFlashDfcWriteFIFO
	Description:
		Write to data buffer of DFC from a buffer.Bytes can be same as dataSize, 
		also can be dataSize-paddingSize, but cant be random value, the left will 
		be automatically padded by WriteFIFO.
  Input Parameters:
	  pContext
	    Pointer to DFC context structure
	  bytes
	    Indicating how much data should be read into buffer. 
	  dataSize
	    Specifing length of data transferred to/from DFC, which includes 
		  padding bytes
		pBuffer
		  Pointer to the data buffer where data will be taken from to be written to 
		  DFC data buffer
	Output Parameters:
	  None
	Returns:
		None  
*******************************************************************************/
void 
XllpFlashDfcWriteFIFO(P_XLLP_DFC_CONTEXT_T pContext, 
		P_XLLP_UINT8_T pBuffer, XLLP_UINT32_T bytes) 
{
	P_XLLP_DFC_REG_T pDFCReg = pContext->pDFCReg;
	XLLP_UINT32_T i=0;
	P_XLLP_UINT32_T temp;

	bytes = (bytes & 0xFFFFFFFC);
	
	while (i < bytes) {
	    temp = (P_XLLP_UINT32_T)pBuffer;
	    pDFCReg->nddb = *temp;
	    i += 4;
	    pBuffer += 4;
	}
}
			
/******************************************************************************
	XllpFlashDfcReadDFCStatus

	Description:
		Read data flash controller status register.
  Input Parameters:
	  pContext
	    Pointer to DFC context structure
	Output Parameters:
	  pStatus
	    Used to retrieve a content of NDSR register back to caller
	Returns:
		None  
*******************************************************************************/
void 
XllpFlashDfcReadDFCStatus(P_XLLP_DFC_CONTEXT_T pContext, 
		P_XLLP_UINT32_T pStatus)
{
	*pStatus = (pContext->pDFCReg->ndsr) &  XLLP_DFC_NDSR_RESERVED;
	return;
}
			
/******************************************************************************
	XllpFlashDfcReadPageCounter

	Description:
		This function reads number of pages completed for read and program commands. 
		It takes into the account if the operation is for CS0 or CS1 depending on 
		settings of  chipSelect parameter of DFC Mode structure
  Input Parameters:
	  pContext
	    Pointer to DFC context structure
	Output Parameters:
	  pPageCounter
	    Used to retrieve number of pages  from NDPCR register back to caller
	Returns:
		None  
*******************************************************************************/
void 
XllpFlashDfcReadPageCounter(P_XLLP_DFC_CONTEXT_T pContext, 
		P_XLLP_UINT32_T pPageCounter)
{
	XLLP_UINT32_T pgcnt = pContext->pDFCReg->ndpcr;
	*pPageCounter = pContext->pDFCMode->chipSelect ? \
		(pgcnt>>16) & 0x3F : pgcnt & 0x3F;
	return;
}

/******************************************************************************
	XllpFlashDfcReadBadBlockAddr

	Description:
		This function reads bad block address in units of block starting from 0 if 
		bad block is detected. It takes into the account if the operation is for 
		CS0 or CS1  depending on settings of chipSelect parameter of DFC Mode 
		structure.
  Input Parameters:
	  pContext
	    Pointer to DFC context structure
	Output Parameters:
	  pBadBlockAddr
	    Used to retrieve bad block address back to caller if bad block is detected
	Returns:
		None  
*******************************************************************************/
void 
XllpFlashDfcReadBadBlockAddr(P_XLLP_DFC_CONTEXT_T pContext, 
		P_XLLP_UINT32_T pBadBlockAddr)
{
	XLLP_UINT32_T ndbdr = pContext->pDFCMode->chipSelect ? \
		pContext->pDFCReg->ndbdr1 : pContext->pDFCReg->ndbdr0;
	if (512 == pContext->pFlashInfo->pageSize) {
		*pBadBlockAddr = ndbdr;
	} else if (2048 == pContext->pFlashInfo->pageSize) {
		*pBadBlockAddr = ndbdr;
	}
	return;
}

/******************************************************************************
	XllpFlashDfcEnableInt

	Description:
		This function is used to enable DFC interrupts.	The bits in intMask will be
		used to unmask NDCR register to enable corresponding interrupts.
  Input Parameters:
	  pContext
	    Pointer to DFC context structure
	  intMask
	  	Specifies what interrupts to enable
	Output Parameters:
		None  
	Returns:
		None  
*******************************************************************************/
void 
XllpFlashDfcEnableInt(P_XLLP_DFC_CONTEXT_T pContext, 
		XLLP_UINT32_T intMask)
{
	XLLP_UINT32_T ndcr = pContext->pDFCReg->ndcr;
	ndcr &= ~intMask;
	pContext->pDFCReg->ndcr = ndcr;
	return;
}
			
/******************************************************************************
	XllpFlashDfcDisableInt

	Description:
		This function is used to disable DFC interrupts.
		The bits inintMask will be used to mask NDCR register to disable 
		corresponding interrupts.
	Input Parameters:
	  pContext
	    Pointer to DFC context structure
	  intMask
	  	Specifies what interrupts to disable
	Output Parameters:
		None  
	Returns:
		None  
*******************************************************************************/
void 
XllpFlashDfcDisableInt(P_XLLP_DFC_CONTEXT_T pContext, 
		XLLP_UINT32_T intMask)
{
	XLLP_UINT32_T ndcr = pContext->pDFCReg->ndcr;
	ndcr |= intMask;
	pContext->pDFCReg->ndcr = ndcr;
	return;
}

/******************************************************************************
	XllpFlashDfcClearInt

	Description:
		This function is used to disable DFC interrupts.
		The bits inintMask will be used to clear corresponding interrupts in NDCR register
	Input Parameters:
	  pContext
	    Pointer to DFC context structure
	  intMask
	  	Specifies what interrupts to clear
	Output Parameters:
		None  
	Returns:
		None  
*******************************************************************************/
void 
XllpFlashDfcClearInt(P_XLLP_DFC_CONTEXT_T pContext, 
		XLLP_UINT32_T intMask)
{
	
	DFC_WORKAROUND_ENTER();
	pContext->pDFCReg->ndsr = intMask;
	DFC_WORKAROUND_EXIT();
	return;
}

//			
// high level primitives
//


/******************************************************************************
	XllpFlashDfcInit

	Description:
	  This function does entire DFC initialization according to the NAND flash 
	  type currently used with platform, including setting MFP, set flash timing,
	  set DFC mode, configuring specified flash parameters in DFC, clear ECC logic
	  and page count register. 
  Input Parameters:
	  pContext	--	Pointer to DFC context structure
  Output Parameters:
    None
  Returns:
    XLLP_STATUS_SUCCESS 
      if MFPRs are set correctly
    XLLP_STATUS_NO_RESOURCES 
      if the MFP ownership is already assigned to another device.
    XLLP_STATUS_UNSUPPORTED 
      if specified flash is not support by check bytes per page and pages per
      block
******************************************************************************/
XLLP_STATUS_T 
XllpFlashDfcInit(P_XLLP_DFC_CONTEXT_T pContext, XLLP_DFC_FLASH_TYPE type)
{
	XLLP_STATUS_T status;
	P_XLLP_DFC_REG_T pDFCReg = pContext->pDFCReg;
	P_XLLP_DFC_FLASH_SPECS_T pFlashInfo;
	XLLP_UINT32_T ndcr = 0x00000FFF; //disable all interrupts
	
	status = XllpFlashDfcGetFlashInfo(type, &pFlashInfo);
	if (status != XLLP_STATUS_SUCCESS)
		return status;
	pContext->pFlashInfo = pFlashInfo;

	status = XllpFlashDfcSetGPIO(pContext);
	if (status != XLLP_STATUS_SUCCESS)
		return status;
	XllpFlashDfcSetTiming(pContext);
	//XllpFlashDfcSetClock(pContext, XLLP_BOOL_TRUE);
	
	if (pFlashInfo->enableBusArbiter)
		ndcr |= XLLP_DFC_CTRL_ND_ARB_EN;
		
	if (64 == pFlashInfo->pagePerBlock)
		ndcr |= XLLP_DFC_CTRL_PG_PER_BLK;
	else if (32 != pFlashInfo->pagePerBlock)
		return XLLP_STATUS_UNSUPPORTED;
	
	if (pFlashInfo->rowAddrStart)
		ndcr |= XLLP_DFC_CTRL_RA_START;
		
	ndcr |=  (pFlashInfo->byteCntReadID)<<16;
	
	ndcr |= (pFlashInfo->dfcMode) << 21;
	
	if (pFlashInfo->ncsx)
		ndcr |= XLLP_DFC_CTRL_NCSX;
		
	if (2048 == pFlashInfo->pageSize)
		ndcr |= XLLP_DFC_CTRL_PAGE_SZ;
	else if (512 != pFlashInfo->pageSize)
		return XLLP_STATUS_UNSUPPORTED;
		
	if (16 == pFlashInfo->widthFlash)
		ndcr |= XLLP_DFC_CTRL_DWIDTH_M;
	else if (8 != pFlashInfo->widthFlash)
		return XLLP_STATUS_UNSUPPORTED;
		
	if (16 == pFlashInfo->widthDFC)
		ndcr |= XLLP_DFC_CTRL_DWIDTH_C;
	else if (8 != pFlashInfo->widthDFC)
		return XLLP_STATUS_UNSUPPORTED;
		
	pDFCReg->ndcr = ndcr;
	
	//XllpFlashDfcClearECC(pContext);
	//XllpFlashDfcClearPGCNT(pContext);
	
	XllpFlashDfcSetDMA(pContext);
	XllpFlashDfcSetECC(pContext);
	XllpFlashDfcSetSpare(pContext);
	
	return XLLP_STATUS_SUCCESS;	
}
/******************************************************************************
	XllpFlashDfcInitNOGPIO

	Description:
	  This function does entire DFC initialization according to the NAND flash 
	  type currently used with platform, including set flash timing,
	  set DFC mode, configuring specified flash parameters in DFC, clear ECC logic
	  and page count register. The only difference with XllpFlashDfcInit is that 
	  it does not set MFP&GPIO, very useful in OS loader
  Input Parameters:
	  pContext	--	Pointer to DFC context structure
  Output Parameters:
    None
  Returns:
    XLLP_STATUS_SUCCESS 
      if MFPRs are set correctly
    XLLP_STATUS_NO_RESOURCES 
      if the MFP ownership is already assigned to another device.
    XLLP_STATUS_UNSUPPORTED 
      if specified flash is not support by check bytes per page and pages per
      block
******************************************************************************/
XLLP_STATUS_T 
XllpFlashDfcInitNOGPIO(P_XLLP_DFC_CONTEXT_T pContext, XLLP_DFC_FLASH_TYPE type)
{
	P_XLLP_DFC_REG_T pDFCReg = pContext->pDFCReg;
	P_XLLP_DFC_FLASH_SPECS_T pFlashInfo;
	XLLP_UINT32_T ndcr = 0x00000FFF; //disable all interrupts
	XLLP_STATUS_T status;
	
	status = XllpFlashDfcGetFlashInfo(type, &pFlashInfo);
	if (status != XLLP_STATUS_SUCCESS)
		return status;
	pContext->pFlashInfo = pFlashInfo;

	XllpFlashDfcSetTiming(pContext);
	//XllpFlashDfcSetClock(pContext, XLLP_BOOL_TRUE);
	
	if (pFlashInfo->enableBusArbiter)
		ndcr |= XLLP_DFC_CTRL_ND_ARB_EN;
		
	if (64 == pFlashInfo->pagePerBlock)
		ndcr |= XLLP_DFC_CTRL_PG_PER_BLK;
	else if (32 != pFlashInfo->pagePerBlock)
		return XLLP_STATUS_UNSUPPORTED;
	
	if (pFlashInfo->rowAddrStart)
		ndcr |= XLLP_DFC_CTRL_RA_START;
		
	ndcr |=  (pFlashInfo->byteCntReadID)<<16;
	
	ndcr |= (pFlashInfo->dfcMode) << 21;
	
	if (pFlashInfo->ncsx)
		ndcr |= XLLP_DFC_CTRL_NCSX;
		
	if (2048 == pFlashInfo->pageSize)
		ndcr |= XLLP_DFC_CTRL_PAGE_SZ;
	else if (512 != pFlashInfo->pageSize)
		return XLLP_STATUS_UNSUPPORTED;
		
	if (16 == pFlashInfo->widthFlash)
		ndcr |= XLLP_DFC_CTRL_DWIDTH_M;
	else if (8 != pFlashInfo->widthFlash)
		return XLLP_STATUS_UNSUPPORTED;
		
	if (16 == pFlashInfo->widthDFC)
		ndcr |= XLLP_DFC_CTRL_DWIDTH_C;
	else if (8 != pFlashInfo->widthDFC)
		return XLLP_STATUS_UNSUPPORTED;
		
	pDFCReg->ndcr = ndcr;
	
	//XllpFlashDfcClearECC(pContext);
	//XllpFlashDfcClearPGCNT(pContext);
	
	XllpFlashDfcSetDMA(pContext);
	XllpFlashDfcSetECC(pContext);
	XllpFlashDfcSetSpare(pContext);
	
	return XLLP_STATUS_SUCCESS;	
}

// This macro will be used in following NAND operation functions.
// It is used to clear command buffer to ensure cmd buffer is empty
// in case of operation is timeout
#define ClearCMDBuf()          \
{                               \
		XllpFlashDfcStop(pContext); \
		XllpOstDelayMicroSeconds(pContext->pOSTReg, NAND_OTHER_TIMEOUT);					\
		return status;              \
}

/******************************************************************************
	XllpFlashDfcWrite

	Description:
		This function does page programming. It supports multiple page programming. 
		The function behaves in according to settings in DFC mode structure of DFC
		context. It is up to caller to construct the data in the buffer: data + 
		spare + padding, depending on the mode. Before calling this function, 
		DFC modes must be in consistent state with value specified in context struct
  Input Parameters:
	  pContext
	    Pointer to DFC context structure
	  pBuffer
      Pointer to the data buffer where data will be taken from to be written
      to DFC data buffer
		startPage
      the starting page number of where this function will take place
    numPages
      It specifies the number of pages of data to be transferred
    timeout
	    Timeout value for this transaction, in units of micro seconds
	Output Parameters:
	  pBadBlockNum
      Used to pass the bad block Infomation back to caller if bad block is 
      detected
	Returns:
		XLLP_STATUS_SUCCESS if execution succeeds
		XLLP_STATUS_FAILURE if bad block is detected
		XLL_STATUS_TIMEOUT if timeout
*******************************************************************************/
XLLP_STATUS_T 
XllpFlashDfcWrite (P_XLLP_DFC_CONTEXT_T pContext, 
		P_XLLP_UINT8_T pBuffer, XLLP_UINT32_T startPage, 
		XLLP_UINT32_T numPages, XLLP_UINT32_T timeout, 
		P_XLLP_UINT32_T pBadBlockNum)
{
	P_XLLP_DFC_FLASH_SPECS_T	pFlashInfo = pContext->pFlashInfo;
	P_XLLP_DFC_MODE_T	pDFCMode = pContext->pDFCMode;
	
	XLLP_UINT32_T pageSize = pFlashInfo->pageSize; // 512 or 2048
	XLLP_UINT32_T addr;
	XLLP_UINT32_T i=0;
	P_XLLP_UINT8_T buf = pBuffer;
	XLLP_DFC_DATA_SIZE_T dataSize;
	XLLP_DFC_PADDING_SIZE_T paddingSize;
	XLLP_STATUS_T status = XLLP_STATUS_WRONG_PARAMETER;
	XLLP_UINT32_T event, eventOut, ndbbr;

	addr = pageSize * startPage;	
	XllpFlashDfcGetPattern(pContext, 
	  (XLLP_UINT16_T)pFlashInfo->program, 
		&dataSize, &paddingSize);
	
	while(i < numPages) {
		// Send command
		status = XllpFlashDfcSendCommand(pContext, (XLLP_UINT16_T)pFlashInfo->program,
		 addr, 1);
		if (status != XLLP_STATUS_SUCCESS)
			break; 
		
		// Wait for WRDREQM(write FIFO need to be filled due to program command) 
		status = XllpFlashDfcWaitEvent(pContext, XLLP_DFC_STATUS_WRDREQ, &eventOut,  timeout, 
			XLLP_FALSE);
		if (status != XLLP_STATUS_SUCCESS)
			break;//Timeout
		
		// Write FIFO with a page of data	
		XllpFlashDfcWriteFIFOPartial(pContext, buf, (XLLP_UINT32_T)dataSize, dataSize);
		
		if (pDFCMode->chipSelect) {
			event = XLLP_DFC_STATUS_CS1_BBD | XLLP_DFC_STATUS_CS1_CMDD;
		}else {
			event = XLLP_DFC_STATUS_CS0_BBD | XLLP_DFC_STATUS_CS0_CMDD;
		}
		// Wait for BBDM(bad block detected) and
		//          CMDDM(command done successfully)
		status = XllpFlashDfcWaitEvent(pContext, event, &eventOut, timeout, 
			XLLP_FALSE);
			
		if (status != XLLP_STATUS_SUCCESS)
			break;//Timeout
		
		//XllpFlashDfcReadDFCStatus(pContext, &ndsr);
		if (eventOut & (pDFCMode->chipSelect ?
			 XLLP_DFC_STATUS_CS1_BBD : XLLP_DFC_STATUS_CS0_BBD)) {
			// Bad block detected
			
			// test purpose since we can get bad block address from page address
			// it can be used on multiple page program
			XllpFlashDfcReadBadBlockAddr(pContext, &ndbbr);
			pFlashInfo->NDBBR2Addr((XLLP_UINT16_T)pFlashInfo->program, ndbbr, &addr);
			*pBadBlockNum = addr / (pFlashInfo->pageSize * pFlashInfo->pagePerBlock);
			
			return XLLP_STATUS_FAILURE;
		}
		buf += dataSize;	
		addr += pageSize;
		i++;
	}
	if (status != XLLP_STATUS_SUCCESS) 
		ClearCMDBuf();
	
	return status;
}

/******************************************************************************
	XllpFlashDfcRead
	
	Description:
		This function dose the page read. It supports multiple page operation.  The
		function behaves in according to settings in DFC mode structure of DFC 
		handle. It is up to caller to analyze the data in the buffer: data + spare
		+ padding, depending on the mode. Caller can get padding and data size by 
		XllpFlashDfcGetPattern.Before calling this function, DFC modes must be 
		in consistent state with value specified in context structure. In case of 
		double bit error, the function will try three(might change) times mostly
  Input Parameters:
	  pContext
	    Pointer to DFC context structure
	  bufBytesPerPG
	    bytes number per page in buffer. It can be full size(including padding
	    bytes) or data only or data+spare.
	  startPage
      the starting page number of where this function will take place
    numPages
      It specifies the number of pages of data to be transferred
    timeout
	    Timeout value for this transaction, in units of micro seconds
	Output Parameters:
	  pBuffer
      user provided buffer used to receive data from flash.
    eccErrorPageNum
      page number whose double bit ECC error is detected
	Returns:
		XLLP_STATUS_SUCCESS if execution succeeds
		XLLP_STATUS_FAILURE if double bit ECC error is detected
		XLL_STATUS_TIMEOUT if timeout
*******************************************************************************/
#define READ_ATTEMPT 3
			
XLLP_STATUS_T 
XllpFlashDfcRead (P_XLLP_DFC_CONTEXT_T pContext, 
		P_XLLP_UINT8_T pBuffer, XLLP_UINT32_T bufBytesPerPG, 
		XLLP_UINT32_T startPage, XLLP_UINT32_T numPages, 
		XLLP_UINT32_T timeout, P_XLLP_UINT32_T eccErrorPageNum)
{
	P_XLLP_DFC_FLASH_SPECS_T	pFlashInfo = pContext->pFlashInfo;
	
	XLLP_UINT32_T pageSize = pFlashInfo->pageSize; // 512 or 2048
	XLLP_UINT32_T addr;
	XLLP_UINT32_T i=0;
	P_XLLP_UINT8_T buf = pBuffer;
	XLLP_DFC_DATA_SIZE_T dataSize;
	XLLP_DFC_PADDING_SIZE_T paddingSize;
	XLLP_STATUS_T status = XLLP_STATUS_WRONG_PARAMETER;
	XLLP_UINT32_T event, eventOut;
	XLLP_UINT32_T try_count;

	addr = startPage *pageSize;	
	XllpFlashDfcGetPattern(pContext, (XLLP_UINT16_T)pFlashInfo->read1, 
		&dataSize, &paddingSize);
		
	if (bufBytesPerPG > (XLLP_UINT32_T)dataSize)
	  return XLLP_STATUS_INVALID_STATE;
	  
	while(i < numPages) {
		status = XLLP_STATUS_FAILURE;
		try_count = READ_ATTEMPT;
		
		while (status!=XLLP_STATUS_SUCCESS && try_count--) {
			// Send command
			status = XllpFlashDfcSendCommand(pContext, (XLLP_UINT16_T)pFlashInfo->read1,
			  addr, 1);
			if (status != XLLP_STATUS_SUCCESS)
				ClearCMDBuf();
			
			event = XLLP_DFC_STATUS_RDDREQ | XLLP_DFC_STATUS_DBERR;
			// Wait for RDDREQM(read data available at FIFO) and 
			//          DBERRM(double bit error)
			status = XllpFlashDfcWaitEvent(pContext, event, &eventOut, timeout, 
				XLLP_FALSE);
				
			if (status != XLLP_STATUS_SUCCESS)
				ClearCMDBuf();
		
			//XllpFlashDfcReadDFCStatus(pContext, &ndsr);
			
			if (eventOut & XLLP_DFC_STATUS_DBERR ) {
				// Double bit error
				*eccErrorPageNum = addr / pageSize; 
				status = XLLP_STATUS_FAILURE;
			}else
				status = XLLP_STATUS_SUCCESS;
			
				
			XllpFlashDfcReadFIFOPartial(pContext, buf, bufBytesPerPG, dataSize);
#ifdef CACHE_READ
			if (pFlashInfo->chipID == 0xba20){
				status = XllpFlashDfcSendCommand(pContext, (XLLP_UINT16_T)0x34,
				  0xFFFFFFFF, 0);
				if (status != XLLP_STATUS_SUCCESS){
					ClearCMDBuf();
					printf("send cache read exit command: failed\n");
				}else{
					status = XllpFlashDfcWaitEvent(pContext, XLLP_DFC_STATUS_RDDREQ ,
							&eventOut, NAND_OTHER_TIMEOUT, XLLP_FALSE);
					if (status != XLLP_STATUS_SUCCESS){
						ClearCMDBuf();
						printf("wait for cache read exit completed: timeout\n");
					}
				}
				ClearCMDBuf();
			}
#endif
		}
		if (status != XLLP_STATUS_SUCCESS)
			return status;
		// Read FIFO with a page of data	
		buf += bufBytesPerPG;	
		addr += pageSize;
		i++;
	}
	return status;
}

/******************************************************************************
	XllpFlashDfcErase
	
	Description:
		This function erases flash. It supports multiple block operation. 
  Input Parameters:
	  pContext
	    Pointer to DFC context structure
	  startBlock
      the block number of where this function will take place
    numBlocks
      number of blocks caller hope to program
		timeout
	    Timeout value for this transaction, in units of micro seconds
	Output Parameters:
	  pBadBlockNum
      Used to pass the bad block Infomation back to caller if bad block is 
      detected
	Returns:
		XLLP_STATUS_SUCCESS if execution succeeds
		XLLP_STATUS_FAILURE if bad block is detected
		XLL_STATUS_TIMEOUT if timeout
*******************************************************************************/			
XLLP_STATUS_T XllpFlashDfcErase (P_XLLP_DFC_CONTEXT_T pContext, 
			XLLP_UINT32_T startBlock, XLLP_UINT32_T numBlocks, 
			XLLP_UINT32_T timeout, P_XLLP_UINT32_T pBadBlockNum)
{
	P_XLLP_DFC_FLASH_SPECS_T	pFlashInfo = pContext->pFlashInfo;
	P_XLLP_DFC_MODE_T	pDFCMode = pContext->pDFCMode;
	
	XLLP_UINT32_T pageSize = pFlashInfo->pageSize; // 512 or 2048
	XLLP_UINT32_T pagePerBlock = pFlashInfo->pagePerBlock; //32 or 64
	XLLP_UINT32_T addr;
	XLLP_UINT32_T i=0;
	XLLP_STATUS_T status = XLLP_STATUS_WRONG_PARAMETER;
	XLLP_UINT32_T event, eventOut, ndbbr;
	
	addr = startBlock * pagePerBlock * pageSize;
	
	while(i < numBlocks) {
		// Send command
		status = XllpFlashDfcSendCommand(pContext, (XLLP_UINT16_T)pFlashInfo->erase, 
		  addr, 0);
		
		if (status != XLLP_STATUS_SUCCESS)
			ClearCMDBuf();
		
		if (pDFCMode->chipSelect) {
			event = XLLP_DFC_STATUS_CS1_BBD | XLLP_DFC_STATUS_CS1_CMDD;
		}else {
			event = XLLP_DFC_STATUS_CS0_BBD | XLLP_DFC_STATUS_CS0_CMDD;
		}
		// Wait for BBDM(bad block detected) and
		//          CMDDM(command done successfully)
		status = XllpFlashDfcWaitEvent(pContext, event, &eventOut, timeout, 
			XLLP_FALSE);
			
		if (status != XLLP_STATUS_SUCCESS)
			ClearCMDBuf();
		
		//XllpFlashDfcReadDFCStatus(pContext, &ndsr);
		if (eventOut & (pDFCMode->chipSelect ?
			 XLLP_DFC_STATUS_CS1_BBD : XLLP_DFC_STATUS_CS0_BBD)) {
			// Bad block detected
			
			// test purpose since we can get bad block address from page address
			// it can be used on multiple page program
			XllpFlashDfcReadBadBlockAddr(pContext, &ndbbr);
			pFlashInfo->NDBBR2Addr((XLLP_UINT16_T)pFlashInfo->erase, ndbbr, &addr);
			*pBadBlockNum = addr / (pageSize * pagePerBlock);
			return XLLP_STATUS_FAILURE;
		}
		addr += pagePerBlock * pageSize;
		i++;
	}
	return status;
}

/******************************************************************************
	XllpFlashDfcReadID
	
	Description:
		It gets manufacturer ID and device ID of flash by issuing ReadID command
  Input Parameters:
	  pContext
	    Pointer to DFC context structure
	Output Parameters:
	  pMakerID
      Pointer to get manufacturer ID
    pDevID
      Pointer to get device ID
	Returns:
		XLLP_STATUS_SUCCESS if execution succeeds
		XLL_STATUS_TIMEOUT if timeout
*******************************************************************************/			
XLLP_STATUS_T
XllpFlashDfcReadID (P_XLLP_DFC_CONTEXT_T pContext, 
			P_XLLP_UINT8_T pMakerID, P_XLLP_UINT8_T pDevID)
{
	P_XLLP_DFC_FLASH_SPECS_T	pFlashInfo = pContext->pFlashInfo;
	//P_XLLP_DFC_MODE_T	pDFCMode = pContext->pDFCMode;
	XLLP_STATUS_T status;
	XLLP_UINT32_T count;
	XLLP_UINT32_T eventOut;
	//XLLP_UINT32_T event;
	XLLP_UINT8_T tmp[(XLLP_DFC_DATA_SIZE_ID + 3) & 4];
	
	// Send command
	status = XllpFlashDfcSendCommand(pContext, (XLLP_UINT16_T)pFlashInfo->readID, 0xFFFFFFFF, 0);
	if (status != XLLP_STATUS_SUCCESS)
		ClearCMDBuf();
		
	// Wait for RDDREQ(read data availabe at FIFO)
	status = XllpFlashDfcWaitEvent(pContext, XLLP_DFC_STATUS_RDDREQ, &eventOut, 
		NAND_OTHER_TIMEOUT, XLLP_FALSE);
	if (status != XLLP_STATUS_SUCCESS)
		ClearCMDBuf();
		 
	count = pContext->pFlashInfo->byteCntReadID;
	// read FIFO to get ID
	XllpFlashDfcReadFIFOPartial(pContext, (P_XLLP_UINT8_T)tmp, count, XLLP_DFC_DATA_SIZE_ID);
	*pMakerID = tmp[0];
	*pDevID = tmp[1];

	return XLLP_STATUS_SUCCESS;
}

/******************************************************************************
	XllpFlashDfcReadStatus

	Description:
		It gets current status Infomation of flash by issuing command of Read Status. 
		DFC has capability to enable the automatic checking of the program/erase 
		status by issuing 70H command. This function is provided here for those 
		flash with Read_Status command other than 70H or drivers who want to do 
		manual checking
  Input Parameters:
	  pContext
	    Pointer to DFC context structure
	Output Parameters:
	  pStatus
	    Pointer to get status Infomation
	Returns:
		XLLP_STATUS_SUCCESS if execution succeeds
		XLL_STATUS_TIMEOUT if timeout
*******************************************************************************/
XLLP_STATUS_T 
XllpFlashDfcReadStatus (P_XLLP_DFC_CONTEXT_T pContext, 
			P_XLLP_UINT8_T pStatus)
{
	P_XLLP_DFC_FLASH_SPECS_T	pFlashInfo = pContext->pFlashInfo;
	XLLP_STATUS_T status;
	XLLP_UINT32_T eventOut;
	
	// Send command
	status = XllpFlashDfcSendCommand(pContext, (XLLP_UINT16_T)pFlashInfo->readStatus, 
		0xFFFFFFFF, 0);
	if (status != XLLP_STATUS_SUCCESS)
		ClearCMDBuf();
	
	// Wait for RDDREQM(read data availabe at FIFO)
	status = XllpFlashDfcWaitEvent(pContext, XLLP_DFC_STATUS_RDDREQ, &eventOut, 
		NAND_OTHER_TIMEOUT, XLLP_FALSE);
		
	if (status != XLLP_STATUS_SUCCESS) 
		ClearCMDBuf();
		
	// read FIFO to get status
	XllpFlashDfcReadFIFOPartial(pContext, pStatus, 1, XLLP_DFC_DATA_SIZE_STATUS);
	
	return XLLP_STATUS_SUCCESS;
}

/******************************************************************************
	XllpFlashDfcReset

	Description:
		It reset the flash. The function can be called at any time when the device 
		is in Busy state during random read/program/erase mode and reset operation 
		will abort all these operations. After reset operation the device is ready 
		to wait for next command
	Input Parameters:
	  pContext
	    Pointer to DFC context structure
	Output Parameters:
	   None
	Returns:
		XLLP_STATUS_SUCCESS if execution succeeds
		XLL_STATUS_TIMEOUT if timeout
*******************************************************************************/
XLLP_STATUS_T 
XllpFlashDfcReset (P_XLLP_DFC_CONTEXT_T pContext)
{
	P_XLLP_DFC_FLASH_SPECS_T	pFlashInfo = pContext->pFlashInfo;
	XLLP_UINT32_T event, eventOut;
	XLLP_STATUS_T status;
	
	// Send command
	XllpFlashDfcSendCommand(pContext, (XLLP_UINT16_T)pFlashInfo->reset, 
	  0xFFFFFFFF, 0);
	
	event = (pContext->pDFCMode->chipSelect)? \
			XLLP_DFC_INT_CS1_CMDDM : XLLP_DFC_INT_CS0_CMDDM;
	
	// Wait for CMDDM(command done successfully)
	status = XllpFlashDfcWaitEvent(pContext, event, &eventOut, 
		NAND_OTHER_TIMEOUT, XLLP_FALSE);
		
	if (status != XLLP_STATUS_SUCCESS) 
		ClearCMDBuf();
	
	// wait 10 ms until flash device is stable
	XllpOstDelayMilliSeconds(pContext->pOSTReg, 10);	
	
	return XLLP_STATUS_SUCCESS;
}


/******************************************************************************
	Description:
		It gets block lock status Infomation of flash by issuing command of 
		Read Lock Status. 
   Input Parameters:
	  pContext
	    Pointer to DFC context structure
	Output Parameters:
	  pStatus
	    Pointer to get status Infomation
	Returns:
		XLLP_STATUS_SUCCESS if execution succeeds
		XLL_STATUS_TIMEOUT if timeout
*******************************************************************************/
XLLP_STATUS_T 
XllpFlashDfcReadLockStatus (P_XLLP_DFC_CONTEXT_T pContext, XLLP_UINT32_T block,
							P_XLLP_UINT8_T pStatus)
{
	P_XLLP_DFC_REG_T pDFCReg = pContext->pDFCReg;
	P_XLLP_DFC_FLASH_SPECS_T	pFlashInfo = pContext->pFlashInfo;
	P_XLLP_DFC_MODE_T	pDFCMode = pContext->pDFCMode;
	XLLP_UINT32_T pageSize = pContext->pFlashInfo->pageSize;
	XLLP_UINT32_T pagesPerBlock = pContext->pFlashInfo->pagePerBlock;
	XLLP_UINT32_T event, eventOut;
	XLLP_UINT32_T cmd = pFlashInfo->lockStatus;
	XLLP_STATUS_T status;
	XLLP_UINT32_T ndcb0=0, ndcb1=0, ndcb2=0;
	
	// Clear DFC status register
	pDFCReg->ndsr = XLLP_DFC_NDSR_CLEAR;

	// Set ND_RUN
	pDFCReg->ndcr |= XLLP_DFC_CTRL_ND_RUN;

    // Wait for write command request
	status = XllpFlashDfcWaitEvent(pContext, XLLP_DFC_STATUS_WRCMDREQ, 
		&eventOut, NAND_CMD_TIMEOUT, XLLP_FALSE);
	if (status != XLLP_STATUS_SUCCESS)
	  return status; //Timeout

	cmd &= 0xFF;
  	ndcb0 = (1<<25) | (pDFCMode->chipSelect<<24) | (XLLP_DFC_CMD_READ_ID | 
  	         XLLP_DFC_NDCB_DBC_SINGLE | (3 << XLLP_DFC_NDCB_ADDR_CYC_SHIFT) | cmd);

  	// starting block address
	ndcb1 = (block * pagesPerBlock * pageSize) >> 9;
  	
	// Write to DFC command register
	pDFCReg->ndcb0 = ndcb0;
	pDFCReg->ndcb0 = ndcb1;
	pDFCReg->ndcb0 = ndcb2;
	
	// Wait for RDDREQ(read data availabe at FIFO)
	status = XllpFlashDfcWaitEvent(pContext, XLLP_DFC_STATUS_RDDREQ, &eventOut, 
		NAND_OTHER_TIMEOUT, XLLP_FALSE);
	if (status != XLLP_STATUS_SUCCESS) 
		return status; 
		
	// read FIFO 
	XllpFlashDfcReadFIFO(pContext, pStatus, 8);

	if (pDFCMode->chipSelect) {
		event = XLLP_DFC_STATUS_CS1_CMDD;
	}else {
		event = XLLP_DFC_STATUS_CS0_CMDD;
	}
    
    // Wait for operation to complete 	
	status = XllpFlashDfcWaitEvent(pContext, event, &eventOut, 
		NAND_OTHER_TIMEOUT, XLLP_FALSE);
	if (status != XLLP_STATUS_SUCCESS)
		return status;
	
	return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T
XllpFlashDfcUnlock (
		P_XLLP_DFC_CONTEXT_T __attribute__((unused)) pContext, 
		XLLP_UINT32_T __attribute__((unused)) low_addr, 
		XLLP_UINT32_T __attribute__((unused)) high_addr
		)
{
	P_XLLP_DFC_FLASH_SPECS_T pFlashInfo = pContext->pFlashInfo;
	P_XLLP_DFC_MODE_T pDFCMode = pContext->pDFCMode;

	unsigned char buf[50];
	XLLP_DFC_DATA_SIZE_T dataSize;
	XLLP_DFC_PADDING_SIZE_T paddingSize;
	XLLP_STATUS_T status = XLLP_STATUS_WRONG_PARAMETER;
	XLLP_UINT32_T event, eventOut;

	if (pFlashInfo->unlockAddrCycles == 0) {
		printf("the current NAND chip does not support Block Unlocking.\n");
		return XLLP_STATUS_UNSUPPORTED;
	} else printf("Unlocking nand ... ");

	// Send command
	status = XllpFlashDfcSendCommand(pContext, 
		(XLLP_UINT16_T)pFlashInfo->unlock_low, 0x0, 0);
	if (status != XLLP_STATUS_SUCCESS)
		return -1; 

	// Wait for WRDREQM(write FIFO need to be filled due to program command) 
	status = XllpFlashDfcWaitEvent(pContext, XLLP_DFC_STATUS_RDDREQ, 
		&eventOut, 10, XLLP_FALSE);
	if (status != XLLP_STATUS_SUCCESS)
		return -1; //Timeout

	XllpFlashDfcGetPattern(pContext, 
		(XLLP_UINT16_T)pFlashInfo->readID, &dataSize, &paddingSize);

	// Write FIFO with a page of data	
	XllpFlashDfcReadFIFOPartial(pContext, buf, 
		(XLLP_UINT16_T)pFlashInfo->byteCntReadID, dataSize);

	if (pDFCMode->chipSelect) {
		event = XLLP_DFC_STATUS_CS1_CMDD;
	}else {
		event = XLLP_DFC_STATUS_CS0_CMDD;
	}

	// Wait for BBDM(bad block detected) and
	//			CMDDM(command done successfully)
	status = XllpFlashDfcWaitEvent(pContext, event,
		&eventOut, 10, XLLP_FALSE);

	if (status != XLLP_STATUS_SUCCESS)
		return -1; //Timeout

	status = XllpFlashDfcSendCommand(pContext, 
		(XLLP_UINT16_T)pFlashInfo->unlock_high,	high_addr, 0);
	if (status != XLLP_STATUS_SUCCESS)
		return -1;

	// Wait for WRDREQM(write FIFO need to be filled due to program command) 
	status = XllpFlashDfcWaitEvent(pContext, 
		XLLP_DFC_STATUS_RDDREQ, &eventOut, 10, XLLP_FALSE);
	if (status != XLLP_STATUS_SUCCESS)
		return -1; //Timeout

	XllpFlashDfcGetPattern(pContext, 
		(XLLP_UINT16_T)pFlashInfo->readID, &dataSize, &paddingSize);

	// Write FIFO with a page of data	
	XllpFlashDfcReadFIFOPartial(pContext, buf, 
		(XLLP_UINT16_T)pFlashInfo->byteCntReadID, dataSize);
	
	// Wait for BBDM(bad block detected) and
	//			CMDDM(command done successfully)
	status = XllpFlashDfcWaitEvent(pContext, event, &eventOut, 10, 
		XLLP_FALSE);
		
	if (status != XLLP_STATUS_SUCCESS)
		return -1; //Timeout
	else printf("Done.\n");

	return status;
}

XLLP_STATUS_T
XllpFlashDfcLock (P_XLLP_DFC_CONTEXT_T pContext)
{
	P_XLLP_DFC_FLASH_SPECS_T	pFlashInfo = pContext->pFlashInfo;
	P_XLLP_DFC_MODE_T	pDFCMode = pContext->pDFCMode;
	XLLP_UINT32_T event, eventOut;
	XLLP_STATUS_T status;

	// Send command
	status = XllpFlashDfcSendCommand(pContext, (XLLP_UINT16_T)pFlashInfo->lock, 0xFFFFFFFF, 0);
	if (status != XLLP_STATUS_SUCCESS)
	  return status; 
	
	// Clear event bits
	if (pDFCMode->chipSelect) {
		event = XLLP_DFC_STATUS_CS1_CMDD;
	}else {
		event = XLLP_DFC_STATUS_CS0_CMDD;
	}
  
    // Wait for operation to complete 	
	status = XllpFlashDfcWaitEvent(pContext, event, &eventOut, NAND_OTHER_TIMEOUT, XLLP_FALSE);
	if (status != XLLP_STATUS_SUCCESS)
	  return status;

	XllpFlashDfcClearInt(pContext, XLLP_DFC_NDSR_CLEAR);	

	return XLLP_STATUS_SUCCESS;
}

/*
XLLP_STATUS_T 
XllpFlashDfcUnlock (P_XLLP_DFC_CONTEXT_T pContext, 
			XLLP_UINT32_T startBlock, XLLP_UINT32_T numBlocks)
{
	P_XLLP_DFC_REG_T pDFCReg = pContext->pDFCReg;
	P_XLLP_DFC_FLASH_SPECS_T	pFlashInfo = pContext->pFlashInfo;
	P_XLLP_DFC_MODE_T	pDFCMode = pContext->pDFCMode;
	XLLP_UINT32_T pageSize = pContext->pFlashInfo->pageSize;
	XLLP_UINT32_T pagesPerBlock = pContext->pFlashInfo->pagePerBlock;
	XLLP_UINT32_T event, eventOut, cmd;
	XLLP_STATUS_T status;
	XLLP_UINT32_T ndcb0=0, ndcb1=0, ndcb2=0;
	XLLP_UINT32_T endBlock = startBlock + numBlocks;
	XLLP_UINT8_T tmp[7];

	// Map this as two Reset commands.
	// First command would be:
	// NDCB0 : CMD1 = 23h, ADDR_CYC = 011, DBC = 0, NC = 1, CMD_TYPE = Reset
	// NDCB1 : 3 address bytes (start block address)
	// NDCB2 : Zeros

	// Clear DFC status register
	pDFCReg->ndsr = XLLP_DFC_NDSR_CLEAR;

	// Set ND_RUN
	pDFCReg->ndcr |= XLLP_DFC_CTRL_ND_RUN;

	status = XllpFlashDfcWaitEvent(pContext, XLLP_DFC_STATUS_WRCMDREQ, &eventOut, NAND_CMD_TIMEOUT, XLLP_FALSE);
	if (status != XLLP_STATUS_SUCCESS)
	{
		status |= 0xA0;
		return status; //Timeout
	}
	cmd = pFlashInfo->unlock & 0xFF;
//  	ndcb0 = (pDFCMode->chipSelect<<24) | XLLP_DFC_CMD_RESET | XLLP_DFC_NDCB_NC |
//  			 XLLP_DFC_NDCB_DBC_SINGLE | (3 << XLLP_DFC_NDCB_ADDR_CYC_SHIFT) | cmd;
  	ndcb0 = (pDFCMode->chipSelect<<24) | XLLP_DFC_CMD_READ_ID | XLLP_DFC_NDCB_NC |
  			 XLLP_DFC_NDCB_DBC_SINGLE | (3 << XLLP_DFC_NDCB_ADDR_CYC_SHIFT) | cmd;
  	
  	// starting block address
	ndcb1 = (startBlock * pagesPerBlock * pageSize) >> 9;
	
	// Write to DFC command register
	pDFCReg->ndcb0 = ndcb0;
	pDFCReg->ndcb0 = ndcb1;
	pDFCReg->ndcb0 = ndcb2;

	// Wait for RDDREQ(read data availabe at FIFO)
	status = XllpFlashDfcWaitEvent(pContext, XLLP_DFC_STATUS_RDDREQ, &eventOut, 
		NAND_OTHER_TIMEOUT, XLLP_FALSE);
	if (status != XLLP_STATUS_SUCCESS)
	{
		return status;
	}
	 
	// read FIFO to get ID
	XllpFlashDfcReadFIFO(pContext, (P_XLLP_UINT8_T)tmp, 8);

	if (pDFCMode->chipSelect) {
		event =  XLLP_DFC_STATUS_CS1_CMDD;
	}else {
		event =  XLLP_DFC_STATUS_CS0_CMDD;
	}
    
    // Wait for operation to complete 	
	status = XllpFlashDfcWaitEvent(pContext, event, &eventOut, 
		NAND_OTHER_TIMEOUT, XLLP_FALSE);
	{
		return status; //Timeout
	}

	// Second command would be:
	// NDCB0 : CMD1 = 24h, ADDR_CYC = 011, DBC = 0, NC = 0, CMD_TYPPE = Reset
	// NDCB1 : 3 address bytes (end block address)
	// NDCB2 : Zeros
	
	// Clear DFC status register
	pDFCReg->ndsr = XLLP_DFC_NDSR_CLEAR;

	// Set ND_RUN
	pDFCReg->ndcr |= XLLP_DFC_CTRL_ND_RUN;

	status = XllpFlashDfcWaitEvent(pContext, XLLP_DFC_STATUS_WRCMDREQ, &eventOut, 
		NAND_CMD_TIMEOUT, XLLP_FALSE);
	if (status != XLLP_STATUS_SUCCESS)
	{
		status |= 0xC0;
		return status; //Timeout
	}

	ndcb0=0, ndcb1=0, ndcb2=0;
	cmd = (pFlashInfo->unlock>>8) & 0xFF;
//  	ndcb0 = (pDFCMode->chipSelect<<24) | XLLP_DFC_CMD_RESET |
//  			 XLLP_DFC_NDCB_DBC_SINGLE  | (3 << XLLP_DFC_NDCB_ADDR_CYC_SHIFT) | cmd;
	ndcb0 = (pDFCMode->chipSelect<<24) | XLLP_DFC_CMD_READ_ID |
		 	 XLLP_DFC_NDCB_DBC_SINGLE  | (3 << XLLP_DFC_NDCB_ADDR_CYC_SHIFT) | cmd;
  	
  	// starting block address
	ndcb1 = (endBlock * pagesPerBlock * pageSize) >> 9;
	
	// Write to DFC command register
	pDFCReg->ndcb0 = ndcb0;
	pDFCReg->ndcb0 = ndcb1;
	pDFCReg->ndcb0 = ndcb2;

	// Wait for RDDREQ(read data availabe at FIFO)
	status = XllpFlashDfcWaitEvent(pContext, XLLP_DFC_STATUS_RDDREQ, &eventOut, 
		NAND_OTHER_TIMEOUT, XLLP_FALSE);
	if (status != XLLP_STATUS_SUCCESS)
	{
		return status;
	}
	 
	// read FIFO to get ID
	XllpFlashDfcReadFIFO(pContext, (P_XLLP_UINT8_T)tmp, 8);

	if (pDFCMode->chipSelect) {
		event =  XLLP_DFC_STATUS_CS1_CMDD;
	}else {
		event =  XLLP_DFC_STATUS_CS0_CMDD;
	}
	
	status = XllpFlashDfcWaitEvent(pContext, event, &eventOut, NAND_OTHER_TIMEOUT, XLLP_FALSE);
	if (status != XLLP_STATUS_SUCCESS) 
	{
		return status; //Timeout
	}
	
	return XLLP_STATUS_SUCCESS;
}
*/
