/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/** FILENAME:	xllp_pm_suspend.c
**
** PURPOSE:	contain APIs for Suspend/Resume.
**
******************************************************************************/

#include "xllp_pm_suspend.h"
#include "xllp_dfc_proc.h"

#define AD3ER_MASK  (XLLP_PM_WE_EXTERNAL0 | XLLP_PM_WE_EXTERNAL1 |\
					XLLP_PM_WE_GENERIC0 | XLLP_PM_WE_GENERIC1 |   \
					XLLP_PM_WE_GENERIC2 | XLLP_PM_WE_GENERIC3 |   \
					XLLP_PM_WE_GENERIC4 | XLLP_PM_WE_GENERIC5 |   \
					XLLP_PM_WE_GENERIC6 | XLLP_PM_WE_GENERIC7 |   \
					XLLP_PM_WE_GENERIC8 | XLLP_PM_WE_GENERIC9 |   \
					XLLP_PM_WE_GENERIC10 | XLLP_PM_WE_GENERIC11 | \
					XLLP_PM_WE_GENERIC12 | XLLP_PM_WE_GENERIC13 | \
					XLLP_PM_WE_OTG | XLLP_PM_WE_USIM0 |           \
					XLLP_PM_WE_USIM1 | XLLP_PM_WE_KP |            \
					XLLP_PM_WE_MUX2 | XLLP_PM_WE_MUX3 |           \
					XLLP_PM_WE_MSL0 | XLLP_PM_WE_USB2 |           \
					XLLP_PM_WE_USBH | XLLP_PM_WE_TSI |            \
					XLLP_PM_WE_OST | XLLP_PM_WE_RTC)
					
#define AD2D1ER_MASK  (XLLP_PM_WE_OST | XLLP_PM_WE_RTC)

#define AD2D0ER_MASK  (XLLP_PM_WE_EXTERNAL0 | XLLP_PM_WE_EXTERNAL1 |\
					XLLP_PM_WE_GENERIC0 | XLLP_PM_WE_GENERIC1 |     \
					XLLP_PM_WE_GENERIC2 | XLLP_PM_WE_GENERIC3 |     \
					XLLP_PM_WE_GENERIC4 | XLLP_PM_WE_GENERIC5 |     \
					XLLP_PM_WE_GENERIC6 | XLLP_PM_WE_GENERIC7 |     \
					XLLP_PM_WE_GENERIC8 | XLLP_PM_WE_GENERIC9 |     \
					XLLP_PM_WE_GENERIC10 | XLLP_PM_WE_GENERIC11 |   \
					XLLP_PM_WE_GENERIC12 | XLLP_PM_WE_GENERIC13 |   \
					XLLP_PM_WE_OTG | XLLP_PM_WE_USIM0 |             \
					XLLP_PM_WE_USIM1 | XLLP_PM_WE_KP |              \
					XLLP_PM_WE_MUX2 | XLLP_PM_WE_MUX3 |             \
					XLLP_PM_WE_MSL0 | XLLP_PM_WE_USB2 |             \
					XLLP_PM_WE_USBH | XLLP_PM_WE_TSI |              \
					XLLP_PM_WE_OST | XLLP_PM_WE_RTC)

#define AD1D0ER_MASK  (XLLP_PM_WE_EXTERNAL0 | XLLP_PM_WE_EXTERNAL1 |\
					XLLP_PM_WE_GENERIC0 | XLLP_PM_WE_GENERIC1 |     \
					XLLP_PM_WE_GENERIC2 | XLLP_PM_WE_GENERIC3 |     \
					XLLP_PM_WE_GENERIC4 | XLLP_PM_WE_GENERIC5 |     \
					XLLP_PM_WE_GENERIC6 | XLLP_PM_WE_GENERIC7 |     \
					XLLP_PM_WE_GENERIC8 | XLLP_PM_WE_GENERIC9 |     \
					XLLP_PM_WE_GENERIC10 | XLLP_PM_WE_GENERIC11 |   \
					XLLP_PM_WE_GENERIC12 | XLLP_PM_WE_GENERIC13 |   \
					XLLP_PM_WE_OTG | XLLP_PM_WE_USIM0 |             \
					XLLP_PM_WE_USIM1 | XLLP_PM_WE_KP |              \
					XLLP_PM_WE_MUX2 | XLLP_PM_WE_MUX3 |             \
					XLLP_PM_WE_MSL0 | XLLP_PM_WE_USB2 |             \
					XLLP_PM_WE_USBH | XLLP_PM_WE_TSI |              \
					XLLP_PM_WE_OST | XLLP_PM_WE_RTC |               \
					XLLP_PM_WE_MLCD)


void XllpPmClearAllWakeupSource(
		P_XLLP_PM_SUSPEND_CONTEXT_T   p_pm_suspend_context,
		XLLP_PM_MODE_T to_mode,
		XLLP_PM_MODE_T from_mode)
{
	/* deepsleep --> run */
	if ((to_mode == XLLP_PM_RUN) &&
		(from_mode == XLLP_PM_DEEPSLEEP))
	{
		p_pm_suspend_context->spmu->pwer &= ~(XLLP_PWER_WER0 | XLLP_PWER_WER1 |
										   XLLP_PWER_WEF0 | XLLP_PWER_WEF1 |
										   XLLP_PWER_WERTC);
		p_pm_suspend_context->spmu->pwsr = XLLP_PWSR_EDR0 | XLLP_PWSR_EDR1 |
										   XLLP_PWSR_EDF0 | XLLP_PWSR_EDF1 |
										   XLLP_PWSR_EERTC;
	}

	/* sleep --> run */
	if ((to_mode == XLLP_PM_RUN) &&
		(from_mode == XLLP_PM_SLEEP))
	{
		p_pm_suspend_context->spmu->pwer &= ~(XLLP_PWER_WER0 | XLLP_PWER_WER1 |
										   XLLP_PWER_WEF0 | XLLP_PWER_WEF1 |
										   XLLP_PWER_WERTC);
		p_pm_suspend_context->spmu->pwsr |= XLLP_PWSR_EDR0 | XLLP_PWSR_EDR1 |
										   XLLP_PWSR_EDF0 | XLLP_PWSR_EDF1 |
										   XLLP_PWSR_EERTC;
		p_pm_suspend_context->bpmu->ad3er &= ~AD3ER_MASK;
		p_pm_suspend_context->bpmu->ad3sr = AD3ER_MASK;
	}


	/* standby --> run */
	if ((to_mode == XLLP_PM_RUN) &&
		(from_mode == XLLP_PM_STANDBY))
	{
		p_pm_suspend_context->bpmu->ad2d0er &= ~AD2D0ER_MASK;
		p_pm_suspend_context->bpmu->ad2d0sr = AD2D0ER_MASK;		
	}	

	/* standby --> lcdrefresh */
	if ((to_mode == XLLP_PM_LCDREFRESH) &&
		(from_mode == XLLP_PM_STANDBY))
	{
		p_pm_suspend_context->bpmu->ad2d1er &= ~AD2D1ER_MASK;
		p_pm_suspend_context->bpmu->ad2d1sr = AD2D1ER_MASK;
											
	}	

	/* lcdrefresh --> run */
	if ((to_mode == XLLP_PM_RUN) &&
		(from_mode == XLLP_PM_LCDREFRESH))
	{
		p_pm_suspend_context->bpmu->ad1d0er &= ~AD1D0ER_MASK;
		p_pm_suspend_context->bpmu->ad1d0sr = AD1D0ER_MASK;
	}	
}

void XllpPmClearWakeupSource(
		P_XLLP_PM_SUSPEND_CONTEXT_T   p_pm_suspend_context,
		XLLP_PM_MODE_T to_mode,
		XLLP_PM_MODE_T from_mode,
		XLLP_PM_WAKEUP_SRC_T src)
{
	/* deepsleep --> run */
	if ((to_mode == XLLP_PM_RUN) &&
		(from_mode == XLLP_PM_DEEPSLEEP))
	{
		if (src & XLLP_PM_WE_RTC)
		{
			p_pm_suspend_context->spmu->pwer &= ~XLLP_PWER_WERTC;
			p_pm_suspend_context->spmu->pwsr = XLLP_PWSR_EERTC;
		}
		
		if (src & XLLP_PM_WE_EXTERNAL0)
		{
			p_pm_suspend_context->spmu->pwer &= ~(XLLP_PWER_WER0 | XLLP_PWER_WEF0);
			p_pm_suspend_context->spmu->pwsr = XLLP_PWSR_EDR0 | XLLP_PWSR_EDF0;
		}
		
		if (src & XLLP_PM_WE_EXTERNAL1)
		{
			p_pm_suspend_context->spmu->pwer &= ~(XLLP_PWER_WER1 | XLLP_PWER_WEF1);
			p_pm_suspend_context->spmu->pwsr = XLLP_PWSR_EDR1 | XLLP_PWSR_EDF1;
		}
	}

	/* sleep --> run */
	if ((to_mode == XLLP_PM_RUN) &&
		(from_mode == XLLP_PM_SLEEP))
	{
		if (src & XLLP_PM_WE_RTC)
		{
			p_pm_suspend_context->spmu->pwer &= ~XLLP_PWER_WERTC;
			p_pm_suspend_context->spmu->pwsr = XLLP_PWSR_EERTC;
		}
		
		if (src & XLLP_PM_WE_EXTERNAL0)
		{
			p_pm_suspend_context->spmu->pwer &= ~(XLLP_PWER_WER0 | XLLP_PWER_WEF0);
			p_pm_suspend_context->spmu->pwsr = XLLP_PWSR_EDR0 | XLLP_PWSR_EDF0;
		}
		
		if (src & XLLP_PM_WE_EXTERNAL1)
		{
			p_pm_suspend_context->spmu->pwer &= ~(XLLP_PWER_WER1 | XLLP_PWER_WEF1);
			p_pm_suspend_context->spmu->pwsr = XLLP_PWSR_EDR1 | XLLP_PWSR_EDF1;
		}

		p_pm_suspend_context->bpmu->ad3er &= ~(src & AD3ER_MASK);
		p_pm_suspend_context->bpmu->ad3sr = src & AD3ER_MASK;											
	}


	/* standby --> run */
	if ((to_mode == XLLP_PM_RUN) &&
		(from_mode == XLLP_PM_STANDBY))
	{
		p_pm_suspend_context->bpmu->ad2d0er &= ~(src & AD2D0ER_MASK);
		p_pm_suspend_context->bpmu->ad2d0sr = src & AD2D0ER_MASK;			
	}	

	/* standby --> lcdrefresh */
	if ((to_mode == XLLP_PM_LCDREFRESH) &&
		(from_mode == XLLP_PM_STANDBY))
	{
		p_pm_suspend_context->bpmu->ad2d1er &= ~(src & AD2D1ER_MASK);
		p_pm_suspend_context->bpmu->ad2d1sr = src & AD2D1ER_MASK;
	}	

	/* lcdrefresh --> run */
	if ((to_mode == XLLP_PM_RUN) &&
		(from_mode == XLLP_PM_LCDREFRESH))
	{
		p_pm_suspend_context->bpmu->ad1d0er &= ~(src & AD1D0ER_MASK);
		p_pm_suspend_context->bpmu->ad1d0sr = src & AD1D0ER_MASK;
	}
}

void XllpPmSelectWakeupSource(
		P_XLLP_PM_SUSPEND_CONTEXT_T   p_pm_suspend_context,
		XLLP_PM_MODE_T to_mode,
		XLLP_PM_MODE_T from_mode, 
		XLLP_PM_WAKEUP_SRC_T src)
{
	/* deepsleep --> run */
	if ((to_mode == XLLP_PM_RUN) &&
		(from_mode == XLLP_PM_DEEPSLEEP))
	{
		if (src & XLLP_PM_WE_RTC)
		{
			p_pm_suspend_context->spmu->pwer |= XLLP_PWER_WERTC;
		}
		
		if (src & XLLP_PM_WE_EXTERNAL0)
		{
			p_pm_suspend_context->spmu->pwer |= (XLLP_PWER_WER0 | XLLP_PWER_WEF0);
		}
		
		if (src & XLLP_PM_WE_EXTERNAL1)
		{
			p_pm_suspend_context->spmu->pwer |= (XLLP_PWER_WER1 | XLLP_PWER_WEF1);
		}
	}

	/* sleep --> run */
	if ((to_mode == XLLP_PM_RUN) &&
		(from_mode == XLLP_PM_SLEEP))
	{
		if (src & XLLP_PM_WE_RTC)
		{
			p_pm_suspend_context->spmu->pwer |= XLLP_PWER_WERTC;
		}
		
		if (src & XLLP_PM_WE_EXTERNAL0)
		{
			p_pm_suspend_context->spmu->pwer |= (XLLP_PWER_WER0 | XLLP_PWER_WEF0);
		}
		
		if (src & XLLP_PM_WE_EXTERNAL1)
		{
			p_pm_suspend_context->spmu->pwer |= (XLLP_PWER_WER1 | XLLP_PWER_WEF1);
		}

		p_pm_suspend_context->bpmu->ad3er |= (src & AD3ER_MASK);
	}


	/* standby --> run */
	if ((to_mode == XLLP_PM_RUN) &&
		(from_mode == XLLP_PM_STANDBY))
	{
		p_pm_suspend_context->bpmu->ad2d0er |= (src & AD2D0ER_MASK);
	}	

	/* standby --> lcdrefresh */
	if ((to_mode == XLLP_PM_LCDREFRESH) &&
		(from_mode == XLLP_PM_STANDBY))
	{
		p_pm_suspend_context->bpmu->ad2d1er |= (src & AD2D1ER_MASK);
	}	

	/* lcdrefresh --> run */
	if ((to_mode == XLLP_PM_RUN) &&
		(from_mode == XLLP_PM_LCDREFRESH))
	{
		p_pm_suspend_context->bpmu->ad1d0er |= (src & AD1D0ER_MASK);
	}
}

XLLP_STATUS_T XllpPmQueryWakeupSource(
		P_XLLP_PM_SUSPEND_CONTEXT_T   p_pm_suspend_context,
		XLLP_PM_MODE_T to_mode,
		XLLP_PM_MODE_T from_mode, 
		XLLP_PM_WAKEUP_SRC_T *psrc)
{
	*psrc = 0;
		
	/* deepsleep --> run */
	if ((to_mode == XLLP_PM_RUN) &&
		(from_mode == XLLP_PM_DEEPSLEEP))
	{
		if (p_pm_suspend_context->spmu->pwsr &= XLLP_PWSR_EERTC)
			*psrc |= XLLP_PM_WE_RTC;
		
		if ((p_pm_suspend_context->spmu->pwsr &= XLLP_PWSR_EDR0) ||
			(p_pm_suspend_context->spmu->pwsr &= XLLP_PWSR_EDF0))
			*psrc |= XLLP_PM_WE_EXTERNAL0;
			
		if ((p_pm_suspend_context->spmu->pwsr &= XLLP_PWSR_EDR1) ||
			(p_pm_suspend_context->spmu->pwsr &= XLLP_PWSR_EDF1))
			*psrc |= XLLP_PM_WE_EXTERNAL1;
	}

	/* sleep --> run */
	if ((to_mode == XLLP_PM_RUN) &&
		(from_mode == XLLP_PM_SLEEP))
	{
		if (p_pm_suspend_context->spmu->pwsr &= XLLP_PWSR_EERTC)
			*psrc |= XLLP_PM_WE_RTC;
		
		if ((p_pm_suspend_context->spmu->pwsr &= XLLP_PWSR_EDR0) ||
			(p_pm_suspend_context->spmu->pwsr &= XLLP_PWSR_EDF0))
			*psrc |= XLLP_PM_WE_EXTERNAL0;
			
		if ((p_pm_suspend_context->spmu->pwsr &= XLLP_PWSR_EDR1) ||
			(p_pm_suspend_context->spmu->pwsr &= XLLP_PWSR_EDF1))
			*psrc |= XLLP_PM_WE_EXTERNAL1;

		*psrc |= (p_pm_suspend_context->bpmu->ad3sr & AD3ER_MASK);
	}


	/* standby --> run */
	if ((to_mode == XLLP_PM_RUN) &&
		(from_mode == XLLP_PM_STANDBY))
	{
		*psrc |= (p_pm_suspend_context->bpmu->ad2d0sr & AD2D0ER_MASK);
	}	

	/* standby --> lcdrefresh */
	if ((to_mode == XLLP_PM_LCDREFRESH) &&
		(from_mode == XLLP_PM_STANDBY))
	{
		*psrc |= (p_pm_suspend_context->bpmu->ad2d1sr & AD2D1ER_MASK);
	}	

	/* lcdrefresh --> run */
	if ((to_mode == XLLP_PM_RUN) &&
		(from_mode == XLLP_PM_LCDREFRESH))
	{
		*psrc |= (p_pm_suspend_context->bpmu->ad1d0sr & AD1D0ER_MASK);
	}

	return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T XllpPmCopyBootCode(
		P_XLLP_PM_SUSPEND_CONTEXT_T   p_pm_suspend_context,
		XLLP_UINT32_T boot_addr)
{
	XLLP_UINT32_T eccErrorPageNum;
	XLLP_STATUS_T status;
		
	status = XllpFlashDfcRead(p_pm_suspend_context->p_dfc_context,
				(P_XLLP_UINT8_T)boot_addr, 
				p_pm_suspend_context->p_dfc_context->pFlashInfo->pageSize,
				0, 32, 
				NAND_CMD_TIMEOUT, &eccErrorPageNum);

	return status;
}


