/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/** 
**  FILENAME:       xllp_pmu.c
**
**	CORE STEPPING:  
**
**  PURPOSE: contains all PMU C function.
**           
**                  
**
*/

#include "xllp_pmu.h"
/**
	Select one event including PMU and PML envent for PMU counter
@par	
	This function selects one event including Manzano and Monahans event. 
When type is Monahans PML Event, it is Monahans PML Event Number OR MONAHANS_EVENT_MASK.  
Other words, when type is Manzano event, bit31 is zero. When type is Monahans PML Event, 
bit31 is one.

@par
	We only use Monahans PML first four event selectors because manzano has only 4 counters 
and every selector can choose all PML events.  We use 1:1 map from PMU counter to PML selector. 
So counter 0 use PML_SEL0, counter1 use PML_SEL1 and so on.

@param	
	counter		PMU Counter Number. It must is between 0 and 3
	type		PMU And PML type 
	base		PML register base virtual address

@return 
	old event type before call this function.

@remarks
	 required kernel/supervisor mode	
*/
XLLP_PMU_EVENT_TYPE XllpPMUEventSelect(int counter, XLLP_PMU_EVENT_TYPE type, P_XLLP_PML_T base)
{
	XLLP_INT32_T oldevent,value,pmuevent,shift;

	if( counter<0||counter>3 )
	{
		return XLLP_PMU_EVENT_INVALIDATE;
	}
	shift=counter*8;

	value=XllpReadPMUReg((XLLP_INT32_T)XLLP_PMU_EVTSEL);
	pmuevent=(value>>shift)&0xFF;

	if( pmuevent>=XLLP_PMU_EVENT_ASSP_0 && pmuevent<=XLLP_PMU_EVENT_ASSP_3 )
	{
		oldevent=XLLP_MONAHANS_EVENT_MASK|(*(base+counter));

	}else
	{
		oldevent=pmuevent;
	}

	if(type&XLLP_MONAHANS_EVENT_MASK)
	{//PML Event
		value&=~(0xFF<<shift);
		value|=(XLLP_PMU_EVENT_ASSP_0+counter)<<shift;
		*(base+counter)=type&(~XLLP_MONAHANS_EVENT_MASK);
	}else
	{//PMU Event
		value&=~(0xFF<<shift);
		value|=(type&0xFF)<<shift;
	}
	XllpWritePMUReg((XLLP_INT32_T)XLLP_PMU_EVTSEL, value);
	
	return oldevent;
}

