/*
 * dlfile.c: command wrapper for xmodem/uuencode download to
 *			 arbitary memory loc
 *
 * Copyright (C) 2002 Stefan Eletzhofer <stefan.eletzhofer@eletztrick.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: dlfile.c,v 1.3 2003/04/03 14:40:55 seletz Exp $"

/**********************************************************************
 * Includes
 */

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/main.h>
#include <blob/errno.h>
#include <blob/debug.h>
#include <blob/types.h>
#include <blob/serial.h>
#include <blob/util.h>
#include <blob/uucodec.h>
#include <blob/xmodem.h>
#include <blob/md5.h>
#include <blob/md5support.h>


/**********************************************************************
 * defines
 */

/* this will send a cold shiver through erik's spine ... */
#define	ERR( x )		{ ret = x; goto DONE; }

extern blob_status_t blob_status;

/*********************************************************************
 * dlfile
 *
 * AUTOR:	Stefan Eletzhofer
 * REVISED:
 *
 * download a file to a memory location. Does also print a md5
 * checksum of the downloaded data.
 *
 */
int dlfile_cmd( int argc, char *argv[] )
{
	int ret = 0, len;
	u32 address;//, len;
	u32 digest[4];

	if ( argc < 3 )
		ERR( -EINVAL );

	ret = strtou32(argv[1], &address);
	if ( ret < 0 )
		ERR( -EINVAL );

	ret = strtou32(argv[2], (u32 *)&len);
	if ( ret < 0 )
		ERR( -EINVAL );

	dprintf("adr=0x%08x, len=0x%08x\n", address, len );

#if defined(CONFIG_XMODEM_SUPPORT)
	printf( "XMODEM download\n" );
#else
	printf( "UUENCODE download\n" );
#endif

	/* switch terminal speed if necessary */
	if (blob_status.terminalSpeed != blob_status.downloadSpeed) {
		SerialOutputString( "Switch to download speed, please.\n" );
		SerialOutputString("You have 60 seconds to start downloading.\n");
	
		serial_init(blob_status.downloadSpeed);
	} else {
		SerialOutputString("You have 60 seconds to start downloading.\n");
	}

	/* download the data */
#if defined(CONFIG_XMODEM_SUPPORT)
	ret = XModemReceive( (char *)address, len );
#else
	ret = UUDecode((char *)address, len);
#endif

	if ( ret == len ) {
		SerialOutputString("Received ");
		SerialOutputDec(ret);
		SerialOutputString(" (0x");
		SerialOutputHex(ret);
		SerialOutputString(") bytes.\n");
		ret = 0;
	} else {
		SerialOutputString("error during download\n");
	}


	if (blob_status.terminalSpeed != blob_status.downloadSpeed) {
		SerialOutputString("\n(Please switch your terminal emulator back to terminal speed\n");
		serial_init(blob_status.terminalSpeed);
	}

	/* do a md5 sum of the downloaded data */
	md5_buffer( (const char *)address, len, digest );
	SerialOutputString("md5 checksum: ");
	print_md5_digest(digest);
	serial_write( '\n' );


	ret = 0;
DONE:
	return ret;
}

char dlfile_help[] = "dlfile destination_address len\n\ndownload a file to memory\n";
