/*
 * ferase.c: command wrapper for erasing arbitary flash
 *			regions
 *
 * Copyright (C) 2002 Stefan Eletzhofer <stefan.eletzhofer@eletztrick.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: ferase.c,v 1.3 2003/04/03 14:40:55 seletz Exp $"

/**********************************************************************
 * Includes
 */

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/errno.h>
#include <blob/debug.h>
#include <blob/types.h>
#include <blob/serial.h>
#include <blob/util.h>
#include <blob/flash.h>


/**********************************************************************
 * defines
 */

/* this will send a cold shiver through erik's spine ... */
#define	ERR( x )		{ ret = x; goto DONE; }


/*********************************************************************
 * ferase_cmd
 *
 * AUTOR:	SELETZ
 * REVISED:
 *
 * Command wrapper for low-level flash erase
 *
 */
int ferase_cmd( int argc, char *argv[] )
{
	int ret = 0;
	u32	dest	= 0L;
	u32	len	= 0L;

	if ( argc < 3 )				ERR( -EINVAL );

	ret = strtou32( argv[1], &dest );
	if ( ret < 0 )				ERR( -EINVAL );

	ret = strtou32( argv[2], &len );
	if ( ret < 0 )				ERR( -EINVAL );

	if ( len & (0x3) ) {
		len = (len>>2)  + 1;
	} else {
		len = len>>2;
	}

	dprintf( "start=0x%08x, len=0x%08x words\n", dest, len );

	ret = flash_erase_region( (u32 *)dest, len );
DONE:
	return ret;
}
char ferase_help[] = "ferase destadr size(bytes)\n"
"flash a memory region\n";
