/*
 * memcpy.c: copy memory command
 *
 * Copyright (C) 2001 Stefan Eletzhofer <stefan.eletzhofer@eletztrick.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: memcpy.c,v 1.3 2003/04/03 14:40:57 seletz Exp $"

/**********************************************************************
 * Includes
 */

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/errno.h>
#include <blob/debug.h>
#include <blob/types.h>
#include <blob/util.h>


/**********************************************************************
 * defines
 */

/* this will send a cold shiver through erik's spine ... */
#define	ERR( x )		{ ret = x; goto DONE; }


/**********************************************************************
 * prototypes
 */

/*********************************************************************
 * CmdMemcpy
 *
 * AUTOR:		SELETZ
 * REVISED:
 *
 * Command wrapper for memcpy utility function.
 *
 */
int memcpy_cmd( int argc, char *argv[] )
{
	int ret = 0;
	u32 src, dest, len;

	if ( argc < 4 )
		ERR( -EINVAL );

	ret = strtou32( argv[1], &src );
	if ( ret < 0 )
		ERR( -EINVAL );

	ret = strtou32( argv[2], &dest );
	if ( ret < 0 )
		ERR( -EINVAL );

	ret = strtou32( argv[3], &len );
	if ( ret < 0 )
		ERR( -EINVAL );

	/* check alignment of src and dest */
	if((src & 0x03) || (dest & 0x03))
		ERR(-EALIGN);

	/* counted in words */
	if ( len & 0x00000003 ) {
		len = ( len >> 2 ) + 1;
	} else {
		len = len >> 2;
	}

	dprintf("src=0x%08x, dest=0x%08x, %d words\n", src, dest, len );

	MyMemCpy( (u32 *)dest, (const u32 *)src, len);

	printf("%d words copied.\n", len );

	ret = 0;

DONE:
	return ret;
}

char memcpy_help[] = "memcpy src dst len\n"
"copy len bytes from src to dst\n";
