/*
 * reboot.c: Reboot board
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * $Id: reboot.c,v 1.3 2003/02/11 00:02:48 erikm Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * aint with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: reboot.c,v 1.3 2003/02/11 00:02:48 erikm Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/serial.h>
#include <blob/reboot.h>
#include <blob/util.h>




int reboot_cmd(int __attribute__((unused)) argc,
		char __attribute__((unused)) *argv[])
{
typedef enum {
        PU_REASON_HARDWARE_RESET,
        PU_REASON_WATCHDOG,
        PU_REASON_RTC_ALARM,
        PU_REASON_PWR_KEY_PRESS,
        PU_REASON_USB_CABLE,
        PU_REASON_USB_FACTORY,
        PU_REASON_USB_CHARGER,
        PU_REASON_REBOOT_NORMAL,
        PU_REASON_REBOOT_FACTORY,
        PU_REASON_REBOOT_RECOVERY,
        PU_REASON_REBOOT_FOTA,
        PU_REASON_REBOOT_ASSERT,
        PU_REASON_LOW_PWR_RESET, 
        PU_REASON_MAX
} powerup_reason_t;
typedef enum {
        REBOOT_NORMAL,
        REBOOT_FACTORY,
        REBOOT_FLASH,
        REBOOT_RECOVERY,
        REBOOT_FOTA,
        REBOOT_ASSERT,
        REBOOT_DKB=0xfd,
        REBOOT_MAX
} reboot_setting_t;


	printf("Rebooting...\n\n");
/*
    if (!strncmp("hd_reset", argv[1], 8)){
		*(unsigned int *)0x40f5008 = PU_REASON_HARDWARE_RESET;
	    printf("Rebooting hd_reset\n");
    }
	else if (!strncmp("watchdog", argv[1], 8)){
		*(unsigned int *)0x40f5008 = PU_REASON_WATCHDOG;
	    printf("Rebooting watchdog\n");
    }
	else if (!strncmp("rtc_alarm", argv[1], 9)){
		*(unsigned int *)0x40f5008 = PU_REASON_RTC_ALARM;
	    printf("Rebooting rtc_alarm\n");
    }
    else if (!strncmp("pwr_key", argv[1], 7)){
		*(unsigned int *)0x40f5008 = PU_REASON_PWR_KEY_PRESS;
	    printf("Rebooting pwr_key\n");
    }
	else if (!strncmp("usb_cable", argv[1], 9)){
		*(unsigned int *)0x40f5008 = PU_REASON_USB_CABLE;
	    printf("Rebooting usb_cable\n");
    }
    else if (!strncmp("usb_factory", argv[1], 8)){
		*(unsigned int *)0x40f5008 = PU_REASON_USB_FACTORY;
	    printf("Rebooting usb_factory\n");
    }
    else if (!strncmp("usb_charger", argv[1], 8)){
		*(unsigned int *)0x40f5008 = PU_REASON_USB_CHARGER;
	    printf("Rebooting usb_charger\n");
    }
    else */
    if (!strncmp("normal", argv[1], 6)){
		*(unsigned int *)0x40f50008 = REBOOT_NORMAL;
//        printf("ARSR = 0x%x\n", *(unsigned int *)0x40F40004);
        *(unsigned int *)0x40F40004 = (*(unsigned int *)0x40F40004)|0x8;
//        printf("ARSR or = 0x%x\n", *(unsigned int *)0x40F40004);
	    printf("Rebooting normal\n");
    }
	else if (!strncmp("factory", argv[1], 7)){
		*(unsigned int *)0x40f50008 = REBOOT_FACTORY;
        *(unsigned int *)0x40F40004 = (*(unsigned int *)0x40F40004)|0x8;
	    printf("Rebooting factory\n");
    }
    else if (!strncmp("recovery", argv[1], 8)){
		*(unsigned int *)0x40f50008 = REBOOT_RECOVERY;
        *(unsigned int *)0x40F40004 = (*(unsigned int *)0x40F40004)|0x8;
	    printf("Rebooting recovery\n");
    }
	else if (!strncmp("fota", argv[1], 4)){
		*(unsigned int *)0x40f50008 = REBOOT_FOTA;
        *(unsigned int *)0x40F40004 = (*(unsigned int *)0x40F40004)|0x8;
	    printf("Rebooting fota\n");
    }
    else if (!strncmp("assert", argv[1], 7)){
		*(unsigned int *)0x40f50008 = REBOOT_ASSERT;
        *(unsigned int *)0x40F40004 = (*(unsigned int *)0x40F40004)|0x8;
	    printf("Rebooting assert\n");
    }
    else if (!strncmp("flash", argv[1], 7)){
		*(unsigned int *)0x40f50008 = REBOOT_FLASH;
        *(unsigned int *)0x40F40004 = (*(unsigned int *)0x40F40004)|0x8;
	    printf("Rebooting assert\n");
    }/*
	else if (!strncmp("low_pwr", argv[1], 7)){
		*(unsigned int *)0x40f5008 = PU_REASON_LOW_PWR_RESET;
	    printf("Rebooting low_pwr\n");
    }*/
	else {
	}

	serial_flush_output();
	
	reboot_system();

	/* never reached, but anyway... */
	return 0;
}

char reboot_help[] = "reboot\n"
"Reboot system\nhd_reset\nwatchdog\nrtc_alarm\npwr_key\nusb_cable\nusb_factory\nusb_charger\nnormal\nfactory\nrecovery\nfota\nassert\nflash\nlow_pwr\n";
