/*
 * tftp.c: tftp download command
 *
 * Copyright (C) 2003 Stefan Eletzhofer <stefan.eletzhofer@eletztrick.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: tftp.c,v 1.5 2003/09/04 01:10:23 russd Exp $"

/**********************************************************************
 * Includes
 */

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>
#include <blob/errno.h>
#include <blob/debug.h>
#include <blob/types.h>
#include <blob/util.h>

#include <net/net.h>


#if !defined( TFTP_RAM_START ) 
# warning "Please define TFTP_RAM_START for your arch!, tftp command inoperable"
#define TFTP_RAM_START 0xdeadbeef
#endif

/**********************************************************************
 * defines
 */
#undef TFTP_DEBUG
#ifdef TFTP_DEBUG
static int dbg = 1;
#	define DBG( x, args... )	if ( dbg>=x ) printf( args )
#else
#	define DBG( x, args... )
#endif

/*********************************************************************
 * usage
 */
char tftp_help[] = "tftp command.\n"
"usage: tftp filename [--kernel | --ramdisk]\n"
"\tdownloads file over TFTP to RAM.\n"
"\tUse setip to set client and server IP addresses\n";

/**********************************************************************
 * command
 */
int tftp_cmd(int argc, char *argv[])
{
	int ret = 0;
	extern int do_tftp(char *file, unsigned long addr, unsigned long *size);
	unsigned long addr = TFTP_RAM_START;
	unsigned long size = 0;
	char *tftp_file = "update.tar";

	if (TFTP_RAM_START == 0xdeadbeef) return -EINVAL;

	if ( argc>1 ) {
		tftp_file = argv[1];

		/* options */
		if ( argc>2 ) {
			if ( !strncmp("--kernel", argv[2], 8) )
				addr = KERNEL_RAM_BASE;
			else if ( !strncmp("--ramdisk", argv[2], 9) )
				addr = RAMDISK_RAM_BASE;
			else {
				printf("TFTP: unknown option '%s'", argv[2]);
			}
		}
	} else {
		printf("%s", tftp_help);
		return 0;
	}

	ret = do_tftp( tftp_file, addr , &size );
	if ( ret ) {
		return -EINVAL;
	}

	printf( "%s: file '%s' loaded via tftp to address 0x%08lx.\n",
			__FUNCTION__, tftp_file, addr );
	return 0;
}


