/*-------------------------------------------------------------------------
 * Filename:      xdownload.c
 * Version:       $Id: xdownload.c,v 1.1 2003/09/04 17:47:00 russd Exp $
 * Copyright:     Copyright (C) 1999, Jan-Derk Bakker
 * Author:        Jan-Derk Bakker <J.D.Bakker@its.tudelft.nl>
 * Description:   xmodem download command
 * Created at:    Mon Aug 23 20:00:00 1999
 * Modified by:   Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
 * Modified at:   Sat Mar 25 14:31:16 2000
 *-----------------------------------------------------------------------*/
/*
 * xmodem.c: xmodem download command
 *
 * Copyright (C) 1999 2000 2001
 *   Jan-Derk Bakker (J.D.Bakker@its.tudelft.nl) and
 *   Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: xdownload.c,v 1.1 2003/09/04 17:47:00 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/download.h>
#include <blob/command.h>
#include <blob/errno.h>
#include <blob/error.h>
#include <blob/md5.h>
#include <blob/md5support.h>
#include <blob/serial.h>
#include <blob/xmodem.h>
#include <blob/partition.h>
#include <blob/time.h>


int xdownload_cmd(int argc, char *argv[])
{
	u32 startAddress;
	int bufLen;
	int *numRead;
	int retval;
	u32 *digest;

	if(argc < 2)
		return -ENOPARAMS;

	retval = set_download_parameters(argv[1], &startAddress,
					 &bufLen, &numRead, &digest);
	if (retval < 0)
		return 0;

	if (blob_status.terminalSpeed != blob_status.downloadSpeed) {
		printf("Switching to ");
		PrintSerialSpeed(blob_status.downloadSpeed);
		printf(" baud\n");

		printf("Switch your terminal emulator to the same speed and\n");
		printf("start sending using the XMODEM protocol (repeated ^X) to quit.\n");
		printf(PACKAGE " will switch back to ");
		PrintSerialSpeed(blob_status.terminalSpeed);
		printf(" baud after XMODEM succeeds or times out.\n");
	
		serial_init(blob_status.downloadSpeed);
	} else {
		printf("Start sending using the XMODEM protocol (repeated ^X to quit).\n\n");
	}

	*numRead = XModemReceive((char *) startAddress, bufLen);

	/* There is a race condition somewhere...  We need this delay */
	/* else the report is not printed out. TomW 4/30/2002 */
	msleep(100);

	printf("\n\n");
	serial_flush_output();
	
	if (blob_status.terminalSpeed != blob_status.downloadSpeed) {
		printf("\n(Please switch your terminal emulator back to ");
		PrintSerialSpeed(blob_status.terminalSpeed);
		printf(" baud)\n");
	}
	
	if(*numRead < 0) {
		/* something went wrong */
		retval = *numRead;
		
		/* reload the correct memory */
		do_reload(argv[1]);

		serial_init(blob_status.terminalSpeed);
		return retval;
	}

	do_md5sum((u32 *)startAddress, *numRead, digest);

	printf("Received %i (0x%08x) bytes", *numRead, *numRead);
#ifdef BLOB_DEBUG
//	printf("at 0x%08x", (unsigned int)startAddress);
#ifdef CONFIG_MD5_SUPPORT
//	printf(", MD5: ");
//	print_md5_digest(digest);
#endif
#endif
	printf("\n");


	serial_init(blob_status.terminalSpeed);

	return 0;
}

char xdownload_help[] = "xdownload {blob|param|kernel|ramdisk}\n"
"Download <argument> image to RAM using xmodem\n";

