/*
 * i2c-gpio.c: i2c on a pair of GPIO pins
 *
 * Copyright (C) 2001 Hewlett-Packard Comapny
 * Written by Christopher Hoover <ch@hpl.hp.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: i2c-gpio.c,v 1.4 2003/08/06 22:55:41 abz Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/i2c.h>
#include <blob/i2c-gpio.h>
#include <blob/arch.h>
#include <blob/errno.h>
#include <blob/util.h>

#ifdef DEBUG
#define DPRINTF(...) do { printf(__VA_ARGS__); } while (0)
#else
#define DPRINTF(...) do { } while (0)
#endif

////////////////////////////////////////////////////////////////////////

static inline int set_scl(struct i2c_bus *bus, unsigned sense) {
	return (bus->set_scl)(bus, sense);
}
static inline int set_sda(struct i2c_bus *bus, unsigned sense) {
	return (bus->set_sda)(bus, sense);
}
static inline int get_scl(struct i2c_bus *bus) {
	return (bus->get_scl)(bus);
}
static inline int get_sda(struct i2c_bus *bus) {
	return (bus->get_sda)(bus);
}

////////////////////////////////////////////////////////////////////////

static inline void delay(const struct i2c_bus *bus) 
{
	struct i2c_bus_gpio_private *private =
		(struct i2c_bus_gpio_private *) bus->private;

	/* ### use msleep?  no, it may not be around early enough */
	volatile int i;
	for (i = 0; i < private->delay; i++)
		;
}

int init_bus_gpio(struct i2c_bus *bus)
{
	struct i2c_bus_gpio_private *private =
		(struct i2c_bus_gpio_private *) bus->private;

	GAFR &= ~(private->sda_gpio | private->scl_gpio);
	GPDR &= ~(private->sda_gpio | private->scl_gpio);
	GPCR = (private->sda_gpio | private->scl_gpio);

	set_sda_gpio(bus, 1);
	set_scl_gpio(bus, 1);

	delay(bus);
	delay(bus);

	return 0;
}

int set_sda_gpio(struct i2c_bus *bus, unsigned sense)
{
	struct i2c_bus_gpio_private *private =
		(struct i2c_bus_gpio_private *) bus->private;

	if (sense)
		/* float high */
		GPDR &= ~(private->sda_gpio);
	else
		/* pull low */
		GPDR |= (private->sda_gpio);

	delay(bus);

	return 0;
}

int get_sda_gpio(struct i2c_bus *bus)
{
	struct i2c_bus_gpio_private *private =
		(struct i2c_bus_gpio_private *) bus->private;

	return (GPLR & (private->sda_gpio)) ? 1 : 0;
}

int set_scl_gpio(struct i2c_bus *bus, unsigned sense)
{
	struct i2c_bus_gpio_private *private =
		(struct i2c_bus_gpio_private *) bus->private;

	if (sense) {
		int i = 0;

		/* float high */
		GPDR &= ~(private->scl_gpio);

		while (!get_scl(bus)) {
			if (i++ > 10) {
				DPRINTF(__FUNCTION__ ": scl did not go high\n");
				return -ETIMEOUT;
			}
			delay(bus);
		}
	} else
		/* pull low */
		GPDR |= (private->scl_gpio);

	delay(bus);

	return 0;
}

int get_scl_gpio(struct i2c_bus *bus)
{
	struct i2c_bus_gpio_private *private =
		(struct i2c_bus_gpio_private *) bus->private;

	return (GPLR & (private->scl_gpio)) ? 1 : 0;
}

