/*-------------------------------------------------------------------------
 * Filename:      time-s3c2500.c
 * Version:       $Id: time-s3c2500.c,v 1.1 2003/09/04 18:03:10 russd Exp $
 * Copyright:     Copyright (C) 1999, Erik Mouw
 * Author:        Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
 * Description:   Timer for the s3c2500
 * Created at:    Tue Aug 24 21:08:25 1999
 * Modified by:   Russ Dill <Russ.Dill@asu.edu>
 * Modified at:   Sat Aug 23 17:50:00 2003
 *-----------------------------------------------------------------------*/
/*
 * time-s3c2500.c: Timer functions for blob
 *
 * Copyright (C) 1999 2000 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: time-s3c2500.c,v 1.1 2003/09/04 18:03:10 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/init.h>
#include <blob/led.h>
#include <blob/arch.h>
#include <blob/time.h>
#include <blob/util.h>

#ifndef BUS_SPEED
#  error You must define BUS_SPEED in your arch file
#endif

/* at a 133MHz bus speed, this will only count up to 32.2s, so
 * do a weird counter thing. As long as its checked every 32.2s
 * it should work fine */


static u32 timer;
static u32 last_val;

static void s3c2500_timer_init(void)
{
	TMOD = 0;
	barrier();
	TDATA0 = 0xFFFFFFFFL;
	barrier();
	TMOD = TIMER_TE0;
	last_val = timer = 0;
}

/* returns the time in 1/TICKS_PER_SECOND seconds */
static u32 s3c2500_timer_read(void)
{
	u32 val = -TCNT0;
	if (last_val >= val) timer++;
	last_val = val;
	return (timer << 24) | (val >> 8);
}

/* not sure what these overflow functions are supposed to do
 * but I can't tell if there is an overflow */
static int s3c2500_timer_overflow(void)
{
	return 0;
}

static void s3c2500_timer_overflow_clear(void)
{
}

/* export time driver */
timer_driver_t s3c2500_timer_driver = {
	init:		s3c2500_timer_init,
	read:		s3c2500_timer_read,
	overflow:	s3c2500_timer_overflow,
	overflow_clear:	s3c2500_timer_overflow_clear,
	tps:		(GET_MHz(BUS_SPEED) * 1000000) >> 8,
};
