/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2000, 2001 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:       usb_descriptors.c
**
**  PURPOSE:   		This files sets up USB descriptors from TIM or uses default.
**
**  LAST MODIFIED: $Modtime: 4/04/06 4:26p $
**
******************************************************************************/

#include "general.h"
#include "misc.h"
#include "errors.h"
#include "tim.h"
#include "xllp_defs.h"
#include "xllp_u2d.h"
#include "xllp_udc.h"
#include "usb_descriptors.txt"

#include "protocolmanager.h"
#include "usb_descriptors.h"

OPT_USB_DESCRIPTOR_LOADS USBDescriptors[NUM_USB_DESCRIPTORS];
UINT_T USBStringIndexTable[MAX_USB_STRINGS];
static pTIM  pTIMUSB_h = NULL;			// For Reference later by the USB driver for Host Device Requests in TIM

void CheckAndFillStringIndexTable (UINT_T index)
{
 pUSBDeviceDescriptor pUSBDevDesc;
 pUSBConfigDescriptor pUSBConfigDesc;

	// Is this a Device Descriptor? If so save off string index's
	if ((DESCRIPTOR_RES_ID | index) == USB_DEVICE_DESCRIPTOR)
	{
		pUSBDevDesc = (pUSBDeviceDescriptor)USBDescriptors[index].pDesPayload;
		if ((pUSBDevDesc->iManufacturer < MAX_USB_STRINGS) && (pUSBDevDesc->iManufacturer != 0))
			USBStringIndexTable[pUSBDevDesc->iManufacturer] = USB_MANUFACTURER_STRING_DESCRIPTOR;
		if ((pUSBDevDesc->iProduct < MAX_USB_STRINGS) && (pUSBDevDesc->iProduct != 0))
			USBStringIndexTable[pUSBDevDesc->iProduct] = USB_PRODUCT_STRING_DESCRIPTOR;
		if ((pUSBDevDesc->iSerialNumber < MAX_USB_STRINGS) && (pUSBDevDesc->iSerialNumber != 0))
			USBStringIndexTable[pUSBDevDesc->iSerialNumber] = USB_SERIAL_STRING_DESCRIPTOR;
	}
	// Is this a Config Descriptor? If so save off string index's
	if ((DESCRIPTOR_RES_ID | index) == USB_CONFIG_DESCRIPTOR)
	{
		pUSBConfigDesc = (pUSBConfigDescriptor)USBDescriptors[index].pDesPayload;
		if ((pUSBConfigDesc->InterDesc.iInterface <	MAX_USB_STRINGS) && (pUSBConfigDesc->InterDesc.iInterface != 0))
			USBStringIndexTable[pUSBConfigDesc->InterDesc.iInterface] = USB_INTERFACE_STRING_DESCRIPTOR;
	}
	return;
}

/* This routine fills out an array of descriptors from either the reserved area or by defaults *
* Format is in the form of:
*
*/
UINT_T SetUpUSBDescriptors (pTIM pTIM_h, UINT_T TIM_Use, UINT_T PortType)
{
 UINT_T Retval = NoError;
 pWTP_RESERVED_AREA_HEADER pWRAH = NULL;
 UINT_T 	i;

 // Save for later
 pTIMUSB_h = pTIM_h;

 // Check for USB Device Descriptor
 for (i=0; i<NUM_USB_DESCRIPTORS; i++)
 {
	if (TIM_Use)
	{
 		pWRAH = FindPackageInReserved (&Retval, pTIM_h, (DESCRIPTOR_RES_ID | i));
 		if (Retval == NoError)
		{
			USBDescriptors[i].DesSize = pWRAH->Size - sizeof (WTP_RESERVED_AREA_HEADER);
			USBDescriptors[i].pDesPayload = (UINT_T *)(pWRAH + 1);	// Go past the header
			CheckAndFillStringIndexTable(i);
		}
	}
	if ((!TIM_Use) || (Retval != NoError))
	{
	 	switch (DESCRIPTOR_RES_ID | i)
		{
		 case USB_DEVICE_DESCRIPTOR:
			 if(PortType == U2D_USB_D)
			 {
				USBDescriptors[i].DesSize = sizeof (U2DDeviceDesc);
				USBDescriptors[i].pDesPayload = (UINT_T *) U2DDeviceDesc;
			 	CheckAndFillStringIndexTable(i);
			 }
			 else
			 {
		 		USBDescriptors[i].DesSize = sizeof (UsbDeviceDesc);
				USBDescriptors[i].pDesPayload = (UINT_T *) UsbDeviceDesc;
				CheckAndFillStringIndexTable(i);
			 }
		 	break;
		 case USB_CONFIG_DESCRIPTOR:
			 if(PortType == U2D_USB_D)
			 {
		 		USBDescriptors[i].DesSize = sizeof (U2DConfigDesc);
				USBDescriptors[i].pDesPayload = (UINT_T *) U2DConfigDesc;
				CheckAndFillStringIndexTable(i);
			 }
			 else
			 {
		 		USBDescriptors[i].DesSize = sizeof (UsbConfigDesc);
				USBDescriptors[i].pDesPayload = (UINT_T *) UsbConfigDesc;
			 	CheckAndFillStringIndexTable(i);
			 }
		 	break;
		 case USB_INTERFACE_DESCRIPTOR:
		 	break;
		 case USB_LANGUAGE_STRING_DESCRIPTOR:
		 	USBDescriptors[i].DesSize = sizeof (UsbStringDesc_LangID);
			USBDescriptors[i].pDesPayload = (UINT_T *) UsbStringDesc_LangID;
		 	USBStringIndexTable[0] = USB_LANGUAGE_STRING_DESCRIPTOR;
		 	break;
		 case USB_MANUFACTURER_STRING_DESCRIPTOR:
		 	USBDescriptors[i].DesSize = sizeof (UsbStringDesc_ManufacturerStr);
			USBDescriptors[i].pDesPayload = (UINT_T *) UsbStringDesc_ManufacturerStr;
		 	break;
		 case USB_PRODUCT_STRING_DESCRIPTOR:
		 	USBDescriptors[i].DesSize = sizeof (UsbStringDesc_ProductStr);
			USBDescriptors[i].pDesPayload = (UINT_T *) UsbStringDesc_ProductStr;
		 	break;
		 case USB_SERIAL_STRING_DESCRIPTOR:
		 	USBDescriptors[i].DesSize = sizeof (UsbStringDesc_SerialNumStr);
			USBDescriptors[i].pDesPayload = (UINT_T *) UsbStringDesc_SerialNumStr;
		 	break;
		 case USB_INTERFACE_STRING_DESCRIPTOR:
		 	USBDescriptors[i].DesSize = sizeof (UsbStringDesc_InterfaceStr);
			USBDescriptors[i].pDesPayload = (UINT_T *) UsbStringDesc_InterfaceStr;
		 	break;
		 case USB_DEFAULT_STRING_DESCRIPTOR:
		 	USBDescriptors[i].DesSize = sizeof (UsbStringDesc_DefaultStr);
			USBDescriptors[i].pDesPayload = (UINT_T *) UsbStringDesc_DefaultStr;
			USBStringIndexTable[5] = USB_DEFAULT_STRING_DESCRIPTOR;			// Index 5 is the default id.
		 	break;
		 case USB_ENDPOINT_DESCRIPTOR:
		 	break;
		}
	 }
  }
  return Retval;
}

/* This routine will search the TIM and find the Vendor specific requested package.
   If Retval == NoError then success. Failure otherwise.
*/
UINT_T FindVendorRequestInTIM (UINT8 bRequest, UINT8 bmRequestType, PUINT32 pTxBuffEp0, PINT32 ppacketLength)
{
    pWTP_RESERVED_AREA_HEADER pWRAH;
	pUSB_VENDOR_REQ pUSBVREQ;
	UINT_T Retval;

	// Any USB Vendor Request packages?
	pWRAH = FindFirstPackageTypeInReserved (&Retval, pTIMUSB_h, USBVENDORREQ);
	if (Retval != NoError)
		return Retval;
	do
	{
		pUSBVREQ = (pUSB_VENDOR_REQ) pWRAH;
		if ((pUSBVREQ->bRequest == bRequest) && (pUSBVREQ->bmRequestType == bmRequestType))
		{
			*ppacketLength = pUSBVREQ->wLength;
			(*pTxBuffEp0) = (UINT_T) &pUSBVREQ->wData;
			return Retval;
		}
		pWRAH = FindNextPackageTypeInReserved(&Retval);
	}
	while (Retval == NoError);
	return Retval;
}
