/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	UsbDownload.c
**
**  PURPOSE: 	Main USB download program.  Handles all control aspects of
** 				USB download
**
******************************************************************************/

#include "usbdownload.h"
#include "misc.h"
// UDC Controller Registers' Base
#define XLLP_UDC_REGISTERS_BASE		(0x40600000)
#define UdcEndpoint2FIFOAddr 	0x40600308
#define UdcEndpoint1FIFOAddr 	0x40600304
#define MaxEndpointPacketSize	64       // Max size in bytes
#define borqs_package_lost_max  0x800

extern void EnableIrqInterrupts(void);
extern void DisableIrqInterrupts(void);

static XLLP_DMAC_COMMAND_T UdcDmaRxCommand;
static XLLP_DMAC_CHANNEL_T UdcDmaRxChannel   = USB_RX_CHANNEL;

XLLP_UDC_T	DeviceControl;
volatile P_XLLP_UDC_T   UsbDeviceController = &DeviceControl;
volatile P_XLLP_UDC_T  	pUdcHandle;
volatile P_XLLP_UDC_REGISTERS_T pRegs;
static volatile int borqs_CurrentDownloadAddress;
static volatile int borqs_package_lost_counts = 0;
extern int borqs_package_lost;
const XLLP_UDC_EP_T TransmitEndpointNum = UDC_EP_1;
const XLLP_UDC_EP_T ReceiveEndpointNum  = UDC_EP_2;

void UdcInit()
{
	UdcDmaRxChannel = USB_RX_CHANNEL;

	UdcDmaRxCommand.aSize		= XLLP_DMAC_BURSTSIZE_32;
	UdcDmaRxCommand.aWidth      = XLLP_DMAC_WIDTH_32;
	UdcDmaRxCommand.aEndian     = 0;
	UdcDmaRxCommand.aFlyByT     = 0;
	UdcDmaRxCommand.aFlyByS     = 0;
	UdcDmaRxCommand.aEndIrqEn   = 1;
	UdcDmaRxCommand.aStartIrqEn = 0;
	UdcDmaRxCommand.aAddrMode   = 0;
	UdcDmaRxCommand.aCmpEn      = 0;
	UdcDmaRxCommand.aFlowTrg    = 0;
	UdcDmaRxCommand.aFlowSrc    = 1;
	UdcDmaRxCommand.aIncTrgAddr = 1;
	UdcDmaRxCommand.aIncSrcAddr = 0;

	// Initialize the handles
	pUdcHandle = UsbDeviceController;
	// Initialize the USB context structure
	XllpUdcSWInit (pUdcHandle, (P_XLLP_UDC_REGISTERS_T)XLLP_UDC_REGISTERS_BASE);
	pRegs      = (P_XLLP_UDC_REGISTERS_T)pUdcHandle->pRegsBase;

	EnablePeripheralIRQInterrupt(USB_CLIENT_INT);
	// Enable IRQ interrupt
	//EnableIrqInterrupts();

	// Setup UDC
	//BootRomUdcHWSetup();
    XllpUdcHWSetup ((P_XLLP_UDC_T)UsbDeviceController);
    pUdcHandle->enumerateComplete = XLLP_FALSE;

	//Disable transmit endpoint's interrupts
	XllpUdcDisableInterrupt (pUdcHandle,(XLLP_UDC_INTERRUPTS_T)TransmitEndpointNum,PACKET_COMPL_INT);
	XllpUdcDisableInterrupt (pUdcHandle,(XLLP_UDC_INTERRUPTS_T)TransmitEndpointNum,FIFO_ERROR_INT);
}
/**
 * UDC DMA interrupts entry point.
 **/
void UdcDmaHandler(void)
{
	UINT_T Regval;

	XllpDmacStopTransfer(UdcDmaRxChannel);

	//if there's data left, download it
	if(getProtocolISR()->DataLeft > 0)
	{
		UdcReceiveDmaUpdate();
	}
	//else shutdown DMA client
	else
	{
		// Disable the STOP IRQ Enable interrupt
		XllpDmacDisableStopIrqInterrupt(UdcDmaRxChannel);
		// Unmap device from DMA channel
		XllpDmacUnMapDeviceToChannel(XLLP_DMAC_USB_ENDPOINT_B, UdcDmaRxChannel);
		// Clear all DMA related interrupts
		XllpDmacClearDmaInterrupt(UdcDmaRxChannel);
		DisableUdcDmaUsage();
		DisablePeripheralIRQInterrupt(DMA_CNTL_INT);

		// Clear the UDC interrupts and turn off DMA
		Regval = pRegs->UDCCSR[ReceiveEndpointNum];
		Regval |= (XLLP_UDC_UDCCSR_PC | XLLP_UDC_UDCCSR_SP);
		Regval &= ~XLLP_UDC_UDCCSR_DME;
		pRegs->UDCCSR[ReceiveEndpointNum] = Regval;

		pRegs->UDCISR0 |= (XLLP_UDC_UDCISR0_IRB_0 | XLLP_UDC_UDCISR0_IRB_1);

		//we just finished receiving Data Cmd.  Now turn off interrupt
		XllpUdcDisableInterrupt (pUdcHandle,(XLLP_UDC_INTERRUPTS_T)ReceiveEndpointNum,PACKET_COMPL_INT);
		XllpUdcDisableInterrupt (pUdcHandle,(XLLP_UDC_INTERRUPTS_T)ReceiveEndpointNum,FIFO_ERROR_INT);

		getProtocolISR()->CommandReceived = TRUE;
		ClearPortInterruptFlag();
        borqs_package_lost_counts = 0;
	}
}

/**
 * Top level USB1.1 Interrupt handler
 **/
void  UdcInterruptHandler(void)
{
	//Call to clear out any Endpoint 0 interrupts
	UdcEndpoint0InterruptHandler();

	if(pUdcHandle->enumerateComplete) {

		// do not allow downloads if the fuse bit is set, only
		if(getProtocolISR()->DownloadsEnabled == 0)
		{
			// Disable first level UDC interrupt
			DisablePeripheralIRQInterrupt(USB_CLIENT_INT);

			pRegs->UDCICR0 &= ~XLLP_UDC_UDCICR0_ENABLE_ALL;
			pRegs->UDCICR1 &= ~(XLLP_UDC_UDCICR1_EVENTS | XLLP_UDC_UDCICR1_ENABLE_ALL);

			pRegs->UDCISR0 = 0xFFFFFFFF;
			pRegs->UDCISR1 = 0xFFFFFFFF;

		}
		else
		{

			//if we have an interrupt on Endpoint B (which is our receive endpoint)
			if((pRegs->UDCISR0 & XLLP_UDC_UDCISR0_IRB_0) == XLLP_UDC_UDCISR0_IRB_0)
			{
				UdcReceiveInterruptHandler();
			}
			//if we have any transmit interrupts, clear them
			if((pRegs->UDCISR0 & XLLP_UDC_UDCISR0_IRA_0) == XLLP_UDC_UDCISR0_IRA_0)
			{
				pRegs->UDCCSR[TransmitEndpointNum] |= XLLP_UDC_UDCCSR_PC;
				pRegs->UDCISR0 |= (XLLP_UDC_UDCISR0_IRA_0 | XLLP_UDC_UDCISR0_IRA_1);
			}
		}
	}
}

void UdcTransmit(UINT_T len, UINT8_T* SrcAddr)
{
	//load packet into Transmit FIFO
	XllpUdcWriteDwordFifo(&pRegs->UDCDR[TransmitEndpointNum], SrcAddr, len);
	//set Short Packet bit to send packet
	pRegs->UDCCSR[TransmitEndpointNum] |= XLLP_UDC_UDCCSR_SP;

	//turn Received endpoint interrupts back on
	XllpUdcEnableInterrupt (pUdcHandle,(XLLP_UDC_INTERRUPTS_T)ReceiveEndpointNum,PACKET_COMPL_INT);
	XllpUdcEnableInterrupt (pUdcHandle,(XLLP_UDC_INTERRUPTS_T)ReceiveEndpointNum,FIFO_ERROR_INT);
}
/*
 *	This function updates the DMA descriptor and starts the transfer
 */
void UdcReceiveDmaUpdate()
{
	pProtocolISR pISR = getProtocolISR();
	UINT_T PacketLength,Regval;
	static __attribute__((aligned(16)))XLLP_DMAC_DESCRIPTOR_T UdcDmaDescArray[2];
	P_XLLP_DMAC_DESCRIPTOR_T UdcDmaDesc[3];

	UdcDmaDesc[0] = &UdcDmaDescArray[0];
	UdcDmaDesc[1] = &UdcDmaDescArray[1];

	PacketLength = pRegs->UDCBCR[ReceiveEndpointNum] & XLLP_UDC_UDCBCR_BC_MASK;
#if 1
    if(PacketLength == 0)
    {
        if(++borqs_package_lost_counts > borqs_package_lost_max)
        {
            borqs_package_lost = 0x1000;
            borqs_package_lost_counts = 0;
            // Disable the STOP IRQ Enable interrupt
            XllpDmacDisableStopIrqInterrupt(UdcDmaRxChannel);
            // Unmap device from DMA channel
            XllpDmacUnMapDeviceToChannel(XLLP_DMAC_USB_ENDPOINT_B, UdcDmaRxChannel);
            // Clear all DMA related interrupts
            XllpDmacClearDmaInterrupt(UdcDmaRxChannel);
            DisableUdcDmaUsage();
            DisablePeripheralIRQInterrupt(DMA_CNTL_INT);

            // Clear the UDC interrupts and turn off DMA
            Regval = pRegs->UDCCSR[ReceiveEndpointNum];
            Regval |= (XLLP_UDC_UDCCSR_PC | XLLP_UDC_UDCCSR_SP);
            Regval &= ~XLLP_UDC_UDCCSR_DME;
            pRegs->UDCCSR[ReceiveEndpointNum] = Regval;

            pRegs->UDCISR0 |= (XLLP_UDC_UDCISR0_IRB_0 | XLLP_UDC_UDCISR0_IRB_1);

            //we just finished receiving Data Cmd.  Now turn off interrupt
            XllpUdcDisableInterrupt (pUdcHandle,(XLLP_UDC_INTERRUPTS_T)ReceiveEndpointNum,PACKET_COMPL_INT);
            XllpUdcDisableInterrupt (pUdcHandle,(XLLP_UDC_INTERRUPTS_T)ReceiveEndpointNum,FIFO_ERROR_INT);

//          getProtocolISR()->CommandReceived = TRUE;
            ClearPortInterruptFlag();
            return;
        }
    }else borqs_package_lost_counts = 0;
#endif

	pISR->DataLeft			-= PacketLength;
	UdcDmaRxCommand.aLen	= PacketLength;

	XllpDmacFillLinkedDesc(UdcDmaDesc[0],
			UdcDmaDesc[1],
			XLLP_DMAC_DESC_STOP_CHANNEL,
			XLLP_DMAC_DISABLE_DESC_BRANCH,
			(XLLP_UINT32_T)UdcEndpoint2FIFOAddr,
			(XLLP_UINT32_T)pISR->CurrentDownloadAddress,
			&UdcDmaRxCommand);

	XllpDmacCfgChannelDescTransfer(UdcDmaDesc[0],
		UdcDmaRxChannel,
		XLLP_DMAC_USB_ENDPOINT_B,
		XLLP_DMAC_ALIGNMENT_ON);

	// Start DMA transfer
	XllpDmacStartTransfer(UdcDmaRxChannel);

	pISR->CurrentDownloadAddress += PacketLength;
}

void UdcReceiveInterruptHandler()
{
	UINT_T Retval = NoError;
	UINT_T Tmp    = 0;
	UINT8_T Buff[20];
	UINT_T PacketLength = 0;
	pProtocolISR pISR = getProtocolISR();
	pProtocolCmd pCmd = getProtocolCmd();

	//first, Disable endpoint's interrupts
	// turn off ICR - Interrupt Control Register
	XllpUdcDisableInterrupt (pUdcHandle,(XLLP_UDC_INTERRUPTS_T)ReceiveEndpointNum,PACKET_COMPL_INT);

	// Disable endpoint's FIFO error interrupt
	XllpUdcDisableInterrupt (pUdcHandle,(XLLP_UDC_INTERRUPTS_T)ReceiveEndpointNum,FIFO_ERROR_INT);


	// Read the UDCCSRx register
	Tmp = pRegs->UDCCSR[ReceiveEndpointNum];
	if (((Tmp & XLLP_UDC_UDCCSR_PC) == XLLP_UDC_UDCCSR_PC))
	{
		PacketLength = pRegs->UDCBCR[ReceiveEndpointNum] & XLLP_UDC_UDCBCR_BC_MASK;
		// Read Receive Endpoint's FIFO
		if(PacketLength == PREAMBLE_SIZE) {
			XllpUdcReadDwordFifo(&pRegs->UDCDR[ReceiveEndpointNum], &Buff[0], PREAMBLE_SIZE);
            if( ProtocolVerifyPreamble( Buff ) == TRUE )
			{
				pISR->CommandReceived = FALSE;
				pISR->PreambleReceived = TRUE;
				pISR->InterruptPort = DIFF_USB_D;
				#if !VERBOSE_DEBUG  //leave this on for dumping messages
#ifndef FBW_CODE
				ShutdownPort(FFUART_D);
#endif
				#endif
                ShutdownPort(U2D_USB_D);

				ClearPortInterruptFlag();
				SendResponse(Buff, PREAMBLE_SIZE);
			}
		} else if(PacketLength >= 8){
			//read constant part of command packet
			XllpUdcReadDwordFifo(&pRegs->UDCDR[ReceiveEndpointNum], (void *)&(pCmd->Command), 8);
			PacketLength -= 8;
			//update Interrupt Status Flags
			pISR->CurrentCommand = pCmd->Command;
			pISR->DataLeft = pCmd->Length;
			pISR->CommandReceived = TRUE;

			switch(pISR->CurrentCommand)
			{
			case MessageCmd:
				// Clear the interrupt bits, or UDC will not like it!!!
				//-----------------------------------------------------
				pRegs->UDCCSR[ReceiveEndpointNum] |= XLLP_UDC_UDCCSR_PC;
				pRegs->UDCISR0 |= (XLLP_UDC_UDCISR0_IRB_0 | XLLP_UDC_UDCISR0_IRB_1);
				//try and send a message
				Retval = HandleMessageCmd();
				return;
			case DownloadDataCmd:
                pISR->CommandReceived = FALSE;
                borqs_CurrentDownloadAddress = pISR->CurrentDownloadAddress;
				// Enable UDC DMA usage in DMA handler
				EnableUdcDmaUsage();
				EnablePeripheralIRQInterrupt(DMA_CNTL_INT);

				//read rest of data out of FIFO
				XllpUdcReadDwordFifo(&pRegs->UDCDR[ReceiveEndpointNum], (UINT32_T*)pISR->CurrentDownloadAddress, PacketLength);
				pRegs->UDCCSR[ReceiveEndpointNum] |= XLLP_UDC_UDCCSR_PC;

                // These instructions will allow time for setting the "PC" bit to
                // change the USB 1.1 peripheral registers ...
                //---------------------------------------------------------------
				pISR->CurrentDownloadAddress += PacketLength;
				pISR->DataLeft -= PacketLength;

                // This get's the next FIFO buffer for USB 1.1 peripheral
                //-------------------------------------------------------
				PacketLength = pRegs->UDCBCR[ReceiveEndpointNum] & XLLP_UDC_UDCBCR_BC_MASK;
				XllpUdcReadDwordFifo(&pRegs->UDCDR[ReceiveEndpointNum], (UINT32_T*)pISR->CurrentDownloadAddress, PacketLength);

				// Enable UDC DMA usage
				pRegs->UDCCSR[ReceiveEndpointNum] |= XLLP_UDC_UDCCSR_DME;

				pISR->CurrentDownloadAddress += PacketLength;
				pISR->DataLeft -= PacketLength;

				pRegs->UDCISR0 |= (XLLP_UDC_UDCISR0_IRB_0 | XLLP_UDC_UDCISR0_IRB_1);
				// Enable the STOP IRQ Enable interrupt
				XllpDmacEnableStopIrqInterrupt(UdcDmaRxChannel);
				UdcReceiveDmaUpdate();
                borqs_package_lost_counts = 0;
                borqs_package_lost = 0;
				return;
			case DisconnectCmd:
				pISR->PreambleReceived = TRUE;
			default:
				// rest of messages: should not have over 4 bytes of data
				// this will need to change if protocol changes
				if(PacketLength > 4)
					PacketLength = 4;

				XllpUdcReadDwordFifo(&pRegs->UDCDR[ReceiveEndpointNum],(void *) &(pCmd->Data[0]), PacketLength);

				//clear rest of received FIFO, incase there are extra bytes in FIFO
				pRegs->UDCCSR[ReceiveEndpointNum] |= XLLP_UDC_UDCCSR_FEF;
				break;
			}
		} else if (PacketLength > 0){
			// read out extra data.  We should never reach this statement
            //clear rest of received FIFO, incase there are extra bytes in FIFO
            pRegs->UDCCSR[ReceiveEndpointNum] |= XLLP_UDC_UDCCSR_FEF;
		}

		// Clear the interrupt bits, or UDC will not like it!!!
		//-----------------------------------------------------
		pRegs->UDCCSR[ReceiveEndpointNum] |= XLLP_UDC_UDCCSR_PC;
		pRegs->UDCISR0 |= (XLLP_UDC_UDCISR0_IRB_0 | XLLP_UDC_UDCISR0_IRB_1);

	} else //check to see if we have error bits.  if so, clear them
		if (((Tmp & XLLP_UDC_UDCCSR_EFE) == XLLP_UDC_UDCCSR_EFE)){
			pRegs->UDCCSR[ReceiveEndpointNum] &= (XLLP_UDC_UDCCSR_PC | XLLP_UDC_UDCCSR_SP);
		}
}

void UdcHWShutdown(void)
{
	// Shutdown the USB device
	BootRomUdcHWShutdown();
}

void borqs_reset_data_transfer(void)
{
    UINT_T Regval;
    pProtocolISR pISR = getProtocolISR();

	XllpDmacStopTransfer(UdcDmaRxChannel);

        // Disable the STOP IRQ Enable interrupt
    XllpDmacDisableStopIrqInterrupt(UdcDmaRxChannel);
    // Unmap device from DMA channel
    XllpDmacUnMapDeviceToChannel(XLLP_DMAC_USB_ENDPOINT_B, UdcDmaRxChannel);
    // Clear all DMA related interrupts
    XllpDmacClearDmaInterrupt(UdcDmaRxChannel);
    DisableUdcDmaUsage();
    DisablePeripheralIRQInterrupt(DMA_CNTL_INT);

    // Clear the UDC interrupts and turn off DMA
    Regval = pRegs->UDCCSR[ReceiveEndpointNum];
    Regval |= (XLLP_UDC_UDCCSR_PC | XLLP_UDC_UDCCSR_SP);
    Regval &= ~XLLP_UDC_UDCCSR_DME;
    pRegs->UDCCSR[ReceiveEndpointNum] = Regval;

    pRegs->UDCISR0 |= (XLLP_UDC_UDCISR0_IRB_0 | XLLP_UDC_UDCISR0_IRB_1);

    //we just finished receiving Data Cmd.  Now turn off interrupt
    XllpUdcDisableInterrupt (pUdcHandle,(XLLP_UDC_INTERRUPTS_T)ReceiveEndpointNum,PACKET_COMPL_INT);
    XllpUdcDisableInterrupt (pUdcHandle,(XLLP_UDC_INTERRUPTS_T)ReceiveEndpointNum,FIFO_ERROR_INT);

    getProtocolISR()->CommandReceived = FALSE;
    ClearPortInterruptFlag();

//    pISR->DataLeft = 0;
    pISR->CurrentDownloadAddress = borqs_CurrentDownloadAddress;
}
