#include "FlexOneNAND_Cmd.h"
#include "OneNAND_Driver.h"
#include "errors.h"

UINT_T FlexOneNAND_HotReset(void);
extern UINT_T CheckReadDisturb(UINT8 *mainBuffer, UINT_T noSectors, UINT16* ECCResultsMain);
extern void memset(void *Addr, unsigned char Val, unsigned long Size);

extern P_FlexOneNAND_Properties_T GetFlexOneNANDProperties(void);
extern P_FLEX_REGISTERS_T pFlexRegs;

UINT_T LoadFlexPage(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt)
{
	UINT_T Retval = NoError;
	UINT_T BlockNum, BlockOffsetMask, BlockByteOffset, StartAddr, SectorAddr, FlashPageAddr;
	UINT_T ER[8];
	P_FlashProperties_T pFlashProp = GetFlashProperties(fbt);
	P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();
	VUINT8_T *pDataRAM, *pBuff;
	int i;
	UINT_T EccFlag = ECC_ON_FLAG; //Turn ECC flag OFF for now. 
	UINT_T noSectors = 4;
	UINT8  mainBuffer[64];
	UINT16_T device_select = 0;

	//Setup Pointers	
	pDataRAM = (VUINT8_T *) FLEX_DATARAM_MAIN_BASE;
	pBuff = (VUINT8_T *) buffer;
	
	// Block size of 4096 = 2^12
	BlockNum = FlashOffset >> pFlexProp->BlockSizeDivShift;
	BlockOffsetMask  = (pFlashProp->BlockSize) - 1;
	BlockByteOffset = FlashOffset & BlockOffsetMask;

    // Compute the byte offset into the first sector(BlockByteOffset)
    //---------------------------------------------
    StartAddr = (BlockNum * pFlashProp->BlockSize) + BlockByteOffset;

	SectorAddr = (StartAddr/SECTOR_SIZE_IN_BYTES)%4;
	FlashPageAddr = StartAddr/FLEX_PAGE_SIZE_IN_BYTES; 
	
	if (FlashOffset >> 10)
		device_select = FLEX_SA2R_DBS_FLAG;

    pFlexRegs->SA1 = (device_select) | (BlockNum & MUX_SA1R_FBA_MASK);

	pFlexRegs->SA2 = device_select;


	pFlexRegs->SA8 = (FlashPageAddr << 2) | SectorAddr;
	pFlexRegs->SBR = (UINT16_T) (SBR_WRITE_DEFAULT) | (SectorAddr & MUX_SBR_BSC_MASK);	

			
	pFlexRegs->INTRPT = INT_STAT_REG_CLEAR;
	pFlexRegs->COMMAND =ONE_NAND_LOAD_SECTOR_INTO_BUFFER;

    if( WaitForCommandCompletion(PEND_READ, (UINT32)ONENAND_TIMEOUT_MS )!= NoError)
    {
       return ReadError;
    }

	// Read ECC Status Registers
	ER[0] = pFlexRegs->ECCSR1 & FLEX_ECCSR_ER_MASK;
	ER[1] = ( pFlexRegs->ECCSR1 >> 8 ) & FLEX_ECCSR_ER_MASK;

	ER[2] = pFlexRegs->ECCSR2 & FLEX_ECCSR_ER_MASK;
	ER[3] = ( pFlexRegs->ECCSR2 >> 8 ) & FLEX_ECCSR_ER_MASK;

	ER[4] = pFlexRegs->ECCSR3 & FLEX_ECCSR_ER_MASK;
	ER[5] = ( pFlexRegs->ECCSR3 >> 8 ) & FLEX_ECCSR_ER_MASK;

	ER[6] = pFlexRegs->ECCSR4 & FLEX_ECCSR_ER_MASK;
	ER[7] = ( pFlexRegs->ECCSR4 >> 8 ) & FLEX_ECCSR_ER_MASK;

	// Host reads data from DataRAM
	for (i=0;i<size;i+=1)
	{
		*pBuff++ = *pDataRAM++; 
	}

	if ( EccFlag & ECC_ON_FLAG)
		pFlexRegs->SYSCONF1  &= CONF1_ECC_ON;
	else  
		pFlexRegs->SYSCONF1  |= CONF1_ECC_OFF;

	if ( EccFlag & ECC_ON_FLAG)
	{
		memset( mainBuffer, (UINT8)~VALID_BLK_MARK, sizeof(mainBuffer));
		if (CheckReadDisturb(mainBuffer, noSectors, (UINT16 *)ER) != NoError)
			Retval = ReadDisturbError;
	}
	return Retval;
}

UINT_T SuperLoadFlexPage()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T LSBPageRecoveryRead()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T PIUpdate()
{
	UINT_T Retval = NoError;
	return Retval;
}

/*
 * ProgramFlexPage:  
 *	This function uses the Program command to program
 *	one full page of data into the OneNAND part
 *
 * Inputs:
 *	address - location in flash to burn data
 *	buffer - pointer to location of data to be written
 *	size - number of bytes to be written from buffer
 *			
 *		Note: address + size CANNOT span multiple pages
 *			This MUST be handled in upper level driver
 *
 * Outputs:
 *	Return value - returns NoError on success
 *		Failures can be:
 *			Flash is Locked
 *			Timeout Error
 *			Program Error
 */
UINT_T ProgramFlexPage(UINT_T address, UINT_T buffer, UINT_T size, FlashBootType_T fbt)
{
	UINT_T Retval = NoError;
	UINT_T BlockNum, PageNum, PageMask, ByteMask, pre_bytes, post_bytes;
	VUINT8_T *pDataRAM, *pBuff;
	UINT16_T device_select = 0;
	P_FlashProperties_T pFlashProp = GetFlashProperties(fbt);
	P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();

	//Initial Calculations:
	//Determine the block we are on
	BlockNum = address >> pFlexProp->BlockSizeDivShift;
	//Determine the page we are on
	PageMask = (pFlashProp->BlockSize - pFlashProp->PageSize);
	PageNum	= (address & PageMask) >> pFlexProp->PageSizeDivShift;
	//Determine pre-bytes for this page
	ByteMask = (pFlashProp->PageSize - 1);
	pre_bytes = (address & ByteMask);
	//Determine left over bytes for this page
	post_bytes = pFlashProp->PageSize - size - pre_bytes;

	//Setup Pointers	
	pDataRAM = (VUINT8_T *) FLEX_DATARAM_MAIN_BASE;
	pBuff = (VUINT8_T *) buffer;

	//Step 1:
	//Set DBS bit in Start Address 2 register
//	pFlexRegs->SA2 = pFlexProp->MuxType << 15;
	if (address >> 10)
		device_select = FLEX_SA2R_DBS_FLAG;

	pFlexRegs->SA2 = device_select;
			   
	//Step 2: Load data into BufferRAM
	//Fill any initial values with 1's
	for( ; pre_bytes > 0 ; pre_bytes -= 1 )
		*pDataRAM++ = 0xFF;
	
	//Write the data 16 bits at a time
	for( ; size > 0 ; size -= 1 )
		*pDataRAM++ = *pBuff++;

	//If there is any leftover space in the page, fill it with 1's
	for( ; post_bytes > 0 ; post_bytes -= 1 )
		*pDataRAM++ = 0xFF;

	//Step 3:
	//Fill out Start Address 1 Register 
	pFlexRegs->SA1 = (device_select) | (BlockNum & FLEX_SA1R_FBA_MASK);

	//Step 4:
	//Read Write Protection Status
	if (pFlexRegs->WPS & FLEX_WPSR_UNLOCKED != FLEX_WPSR_UNLOCKED)
		return FlashLockError;

	//Step 5:
	//Write Page and Sector Addres (must be 0) to Start Address 8
	pFlexRegs->SA8 = (PageNum << 2)| 0x0;

	//Step 6:
	//Fill out Start Buffer Register info (must be 0x0800 for Program)
	pFlexRegs->SBR = (UINT16_T) SBR_WRITE_DEFAULT;

	//Step 7:
	//Set System Config Register - leave at default 
	//uncomment next line to turn of ECC	
   //	pFlexRegs->SYSCONF1 = CONF1_WRITE_DEFAULT | CONF1_ECC_OFF; 

	//Step 8:
	//Clear Interrupt Register
	pFlexRegs->INTRPT = INT_STAT_REG_CLEAR;
	
	//Step 9:
	//Write PROGRAM Cmd to command register
	pFlexRegs->COMMAND = FLEX_PROGRAM_PAGE_FROM_BUFFER;

	//Step 10:
	//Wait for WRITE interrupt
	Retval = WaitForCommandCompletion(PEND_WRITE, (UINT32)ONENAND_TIMEOUT_MS );
    if (Retval != NoError)
		return Retval;

	//Step 11:
	//Read Controller status register
	if ((pFlexRegs->CNTSTAT & FLEX_CSR_ERROR) == FLEX_CSR_ERROR)
		Retval = ProgramError;

	return Retval;
}


UINT_T FinishCacheProgram()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T CacheProgram()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T UnlockFlexArray()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T LockFlexArray()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T LockTightFlexArray()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T UnlockAllFlexBlocks()
{
	UINT_T Retval = NoError;
	P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();

	//Step 1: Write DFS to Start Address 1 Register
	pFlexRegs->SA1 = pFlexProp->MuxType << 15;

	//Step 2: Set DBS for Start Address 2 Register
	pFlexRegs->SA2 = pFlexProp->MuxType << 15;

	//Step 3: Write SBA to SBA Register
	pFlexRegs->SBA = 0;

	//Step 4: Clear Interrupt Register
	pFlexRegs->INTRPT = INT_STAT_REG_CLEAR;
	
	//Step 5: Write all block unlock command
	pFlexRegs->COMMAND = FLEX_ALL_BLOCK_UNLOCK;

	//Step 6: Wait for Interrupt
	Retval = WaitForCommandCompletion(INT_STAT_REG, (UINT32)ONENAND_TIMEOUT_MS*3 );
    if (Retval != NoError)
		return Retval;

	//Step 7: Read Controller Status Register
	if ((pFlexRegs->CNTSTAT & FLEX_CSR_ERROR) == FLEX_CSR_ERROR)
		Retval = GeneralError;
		
	return Retval;
}

//------------------------------------------
// FlexBlockErase()
//------------------------------------------
UINT_T FlexBlockErase( UINT32 blockNumber )
{
	UINT_T Retval;
    FLEX_SA1_REG sa1reg;
    
    //First unlock Blocks
    //-------------------
    UnlockAllFlexBlocks();
    
    // setup the FBA in SA1
    // bits0-9 = blockNumber
    // bit 15 = blocks #'s above 1023
    //-------------------------------
    if( blockNumber > 1023 )
    {
      sa1reg.bits.FLEX_DFS = 1;
      sa1reg.bits.FLEX_RESERVED = 0;
      sa1reg.bits.FLEX_FBA = blockNumber-1024;
    }
    else
    {
      sa1reg.bits.FLEX_DFS = 0;
      sa1reg.bits.FLEX_RESERVED = 0;
      sa1reg.bits.FLEX_FBA = blockNumber;   
    }
      
    //Write FBA & DFS
    //----------------------------
    pFlexRegs->SA1 = sa1reg.value;

	//Step 4:
	//Read Write Protection Status
	if (pFlexRegs->WPS & FLEX_WPSR_UNLOCKED != FLEX_WPSR_UNLOCKED)
		return FlashLockError;
    
    //set DBS in SA2
    //----------------
    if(blockNumber > 1023)
      pFlexRegs->SA2 = ( 1 << 15);
    else
      pFlexRegs->SA2 = 0;
    
	//Clear Interrupt Register
    //ensure write complete
    //------------------------
	pFlexRegs->INTRPT = INT_STAT_REG_CLEAR;
    sa1reg.value = pFlexRegs->INTRPT;
    
    //Write the command
    pFlexRegs->COMMAND = FLEX_BLOCK_ERASE;
    
    //Wait for operation complete
    //---------------------------
	Retval = WaitForCommandCompletion(PEND_ERASE, (UINT32)ONENAND_TIMEOUT_MS*3 );
    if (Retval != NoError)
		return EraseError;

	//Read Controller Status Register
	if ((pFlexRegs->CNTSTAT & FLEX_CSR_ERROR) == FLEX_CSR_ERROR)
		Retval = EraseError;
        
	return Retval;
}

UINT_T FlexEraseSuspend()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T FlexEraseResume()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T ResetFlexNANDCore()
{
	UINT_T Retval = NoError;
	UINT16 IntStatReg=0;
	int count=0;

	do{
		 // This will return if there's no FlexOneNAND device connected.
		 if(count > INT_CLEAR_RETRY_CNT)
		 {
		   return GeneralError;
		 }
		// clear interrupt register
		pFlexRegs->INTRPT = INT_STAT_REG_CLEAR;
		IntStatReg = pFlexRegs->INTRPT;
		count++;
	// This checks for interrupt status register bit 15(INT)
	}while (IntStatReg & INT_STAT_REG);	
	 
	pFlexRegs->COMMAND = FLEX_NAND_CORE_RESET;

	if( WaitForCommandCompletion(INT_STAT_REG_RSTI_PEND, (UINT32)RESET_TIMEOUT_MS )!= NoError)
	{
	   return GeneralError;
	}

	return Retval;
}

UINT_T ResetFlexOneNAND()
{
	UINT_T Retval = NoError;
	UINT16 IntStatReg=0;
	int count=0;

	do{
		 // This will return if there's no FlexOneNAND device connected.
		 if(count > INT_CLEAR_RETRY_CNT)
		 {
		   return GeneralError;
		 }
		// clear interrupt register
		pFlexRegs->INTRPT = INT_STAT_REG_CLEAR;
		IntStatReg = pFlexRegs->INTRPT;
		count++;
	// This checks for interrupt status register bit 15(INT)
	}while (IntStatReg & INT_STAT_REG);	
	 
	pFlexRegs->COMMAND = FLEX_MUX_ONENAND_RESET;

	if( WaitForCommandCompletion(INT_STAT_REG_RSTI_PEND, (UINT32)RESET_TIMEOUT_MS )!= NoError)
	{
	   return GeneralError;
	}

	return Retval;

}
