/******************************************************************************
**	(C)Copyright Marvell. All Rights Reserved. 2007 ?
**
**  THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MARVELL.
**	The copyright notice above does not evidence any actual or intended  
**  publication of such source code.	
**
**  This Module contains Proprietary Information of Marvell and should be treated 
**  as Confidential.
**	  
**  The information in this file is provided for the exclusive use of the licensees 
**  of Marvell.	
**	Such users have the right to use, modify, and incorporate this code into products 
**  for purposes authorized by the license agreement provided they include this notice 
**  and the associated copyright notice with any such product. 
**  The information in this file is provided "AS IS" without warranty.
**
**  FILENAME: OneNAND_Cmd.c
**
**  PURPOSE:  The driver for the One NAND device.
**
******************************************************************************/
#include "OneNAND_Cmd.h"
#include "OneNAND_Driver.h"
#include "errors.h"

extern P_FlexOneNAND_Properties_T GetFlexOneNANDProperties(void);
UINT_T CheckReadDisturb(UINT8 *mainBuffer, UINT_T noSectors, UINT16* ECCResultsMain);
extern void memset(void *Addr, unsigned char Val, unsigned long Size) ;
extern P_ONE_NAND_REGISTERS_T pOneNANDRegs;

#define ARRAY_SIZE(arr) (sizeof(arr) / sizeof((arr)[0]))
struct ecc_layout {
	int	offset;
	int	length;
};

/* In Onenand document and Linux kernel driver. The offset entry
 * 1 is not avaliable. But to keep compatible with native nand
 * bbt, add it in blob. -- yfw
 */
struct ecc_layout onenand_ecc_layout[] = {
	{0, 2},	/* this entry is for mark bad information */
	{2, 3},
	{14, 2},
	{18, 3},
	{30, 2},
	{34, 3},
	{46, 2},
	{50, 3},
	{62, 2},
};
static int convert_oob_after_read(unsigned char *oobbuf, unsigned char *buf)
{
	int i;
	unsigned char *tmp = buf;

	for (i = 0; i < (int)ARRAY_SIZE(onenand_ecc_layout); i++) {
		memcpy(tmp, oobbuf + onenand_ecc_layout[i].offset,
			onenand_ecc_layout[i].length);
		tmp += onenand_ecc_layout[i].length;
	}

	return i;
}

static int convert_oob_before_write(unsigned char *oobbuf, unsigned char *buf)
{
	int i, count = 0;
	unsigned char *tmp = buf;

	for (i = 0; i < ARRAY_SIZE(onenand_ecc_layout); i++) {
		memcpy(oobbuf + onenand_ecc_layout[i].offset,
			tmp, onenand_ecc_layout[i].length);
		tmp += onenand_ecc_layout[i].length;
		count += onenand_ecc_layout[i].length;
	}

	return count;
}
/*
 * LoadSector:  
 *	This function uses the Load command to retrive
 *	up to one full page of data into the OneNAND part
 *
 * Inputs:
 *	address - location in flash to burn data
 *	buffer - pointer to location of data to be written
 *	size - number of bytes to be written from buffer
 *			
 *		Note: address + size CANNOT span multiple pages
 *			This MUST be handled in upper level driver
 *
 * Outputs:
 *	Return value - returns NoError on success
 *		Failures can be:
 *			Timeout Error
 *			Read Error
 */
UINT_T LoadSector(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt, char *oob)
{
	UINT_T Retval = NoError;
	UINT_T BlockNum, BlockOffsetMask, BlockByteOffset, StartAddr, SectorAddr, FlashPageAddr;
	UINT_T BufferSectorCnt, BufferRAMSectorAddr;
	P_FlashProperties_T pFlashProp = GetFlashProperties(fbt);
	P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();
	VUINT8_T *pDataRAM, *pBuff;
	int i;
	UINT_T EccFlag = ECC_ON_FLAG; 
	UINT_T noSectors = 4;
	UINT8  mainBuffer[64];
	UINT16  ECCResultsMain[ECC_BLOCK_SIZE];
	UINT16_T device_select = 0;

	//Setup Pointers	
	pDataRAM = (VUINT8_T *) FLEX_DATARAM_MAIN_BASE;
	pBuff = (VUINT8_T *) buffer;
	
	// Block size of 4096 = 2^12
	BlockNum = FlashOffset >> pFlexProp->BlockSizeDivShift;
	BlockOffsetMask  = (pFlashProp->BlockSize) - 1;
	BlockByteOffset = FlashOffset & BlockOffsetMask;

    // Compute the byte offset into the first sector(BlockByteOffset)
    //---------------------------------------------
    StartAddr = (BlockNum * pFlashProp->BlockSize) + BlockByteOffset;

	SectorAddr = (StartAddr/SECTOR_SIZE_IN_BYTES)%4;
	FlashPageAddr = StartAddr/PAGE_SIZE_IN_BYTES; 
	
	if (FlashOffset >> 11)
		device_select = FLEX_SA2R_DBS_FLAG;

    pOneNANDRegs->SA1 = (device_select) | (BlockNum & MUX_SA1R_FBA_MASK);

	pOneNANDRegs->SA2 = device_select;


	pOneNANDRegs->SA8 = (FlashPageAddr << 2) | SectorAddr;
	pOneNANDRegs->SBR = (UINT16_T) (SBR_WRITE_DEFAULT) | (SectorAddr & MUX_SBR_BSC_MASK);	

			
	pOneNANDRegs->INTRPT = INT_STAT_REG_CLEAR;
	pOneNANDRegs->COMMAND =ONE_NAND_LOAD_SECTOR_INTO_BUFFER;

    if( WaitForCommandCompletion(PEND_READ, (UINT32)ONENAND_TIMEOUT_MS )!= NoError)
    {
       return ReadError;
    }

	for (i=0;i<size;i+=1)
	{
		*pBuff++ = *pDataRAM++; 
	}
    
    if(oob)convert_oob_after_read((unsigned char *)FLEX_DATARAM_SPARE_BASE, oob);

	if ( EccFlag & ECC_ON_FLAG)
		pOneNANDRegs->SYSCONF1  &= CONF1_ECC_ON;
	else  
		pOneNANDRegs->SYSCONF1  |= CONF1_ECC_OFF;

	if ( EccFlag & ECC_ON_FLAG)
	{
		memset( mainBuffer, (UINT8)~VALID_BLK_MARK, sizeof(mainBuffer));
		if (CheckReadDisturb(mainBuffer, noSectors, ECCResultsMain) != NoError)
			Retval = ReadDisturbError;
	}
	return Retval;

}

UINT_T LoadSpareSector()
{
	UINT_T Retval = NoError;
	return Retval;
}

/*
 * ProgramSector:  
 *	This function uses the Program command to program
 *	up to one full page of data into the OneNAND part
 *
 * Inputs:
 *	address - location in flash to burn data
 *	buffer - pointer to location of data to be written
 *	size - number of bytes to be written from buffer
 *			
 *		Note: address + size CANNOT span multiple pages
 *			This MUST be handled in upper level driver
 *
 * Outputs:
 *	Return value - returns NoError on success
 *		Failures can be:
 *			Flash is Locked
 *			Timeout Error
 *			Program Error
 */
extern volatile UINT8_T is_yaffs;
UINT_T ProgramSector(UINT_T address, UINT_T buffer, UINT_T size, FlashBootType_T fbt)
{
	UINT_T Retval = NoError;
	UINT_T BlockNum, PageNum, SectorCount, PageMask, ByteMask, SectorMask, pre_bytes, post_bytes,BufferRAMSectorAddr,SectorAddr;
	VUINT8_T *pDataRAM, *pBuff;
	UINT16_T device_select = 0;
	P_FlashProperties_T pFlashProp = GetFlashProperties(fbt);
	P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();
    UINT32 yaff_buf[10];
	//Initial Calculations:
	PageMask = (pFlashProp->BlockSize - pFlashProp->PageSize);
	SectorMask = pFlashProp->PageSize - (pFlashProp->PageSize >> 2);
	ByteMask = (pFlashProp->PageSize >> 2)- 1;
	
	//Determine the block we are on
	BlockNum = address >> pFlexProp->BlockSizeDivShift;
	//Determine the page we are on
	PageNum	= (address & PageMask) >> pFlexProp->PageSizeDivShift;
	
    #if 0 //this code has bug,drop it temply
    //Determine pre-bytes for this sector
	pre_bytes = address & ByteMask;
	//Determine left over bytes (fill out to end of last sector)
	post_bytes = (pFlashProp->PageSize - size - pre_bytes) & ByteMask;
	//Determine how many sectors we are programming (total size / sector size)
	SectorCount = (pre_bytes + size + post_bytes) >> (pFlexProp->PageSizeDivShift - 2);
    #endif
    pre_bytes = 0;
    post_bytes = pFlashProp->PageSize - size;
    SectorCount = 4;
	//Setup Pointers	
	pDataRAM = (VUINT8_T *) FLEX_DATARAM_MAIN_BASE;
	pBuff = (VUINT8_T *) buffer;

	//Step 1:
	//Set DBS bit in Start Address 2 register
	if (address >> 11)
		device_select = FLEX_SA2R_DBS_FLAG;

	pOneNANDRegs->SA2 = device_select;
	//pOneNANDRegs->SA2 = 0; //make sure DBS bit is clear

	//Step 2: Load data into BufferRAM
	//Fill any initial values with 1's
	#if 0
    for( ; pre_bytes > 0 ; pre_bytes -= 1 )
    {
        borqs_printf1("p");
		*pDataRAM++ = 0xFF;
	}
    #endif
	//Write the data 16 bits at a time
	for( ; size > 0 ; size -= 1 )
		*pDataRAM++ = *pBuff++;

	//If there is any leftover space in the page, fill it with 1's
	for( ; post_bytes > 0 ; post_bytes -= 1 )
		*pDataRAM++ = 0xFF;
    
    if(is_yaffs)
    {char *src,*dst;
     int i;
        src = (char*)(buffer+pFlashProp->PageSize);
        dst = (char*)(&yaff_buf[0]);
        *dst++ = 0xff;*dst++ = 0xff;
        for(i = 0;i < 38;i++)*dst++ = *src++;
        convert_oob_before_write((char*)FLEX_DATARAM_SPARE_BASE, yaff_buf);
    }
	//Step 3:
	//Fill out Start Address 1 Register 
	pOneNANDRegs->SA1 = (device_select) | (BlockNum & MUX_SA1R_FBA_MASK);
	//pOneNANDRegs->SA1 = BlockNum & MUX_SA1R_FBA_MASK;

	//Step 4:
	//Read Write Protection Status
	//if (pFlexRegs->WPS & FLEX_WPSR_UNLOCKED != FLEX_WPSR_UNLOCKED)
	//	return FlashLockError;

	//Step 5:
	//Write Page and Sector Addres (must be 0) to Start Address 8
	pOneNANDRegs->SA8 = (PageNum << 2) | 0x0;

	//Step 6:
	//Fill out Start Buffer Register info
	pOneNANDRegs->SBR = (UINT16_T) (SBR_WRITE_DEFAULT) | (SectorCount & MUX_SBR_BSC_MASK);

	//Step 7:
	//Set System Config Register- leave at default 
	//uncomment next line to turn of ECC		
	//pFlexRegs->SYSCONF1 = CONF1_WRITE_DEFAULT | CONF1_ECC_OFF; 

	//Step 8:
	//Clear Interrupt Register
	pOneNANDRegs->INTRPT = INT_STAT_REG_CLEAR;
	
	//Step 9:
	//Write PROGRAM Cmd to command register
	pOneNANDRegs->COMMAND = ONE_NAND_PROGRAM_SECTOR_FROM_BUFFER;

	//Step 10:
	//Wait for WRITE interrupt
	Retval = WaitForCommandCompletion(PEND_WRITE, (UINT32)ONENAND_TIMEOUT_MS );
    if (Retval != NoError)
		return Retval;

	//Step 11:
	//Read Controller status register
	if ((pOneNANDRegs->CNTSTAT & FLEX_CSR_ERROR) == FLEX_CSR_ERROR)
		Retval = ProgramError;

	return Retval;

}

UINT_T ProgramSpareSector()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T CopyBackProgram()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T UnlockNANDArray()
{
	UINT_T Retval = NoError;
	P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();

	//Step 1: Write DFS to Start Address 1 Register
	pOneNANDRegs->SA1 = pFlexProp->MuxType << 15;

	//Step 2: Set DBS for Start Address 2 Register
	pOneNANDRegs->SA2 = pFlexProp->MuxType << 15;

	//Step 3: Write SBA to SBA Register
	pOneNANDRegs->SBA = 0;

	//Step 4: Clear Interrupt Register
	pOneNANDRegs->INTRPT = INT_STAT_REG_CLEAR;
	
	//Step 5: Write all block unlock command
	pOneNANDRegs->COMMAND = FLEX_ALL_BLOCK_UNLOCK;

	//Step 6: Wait for Interrupt
	Retval = WaitForCommandCompletion(INT_STAT_REG, (UINT32)ONENAND_TIMEOUT_MS );
    if (Retval != NoError)
		return Retval;

	//Step 7: Read Controller Status Register
	if ((pOneNANDRegs->CNTSTAT & FLEX_CSR_ERROR) == FLEX_CSR_ERROR)
		Retval = GeneralError;
		
	return Retval;
}

UINT_T LockNANDArray()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T LockTightNANDArray()
{
	UINT_T Retval = NoError;
	return Retval;
}
UINT_T EraseVerifyRead()
{
	UINT_T Retval = NoError;
	return Retval;
}
UINT_T BlockErase( UINT32 blockNumber )
{
	UINT_T Retval;
    ONE_NAND_SA1_REG sa1reg;
    
    //First unlock Blocks
    //-------------------
    UnlockNANDArray();
    
    // setup the FBA in SA1
    // bits0-10 = blockNumber
    // bit 15 = blocks #'s above 2047
    //-------------------------------
    if( blockNumber > 2047 )
    {
      sa1reg.bits.ONE_NAND_DFS = 1;
      sa1reg.bits.ONE_NAND_RESERVED = 0;
      sa1reg.bits.ONE_NAND_FBA = blockNumber-2047;
    }
    else
    {
      sa1reg.bits.ONE_NAND_DFS = 0;
      sa1reg.bits.ONE_NAND_RESERVED = 0;
      sa1reg.bits.ONE_NAND_FBA = blockNumber;   
    }
      
    //Write FBA & DFS
    //-------------------------------
    pOneNANDRegs->SA1 = sa1reg.value;
    
    //set DBS in SA2
    //----------------
    if(blockNumber > 2047)
      pOneNANDRegs->SA2 = ( 1 << 15);
    else
      pOneNANDRegs->SA2 = 0;
    
	//Clear Interrupt Register
    //ensure write complete
    //------------------------
	pOneNANDRegs->INTRPT = INT_STAT_REG_CLEAR;
    sa1reg.value = pOneNANDRegs->INTRPT;
    
    //Write the command
    pOneNANDRegs->COMMAND = ONE_NAND_BLOCK_ERASE;
    
    //Wait for operation complete
    //---------------------------
	Retval = WaitForCommandCompletion(PEND_ERASE, (UINT32)ONENAND_TIMEOUT_MS*3 );
    if (Retval != NoError)
		return EraseError;

	//Read Controller Status Register
	if ((pOneNANDRegs->CNTSTAT & ONENAND_CSR_ERROR) == ONENAND_CSR_ERROR)
		Retval = EraseError;
        
	return Retval;
}

UINT_T MultiBlockErase()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T EraseSuspend()
{
	UINT_T Retval = NoError;
	return Retval;
}
UINT_T EraseResume()
{
	UINT_T Retval = NoError;
	return Retval;
}
UINT_T ResetNANDCore()
{
	UINT_T Retval = NoError;
	UINT16 IntStatReg=0;
	int count=0;

	do{
		 // This will return if there's no FlexOneNAND device connected.
		 if(count > INT_CLEAR_RETRY_CNT)
		 {
		   return GeneralError;
		 }
		// clear interrupt register
		pOneNANDRegs->INTRPT = INT_STAT_REG_CLEAR;
		IntStatReg = pOneNANDRegs->INTRPT;
		count++;
	// This checks for interrupt status register bit 15(INT)
	}while (IntStatReg & INT_STAT_REG);	
	 
	pOneNANDRegs->COMMAND = ONE_NAND_CORE_RESET;

	if( WaitForCommandCompletion(INT_STAT_REG_RSTI_PEND, (UINT32)RESET_TIMEOUT_MS )!= NoError)
	{
	   return GeneralError;
	}

	return Retval;
}
UINT_T ResetOneNAND()
{
	UINT_T Retval = NoError;
	UINT16 IntStatReg=0;
	int count=0;

	do{
		 // This will return if there's no FlexOneNAND device connected.
		 if(count > INT_CLEAR_RETRY_CNT)
		 {
		   return GeneralError;
		 }
		// clear interrupt register
		pOneNANDRegs->INTRPT = INT_STAT_REG_CLEAR;
		IntStatReg = pOneNANDRegs->INTRPT;
		count++;
	// This checks for interrupt status register bit 15(INT)
	}while (IntStatReg & INT_STAT_REG);	
	 
	pOneNANDRegs->COMMAND = ONE_NAND_RESET;

	if( WaitForCommandCompletion(INT_STAT_REG_RSTI_PEND, (UINT32)RESET_TIMEOUT_MS )!= NoError)
	{
	   return GeneralError;
	}

	return Retval;
}
UINT_T OTPAccess()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT32 ComputeDivisorShiftFromPowerOf2( UINT32 power )
{
  UINT32 shift = 0;

  if( (power == 0) || (power & 0x1))
    return 0;

  while( !(power & 0x01) )
  {
    shift +=1;
    power >>= 1;
  }


  //Only one bit can be set in power...
  //otherwise it's not a power of 2...
  //----------------------------------
  power >>=1; //Power should now == 0 if only one bit set
  if( power )
    return 0;

  return shift;
}
