/******************************************************************************
**  COPYRIGHT  2007 Marvell Inernational Ltd.
**  All Rights Reserved
******************************************************************************/

#include "OneNAND_Driver.h"
#include "FlexOneNAND_Cmd.h"
#include "OneNAND_Cmd.h"
#include "errors.h"

P_FLEX_REGISTERS_T pFlexRegs = (P_FLEX_REGISTERS_T)FLEX_ONENAND_REG_BASE;
P_ONE_NAND_REGISTERS_T pOneNANDRegs = (P_ONE_NAND_REGISTERS_T)FLEX_ONENAND_REG_BASE;

extern UINT_T ResetOneNAND(void);
extern UINT_T ResetFlexOneNAND(void);
extern UINT_T LoadSector(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt, char *oob);
extern UINT_T LoadFlexPage(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt);
extern UINT32 ComputeDivisorShiftFromPowerOf2( UINT32 power );

UINT noOfBlocks;
FlexOneNAND_Properties_T FlexOneNAND_Prop;  // Only need one

#define LOAD_ADDRESS 0x0
#define BLOCK_SIZE 0x20000
#define DATA_BUFFER_SIZE 250

//extern UINT32 RelocateBlock( UINT32 *badBlock, P_RelocTable_T pBBT );

P_FlexOneNAND_Properties_T GetFlexOneNANDProperties(void)
{
    return &FlexOneNAND_Prop;
}

 //-----------------------------------------------------------------------
// ONENAND_Init()
//
//     Bring-up the OneNAND infrastructure
//
//     RETURNS 0 if all went well
//-----------------------------------------------------------------------
UINT_T InitializeFlexOneNANDDevice(UINT8_T FlashNum, FlashBootType_T FlashBootType)
{

  UINT16 FlexManID=0, OneNANDManID=0, DevID = 0, DevType, DevDensity;
  UINT_T deviceFound = FALSE;
  UINT_T Retval = NoError;
  int i;

    //Initialize Flash properties
  P_FlashProperties_T pFlashProp = GetFlashProperties(FlashBootType);
  P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();

    //Setup flash interface data & function pointers
    //No Erase or Finalize functions for Bootrom
    //----------------------------------------------
//    pFlashProp->FinalizeFlash     = &FinalizeFlash;
    pFlashProp->ReadFromFlash       = &Read_Flash;
    pFlashProp->WriteToFlash        = &Write_Flash;
    pFlashProp->EraseFlash          = &Erase_Flash;
    pFlashProp->ResetFlash          = &ResetONENAND;
    pFlashProp->UseBBM              = 1;
    pFlashProp->TimLoadAddress      = TIMOffset_ONENAND;
  //Setup the OneNAND chip select
  ChipSelectOneNAND();

  // Read manufacturer ID for Flex and OneNAND devices.
  FlexManID = pFlexRegs->MANFID;
  OneNANDManID = pOneNANDRegs->MANFID;

  if (FlexManID == 0x00EC || OneNANDManID == 0x00EC)
  {
    deviceFound = TRUE;
    DevID = pFlexRegs->DEVID;
    DevType = DevID >> 8;
    if ( DevType == 2)
        pFlexProp->DeviceType = FLEX_ONE_NAND;
    else if ( DevType == 0 )
        pFlexProp->DeviceType = ONE_NAND;
  }

  if (deviceFound == FALSE)
  {

        Retval = ResetFlexOneNAND();
        if ( Retval == NoError)
        {
            deviceFound = TRUE;
            DevID = pFlexRegs->DEVID;
            pFlexProp->DeviceType = FLEX_ONE_NAND;
        }
        else
        {
            Retval= ResetOneNAND();
            if ( Retval == NoError)
            {
                deviceFound = TRUE;
                DevID = pOneNANDRegs->DEVID;
                pFlexProp->DeviceType = ONE_NAND;
            }
        }
  }
  if (deviceFound == TRUE)
  {
            DevID = pFlexRegs->DEVID;
            DevType = DevID >> 8;
            DevDensity = (DevID >> 4) & 0xF;
            if (DevID & 0x4)
                pFlexProp->MuxType = DUAL_CHIP;
            else
                pFlexProp->MuxType = SINGLE_CHIP;
            switch(DevType){
            case 0:
                pFlashProp->BlockSize = 128*KBYTES;
                pFlashProp->PageSize = 2*KBYTES;

            break;
            case 2:
                pFlashProp->BlockSize = 256*KBYTES;
                pFlashProp->PageSize = 4*KBYTES;

            break;
            }
            pFlexProp->BlockSizeDivShift = ComputeDivisorShiftFromPowerOf2( pFlashProp->BlockSize );
            pFlexProp->PageSizeDivShift = ComputeDivisorShiftFromPowerOf2( pFlashProp->PageSize );
            pFlexProp->NumOfBlocks = ((16 * MEGABYTES) << DevDensity) >> pFlexProp->BlockSizeDivShift;

            pFlashProp->NumBlocks = pFlexProp->NumOfBlocks;
  }
  
    borqs_printf2("onenand->pagesize=", pFlashProp->PageSize, 4);
    borqs_printf2("onenand->blocksize=", pFlashProp->BlockSize, 4);
    borqs_printf2("onenand->numofblocks=", pFlashProp->NumBlocks, 4);
    borqs_printf2("onenand->oobsize=", pFlashProp->PageSize >> 5, 4);
  if (!deviceFound)
        return FlashDriverInitError;

  return NoError;
}

UINT_T ResetONENAND(FlashBootType_T fbt)
{
    P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();
    if(pFlexProp->DeviceType == ONE_NAND)
        ResetOneNAND();
    else
        ResetFlexOneNAND();
    return NoError;
}

UINT_T WaitForCommandCompletion( UINT16 eventGroup, UINT32 TimeOutMillisec )
{
  UINT32 startTime;
  UINT_T Retval = NoError;
  UINT16 IntStatReg;

  startTime = GetOSCR0();

  do
  {
    if (pFlexRegs->INTRPT == eventGroup )
     return Retval;

  }while( OSCR0IntervalInMilli(startTime, GetOSCR0()) < TimeOutMillisec );

  return TimeOutError;
}

/* Read_Flash
 *  This function calls the relative Flash device read command to read data.
 *  This function will call the lower level driver to read on a block basis, in
 *  which to use the FlashManagement BBT
 *
 *  Inputs:
 *      flash_addr - absolute address of where to begin read in flash device
 *      buffer - pointer to location of where to load the data to
 *      total_size - amount of data, in bytes, to read
 *      fbt - flash boot type:  BOOT_FLASH or SAVE_STATE_FLASH
 *  Outputs:
 *      None
 *  Retval value:
 *      NoError - on successful completion
 *      <Other> - Flash device specific read error
 */
UINT_T Read_Flash(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt)
{
    UINT_T Retval = NoError;
    P_FlashProperties_T pFlashProp = GetFlashProperties(fbt);
    P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();
    UINT_T PageMask, ByteMask, read_size;

    PageMask = pFlashProp->BlockSize - pFlashProp->PageSize;
    ByteMask = pFlashProp->PageSize - 1;
    do
    {
        //calculate how much to read from buffer
        //  note: this normally should be a page size,
        //      Exceptions:
        //          - Start reading in the middle of a page
        //          - End reading in the middle of a page
        read_size = pFlashProp->PageSize - (FlashOffset & ByteMask);
        read_size = read_size > size ? size : read_size;

        //Write the page

        if(pFlexProp->DeviceType == ONE_NAND)
            //Retval = ProgramSector(address, buffer, write_size, fbt);
            Retval = LoadSector(FlashOffset, buffer, read_size, fbt, NULL);
        else
            //Retval = ProgramFlexPage(address, buffer, write_size, fbt);
            Retval = LoadFlexPage(FlashOffset, buffer, read_size, fbt);

        //Check and handle errors
        switch(Retval){
        case NoError:
            FlashOffset += read_size;
            size -= read_size;
            buffer += read_size;
            break;
        case ReadDisturbError:
            //TBD
            break;
        case TimeOutError:
        default:
            //Reset device
            if(pFlexProp->DeviceType == FLEX_ONE_NAND)
                Retval = ResetFlexOneNAND();
            else
                Retval = ResetOneNAND();
            // No need to loop around again. Return after the reset.
            return Retval;
            break;
        }
    }
    while ((size > 0) && (Retval == NoError));

    return Retval;
}
extern volatile UINT8_T is_yaffs;
UINT_T Write_Flash(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt)
{
    UINT_T Retval = NoError;
    UINT_T PageMask, ByteMask, write_size;
    P_FlashProperties_T pFlashProp = GetFlashProperties(fbt);
    P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();

    PageMask = pFlashProp->BlockSize - pFlashProp->PageSize;
    ByteMask = pFlashProp->PageSize - 1;
    do{
        //calculate how much to write from buffer
        //  note: this normally should be a page size,
        //      Exceptions:
        //          - Start writing in the middle of a page
        //          - End writing in the middle of a page
        write_size = pFlashProp->PageSize - (FlashOffset & ByteMask);
        write_size = write_size > size ? size : write_size;

        //Write the page

        if(pFlexProp->DeviceType == ONE_NAND)
            Retval = ProgramSector(FlashOffset, buffer, write_size, fbt);
        else
            Retval = ProgramFlexPage(FlashOffset, buffer, write_size, fbt);

        //Check and handle errors
        switch(Retval){
        case NoError:
            FlashOffset += write_size;
            size -= write_size;
            buffer += write_size;
            if(is_yaffs)
            {
                buffer += 64;
            }
            //borqs_printf1("w");
            break;
        case ProgramError:
            //Relocate Block?
            borqs_printf("w+ProgramError");
            break;
        case FlashLockError:
            //Unlock Device
            if(pFlexProp->DeviceType == ONE_NAND)
                Retval =  UnlockNANDArray();
            else
                Retval = UnlockAllFlexBlocks();
            borqs_printf("w+FlashLockError");
            break;
        case TimeOutError:
        default:
            borqs_printf("w+TimeOutError+others");
            //Reset device
            if(pFlexProp->DeviceType == FLEX_ONE_NAND)
                Retval =  ResetFlexOneNAND();
            else
                Retval = ResetOneNAND();
            // The device has been reset. No need to loop around again.
            return Retval;
            break;
        }
    }while ((size > 0) && (Retval == NoError));

    return Retval;
}

UINT_T Erase_Flash(UINT_T FlashOffset, UINT_T size, FlashBootType_T fbt)
{
    UINT_T Retval = NoError;
    UINT_T BlkNum;
    P_FlexOneNAND_Properties_T pDeviceProp;

    //Get Device properties pointer
    //-----------------------------
    pDeviceProp = GetFlexOneNANDProperties();

    //Calc block num
    //------------------------------------
    BlkNum = FlashOffset >> (pDeviceProp->BlockSizeDivShift);
    do{
        //Erase block
        if(pDeviceProp->DeviceType == ONE_NAND)
            Retval = BlockErase(BlkNum);
        else
            Retval = FlexBlockErase(BlkNum);

        if(Retval != NoError)
        {
            borqs_printf2("Erase Error:", BlkNum, 4);
        }
        //If there is a Flash Lock error,
        //we need to Unlock device and try erase again
        if(Retval == FlashLockError)
        {
            //Unlock Device
            if(pDeviceProp->DeviceType == ONE_NAND)
                Retval = UnlockNANDArray();
            else
                Retval = UnlockAllFlexBlocks();
            if(Retval == NoError)
                continue;
        }
        break;
    } while (TRUE);

    return Retval;
}

UINT_T CalculateBlockNumber(FlashBootType_T fbt)
{
    UINT_T Retval = NoError;
    UINT16 DevID = 0, DevDensity;

    P_FlashProperties_T pFlashProp = GetFlashProperties(fbt);
    P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();
    DevID = pFlexRegs->DEVID;

    DevDensity = (DevID >> 4) & 0xF;

    pFlexProp->NumOfBlocks = ((128 * MEGABYTES) / pFlashProp->BlockSize) << DevDensity;


    return Retval;
}

UINT_T CheckReadDisturb(UINT8 *pMBuf, UINT_T noSectors, UINT16* ECCResultsMain)
{
  UINT_T Retval = NoError;
  P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();
  int i;
  UINT16 ECCStatus, ECCMainMask, ECCResult, ECCResultByte;
  UINT8 ECCResultWord, ECCResultBit;
  UINT8 ChkMask;

    if ( pFlexProp->DeviceType == ONE_NAND)
    {
        ECCStatus = pOneNANDRegs->ECCSR;
        if ( ECCStatus == 0 )
            return Retval;

        for (i = 0;i < noSectors;i++)
        {
            ECCMainMask = (0x04 << (i * 4));

            if (pMBuf != NULL)
            {
                if (ECCStatus & ECCMainMask)
                {
                    /* 1-Bit error */
                    ECCResult = ECCResultsMain[i];

                    ECCResultWord = (UINT8)((ECCResult & 0x0ff0)>>4);
                    ECCResultBit = (UINT8)(ECCResult & 0xf);

                    ECCResultByte = i * 512 + ECCResultWord * 2;

                    if (ECCResultBit >= 8)
                    {
                        ECCResultByte++;
                        ECCResultBit -= 8;
                    }

                    ChkMask = 0x01 << ECCResultBit;

                    if ((pMBuf[ECCResultByte] & ChkMask) != 0)
                    {
                        return ReadDisturbError;
                    }
                }
            }
        }
    }
    else
    {
        if ( pFlexRegs->ECCSR1 == 0 ||
             pFlexRegs->ECCSR2 == 0 ||
             pFlexRegs->ECCSR3 == 0 ||
             pFlexRegs->ECCSR4 == 0 )
            return Retval;

        else
        {
            for (i = 0;i < noSectors;i++)
            {
                ECCMainMask = (0x1 << (i * 8));

                if (pMBuf != NULL)
                {
                    if ( (pFlexRegs->ECCSR1 & ECCMainMask) ||
                        (pFlexRegs->ECCSR2 & ECCMainMask) ||
                        (pFlexRegs->ECCSR3 & ECCMainMask) ||
                        (pFlexRegs->ECCSR4 & ECCMainMask) )
                    {
                        /* 1-Bit error */
                        ECCResult = ECCResultsMain[i];

                        ECCResultWord = (UINT8)((ECCResult & 0x1f00)>>8);
                        ECCResultBit = (UINT8)(ECCResult & 0xf);

                        ECCResultByte = i * 512 + ECCResultWord * 2;

                        if (ECCResultBit >= 8)
                        {
                            ECCResultByte++;
                            ECCResultBit -= 8;
                        }

                        ChkMask = 0x01 << ECCResultBit;

                        if ((pMBuf[ECCResultByte] & ChkMask) != 0)
                        {
                            return ReadDisturbError;
                        }
                    }
                }
            }
        }
    }
  return Retval;
}














