/*--------------------------------------------------------------------------------------------------------------------
(C) Copyright 2006, 2007 Marvell DSPC Ltd. All Rights Reserved.
-------------------------------------------------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------------------------------------------------
INTEL CONFIDENTIAL
Copyright 2006 Intel Corporation All Rights Reserved.
The source code contained or described herein and all documents related to the source code (Material) are owned
by Intel Corporation or its suppliers or licensors. Title to the Material remains with Intel Corporation or
its suppliers and licensors. The Material contains trade secrets and proprietary and confidential information of
Intel or its suppliers and licensors. The Material is protected by worldwide copyright and trade secret laws and
treaty provisions. No part of the Material may be used, copied, reproduced, modified, published, uploaded, posted,
transmitted, distributed, or disclosed in any way without Intels prior express written permission.

No license under any patent, copyright, trade secret or other intellectual property right is granted to or
conferred upon you by disclosure or delivery of the Materials, either expressly, by implication, inducement,
estoppel or otherwise. Any license under such intellectual property rights must be express and approved by
Intel in writing.
-------------------------------------------------------------------------------------------------------------------*/

/************************************************************************/
/*                                                                      */
/* Title: I2C pacakge header file                                       */
/*                                                                      */
/* Filename:  I2C.h                                                    */
/*                                                                      */
/* Target, subsystem: Common Platform, HAL                              */
/************************************************************************/
#ifndef _I2C_H_

  #define _I2C_H_
/*----------------------------------------------------------------------*/
#include "errors.h"


#define I2C_ICR_REG_INIT_VALUE      0x014E0   //all interrupts are disabled, except of the Bus Error and arbitration lost

//MICCO_BO
#define PMC_I2C_WRITE_SLAVE_ADDRESS               0x68
#define PMC_I2C_READ_SLAVE_ADDRESS                0x69

typedef enum
{
	I2C_FALSE = FALSE,
	I2C_TRUE  = TRUE
} I2C_BOOL;


/*********************** Sub-Package Parameters *************************/


typedef enum
{
    SLOW_MODE_ENABLE = 0x7fff,
    FAST_MODE_ENABLE = 0x8000
}I2C_FAST_MODE;


//ICAT EXPORTED ENUM
typedef enum
{
    I2C_RC_OK,
    I2C_RC_NOT_OK,
    I2C_RC_INVALID_DATA_SIZE,
    I2C_RC_INVALID_DATA_PTR,
    I2C_RC_TOO_MANY_REGISTERS,
    I2C_RC_TIMEOUT_ERROR,                               // 5
    I2C_RC_CHIP_BUSY,                                   // 6
    I2C_RC_INVALID_GENERAL_CALL_SLAVE_ADDRESS,
	I2C_RC_UNREGISTER_ERR,
	I2C_RC_MESSAGE_QUEUE_IS_FULL,
    I2C_ISR_UNEXPECTED_INTERRUPT,                       // 0xA
	I2C_ISR_BUS_ERROR,                                  // 0xB
	I2C_ISR_BUS_BUSY,                                   // 0xC
	I2C_ISR_CALL_BACK_FUNCTION_ERR,
	I2C_ISR_ARBITRATION_LOSS,
	I2C_RC_ILLEGAL_USE_OF_API
}I2C_ReturnCode;




typedef struct
{
    UINT8_T 			  			scl;    	// i2c_clk
    UINT8_T 			  			sda; 		// i2c_data
}  I2C_IBMR;



/*************** Prototypes *****************/



I2C_ReturnCode I2CConfigure( void );
void I2CBusStateGetDirect ( I2C_IBMR   *i2c_ibmr );
I2C_ReturnCode I2CMasterSendDataDirect( UINT8_T *data , UINT16_T dataSize , UINT8_T slaveAddress );
I2C_ReturnCode I2CMasterReceiveDataDirect ( UINT8_T 						   *cmd,
											UINT16_T 	 						cmdLength,
											UINT8_T 							writeSlaveAddress,
		                                    UINT8_T 						   *designatedRxBufferPtr,
											UINT16_T 	 						dataSize,
											UINT8_T 							readSlaveAddress );

/*----------------------------------------------------------------------*/

#endif  /* _I2C_H_ */
