/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2006 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	SaveState.h
**
**  PURPOSE: 	Structures for WTM save state information used during
** 					trusted boot operations
**
******************************************************************************/

#ifndef __save_state_h
#define __save_state_h

#define MAGICID	0x57545006	//"WTP06"
#define SLOTZEROSIZE	2048 //size in bytes of fixed state
#define SLOTNSIZE		512 //size of dynamic state in bytes

//*************** WTM Based Defines*****************************
#define WTM_KEY_RESTRICT_PRIV 		0x4000
#define WTM_SLOT_0_ID				0
#define WTM_SLOT_1_ID				1
#define WTM_KEY_ID					1
#define WTM_INSTALL_SUCCESS			0x01010101
#define WTM_NEW_DEVICE				0x02020202
#define WTM_BACKUP_RESET			0x03030303
#define WTM_COMPROMISED				0x04040404

//Dynamic portion of the save state that can reside at any slot 1 - max
// minimum size is 512 Bytes or 1 page for NAND.
typedef struct {
	UINT_T	MagicNo;
	UINT_T	RNGSeed[5];
	UINT_T	WTMStateCipher[20];
	UINT_T	BRStatus;
	UINT_T	WTPSPStatus;
	UINT_T	DKBOBMStatus;
	UINT_T	SoftStateCipher[20];
}DynamicState, *pDynamicState;


//Fixed Portion of the save state resides in slot 0
// size is 2048 KB

typedef struct {
	UINT_T	WTMStateSubKey[26];
	UINT_T	SoftStateSubKey[26];
	UINT_T	SoftStateUserKey[26];
	UINT_T	UCodeUpdateSubKey[26];
	UINT_T  UCodeSize;
	UINT_T  UCodeOEMHeaderSize;
	UINT_T	WTPSPRetCode;
}FixedState, *pFixedState;


typedef struct {
	FixedState		SlotZero;
	DynamicState	SlotN;
	UINT_T			NextSlotAvailable;
}SaveState, *pSaveState;

typedef enum
{
	SLOT0			= 0,
	SLOT1 			= 1
} SLOT_TYPE_T;

#endif
