;/******************************************************************************
;**	(C) Copyright 2007 Marvell International Ltd. �
;**	All Rights Reserved
;******************************************************************************/

;
; INTEL CONFIDENTIAL
; Copyright 2004-2005 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code (Material) are owned by Intel Corporation
; or its suppliers or licensors.  Title to the Material remains with
; Intel Corporation or its suppliers and licensors. The Material contains
; trade secrets and proprietary and confidential information of Intel
; or its suppliers and licensors. The Material is protected by worldwide
; copyright and trade secret laws and treaty provisions. No part of the
; Material may be used, copied, reproduced, modified, published, uploaded,
; posted, transmitted, distributed, or disclosed in any way without Intel's
; prior express written permission.
;
; No license under any patent, copyright, trade secret or other intellectual
; property right is granted to or conferred upon you by disclosure or
; delivery of the Materials, either expressly, by implication, inducement,
; estoppel or otherwise. Any license under such intellectual property rights
; must be express and approved by Intel in writing.
;
;
;*********************************************************************************
;
;  FILENAME:    xlli_LowLev_Init.s
;
;  PURPOSE:     This XLLI source file contains XLLI code for both the Bulverde
;               and Monahnas platforms. The Bulverde specific code will be
;               removed after Monahans is running on Zylonite.
;
;  LAST MODIFIED:  20-Apr-2005
;       Added conditional for stack pointer: "xlli_stack_pointer"
;
;******************************************************************************
;
; NOTES:
;
; The use of these subroutines and/or the order in which they are called is, for the most part,
; arbitrary and is left up to the user. Only a few subroutines must be called in a specific order.
; For example, it makes no sense to go to virtual mode before the MMU is initialized and
; the page table is set up.... and, in order to set up the page table, memory needs to be initialized
; first.
;
        INCLUDE xlli_platform_defs_fbw.INC         ; Monahans specific include file
        INCLUDE xlli_board_defs_fbw.INC        ; Tavor EVB specific settings
;
        AREA    |.text|, CODE, READONLY, ALIGN=5         ; Align =5 required for "ALIGN 32" feature to work.
;
; List of Low Level Init functions in this source code include:
;
        EXPORT mini_xlli_Dmem_initP1         ; Dynamic Memory Controller Initialization Part #1
        EXPORT mini_xlli_Rcomp               ; Resistive compensation routine for DDR SDRAM
        EXPORT mini_xlli_DDR_HCAL            ; Hardware delay line calibration routine
        EXPORT mini_xlli_DDR_SCAL            ; Software delay line calibration routine
        EXPORT mini_xlli_Dmem_initP2         ; Dynamic Memory Controller Initialization Part #2
        EXPORT mini_xlli_DDR_HCAL_EVAL

;        GLOBAL  mini_xlli_EMPI_offset_table  ; EMPI Pin Offset Table
;
;        LTORG
;
;mini_xlli_EMPI_offset_table
;        DCW     0x24C,  0x254,  0x25C,  0x264,  0x26C,  0x274,  0x27C,  0x284   ; DF_IO1 -> DF_IO7
;        DCW     0x250,  0x258,  0x260,  0x268,  0x270,  0x278,  0x280,  0x288   ; DF_I08 -> DF_IO15
;        DCW     0x23C,  0x240,  0x244,  0x248                                   ; DF_ADDR0 -> DF_ADDR3
;        DCW     0x238,  0x204,  0x208,  0x20C,  0x210,  0x214,  0x218,  0x21C   ; (misc)
;        DCW     0x220,  0x224,  0x228,  0x22C,  0x230,  0x234,  0x238           ; (misc)
;        DCW     0x000           ; An offset of zero marks the end of the table
        ALIGN   4

;
;*******************************************************************************
;
; **************************************
; **********                  **********
; ********** xlli_Dmem_InitP1 **********   INITALIZE DYNAMIC MEMORY CONTROLLER
; **********                  **********           (PART 1 OF 2 PARTS)
; **************************************
;
; The sequence below is based on the recommended memory initializing procedure
; recommended in the (Monahans) External Memory Controller Manual
;
;       INPUT PARAMETERS:
;
;               NONE
;
;       RETURNED STATUS/VALUE(s):
;
;               NONE
;

mini_xlli_Dmem_initP1        FUNCTION
        IF :DEF: xlli_stack_pointer     ; If stack pointer has been defined
        stmfd   sp!,    {r0-r4, lr}             ; Save r0-r4, & link on stack
        ENDIF
;
; STEP 1: Delay at least 100uS to allow internal clocks for SDCLK to stabilize.
;       This is a required hardware reset power-on wait period. XLLI code waits
;       a little bit longer - just to be sure.
;
        ldr     r0,     =xlli_OSTREGS_PHYSICAL_BASE     ; Load OS timer base address
        ldr     r2,     [r0, #xlli_OSCR0_offset]        ; Fetch starting value of OSCR0
;        add     r2,     r2,     #0x150                  ; Actually, 0x145 is about 100uS
;        add     r2,     r2,     #0x600000                  ; Work around because some problem with ddr - increase delay to 400ms
		add     r2,     r2,     #0x2A0                  ;
xlli_d1 ldr     r1,     [r0, #xlli_OSCR0_offset]        ; Fetch current OSCR0 value
        cmp     r1,     r2                              ; Is the timer past the time out value?
        bmi     xlli_d1                                 ; No - Loop until it is
;
;       Set up EMPI register
;
        ldr     r4,     =xlli_DMEMREGS_PHYSICAL_BASE    ; Dynamic Memory Control Reg Base
        ldr     r3,     =xlli_EMPI_value                ; Get EMPI value
        str     r3,     [r4, #xlli_EMPI_offset]         ; Load the EMPI register
;
; STEP 2: Prevent Power Down Command by setting CMA[APDD] bit. (A0 Silicon only)
;
 IF :DEF: PV
				ldr			r0,			[r4, #xlli_CMA_offset]          ; Read CMA content
				orr     r0,     r0,  #0x01000000                ; Set this bit to enable a workaround for a DDR stability problem.
 ELSE
        mov     r0,     #xlli_CMA_APDD                  ; Set this bit to prevent power down
 ENDIF
        str     r0,     [r4, #xlli_CMA_offset]          ; Set the bit
;
; STEP 3: Program MDCNFG then enable the occupied DRAM partitions.
;         MDCNFG[DMCEN] must remain deasserted!
;
        ldr     r2,     =xlli_MDCNFG_value              ; Get user config values
        and     r0,     r2,     #xlli_MDCNFG_DSCE       ; Save the DSCE bits
 ; change for A0: DDR should be configured on 32 bits (in Z0 - 16 bits)
        ;ldr     r3,     =0x8000077C                    ; 16 bits Mask to clear reserved, DMCEN & DSCE bits
        ;ldr     r3,     =0x80000778                    ; ;32 bits Mask to clear reserved, DMCEN & DSCE bits
				ldr     r3,     =xlli_MDCNFG_clr_DMCEN_DSCE     ; ;32 bits Mask to clear reserved, DMCEN & DSCE bits
        and     r2,     r2,     r3                      ; Clear reserved, DMCEN & DSCE bits
        str     r2,     [r4, #xlli_MDCNFG_offset]       ; Write to controller
        ldr     r2,     [r4, #xlli_MDCNFG_offset]       ; Insure write is complete
        orr     r2,     r2,  r0                         ; Enable selected partition(s)
        orr     r2,     r2,  #xlli_MDCNFG_DE1           ; Enable selected partition(s)
        str     r2,     [r4, #xlli_MDCNFG_offset]       ; Write to controller
        ldr     r2,     [r4, #xlli_MDCNFG_offset]       ; Insure write is complete
;
;       Set in Tavor Address Isolation Control Register (TAI) 7/31/2006 5:53PM
;
        ldr     r2,     [r4, #xlli_TAI_offset]         ; Get  TAI
        orr     r2 ,    r2,  #0x80000000               ; Set bit 31
        str     r2,     [r4, #xlli_TAI_offset]         ; Set  TAI

; Step 4: Wait a typical NOP power-up waiting period required by SDRAM (Typically
;         200 uS) Steps 5 & 6 can be executited during this waiting period. However,
;         xlli will do the wait now and then perform steps 5 & 6.
;
        ldr     r0,     =xlli_OSTREGS_PHYSICAL_BASE     ; Load OS timer base address
        ldr     r2,     [r0, #xlli_OSCR0_offset]        ; Fetch starting value of OSCR0
        add     r2,     r2,     #0x2A0                  ; Actually, 0x28A is about 200uS
xlli_m2 ldr     r1,     [r0, #xlli_OSCR0_offset]        ; Fetch current OSCR0 value
        cmp     r1,     r2                              ; Is the timer past the time out value?
        bmi     xlli_m2                                 ; No - Loop until it is
;
;       In early silicon it has been observed that the Rcomp eval may not complete
;       although the SWEVAL bit was set. The edits below will set this bit up to
;       four times before exiting.
;
        mov     r1,     #4              ; Set up Rcomp retry loop count
;
; STEP 5: Perform Resistive Compensation calibration cycle (RCOMP)
;
xlli_m3
 IF :DEF: PV
 				ldr     r3,     =(xlli_RCOMP_RCRNG_value<<25)
 ELSE
        ldr     r3,     =((xlli_RCOMP_RCRNG_value<<25) | xlli_RCOMP_REI | xlli_RCOMP_RCNOP)
 ENDIF
        str     r3,     [r4, #xlli_RCOMP_offset]        ; Write inital values to RCOMP
        orr     r3,     r3,     #xlli_RCOMP_SWEVAL      ; Set bit to perform the RCOMP
        str     r3,     [r4, #xlli_RCOMP_offset]        ; Start the RCOMP evaluation
;
;       Wait for Rcomp evaluation cycle to complete - RCOMP[SWEVAL] clears.
;       A "time out" counter is in this loop - just in case the eval bit doesn't clear
;       so there is an exit path.
;
        mov     r0,     #0x10000                        ; Set up arbitrary time out counter
xlli_m4 ldr     r2,     [r4, #xlli_RCOMP_offset]        ; Fetch status
        subs    r0,     r0,     #1                      ; Decrement loop count
        beq     xlli_m9                                 ; Branch if timed out
        ands    r2,     r2,     #xlli_RCOMP_SWEVAL      ; Is the evaluation complete?
        bne     xlli_m4                                 ; No - loop until done.

                IF :DEF: xlli_stack_pointer     ; If stack pointer has been defined
xlli_m8 ldmfd   sp!,    {r0-r4, pc}             ; Return to caller via the stack
                ELSE
xlli_m8 mov     pc,     lr              	; Non stack return
                ENDIF

xlli_m9 subs    r1,     r1,     #1      ; Decrement re-try count
        bne     xlli_m3                 ; Try again if count is not zero
        b       xlli_m8                 ; Otherwise - time to give up on this!!!

        ENDFUNC
;
;*******************************************************************************
;
; ********************************
; **********            **********
; ********** mini_xlli_Rcomp **********   PERFORM RCOMP CALCULATION
; **********            **********
; ********************************
;
; Performs Rcomp calculations
;
;       INPUT PARAMETER REQUIRED:
;
;               R0 = Dynamic Memory Control Reg Base Address
;               R8 = Slew Rate   -- only in PV
;               R3 = XCode value to use if XCode from DMCISR2 is 0   -- only in PV
;       RETURNED STATUS/VALUE(s):
;
;               NONE
;
;       NOTES:
;
;       The user must also re-calibrate the DDR-SDRAM strobes by calling either
;       the xlli_DDR_HCAL or the xlli_DDR_SCAL routine after this subroutine.
;
;       It is left up to the user to (optionally) disable the Dynamic Memory
;       controller before this routine is called and to re-enable it after
;       returning from the HCAL or SCAL routine.
;
;       new one: from Leon's xlli
mini_xlli_Rcomp      FUNCTION

        stmfd   sp!,    {r0-r8, lr}             ; Save r0-r8, & link on stack


        mov     r4,     R0      ; Copy Dynamic Memory Control Reg Base
;
;       Obtain the NCODE and PCODE values from the DMCISR register and use this (For P only)
;       starting value to calculate the final NSLEW and PSLEW values to be
;       programmed into each pad group.
;
        ldr     r2,     [r4, #xlli_DMCISR_offset]       ; Get results of the evaluation
    IF :DEF: PV
        and     r6,     r2,     #0x000F8000             ; Extract NCODE
				orr			r6,			r6,			#0x00080000							;
    ELSE
        and     r6,     r2,     #0x003F8000             ; Extract NCODE
    ENDIF
        mov     r6,     r6,     LSR #15                 ; and right justify
    IF :DEF: PV
        and     r5,     r2,     #0x07C00000             ; Extract PCODE
    ELSE
        and     r5,     r2,     #0x1FC00000             ; Extract PCODE
    ENDIF
        mov     r5,     r5,     LSR #22                 ; and right justify



    IF :DEF: PV

; 	Obtain XCODE value from DMCISR2
	; for now, only use predefined XCODE value (in R3) - mdbrown
;    ldr     r7,     [r4, #xlli_DMCISR2_offset]       ; Get results of the evaluation
;    and 	r7, 	r7, 	#0xF					 ; Keep only the low 4 bits
;    cmp 	r7, 	#0
;    bne		PrepareToFillPads
		mov		r7, 	r3
    ELSE
;
;       At this point: r5 = PCODE, r6 = NCODE
;
;       The PSLEW and NSLEW values are calculated as follows:
;
;       PSLEW = 1.9473 - 0.051(PCODE) + 0.1914(NCODE)
;       NSLEW = - 2.0786 + 0.2739(PCODE) + 0.0279(NCODE)
;
;       To gain the desired accuracy, the constant values above are multiplied
;       by 10,000 (so 0.051 will become 510) and the math is done as integer math
;       rather than floating point. When the calculations are done, the result
;       is divided by 10,000 and rounded up/down to the closest integer value
;       for PSLEW and NSLEW.
;
;       Do the PSLEW calculation first.
;
        ldr     r3,     =1914           ; load 0.1914 * 10,000 into r3
        mul     r2,     r3,     r6      ; r2 = 1914(NCODE)
        ldr     r3,     =510            ; load 0.051 * 10,000 into r3
        mul     r1,     r3,     r5      ; r1 = 510(PCODE)
        sub     r2,     r2,     r1      ; subtract the PCODE from the NCODE
        ldr     r3,     =19473          ; load 1.9473 * 10,000 into r3
        add     r0,     r2,     r3      ; r0 = PSLEW * 10,000 at this point
        ldr     r1,     =5000           ; Set up the midpoint of 10,000
        add     r0,     r0,     r1      ; Add to PSLEW (will provide rounded PSLEW after divide)
        ldr     r1,     =10000          ; Set up divisor
        bl      xlli_U32Divide          ; Divide!
        mov     r7,     r2              ; Save the calculated PSLEW value in r7
        and     r7,     r7,   #0x0F     ; Clear not used bits 7/31/2006 5:41PM

;
;       Do the NSLEW calculation.
;
        ldr     r3,     =2739           ; load 0.2739 * 10,000 into r3
        mul     r1,     r3,     r5      ; r1 = 2739(PCODE)
        ldr     r3,     =279            ; load 0.0279 * 10,000 into r3
        mul     r2,     r3,     r6      ; r2 = 279(NCODE)
        add     r1,     r1,     r2      ; Add the NCODE from the PCODE
        ldr     r3,     =20786          ; load 2.0786 * 10,000 into r3
        sub     r0,     r1,     r3      ; r0 = NSLEW * 10,000 at this point
        ldr     r1,     =5000           ; Set up the midpoint of 10,000
        add     r0,     r0,     r1      ; Add to NSLEW (will provide rounded NSLEW after divide)
        ldr     r1,     =10000          ; Set up divisor
        bl      xlli_U32Divide          ; Divide!
        mov     r8,     r2              ; Save the calculated NSLEW value in r8
        and     r8,     r8,   #0x0F     ; Clear not used bits 7/31/2006 5:41PM

    ENDIF

;
;       At this point the registers we care about contain the following:
;
;               r4 = Dynamic Memory Control Reg Base Address
;               r5 = PCODE value
;               r6 = NCODE value
;
;               r7 = XCODE value  --  For PV
;               r8 = Slew Rate Value --  For PV
;
;               r7 = PSLEW value  --  For P
;               r8 = NSLEW value  --  For P
;
;       Next step is to combine the NCODE, PCODE, NSLEW, PSLEW values and program
;       all the pad registers to the same value.
;
PrepareToFillPads
        mov     r5,     r5,     LSL #24 ; Move the PCODE into position (LSB to bit 24)
        mov     r6,     r6,     LSL #16 ; Move the NCODE into position (LSB to bit 16)
    IF :DEF: PV
        mov     r7,     r7,     LSL #8  ; Move the XCODE into position (LSB to bit 8)
        orr     r8,     r8,     r7      ; OR the SLEW and XCODE values together
    ELSE
        mov     r7,     r7,     LSL #8  ; Move the PSLEW into position (LSB to bit 8)
        orr     r8,     r8,     r7      ; OR the NSLEW and PSLEW values together
    ENDIF
        orr     r8,     r8,     r6      ; OR in the NCODE value
        orr     r8,     r8,     r5      ; OR in the PCODE value

;
;       The PAD register data has been assembled into r3. Now the code needs
;       to send this data to the selected PAD register.
;
        str     r8,     [r4, #xlli_PAD_SDCLK_offset]    ;
        str     r8,     [r4, #xlli_PAD_MDMSB_offset]    ;

    IF :DEF: PV
        str     r8,     [r4, #xlli_PAD_CMD_offset]      ; Load the pad registers
        str     r8,     [r4, #xlli_PAD_QUAD_offset]    	;
    ELSE

        str     r8,     [r4, #xlli_PAD_MA_offset]       ; Load the pad registers
        str     r8,     [r4, #xlli_PAD_MDLSB_offset]    ;
        str     r8,     [r4, #xlli_PAD_DMEM_offset]     ;
        str     r8,     [r4, #xlli_PAD_SDCS_offset]     ;
        str     r8,     [r4, #xlli_PAD_SMEM_offset]     ;
        str     r8,     [r4, #xlli_PAD_SCLK_offset]     ;
    ENDIF
;
;       Set the UPDATE bit in the Rcomp register
;
        ldr     r3,     [r4, #xlli_RCOMP_offset]        ; Fetch RCOMP register value
        orr     r3,     r3,     #xlli_RCOMP_UPDATE      ; Set the UPDATE bit
        str     r3,     [r4, #xlli_RCOMP_offset]        ; Write RCOMP register back
;
;       All done - return to caller
;

        ldmfd   sp!,    {r0-r8, pc}             ; Return to caller via the stack
;
;
xlli_NPCODES    DCW     0,0,0,0,0,0,0,0         ; Eight 16-bit words
;
;
;        INCLUDE  xlli_Rcomp_table.inc   ;

        ENDFUNC
;
;
; *****************************************************
; **********                                 **********
; ********** HARDWARE DELAY LINE CALIBRATION **********
; **********                                 **********
; *****************************************************
;
; This xlli subroutine performs a hardware calibration for the dynamic
; memory strobe delay lines DQS[3:0].
;
;       NOTES:
;
;       In 16-bit bus mode, all functions corresponding to strobes DQS[3:2] are disabled.
;       DDR_SCAL (SW calibration enable) should be clear if using SDR_DRAM.
;
;       INPUT PARAMETERS:
;
;               r0 = Address of dynamic memory control registers base (Physical 0x4810_0000).
;
;       RETURNED STATUS/VALUE(s):
;
;               NONE
;

mini_xlli_DDR_HCAL   FUNCTION

                IF :DEF: xlli_stack_pointer     ; If stack pointer has been defined
        stmfd   sp!,    {r1-r2, lr}                ; Save used registers
                ENDIF

				ldr     r2,     =(xlli_DDR_HCAL_HCEN | xlli_DDR_HCAL_HCPROG |  xlli_DDR_HCAL_HCOD)
				bl		  isDDRFreqLessThanZOOMHz
				cmp		  r1, #0x1
				orreq	  r2, r2, #xlli_DDR_HCAL_HCPW
    		str     r2, [r0, #xlli_DDR_HCAL_offset]     ; Load data into DDR_HCAL register
;
;       Wait for hardware calibration to complete
;
xlli_h1 ldr     r1,     [r0, #xlli_DMCISR_offset]       ; Get results of the evaluation
        ands    r1,     r1,     #xlli_DMCISR_DLP        ; Extract DLP bit
;        beq     xlli_h1                                 ; Loop until set
        str     r1,     [r0, #xlli_DMCISR_offset]       ; Clear sticky DLP bit

                IF :DEF: xlli_stack_pointer     ; If stack pointer has been defined
        ldmfd   sp!,    {r1, pc}                ; Return to caller via the stack
                ELSE
        mov     pc,     lr                      ; Non stack return
                ENDIF
        ENDFUNC

;
;*******************************************************************************
;
;
;*******************************************************************************
;
; **************************************
; **********                  **********
; ********** xlli_Dmem_InitP2 **********   INITALIZE DYNAMIC MEMORY CONTROLLER
; **********                  **********           (PART 2 OF 2 PARTS)
; **************************************
;
; The sequence below is based on the recommended memory initializing procedure
; recommended in the (Monahans) External Memory Controller Manual
; (Issue 16860 - Section 1.4.3.6)
;
;       INPUT PARAMETER REQUIRED:
;
;               R0 = Dynamic Memory Control Reg Base Address
;
;       RETURNED STATUS/VALUE(s):
;
;               NONE
;

mini_xlli_Dmem_initP2        FUNCTION

                IF :DEF: xlli_stack_pointer     ; If stack pointer has been defined
        stmfd   sp!,    {r0-r2, lr}             ; Save r0-r2, & link on stack
                ENDIF
;
; STEP 7: Write the Mode Register Set Config Register (MDMRS)
;
        ldr     r0,     =xlli_DMEMREGS_PHYSICAL_BASE    ; Dynamic Memory Control Reg Base
        ldr     r2,     =xlli_MDMRS_value               ; Get config values
        orr     r2,     r2,     #(xlli_MDMRS_MSCS0 | xlli_MDMRS_MDCOND)      ; Set the "Condition DRAM" bit
        orr     r2,     r2,     #(xlli_MDMRS_MSCS1)
        str     r2,     [r0, #xlli_MDMRS_offset]        ; Update register
        ldr     r2,     [r0, #xlli_MDMRS_offset]        ; Insure write is complete
;
; STEP 8: Program the MDREFR with the desired refresh interval
;

        ldr     r1,     =xlli_MDREFR_value              ; Fetch inital refresh value
        str     r1,     [r0, #xlli_MDREFR_offset]       ; Write to register
        ldr     r1,     [r0, #xlli_MDREFR_offset]       ; Insure the write completes                           ; No - Loop until time runs out                              ; No - loop until they are

        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ; delay before enabling DDR (by docs should be 100u
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ; 	MOV  r3, #0xFFFFFFF0          ;  set r0 before reading to be able to find a good value with the debugger...
;delay
		;sub  r3, r3, #1
		;cmp  r3, #0
		;bpl  delay
;
; STEP 9: Enable the dynamic memory controller
;
        ldr     r1,     [r0, #xlli_MDCNFG_offset]       ; Get the MDCNFG value
        orr     r1,     r1,     #xlli_MDCNFG_DMCEN      ; Set the enable bit
        str     r1,     [r0, #xlli_MDCNFG_offset]       ; Write to controller
        ldr     r1,     [r0, #xlli_MDCNFG_offset]       ; Insure write is complete


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;
; STEP 9.1: Write the Extended Mode Register Set Config Register (MDEMRS)
;
;        ldr     r2,     [r0, #xlli_MDMRS_offset]        ; Get MDMRS values
;        orr     r2,     r2,  #(xlli_BIT_16)
;        str     r2,     [r0, #xlli_MDMRS_offset]        ; Update register
;        ldr     r2,     [r0, #xlli_MDMRS_offset]        ; Insure write is complete

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;
; STEP 10: Optionally Enable auto-power-down by clearing the CMA[APDD] bit
;
;        mov     r2,     #0                      ; Clear a work register - with APD
 IF :LNOT: :DEF: PV
        mov     r2,     #0x02000000             ; Set  Auto Power Down Disable bit 8/13/2006 3:31PM - NO APD
        str     r2,     [r0, #xlli_CMA_offset]  ; Write to CMA[APDD] bit
 ENDIF
;
;       SDRAM should be ready for use at this point - return to caller
;
                IF :DEF: xlli_stack_pointer     ; If stack pointer has been defined
        ldmfd   sp!,    {r0-r2, pc}             ; return to caller via the stack
                ELSE
        mov     pc,     lr                      ; Non stack return
                ENDIF

        ENDFUNC

;*********************************************************************************
;
;       ******************
;       *                *
;       * xlli_U32Divide * Subroutine
;       *                *
;       ******************
;
; This subroutine performs an unsigned 32 bit divide
;
; PARAMETER PASSING:
;
; r0  contains the dividend
; r1  contains the divisor
; r2  will contain the result on subroutine exit
; r3  will contain the remainder on subroutine exit
;
xlli_U32Divide  FUNCTION

                IF :DEF: xlli_stack_pointer     ; If stack pointer has been defined
        stmfd   sp!,    {r0, r1, r4, r5, lr}    ; Save used registers
                ENDIF

        mov     r2,     #0      ; Clear result
        mov     r3,     #0      ; Clear remainder
        mov     r5,     #32     ; load r5 with loop count
        mrs     r4,     CPSR                    ; Get CPSR
        bic     r4,     r4,     #0x20000000     ; Clear carry bit
;
;       Main division loop begins here
;
xlli_D1 msr     CPSR_f,         r4      ; write flags back
        adcs    r2,     r2,     r2      ; Shift C bit into result
        adcs    r0,     r0,     r0      ; Rotate r0 left 1 bit through C bit
        adcs    r3,     r3,     r3      ; Rotate data left, into r3
        subs    r3,     r3,     r1      ; Subtract dividend
        blt     xlli_D2                 ; Branch if negative
        mrs     r4,     CPSR            ; Get CPSR
        orr     r4,     r4,     #0x20000000     ; Set carry bit
        msr     CPSR_f,         r4      ; write flags back
        b       xlli_D3                 ; Check loop count

xlli_D2 add     r3,     r3,     r1      ; Restore r3
        mrs     r4,     CPSR            ; Get CPSR
        bic     r4,     r4,     #0x20000000     ; Clear carry bit
        msr     CPSR_f,         r4      ; write flags back

xlli_D3 mrs     r4,     CPSR            ; Get CPSR
        subs    r5,     r5,     #1      ; decrement loop count
        bne     xlli_D1                 ; Loop until done
;
;       Shift the last bit into the result and return to caller
;
        msr     CPSR_f,         r4      ; write flags back
        adcs    r2,     r2,     r2      ; Shift C bit into result
        adcs    r0,     r0,     r0      ; Rotate r0 left 1 bit through C bit

                IF :DEF: xlli_stack_pointer     ; If stack pointer has been defined,
        ldmfd   sp!,    {r0, r1, r4, r5, pc}    ; Return to caller vis the stack
                ELSE
        mov     pc,     lr                      ; Non stack return
                ENDIF
        ENDFUNC

;;
;;*******************************************************************************
;;
;; ************************************
;; **********                **********
;; ********** xlli_GPIO_init **********  INIT THE GPIOs FOR THIS PLATFORM
;; **********                **********  (TavorVERSION)
;; ************************************
;;
;;       GPIO direction and output levels are set by this subroutine according
;;       to the values in the platform include files. Output levels are set
;;       so the platform will draw a minimal amount of power.
;;
;;       INPUT PARAMETERS:
;;
;;               NONE
;;
;;       RETURNED STATUS/VALUE(s):
;;
;;               NONE
;;
;mini_xlli_GPIO_init  FUNCTION
;
;                IF :DEF: xlli_stack_pointer     ; If stack pointer has been defined
;        stmfd   sp!,    {r0-r2, lr}             ; Save r0-r2, & link on stack
;                ENDIF
;
;        ldr     r0,     =xlli_GPIOREGS_PHYSICAL_BASE    ; GPIO regs base address
;;
;;       Set any GPIOs that should be HIGH after GPIOs are configured.
;;
;        ldr     r1,     =xlli_GPSR0_value       ; Set bits for GPSR0
;        str     r1,     [r0, #xlli_GPSR0_offset]; GPIOs 31:0
;
;        ldr     r2,     =xlli_GPSR1_value       ; Set bits for GPSR1
;        str     r2,     [r0, #xlli_GPSR1_offset]; GPIOs 63:32
;
;        ldr     r1,     =xlli_GPSR2_value       ; Set bits for GPSR2
;        str     r1,     [r0, #xlli_GPSR2_offset]; GPIOs 95:64
;;
;;       Set any GPIOs that should be LOW after GPIOs are configured.
;;
;        ldr     r1,     =xlli_GPCR0_value       ; Set bits for GPCR0
;        str     r1,     [r0, #xlli_GPCR0_offset]; GPIOs 31:0
;
;        ldr     r2,     =xlli_GPCR1_value       ; Set bits for GPCR1
;        str     r2,     [r0, #xlli_GPCR1_offset]; GPIOs 63:32
;
;        ldr     r1,     =xlli_GPCR2_value       ; Set bits for GPCR2
;        str     r1,     [r0, #xlli_GPCR2_offset]; GPIOs 95:64
;;
;;       Set up GPIO direction registers.
;;
;        ldr     r1,     =xlli_GPDR0_value       ; Direction values for GPDR0
;        str     r1,     [r0, #xlli_GPDR0_offset]; GPIOs 31:0
;
;        ldr     r2,     =xlli_GPDR1_value       ; Direction values for GPDR1
;        str     r2,     [r0, #xlli_GPDR1_offset]; GPIOs 63:32
;
;        ldr     r1,     =xlli_GPDR2_value       ; Direction values for GPDR2
;        str     r1,     [r0, #xlli_GPDR2_offset]; GPIOs 95:64
;;
;;       GPIO Setup complete
;;
;                IF :DEF: xlli_stack_pointer
;        ldmfd   sp!,    {r0-r2, pc}     ; Return to caller (via stack)
;                ELSE
;        mov     pc,     lr              ; Return to caller (no stack
;                ENDIF
;
;        ENDFUNC
;        LTORG
;
;*******************************************************************************
;
; ************************************
; **********                **********
; ********** xlli_MFPR_init **********  INIT THE MFPRs FOR THIS PLATFORM
; **********                **********
; ************************************
;
;       The Multi-Function Pin registers are set to alternate function zero
;       and all other pin parameters are set so the platform will draw a
;       minimum amount of power.
;
;       INPUT PARAMETERS:
;
;               NONE
;
;       RETURNED STATUS/VALUE(s):
;
;               NONE
;


;********************************************************************************************
;
; *****************************************************
; **********                                 **********
; ********** SOFTWARE DELAY LINE CALIBRATION **********
; **********                                 **********
; *****************************************************
;
; This xlli subroutine performs a software calibration for the dynamic
; memory strobe delay lines DQS[3:0].
;
;       NOTES:
;
;       In 16-bit bus mode, all functions corresponding to strobes DQS[3:2] are disabled.
;       DDR_SCAL (SW calibration enable) should be clear if using SDR_DRAM. Memory should
;       be operating at maximum frequency to use the software calibration method.
;
;       INPUT PARAMETERS:
;
;               r0 = Address of dynamic memory control registers base (Physical 0x4810_0000).
;               r1 = Address of 8-byte DDR-SDRAM area to use for strobe delay test.
;
;       RETURNED STATUS/VALUE(s):
;
;               NONE
;
;       REGISTERS USED: R0 - r7 (all are preserved)
;

mini_xlli_DDR_SCAL   FUNCTION

         stmfd   sp!,    {r2-r9, lr}            ; Save used registers
;
;       Store a test pattern to memory
;
        ldr     r2,     =0x124837EC     ; Test pattern
        mvn     r3,     r2              ; Invert test pattern in r3
        str     r2,     [r1]            ; Store test pattern
        str     r3,     [r1, #4]        ; Store inverted pattern
                                        ; Disable HW calibration 7/17/2006 6:26PM
        ldr     r6,     [r0, #xlli_DDR_HCAL_offset]     ; Get  HCAL register
        bic     r6,     r6,     #0x80000000             ; Disable HW Calibration
        str     r6,     [r0, #xlli_DDR_HCAL_offset]     ; Set up HCAL register

;
;       Set up DDR_SCAL register to do software calibration. Inital conditions
;       are set up by writing 0x80000000 to the DDR_SCAL register
;
        ldr     r4,     =0x00808000                     ; Inital value
        str     r4,     [r0, #xlli_DDR_SCAL_offset]     ; Set up SCAL register
        ldr     r4,     [r0, #xlli_DDR_SCAL_offset]     ; Read back to latch
        ldr     r4,     =0x80808000                     ; Enable software cal, & auto increment by 2
        str     r4,     [r0, #xlli_DDR_SCAL_offset]     ; Set up SCAL register
        mov     r6,     #0                              ; Clear a work register
        mov     r7,     #0                              ; Clear a work register
        mov     r8,     #0                              ; Clear a work register
        mov     r9,     #0                              ; Clear a work register

        b       New_tsts                                ; 7/16/2006 7:45PM
;
;       Read the test pattern back from SDRAM saving data for later evaluation.
;       There are four strobe delays line to be calibrated, one strobe for
;       byte of data.
;
;       When the data is being read back, it may change from bad to good and
;       back to bad several times before the real data eye becomes apparent.
;       This code looks for the largest number of good reads for each byte
;       and sets the delay window for the center of this data eye. Four registers
;       are used to keep track of the largest window. R6 ends up containing the
;       delay setting of the last BAD read for the largest window in the
;       approprite byte for the stobe. R7 contains the number of sucessful reads
;       (again by byte) before the data went bad again. R8 and r9 perform essentally
;       the same function but are the "working rregisters" during the readback
;       process. Whenever the successful count value in a byte of r9 becomes larger
;       than the sucessful count value in r7, the data for that byte is copied from
;       r8 into r6 and from r9 into r7. Thus, at the end of the calibration, each
;       byte in r6 will have the starting dealy count for the coresponding strobe,
;       and r7 will contain the width of the dealy in successful reads. The delay
;       to the center of the data eye is the value in each r6 bye added to the
;       count in r7 if the dealys are incremented by two, or is r7 divided by
;       two if the delay increment is 1 on sucessive reads.
;
;       r2 contins expected test pattern.
;       r3 contains the data that is read back (below)
;       r4, r5 are temp use registers
;       r6 saves (in position) the dealy start value of the largest data eye
;       r7 saves (in position) the number of successful reads for the largest data eye
;       r8 saves the delay time from the latest INVALID read (by byte)
;       r9 saves the count of the number of successful reads
;
xlli_sr ldr     r3,     [r1]                            ; Read test pattern data
;
;       ************************
;       ***** BYTE 0 TESTS *****
;       ************************
;
        and     r4,     r2,     #0xFF   ; r4 = expected test pattern for byte 0
        and     r5,     r3,     #0xFF   ; r5 = returned test pattern for byte 0
        cmp     r4,     r5              ; Does the byte data match?
        beq     xlli_s0s                ; Yes - skip over the following
;
;       Update r8 with this delay time and zero out coresponding byte in r9
;
        ldr     r4,     [r0, #xlli_DDR_SCAL_offset]     ; Read SCAL register
        and     r4,     r4,     #0x7F                   ; Get the delay value
        bic     r8,     r8,     #0x7F                   ; Clear the previous data
        orr     r8,     r8,     r4                      ; Save this delay value
        bic     r9,     r9,     #0xFF                   ; Zero out the count in r9
        b       xlli_s1                                 ; Test next byte
;
;       Increment count in r9 and copy r8/r9 data to r6/r7 if count in r9
;       exceeds the count in r7.
;
xlli_s0s
        add     r9,     r9,     #1      ; Increment count in r9
        and     r4,     r7,     #0xFF   ; Get byte 0 count in r7 (largest data eye size)
        and     r5,     r9,     #0xFF   ; Get byte 0 count in r9 (working register)
        cmp     r4,     r5              ; Is this data eye larger than the saved one?
        bpl     xlli_s1                 ; No - test next byte
;
;       A larger data eye has been found - copy r8/r9 data into r6/r7
;
        bic     r6,     r6,     #0xFF   ; Clear data in r6
        bic     r7,     r7,     #0xFF   ; Clear data in r7
        and     r4,     r8,     #0xFF   ; Extract r8 data into r4
        and     r5,     r9,     #0xFF   ; Extract r9 data into r5
        orr     r6,     r6,     r8      ; Copy r8 data into r6
        orr     r7,     r7,     r9      ; Copy r9 data into r7
;
;       ************************
;       ***** BYTE 1 TESTS *****
;       ************************
;
xlli_s1 and     r4,     r2,     #0xFF00 ; r4 = expected test pattern for byte 0
        and     r5,     r3,     #0xFF00 ; r5 = returned test pattern for byte 0
        cmp     r4,     r5              ; Does the byte data match?
        beq     xlli_s1s                ; Yes - skip over the following
;
;       Update r8 with this delay time and zero out coresponding byte in r9
;
        ldr     r4,     [r0, #xlli_DDR_DLS_offset]      ; Read SCAL register
        and     r4,     r4,     #0x7F00                 ; Get the delay value
        bic     r8,     r8,     #0x7F00                 ; Clear the previous data
        orr     r8,     r8,     r4                      ; Save this delay value
        bic     r9,     r9,     #0xFF00                 ; Zero out the count in r9
        b       xlli_s2                                 ; Test next byte
;
;       Increment count in r9 and copy r8/r9 data to r6/r7 if count in r9
;       exceeds the count in r7.
;
xlli_s1s
        add     r9,     r9,     #0x100  ; Increment count in r9
        and     r4,     r7,     #0xFF00 ; Get byte 0 count in r7 (largest data eye size)
        and     r5,     r9,     #0xFF00 ; Get byte 0 count in r9 (working register)
        cmp     r4,     r5              ; Is this data eye larger than the saved one?
        bpl     xlli_s2                 ; No - test next byte
;
;       A larger data eye has been found - copy r8/r9 data into r6/r7
;
        bic     r6,     r6,     #0xFF00 ; Clear data in r6
        bic     r7,     r7,     #0xFF00 ; Clear data in r7
        and     r4,     r8,     #0xFF00 ; Extract r8 data into r4
        and     r5,     r9,     #0xFF00 ; Extract r9 data into r5
        orr     r6,     r6,     r8      ; Copy r8 data into r6
        orr     r7,     r7,     r9      ; Copy r9 data into r7
;
;       ************************
;       ***** BYTE 2 TESTS *****
;       ************************
;
xlli_s2 and     r4,     r2,     #0xFF0000       ; r4 = expected test pattern for byte 0
        and     r5,     r3,     #0xFF0000       ; r5 = returned test pattern for byte 0
        cmp     r4,     r5                      ; Does the byte data match?
        beq     xlli_s2s                        ; Yes - skip over the following
;
;       Update r8 with this delay time and zero out coresponding byte in r9
;
        ldr     r4,     [r0, #xlli_DDR_DLS_offset]      ; Read SCAL register
        and     r4,     r4,     #0x7F0000               ; Get the delay value
        bic     r8,     r8,     #0x7F0000               ; Clear the previous data
        orr     r8,     r8,     r4                      ; Save this delay value
        bic     r9,     r9,     #0xFF0000               ; Zero out the count in r9
        b       xlli_s3                                 ; Test next byte
;
;       Increment count in r9 and copy r8/r9 data to r6/r7 if count in r9
;       exceeds the count in r7.
;
xlli_s2s
        add     r9,     r9,     #0x10000         ; Increment count in r9
        and     r4,     r7,     #0xFF0000       ; Get byte 0 count in r7 (largest data eye size)
        and     r5,     r9,     #0xFF0000       ; Get byte 0 count in r9 (working register)
        cmp     r4,     r5                      ; Is this data eye larger than the saved one?
        bpl     xlli_s3                         ; No - test next byte
;
;       A larger data eye has been found - copy r8/r9 data into r6/r7
;
        bic     r6,     r6,     #0xFF0000       ; Clear data in r6
        bic     r7,     r7,     #0xFF0000       ; Clear data in r7
        and     r4,     r8,     #0xFF0000       ; Extract r8 data into r4
        and     r5,     r9,     #0xFF0000       ; Extract r9 data into r5
        orr     r6,     r6,     r8              ; Copy r8 data into r6
        orr     r7,     r7,     r9              ; Copy r9 data into r7
;
;       ************************
;       ***** BYTE 3 TESTS *****
;       ************************
;
xlli_s3 and     r4,     r2,     #0xFF000000     ; r4 = expected test pattern for byte 0
        and     r5,     r3,     #0xFF000000     ; r5 = returned test pattern for byte 0
        cmp     r4,     r5                      ; Does the byte data match?
        beq     xlli_s3s                        ; Yes - skip over the following
;
;       Update r8 with this delay time and zero out coresponding byte in r9
;
        ldr     r4,     [r0, #xlli_DDR_DLS_offset]      ; Read SCAL register
        and     r4,     r4,     #0x7F000000             ; Get the delay value
        bic     r8,     r8,     #0x7F000000             ; Clear the previous data
        orr     r8,     r8,     r4                      ; Save this delay value
        bic     r9,     r9,     #0xFF000000             ; Zero out the count in r9
        b       xlli_s4                                 ; Test next byte
;
;       Increment count in r9 and copy r8/r9 data to r6/r7 if count in r9
;       exceeds the count in r7.
;
xlli_s3s
        add     r9,     r9,     #0x1000000       ; Increment count in r9
        and     r4,     r7,     #0xFF000000     ; Get byte 0 count in r7 (largest data eye size)
        and     r5,     r9,     #0xFF000000     ; Get byte 0 count in r9 (working register)
        cmp     r4,     r5                      ; Is this data eye larger than the saved one?
        bpl     xlli_s4                         ; No - test next byte

;       A larger data eye has been found - copy r8/r9 data into r6/r7
;
        bic     r6,     r6,     #0xFF000000     ; Clear data in r6
        bic     r7,     r7,     #0xFF000000     ; Clear data in r7
        and     r4,     r8,     #0xFF000000     ; Extract r8 data into r4
        and     r5,     r9,     #0xFF000000     ; Extract r9 data into r5
        orr     r6,     r6,     r8              ; Copy r8 data into r6
        orr     r7,     r7,     r9              ; Copy r9 data into r7
;
;       Write the inverted test pattern to SDRAM to "pre-charge" the data line
;       to the opposite polarity of the expected data.
;
xlli_s4 mvn     r3,     r2                      ; Invert test pattern in r3
        str     r3,     [r1, #4]                ; Store inverted pattern to precharge data lines
;
;       Check to see if the delay time has been maxed out
;
        ldr     r4,     [r0, #xlli_DDR_DLS_offset]     ; Read SCAL register
        and     r4,     r4,     #0x7E           ; Get the present delay setting
        cmp     r4,     #0x7E                   ; Is the delay time maxed out?
        bne     xlli_sr                         ; Nope, Do another data read
;
;       The data eye has been painted and all delay data has been collected.
;       Locate the center of the data eye and program these values in for the
;       strobe delay times.
;
;       Recall that:
;       r6 saves (in position) the start dealy value for the largest data eye found.
;       r7 saves (in position) the number of successful reads in this data eye.
;
;       Because the XLLI code set up the SCAL register to increment dealy units
;       by a factor of two and r7 contains the number of successful reads, the
;       center of the data eye is found simply by adding r7 to r6.
;
        add     r5,     r6,     r7      ; Generate data for SCAL register
;
;     New 7/16/2006 7:46PM
;
New_tsts
       mov    r3,   r2         ; Keep test pattern
       mov    r2,   #0xFF      ; BYTE 0
       bl     xlli_DDR_SCAL_EVAL
       mov    r2,   #0xFF      ; BYTE 0
       bl     xlli_DDR_SCAL_EVAL       ; For BYTE 0 - twice - because it is possible first begin was wrong
       orr    r6,   r6,   r2   ; Keep in r6 on the BYTE  0 position
       mov    r2,   #0xFF00    ; BYTE 1
       bl     xlli_DDR_SCAL_EVAL
       orr    r6,   r6,   r2   ; Keep in r6 on the BYTE  1 position
       mov    r2,   #0xFF0000   ; BYTE 2
       bl     xlli_DDR_SCAL_EVAL
       orr    r6,   r6,   r2   ; Keep in r6 on the BYTE  2 position
       mov    r2,   #0xFF000000  ; BYTE 3
       bl     xlli_DDR_SCAL_EVAL
       orr    r6,   r6,   r2   ; Keep in r6 on the BYTE  3 position



;
;       Update the DDR_SCAL register
;
        orr     r6,     r6,     #0x80000000             ; Set this bit to update register
        str     r6,     [r0, #xlli_DDR_SCAL_offset]     ; Set up SCAL register

        ldmfd   sp!,    {r2-r9, pc}             ; Return to caller via the stack
        ENDFUNC

        ;*********************************************************************************
;
;       **********************
;       *                    *
;       * xlli_DDR_SCAL_EVAL * Subroutine
;       *                    *
;       **********************
;
; This xlli subroutine performs a part of software calibration for the dynamic
; memory strobe delay lines - perform tumnning line per byte.
;
;
;       INPUT PARAMETERS:
;
;               r0 = Address of dynamic memory control registers base (Physical 0x4810_0000).
;               r1 = Address of 8-byte DDR-SDRAM area to use for strobe delay test.
;               r2 = Mask of BYTE for testing.
;               r3 = Pattern - 4-byte  to use for strobe delay test.

;
;       RETURNED STATUS/VALUE(s):
;
;               r2 = Mask for new value for set in SCAL register
;
;
;
xlli_DDR_SCAL_EVAL  FUNCTION

        stmfd   sp!,    {r3-r12, lr}    ; Save used registers

        ldr     r7,     =0x80808080     ; Mask for unused bits
        mov     r8,     r2              ; Build mask for SCAL register
        bic     r8,     r8,   r7        ; Clear unused bits
        mov     r7,     #0              ; Clear a work register
        mov     r4,     #0              ; Clear a work register
        mov     r6,     #0              ; Clear a work register
                                        ;  Keep OK windows
        mov     r9,     #0              ; Clear a work register - Prev begin
        mov     r10,    #0              ; Clear a work register - Prev end
        mov     r11,    #0              ; Clear a work register - Current begin
        mov     r12,    #0              ; Clear a work register - Current end


four	str     r3,     [r1]            ; Store test pattern
        ldr     r5,     [r1]            ; Get value from DDR
        and     r4,     r3,    r2       ; Expected test pattern for evaluated byte
        and     r5,     r5,    r2       ; Byte from DDR
        cmp     r4,     r5              ; Does the byte data match?
        beq     one                     ; Yes - skip over the following
;
                                        ; NO
        mov     r6,    #0                ; Clear OK flag
        sub     r4,   r10,   r9         ; Last window size
        sub     r7,   r12,   r11        ; New  window size
        cmp     r7,   r4                ; Compare
        blt     two
        mov     r9,   r11               ; Keep last good Begin value
        mov     r10,  r12               ; Keep last good End value
        b       two                      ; Continue
one
        ldr     r4,    [r0,#xlli_DDR_DLS_offset]     ; Read SCAL register
        and     r4,    r4   ,r8         ; Byte rate SSDLV
        mov     r12,   r4               ; Keep last OK value
        cmp     r6,    #0               ; First OK after fault
        bne     two                     ; NO - skip
        mov     r6,    r4               ; Yes - keep current value
        mov     r11,   r4               ; Keep current begin
two
        mvn     r7,     r3              ; Invert test pattern in r4
        str     r7,     [r1, #4]        ; Store inverted pattern to precharge data lines

        cmp     r10,   #0               ; First step ?
        beq     four                     ; Yes - loop
        ldr     r4,    [r0,#xlli_DDR_DLS_offset]     ; Read SCAL register
        and     r4,    r4   ,r8         ; Byte rate SSDLV
;        cmp     r4,    r10              ; Begin once again
;        blt     %B4
        cmp     r9,   r4               ; NO -  End < Begin
        blt     four                     ; NO - loop
                                        ; Yes - exit from loop
                                        ; Test windows sizes
        sub     r4,   r10,   r9         ; Last window size
        sub     r7,   r12,   r11        ; New  window size
        cmp     r7,   r4                ; Compare
        blt     five
        mov     r9,   r11               ; Keep last good Begin value
        mov     r10,  r12               ; Keep last good End value
five
        add     r4,   r10,  r9          ; Average for center of better window
        ldr     r7,     =0x01010101     ; Mask for clear last bit
        and     r7,    r7,   r2         ;
        bic     r4,    r4,   r7
        mov     r4,    r4,  LSR #1      ; Divide by 2
        ldr     r7,    =0x02020202
        and     r7,    r7,   r2         ;
        sub     r4,   r4,   r7          ; Minus previous (losed) values
        mov     r2,   r4
        ldmfd   sp!,    {r3-r12, pc}    ; Return to caller vis the stack
        ENDFUNC
;
; *****************************************************
; **********                                 **********
; ********** HARDWARE DELAY LINE EVALUATION  **********
; **********                                 **********
; *****************************************************
;
; This xlli subroutine performs a hardware calibration for the dynamic
; memory strobe delay lines DQS[3:0].
;
;       NOTES:
;
;       In 16-bit bus mode, all functions corresponding to strobes DQS[3:2] are disabled.
;       DDR_SCAL (SW calibration enable) should be clear if using SDR_DRAM.
;
;       INPUT PARAMETERS:
;
;               r0 = Address of dynamic memory control registers base (Physical 0x4810_0000).
;
;       RETURNED STATUS/VALUE(s):
;
;               NONE
;
;       REGISTERS USED: R0 - r1 (all are preserved)
;

mini_xlli_DDR_HCAL_EVAL   FUNCTION

        stmfd   sp!,    {r0-r7, lr}        ; Save used registers

;;       ldr     r1,     =(xlli_DDR_HCAL_HCEN | xlli_DDR_HCAL_HCPROG | xlli_DDR_HCAL_HCPW | xlli_DDR_HCAL_HCOD)
;;        ldr     r1,     =(xlli_DDR_HCAL_HCEN | xlli_DDR_HCAL_HCPROG | xlli_DDR_HCAL_HCOD)  ;; 7/16/2006 3:25PM

        ldr     r1,     [r0, #xlli_DDR_SCAL_offset]     ; Get  SCAL register
        bic     r1,     r1,     #0x80000000             ; Disable  SW calibration  bit to update register
        str     r1,     [r0, #xlli_DDR_SCAL_offset]     ; Set up SCAL register

				ldr     r7,     =(xlli_DDR_HCAL_HCEN | xlli_DDR_HCAL_HCPROG |  xlli_DDR_HCAL_HCOD)

;				bl		isDDRFreqLessThanZOOMHz
;				cmp		r1, #0x1
;				orreq	r7, r7, #xlli_DDR_HCAL_HCPW
				orr	r7, r7, #xlli_DDR_HCAL_HCPW
    		str     r7,     [r0, #xlli_DDR_HCAL_offset]     ; Load data into DDR_HCAL register

;
;       Wait for hardware calibration to complete
;
one_	ldr     r1,     [r0, #xlli_DMCISR_offset]       ; Get results of the evaluation
        ands    r1,     r1,     #xlli_DMCISR_DLP        ; Extract DLP bit
        beq     one_                                     ; Loop until set
        str     r1,     [r0, #xlli_DMCISR_offset]       ; Clear sticky DLP bit
;
;                         Calculation SW/HW 7/17/2006 1:44PM
;
        mov     r7,     r0                              ; Keep r0
        ldr     r5,     [r0,#xlli_DDR_SCAL_offset]       ; SW Calibration results
        ldr     r6,     [r0,#xlli_DMCISR_offset]         ; HW Calibration results
        and     r1,     r6,  #0x7f                        ; Value of HW
        and     r0,     r5,  #0x7f                       ; BYTE 0  for SW Calibration
        mov     r0,     r0,   LSL #3                    ; Multiple by 8
        bl      xlli_U32Divide                         ; Divide  8*SW/HW
        mov     r3,     r3 ,  LSL #1                   ; Remainder *2
        cmp     r3,     r1                             ; Remainder is Greater than Divider/2
        blt     two_                                     ; NO - skip
        add     r2,   r2,   #1                         ; Round result if need
two_	and     r2,   r2,   #0x0f                      ; 4 - bits of result
        mov     r4,   r2,   LSL #6                     ;In Right position for BYTE 0 in r4
        and     r0,     r5, #0x7F00                     ; BYTE 1  for SW Calibration
        mov     r0,     r0,   LSR #5                    ; Move to Byte 0 and Multiple by 8
        bl      xlli_U32Divide                         ; Divide  8*SW/HW
        mov     r3,     r3 ,  LSL #1                   ; Remainder *2
        cmp     r3,     r1                             ; Remainder is Greater than Divider/2
        blt     three_                                     ; NO - skip
        add     r2,   r2,   #1                         ; Round result if need
three_	and     r2,   r2,   #0x0f                      ; 4 - bits of result
        mov     r2,   r2,   LSL #10                     ;In Right position for BYTE 1 in r2
        orr     r4,   r4,   r2                         ; Set in r4
        and     r0,     r5,# 0x7F0000                  ; BYTE 2  for SW Calibration
        mov     r0,     r0,   LSR #13                   ; Move to Byte 0 and Multiple by 8
        bl      xlli_U32Divide                         ; Divide  8*SW/HW
        mov     r3,     r3 ,  LSL #1                   ; Remainder *2
        cmp     r3,     r1                             ; Remainder is Greater than Divider/2
        blt     four_                                     ; NO - skip
        add     r2,   r2,   #1                         ; Round result if need
four_   and     r2,   r2,   #0x0f                      ; 4 - bits of result
        mov     r2,   r2,   LSL #14                     ;In Right position for BYTE 2 in r2
        orr     r4,   r4,   r2                         ; Set in r4
        and     r0,     r5,  #0x7F000000                ; BYTE 3  for SW Calibration
        mov     r0,     r0,   LSR #21                   ; Move to Byte 0 and Multiple by 8
        bl      xlli_U32Divide                         ; Divide  8*SW/HW
        mov     r3,     r3 ,  LSL #1                   ; Remainder *2
        cmp     r3,     r1                             ; Remainder is Greater than Divider/2
        blt     five_                                     ; NO - skip
        add     r2,   r2,   #1                         ; Round result if need
five_   and     r2,   r2,   #0x0f                      ; 4 - bits of result
        mov     r2,   r2,   LSL #18                     ;In Right position for BYTE 2 in r2
        orr     r4,   r4,   r2                         ; Set in r4
        ldr     r6,     [r7,#xlli_DDR_HCAL_offset]       ; Get HW Calibration register
        bic     r6,   r6,  #0x00fc0                    ; Clear position of HCOFF0..3
        bic     r6,   r6,  #0x3f000                    ; Clear position of HCOFF0..3
        bic     r6,   r6,  #0x0001f                    ; Clear position of HCRNG

 IF :DEF: PV
        orr     r6,   r6,  r4                          ; Set new values of  HCOFF0..3
 ENDIF
        orr     r6,   r6,  #2                          ; Ser range of   HCRNG  to 2
        str     r6,   [r7,#xlli_DDR_HCAL_offset]       ; Set HW Calibration register

 IF :DEF: PV
        ldr     r6,   [r7,#xlli_DDR_SCAL_offset]       ; Read SW Calibration value
        orr     r6,   r6,	#0x80000000						 			 ; Enable Hybrid Calibration (both HCAL and SCAL registers enabled)
        str     r6,   [r7,#xlli_DDR_SCAL_offset]       ; Set SW Calibration register
 ENDIF


        ldmfd   sp!,    {r0-r7, pc}        ; Return to caller
        ENDFUNC

;
; This Routine returns TRUE if DDR frequency is lower than 200MHz
; It uses r1 as its return register!
;
isDDRFreqLessThanZOOMHz
        stmfd   sp!,    {r0,r2}        ; Save used registers

				ldr     r1,     =xlli_CLKREGS_PHYSICAL_BASE         ; Get clocks register base
				ldr 		r0,			[r1, #xlli_ACSR_offset]  		;read ACSR - check what is the DDR frequency
				ldr 		r2,			=~(xlli_BIT_12 | xlli_BIT_13)
				bic 		r0,			r0,	r2
				mov			r1,			#0x1
				cmp 		r0,			#0x2000             ;is it greater than b10?
				blt			lessThan
				mov			r1,			#0x0

lessThan
				ldmfd   sp!, 		{r0,r2}
				BX      lr

        END
