/******************************************************************************
**
**  COPYRIGHT (C) 2002, 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:   Flash.c
**
**  PURPOSE:    Contain template OEM boot code flash operations
**
******************************************************************************/

#include "flash.h"
#include "nand.h"
#include "general.h"
#include "xllp_dfc.h"

NAND_Properties_T NAND_Prop;    // Only need one

P_NAND_Properties_T GetNANDProperties(void)
{
    return &NAND_Prop;
}

/*
 * Function initializes NAND device and fills out Flash Properties Struct
 */

UINT_T InitializeNANDDevice(UINT8_T FlashNum, FlashBootType_T FlashBootType)
{
    UINT_T Retval = NoError;
    P_NAND_Properties_T pNandP = GetNANDProperties();
    P_FlashProperties_T pFlashP = GetFlashProperties(FlashBootType);

    pNandP->FlashBusWidth = (FlashNum == 0x4) ? FlashBusWidth16 : FlashBusWidth8;

    Retval = XllpDfcInit(pNandP->FlashBusWidth, pNandP);

    if (Retval != NoError)
        return DFCInitFailed;

    //define functions
    pFlashP->ReadFromFlash = &ReadNAND;
    pFlashP->WriteToFlash = &WriteNANDWithBBM;
    pFlashP->EraseFlash =  &EraseNANDWithBBM;
    pFlashP->ResetFlash = &ResetNAND;
    pFlashP->BlockSize = pNandP->BlockSize;
    pFlashP->PageSize = pNandP->PageSize;
    pFlashP->UseBBM = 1;
    pFlashP->bUseSpareArea = 0;
    pFlashP->SASize = 0;

    pFlashP->NumBlocks = pNandP->NumOfBlocks;

    // init the TIM load address
    //---------------------------------------
    pFlashP->TimLoadAddress = TIMOffset_NAND;
    return Retval;
}

/*
 This routine will read in data from the DFC.
*/
UINT_T ReadNAND (UINT_T Address, UINT_T Buffer, UINT_T ReadAmount, FlashBootType_T FlashBootType)
{
    return xdfc_read((UINT_T *)Buffer, Address, ReadAmount, NULL, GetNANDProperties());
}

/* 
	WriteNANDWithBBM - This function is a wrapper to work with DFC
 */

UINT_T WriteNANDWithBBM (UINT_T flash_addr, UINT_T source, UINT_T WriteAmount, FlashBootType_T FlashBootType)
{
    UINT_T Retval = NoError;
    P_NAND_Properties_T pNandP = GetNANDProperties();
    P_FlashProperties_T pFlashP = GetFlashProperties(FlashBootType);
    Retval = xdfc_write((UINT_T *) source, flash_addr, WriteAmount, pFlashP->bUseSpareArea, pFlashP->SASize, pNandP);
    return Retval;  // Return value
}

/*
 *
 *  Erases one block at a time
 *      note:  if user inputs determine that a partial block should be erased,
 *              this function will erase that WHOLE block: no partial blocks allowed
 *
 */
UINT_T EraseNANDWithBBM (UINT_T flashoffset, UINT_T size, FlashBootType_T fbt)
{
    return xdfc_erase(flashoffset, GetNANDProperties());
}

//Wrapper to link the dfc reset routine to the flash API
UINT_T ResetNAND(FlashBootType_T fbt)
{
	return xdfc_reset(GetNANDProperties());
}


