/******************************************************************************
**  (C) Copyright 2007 Marvell International Ltd.  
**  All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2002, 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:   DKB.c
**
**  PURPOSE:    DKB main routine
**
**  History:    Initial Creation 5/16/05
******************************************************************************/

#include "flash.h"
#include "protocolmanager.h"
#include "platform_pxa_font.h"
#include "interrupts.h"
#if TRUSTED
    #include "dkb_security.h"
#endif
#include "i2c.h"
#include "tim.h"
#include "platform_setup.h"
#include "downloader.h"
#include "misc.h"
#include "FlasherBinary.h"
#ifdef FBW_CODE
#include "general.h"
#endif
#include "RB_Check.h"


volatile UINT8_T is_yaffs = 0;
volatile UINT8_T tim_flash = 0;
volatile UINT32_T volume_yaffs = 0;
volatile UINT8_T block0_reset = 0;
typedef struct 
{
    int iFromOffset;    //Offset from the file position 0       
    char bDownloadToFlash;//0 Not to flash,1 to Flash
    int iFileSize;
    int  iFlashAddress;
    char bYaffsFile;    //Only for User&System Yaffs
    int iYaffsSize; //For Yaffs Item use only
    char szFileName[256];
}STPreTIMItem,*PSTPreTIMItem;

struct stHead
{
    char magic[4];
    int  items;
    #define BITMES_MAX_NUMS 20
    STPreTIMItem bitmes[BITMES_MAX_NUMS];
}__attribute__((packed));

typedef  struct  stHead STHead;
static STHead *phead;
static STPreTIMItem *ptime;
//static STHead head;
void fota_reboot(void);

#define pretim_download() \
{ \
	AddMessage((UINT8_T*) ("*** PreTIM ***\0")); \
    do \
    { \
     FS_Retval = HandleRequest((UINT_T)(PRETIMLOADADDRESS+4), PRETIMIDENTIFIER, NULL); \
     if (FS_Retval.ErrorCode != NoError) \
            AddMessage((UINT8_T*) ("*** Download of PreTIM failed...retrying\0")); \
    } \
    while (FS_Retval.ErrorCode != NoError); \
     \
    phead = (STHead*)(PRETIMLOADADDRESS); \
    if(phead->items >= 9)block0_reset = 1; \
    else block0_reset = 0; \
}


inline void pretim_watchdog_start(TIM *pTIM_h)
{
    if( 1 || (pTIM_h->pImg[1].ImageID != OBMIDENTIFIER) )
    {
        /*pick up from blob/src/lib/reboot-mhn.c*/
        #define OSMR3		*(volatile UINT32_T *)(0x40A0000C)
        #define OSCR		*(volatile UINT32_T *)(0x40A00010)
        #define OWER		*(volatile UINT32_T *)(0x40A00018)  /* OS Timer Watchdog Enable Register */
        #define OSSR		*(volatile UINT32_T *)(0x40A00014)  /* OS Timer Status Register */
        #define OWER_WME	(1 << 0)	/* Watchdog Match Enable */
        #define OSSR_M3		(1 << 3)	/* Match status channel 3 */
        OWER = OWER_WME;
        OSSR = OSSR_M3;
        OSMR3 = OSCR + 368640;	/* ... in 100 ms */
        for(;;);
    }
}
// Routines defined outside
extern void   TransferControl(UINT_T, UINT_T, UINT_T, UINT_T);
void CheckForErrors(UINT_T Code);
extern UINT_T SetUpUSBDescriptors (pTIM pTIM_h, UINT_T TIM_Use, UINT_T);

UINT_T DownloadTIMImages( TIM *pTIM_h, pFUSE_SET pFuses );
UINT_T DownloadFBFImage( TIM *pTIM_h, pFUSE_SET pFuses );
UINT_T SetTIMPointers( UINT8_T * StartAddr, TIM *pTIM_h );
UINT_T LoadTim( UINT8_T *TIMArea, TIM *pTIM_h );
UINT CalcImageChecksum( UINT_T* DownloadArea, UINT_T ImageLength, UINT_T ImageType, UINT_T PageSize, UINT_T SpareAreaSize);
UINT_T CopyImageToFlash( UINT_T* DownloadArea, UINT_T FlashStartAddr, UINT_T Length, UINT_T PageSize, UINT_T SpareAreaSize);

//#define VERIFY_WRITE
#ifdef VERIFY_WRITE
UINT_T VerifyWrite(UINT_T* DownloadArea, UINT_T FlashStartAddress, UINT_T PageSize, UINT_T SpareAreaSize, UINT_T ImageLength);
#endif

// Reference
void _mod0(void) {while (1);}

void DKBBootMain(UINT_T PlatformSettings)
{
    UINT_T   Retval = NoError;
    FUNC_STATUS FS_Retval;
    UINT8_T OBMOrder,i;                 // Keeps track of which order the OBM is in
    FUSE_SET  Fuses;                    // Fuse information passed in from BootROM
    UINT8_T PlatformInitialized = 0;    // Keep track if the platform has been initialized or not

    // TIM
    TIM TIM_h;                          // Our struct to work our way around the TIM
    UINT_T ImageRequested = TIMIDENTIFIER;

    unsigned int usb_default_port;
    UINT_T KeyDataC[]    = {0xF07DE2F1, 0x78312A1F, 0xBB7AEE8C, 0x776DB5C1, 0x94D1022B};

    // Initialize any fuse information passed in by the BootROM
    Fuses.value = PlatformSettings;

    #if DKBTEST
     PlatformSettings = 0x1;
     Fuses.value = 0;
     Fuses.bits.USBPort = 0;            // Default to Differential
     Fuses.bits.UARTPort = 0;           // Default to FFUART
     Fuses.bits.PlatformState = 0x2;    // Override with x16 NAND
    #endif


    InitMessageQueue();

    // Initialize the ports
    EnableIrqInterrupts();
    //Retval = InitPort(Fuses.bits.UARTPort, &Fuses);
    Retval = InitPort(FFUART_D, &Fuses);
#ifdef FBW_CODE
    //borqs_uart_open();
#endif

#if 0
    //read the GPIO's to determine port configuration
    getPlatformPortSelection(&usb_default_port);
    Retval = PlatformPreFlashSetup();
    SetUpUSBDescriptors (NULL, 0, usb_default_port);
    InitPort(usb_default_port, &Fuses);
    AddMessage((UINT8_T*) ("*** DKB ***\0"));
	pretim_download();
#endif

    // Print out who we are
    AddMessage((UINT8_T*) ("*** DKB ***\0"));
    if( Retval != NoError)
        AddMessage((UINT8_T*)("*** Could not raise voltage\0"));

    LCD_RECT rect;
#if 0
    rect.x0 = 0;
    rect.x1 = 320-1;
    rect.y0 = 0;
    rect.y1 = 480-1;
    lcd_fill_rect(&rect,FONT_COLOR_BLACK);

    rect.x0 = 50;
    rect.x1 = 320-50;
    rect.y0 = 50;
    rect.y1 = 480-100;
    lcd_draw_rect(&rect,FONT_COLOR_GREEN);
    rect.x0 = 50+1;
    rect.x1 = 320-50-1;
    rect.y0 = 50+1;
    rect.y1 = 480-100-1;
    lcd_draw_rect(&rect,FONT_COLOR_BLUE);


    rect.x0 += 2;
    rect.x1 -= 1;
    rect.y0 += 2;
    rect.y1 -= 2;
    lcd_fill_rect(&rect,FONT_COLOR_GREEN);
#endif
    //lcd_printf("FOTA processing..",rect.x0+8,200,2,FONT_COLOR_RED,FONT_COLOR_GREEN,LCD_DRAWMODE_NORMAL);
    lcd_printf("FOTA processing..",rect.x0+8,200,2,FONT_COLOR_RED,FONT_COLOR_BLACK,LCD_DRAWMODE_NORMAL);
    //fota_reboot();


    if (Fuses.bits.PlatformState > 0)
    {
        PlatformInitialized = 1;
        AddMessage((UINT8_T*)("*** Platform Fuses already initialized\0"));
    }

#if 0
    // Download TIM/NTIM or FBF MasterBlockHeader, can't proceed otherwise
    do
    {
        // request the TIM first as originally implemented
        ImageRequested = TIMIDENTIFIER;
        FS_Retval = HandleRequest((UINT_T)TIMLOADADDRESS, ImageRequested, NULL); //, Interrupt_Off_Flag);
        if (FS_Retval.ErrorCode == NoError)
            break;

//        AddMessage((UINT8_T*) ("*** Download TIM failed...try to download FBF instead\0"));

        ImageRequested = FBFIDENTIFIER;
        // since request for TIM failed, let's see if there is an FBF image to download
        FS_Retval = HandleRequest((UINT_T)MASTER_BLOCK_HEADER_ADDR, ImageRequested, NULL); //, Interrupt_Off_Flag);
        if (FS_Retval.ErrorCode == NoError)
            break;

        AddMessage((UINT8_T*) ("*** Retrying download request...\0"));
    }
    while (FS_Retval.ErrorCode != NoError);

#if TRUSTED
    // Enable Caddo
    Retval = EnableDisableCaddo(ENABLE_CADDO);       // Enable the Caddo Unit
    Retval |= WtmReset(1);
#endif

    // Fill in TIM pointers
    SetTIMPointers( (UINT8_T *)TIMLOADADDRESS, &TIM_h );

    // check image type last requested and proceed to download image(s)
    if ( ImageRequested == FBFIDENTIFIER )
    {
        Retval = DownloadFBFImage( &TIM_h, &Fuses );
    }
    else
        // else continue processing TIM
        Retval = DownloadTIMImages( &TIM_h, &Fuses );
#endif
    int FlashNumber;
    SetTIMPointers( (UINT8_T *)TIMLOADADDRESS, &TIM_h );
    FlashNumber = (UINT8_T)(TIM_h.pConsTIM->FlashInfo.BootFlashSign) & 0xFF;
    Configure_Flashes(FlashNumber, BOOT_FLASH);
    AddMessage("Configure flashes success");


#if 0
    char *buf = (char *)0x80100000, *buf2 = (char *)0x80101000, k[] = "XXXXX!!!!!!!!!!!!!!!!!11Hello", *p, m[] = "world";
    int j = 0;
    p = buf;
    while (k[j] != '\0')
    {
        *p = k[j];       
        p++;
        j++;
    }
    *p = '\0';

    AddMessage("buf = ");
    AddMessage(buf);

    EraseFlash(0x0F600000, 2048, BOOT_FLASH);
    WriteFlash(0x0F600000, 0x80100000, 2048, BOOT_FLASH);
    AddMessage("after Write_Flash");
    ReadFlash(0x0F600000, 0x80101000, 2048, BOOT_FLASH);
    AddMessage("after Read_Flash\r\n");
    AddMessage(buf2);

#if 1
    j = 0;
    p = buf;
    while (m[j] != '\0')
    {
        *p = m[j];       
        p++;
        j++;
    }
    *p = '\0';

    EraseFlash(0x0F600000, 2048, BOOT_FLASH);
    WriteFlash(0x0F600000, 0x80100000, 2048, BOOT_FLASH);
    ReadFlash(0x0F600000, 0x80101000, 2048, BOOT_FLASH);
    AddMessage("after second write flash\r\n");
    AddMessage(buf2);
    //AddMessage(buf);
#endif
while(1);
#endif




    
    //clear the FOTA Mode flag from register
    volatile unsigned int *PSPR = (volatile unsigned int *)0x40F50008;  //regiseter address

    char *p = (char *)0x80200000;
    for(i=0; i< 64; i++)
        p[i] = 0;
   
  /* // obm will do the FOTA flag check  
    //check whether only File System update
    AddMessage("Going to check FS update flag.\n");
    //ReadFlash(0x0f400000, p, 2048, BOOT_FLASH); //read first 32 bytes of the first block 
    read_flash_with_align(0x0F400000, p, 32);
    AddMessage("\nFS flag = ");
    AddMessage(p);

    if(p[0] == 'S' && p[1] == 'S' && p[2] == 'S' && p[3] =='S' 
       && p[4] == '5' && p[5] == 'B' && p[6] == '5' && p[7] =='B')
    {// only file system update
        *PSPR = 3; //reboot to recovery mode
        fota_reboot();
    }
  */
    AddMessage("!!!!!!!!!!!Going to do raw image update!!!!!!!!!!.\n");
    Retval = FOTA_Main();
    AddMessage("!!!!!!!!!!!FOTA raw image update finish!!!!!!!!!!.\n");

/* //obm will protect this
    if(Retval == 0) //FOTA success, going to check File System update flag
    {
       char *buf = (char *)0x80200000;
   
       //ReadFlash(0x0f400000, buf, 2048, BOOT_FLASH); //read first 32 bytes of the first block 
       read_flash_with_align(0x0F400000, buf, 32);
       AddMessage("FS update flag = ");
       AddMessage(buf);
       if(buf[4] == '5' && buf[5] == 'B' && buf[6] == '5' && buf[7] =='B')
           *PSPR = 3; //reboot to recovery mode
       else
           *PSPR = 0; //reboot to normal mode
    }
*/     
    *PSPR = 0; //reboot to normal mode
    fota_reboot();
/*
#if 1
    char *buf = (char *)0x80100000, *buf2 = (char *)0x80101000, k[] = "XXXXX!!!!!!!!!!!!!!!!!11Hello", *p, m[] = "world";
    int j = 0;
    p = buf;
    while (k[j] != '\0')
    {
        *p = k[j];       
        p++;
        j++;
    }
    *p = '\0';

    AddMessage("buf = ");
    AddMessage(buf);

    EraseFlash(0x0F600000, 2048, BOOT_FLASH);
    WriteFlash(0x0F600000, 0x80100000, 2048, BOOT_FLASH);
    ReadFlash(0x0F600000, 0x80101000, 2048, BOOT_FLASH);
    AddMessage("after write flash");
    AddMessage(buf2);

#if 1
    j = 0;
    p = buf;
    while (m[j] != '\0')
    {
        *p = m[j];       
        p++;
        j++;
    }
    *p = '\0';

    EraseFlash(0x0F600000, 2048, BOOT_FLASH);
    WriteFlash(0x0F600000, 0x80100000, 2048, BOOT_FLASH);
    ReadFlash(0x0F600000, 0x80101000, 2048, BOOT_FLASH);
    AddMessage("after second write flash");
    AddMessage(buf2);
    AddMessage(buf);
#endif

#endif
*/




    while (1);
    // Check for Errors
    CheckForErrors(Retval);

#if COPYIMAGESTOFLASH
    if( tim_flash || (ImageRequested == FBFIDENTIFIER) )
    {
        // erase the area of flash where the image will be burned
        AddMessageError(PlatformBusy);
        Retval = EraseFlash(0, MAX_TIM_SIZE, BOOT_FLASH);
        CheckForErrors(Retval);

        // Copy the Image
        AddMessage((UINT8_T*) ("*** Copy Image to Flash...\0"));
        Retval = WriteFlash(0, TIMLOADADDRESS, MAX_TIM_SIZE, BOOT_FLASH);
        CheckForErrors(Retval);
        AddMessageError(PlatformReady);
    }
#endif

#if TRUSTED
     Retval = FinalizeSecurity (&TIM_h, PlatformSettings, PlatformInitialized);
     CheckForErrors(Retval);
#endif

    Retval = Finalize_Flashes(BOOT_FLASH);
    CheckForErrors(Retval);

#if TRUSTED
     Retval = Finalize_Flashes(SAVE_STATE_FLASH);
    CheckForErrors(Retval);
#endif

    // Ensure that the WTPTP tool is ready to shutdown
    AddMessageError(PlatformReady);
    // TRANSFER CONTROL TO OBM
    AddMessage((UINT8_T*) ("*** XFer Control To OBM...\0"));
    HandleDisconnect();
#ifdef FBW_CODE
    borqs_uart_close();
#endif
    ShutdownPort(FFUART_D);
    ShutdownPort(ALTUART_D);
    ShutdownPort(DIFF_USB_D);
    ShutdownPort(U2D_USB_D);
    DisableIrqInterrupts();

    {
        typedef unsigned int u32;
        #define io_p2v(x)   (x)
        # define __REG(x)	(*((volatile u32 *)io_p2v(x)))
        #define PMCR		__REG(0x40F50000)
        #define PMCR_BIE       (1 << 0)        /* Interrupt Enable for nBATT_FAULT */
        #define PMCR_BIS       (1 << 1)        /* Interrupt Status for nBATT_FAULT */
        #define PMCR_TIE       (1 << 10)       /* Interrupt Enable for XScale Core Frequency Change */
        #define PMCR_TIS       (1 << 11)       /* Interrupt Status for XScale Core Frequency Change */
        #define PMCR_VIE       (1 << 12)       /* Interrupt Enable for VCC_APPS and VCC_SRAM Voltage Change */
        #define PMCR_VIS       (1 << 13)       /* Interrupt Status for VCC_APPS and VCC_SRAM Voltage Change */
        #define PMCR_SWGR      (1 << 31)       /* Software GPIO Reset */


        #define REBOOT_SETTING_ADDR	(0x5c01c000)
        *((UINT_T *)(REBOOT_SETTING_ADDR)) = 0xfd;

        PMCR = (PMCR & (PMCR_BIE | PMCR_TIE | PMCR_VIE))
                                | PMCR_SWGR;
        for(;;);

    }
    //TransferControl(TIM_h.pImg[1].LoadAddr, TIM_h.pImg[1].ImageSize,DDR_DKB_IMAGE_DOWNLOAD, Fuses.value);
    //pretim_watchdog_start(&TIM_h);
}

void fota_reboot(void)
{
    typedef unsigned int u32;
#define io_p2v(x)   (x)
# define __REG(x)	(*((volatile u32 *)io_p2v(x)))
#define PMCR		__REG(0x40F50000)
#define PMCR_BIE       (1 << 0)        /* Interrupt Enable for nBATT_FAULT */
#define PMCR_BIS       (1 << 1)        /* Interrupt Status for nBATT_FAULT */
#define PMCR_TIE       (1 << 10)       /* Interrupt Enable for XScale Core Frequency Change */
#define PMCR_TIS       (1 << 11)       /* Interrupt Status for XScale Core Frequency Change */
#define PMCR_VIE       (1 << 12)       /* Interrupt Enable for VCC_APPS and VCC_SRAM Voltage Change */
#define PMCR_VIS       (1 << 13)       /* Interrupt Status for VCC_APPS and VCC_SRAM Voltage Change */
#define PMCR_SWGR      (1 << 31)       /* Software GPIO Reset */


#define REBOOT_SETTING_ADDR	(0x5c01c000)
    *((UINT_T *)(REBOOT_SETTING_ADDR)) = 0xfd;

    PMCR = (PMCR & (PMCR_BIE | PMCR_TIE | PMCR_VIE))
        | PMCR_SWGR;
    for(;;);

}



extern volatile UINT8_T block0_reset;
UINT_T DownloadTIMImages( TIM *pTIM_h, FUSE_SET *pFuses )
{
    UINT_T   Retval = NoError;
    FUNC_STATUS FS_Retval;
    UINT8_T ImageOrder = 1;             // Keep Track of Image Order, First Image is TIM->OBM->AnyImage
    UINT_T j = 0;
    UINT8_T FlashNumber = 0;
    P_FlashProperties_T pFlashProps = GetFlashProperties( BOOT_FLASH );

    // Validate TIM
    AddMessage((UINT8_T*) ("*** Checking TIM...\0"));
#if TRUSTED
    // Enable Caddo
    Retval = ValidateTIM(pTIM_h);
#endif
    if (Retval != NoError)
    {
        AddMessage((UINT8_T*) ("*** TIM validation failed\0"));
        CheckForErrors(Retval);
    }

    FlashNumber = (UINT8_T)(pTIM_h->pConsTIM->FlashInfo.BootFlashSign) & 0xFF;
    if((pFuses->bits.PlatformState != 0x0) && (FlashNumber != pFuses->bits.PlatformState))
    {
        AddMessage((UINT8_T*) ("*** ERROR: Platform Fuses do NOT match flash settings in TIM!\0"));
        AddMessage((UINT8_T*) ("*** ERROR: Initialize halted here!\0"));
        return InvalidPlatformConfigError;
    }

    // Configure Flashes Based on TIM
    AddMessage((UINT8_T*) ("*** Preparing Flash for first use (may take several minutes)...\0"));

    // Configure_Flashes will create a BBT if one is needed and does not exist
    AddMessageError(PlatformBusy);
    Retval = Configure_Flashes( FlashNumber, BOOT_FLASH );
    CheckForErrors(Retval);
    AddMessageError(PlatformReady);

    //added:    We want to download the images in reverse order
    //          This will make sure we download the OBM (or Boot Image) last
    //          Which means a copy of this image will be in the scratch area, and we can
    //           move it to it's load address without having to read it back from flash

    ImageOrder = pTIM_h->pConsTIM->NumImages - 1;

    ptime = phead->bitmes + ImageOrder;
    // For each image, download, validate and copy
    while (ImageOrder > 0)
    {
        // Download the image
        AddMessage((UINT8_T*) ("*** Downloading Next Image...\0"));
#ifdef FBW_CODE
        borqs_printf(ptime->szFileName);
#endif
        FS_Retval = HandleRequest(DDR_DKB_IMAGE_DOWNLOAD, pTIM_h->pImg[ImageOrder].ImageID, NULL);
        AddMessage((UINT8_T*) ("*** Image Download Complete\0"));
        Retval = FS_Retval.ErrorCode;
        CheckForErrors(Retval);

        // Validate the Image
        AddMessage((UINT8_T*) ("*** Validating Image...\0"));
#if TRUSTED
        Retval = ValidateImage(pTIM_h->pImg[ImageOrder].ImageID, NULL, pTIM_h);
#endif
        CheckForErrors(Retval);
        AddMessage((UINT8_T*) ("*** Image Validation Successful\0"));

#if COPYIMAGESTOFLASH
        #if 0
        // erase the area of flash where the image will be burned
        AddMessageError(PlatformBusy);
        Retval = EraseFlash(pTIM_h->pImg[ImageOrder].FlashEntryAddr, pTIM_h->pImg[ImageOrder].ImageSize, BOOT_FLASH);
        CheckForErrors(Retval);

        // Copy the Image
        AddMessage((UINT8_T*) ("*** Copy Image to Flash...\0"));
        Retval = WriteFlash(pTIM_h->pImg[ImageOrder].FlashEntryAddr, DDR_DKB_IMAGE_DOWNLOAD, pTIM_h->pImg[ImageOrder].ImageSize, BOOT_FLASH);
        CheckForErrors(Retval);
        AddMessageError(PlatformReady);
        #else
        if(is_yaffs = ptime->bYaffsFile)
        {
            volume_yaffs = ptime->iYaffsSize;
            if(volume_yaffs == 0)volume_yaffs = 64*1024*1024;
#ifdef FBW_CODE
            borqs_printf("yaff");
#endif
        }
        if(!block0_reset && !is_yaffs)
        {
            AddMessageError(PlatformBusy);
#ifdef FBW_CODE
            borqs_printf("a");
#endif
            Retval = EraseFlash(pTIM_h->pImg[ImageOrder].FlashEntryAddr, pTIM_h->pImg[ImageOrder].ImageSize, BOOT_FLASH);
            CheckForErrors(Retval);
        }
#ifdef FBW_CODE
        borqs_printf("b");
#endif
        if(ptime->bDownloadToFlash)
        {
            UINT_T total_size;
            UINT_T ddr_buffer;
#ifdef FBW_CODE
            #if !____cache_in_read
            {extern void enable_i_d_cache(void);
                enable_i_d_cache();
            }
           #endif
#else
            {extern void enable_i_d_cache(void);
                enable_i_d_cache();
            }
#endif

            ddr_buffer = DDR_DKB_IMAGE_DOWNLOAD;
            total_size = pTIM_h->pImg[ImageOrder].ImageSize;

            if(pTIM_h->pImg[ImageOrder].ImageID == LOGOIDENTIFIER)
            {
                UINT32_T *p;
                ddr_buffer -= 8;
                p = (UINT32_T*)ddr_buffer;
                p[0] = LOGOIDENTIFIER;
                p[1] = total_size;
                total_size += 8;
            }
#ifdef FBW_CODE
            borqs_printf("writeing");
#endif
            Retval = WriteFlash(pTIM_h->pImg[ImageOrder].FlashEntryAddr, ddr_buffer, total_size, BOOT_FLASH);
		    CheckForErrors(Retval);
            AddMessageError(PlatformReady);
#ifdef FBW_CODE
            #if !____cache_in_read
            {extern void disable_i_d_cache(void);
                disable_i_d_cache();
            }
            #endif
#else
            {extern void disable_i_d_cache(void);
                disable_i_d_cache();
            }
#endif
        }
        #endif
#endif
#ifdef FBW_CODE
        borqs_printf("c");
#endif
        ImageOrder--;
        ptime--;
    }

    if(ptime->bDownloadToFlash)
     tim_flash = 1;
    else 
     tim_flash = 0;

    return Retval;
}


UINT_T DownloadFBFImage( TIM *pTIM_h, FUSE_SET *pFuses )
{
#define DEV_IMAGE_NUMS( dev, img ) (((dev & 0xFFFF)<<16) | (img & 0xFFFF))
#define IMAGE_TYPE(cmds) (cmds>>DLCMD_IMAGE_TYPE_FIELD_BIT) & ((1<<DLCMD_IMAGE_TYPE_FIELD_SIZE_BITS)-1)
#define SPARE_AREA_SIZE(cmds) (cmds>>DLCMD_IMAGE_SPARE_AREA_SZ_BIT) & ((1<<DLCMD_IMAGE_SPARE_AREA_SZ_FIELD_BITS)-1)
#define BLOCK_SIZE 0x20000

    // MasterBlock for FBF file
    MasterBlockHeader* pMasterHeader = (MasterBlockHeader*)MASTER_BLOCK_HEADER_ADDR;
    UINT_T Retval = NoError;
    PDeviceHeader pDevHeader = 0;
    PImageStruct pImage = 0;
    int devnum = 0;
    int imagenum = 0;
    FUNC_STATUS FS_Retval;
    UINT_T DeviceImageID = DEV_IMAGE_NUMS( 0, 0  ); // device 0, image 0;
    pIMAGE_INFO pImageInfo = 0;
    UINT_T lw = 0;
    UINT_T ImageType = 0;
    UINT_T BlockSize = 0;
    UINT_T PageSize = 0;
    UINT_T SpareAreaSize = 0;
    UINT_T ImageChecksum = 0;
    UINT8_T FlashNumber = 0;

    if ( stricmpl( pMasterHeader->Unique, "Marvell_FBF", UNIQUE_SIZE ) !=0 )
        return UnknownImageError;

    // At this point we have already downloaded the MasterBlockHeader
    if ( pMasterHeader->nOfDevices < 1 )
        return UnknownImageError;

    // For now, we will only support one flash device.  In the future we can support more.  To add support for
    // multiple flash devices, it is necessary to change PlatformState to manage the state for each device.
    if ( pMasterHeader->nOfDevices > 1 )
        return UnknownImageError;

    // We can now proceed to download the individual images from the FBF file on the host.
    // This requires coordination between the host and the target both using the MasterBlockHeader
    // to refer to the image info for each download.
    for ( devnum = 0; devnum < pMasterHeader->nOfDevices; devnum++ )
    {
        // perhaps there is more than flash device to program, so we will iterate for all devices
        pDevHeader = (PDeviceHeader)((int)pMasterHeader + pMasterHeader->deviceHeaderOffset[ devnum ] );

        // if we eventually support more than 1 flash device, this will need to change
        switch( pDevHeader->FlashFamily )
        {
            case SIBLEY_FLASH_FAMILY:
            case TYAX_FLASH_FAMILY:
                if ( pDevHeader->AccessWidth == 16 )
                {
                    FlashNumber = 0x7;  // x16 Tyax on CS2
                }
                else
                    Retval = UnknownImageError;
                break;

            case SDRAM_FAMILY:
                Retval = UnknownImageError;
                break;

            case OEM_CAM_BITS_FLASH_FAMILY:
                Retval = UnknownImageError;
                break;

            case RAW_NAND_FLASH_FAMILY:
                if ( pDevHeader->AccessWidth == 8 )
                {
                    FlashNumber = 0x6; // x8 NAND (8 bit) or
                }
                else if ( pDevHeader->AccessWidth == 16 )
                {
                    FlashNumber = 0x4; // x16 NAND (16 bit) or
                }
                else
                    Retval = UnknownImageError;
                break;

            case SAMSUNG_ONENAND:
                FlashNumber = 0x2; // (includes MuxOneNAND and FlexOneNAND)
                break;

            case MDOC_NAND:
                FlashNumber = 0x1;  // MSystems
                break;


            default:
                Retval = UnknownImageError;
        }

        CheckForErrors(Retval);

        if((pFuses->bits.PlatformState != 0x0) && (FlashNumber != pFuses->bits.PlatformState))
        {
            AddMessage((UINT8_T*) ("*** ERROR: Platform Fuses do NOT match flash settings in FBF!\0"));
            AddMessage((UINT8_T*) ("*** ERROR: Initialize halted here!\0"));
            return InvalidPlatformConfigError;
        }

        // Configure Flashes Based on DevHeader->FlashFamily
        AddMessage((UINT8_T*) ("*** Preparing Flash for first use (may take several minutes)...\0"));

        // Configure_Flashes will create a BBT if one is needed and does not exist
        AddMessageError(PlatformBusy);
        Retval = Configure_Flashes( FlashNumber, BOOT_FLASH );
        CheckForErrors(Retval);
        AddMessageError(PlatformReady);

        BlockSize = GetBlockSize( BOOT_FLASH );
        PageSize = GetPageSize( BOOT_FLASH );

        for ( imagenum = 0; imagenum < pDevHeader->nOfImages; imagenum++ )
        {
            pImage = &pDevHeader->imageStruct[ imagenum ];

            ImageType = IMAGE_TYPE(pImage->commands);

            // set up use of spare area bytes according to pImage->commands field
            SpareAreaSize = SPARE_AREA_SIZE(pImage->commands);
            SetUseSpareArea( (SpareAreaSize > 0), BOOT_FLASH );
            SetSpareAreaSize( SpareAreaSize, BOOT_FLASH );

            if ( pImage->commands & DLCMD_DO_ERASE_BLOCKS )
            {
                //
                AddMessageError(PlatformBusy);
                Retval = EraseFlash(pImage->Flash_Start_Address, pImage->length, BOOT_FLASH);
                CheckForErrors(Retval);
                AddMessageError(PlatformReady);
            }

            if ( pImage->commands & DLCMD_WRITE_IMAGE )
            {
                // when downloading images from an FBF file, the DeviceImageID refers to the arrays in the
                // MasterBlockHeader.  The host has a copy.  DeviceImageID indicates the array indicies for
                // the device header array and the image struct array in the MasterBlockHeader.
                // The macro DEVIMAGE_ID takes care of putting the 16-bit fields in the right bit positions.
                DeviceImageID = DEV_IMAGE_NUMS( devnum, imagenum );

                FS_Retval = HandleRequest(MBH_IMAGE_DOWNLOAD_AREA, DeviceImageID, NULL); //, Interrupt_Off_Flag);
                if (FS_Retval.ErrorCode != NoError)
                {
                    AddMessage((UINT8_T*) ("*** Download of image in FBF file failed...\0"));
                    Retval = UnknownImageError;
                    CheckForErrors(Retval);
                }

                AddMessageError(PlatformBusy);
                // verify downloaded checksum matches FBF image header
                ImageChecksum = CalcImageChecksum( (UINT_T*)MBH_IMAGE_DOWNLOAD_AREA, pImage->length, ImageType, PageSize, SpareAreaSize);
                if ( ImageChecksum != pImage->ChecksumFormatVersion2 )
                {
                   AddMessage((UINT8_T*) ("*** Download of image in FBF file failed, checksum error...\0"));
                   Retval = CRCFailedError;
                   CheckForErrors(Retval);
                }

#if COPYIMAGESTOFLASH
                Retval = CopyImageToFlash((UINT_T*)MBH_IMAGE_DOWNLOAD_AREA, pImage->Flash_Start_Address, pImage->length, PageSize, SpareAreaSize);
                CheckForErrors(Retval);
#endif
                AddMessageError(PlatformReady);
            }

#ifdef VERIFY_WRITE  // for debug only as this takes a while and should not be necessary in production code
            if ( (pImage->commands & DLCMD_DO_VERIFY_WRITE) && (pImage->commands & DLCMD_WRITE_IMAGE) )
            {
                AddMessageError(PlatformBusy);
                // Copy the Image to flash
                AddMessage((UINT8_T*) ("*** Verify Image Written to Flash...\0"));
                Retval = VerifyWrite((UINT_T*)MBH_IMAGE_DOWNLOAD_AREA, pImage->Flash_Start_Address, PageSize, SpareAreaSize, pImage->length, fbt);
                CheckForErrors(Retval);
                AddMessageError(PlatformReady);
            }
#endif
        }
    }

    AddMessageError(PlatformBusy);
    // turn off use of spare area since we are done with OS images
    // any write to flash from here on do not use the spare area
    SetUseSpareArea( 0, BOOT_FLASH );
    SetSpareAreaSize( 0, BOOT_FLASH );

    // Copy the Image to flash
    AddMessage((UINT8_T*) ("*** All images written to flash successfully...\0"));
    // add a check for any stray error codes
    CheckForErrors(Retval);

    // load the tim from flash to validate it
    Retval = LoadTim((UINT8_T *)TIMLOADADDRESS, pTIM_h);
    CheckForErrors(Retval);

#if TRUSTED
    Retval = ValidateTIM(pTIM_h);
    CheckForErrors(Retval);
#endif

    pImageInfo = FindImageInTIM(pTIM_h, OBMIDENTIFIER);
    if ( pImageInfo == 0 )
    {
        Retval = ImageNotFound;
        CheckForErrors(Retval);
    }

    Retval = ReadFlash(pImageInfo->FlashEntryAddr, DDR_DKB_IMAGE_DOWNLOAD, pImageInfo->ImageSize, BOOT_FLASH);
    CheckForErrors(Retval);

#if TRUSTED
    Retval = ValidateImage(OBMIDENTIFIER, NULL, pTIM_h);
    CheckForErrors(Retval);
#endif
    AddMessage("Validation Successful!\0");

    AddMessageError(PlatformReady);

    // notify WTPTP that no more image requests will be coming
    FS_Retval = HandleRequest(0, 0xFFFFFFFF, NULL);

    return Retval;
}

void CheckForErrors(UINT_T Code)
{
    // Fatal error occured. Loop forever. BSP can optinally handle this differently.
    if (Code != NoError)
    {
        // Print out why we failed
        AddMessageError(Code);
        HandleDisconnect();
        _mod0();
    }
    return;
}

UINT_T SetTIMPointers( UINT8_T * StartAddr, TIM *pTIM_h )
{
    pTIM_h->pConsTIM = (pCTIM) StartAddr;   // Overlap Constant Part of TIM with actual TIM...
    pTIM_h->pImg = (pIMAGE_INFO) (StartAddr + sizeof (CTIM));
    pTIM_h->pKey = (pKEY_MOD) ((UINT_T)pTIM_h->pImg + ((pTIM_h->pConsTIM->NumImages) * sizeof (IMAGE_INFO)));
    pTIM_h->pReserved = (PUINT) ((UINT_T)pTIM_h->pKey + ((pTIM_h->pConsTIM->NumKeys) * sizeof (KEY_MOD)));
    pTIM_h->pTBTIM_DS = (pPLAT_DS) ((UINT_T)pTIM_h->pReserved + pTIM_h->pConsTIM->SizeOfReserved);
    return (NoError);
}

UINT_T LoadTim( UINT8_T *TIMArea, TIM *pTIM_h )
{
    UINT_T Retval = NoError;
    UINT_T s_addr, d_addr;//source and destination address
    UINT_T TimSize = 0;
    P_FlashProperties_T pFlashProp = GetFlashProperties(BOOT_FLASH);

    // Determine default TIM address
    //read up to (including) the first Image_Info field of the TIM
    //then read the rest of the TIM based on the info read
    // - for XIP, this will let us know whether we can just leave the TIM in flash
    // - for NAND, this will keep us from reading unwritten pages
    TimSize = sizeof(VERSION_I) + sizeof(FLASH_I) + 12 + sizeof(IMAGE_INFO);
    Retval = ReadFlash(pFlashProp->TimLoadAddress, (UINT_T)TIMArea, TimSize, BOOT_FLASH);
    if (Retval != NoError) return Retval;

    Retval = SetTIMPointers(TIMArea, pTIM_h);
    if( pTIM_h->pConsTIM->VersionBind.Identifier != TIMIDENTIFIER )
        Retval = TIMNotFound;
    else
    {
        TimSize = pTIM_h->pImg[0].ImageSize;
        s_addr = pTIM_h->pImg[0].FlashEntryAddr;
        d_addr = pTIM_h->pImg[0].LoadAddr;
        Retval = ReadFlash(s_addr, d_addr, TimSize, BOOT_FLASH);
        if (Retval != NoError)
            return Retval;
        Retval = SetTIMPointers((UINT8_T*)d_addr, pTIM_h);
        if( pTIM_h->pConsTIM->VersionBind.Identifier != TIMIDENTIFIER )
            Retval = TIMNotFound;
    }

    return Retval;
}

UINT_T VerifyWrite(UINT_T* DownloadArea, UINT_T FlashStartAddress, UINT_T PageSize, UINT_T SpareAreaSize, UINT_T ImageLength)
{
    UINT_T Retval = NoError;
    UINT_T BytesRead = 0;
    UINT_T ReadSize = PageSize;
    UINT_T BlockReadBuffer = TEMP_BUFFER_AREA;
    UINT_T FlashReadAddr = FlashStartAddress;

    while ( BytesRead < ImageLength )
    {
        // the last buffer may contain bytes not in the image so limit the number of bytes to compare
        if ( (BytesRead + PageSize) > ImageLength )
            ReadSize = ImageLength - BytesRead;

        // read a full buffer
        Retval = ReadFlash(FlashReadAddr, BlockReadBuffer, ReadSize, BOOT_FLASH);
        CheckForErrors(Retval);

        if ( 0 != memcmp( (char*)BlockReadBuffer, DownloadArea + (BytesRead/4), ReadSize ) )
            return GeneralError;

        if ( GetUseSpareArea( BOOT_FLASH ) )
            BytesRead += ReadSize + SpareAreaSize;
        else
            BytesRead += ReadSize;

        // ignoring spare area in flash on flash read
        FlashReadAddr += ReadSize;
    }
    return Retval;
}


UINT CalcImageChecksum( UINT_T* DownloadArea, UINT_T ImageLength, UINT_T ImageType, UINT_T PageSize, UINT_T SpareAreaSize )
{
    UINT ImageChecksum = 0;
    UINT32* ptr32 = DownloadArea;
    UINT32* pEnd = ptr32 + (ImageLength / sizeof(UINT32));
    UINT BytesSummed = 0;
    UINT bUseSpareArea = 0;

    // position to skip over spare area bytes when calculating checksum
    if ( ImageType == DLCMD_WINDOWS_MOBILE_IMAGE_TYPE || ImageType == DLCMD_CUSTOMIZED_IMAGE_TYPE)
        if ( GetUseSpareArea( BOOT_FLASH ) )
            bUseSpareArea = 1;

    while ( ptr32 < pEnd )
    {
        // skip over spare area bytes
        if ( bUseSpareArea && (BytesSummed > 0) && (0 == (BytesSummed % PageSize)) )
        {
            ptr32 += SpareAreaSize/sizeof(UINT32);
            BytesSummed = 0;
            continue;
        }

        // checksum format version 2 algorithm as defined by flasher
        ImageChecksum ^= (*ptr32);
        ptr32++;
        BytesSummed += sizeof(UINT32);
    }
    return ImageChecksum;
}


UINT_T CopyImageToFlash( UINT_T* DownloadArea, UINT_T FlashStartAddr, UINT_T Length, UINT_T PageSize, UINT_T SpareAreaSize)
{
    UINT_T Retval = NoError;
    UINT32* ptr32 = DownloadArea;
    UINT32* pEnd = ptr32 + (Length / sizeof(UINT32));
    UINT_T  FlashWriteAddr = FlashStartAddr;
    UINT i;

    // all 0xff's buffer for comparing with image padding
    char FFsBuffer[4*1024];
    for ( i=0; i<(4*1024); i++ )
    {
        // fill buffer with all F's
        FFsBuffer[i] = 0xff;
    }

    // Copy the Image to flash
    AddMessage((UINT8_T*) ("*** Copy Image to Flash...\0"));

    while ( ptr32 < pEnd )
    {
        // skip writing any page that contains all f's
        if ( 0 != memcmp( FFsBuffer, ptr32, PageSize ) )
        {
            Retval = WriteFlash(FlashWriteAddr, (UINT_T)ptr32, PageSize, BOOT_FLASH);
            CheckForErrors(Retval);
        }
        ptr32 += (PageSize+SpareAreaSize)/sizeof(UINT32);
        FlashWriteAddr += PageSize/sizeof(UINT32);
    }
    return Retval;
}
