#ifndef	__RB_ERRORS_H__
#define	__RB_ERRORS_H__
/*____________  BASIC  TYPES _________________________________________*/

typedef long RB_RETCODE;

/*
 *  Baundries for the RB_RETCODE code range
 */
#define RB_MAX_CODE_RANGE			(0xFFFFFF)

/*________________  Return code definition  __________________*/


/* 
 * Return value is a 32 bit size layout:
 *
 *   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *  +-+-+-+-+-+-+-+-+-----------------------------------------------+
 *  |S|r|r|r|r|r|r|r|                C  o  d  e                     |
 *  +-+-+-+-+-+-+-+-+-----------------------------------------------+
 *
 * Where:
 *
 * S        - Status bit indicates success/error
 *
 * r        - reserved bit
 *  
 *   
 * Code     - is the status code,  range: 0x0000 to RB_MAX_CODE_RANGE
 */


/*
 * indication for success in RB_RETCODE
 */
#define RB_SUCCESS_STATUS		(0)

/*
 * indication for error in RB_RETCODE
 */
#define RB_ERROR_STATUS	        (1)

/*
 *	Compose RB_RETCODE:
 */
#define  RB_MAKE_RETCODE(status, code)	(long)(((long)(status)<<31)|(code))

/*
 *	compose success and error codes
 */
#define  RB_MAKE_SUCCESS(rc)    RB_MAKE_RETCODE(RB_SUCCESS_STATUS, rc)
#define  RB_MAKE_ERROR(rc)      RB_MAKE_RETCODE(RB_ERROR_STATUS, rc)


/* _____________ return codes querying ________________ */

/*
 *	Test if composed returned value is error
 */
#define  RB_IS_ERROR(rc)        ( ((rc)>>31) == RB_ERROR_STATUS)
#define  RB_IS_SUCCESS(rc)		( ((rc)>>31) == RB_SUCCESS_STATUS)
#define  RB_CODE(rc)            ((rc)&RB_MAX_CODE_RANGE)


/* __________________________ general codes ________________________________________*/
#define  S_RB_SUCCESS               RB_MAKE_SUCCESS(0)
#define  E_RB_FAILURE               RB_MAKE_ERROR(0)

/* __________________________ specific success codes  ______________________________*/
#define  S_RB_NO_UPDATE             RB_MAKE_SUCCESS(0)
#define  S_RB_DONE                  RB_MAKE_SUCCESS(0)

/* __________________________ invocation errors ____________________________________*/
#define  E_RB_BAD_PARAMS			RB_MAKE_ERROR(2)    /* error in a run parameter */
#define  E_RB_NO_VALID_UPI			RB_MAKE_ERROR(3)    /* no valid UPI found       */
#define  E_RB_BUSY					RB_MAKE_ERROR(4)    /* future use               */

/* __________________________ update package errors ________________________________*/
#define  E_RB_PKG_TOO_SHORT			RB_MAKE_ERROR(11)	/* expected length error    */
#define  E_RB_PKG_TOO_LONG			RB_MAKE_ERROR(12)	/* expected length error    */
#define  E_RB_PKG_CORRUPTED			RB_MAKE_ERROR(13)	/* structural error         */
#define  E_RB_SOURCE_CORRUPTED		RB_MAKE_ERROR(14)	/* signature error          */
#define  E_RB_PKG_NOT_AUTHORIZED    RB_MAKE_ERROR(15)   /* foreign key not signed   */
#define  E_RB_WRONG_UPDATE			RB_MAKE_ERROR(16)	/* not for current version  */
#define  E_RB_WRONG_UPI_VER			RB_MAKE_ERROR(17)	/* non-compatible UPI		*/
#define  E_RB_WRONG_UPI_UPDATE		RB_MAKE_ERROR(18)   /* update for UPI does not
													       match its version        */
#define  E_RB_UPDATE_SECTOR_SIG		RB_MAKE_ERROR(19)   /* update for UPI does not
													       match its version        */
/*___________________________ resources errors _____________________________________*/
#define  E_RB_NOT_ENOUGH_RAM		RB_MAKE_ERROR(30)	/* given RAM is not enough  */ 
#define  E_RB_BAD_RAM				RB_MAKE_ERROR(31)	/* does not behave as RAM   */
#define  E_RB_NO_ROOM_FOR_NEW		RB_MAKE_ERROR(32)	/* new version is too big   */
#define  E_RB_WRITE_ERROR			RB_MAKE_ERROR(33)	/* flash writing failure    */
#define  E_RB_ERASE_ERROR			RB_MAKE_ERROR(34)	/* flash erasing failure    */
#define  E_RB_READ_ERROR			RB_MAKE_ERROR(35)	/* flash reading failure    */
#define  E_RB_MALLOC_ERROR			RB_MAKE_ERROR(36)	/* memory allocation failure*/

/*___________________________ final-stage errors ___________________________________*/
#define  E_RB_NEW_CORRUPTED			RB_MAKE_ERROR(40)   /* can not restore new      */
#define  E_RB_NEW_NOT_SIGNED		RB_MAKE_ERROR(41)   /* signature error of new   */
#define  E_RB_NEW_NOT_AUTHORIZED	RB_MAKE_ERROR(42)	/* foreign key not signed   */
#define  E_RB_INVALID_SUPPORT_FUNC	RB_MAKE_ERROR(43)	/* foreign key not signed   */

/*________________________ recommended File System update errors ___________________*/

#define  E_RB_OPENFILE_ONLYR		RB_MAKE_ERROR(205)  /* file does not exist      */
#define  E_RB_OPENFILE_WRITE		RB_MAKE_ERROR(206)  /* RO or no access rights   */
#define  E_RB_DELETEFILE_NOFILE		RB_MAKE_ERROR(207)	/* file does not exist      */
#define  E_RB_DELETEFILE			RB_MAKE_ERROR(208)	/* no access rights         */
#define  E_RB_RESIZEFILE			RB_MAKE_ERROR(209)	/* cannot resize file       */
#define  E_RB_READFILE_SIZE			RB_MAKE_ERROR(210)	/* cannot read specfied size*/
#define  E_RB_CLOSEFILE_ERROR		RB_MAKE_ERROR(211)	/* cannot close file handle*/
	

#endif //__RB_ERRORS_H__
