/******************************************************************************
**  (C) Copyright 2007 Marvell International Ltd.  
**  All Rights Reserved
******************************************************************************/

/********************************************************************************
**  COPYRIGHT 2007 Marvell Inernational Ltd.
**  All Rights Reserved
**
**  COPYRIGHT (C) 2002, 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:   Misc.c
**
**  PURPOSE:    Contain helpful misc. functions
**
******************************************************************************/

#include "misc.h"

static unsigned char*       StrPtr8;
static unsigned int         DataWidth;

UINT_T CheckReserved (pTIM pTIM_h)
{
    pWTP_RESERVED_AREA pWRA = (pWTP_RESERVED_AREA) pTIM_h->pReserved;

    // Make sure that the TIM has a credible size
    if(pTIM_h->pConsTIM->SizeOfReserved == 0)
        return NotFoundError;

    // Older TIM's had old reserved fields definition
    if (pTIM_h->pConsTIM->VersionBind.Version == (0x00030101))
        return NotFoundError;

    // Was this area in reserved field created by a WTP compliant tool so we can parse?
    if (pWRA->WTPTP_Reserved_Area_ID != WTPRESERVEDAREAID)
        return NotFoundError;

    return NoError;
}

// Finds a Package of WTP recognized information in the reserved area based on identifier
pWTP_RESERVED_AREA_HEADER FindPackageInReserved (UINT_T * Retval, pTIM pTIM_h, UINT_T Identifier)
{
    UINT_T Count = 0;
    pWTP_RESERVED_AREA_HEADER pWRAH = NULL;
    pWTP_RESERVED_AREA pWRA = (pWTP_RESERVED_AREA) pTIM_h->pReserved;

    *Retval = CheckReserved(pTIM_h);
    if (*Retval != NoError)
        return NULL;

    // Start from the begining
    pWRAH = (pWTP_RESERVED_AREA_HEADER) (pWRA + 1);
    //NOTE: the '+ 1' is deceiving.  Should be '+ sizeof(pWTP_RESERVED_AREA_HEADER)'

    while ((pWRAH->Identifier != Identifier) && (pWRAH->Identifier != TERMINATORID) && (Count < pWRA->NumReservedHeaders))
    {
            pWRAH = (pWTP_RESERVED_AREA_HEADER) ((UINT_T) pWRAH + ((pWRAH->Size + 3) & ~3));    // Skip to the next one
            Count++;
    }
    if (pWRAH->Identifier != Identifier)
        *Retval = NotFoundError;

    return pWRAH;
}

//--------------------------------------------------------------------------------------------------------
// These 2 functions allow for more than one instance of each package "TYPE" to be retrieved
// from the reserved area.
// The TIM is expected to be a static data area.
//--------------------------------------------------------------------------------------------------------

//------------------------------------------------------
// This data is shared between typical Getfirst/GetNext
// functions below.
//------------------------------------------------------
static pTIM pGetNextTim_h = NULL;
static pWTP_RESERVED_AREA_HEADER pLastIdFound = NULL;
static UINT_T lastIdType = 0x21212121; // = "!!!!"
static UINT_T reservedPackageCount = 0;

//---------------------------------------------------------------------------------------------------
//  FindFirstPackageInReserved()
//
//   Starting from the begining of the reserved area, search for the first instance of
//   a reserved package of the type specified by "Identifier"
//
//   update the static data for FindNextPackageInReserved()
//
//---------------------------------------------------------------------------------------------------
pWTP_RESERVED_AREA_HEADER FindFirstPackageTypeInReserved (UINT_T * Retval, pTIM pTIM_h, UINT_T Identifier)
{
  UINT_T Count = 0;
  pWTP_RESERVED_AREA_HEADER pWRAH;
  pWTP_RESERVED_AREA pWRA  = (pWTP_RESERVED_AREA) pTIM_h->pReserved;

  *Retval = CheckReserved(pTIM_h);
  if (*Retval != NoError)
    return NULL;

  // Start from the begining of the reserved area
  //---------------------------------------------
  pWRAH = (pWTP_RESERVED_AREA_HEADER) (pWRA + 1);

  while( (pWRAH->Identifier != Identifier)   &&
         (pWRAH->Identifier != TERMINATORID) &&
         (Count < pWRA->NumReservedHeaders)     )
  {
     pWRAH = (pWTP_RESERVED_AREA_HEADER) ((UINT_T) pWRAH + ((pWRAH->Size + 3) & ~3));   // Skip to the next one
     //((UINT_T)pWRAH) += (pWRAH->Size + 3) & ~3;   // Skip to the next one
     Count++;
  }

  if (pWRAH->Identifier != Identifier)
  {
    *Retval       = NotFoundError;
    pLastIdFound  = NULL;
    pGetNextTim_h = NULL;
    lastIdType    = 0x21212121;
    reservedPackageCount = 0;
    return NULL;
  }
  else
  {
    pLastIdFound         = pWRAH;
    lastIdType           = Identifier;
    reservedPackageCount = Count;
    pGetNextTim_h        = pTIM_h;
    return pWRAH;
  }
}

//---------------------------------------------------------------------------------------------------
//  FindNextPackageInReserved()
//
//
//
//---------------------------------------------------------------------------------------------------
pWTP_RESERVED_AREA_HEADER FindNextPackageTypeInReserved(UINT_T *Retval )
{
    pWTP_RESERVED_AREA_HEADER pWRAH;
    pWTP_RESERVED_AREA pWRA = (pWTP_RESERVED_AREA) pGetNextTim_h->pReserved;
    *Retval = NoError;

    if( (lastIdType    == 0x21212121) ||
        (pLastIdFound  == NULL)       ||
        (pGetNextTim_h == NULL)          )
    {
        *Retval = NotFoundError;
        return NULL;
    }

    // Start from the last item of Identifier type found
    //--------------------------------------------------
    pWRAH = pLastIdFound;
    pWRAH = (pWTP_RESERVED_AREA_HEADER) ((UINT_T) pWRAH + pWRAH->Size); // Skip to the next one
     //(UINT_T)pWRAH += pWRAH->Size;      // Skip to the next one
    reservedPackageCount++;
    while(  (pWRAH->Identifier != lastIdType)   &&
            (pWRAH->Identifier != TERMINATORID) &&
            (reservedPackageCount < pWRA->NumReservedHeaders)  )
    {
        pWRAH = (pWTP_RESERVED_AREA_HEADER) ((UINT_T) pWRAH + ((pWRAH->Size + 3) & ~3));    // Skip to the next one
        //((UINT_T)pWRAH) += (pWRAH->Size + 3) & ~3;    // Skip to the next one
        reservedPackageCount++;
    }

    // Reset the get First / Next interaction
    //---------------------------------------
    if( pWRAH->Identifier != lastIdType )
    {
      *Retval      = NotFoundError;
      pLastIdFound = NULL;
      lastIdType   = 0x21212121;
      reservedPackageCount = 0;
      return NULL;
    }
    else
    {
      pLastIdFound = pWRAH;
      return pWRAH;
    }
}

//mdb - CRC disabled
#if 0
/********************   CRC Check Sum ******************************************************
*  Calculates a CRC Checksum of an image given its address and length (in bytes.)  This
*  should be used on Non-Trusted images.
*
*******************************************************************************************/
UINT32_T CheckSum(UINT8_T *Address, UINT_T length, UINT8_T *SkipAddr){
    UINT32_T Key = 0x04C11DB7;
    UINT32_T Val = 0xFFFFFFFF;
    UINT_T i, t; //t keeps track of bit position
    UINT8_T CurrentByte = Address[0];

    i = 0; t = 0;

    while(i < length)
    {
        if( ( (( Val >> 31)&0x1) ^ ((CurrentByte >> t) & 0x1) ) == 1)
            Val = ( Val << 1 ) ^ Key;
        else
            Val = Val << 1;

        if(t == 7)
        {
            t = 0;
            i++;
            if((SkipAddr == NULL) || ((&Address[i] < SkipAddr) || (&Address[i] > (SkipAddr + 3))))
                CurrentByte = Address[i];
            else
                CurrentByte = 0;
        }
        else t++;
    }
    return Val;
}

UINT_T ValidateCheckSum(UINT_T ImageID, pTIM pTIM_h)
{
    UINT_T Retval = NoError;
    UINT_T StartAddr = 0;
    UINT_T ImageSize;
    UINT_T checksum;
    pIMAGE_INFO pImageInfo;
    UINT8_T *SkipAddr = NULL;

    // First find the image in the TIM, then get relavent data.  We always validate
    // from the load address and require that the image has been loaded first
    pImageInfo = FindImageInTIM(pTIM_h, ImageID);
    if (pImageInfo == NULL)
        return ImageNotFound;
    if (pImageInfo->ImageSizeToHash == 0)
        return NoError;

    ImageSize = pImageInfo->ImageSize;
    checksum  = pImageInfo->Hash[0];

    // Even though the image size to validate is too large attempt to do the whole image
    // but not more than the whole image.  Worst case the CRC will fail anyways ending in the same result.
    if( pImageInfo->ImageSizeToHash > ImageSize )
        pImageInfo->ImageSizeToHash = ImageSize;

    // Determine Starting Address
    switch (ImageID)
    {
    case TIMIDENTIFIER:
        StartAddr = (UINT_T) pTIM_h->pConsTIM;                      // ISRAM to Flash
        SkipAddr = (UINT8_T*)&(pImageInfo->Hash[0]);
        break;
#if (DKB)
    default:                                                        // All other images in scratch area
        StartAddr = DDR_DKB_IMAGE_DOWNLOAD;
        break;
#else
    default:
        StartAddr = pImageInfo->LoadAddr;                       // OBM should have be linked with this address
        break;
#endif
    }

    if(checksum != CheckSum((UINT8_T*)StartAddr, pImageInfo->ImageSizeToHash, SkipAddr))
        Retval = CRCFailedError;

    return Retval;
}

#endif

pIMAGE_INFO FindImageInTIM(pTIM pTIM_h, UINT_T ImageID)
{
    pIMAGE_INFO pImageInfo = NULL;
    UINT8_T found = FALSE;
    UINT8_T i;

    for( i = 0; i < pTIM_h->pConsTIM->NumImages; i++)
    {
        pImageInfo = (pIMAGE_INFO)&pTIM_h->pImg[i];
        if(pImageInfo->ImageID == ImageID)
        {
            found = TRUE;
            break;
        }
    }
    if (found == TRUE)
        return pImageInfo;
    else
        return NULL;
}

/******************************************************************************
 * void Delay(int usec)
 *
 *   Description:  Delays usec microseconds (accurate to within ~500 usecs)
 *   Parameters:    usec - time to delay in microseconds
 *   Return: none
 ******************************************************************************/
void Delay(int uSec)
{
    int delaytime = (int)(TIMER_FREQ(uSec)); // Made TIMER_FREQ into macro to
                                             // avoid using the built-in library
                                             // this reduces the code footprint
    // make sure OSCR has enough time to reset to zero
    do
    {
        TIMER_CLEAR;
    } while (TIMER_GET>100);

    // now do the wait loop
    while (TIMER_GET < delaytime);

    return;
}

int ConvertStringToInteger8(const unsigned char* StringPtr, unsigned int Count)
{
    int  Output = 0;
    unsigned char c;
    while (*StringPtr)
    {
        Output *= Count;
        c = *StringPtr++;
        if (c >= '0' && c <= '9')
            Output += (c-'0');
        else
            Output += ((c|' ')-'a'+10);
    }
    return (Output);
}

int DivideTwoNumbers(int Numerator,int Denominator)
{
    int Tmp    = 0;
    int Output = 0;
    int Count;
    for (Count = 28; Count >= 0; Count -= 4)
    {
        Tmp    <<= 4;
        Output <<= 4;
        Tmp |= (Numerator >> Count)&0xf;
        while(Tmp >= 0)
        {
            Tmp -= Denominator;
            Output++;
        }
        Tmp += Denominator;
        Output--;
    }
    return(Output);
}

int ModOfTwoNumbers(int Numerator, int Denominator)
{
    return(Numerator - (Denominator * DivideTwoNumbers(Numerator, Denominator)));
}

void ConvertLongIntToBuf8(int DataValue, int Count)
{
    if (DataValue >= Count)
    {
        ConvertLongIntToBuf8(DivideTwoNumbers(DataValue, Count), Count);
        StrPtr8++;
        DataWidth--;
        DataValue = ModOfTwoNumbers(DataValue, Count);
    }
    if (DataWidth > 0)
        *StrPtr8 = (unsigned char)(DataValue + (DataValue < 10 ? '0' : ('a'-10)));
}

void ConvertIntToBuf8(unsigned char* StringPtr,unsigned int Value, int Width,unsigned int Count)
{
    StrPtr8    = StringPtr;
    DataWidth  = Width;
    if (Value >= Count)
    {
        ConvertLongIntToBuf8(DivideTwoNumbers(Value,Count),Count);
        StrPtr8++;
        DataWidth--;
        Value=ModOfTwoNumbers(Value,Count);
    }
    if (DataWidth-->0)
        *StrPtr8++=(unsigned char)(Value + (Value < 10 ? '0' : ('a'-10)));
    while (DataWidth-- > 0)
        *StrPtr8++ = ' ';
}

/**
 * Memory set function
 **/
void memset(void *Addr, unsigned char Val, unsigned long Size)
{

    int i;
    for(i=0; i < Size; i++ )
    {
        ((unsigned char*)Addr)[i] = Val;
    }
}

void *memcpy(void *dest, const void *src, unsigned int n)
{
    const unsigned char *s = (const unsigned char*)src;
    unsigned char *d = (unsigned char *)dest;

    while(n-- > 0)
        *d++ = *s++;

    return dest;
}

// Timer routines
UINT_T GetOSCR0(void)
{
    return (*(volatile UINT_T *)0x40A00010);
}

UINT_T OSCR0IntervalInSec(UINT_T Before, UINT_T After)
{
    UINT_T Temp = (After - Before);
    return (Temp/(3250000));
}

UINT_T OSCR0IntervalInMicro(UINT_T Before, UINT_T After)
{
    UINT_T Temp = (After - Before);
    Temp        = (Temp * 13);
    return (Temp/4);
}

UINT_T OSCR0IntervalInMilli(UINT_T Before, UINT_T After)
{
    UINT_T Temp = (After - Before);
    return (Temp/(3250));
}

INT_T memcmp( const void *buffer1, const void *buffer2, INT_T count)
{
    UINT8_T* buf1 = (UINT8_T*)buffer1;
    UINT8_T* buf2 = (UINT8_T*)buffer2;

  while(count)
  {
    if( *(buf1) != *(buf2) )
      return( (*buf1 < *buf2) ? -1 : 1 );

    buf1 += sizeof(UINT8_T);
    buf2 += sizeof(UINT8_T);
    count--;
  }

  return 0;
}

/*
   String Compare function - case sensitive
   Will compare str1 to str2, character to character, until a NULL character is hit
   or until maxlength is reached
   If no NULL character is present, there could be serious issues

      Return values:
      -1 = str1 < str2
       0 = Match
       1 = str1 > str2

 */
int strcmpl( char* str1, char* str2, int maxlength )
{
    int length = maxlength;
    while((*str1 != NULL) && (*str2 != NULL) && --length)
    {
        if (*str1 != *str2)
        {
            // check the char that was different to determine results
            if ( *str1 > *str2 )
                return 1;
            else
                return -1;
        }
        str1++; str2++;
    }

    // strings compared so far, but did we reach the end of both?
    if ( *str1 != NULL && *str2 != NULL )
        // did not find the NULLs but have reached the maxlength to compare
        return 0;

    if ( *str1 == NULL && *str2 == NULL )
        return 0;

    if ( *str1 != NULL && *str2 == NULL )
        // "abcde" > "abcd"
        return 1;
    else
        // "abcd" < "abcde"
        return -1;
}


/*
   String Compare function - case insensitive
   Will compare str1 to str2 ignoring case, character to character, until a NULL character is hit
   or until maxlength is reached
   If no NULL character is present, there could be serious issues

      Return values:
      -1 = str1 < str2
       0 = Match
       1 = str1 > str2
 */
int stricmpl( char* str1, char* str2, int maxlength)
{
    int length = maxlength;
    while((*str1 != NULL) && (*str2 != NULL) && --length)
    {
        if ( ((*str1) & 0xDF) != ((*str2) & 0xDF) )
        {
            // check the char that was different to determine results
            if ( ((*str1) & 0xDF) > ((*str2) & 0xDF) )
                return 1;
            else
                return -1;
        }
        str1++; str2++;
    }

    // strings compared so far, but did we reach the end of both?
    if ( *str1 != NULL && *str2 != NULL )
        // did not find the NULLs but have reached the maxlength to compare
        return 0;

    if ( *str1 == NULL && *str2 == NULL )
        return 0;

    if ( *str1 != NULL && *str2 == NULL )
        // "abcde" > "abcd"
        return 1;
    else
        // "abcd" < "abcde"
        return -1;
}


INT_T strlen (const char * str)
{
    int i = 0;
    while(str[i++] != '\0')
        if(i > 1000)
            break;
    return --i;
}

void __aeabi_idiv0( void ){

    //dummy function for the compiler
    // need to investigate the in32t_divide.o and x0_a000.o libraries from XDB
    return;
}

//The below functions are pulled in from a library when built by SDT compiler (NDT flag)
//When build by linux compiler, we need to define the calls ourselves
#ifndef NDT

void __aeabi_idiv(void){
    asm("b DivideTwoNumbers");
    return;
}

void __aeabi_uidiv(void){
    asm("b DivideTwoNumbers");
    return;
}

void __aeabi_uidivmod(void){
    asm("b ModOfTwoNumbers");
    return;
}

#endif
#ifdef FBW_CODE
void bytes2hex(int value, char *out_buf, char bytes, char pad)
{
	char *p;

	inline char bcd2hex(char BCD)
	{
		return (BCD > 9) ? (BCD-10+'A'):(BCD+'0');
	}
	inline void char2hex(char value,char **out)
	{
	*(*out)++ = bcd2hex(value >> 4);
	*(*out)++ = bcd2hex(value & 0x0f);
	}

	p = out_buf;
    if(pad)
    {
        *p++ = '0';
        *p++ = 'x';
    }
	if(bytes > 3)char2hex(value >> 24,&p);
	if(bytes > 2)char2hex(value >> 16,&p);
	if(bytes > 1)char2hex(value >>  8,&p);
	char2hex(value &0xff,&p);
	*p = 0;
}
#endif


