;/******************************************************************************
;**	(C) Copyright 2007 Marvell International Ltd. 
;**	All Rights Reserved
;******************************************************************************/

;******************************************************************************
;*
;* INTEL CONFIDENTIAL
;* Copyright 2003-2005 Intel Corporation All Rights Reserved.
;*
;* The source code contained or described herein and all documents
;* related to the source code (Material) are owned by Intel Corporation
;* or its suppliers or licensors.  Title to the Material remains with
;* Intel Corporation or its suppliers and licensors. The Material contains
;* trade secrets and proprietary and confidential information of Intel
;* or its suppliers and licensors. The Material is protected by worldwide
;* copyright and trade secret laws and treaty provisions. No part of the
;* Material may be used, copied, reproduced, modified, published, uploaded,
;* posted, transmitted, distributed, or disclosed in any way without Intels
;* prior express written permission.
;*
;* No license under any patent, copyright, trade secret or other intellectual
;* property right is granted to or conferred upon you by disclosure or
;* delivery of the Materials, either expressly, by implication, inducement,
;* estoppel or otherwise. Any license under such intellectual property rights
;* must be express and approved by Intel in writing.
;*
;*  FILENAME:   MMUenable.inc
;*
;*  PURPOSE:    Code intented to enable the MMU, and make things virtual.
;*
;*****************************************************************************

   INCLUDE macro.MAC

   def TTB_BASE,              0x80040000
   def TTB_PAGE_BASE,         0x80046000

;*************************************************** *
;* Memory Map with virtual to physical translations * *
;*************************************************** *
   B MemMapEnd
;ALIGN   32
    INCLUDE memmap.INC
MemMapEnd

;***************************************************************
;*
;* Regardless of reset reason, disable MMU and flush things out
;*
;***************************************************************
;ALIGN   32
   ldr      r0, =0x2043             ; enable access to all coprocessors
   mcr      p15, 0, r0, c15, c1, 0
   CPWAIT   r0

   ldr      r0, =0x00000078         ; zero for off ([6:3] must be ones)
   mcr      p15, 0, r0, c1, c0, 0   ; turn off MMU, I/D caches, write buffer
   CPWAIT   r0

   ldr      r0, =0x00000000         ; zero for off
   mcr      p15, 0, r0, c8, c7, 0   ; flush/invalidate I/D TLBs
   mcr      p15, 0, r0, c7, c7, 0   ; flush/invalidate I/D caches
   mcr      p15, 0, r0, c7, c10, 4  ; drain the write buffer
   CPWAIT   r0

   ldr      r0, =0x00000000         ; zero for off
   mcr      p15, 0, r0, c3, c0, 0   ; grant manager access to all domains
   CPWAIT   r0

;********************************************************************************
;*
;* Build translation table, rooted at TTB_BASE
;*
;********************************************************************************

   ;**************************************
   ;* Load the memory map
   ;**************************************
;   ldr      r0, =__main
   add      r0, pc, #OEMAddressTable-(.+8)   ; (r0) = OEMAddressTable phy addr
   mov      r1, r0                           ; (r1) = ptr to MemoryMap array

   ;*****************************
   ;* Build TTB
   ;*****************************
   ldr      r9, =(TTB_BASE & 0xFFFFC000)     ; (r9) = phys addr of 1st level table
   mcr      p15, 0, r9, c2, c0, 0            ; set TTB

   ;**************************************************
   ;* Map cacheable, bufferable section descriptors
   ;**************************************************

   ;*
   ;* Go through each entry in OEMAddressTable[]
   ;*
   ;*   r2 holds OEMAddressTable[virtual_address]
   ;*   r3 holds OEMAddressTable[physical_address]
   ;*   r4 holds OEMAddressTable[#MB]
   ;*   r5 holds OEMAddressTable[#KB]
   ;*   r0 holds OEMAddressTable[properties]
   ;*
OEMAT_BEGIN
   ldr      r2, [r1], #4   ; r2 = virtual
   ldr      r3, [r1], #4   ; r3 = physical
   ldr      r4, [r1], #4   ; r4 = num MB
   ldr      r5, [r1], #4   ; r5 = num KB
   ldr      r0, [r1], #4   ; r0 = page properties

   cmp      r4, #0         ; end of table?
   bne      OEMAT_MB

   cmp      r5, #0         ; mapping KBs or MBs?
   beq      OEMAT_END

OEMAT_KB
   mov      r7, r2         ; will need VA later

   ; Create 1st level descriptor address --> [r2]
   ldr      r6, =0xFFF00000
   and      r2, r2, r6              ; only VA[31:20] are valid
   orr      r2, r9, r2, LSR #18     ; address: r2=(TTB[31:14] | VA[31:20] >> 18)

   ; Create 1st level descriptor
   ldr      r6, =(TTB_PAGE_BASE | 0x01)

   ; Store 1st level descriptor (then it will be no longer needed)
   str      r6, [r2], #0

   ; Create 2nd level descriptor address
   ldr      r6, =TTB_PAGE_BASE      ; page base
   ldr      r2, =0x000FF000         ; mask for [19:12]
   and      r7, r2, r7
   orr      r2, r6, r7, LSR #10     ; address: r2=(TTB_PAGE_BASE(31:10) | VA[19:12] >> 10)

   ; Create 2nd level descriptor
   ldr      r6, =0xFFFF0000         ; mask
   and      r3, r3, r6              ; PA[31:16]
   ldr      r6, =0x0FFD             ; AP[3:0] user/sup R/W; 64KB
   orr      r3, r3, r6

OEMAT_KB2
   ; Store descriptor
   ldr      r6, =0x10               ; as the upper 4 bits of the page index and the
OEMAT_OVERLAP                       ; low-order 4 bits of the second-level table index
   str      r3, [r2], #0x04         ; overlap, each page table entry for a large page
   sub      r6, r6, #1              ; must be duplicated 16 times in consecutive
   cmp      r6, #0                  ; memory locations in the page table.
   bne      OEMAT_OVERLAP

   add      r3, r3, #0x00010000
   sub      r5, r5, #1
   cmp      r5, #0
   bne      OEMAT_KB2

   ; done; skip MB processing
   b        OEMAT_NEXT

OEMAT_MB
   ldr      r6, =0xFFF00000         ; common mask (assume r6 uncorrupted by "and")

   ; Create descriptor address
   ;   (r2) holds the &descriptor
   ;   (r0) holds the actual section descriptor
   ;
   and      r2, r2, r6              ; only VA[31:20] are valid
   orr      r2, r9, r2, LSR #18     ; build the address: r2=(TTB[31:14] | VA[31:20] >> 18)

   ; Create descriptor
   ;
   orr      r0, r0, #0x0002         ; set 1MB section descriptor
   orr      r0, r0, #0x0400         ; access permission (sup. R/W)
   and      r3, r3, r6              ; only PA[31:20] are valid for descriptor
   orr      r0, r3, r0              ; build the descriptor: r0=(PA[31:20] | rest of desc)

   ; Store descriptor at the proper physical address
   ;
OEMAT_MB2
   str      r0, [r2], #4
   add      r0, r0, #0x00100000     ; (r0) = THE section descriptor for next 1MB mapping
   sub      r4, r4, #1              ; decrement number of MB left
   cmp      r4, #0
   bne      OEMAT_MB2               ; map next MB

OEMAT_NEXT
   bic      r0, r0, #0xF0000000     ; clear section base address field
   bic      r0, r0, #0x0FF00000     ; clear section base address field
   b        OEMAT_BEGIN             ; get and process next table element

OEMAT_END
; End of table creation


;**************************************
;*
;* Activate MMU
;*
;**************************************
   mov      r1, #0x3
   mcr      p15, 0, r1, c3, c0, 0   ; set up access to domain 0
   mcr      p15, 0, r0, c8, c7, 0   ; flush I + D TLBs
   mcr      p15, 0, r1, c7, c10, 4  ; drain write and fill buffers
   CPWAIT   r0

   mov      r1, #0x78               ; bits [6:31] must be written as 1's
   orr      r1, r1, #0x0001         ; Enable MMU
;   orr      r1, r1, #0x1000         ; Enable Icache
;   orr      r1, r1, #0x0004         ; Enable Dcache
;   orr      r1, r1, #0x0800         ; Enable BTB

   mcr      p15, 0, r1, c1, c0, 0   ; MMU now on, all memory accesses now virtual
   mrc      p15, 0, r1, c2, c0, 0
   mov      r1,  r1                     ; before continuing
   sub      pc,  pc,  #4


     ;************************************************************;
   ;****************************************************************;
   ;******'                                                  `******;
   ;******   The rest of the code will run in virtual space   ******;
   ;******,                                                  ,******;
   ;****************************************************************;
     ;************************************************************;

   ldr   r0, =__JUMP_TO_C_CODE
   MOV   pc, r0

   END



