/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2006 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	PlatformConfig.c
**
**  PURPOSE: 	Contain platform specific configuration routines
**
******************************************************************************/

#include "platformconfig.h"
#include "general.h"
#include "usbdownload.h"

/*
 *  Table definitions for multi function pin registers
 */


CS_REGISTER_PAIR_S io_pins[] =
{
 		(int *) (APPS_PAD_BASE | 0x023C), 0x1001, 0x0,	//ND_IO0
		(int *) (APPS_PAD_BASE | 0x0240), 0x1001, 0x0,	//ND_IO1
		(int *) (APPS_PAD_BASE | 0x0244), 0x1001, 0x0,	//ND_IO2
		(int *) (APPS_PAD_BASE | 0x0248), 0x1001, 0x0,	//ND_IO3
		(int *) (APPS_PAD_BASE | 0x0264), 0x1001, 0x0,	//ND_IO4
		(int *) (APPS_PAD_BASE | 0x0268), 0x1001, 0x0,	//ND_IO5
		(int *) (APPS_PAD_BASE | 0x026C), 0x1001, 0x0,	//ND_IO6
		(int *) (APPS_PAD_BASE | 0x0270), 0x1001, 0x0,	//ND_IO7
		(int *) (APPS_PAD_BASE | 0x0274), 0x1041, 0x0,	//ND_IO8
		(int *) (APPS_PAD_BASE | 0x0278), 0x1041, 0x0,	//ND_IO9
		(int *) (APPS_PAD_BASE | 0x027C), 0x1041, 0x0,	//ND_IO10
		(int *) (APPS_PAD_BASE | 0x0280), 0x1041, 0x0,	//ND_IO11
		(int *) (APPS_PAD_BASE | 0x0284), 0x1041, 0x0,	//ND_IO12
		(int *) (APPS_PAD_BASE | 0x0288), 0x1041, 0x0,	//ND_IO13
		(int *) (APPS_PAD_BASE | 0x028C), 0x1041, 0x0,	//ND_IO14
		(int *) (APPS_PAD_BASE | 0x0290), 0x1041, 0x0,	//ND_IO15
		0x0,0x0,0x0 // termination
};

CS_REGISTER_PAIR_S common_pins[]=
{
		(int *) (APPS_PAD_BASE | 0x0218), 0x1001, 0x0, 	//DF_nADV1_ALE
		(int *) (APPS_PAD_BASE | 0x020C), 0x1001, 0x0,	//ND_CLE
		0x0,0x0,0x0 //termination
};

CS_REGISTER_PAIR_S cs2_pins[]=
{
		(int *) (APPS_PAD_BASE | 0x0224), 0x1003, 0x0, 	//DF_nADV2_ALE
		(int *) (APPS_PAD_BASE | 0x0214), 0x1001, 0x0, 	//DF_SCLK_E
		(int *) (APPS_PAD_BASE | 0x0254), 0x1101, 0x0,	//nLUA
		(int *) (APPS_PAD_BASE | 0x0258), 0x1101, 0x0, 	//nLLA
		0x0,0x0,0x0 //termination
};

CS_REGISTER_PAIR_S cs2_reg[]=
{
		(int *) (SMEMC_BASE	   | 0x0088), 0x0320919,	0x0, 	//SMEMC_CSADRCFG_2
		(int *) (SMEMC_BASE	   | 0x000C), 0x07FF8,		0x0,	//SMEMC_MSC1
		0x0,0x0,0x0 //termination
};

CS_REGISTER_PAIR_S cs0_reg[]=
{
		(int *) (SMEMC_BASE	   | 0x00A0), 0x1,			0x0, 	//SMEMC_CSMSADRCFG
		(int *) (SMEMC_BASE	   | 0x0080), 0x0320919,	0x0, 	//SMEMC_CSADRCFG_0
		(int *) (SMEMC_BASE	   | 0x0008), 0x07FF8,		0x0, 	//SMEMC_MSC0
		(int *) (0x48100000),  			  0x4,			0x0,	//MDCNFG
		(int *) (0x48100090),  			  0x01000000,	0x0,	//EMPI
		0x0,0x0,0x0 //termination
};

CS_REGISTER_PAIR_S msys_pins[]=
{
  /* CS for M-SYSTEMS H3 DEVICE and OneNAND */
		(int *) (APPS_PAD_BASE | 0x0224), 0x1003, 0x0, 	//DF_nADV2_ALE
		(int *) (APPS_PAD_BASE | 0x025C), 0x1C40, 0x0,	//DF_ADDR2
		(int *) (APPS_PAD_BASE | 0x0214), 0x1001, 0x0, 	//DF_SCLK_E
		(int *) (APPS_PAD_BASE | 0x0258), 0x1101, 0x0, 	//nLLA
		0x0,0x0,0x0 //termination
};

CS_REGISTER_PAIR_S dfc_pins[]=
{
		(int *) (APPS_PAD_BASE | 0x022C), 0x1001, 0x0, 	//ND_nCS0
		(int *) (APPS_PAD_BASE | 0x0234), 0x1001, 0x0, 	//ND_nWE
		(int *) (APPS_PAD_BASE | 0x0238), 0x1001, 0x0,	//ND_nRE_nOE
		(int *) (APPS_PAD_BASE | 0x0228), 0x1C41, 0x0, 	//ND_INT_RnB
		0x0,0x0,0x0 //termination
};

CS_REGISTER_PAIR_S u2d_pins[]=
{
		(int *) (APPS_PAD_BASE | 0x0618), 0x0447, 0x0, 	//GPIO30 ULPI_D1
		(int *) (APPS_PAD_BASE | 0x0610), 0xA444, 0x0, 	//GPIO31 ULPI_D0
		(int *) (APPS_PAD_BASE | 0x061C), 0x0445, 0x0, 	//GPIO33 ULPI_D2
		(int *) (APPS_PAD_BASE | 0x0620), 0x0455, 0x0, 	//GPIO34 ULPI_D3
		(int *) (APPS_PAD_BASE | 0x0628), 0x0455, 0x0, 	//GPIO35 ULPI_D4
		(int *) (APPS_PAD_BASE | 0x062C), 0x0455, 0x0, 	//GPIO36 ULPI_D5
		(int *) (APPS_PAD_BASE | 0x0614), 0xA455, 0x0, 	//GPIO41 ULPI_D6
		(int *) (APPS_PAD_BASE | 0x0624), 0xA455, 0x0, 	//GPIO42 ULPI_D7
		(int *) (APPS_PAD_BASE | 0x0634), 0x8454, 0x0, 	//GPIO38 ULPI_CLK
		(int *) (APPS_PAD_BASE | 0x0638), 0xC454, 0x0,	//GPIO39 ULPI_STP
		(int *) (APPS_PAD_BASE | 0x0630), 0x8454, 0x0, 	//GPIO37 ULPI_DIR
		(int *) (APPS_PAD_BASE | 0x063C), 0x8454, 0x0, 	//GPIO40 ULPI_NXT
		0x0,0x0,0x0 //termination
};

//-----------------------------------------------------------------------
// ConfigRegSetup
//
//
//-----------------------------------------------------------------------
void ConfigRegSetup( P_CS_REGISTER_PAIR_S regPtr)
{
    UINT32_T i,tmp;

	while(regPtr->registerAddr != 0x0)
    {
	  tmp = *(regPtr->registerAddr);
	  tmp &= 0x000003F8;
      *(regPtr->registerAddr) = regPtr->regValue | tmp;
	  tmp = *(regPtr->registerAddr);  // ensure write complete
      regPtr++;
    }
}

//-----------------------------------------------------------------------
// ConfigRegResume
//
//
//-----------------------------------------------------------------------
void ConfigRegResume( P_CS_REGISTER_PAIR_S regPtr)
{
    UINT32_T i,tmp;

	while(regPtr->registerAddr != 0x0)
    {
      *(regPtr->registerAddr) &= 0xFFFFFFF7;
      tmp = *(regPtr->registerAddr);  // ensure write complete
      regPtr++;
    }
}

//-----------------------------------------------------------------------
// ConfigRegSave
//
//
//-----------------------------------------------------------------------
void ConfigRegSave( P_CS_REGISTER_PAIR_S regPtr)
{
    UINT32_T i,tmp;

	while(regPtr->registerAddr != 0x0)
    {
      regPtr->defaultValue = *(regPtr->registerAddr);
      regPtr++;
    }
}

//-----------------------------------------------------------------------
// ConfigRegRestore
//
//
//-----------------------------------------------------------------------
void ConfigRegRestore( P_CS_REGISTER_PAIR_S regPtr)
{
    UINT32_T i,tmp;

	while(regPtr->registerAddr != 0x0)
    {
      *(regPtr->registerAddr) = regPtr->defaultValue;
      tmp = *(regPtr->registerAddr);  // ensure write complete
      regPtr++;
    }
}

void ChipSelect2(void){
	UINT_T	AscrReg;

	AscrReg = *ASCR;

 	if(AscrReg & BIT0){
	   ConfigRegSetup(cs2_reg );
	   ConfigRegResume(cs2_pins );
	   ConfigRegResume(common_pins );
	   ConfigRegResume(io_pins );
	}else{
	   ConfigRegSetup(cs2_reg );
	   ConfigRegSetup(cs2_pins );
	   ConfigRegSetup(common_pins );
	   ConfigRegSetup(io_pins );
  }

}

UINT_T ChipSelect0(void){
/*
	UINT_T	AscrReg;

	AscrReg = *ASCR;

 	if(AscrReg & BIT0){
	   ConfigRegSetup(cs0_reg );
	   ConfigRegResume(common_pins );
	   ConfigRegResume(io_pins );
	}else{
	   ConfigRegSetup(cs0_reg );
	   ConfigRegSetup(common_pins );
	   ConfigRegSetup(io_pins );
  }
 */
 //Tavor P does not support CS0 on EMPI

	return InvalidPlatformConfigError;


}


//-----------------------------------------------------------------------
// ChipSelect Msys and OneNAND
//
//
//-----------------------------------------------------------------------
void ChipSelectMsys( void )
{
    UINT_T  AscrReg;

	AscrReg = *ASCR;

	if(AscrReg & BIT0){
	   ConfigRegSetup(cs2_reg );
	   ConfigRegResume(msys_pins );
	   ConfigRegResume(common_pins );
	   ConfigRegResume(io_pins );
	}else{
	   ConfigRegSetup(cs2_reg );
	   ConfigRegSetup(msys_pins );
	   ConfigRegSetup(common_pins );
	   ConfigRegSetup(io_pins );
  }

}

void ChipSelectOneNAND( void)
{
	ChipSelectMsys();
}

//-----------------------------------------------------------------------
// ChipSelect for DFC
//
//
//-----------------------------------------------------------------------

void ChipSelectDFC( void )
{
   UINT_T  AscrReg;

	AscrReg = *ASCR;

	if(AscrReg & BIT0){
	   ConfigRegResume(dfc_pins );
	   ConfigRegResume(common_pins );
	   ConfigRegResume(io_pins );
	}else{
	   ConfigRegSetup(dfc_pins );
	   ConfigRegSetup(common_pins );
	   ConfigRegSetup(io_pins );
  }


}

//------------------------------------------------------------------------
//
//
//------------------------------------------------------------------------
UINT_T *PlatformGetOneNANDBaseAddr( void )
{
  return (void *)0x10000000;
}

void RestoreDefaultConfig(void){

	ConfigRegRestore(dfc_pins);
	ConfigRegRestore(cs2_reg );
	ConfigRegRestore(cs0_reg );
	ConfigRegRestore(common_pins );
	ConfigRegRestore(io_pins );

}

void SaveDefaultConfig(void){

	ConfigRegSave(dfc_pins);
	ConfigRegSave(cs2_reg );
	ConfigRegSave(cs0_reg );
	ConfigRegSave(common_pins );
	ConfigRegSave(io_pins );

}



UINT_T getPlatformPortSelection(unsigned int *default_usb_port){
	UINT_T port;

	//port = (*GPLRa >> 4) & 3 ;
#if USB1
	*default_usb_port = DIFF_USB_D;
#elif USB2
	*default_usb_port = U2D_USB_D;
#else
	*default_usb_port = -1;
#endif
	port = 0x3; // send default configuration
	return port;
}

UINT_T PlatformUARTConfig(void){

	// Setup the UART GPIOs
	*GPSRy  = 0x00400000; // Sets GP54 as output -> GP54 = FFTXD
	*GPDRy  = 0x00400000; // Sets GP54 as output -> GP54 = FFTXD

	// Program the respective UART1 PAD registers
	*GPIO53_TAVOR |= 0x441;  // FFRXD as Alt-Fn1
	*GPIO54_TAVOR |= 0x441;  // FFTXD as Alt-Fn1

	return (NoError);
}

UINT_T PlatformAltUARTConfig(void){

	// Setup the UART GPIOs
	*GPSRy  = 0x80000000; // Sets GP63 as output -> GP63 = FFTXD
	*GPDRy  = 0x80000000; // Sets GP63 as output -> GP63 = FFTXD

	// Program the respective UART1 PAD registers
	*GPIO64_TAVOR |= 0x442;  // FFRXD as Alt-Fn2
	*GPIO63_TAVOR |= 0x442;  // FFTXD as Alt-Fn2

	return (NoError);
}


/*****************************************
**		UDC 1.1 Specific Functions		**
******************************************/

/**
 * Configure USB PORT_2 for Single Ended USB Operation
 **/
UINT_T SetupSingleEndedPort2(void)
{

	return(NotSupportedError);
}

void PlatformUsbInit(UINT_T port)
{

#if USB1
   if(port == SE_USB_D)
        SetupSingleEndedPort2();
   if(port == DIFF_USB_D)
	{
		// Bring all H/W into safe state
		// Kill USB interrupts
		// Kill USB 1.1 peripheral
		// Remove XCVR pull-up, leave XCVR connected
		//-------------------------------------------
		DisablePeripheralIRQInterrupt(USB_CLIENT_INT);
		*UDCCR_REG = 0;
		*UP2OCR = 0x00020000;

		UdcInit();
		*UP2OCR = 0x00020010; // Set Host Port 2 transceiver  output enable
		*ASCR &= 0x7fffffff;  // clear RDH
	}
#endif
}

UINT_T PlatformUsbTransmit(UINT_T length, UINT8_T* Buff)
{
#if USB1
	UdcTransmit(length, Buff);
#endif
	return NoError;
}

void PlatformUsbDmaHandler()
{
#if USB1
	UdcDmaHandler();
#endif
}

void PlatformUsbInterruptHandler()
{
#if USB1
	UdcInterruptHandler();
#endif
}

void PlatformUsbShutdown()
{
#if USB1
	*UP2OCR = 0x0;
	UdcHWShutdown();
#endif
}


/**********************************
**		U2D Specific Calls       **
***********************************/

void PlatformU2DInit()
{
#if USB2
	// clear out OTG registers
	//*U2DOTGCR_ = 0x18;

	*OSCC |= 0x800; // turn on CLK_POUT

	//setup ULPI Pads
	ConfigRegSetup(u2d_pins);

	//Enable ULPI wrapper
	//*U2DOTGCR_ |= 0x1;
	//Enable line status interrupts
	//*U2DOTGICR_ |= 0x00007E7E;
	//Make sure we're in client mode
	//*U2DOTGUCR_ = 0x030A0000;
	//Initialize U2D software variables
	U2DDownloadInit();
	//turn on hardware...i think
	BootRomU2DHWSetup();
#endif
}

UINT_T PlatformU2DTransmit(UINT_T length, UINT8_T* Buff, UINT_T WaitState)
{
#if USB2
	return U2DTransmit(length, Buff, WaitState);
#else
	return NoError;
#endif
}

void PlatformU2DInterruptHandler()
{
#if USB2
	ImageU2DInterruptHandler();
#endif
}

void PlatformU2DShutdown()
{
#if USB2
	//Disable UTMI interface
	//*U2DOTGCR_ |= 0x10;
	//Disconnect pull-ups
	//*U2DOTGUCR_ = 0x020A0600;
	//Make sure all ULPI interrupts are off
	//*U2DOTGICR_ = 0;
	//Re-enable UTMI interface - forces PHY into a reset
	//*U2DOTGCR_ &= ~0x10;
	BootRomU2DHWShutdown();
	//Turn off ULPI wrapper
	//*U2DOTGCR_ = 0;
	*OSCC &= ~0x800; // turn off CLK_POUT
#endif
}



