;/******************************************************************************
;**	(C) Copyright 2007 Marvell International Ltd. 
;**	All Rights Reserved
;******************************************************************************/

;******************************************************************************
;**
;**  COPYRIGHT (C) 2006 Intel Corporation.
;**
;**  This software as well as the software described in it is furnished under
;**  license and may only be used or copied in accordance with the terms of the
;**  license. The information in this file is furnished for informational use
;**  only, is subject to change without notice, and should not be construed as
;**  a commitment by Intel Corporation. Intel Corporation assumes no
;**  responsibility or liability for any errors or inaccuracies that may appear
;**  in this document or any software that may be provided in association with
;**  this document.
;**  Except as permitted by such license, no part of this document may be
;**  reproduced, stored in a retrieval system, or transmitted in any form or by
;**  any means without the express written consent of Intel Corporation.
;**
;**  FILENAME:	ddr_init.s
;**
;**  PURPOSE: 	DDR initialization code
;**
;******************************************************************************

    INCLUDE xlli_platform_defs.INC
    INCLUDE xlli_board_defs.INC

   EXPORT DDRInit

    IMPORT mini_xlli_Dmem_initP1
    IMPORT mini_xlli_Dmem_initP2
    IMPORT mini_xlli_Rcomp
    IMPORT mini_xlli_DDR_HCAL_EVAL



   AREA    |text|, CODE, READONLY, ALIGN=5         ; Align =5 required for "ALIGN 32" feature to work.

DDRInit	FUNCTION

 	;   Set the core Frequency and the DDR data rate

    IF :DEF: xlli_stack_pointer     ; If stack pointer has been defined
    stmfd   sp!,    {r0-r11, lr}            ; Save r0-r11, & link on stack
    ENDIF

;;;;;;;;;;;; Start Tavor Specific

    ;init ddr using mini_xlli
    ; Initialize the Monahans DMEMC(1)
    ldr     r4,     =xlli_DMEMREGS_PHYSICAL_BASE    ; Dynamic Memory Control Reg Base

    ldr    r0, [r4, #xlli_MDCNFG_offset]
    ldr    r1, =0x8000077F                      ; clear bit 30, DMC_EN
    and    r1, r1, r0
    str    r1, [r4,#xlli_MDCNFG_offset]

    ldr    r0, [r4, #xlli_MDCNFG_offset]
    ldr    r1, =0xC000077C                      ; clear bits 0,1.  Disable both partitions
    and    r1, r1, r0
    str    r1, [r4,#xlli_MDCNFG_offset]

    ldr    r0, [r4, #xlli_MDCNFG_offset]
    ldr    r1, =0x00000000                      ; clear all bits at this point.
    and    r1, r1, r0
    str    r1, [r4,#xlli_MDCNFG_offset]
    ldr    r0, [r4, #xlli_MDCNFG_offset]       ; final read-back

    ;----------------------------------------------------------
    ; ACCR Register Fields:
    ;
    ;   XL    - bits 0-4   Run mode multiplier (Xscale PLL) (0x4, 0x8, 0xc, 0x10, 0x18, 0x1F)
    ;   XN    - bits 8-10  Turbo mode (0x1, 0x2, 0x4) - if turbo enabled
    ;   DMCFS - bits 12-13 DDR Freq (0=26, 1=156, 2=208, 3=260)
    ;   HSIO  - bits 14-15 HighSpeed IO (0=104, 1=156, 2=208)
    ;   SFLFS - bits 18-19 SRAM Freq (0=104, 1=156, 2=208, 3=312)
    ;   SMCFS - bits 23-25 Static Memory Freq-XIP (0=78, 2=104, 4=156, 5=208)
    ;----------------------------------------------------------
    ;
    ; Core Freq:        Turbo DISABLED - XL * 13
    ;                   Turbo ENABLED  - XL * 13 * XN
    ; Set the core Frequency to 624 MHz and the DDR data rate to 260 MHz.
    ;
    ldr     r1,     =xlli_CLKREGS_PHYSICAL_BASE         ; Get clocks register base

    ;ldr     r2,     =0x1108                             ; Xscale = 104, DDR = 156, SRAM=104, XIP=78
    ;ldr     r2,     =0x2108                             ; Xscale = 104, DDR = 208, SRAM=104, XIP=78
    ldr     r2,     =0x3108                             ; Xscale = 104, DDR = 260, SRAM=104, XIP=78

    ;ldr     r2,     =0x0104140c                         ; Xscale = 156/624 (turbo 624), DDR = 156, SRAM=156, XIP=104
    ;ldr     r2,     =0x0104240c                         ; Xscale = 156/624 (XL=12, XN=3, Freq=12*13=156, turbo=12*13*4=624), DDR = 208, SRAM=156, XIP=104
    ;ldr     r2,     =0x0104340c                         ; Xscale = 156/624 (turbo 624), DDR = 260, SRAM=156, XIP=104
    ;ldr     r2,     =0x0104220c                         ; Xscale = 156/624 (XL=24, XN=2, Freq=12*13=312, turbo=24*13*2=624), DDR = 208, SRAM=156, XIP=104
    ;ldr     r2,     =0x01042218                         ; Xscale = 312/624 (XL=24, XN=2, Freq=12*13=312, turbo=24*13*2=624), DDR = 208, SRAM=156, XIP=104
    ;ldr     r2,     =0x092FB218                         ; Xscale = 312/624 (XL=24, XN=2, Freq=24*13=312, turbo=24*13*2=624), DDR = 260, HSIO = 208MHz, SRAM=312, XIP=104

    str     r2,     [r1, #xlli_ACCR_offset]             ; Write onto ACCR register

    ;Loop until command is written before freq change
SINK
    ldr     r0,     [r1, #xlli_ACCR_offset]             ; Read ACCR register
    and     r0, r0, r2
    cmp     r0, r2
    bne     SINK

    ;Enabling Turbo & F... modes
 ;   mov     r2,  #3                                     ; Set the F,T bits
 ;   mcr     p14, 0, r2, c6, c0, 0                       ; initiate the frequency change sequence


    ; Initialize the Monahans DMEMC(2)
    ;
    ldr     r4,     =xlli_DMEMREGS_PHYSICAL_BASE    ; Dynamic Memory Control Reg Base

    ;
    ; Configure the EMPI and DMC
    ;
    ;;;;;;;;;;
    ; comment Configuration of EMPI and DMC for QT testing
    ;;;;;;;;;;
    bl      mini_xlli_Dmem_initP1                        ; init DDR-SDRAM (Part 1)

    ldr     r0,     =xlli_DMEMREGS_PHYSICAL_BASE    ; Dynamic Mem Controller base


    ;;;;;;;;;;;;;
    ;; THESE VALUES NEED TO BE FILLED IN!!!
	;;;;;;;;;;;;;
	ldr		r5,     =0x10		; PCODE value
	ldr		r6,		=0x10		; NCODE value
	ldr		r7,		=0x6		; XCODE value
	ldr		r8,     =0x01		; SLEWRATE
	;;;;;;;;;;;;;;


    bl      mini_xlli_Rcomp

    ldr     r0,     =xlli_DMEMREGS_PHYSICAL_BASE    ; Dynamic Mem Controller base
    bl      mini_xlli_Dmem_initP2                        ; init DDR-SDRAM (Part 2)
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ; Additional conf
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ldr     r0,     =xlli_DMEMREGS_PHYSICAL_BASE	; Dynamic Mem Controller base
    ldr     r1,     =xlli_SDRAM_PHYSICAL_BASE		; DDR SDRAM base address
    bl      mini_xlli_DDR_HCAL_EVAL					; Perform HW Strobe line evaluation



;;;;;;;;;; End Tavor specific

    IF :DEF: xlli_stack_pointer     ; If stack pointer has been defined,
    ldmfd   sp!,    {r0-r11, pc}            ; Return to caller via the stack
    ELSE
    mov     pc,     lr                      ; Non stack return
    ENDIF

	ENDFUNC
   END



