.EXPORT_ALL_VARIABLES:
TOPDIR :=$(shell pwd)/..
INCLUDE_DIRS = -I $(TOPDIR)/DMA 	\
	-I $(TOPDIR)/Download		\
	-I $(TOPDIR)/DSVL		\
	-I $(TOPDIR)/Flash		\
	-I $(TOPDIR)/MainProgram 	\
	-I $(TOPDIR)/Misc 		\
	-I $(TOPDIR)/SecureBoot		\
	-I $(TOPDIR)/StartUp

TRUSTED = 0
WINCE = 0
LINUX = 0
NOR_ = 0
NAND = 0
ONENAND = 0
MSYS = 0
SDRAM = 0
RESUME = _
BUILD = Release

#USB flags for 1.1, 2.0, and Chip Idea: default disabled
USB1 = 0
USB2 = 0
USBCI = 0


ifeq "$(PLATFORM)" "TAVOR"
ifeq "$(BOARD_TYPE)" "EVB_PV"
USB1  = 1
USB2  = 0
USBCI = 1
else
ifeq "$(BOARD_TYPE)" "SAAR_PV"
USB1  = 1
USB2  = 0
USBCI = 1
else
USB1  = 1
USB2  = 0
USBCI = 0
endif
endif


endif

#Set TAVOR specific flags to a default value, so MH builds don't break
SW_UPGRADE_ENABLE = 1
ASM_CHIP_FLAG = PPPP SETA 1

ifeq	"$(OPTION)" "SRAM"
	StartUp = startup_sram.lds
else
	StartUp = startup.lds
endif


CROSS_COMPILE = arm-linux-
CFLAGS	= -w -g -Os -fpic -fasm -nostdinc -fno-builtin -mcpu=xscale \
	-DTRUSTED=$(TRUSTED) -DOBM=1 -DDKB=0 -DBOOTROM=0 \
	-DUSB1=$(USB1) -DUSB2=$(USB2) -DUSBCI=$(USBCI) -DJTAG_USED=$(TRUSTED) \
	-DNOR_CODE=$(NOR_) -DNAND_CODE=$(NAND) -DONENAND_CODE=$(ONENAND) -DMDOC_CODE=$(MSYS) 

AFLAGS  = -Wall -mcpu=xscale -mapcs-stack-check
LDFLAGS = -e __main -nostdlib -format elf32-littlearm -T $(StartUp) -O -s --gc-sections

INCLUDE_DIRS += -I $(TOPDIR)/$(PLATFORM)/INC -I $(TOPDIR)/$(PLATFORM)/DDR
OUTDIR = $(TOPDIR)/$(PLATFORM)/$(BUILD)

ifeq	"$(PLATFORM)" "TAVOR"

	ifeq	"$(OBM_MODE)" "MODE0"
	ObmMode = MODE0
	else
	ifeq	"$(OBM_MODE)" "MODE2"
	ObmMode = MODE2
	else
	ifeq	"$(OBM_MODE)" "MODE3"
	ObmMode = MODE3
	else
	ifeq	"$(OBM_MODE)" "MODE4"
	ObmMode = MODE4
	else
	ifeq	"$(OBM_MODE)" "MODE6"
	ObmMode = MODE6
	else
	ObmMode = MODE1
	endif
	endif
	endif
	endif
	endif



	ifeq	"$(FLASH_TYPE)" "NOFLASH"
	FlashType = noflash
	Flash_Type = NOFLASH
	NOR_ = 0
	MSYS = 0
	ONENAND = 0
	NAND = 0
	else
	ifeq	"$(FLASH_TYPE)" "NOR"
	FlashType = nor
	Flash_Type = NOR
	NOR_ = 1
	MSYS = 0
	ONENAND = 0
	NAND = 0
	else
	ifeq	"$(FLASH_TYPE)" "MDOC"
	FlashType = mdoc
	Flash_Type = MDOC
	NOR_ = 0
	MSYS = 1
	ONENAND = 0
	NAND = 0
	else
	ifeq	"$(FLASH_TYPE)" "ONENAND"
	FlashType = oneNand
	Flash_Type = ONENAND
	NOR_ = 0
	MSYS = 0
	ONENAND = 1
	NAND = 0
	else
	ifeq	"$(FLASH_TYPE)" "NAND16"
	FlashType = nand16
	Flash_Type = NAND16
	NOR_ = 0
	MSYS = 0
	ONENAND = 0
	NAND = 1
	else
	FlashType = nand8
	Flash_Type = NAND8
	NOR_ = 0
	MSYS = 0
	ONENAND = 0
	NAND = 1
	endif
	endif
	endif
	endif
	endif




	ifeq	"$(BOARD_TYPE)" "YARDEN"
	boardType = YARDEN
	SW_UPGRADE_ENABLE = 0
	CFLAGS += -DYARDEN
	else
	ifeq	"$(BOARD_TYPE)" "EVB"
	boardType = EVB
	CFLAGS += -DEVB2
	else
	ifeq	"$(BOARD_TYPE)" "SAAR"
	boardType = SAAR
	CFLAGS += -DSAAR
	else
	ifeq	"$(BOARD_TYPE)" "SAAR_PV"
	boardType = SAAR_PV
	AFLAGS += --defsym PV=1
	CFLAGS += -DSAAR_PV -DPV_CHIP -DDDR_WORKAROUND
	else
	ifeq	"$(BOARD_TYPE)" "EVB_PV"
	boardType = EVB_PV
	AFLAGS += --defsym PV=1
	CFLAGS += -DPV -DPV_CHIP
	else
	ifeq	"$(BOARD_TYPE)" "P65"
	boardType = P65
	else
	boardType = B0
	endif
	endif
	endif
	endif
	endif
	endif



	ifeq	"$(OPTION)" "DDR"
	OptionDir = _DDR
	else
	ifeq	"$(OPTION)" "SRAM"
	OptionDir = _SRAM
	else
	OptionDir =
	endif
	endif


	OUTDIR = $(TOPDIR)/$(PLATFORM)/$(BUILD)/$(boardType)/$(Flash_Type)


endif

ifneq "$(PRODUCT)" "None"
ifeq "$(PRODUCT)" "SPARK"
	ProductType =  SPARK
else 
ifeq "$(PRODUCT)" "BLAZE"
	ProductType = BLAZE 
else 
ifeq "$(PRODUCT)" "BENZINA"
	ProductType = BENZINA
endif
endif
endif
endif

ifeq "$(PRODUCT)" "None"
	ProductType = NONE
endif



ifeq	"$(TRUSTED)" "1"
	INCLUDE_DIRS += -I $(TOPDIR)/DSVL -I $(TOPDIR)/SecureBoot
	TYPE = T
	LIBS = $(TOPDIR)/DSVL/WtmDSVL.a
	SECUREOBJS = jtag.o obm_security.o
else
	TYPE = NT
	LIBS =
	SECUREOBJS =
endif

ifeq	"$(WINCE)" "1"
	AFLAGS += --defsym WINCE_BUILD=1
	RESUME = "_WINCE_"
endif
ifeq	"$(LINUX)" "1"
	AFLAGS += --defsym LINUX_BUILD=1
##	RESUME = "_LINUX_"
endif

ifeq	"$(SDRAM)" "128"
	AFLAGS += --defsym ENABLE_CS1_SDRAM=1
else
	AFLAGS += --defsym SINGLE_BANK_SDRAM=0
endif

ifeq	"$(ONENAND)" "1"
	INCLUDE_DIRS += -I $(TOPDIR)/Flash/FlexOneNand
endif

ifeq	"$(MSYS)" "1"
	INCLUDE_DIRS += -I $(TOPDIR)/Flash/M-SYS
endif

ifeq	"$(USBCI)"	"1"
	INCLUDE_DIRS += -I $(TOPDIR)/Download/USB2CI/src -I $(TOPDIR)/Download/USB2CI/inc
endif


INCLUDE_DIRS += -I $(TOPDIR)/Flash/DFC


#####platform specific
ifeq	"$(PLATFORM)" "TAVOR"
	CFLAGS += -D$(ObmMode) -DTAVOR_BOERNE -D$(BOARD_TYPE) -D$(ProductType)
	TAVOROBJS = PPSet.o pinmux.o i2c.o
	INCLUDE_DIRS += -I $(TOPDIR)/$(PLATFORM)/I2C -I $(TOPDIR)/$(PLATFORM)/pin_mux/INC -I $(TOPDIR)/$(PLATFORM)/obm_linux
	OBM = $(OUTDIR)/$(PLATFORM)$(RESUME)$(boardType)_$(TYPE)OBM_$(Flash_Type)_$(ObmMode)$(OptionDir)
else
	OBM = $(OUTDIR)/$(PLATFORM)$(RESUME)$(TYPE)OBM
endif

ifeq "$(SW_UPGRADE_ENABLE)" "1"
CFLAGS += -DRUN_SW_UPGRADE
endif


.PHONY: clean
## Call perl script to make sure assembly files are in GNU format ##
tmp :=$(shell perl makegnuassembly.pl $(PLATFORM))
tmp :=$(shell perl makegnuassembly.pl $(PLATFORM)/INC)
tmp :=$(shell perl makegnuassembly.pl $(PLATFORM)/DDR)
tmp :=$(shell perl makegnuassembly.pl StartUp)
ifeq "$(PLATFORM)" "TAVOR"
	ifeq "$(USBCI)" "1"
		tmp :=$(shell perl makegnuassembly.pl Download/USB2CI/src)
	endif
endif

#USB Objects breakdown
ifeq "$(USB1)" "1"
USB1OBJS = usbdownload.o \
      usbdriver.o \
      xllp_udc.o
else
USB1OBJS =
endif

ifeq "$(USB2)" "1"
USB2OBJS = u2ddriver.o \
      u2download.o \
      xllp_u2d.o \
      xllp_u2d_dma.o
else
USB2OBJS =
endif

ifeq "$(USBCI)" "1"
USBCIOBJS =  ci2download.o \
        ci2driver.o   \
	xllp_ciu2d.o \
        mvUsbDevCh9.o   \
        usbPal.o \
        mvUsbDevMain.o \
        mvUsbDevRecv.o \
        mvUsbDevSend.o \
        mvUsbDevUtl.o \
        mvUsbHsDevCncl.o \
        mvUsbHsDevUtl.o \
        mvUsbHsDevMain.o
else
USBCIOBJS =
endif


#Object Definitions
DOWNLOADOBJS = protocolmanager.o \
      uartdownload.o \
      uart.o \
      usb_descriptors.o

ifeq "$(MSYS)" "1"
M_SYS_OBJS = boot.o \
             msys_interface.o \
             msys_chipselect.o
else
M_SYS_OBJS =
endif

ifeq "$(ONENAND)" "1"
ONENANDOBJS = OneNAND_Driver.o \
		OneNAND_Cmd.o \
		FlexOneNAND_Cmd.o \
		FlexOBMInterface.o
else
ONENANDOBJS =
endif

ifeq "$(NAND)" "1"
NANDOBJS = xllp_dfc.o \
           xllp_dfc_support.o
else
NANDOBJS =
endif

MAINOBJS = obm.o \
           interrupts.o

FLASHOBJS = flash.o \
	    obm_flash.o

MISCOBJS = misc.o


ifeq "$(PLATFORM)" "TAVOR"
I2COBJS = i2c.o
endif


PLATFORMOBJS = platformconfig.o \
			platform_obm.o
BORQSOBJS =    borqs_obm_misc.o \
		powerreason.o \
		PRDCFG_Element_lookup_tbl.o

ifeq	"$(BOARD_TYPE)" "SAAR"
	BORQSLINUXBOJS = obm_pxa930.o \
        obm_pxa930_gpio.o \
      	obm_micco.o \
       	obm_pxa_lcd_jades.o \
       	obm_pxa_charger.o \
       	obm_pxa_ssp_jades.o \
      	obm_pxa_lcd_printf.o \
      	obm_pxa_font.o \
      	gunzip.o  
else
ifeq	"$(BOARD_TYPE)" "SAAR_PV"
		BORQSLINUXBOJS = obm_pxa930.o \
        	obm_pxa930_gpio.o \
        	obm_micco.o\
			obm_pxa_lcd_spark.o \
       		obm_pxa_charger.o \
       		obm_pxa_ssp_spark.o \
      		obm_pxa_lcd_printf.o \
      		obm_pxa_font.o \
      		gunzip.o  
else
endif  
endif  

ifeq "$(PLATFORM)" "TAVOR"
	PLATFORMOBJS +=	pinmux.o
endif

DMAOBJS = xllp_dmac.o

ifeq "$(TRUSTED)" "1"
SECUREOBJS = obm_security.o \
		jtag.o
else
SECUREOBJS =
endif

DDROBJS = mini_xlli_lowlev_init.o	\
	ddr_init.o

OBJS = obm_startup.o \
      $(DDROBJS) \
      $(DOWNLOADOBJS) \
      $(USB1OBJS) \
      $(USB2OBJS) \
      $(USBCIOBJS) \
      $(NANDOBJS) \
      $(MAINOBJS) \
      $(FLASHOBJS) \
      $(MISCOBJS) \
      $(DMAOBJS) \
      $(PLATFORMOBJS) \
      $(BORQSOBJS)            \
      $(BORQSLINUXBOJS) \
      $(M_SYS_OBJS) \
      $(ONENANDOBJS) \
      $(FLEXOBJS) \
      $(SECUREOBJS)

ifeq "$(USBCI)" "1"
	OBJS += usbPals.o
endif

ifeq "$(PLATFORM)" "TAVOR"
	OBJS += $(I2COBJS) PPSet.o
endif

AS	=$(CROSS_COMPILE)as $(AFLAGS)
LD	=$(CROSS_COMPILE)ld $(LDFLAGS)
CC	=$(CROSS_COMPILE)gcc $(CFLAGS)
STRIP	=$(CROSS_COMPILE)strip -R .note -R .comment -S
OBJCOPY	=$(CROSS_COMPILE)objcopy -O binary -R .note -R .comment -S
OBJDUMP	=$(CROSS_COMPILE)objdump
MAKE	=make

OBJECTS = $(patsubst %.o, $(OUTDIR)/%.o, $(OBJS))

"$(OBM).bin" :	$(OBJS)
		@$(LD) -o "$(OBM)-elf32" $(OBJECTS) $(LIBS) -Map "$(OBM).map"
		@$(STRIP) "$(OBM)-elf32" -o "$(OBM)-elf32-stripped"
		@$(OBJCOPY) "$(OBM)-elf32-stripped" "$(OBM).bin"
		@echo Build Successful!


obm_startup.o: $(TOPDIR)/StartUp/obm_startup.S
	@$(AS) --MD dump.txt $(INCLUDE_DIRS) "$(TOPDIR)/StartUp/obm_startup.S" -o "$(OUTDIR)/obm_startup.o" > dump.txt

$(DDROBJS): %.o : $(TOPDIR)/$(PLATFORM)/DDR/%.S
	@$(AS) --MD dump_$*.txt $(INCLUDE_DIRS) "$(TOPDIR)/$(PLATFORM)/DDR/$*.S" -o "$(OUTDIR)/$*.o" > dump.txt

$(DOWNLOADOBJS) $(USB1OBJS) $(USB2OBJS): %.o : $(TOPDIR)/Download/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/Download/$*.c" -o "$(OUTDIR)/$*.o"

$(MAINOBJS): %.o : $(TOPDIR)/MainProgram/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/MainProgram/$*.c" -o "$(OUTDIR)/$*.o"

$(FLASHOBJS): %.o : $(TOPDIR)/Flash/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/Flash/$*.c" -o "$(OUTDIR)/$*.o"

$(MISCOBJS): %.o : $(TOPDIR)/Misc/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/Misc/$*.c" -o "$(OUTDIR)/$*.o"

$(PLATFORMOBJS): %.o : $(TOPDIR)/$(PLATFORM)/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/$(PLATFORM)/$*.c" -o "$(OUTDIR)/$*.o"

$(BORQSOBJS): %.o : $(TOPDIR)/$(PLATFORM)/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/$(PLATFORM)/$*.c" -o "$(OUTDIR)/$*.o"

$(BORQSLINUXBOJS): %.o : $(TOPDIR)/$(PLATFORM)/obm_linux/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/$(PLATFORM)/obm_linux/$*.c" -o "$(OUTDIR)/$*.o"

$(DMAOBJS): %.o : $(TOPDIR)/DMA/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/DMA/$*.c" -o "$(OUTDIR)/$*.o"

ifeq "$(TRUSTED)" "1"
$(SECUREOBJS): %.o : $(TOPDIR)/SecureBoot/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/SecureBoot/$*.c" -o "$(OUTDIR)/$*.o"
endif

ifeq "$(NAND)" "1"
$(NANDOBJS): %.o : $(TOPDIR)/Flash/DFC/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/Flash/DFC/$*.c" -o "$(OUTDIR)/$*.o"
endif

ifeq "$(ONENAND)" "1"
$(ONENANDOBJS): %.o : $(TOPDIR)/Flash/FlexOneNand/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/Flash/FlexOneNand/$*.c" -o "$(OUTDIR)/$*.o"
endif

ifeq "$(MSYS)" "1"
$(M_SYS_OBJS): %.o : $(TOPDIR)/Flash/M-SYS/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/Flash/M-SYS/$*.c" -o "$(OUTDIR)/$*.o"
endif

ifeq "$(USBCI)" "1"
$(USBCIOBJS): %.o : $(TOPDIR)/Download/USB2CI/src/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/Download/USB2CI/src/$*.c" -o "$(OUTDIR)/$*.o"

usbPals.o: $(TOPDIR)/Download/USB2CI/src/usbPals.S
	@$(AS) $(INCLUDE_DIRS) "$(TOPDIR)/Download/USB2CI/src/usbPals.S" -o "$(OUTDIR)/usbPals.o" > dump.txt
endif

########### TAVOR COMPILED ONLY   #################
ifeq	"$(PLATFORM)" "TAVOR"

PPSet.o : $(TOPDIR)/$(PLATFORM)/DDR/PPSet.S
	@$(AS) $(INCLUDE_DIRS) "$(TOPDIR)/$(PLATFORM)/DDR/PPSet.S" -o "$(OUTDIR)/PPSet.o" > dump.txt

i2c.o : $(TOPDIR)/$(PLATFORM)/I2C/i2c.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/$(PLATFORM)/I2C/i2c.c" -o "$(OUTDIR)/i2c.o"

endif


clean:
	@echo "OBM make clean..."
	rm -rf $(TOPDIR)/TAVOR/Release/SAAR $(TOPDIR)/TAVOR/Release/SAAR_PV
	rm -f $(TOPDIR)/Build/*txt
	rm -f $(TOPDIR)/StartUp/*S
	rm -f $(TOPDIR)/StartUp/*INC 
	rm -f $(TOPDIR)/StartUp/*MAC 
	rm -f $(TOPDIR)/TAVOR/DDR/*S
	rm -rf $(TOPDIR)/TAVOR/BIN/SAAR
	rm -rf $(TOPDIR)/TAVOR/BIN/SAAR_PV
	rm -f $(TOPDIR)/TAVOR/DDR/*INC 
	rm -f $(TOPDIR)/TAVOR/memmap.INC 
	rm -f $(TOPDIR)/TAVOR/tbb_bin_blk1/*bin




