/*******************************************************************************
** File          : $HeadURL$
** Author        : $Author$
** Project       : HSCTRL
** Instances     :
** Creation date :
********************************************************************************
********************************************************************************
** ChipIdea Microelectronica - IPCS
** TECMAIA, Rua Eng. Frederico Ulrich, n 2650
** 4470-920 MOREIRA MAIA
** Portugal
** Tel: +351 229471010
** Fax: +351 229471011
** e_mail: chipidea.com
********************************************************************************
** ISO 9001:2000 - Certified Company
** (C) 2005 Copyright Chipidea(R)
** Chipidea(R) - Microelectronica, S.A. reserves the right to make changes to
** the information contained herein without notice. No liability shall be
** incurred as a result of its use or application.
********************************************************************************
** Modification history:
** $Date$
** $Revision$
*******************************************************************************
*** Description:
***  This file contains USB device chapter9 specific functions general for
*** all devices
***
**************************************************************************
**END*********************************************************/

#include "mvUsbDevApi.h"
#include "mvUsbDevPrv.h"
#include "mvUsbCh9.h"
#include "xllp_u2d.h"
#include "xllp_ciu2d.h"
#include "mvUsbDebug.h"
#include "usb_descriptors.h"
extern UINT_T FindVendorRequestInTIM (XLLP_UINT8_T bRequest, XLLP_UINT8_T bmRequestType,
                                       XLLP_UINT32_T *pTxBuffEp0, XLLP_INT32_T *ppacketLength);
extern OPT_USB_DESCRIPTOR_LOADS USBDescriptors[NUM_USB_DESCRIPTORS];
extern UINT_T   USBStringIndexTable[MAX_USB_STRINGS];
extern XLLP_U2D_EP_CONFIG_TABLE_T defaultU2DEndpointConfigTable[];
extern UINT_T ci2ErrorFlags;
static volatile boolean  ENTER_TEST_MODE = FALSE;
static volatile uint_16  test_mode_index = 0;

//------------------------------------------------------------------
//
//
//------------------------------------------------------------------
void mvUsbCh9GetStatus(_usb_device_handle handle, boolean setup,
                         SETUP_STRUCT* ctrl_req)
{ /* Body */
    uint_8   endpoint;
    uint_16 mv_usb_status=0; // default value

    MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_SETUP, "%s: setup=%d\n", __FUNCTION__, (int)setup);

    if(!setup)
        return;

    switch (ctrl_req->REQUESTTYPE)
    {
       case (REQ_DIR_IN | REQ_RECIP_DEVICE):
          /* Device request */
          _usb_device_get_status(handle, MV_USB_STATUS_DEVICE, &mv_usb_status);
          break;

       case (REQ_DIR_IN | REQ_RECIP_INTERFACE):
          /* Interface request */
          _usb_device_get_status(handle, MV_USB_STATUS_INTERFACE, &mv_usb_status);
          break;

       case (REQ_DIR_IN | REQ_RECIP_ENDPOINT):
          /* Endpoint request */
          endpoint = ctrl_req->INDEX & MV_USB_STATUS_ENDPOINT_NUMBER_MASK;

          _usb_device_get_status(handle, MV_USB_STATUS_ENDPOINT | endpoint, &mv_usb_status);
          break;

       default:
          /* Unknown request */
           MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"%s: Unknown request type 0x%x\n", __FUNCTION__, ctrl_req->REQUESTTYPE);
           CI2_SET_ERROR_BIT( CI2_ERROR_UNHANDLED_STATUS );
          return;
    } /* Endswitch */

    /* Send the requested data */
    mv_usb_status = USB_16BIT_LE(mv_usb_status);
    _usb_device_send_data(handle, 0, (uint_8_ptr)&mv_usb_status, 2 /*size*/);

    /* status phase */
    _usb_device_recv_data(handle, 0, NULL, 0);

    return;
} /* Endbody */

//------------------------------------------------------------------
//  This function is junk ?!?
//
//------------------------------------------------------------------
void mvUsbCh9ClearFeature(_usb_device_handle handle, boolean setup,
                            SETUP_STRUCT* setup_ptr)
{ /* Body */
    uint_8   endpoint;
    uint_16  usb_status;

    MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_SETUP, "%s: setup=%d\n", __FUNCTION__, (int)setup);

    _usb_device_get_status(handle, MV_USB_STATUS_DEVICE_STATE, &usb_status);
    
    // this if statement will always be true... makes no sense  perhaps || instead of && ?
    //---------------------------------------------------------------------------------------
    if ((usb_status != MV_USB_STATE_CONFIG) && (usb_status != MV_USB_STATE_ADDRESS))
    {
        MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"%s: Wrong state %d\n", __FUNCTION__, usb_status);
        _usb_device_stall_endpoint(handle, 0, MV_USB_RECV);
        return;
    } /* Endif */

    if(!setup)
        return;

    switch (setup_ptr->REQUESTTYPE)
    {
        case (REQ_DIR_OUT | REQ_RECIP_DEVICE):
            /* DEVICE */
            switch(setup_ptr->VALUE)
            {
                case DEVICE_REMOTE_WAKEUP:
                    /* clear remote wakeup */
                    _usb_device_get_status(handle, MV_USB_STATUS_DEVICE, &usb_status);
                    usb_status &= ~MV_USB_REMOTE_WAKEUP;
                    _usb_device_set_status(handle, MV_USB_STATUS_DEVICE, usb_status);
                    break;

                case DEVICE_TEST_MODE:
                    /* Exit Test Mode */
                    _usb_device_set_status(handle, MV_USB_STATUS_TEST_MODE, 0);
                    break;

                default:
                    MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"%s: Unknown Device feature %d\n",
                                __FUNCTION__, setup_ptr->VALUE);
                    CI2_SET_ERROR_BIT( CI2_ERROR_UNHANDLED_CLR_FEATURE );
                    return;
            } /* Endif */
            break;

        case (REQ_DIR_OUT | REQ_RECIP_ENDPOINT):
            /* ENDPOINT */
            if (setup_ptr->VALUE != ENDPOINT_HALT)
            {
                MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"%s: Wrong Endpoint feature %d\n",
                            __FUNCTION__, setup_ptr->VALUE);
                CI2_SET_ERROR_BIT( CI2_ERROR_UNHANDLED_CLR_FEATURE );
                return;
            } /* Endif */

            endpoint = setup_ptr->INDEX & MV_USB_STATUS_ENDPOINT_NUMBER_MASK;
            /* unstall */
            _usb_device_set_status(handle, MV_USB_STATUS_ENDPOINT | endpoint, 0);
            break;

        default:
            MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"%s: Unknown REQUEST_TYPE %d\n",
                                __FUNCTION__, setup_ptr->REQUESTTYPE);
            CI2_SET_ERROR_BIT( CI2_ERROR_UNHANDLED_CLR_FEATURE );
                        //_usb_device_stall_endpoint(handle, 0, MV_USB_RECV);
            return;
    } /* Endswitch */

    /* status phase */
    _usb_device_send_data(handle, 0, 0, 0);
}

//---------------------------------------------------------------------------------------
//
//
//---------------------------------------------------------------------------------------
void  mvUsbCh9SetFeature(_usb_device_handle handle, boolean setup,SETUP_STRUCT* setup_ptr)
{
   uint_16                  usb_status;
   uint_8                   endpoint;
   USB_DEV_STATE_STRUCT*    usb_dev_ptr = (USB_DEV_STATE_STRUCT*)handle;

   MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_SETUP, "%s: setup=%d\n", __FUNCTION__, (int)setup);

   if (setup)
   {
      switch (setup_ptr->REQUESTTYPE)
      {
         case (REQ_DIR_OUT | REQ_RECIP_DEVICE):
            /* DEVICE */
            switch (setup_ptr->VALUE)
            {
               case DEVICE_REMOTE_WAKEUP:
                  /* set remote wakeup */
                  _usb_device_get_status(handle, MV_USB_STATUS_DEVICE, &usb_status);
                  usb_status |= MV_USB_REMOTE_WAKEUP;
                  _usb_device_set_status(handle, MV_USB_STATUS_DEVICE, usb_status);
                  break;

               case DEVICE_TEST_MODE:
                  /* Test Mode */
                  if( (setup_ptr->INDEX & 0x00FF) || (usb_dev_ptr->SPEED != MV_USB_SPEED_HIGH) )
                  {
                     MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"%s: Wrong Test mode parameters: mode=%d, speed=%d\n",
                                __FUNCTION__, (setup_ptr->INDEX & 0x00FF), usb_dev_ptr->SPEED);
                     _usb_device_stall_endpoint(handle, 0, MV_USB_RECV);
                     return;
                  } /* Endif */

                  _usb_device_get_status(handle, MV_USB_STATUS_DEVICE_STATE, &usb_status);
                  if( (usb_status == MV_USB_STATE_CONFIG)  ||
                      (usb_status == MV_USB_STATE_ADDRESS) ||
                      (usb_status == MV_USB_STATE_DEFAULT))
                  {
                      /* wait with Set Test mode */
                      ENTER_TEST_MODE = TRUE;
                      test_mode_index = (setup_ptr->INDEX & 0xFF00);
                      MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_INFO,"%s: Prepare for Test mode 0x%x\n",
                                                __FUNCTION__, test_mode_index);
                  }
                  else
                  {
                     MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"%s: Wrong USB state for Test mode: state=%d\n",
                                __FUNCTION__, usb_status);
                     _usb_device_stall_endpoint(handle, 0, MV_USB_RECV);
                     return;
                  } /* Endif */
                  break;

               default:
                    MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"%s: Unknown Device feature %d\n",
                                __FUNCTION__, setup_ptr->VALUE);
                  _usb_device_stall_endpoint(handle, 0, MV_USB_RECV);
                  return;
            } /* Endswitch */
            break;

         case (REQ_DIR_OUT | REQ_RECIP_ENDPOINT):
            /* ENDPOINT */
            if (setup_ptr->VALUE != ENDPOINT_HALT)
            {
                MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"%s: Unknown Endpoint feature %d\n",
                            __FUNCTION__, setup_ptr->VALUE);
                _usb_device_stall_endpoint(handle, 0, MV_USB_RECV);
                return;
            } /* Endif */

            endpoint = setup_ptr->INDEX & MV_USB_STATUS_ENDPOINT_NUMBER_MASK;
            /* set stall */
            _usb_device_set_status(handle, MV_USB_STATUS_ENDPOINT | endpoint, 1);
            break;

         default:
            MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"%s: Unknown REQUEST_TYPE %d\n",
                       __FUNCTION__, setup_ptr->REQUESTTYPE);

            _usb_device_stall_endpoint(handle, 0, MV_USB_RECV);
            return;
      } /* Endswitch */

      /* status phase */
      _usb_device_send_data(handle, 0, 0, 0);
   }
   else
   {
      if (ENTER_TEST_MODE)
      {
         /* Enter Test Mode */
          MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_INFO,"%s: Activate Test mode 0x%x\n", __FUNCTION__, test_mode_index);
         _usb_device_set_status(handle, MV_USB_STATUS_TEST_MODE, test_mode_index);
      } /* Endif */
   } /* Endif */
}

/*FUNCTION*----------------------------------------------------------------
*
* Function Name  : ch9SetAddress
* Returned Value : None
* Comments       :
*     Chapter 9 SetAddress command
*     We setup a TX packet of 0 length ready for the IN token
*     Once we get the TOK_DNE interrupt for the IN token, then
*     we change the ADDR register and go to the ADDRESS state.
*
*END*--------------------------------------------------------------------*/
void mvUsbCh9SetAddress(_usb_device_handle handle,boolean setup, SETUP_STRUCT* setup_ptr)
{
  static uint_8  new_address;
  
  new_address = setup_ptr->VALUE;
  MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ADDR, "usbDisk %s: setup=%d, address=%d\n",
                                   __FUNCTION__, (int)setup, setup_ptr->VALUE); 
  _usb_device_set_status(handle, MV_USB_STATUS_ADDRESS, new_address);     
  _usb_device_send_data(handle, 0, 0, 0); /* ack */
}

//--------------------------------------------------------------------------------------------
// mvUsbCh9GetDescriptior()
//
//--------------------------------------------------------------------------------------------
void mvUsbCh9GetDescriptior(_usb_device_handle handle, boolean setup, SETUP_STRUCT* setup_ptr)
{
   uint_32  pktSize, stringLength, stringIndex, error;
   uint_8  *payload = "UNHANDLED";
   
   pktSize = sizeof(*payload);

   if (setup)
   {
      /* Load the appropriate string depending on the descriptor requested.*/
      switch (setup_ptr->VALUE >> 8)
      {

         case DESC_TYPE_DEVICE:
            pktSize = USBDescriptors[USB_DEVICE_DESCRIPTOR & 0xFF].DesSize;
            payload = (uint_8*)(USBDescriptors[USB_DEVICE_DESCRIPTOR & 0xFF].pDesPayload);
            break;

         case DESC_TYPE_CONFIG:
            pktSize = USBDescriptors[USB_CONFIG_DESCRIPTOR & 0xFF].DesSize;
            payload = (uint_8*)(USBDescriptors[USB_CONFIG_DESCRIPTOR & 0xFF].pDesPayload);
            break;

         case DESC_TYPE_STRING:
            stringIndex = ((setup_ptr->VALUE) & 0xFF);

            // Bounds protection
            if (stringIndex >= MAX_USB_STRINGS)
            {
                stringLength = (setup_ptr->LENGTH);
                if (stringLength > USBDescriptors[USB_DEFAULT_STRING_DESCRIPTOR & 0xFF].DesSize)
                    pktSize = USBDescriptors[USB_DEFAULT_STRING_DESCRIPTOR & 0xFF].DesSize;
                else
                    pktSize = stringLength;

                payload =(uint_8*)(USBDescriptors[USB_DEFAULT_STRING_DESCRIPTOR & 0xFF].pDesPayload);
                break;
            }

            switch (USBStringIndexTable[stringIndex])
            {
                 case USB_LANGUAGE_STRING_DESCRIPTOR:
                 case USB_MANUFACTURER_STRING_DESCRIPTOR:
                 case USB_PRODUCT_STRING_DESCRIPTOR:
                 case USB_SERIAL_STRING_DESCRIPTOR:
                 case USB_INTERFACE_STRING_DESCRIPTOR:
                 case USB_DEFAULT_STRING_DESCRIPTOR:
                      stringLength = (setup_ptr->LENGTH);
                      if (stringLength > USBDescriptors[USBStringIndexTable[stringIndex] & 0xFF].DesSize)
                        pktSize = USBDescriptors[USBStringIndexTable[stringIndex] & 0xFF].DesSize;
                      else
                        pktSize = stringLength;

                      payload = (uint_8*)(USBDescriptors[USBStringIndexTable[stringIndex] & 0xFF].pDesPayload);
                      break;
            #if 0
                case USB_LANGUAGE_STRING_DESCRIPTOR:
                    stringLength = (setup_ptr->LENGTH);
                    if (stringLength > USBDescriptors[USB_LANGUAGE_STRING_DESCRIPTOR & 0xFF].DesSize)
                        pktSize = USBDescriptors[USB_LANGUAGE_STRING_DESCRIPTOR & 0xFF].DesSize;
                    else
                        pktSize = stringLength;

                    payload = (uint_8*)(USBDescriptors[USB_LANGUAGE_STRING_DESCRIPTOR & 0xFF].pDesPayload);
                    break;

                case USB_MANUFACTURER_STRING_DESCRIPTOR:
                    stringLength = (setup_ptr->LENGTH);
                    if (stringLength > USBDescriptors[USB_MANUFACTURER_STRING_DESCRIPTOR & 0xFF].DesSize)
                        pktSize = USBDescriptors[USB_MANUFACTURER_STRING_DESCRIPTOR & 0xFF].DesSize;
                    else
                        pktSize = stringLength;

                    payload = (uint_8*)(USBDescriptors[USB_MANUFACTURER_STRING_DESCRIPTOR & 0xFF].pDesPayload);
                    break;
                case USB_PRODUCT_STRING_DESCRIPTOR:
                    stringLength = (setup_ptr->LENGTH);
                    if (stringLength > USBDescriptors[USB_PRODUCT_STRING_DESCRIPTOR & 0xFF].DesSize)
                        pktSize = USBDescriptors[USB_PRODUCT_STRING_DESCRIPTOR & 0xFF].DesSize;
                    else
                        pktSize = stringLength;

                    payload = (uint_8*)(USBDescriptors[USB_PRODUCT_STRING_DESCRIPTOR & 0xFF].pDesPayload);
                    break;
                case USB_SERIAL_STRING_DESCRIPTOR:
                    stringLength = (setup_ptr->LENGTH);
                    if (stringLength > USBDescriptors[USB_SERIAL_STRING_DESCRIPTOR & 0xFF].DesSize)
                        pktSize = USBDescriptors[USB_SERIAL_STRING_DESCRIPTOR & 0xFF].DesSize;
                    else
                        pktSize = stringLength;
                    payload = (uint_8*)(USBDescriptors[USB_SERIAL_STRING_DESCRIPTOR & 0xFF].pDesPayload);
                    break;

                case USB_INTERFACE_STRING_DESCRIPTOR:
                    stringLength = (setup_ptr->LENGTH);
                    if (stringLength > USBDescriptors[USB_INTERFACE_STRING_DESCRIPTOR & 0xFF].DesSize)
                        pktSize = USBDescriptors[USB_INTERFACE_STRING_DESCRIPTOR & 0xFF].DesSize;
                    else
                        pktSize = stringLength;

                    payload = (uint_8*)(USBDescriptors[USB_INTERFACE_STRING_DESCRIPTOR & 0xFF].pDesPayload);
                    break;
                    
                
                default:
                    stringLength = (setup_ptr->LENGTH);
                    if (stringLength > USBDescriptors[USB_DEFAULT_STRING_DESCRIPTOR & 0xFF].DesSize)
                        pktSize = USBDescriptors[USB_DEFAULT_STRING_DESCRIPTOR & 0xFF].DesSize;
                    else
                        pktSize = stringLength;

                    payload = (uint_8*)(USBDescriptors[USB_DEFAULT_STRING_DESCRIPTOR & 0xFF].pDesPayload);
                    break;
            #endif
                 default:
                   CI2_SET_ERROR_BIT(CI2_ERROR_UNHANDLED_DESCRIPTOR);
                }
            break;  // from DESC_TYPE_STRING

         case DESC_TYPE_INTERFACE:
            pktSize = USBDescriptors[USB_INTERFACE_DESCRIPTOR & 0xFF].DesSize;
            payload = (uint_8*)(USBDescriptors[USB_INTERFACE_DESCRIPTOR & 0xFF].pDesPayload);
            break;

         case DESC_TYPE_ENDPOINT:
            pktSize = USBDescriptors[USB_ENDPOINT_DESCRIPTOR & 0xFF].DesSize;
            payload = (uint_8*)(USBDescriptors[USB_ENDPOINT_DESCRIPTOR & 0xFF].pDesPayload);
            break;

         // intentional fall-thru to default to signal error
         //-----------------------------------------------------
         case DESC_TYPE_QUALIFIER:
              error = CI2_ERROR_DESC_TYPE_QUALIFIER;
              goto FALL_THRU;
            //break;

         case DESC_TYPE_OTHER_SPEED:
              error = CI2_ERROR_DESC_TYPE_OTHER_SPEED;
            //break;
            
FALL_THRU:
         default:

         // don't stall endpoint, but set an error status if we get here
         //-----------------------------------------------------------------
         CI2_SET_ERROR_BIT( error );
         
     } /* Endswitch */

      pktSize = (setup_ptr->LENGTH < pktSize) ? setup_ptr->LENGTH : pktSize;
      _usb_device_send_data (handle, MV_USB_SERVICE_EP0, payload, pktSize);

      /* status phase */
      _usb_device_recv_data(handle, 0, 0, 0);

   } /* End if(setup) */
   return;
}

//---------------------------------------------------------------------------------------
//
//
//---------------------------------------------------------------------------------------
void mvUsbCh9SetDescriptior(_usb_device_handle handle, boolean setup, SETUP_STRUCT* setup_ptr)
{
}

//---------------------------------------------------------------------------------------
//
//
//---------------------------------------------------------------------------------------
void mvUsbCh9GetConfig(_usb_device_handle handle, boolean setup, SETUP_STRUCT* setup_ptr)
{
   _usb_device_get_status(handle, MV_USB_STATUS_DEVICE_STATE, MV_USB_STATE_CONFIG);
}

//---------------------------------------------------------------------------------------
// mvUsbCh9SetConfig()
//
//  Handle the configuration request from the HOST
//---------------------------------------------------------------------------------------
void mvUsbCh9SetConfig(_usb_device_handle handle, boolean setup, SETUP_STRUCT* setup_ptr)
{
  uint_16 usb_state;
  uint_32 i;
  P_XLLP_U2D_EP_CONFIG_TABLE_T cfTable = defaultU2DEndpointConfigTable;
  USB_DEV_STATE_STRUCT_PTR      usb_dev_ptr;

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;

  if (setup)
  {
    _usb_device_get_status(handle, MV_USB_STATUS_CURRENT_CONFIG, &usb_state);
    
    // If the current configuration does not match the requested
    // configuration, deconfigure the EP's in the current config
    // and then configure the endpoints for the requested config
    //----------------------------------------------------------
    if (usb_state != (setup_ptr->VALUE & 0x00FF))
    {
      // Loop thru the config table and shutdown any non-zero EP's
      // in the CURRENT config
      //--------------------------------------------------------------
      for( i=0; (cfTable[i].udcEndpointNum != U2D_ENDPOINT_BAD ); i++)
      {
         if( cfTable[i].udcEndpointNum == 0 )
           continue;

         if( cfTable[i].usbConfigNum == usb_state)
           _usb_device_deinit_endpoint( handle, cfTable[i].udcEndpointNum, cfTable[i].usbEndpointDirection);
      }

      // Now, loop thru the config table and setup
      // all endpoints in the config
      // make sure to un-stall any enabled endpoints
      // Ep0 is self-configured by the silicon
      //--------------------------------------------
      for( i=0; (cfTable[i].udcEndpointNum != U2D_ENDPOINT_BAD ); i++)
      {
        if( cfTable[i].udcEndpointNum == 0 )
          continue;

        if( (cfTable[i].udcEndpointNum == 3) || (cfTable[i].udcEndpointNum == 4))
		{
		 	 _usb_dci_vusb20_enable_endpoint( handle, cfTable[i].udcEndpointNum, cfTable[i].maxPacketSize,
                                      cfTable[i].usbEndpointDirection, cfTable[i].usbEndpointType);
            continue;
		}
 
         if( cfTable[i].usbConfigNum == (setup_ptr->VALUE & 0x00FF))
        {
          _usb_device_init_endpoint( handle, cfTable[i].udcEndpointNum, cfTable[i].maxPacketSize,
                                      cfTable[i].usbEndpointDirection, cfTable[i].usbEndpointType,
                                       (cfTable[i].usbEndpointType == MV_USB_BULK_ENDPOINT ? MV_USB_DEVICE_DONT_ZERO_TERMINATE : 0));
                                       
          // APPLICATION SPECIFIC, if configuration number == 1, then prime Rx endpoint
          //-------------------------------------------------------------------------------------------------
          if( ((setup_ptr->VALUE & 0x00FF) == 1) && (cfTable[i].usbEndpointDirection == USB_OUT) )
            CI2_ResetBufPosition( (XLLP_U2D_EP_T)cfTable[i].udcEndpointNum );
        }
      }     
      
      // set the configuration number in the device context struct
      //----------------------------------------------------------
      _usb_device_set_status(handle, MV_USB_STATUS_CURRENT_CONFIG, (setup_ptr->VALUE & 0x00FF));
    } /* Endif */

   } /* End if (setup) */

   // sets state variable of the device controller
   //---------------------------------------------
   _usb_device_set_status(handle, MV_USB_STATUS_DEVICE_STATE, MV_USB_STATE_CONFIG);
  
   /* status phase */
   _usb_device_send_data(handle, 0, 0, 0);
    
   return;
}

void mvUsbDummyFunc(_usb_device_handle handle, boolean setup, SETUP_STRUCT* setup_ptr)
{}

//---------------------------------------------------------------------------------------
//
//
//---------------------------------------------------------------------------------------
void mvUsbCh9GetInterface(_usb_device_handle handle, boolean setup, SETUP_STRUCT* setup_ptr)
{

}

//---------------------------------------------------------------------------------------
//
//
//---------------------------------------------------------------------------------------
void mvUsbCh9SetInterface(_usb_device_handle handle, boolean setup, SETUP_STRUCT* setup_ptr)
{
}

//---------------------------------------------------------------------------------------
//
//
//---------------------------------------------------------------------------------------
void ch9Class(_usb_device_handle handle, boolean setup, SETUP_STRUCT* setup_ptr)
{
}



//---------------------------------------------------------------------------------------
// mvUsbCh9ProcessVendorRequest()
//
//
//---------------------------------------------------------------------------------------
void mvUsbCh9ProcessVendorRequest( void *handle, boolean setup, SETUP_STRUCT* setup_ptr )
{
   UINT8 ci2NoVendorString[] = "NoVendorString";
   XLLP_INT32_T pktSize, retVal= !NoError;
   void  *pPayload;
         
   retVal = FindVendorRequestInTIM (setup_ptr->REQUEST, setup_ptr->REQUESTTYPE, 
   				                  (XLLP_UINT32_T *)&pPayload, &pktSize);  

   if( retVal != NoError )
   {
     pktSize = sizeof(ci2NoVendorString);
     pPayload = (UINT8 *)ci2NoVendorString;
   }
   
   pktSize = (setup_ptr->LENGTH < pktSize) ? setup_ptr->LENGTH : pktSize;
   _usb_device_send_data (handle, MV_USB_SERVICE_EP0, (UINT8 *)pPayload, pktSize);

   /* status phase */
   _usb_device_recv_data(handle, 0, 0, 0);
}
