/*******************************************************************************
** File          : $HeadURL$
** Author        : $Author$
** Project       : HSCTRL
** Instances     :
** Creation date :
********************************************************************************
********************************************************************************
** ChipIdea Microelectronica - IPCS
** TECMAIA, Rua Eng. Frederico Ulrich, n 2650
** 4470-920 MOREIRA MAIA
** Portugal
** Tel: +351 229471010
** Fax: +351 229471011
** e_mail: chipidea.com
********************************************************************************
** ISO 9001:2000 - Certified Company
** (C) 2005 Copyright Chipidea(R)
** Chipidea(R) - Microelectronica, S.A. reserves the right to make changes to
** the information contained herein without notice. No liability shall be
** incurred as a result of its use or application.
********************************************************************************
** Modification history:
** $Date$
** $Revision$
*******************************************************************************
*** Description:
***  This file contains USB device API specific function to send data.
***
**************************************************************************
**END*********************************************************/

#include "mvUsbDevApi.h"
#include "mvUsbDevPrv.h"

extern uint_32 ci2ErrorFlags;
uint_8 ci2TxBuffer[512];
uint_32 ci2TxIndex = 0;

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_device_send_data
*  Returned Value : USB_OK or error code
*  Comments       :
*        Sends data on a specified endpoint.
*        NOTE: we double buffer to avoid Tx data on stack, so limit for
*              a Tx is 512 bytes,Bootrom currently does not exceed this
*              size for any transmits.
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_device_send_data
   (
      /* [IN] the USB_USB_dev_initialize state structure */
      _usb_device_handle         handle,

      /* [IN] the Endpoint number */
      uint_8                     ep_num,

      /* [IN] buffer to send */
      uint_8_ptr                 buff_ptr,

      /* [IN] length of the transfer */
      uint_32                    size
   )
{ /* Body */
   int 	                        lockKey;
   uint_8                       error;
   XD_STRUCT_PTR                xd_ptr;
   USB_DEV_STATE_STRUCT_PTR     usb_dev_ptr;

   MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_TX, "send_data: ep=%d, pBuf=0x%x, size=%d\n",
                                        ep_num, (unsigned)buff_ptr, (int)size);

   MV_USB_DEBUG_CODE(MV_USB_DEBUG_FLAG_STATS, (mv_usb_send_count++));

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;

   USB_dcache_flush((pointer)buff_ptr, size);

   lockKey = USB_lock();

   // Copy to the static transfer buffer
   // circular buffer, shoud be ok
   //-----------------------------------
   if( ci2TxIndex >= sizeof(ci2TxBuffer) )
     ci2TxIndex = 0;
      
   if( size <= (sizeof(ci2TxBuffer)-ci2TxIndex-1) )
   {
     memcpy((ci2TxBuffer+ci2TxIndex), buff_ptr, size);
   }
   else
   {
     size = (size <= sizeof(ci2TxBuffer)) ? size : sizeof(ci2TxBuffer);
     ci2TxIndex = 0;
     memcpy((ci2TxBuffer+ci2TxIndex), buff_ptr, size);
     CI2_SET_ERROR_BIT(CI2_ERROR_TX_BUFF_TOO_SMALL);
   }
   
   if (!usb_dev_ptr->XD_ENTRIES)
   {
      USB_unlock(lockKey);
      MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"_usb_device_send_data, transfer in progress\n");
      return MV_USB_STATUS_TRANSFER_IN_PROGRESS;
   } /* Endif */

   /* Get a transfer descriptor */
   USB_XD_QGET(usb_dev_ptr->XD_HEAD, usb_dev_ptr->XD_TAIL, xd_ptr);

   usb_dev_ptr->XD_ENTRIES--;

   /* Initialize the new transfer descriptor */
   xd_ptr->EP_NUM = ep_num;
   xd_ptr->BDIRECTION = MV_USB_SEND;
   xd_ptr->WTOTALLENGTH = size;
   xd_ptr->WSOFAR = 0;
   xd_ptr->WSTARTADDRESS = (ci2TxBuffer+ci2TxIndex);
   xd_ptr->BSTATUS = MV_USB_STATUS_TRANSFER_ACCEPTED;
   

   error = _usb_dci_vusb20_add_dTD(handle, xd_ptr);
   
   // New location of index for next Tx
   //----------------------------------
   ci2TxIndex += size;

   USB_unlock(lockKey);

   if (error)
   {
      MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"_usb_device_send_data, transfer failed\n");
      return USBERR_TX_FAILED;
   } /* Endif */
   return error;

} /* EndBody */

