/*******************************************************************************
** File          : $HeadURL$
** Author        : $Author$
** Project       : HSCTRL
** Instances     :
** Creation date :
********************************************************************************
********************************************************************************
** ChipIdea Microelectronica - IPCS
** TECMAIA, Rua Eng. Frederico Ulrich, n 2650
** 4470-920 MOREIRA MAIA
** Portugal
** Tel: +351 229471010
** Fax: +351 229471011
** e_mail: chipidea.com
********************************************************************************
** ISO 9001:2000 - Certified Company
** (C) 2005 Copyright Chipidea(R)
** Chipidea(R) - Microelectronica, S.A. reserves the right to make changes to
** the information contained herein without notice. No liability shall be
** incurred as a result of its use or application.
********************************************************************************
** Modification history:
** $Date$
** $Revision$
*******************************************************************************
*** Description:
***  This file contains the main VUSB_HS Device Controller interface
***  functions.
***
**************************************************************************
**END*********************************************************/

#include "mvUsbDevApi.h"
#include "mvUsbDevPrv.h"
#include "xllp_ciu2d.h"

uint_8_ptr   usb_gadget_virt_base = 0;
uint_8_ptr   usb_gadget_phys_base = 0;

// endpoint count problematic for static delcration
// I am going by the MMP IAS chapter register descriptions
// in that the number of endpoints is always 16
// (MMP IAS USB2.0 OTG/HOST controller prelim doc)
__attribute__((aligned(2048)))uint_8 ciQueueHeadList[2 * 16 * sizeof(VUSB20_EP_QUEUE_HEAD_STRUCT)];

extern void (*mv_pCpuCI2InterruptEnable)(void);
extern void (*mv_pCI2PlatformReady)(void);
extern uint_32 mv_options;


/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_init
*  Returned Value : USB_OK or error code
*  Comments       :
*        Initializes the USB device controller.
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_vusb20_init
(
      /* [IN] the USB device controller to initialize */
      uint_8                     devnum,

      /* [OUT] the USB_dev_initialize state structure */
      _usb_device_handle         handle
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   uint_32                                      temp;
   uint_32                                      phyAddr, total_memory=0;
   uint_8_ptr                                   driver_memory;

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;

   usb_dev_ptr->CAP_REGS_PTR =
       (VUSB20_REG_STRUCT_PTR)USB_get_cap_reg_addr(devnum);

   // We're about to start writing/reading from OTG registers
   // so make sure the clock PLLS are stable before proceeding
   //---------------------------------------------------------
   if( mv_pCI2PlatformReady != NULL )
      mv_pCI2PlatformReady();

   /* Get the base address of the VUSB_HS registers */
   usb_dev_ptr->DEV_PTR =
      (VUSB20_REG_STRUCT_PTR)(((uint_32)usb_dev_ptr->CAP_REGS_PTR) +
       (USB_32BIT_LE(usb_dev_ptr->CAP_REGS_PTR->REGISTERS.CAPABILITY_REGISTERS.CAPLENGTH_HCIVER) &
                                                                EHCI_CAP_LEN_MASK));

   /* Get the maximum number of endpoints supported by this USB controller */
   usb_dev_ptr->MAX_ENDPOINTS =
      (USB_32BIT_LE(usb_dev_ptr->CAP_REGS_PTR->REGISTERS.CAPABILITY_REGISTERS.DCC_PARAMS) &
                                                VUSB20_DCC_MAX_ENDPTS_SUPPORTED);

   temp = (usb_dev_ptr->MAX_ENDPOINTS * 2);

    /****************************************************************
      Consolidated memory allocation
      VUSB20_EP_QUEUE_HEAD_STRUCT = 64 bytes (64*32Ep's = 2048) (now static declaration)

      VUSB20_EP_TR_STRUCT         = 32 bytes (32*32 = 256)
      SCRATCH_STRUCT              = 12 bytes (12*32 = 384)
      ALIGNMENT EXTRA             = 64 bytes

      total should = 256 + 384 + 64 = 704 bytes
    ****************************************************************/
    total_memory = /* this portion statically declared & aligned ((temp * sizeof(VUSB20_EP_QUEUE_HEAD_STRUCT)) + 2048) + */
                  ((MAX_EP_TR_DESCRS * sizeof(VUSB20_EP_TR_STRUCT)) + 64) +
                  (sizeof(SCRATCH_STRUCT)*MAX_EP_TR_DESCRS);

    /* allocate new space from SRAM only if not allocated in previous "cycle" - for OTG mode */
    // if(usb_gadget_virt_base == NULL)    //  for bootrom, always realloc
    {
      usb_gadget_virt_base = (uint_8_ptr) USB_uncached_memalloc( total_memory, &phyAddr);
      if (usb_gadget_virt_base == NULL)
      {
        USB_printf("_usb_dci_vusb20_init, malloc failed\n");
        return USBERR_ALLOC;
      }
	  usb_gadget_phys_base = usb_gadget_virt_base; //phyAddr;
	}

    driver_memory = usb_gadget_virt_base;
	phyAddr = (uint_32)usb_gadget_phys_base;

 /*   USB_printf("USB init: CAP_REGS=0x%x, DEV_REGS=0x%x, MAX_EP=%d, virtAddr=%p, phyAddr=0x%x\n",
                (unsigned)usb_dev_ptr->CAP_REGS_PTR, (unsigned)usb_dev_ptr->DEV_PTR,
                usb_dev_ptr->MAX_ENDPOINTS, driver_memory, phyAddr);              */

    /****************************************************************
      Zero out the memory allocated, and the EP Queue list
    ****************************************************************/
    USB_memzero( (void*)driver_memory, total_memory);
    USB_memzero(ciQueueHeadList, sizeof(ciQueueHeadList));

    /****************************************************************
      Flush the zeroed memory if cache is enabled
    ****************************************************************/
    USB_dcache_flush((pointer)driver_memory, total_memory);

    /****************************************************************
     Keep a pointer to driver memory alloctaion
    ****************************************************************/
    usb_dev_ptr->DRIVER_MEMORY_SIZE = total_memory;
    usb_dev_ptr->DRIVER_MEMORY_VIRT = driver_memory;
    usb_dev_ptr->DRIVER_MEMORY_PHYS = phyAddr;

    /****************************************************************
      Assign QH base
    ****************************************************************/
    usb_dev_ptr->EP_QUEUE_HEAD_BASE = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)ciQueueHeadList;

    //driver_memory += ((temp * sizeof(VUSB20_EP_QUEUE_HEAD_STRUCT)) + 2048);



#if 1
    /* Align the endpoint queue head to 2K boundary */
    // This should pass the pointer unchanged since it's already aligned
    // statically
    usb_dev_ptr->EP_QUEUE_HEAD_PTR =
            (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)usb_dev_ptr->EP_QUEUE_HEAD_BASE;
#else
    //usb_dev_ptr->EP_QUEUE_HEAD_PTR = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
    //            USB_MEM2048_ALIGN((uint_32)usb_dev_ptr->EP_QUEUE_HEAD_BASE);
#endif
    /****************************************************************
      Assign DTD base, add 1 extra struct size to account for
      alignment adjustment
    ****************************************************************/
    usb_dev_ptr->DTD_BASE_PTR = (VUSB20_EP_TR_STRUCT_PTR)driver_memory;
    driver_memory += ((MAX_EP_TR_DESCRS+1) * sizeof(VUSB20_EP_TR_STRUCT));

    /* Align the dTD base to 32 byte boundary */
    usb_dev_ptr->DTD_ALIGNED_BASE_PTR = (VUSB20_EP_TR_STRUCT_PTR)
                        USB_MEM32_ALIGN((uint_32)usb_dev_ptr->DTD_BASE_PTR);

    /****************************************************************
      Assign SCRATCH Structure base
    ****************************************************************/
    /* Allocate memory for internal scratch structure */
    usb_dev_ptr->SCRATCH_STRUCT_BASE = (SCRATCH_STRUCT_PTR)driver_memory;

    usb_dev_ptr->USB_STATE = MV_USB_STATE_UNKNOWN;
    /* Initialize the VUSB_HS controller */
    _usb_dci_vusb20_chip_initialize((pointer)usb_dev_ptr);

    return USB_OK;
} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_chip_initialize
*  Returned Value : USB_OK or error code
*  Comments       :
*        Initializes the USB device controller.
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_chip_initialize
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle
)
{ /* Body */
    USB_DEV_STATE_STRUCT_PTR         usb_dev_ptr;
    VUSB20_REG_STRUCT_PTR            dev_ptr;
    VUSB20_EP_QUEUE_HEAD_STRUCT_PTR  ep_queue_head_ptr;
    VUSB20_EP_TR_STRUCT_PTR          dTD_ptr;
    uint_32                          i, port_control;
    SCRATCH_STRUCT_PTR               temp_scratch_ptr;
    volatile unsigned long           delay;

    MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_INIT, "chip_initialize\n");

    usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
    dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

    /* Stop the controller */
    dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD &= ~(USB_32BIT_LE(EHCI_CMD_RUN_STOP));

    //Ensure in-order read/write
    //--------------------------
    port_control = dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD;

    /* Reset the controller to get default values */
    dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD = USB_32BIT_LE(EHCI_CMD_CTRL_RESET);

    // RETURN HERE: get real-time delay from device data sheet
    //-------------------------------------------------------------
    delay = 0x100000;
    while (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD &
                                    (USB_32BIT_LE(EHCI_CMD_CTRL_RESET)))
    {
        /* Wait for the controller reset to complete */
        delay--;
        if(delay == 0)
            break;
    } /* EndWhile */

    if(delay == 0)
    {
       if( mv_pCI2PlatformReady != NULL )
         mv_pCI2PlatformReady();  // sanity check to trap RESET bit not ready
       USB_printf("USB Init: Wait for RESET completed TIMEOUT\n");
    }

   /* Program the controller to be the USB device controller */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_MODE =
      (USB_32BIT_LE(VUSBHS_MODE_CTRL_MODE_DEV | VUSBHS_MODE_SETUP_LOCK_DISABLE));

   /* Initialize the internal dTD head and tail to NULL */
   usb_dev_ptr->DTD_HEAD = NULL;
   usb_dev_ptr->DTD_TAIL = NULL;

   /* Make sure the 16 MSBs of this register are 0s */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPT_SETUP_STAT = USB_32BIT_LE(0);

   ep_queue_head_ptr = usb_dev_ptr->EP_QUEUE_HEAD_PTR;

   /* Initialize all device queue heads */
   for (i=0; i<(usb_dev_ptr->MAX_ENDPOINTS*2); i++)
   {
      /* Interrupt on Setup packet */
      (ep_queue_head_ptr + i)->MAX_PKT_LENGTH = (USB_32BIT_LE(
          ((uint_32)USB_MAX_CTRL_PAYLOAD << VUSB_EP_QUEUE_HEAD_MAX_PKT_LEN_POS) |
            VUSB_EP_QUEUE_HEAD_IOS));

      (ep_queue_head_ptr + i)->NEXT_DTD_PTR = (USB_32BIT_LE(VUSB_EP_QUEUE_HEAD_NEXT_TERMINATE));
   } /* Endfor */

   /* Configure the Endpoint List Address */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR =
                            (uint_32)USB_32BIT_LE(USB_VIRT_TO_PHYS(usb_dev_ptr, ep_queue_head_ptr));

   if (usb_dev_ptr->CAP_REGS_PTR->REGISTERS.CAPABILITY_REGISTERS.HCS_PARAMS &
                                        USB_32BIT_LE(VUSB20_HCS_PARAMS_PORT_POWER_CONTROL_FLAG))
   {
      port_control = USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX/*[0]*/);
      port_control &= (~EHCI_PORTSCX_W1C_BITS | ~EHCI_PORTSCX_PORT_POWER);
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX/*[0]*/ = USB_32BIT_LE(port_control);
   } /* Endif */

   dTD_ptr = usb_dev_ptr->DTD_ALIGNED_BASE_PTR;

   temp_scratch_ptr = usb_dev_ptr->SCRATCH_STRUCT_BASE;

   /* Enqueue all the dTDs */
   for (i=0; i<MAX_EP_TR_DESCRS; i++)
   {
      dTD_ptr->SCRATCH_PTR = temp_scratch_ptr;
      dTD_ptr->SCRATCH_PTR->FREE = _usb_dci_vusb20_free_dTD;
      /* Set the dTD to be invalid */
      dTD_ptr->NEXT_TR_ELEM_PTR = USB_32BIT_LE(VUSBHS_TD_NEXT_TERMINATE);
      /* Set the Reserved fields to 0 */
      dTD_ptr->SIZE_IOC_STS &= ~(USB_32BIT_LE(VUSBHS_TD_RESERVED_FIELDS));
      dTD_ptr->SCRATCH_PTR->PRIVATE = (pointer)usb_dev_ptr;
      _usb_dci_vusb20_free_dTD((pointer)dTD_ptr);
      dTD_ptr++;
      temp_scratch_ptr++;
   } /* Endfor */

   /* Initialize the endpoint 0 properties */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[0] = USB_32BIT_LE(
      (EHCI_EPCTRL_TX_DATA_TOGGLE_RST | EHCI_EPCTRL_RX_DATA_TOGGLE_RST));
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[0] &= ~(USB_32BIT_LE
      (EHCI_EPCTRL_TX_EP_STALL | EHCI_EPCTRL_RX_EP_STALL));

   /* Enable interrupts */
   i = EHCI_INTR_INT_EN |
       EHCI_INTR_ERR_INT_EN |
       EHCI_INTR_PORT_CHANGE_DETECT_EN |
       EHCI_INTR_RESET_EN |
       EHCI_INTR_DEVICE_SUSPEND;

   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_INTR = USB_32BIT_LE(i);

   usb_dev_ptr->USB_STATE = MV_USB_STATE_UNKNOWN;

   //-------------------------------------------
   // START platform specific options
   // application / platform specific setup
   // currently XCVR type and width select
   //-------------------------------------------
   switch( mv_options & CI_OTG_OPTIONS_XCR_MASK )
   {
     case CI_OTG_OPTIONS_UPLI_XCVR_SELECT:
          _usb_dci_vusb20_set_xcvr_interface( handle, EHCI_PORTSCX_ULPI_XCVR_SELECT, 0 );
          break;

     case CI_OTG_OPTIONS_SERIAL_XCVR_SELECT:
          _usb_dci_vusb20_set_xcvr_interface( handle, EHCI_PORTSCX_SERIAL_XCVR_SELECT, 0 );
          break;

     case CI_OTG_OPTIONS_UTMI_XCVR_SELECT:
     default:

          _usb_dci_vusb20_set_xcvr_interface( handle, EHCI_PORTSCX_UTMI_XCVR_SELECT, 0 );
          if( mv_options & CI_OTG_OPTIONS_8BIT_XCVR_WIDTH )
             _usb_dci_vusb20_set_xcvr_width( handle, EHCI_PORTSCX_PTW_8BIT_SELECT, 0 );
          else
             _usb_dci_vusb20_set_xcvr_width( handle, EHCI_PORTSCX_PTW_16BIT_SELECT, 0 );
          break;
   }

   // Force full speed connect/handshake if indicated
   // "USB DEVICE" we only configure port 0
   //----------------------------------------
   if( mv_options & CI_OTG_OPTIONS_FULL_SPEED )
     _usb_dci_vusb20_set_speed_full(handle, 0); //  RE-ENABLE FS FORCE BEFORE TAPE-OUT, FOR SLE ONLY
   else
     _usb_dci_vusb20_set_speed_high(handle, 0);

   //------------------------------
   // END platform specific options
   //------------------------------

   // Clear any spurious interrupt bits on the
   // device for sanity before proceeding clear
   // by writing one's to appropriate bits
   //------------------------------------------
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_STS &= USB_32BIT_LE( 0xFFFFFFFF );
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.OTGSC |=  USB_32BIT_LE(VUSBHS_OTGSC_INTERRUPT_STATUS_BITS_MASK);

   // enable the b-session valid interrupt keep and other values already set such as IDPU
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.OTGSC |=  USB_32BIT_LE(VUSBHS_OTGSC_BSVIE);
   //Now enable the interrupt line from the device
   //to the CPU core
   //---------------------------------------------
   mv_pCpuCI2InterruptEnable();

   // Start-up Ep0
   //-----------------------------------
   mv_usb_start_ep0 (handle);

} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_free_dTD
*  Returned Value : void
*  Comments       :
*        Enqueues a dTD onto the free DTD ring.
*
*END*-----------------------------------------------------------------*/

void _usb_dci_vusb20_free_dTD
(
      /* [IN] the dTD to enqueue */
      pointer  dTD_ptr
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR     usb_dev_ptr;
   int                          lockKey;

   MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_TRACE, "free_dTD: dTD_ptr=0x%x\n", (unsigned)dTD_ptr);

   MV_USB_DEBUG_CODE(MV_USB_DEBUG_FLAG_STATS, (mv_usb_free_dTD_count++));

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)(((VUSB20_EP_TR_STRUCT_PTR)dTD_ptr)->SCRATCH_PTR->PRIVATE);

   /*
   ** This function can be called from any context, and it needs mutual
   ** exclusion with itself.
   */
   lockKey = USB_lock();

   /*
   ** Add the dTD to the free dTD queue (linked via PRIVATE) and
   ** increment the tail to the next descriptor
   */
   EHCI_DTD_QADD(usb_dev_ptr->DTD_HEAD, usb_dev_ptr->DTD_TAIL, (VUSB20_EP_TR_STRUCT_PTR)dTD_ptr);
   usb_dev_ptr->DTD_ENTRIES++;

   USB_unlock(lockKey);

} /* Endbody */


/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_add_dTD
*  Returned Value : USB_OK or error code
*  Comments       :
*        Adds a device transfer desriptor(s) to the queue.
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_vusb20_add_dTD
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,

      /* [IN] The transfer descriptor address */
      XD_STRUCT_PTR              xd_ptr
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR         usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR            dev_ptr;
   VUSB20_EP_TR_STRUCT_PTR          dTD_ptr, temp_dTD_ptr, first_dTD_ptr = NULL;
   VUSB20_EP_QUEUE_HEAD_STRUCT_PTR  ep_queue_head_ptr;
   uint_32                          curr_pkt_len, remaining_len;
   uint_32                          curr_offset, temp, bit_pos, temp_ep_stat;
   volatile unsigned long           timeout;

   /*********************************************************************
   For a optimal implementation, we need to detect the fact that
   we are adding DTD to an empty list. If list is empty, we can
   actually skip several programming steps esp. those for ensuring
   that there is no race condition.The following boolean will be useful
   in skipping some code here.
   *********************************************************************/
   boolean           list_empty = FALSE;
   boolean           read_safe = FALSE;

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   remaining_len = xd_ptr->WTOTALLENGTH;

   curr_offset = 0;
   temp = (2*xd_ptr->EP_NUM + xd_ptr->BDIRECTION);
   bit_pos = (1 << (16 * xd_ptr->BDIRECTION + xd_ptr->EP_NUM));

   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)usb_dev_ptr->EP_QUEUE_HEAD_PTR + temp;

#if 1
   // mdb debug - just read the reg once and forget about it
   timeout = USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPT_SETUP_STAT);
#else
   //timeout = 0x100;
   while (USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPT_SETUP_STAT))
   {
   /* Wait until all ENDPT_SETUP_STAT bits cleared */
    timeout--;
    if(timeout <= 0)
    {
        MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"%s: Timeout for ENDPT_SETUP_STAT reg = 0x%x\n", __FUNCTION__,
             (unsigned)USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPT_SETUP_STAT));
        break;
    }
   } /* Endif */
#endif

   /*********************************************************************
   This loops iterates through the length of the transfer and divides
   the data in to DTDs each handling the a max of 0x4000 bytes of data.
   The first DTD in the list is stored in a pointer called first_dTD_ptr.
   This pointer is later linked in to QH for processing by the hardware.
   *********************************************************************/

    do
    {
        /* Check if we need to split the transfer into multiple dTDs */
        if (remaining_len > VUSB_EP_MAX_LENGTH_TRANSFER)
        {
            curr_pkt_len = VUSB_EP_MAX_LENGTH_TRANSFER;
        }
        else
        {
            curr_pkt_len = remaining_len;
        } /* Endif */

        /* Get a dTD from the queue */
        EHCI_DTD_QGET(usb_dev_ptr->DTD_HEAD, usb_dev_ptr->DTD_TAIL, dTD_ptr);

        if (!dTD_ptr)
        {
            MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"Error: Can't get dTD\n");
            return USBERR_TR_FAILED;
        } /* Endif */

        MV_USB_DEBUG_CODE(MV_USB_DEBUG_FLAG_STATS, (mv_usb_add_count++));

        MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_TRANSFER,
                        "add_dTD_%d: ep=%d %s, size=%d, CURR_dTD=0x%x, xd_ptr=0x%x, dTD_ptr=0x%x\n",
                        mv_usb_add_count, xd_ptr->EP_NUM, xd_ptr->BDIRECTION ? "SEND" : "RECV",
                        (int)remaining_len, (unsigned)USB_32BIT_LE(ep_queue_head_ptr->CURR_DTD_PTR),
                        (unsigned)xd_ptr, (unsigned)dTD_ptr);

        remaining_len -= curr_pkt_len;

        usb_dev_ptr->DTD_ENTRIES--;

        if (curr_offset == 0)
        {
            first_dTD_ptr = dTD_ptr;
        } /* Endif */

        /* Zero the dTD. Leave the last 4 bytes as that is the scratch pointer */
        USB_memzero((void *) dTD_ptr,(sizeof(VUSB20_EP_TR_STRUCT) - 4));

        /* Initialize the dTD */
        dTD_ptr->SCRATCH_PTR->PRIVATE = handle;

        /* Set the Terminate bit */
        dTD_ptr->NEXT_TR_ELEM_PTR = USB_32BIT_LE(VUSB_EP_QUEUE_HEAD_NEXT_TERMINATE);

        /*************************************************************
        FIX ME: For hig-speed and high-bandwidth ISO IN endpoints,
        we must initialize the multiplied field so that Host can issues
        multiple IN transactions on the endpoint. See the DTD data
        structure for MultiIO field.

        S Garg 11/06/2003
        *************************************************************/

        /* Fill in the transfer size */
        if (!remaining_len)
        {
            dTD_ptr->SIZE_IOC_STS = USB_32BIT_LE((curr_pkt_len <<
                    VUSBHS_TD_LENGTH_BIT_POS) | (VUSBHS_TD_IOC) | (VUSBHS_TD_STATUS_ACTIVE));
        }
        else
        {
            dTD_ptr->SIZE_IOC_STS = USB_32BIT_LE((curr_pkt_len << VUSBHS_TD_LENGTH_BIT_POS)
                                                   | VUSBHS_TD_STATUS_ACTIVE);
        } /* Endif */

        /* Set the reserved field to 0 */
        dTD_ptr->SIZE_IOC_STS &= ~USB_32BIT_LE(VUSBHS_TD_RESERVED_FIELDS);

        /* 4K apart buffer page pointers */
        if(xd_ptr->WSTARTADDRESS != NULL)
        {
            uint_32 physAddr = USB_virt_to_phys((uint_8*)xd_ptr->WSTARTADDRESS + curr_offset);

            dTD_ptr->BUFF_PTR0 = USB_32BIT_LE(physAddr);

            physAddr += 4096;
            dTD_ptr->BUFF_PTR1 = USB_32BIT_LE(physAddr);

            physAddr += 4096;
            dTD_ptr->BUFF_PTR2 = USB_32BIT_LE(physAddr);

            physAddr += 4096;
            dTD_ptr->BUFF_PTR3 = USB_32BIT_LE(physAddr);

            physAddr += 4096;
            dTD_ptr->BUFF_PTR4 = USB_32BIT_LE(physAddr);
        }
        else
        {
            dTD_ptr->BUFF_PTR0 = dTD_ptr->BUFF_PTR1 = dTD_ptr->BUFF_PTR2 = 0;
            dTD_ptr->BUFF_PTR3 = dTD_ptr->BUFF_PTR4 = 0;
        }
        curr_offset += curr_pkt_len;

      /* Maintain the first and last device transfer descriptor per
      ** endpoint and direction
      */
      if (!usb_dev_ptr->EP_DTD_HEADS[temp])
      {
         usb_dev_ptr->EP_DTD_HEADS[temp] = dTD_ptr;
         /***********************************************
         If list does not have a head, it means that list
         is empty. An empty condition is detected.
         ***********************************************/
         list_empty = TRUE;
      } /* Endif */

      /* Check if the transfer is to be queued at the end or beginning */
      temp_dTD_ptr = usb_dev_ptr->EP_DTD_TAILS[temp];

      /* Remember which XD to use for this dTD */
      dTD_ptr->SCRATCH_PTR->XD_FOR_THIS_DTD = (pointer)xd_ptr;

      /* New tail */
      usb_dev_ptr->EP_DTD_TAILS[temp] = dTD_ptr;
      if (temp_dTD_ptr)
      {
         /* Should not do |=. The Terminate bit should be zero */
         temp_dTD_ptr->NEXT_TR_ELEM_PTR = (uint_32)USB_32BIT_LE(USB_VIRT_TO_PHYS(usb_dev_ptr, dTD_ptr));
      } /* Endif */
   } while (remaining_len); /* EndWhile */


   /**************************************************************
   In the loop above DTD has already been added to the list
   However endpoint has not been primed yet. If list is not empty
   we need safter ways to add DTD to the existing list.
   Else we just skip to adding DTD to QH safely.
   **************************************************************/

   if(list_empty == FALSE)
   {
        /*********************************************************
        Hardware v3.2+ require the use of semaphore to ensure that
        QH is safely updated.
        *********************************************************/
        MV_USB_DEBUG_CODE(MV_USB_DEBUG_FLAG_STATS, (mv_usb_add_not_empty_count++));


        /*********************************************************
        Check the prime bit. If set goto done
        *********************************************************/
        if( USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME) & bit_pos)
        {
           goto done;
        }
#ifdef TRIP_WIRE
        read_safe = FALSE;
        timeout = 10000;
        while(read_safe == FALSE)
        {
            timeout--;
            if(timeout <= 0)
            {
                MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"%s: Timeout for ATDTW_TRIPWIRE reg = 0x%x\n", __FUNCTION__,
                    (unsigned)USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD));
                return USBERR_TR_FAILED;
            }

           /*********************************************************
           start with setting the semaphores
           *********************************************************/
           dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD |=
                                          USB_32BIT_LE(EHCI_CMD_ATDTW_TRIPWIRE_SET);

           /*********************************************************
           Read the endpoint status
           *********************************************************/
           temp_ep_stat = USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS) & bit_pos;

           /*********************************************************
           Reread the ATDTW semaphore bit to check if it is cleared.
           When hardware see a hazard, it will clear the bit or
           else we remain set to 1 and we can proceed with priming
           of endpoint if not already primed.
           *********************************************************/
           if( USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD) &
                                          EHCI_CMD_ATDTW_TRIPWIRE_SET)
           {
               read_safe = TRUE;
           }

        }/*end while loop */

        /*********************************************************
        Clear the semaphore
        *********************************************************/
        dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD &=
                                       USB_32BIT_LE(EHCI_CMD_ATDTW_TRIPWIRE_CLEAR);
#else
	/*********************************************************
      Read the endpoint status
    *********************************************************/
    temp_ep_stat = USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS) & bit_pos;

#endif

        /*********************************************************
         * If endpoint is not active, we activate it now.
         *********************************************************/
         if(!temp_ep_stat)
         {
            /* No other transfers on the queue */
            ep_queue_head_ptr->NEXT_DTD_PTR = (uint_32)USB_32BIT_LE(
                        USB_VIRT_TO_PHYS(usb_dev_ptr, first_dTD_ptr));
            ep_queue_head_ptr->SIZE_IOC_INT_STS = 0;

            /* Prime the Endpoint */
            dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME = USB_32BIT_LE(bit_pos);
         }
   }
   else
   {
         /* No other transfers on the queue */
         ep_queue_head_ptr->NEXT_DTD_PTR = (uint_32)USB_32BIT_LE(
                            USB_VIRT_TO_PHYS(usb_dev_ptr, first_dTD_ptr));
         ep_queue_head_ptr->SIZE_IOC_INT_STS = 0;

         /* Prime the Endpoint */
         dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME = USB_32BIT_LE(bit_pos);
   }

done:
	#if 0
    //mdb debug - no more waiting !!! too slow as it is
    /* wait a little */
	{
		 int i;
		 for (i=0; i < 1000; i++);
	}
	#endif

   return USB_OK;
   /* End CR 1015 */
} /* EndBody */


/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_process_tr_complete
*  Returned Value : None
*  Comments       :
*        Services transaction complete interrupt
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_process_tr_complete
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   volatile VUSB20_REG_STRUCT_PTR               dev_ptr;
   volatile VUSB20_EP_TR_STRUCT_PTR             dTD_ptr;
   VUSB20_EP_TR_STRUCT_PTR                      temp_dTD_ptr;
   VUSB20_EP_QUEUE_HEAD_STRUCT_PTR              ep_queue_head_ptr;
   uint_32                                      temp, i, ep_num = 0, direction = 0, bit_pos;
   uint_32                                      remaining_length = 0;
   uint_32                                      actual_transfer_length = 0;
   uint_32                                      counter, errors = 0;
   XD_STRUCT_PTR                                xd_ptr;
   XD_STRUCT_PTR                                temp_xd_ptr = NULL;
   uint_8_ptr                                    buff_start_address = NULL;
   boolean                                      endpoint_detected = FALSE;

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ISR, "process_tr_complete_isr\n");
   MV_USB_DEBUG_CODE(MV_USB_DEBUG_FLAG_STATS, (mv_usb_complete_isr_count++));

   /* We use separate loops for ENDPTSETUPSTAT and ENDPTCOMPLETE because the
   ** setup packets are to be read ASAP
   */

   /* Process all Setup packet received interrupts */
   bit_pos = USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPT_SETUP_STAT);

   if (bit_pos)
   {
      MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ISR, "setup_isr: bit_pos=0x%x\n", (unsigned)bit_pos);
      for(i=0; i<USB_MAX_CONTROL_ENDPOINTS; i++)
      {
         if (bit_pos & (1 << i))
         {
            MV_USB_DEBUG_CODE(MV_USB_DEBUG_FLAG_STATS, (mv_usb_setup_count++));
            _usb_device_call_service(handle, i, TRUE, 0, 0, 8, 0);
         } /* Endif */
      } /* Endfor */
   } /* Endif */

   /* Don't clear the endpoint setup status register here. It is cleared as a
   ** setup packet is read out of the buffer
   */

   /* Process non-setup transaction complete interrupts */
   bit_pos = USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCOMPLETE);

   /* Clear the bits in the register */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCOMPLETE = USB_32BIT_LE(bit_pos);

   MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ISR, "tr_complete: bit_pos = 0x%x\n", (unsigned)bit_pos);
   if (bit_pos)
   {
        MV_USB_DEBUG_CODE(MV_USB_DEBUG_FLAG_STATS, (mv_usb_complete_count++));

        /* Get the endpoint number and the direction of transfer */
        counter = 0;
        for (i=0; i<(MV_USB_MAX_ENDPOINTS*2); i++)
        {
            endpoint_detected = FALSE;
            if ((i < MV_USB_MAX_ENDPOINTS) && (bit_pos & (1 << i)))
            {
                ep_num = i;
                direction = 0;
                endpoint_detected = TRUE;
            }
            else
            {
                if( (i >= MV_USB_MAX_ENDPOINTS) &&
                    (bit_pos & (1 << (i+16-MV_USB_MAX_ENDPOINTS))))
                {
                    ep_num = (i - MV_USB_MAX_ENDPOINTS);
                    direction = 1;
                    endpoint_detected = TRUE;
                }
            }

            if(endpoint_detected)
            {
                temp = (2*ep_num + direction);

                /* Get the first dTD */
                dTD_ptr = usb_dev_ptr->EP_DTD_HEADS[temp];

                ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)usb_dev_ptr->EP_QUEUE_HEAD_PTR + temp;

                /* Process all the dTDs for respective transfers */
                while (dTD_ptr)
                {
                    if (USB_32BIT_LE(dTD_ptr->SIZE_IOC_STS) & VUSBHS_TD_STATUS_ACTIVE)
                    {
                        /* No more dTDs to process. Next one is owned by VUSB */
                        if(counter == 0)
                        {
                            MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ISR, "tr_complete - break: ep=%d %s, bit_pos=0x%x\n",
                                    (unsigned)ep_num, direction ? "SEND" : "RECV", (unsigned)bit_pos);

                            MV_USB_DEBUG_CODE(MV_USB_DEBUG_FLAG_STATS, (mv_usb_empty_complete_count++));
                        }
                        break;
                    } /* Endif */

                    /* Get the correct internal transfer descriptor */
                    xd_ptr = (XD_STRUCT_PTR)dTD_ptr->SCRATCH_PTR->XD_FOR_THIS_DTD;
                    if (xd_ptr)
                    {
                        buff_start_address = xd_ptr->WSTARTADDRESS;
                        actual_transfer_length = xd_ptr->WTOTALLENGTH;
                        temp_xd_ptr = xd_ptr;
                    } /* Endif */

                    /* Get the address of the next dTD */
                    temp_dTD_ptr = (VUSB20_EP_TR_STRUCT_PTR)USB_PHYS_TO_VIRT(usb_dev_ptr,
                                (uint_32)(USB_32BIT_LE(dTD_ptr->NEXT_TR_ELEM_PTR) & VUSBHS_TD_ADDR_MASK) );

                    /* Read the errors */
                    errors = (USB_32BIT_LE(dTD_ptr->SIZE_IOC_STS) & VUSBHS_TD_ERROR_MASK);
                    if (!errors)
                    {
                        /* No errors */
                        /* Get the length of transfer from the current dTD */
                        remaining_length += ((USB_32BIT_LE(dTD_ptr->SIZE_IOC_STS) & VUSB_EP_TR_PACKET_SIZE) >> 16);
                        actual_transfer_length -= remaining_length;
                    }
                    else
                    {
                        MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"complete_tr error: ep=%d %s: error = 0x%x\n",
                                    (unsigned)ep_num, direction ? "SEND" : "RECV", (unsigned)errors);
                        if (errors & VUSBHS_TD_STATUS_HALTED)
                        {
                            /* Clear the errors and Halt condition */
                            ep_queue_head_ptr->SIZE_IOC_INT_STS &= ~errors;
                        } /* Endif */
                    } /* Endif */

                    /* Retire the processed dTD */
                    MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ISR, "tr_complete - cancel: ep=%d %s, bit_pos = 0x%x\n",
                                    (unsigned)ep_num, direction ? "SEND" : "RECV", (unsigned)bit_pos);

                    counter++;
                    _usb_dci_vusb20_cancel_transfer(handle, ep_num, direction);
                    if( (temp_dTD_ptr == NULL) ||
                        (temp_dTD_ptr->SCRATCH_PTR->XD_FOR_THIS_DTD != temp_xd_ptr) )
                    {
                        /* Transfer complete. Call the register service function for the endpoint */
                        MV_USB_DEBUG_CODE(MV_USB_DEBUG_FLAG_STATS, (mv_usb_complete_ep_count[temp]++));
                        MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_TRANSFER,
                            "tr_complete completed: ep=%d %s, pBuf=%p, size=%d, errors=0x%x\n",
                            (unsigned)ep_num, direction ? "SEND" : "RECV", buff_start_address,
                            actual_transfer_length, errors);

                        _usb_device_call_service(handle, ep_num, FALSE, direction,
                                   buff_start_address, actual_transfer_length, errors);
                        remaining_length = 0;
                    } /* Endif */
                    else
                    {
                        MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_TRANSFER, "tr_complete not completed: ep=%d %s\n",
                                (unsigned)ep_num, direction ? "SEND" : "RECV");
                    }
                    dTD_ptr = temp_dTD_ptr;
                    errors = 0;
                } /* End while dTD_ptr*/
            } /* End if endpoint_detected*/
        } /* End for */
        MV_USB_DEBUG_CODE(MV_USB_DEBUG_FLAG_STATS,
               ( {if(mv_usb_complete_max_count < counter) mv_usb_complete_max_count = counter;}));
   } /* Endif */
} /* EndBody */



void _usb_dci_vusb20_enable (_usb_device_handle      handle )
{
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   uint_32                                      vbus_status;


   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   // check the  B-session valid IS and if vbus is good connect
   vbus_status = USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.OTGSC);

//   if( vbus_status & VUSBHS_OTGSC_BSVIS)
   {
	   // Set the Run bit in the command register
	   // This will cause the phy to connect to the USB bus
	   //--------------------------------------------------
	   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD |= USB_32BIT_LE(EHCI_CMD_RUN_STOP);
	   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.OTGSC &= ~USB_32BIT_LE(VUSBHS_OTGSC_INTERRUPT_ENABLE_BITS_MASK);
   }
}

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_isr
*  Returned Value : None
*  Comments       :
*        Services all the VUSB_HS interrupt sources
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_isr ( _usb_device_handle   handle )
{
   /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   uint_32                                      status, non_int_bits, vbus_status;
   uint_8 record_debug  = 0;

   MV_USB_DEBUG_CODE(MV_USB_DEBUG_FLAG_STATS, (mv_usb_isr_count++));

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;

   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

	// check the  B-session valid IS and if vbus is good connect
   vbus_status = USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.OTGSC);

   if( vbus_status & VUSBHS_OTGSC_BSVIS)
   {
	   // Set the Run bit in the command register
	   // This will cause the phy to connect to the USB bus
	   //--------------------------------------------------
	   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD |= USB_32BIT_LE(EHCI_CMD_RUN_STOP);
	   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.OTGSC &= ~USB_32BIT_LE(VUSBHS_OTGSC_INTERRUPT_ENABLE_BITS_MASK);
   }

   // get the interrupt bits that are set
   //------------------------------------
   status = USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_STS);


   // Get the bits that are set but are MASKED
   //------------------------------------------------------------------------------
   non_int_bits = status & ~(USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_INTR));


   if(status == 0)
   {
       MV_USB_DEBUG_CODE(MV_USB_DEBUG_FLAG_STATS, (mv_usb_empty_isr_count++));
       return;
   } /* Endif */


   /* Clear all the interrupts occured */
   // let's reset individual interrupt bits after each is serviced in case
   // state machine inside OTG silicon needs single acknowledge at a time
   // to advance state etc.
   //--------------------------------------------------------------------------
   //dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_STS = USB_32BIT_LE(status);

   if (status & EHCI_STS_INT)
   {
      do
	  {
      	 _usb_dci_vusb20_process_tr_complete((pointer)usb_dev_ptr);
	  } while ( dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCOMPLETE);

      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_STS = USB_32BIT_LE( EHCI_STS_INT );

   } /* Endif */

   if (status & EHCI_STS_RESET)
   {
      _usb_dci_vusb20_process_reset((pointer)usb_dev_ptr);
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_STS = USB_32BIT_LE( EHCI_STS_RESET );
   } /* Endif */

   if (status & EHCI_STS_PORT_CHANGE)
   {
      _usb_dci_vusb20_process_port_change((pointer)usb_dev_ptr);
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_STS = USB_32BIT_LE( EHCI_STS_PORT_CHANGE );
   } /* Endif */

   if (status & EHCI_STS_ERR)
   {
      _usb_dci_vusb20_process_error((pointer)usb_dev_ptr);
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_STS = USB_32BIT_LE( EHCI_STS_ERR );
   } /* Endif */

   if (status & EHCI_STS_SOF)
   {
   //We don't have a process to handle SOF interrupts, so just clear the int source bit
      //_usb_dci_vusb20_process_SOF((pointer)usb_dev_ptr);
         dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_STS = USB_32BIT_LE( EHCI_STS_SOF );
   }

   if (status & EHCI_STS_SUSPEND)
   {
      _usb_dci_vusb20_process_suspend((pointer)usb_dev_ptr);
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_STS = USB_32BIT_LE( EHCI_STS_SUSPEND );
   } /* Endif */

	//Now clear the Masked off interrupt bits
	dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_STS	= non_int_bits;

} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_process_reset
*  Returned Value : None
*  Comments       :
*        Services USB BUS reset interrupt
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_process_reset
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR          usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR             dev_ptr;
   uint_32                           temp;
   uint_8                            i;

   MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ISR, "process_reset\n");
   MV_USB_DEBUG_CODE(MV_USB_DEBUG_FLAG_STATS, (mv_usb_reset_count++));

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   /* Inform the application so that it can cancel all previously queued transfers */
   _usb_device_call_service(usb_dev_ptr, MV_USB_SERVICE_BUS_RESET, 0, 0, 0, 0, 0);

   /* The address bits are past bit 25-31. Set the address */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.DEVICE_ADDR &= ~USB_32BIT_LE(0xFE000000);

   /* Clear all the setup token semaphores */
   temp = USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPT_SETUP_STAT);
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPT_SETUP_STAT = USB_32BIT_LE(temp);

   /* Clear all the endpoint complete status bits */
   temp = USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCOMPLETE);
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCOMPLETE = USB_32BIT_LE(temp);

   // Wait for all primes to flush
   //------------------------------------------
   while (USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME) & 0xFFFFFFFF)
   {
      /* Wait until all ENDPTPRIME bits cleared */
   } /* Endif */

   /* Write 1s to the Flush register */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTFLUSH = USB_32BIT_LE(0xFFFFFFFF);

   /* Unstall all endpoints */
   for (i=0;i<usb_dev_ptr->MAX_ENDPOINTS;i++)
   {
      _usb_dci_vusb20_unstall_endpoint(handle, i, 0);
      _usb_dci_vusb20_unstall_endpoint(handle, i, 1);
   } /* Endfor */

   if (USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX/*[0]*/) &
                                                EHCI_PORTSCX_PORT_RESET)
   {
      usb_dev_ptr->BUS_RESETTING = TRUE;
      usb_dev_ptr->USB_STATE = MV_USB_STATE_POWERED;
   }
#if 0
   else
   {
      // this is a sanity check/ error recovery; we should not be
      // in this routine if this bit is not set in addition to the URI (reset)
      // bit in the STS register
      //----------------------------------------------------------------------
      _usb_dci_vusb20_chip_initialize((pointer)usb_dev_ptr);
      MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_INIT, "process_reset, Chip reinit hw\n");
      return;
   } /* Endif */
#endif
} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_process_suspend
*  Returned Value : None
*  Comments       :
*        Services suspend interrupt
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_process_suspend
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR     usb_dev_ptr;

   MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ISR, "process_suspend\n");
   MV_USB_DEBUG_CODE(MV_USB_DEBUG_FLAG_STATS, (mv_usb_suspend_count++));

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;

   usb_dev_ptr->USB_DEV_STATE_B4_SUSPEND = usb_dev_ptr->USB_STATE;

   usb_dev_ptr->USB_STATE = MV_USB_STATE_SUSPEND;

   /* Inform the upper layers */
   _usb_device_call_service(usb_dev_ptr, MV_USB_SERVICE_SLEEP, 0, 0, 0, 0, 0);

} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_process_SOF
*  Returned Value : None
*  Comments       :
*        Services SOF interrupt
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_process_SOF
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;

   MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ISR, "process_SOF\n");

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   /* Inform the upper layer */
   _usb_device_call_service(usb_dev_ptr, MV_USB_SERVICE_SOF, 0, 0, 0,
      USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_FRINDEX), 0);

} /* EndBody */


/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_process_port_change
*  Returned Value : None
*  Comments       :
*        Services port change detect interrupt
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_process_port_change
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;

   MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ISR, "process_port_change\n");
   MV_USB_DEBUG_CODE(MV_USB_DEBUG_FLAG_STATS, (mv_usb_port_change_count++));

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   if (usb_dev_ptr->BUS_RESETTING) {
      /* Bus reset operation complete */
      usb_dev_ptr->BUS_RESETTING = FALSE;
   } /* Endif */

   if (!(USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX/*[0]*/) &
                                                EHCI_PORTSCX_PORT_RESET))
   {
      /* Get the speed */
      if (USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX/*[0]*/) &
                                            EHCI_PORTSCX_PORT_HIGH_SPEED)
      {
         usb_dev_ptr->SPEED = MV_USB_SPEED_HIGH;
      }
      else
      {
         usb_dev_ptr->SPEED = MV_USB_SPEED_FULL;
      } /* Endif */

      /* Inform the upper layers of the speed of operation */
      _usb_device_call_service(usb_dev_ptr, MV_USB_SERVICE_SPEED_DETECTION, 0, 0,
                                0, usb_dev_ptr->SPEED, 0);

   } /* Endif */

   if (USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX/*[0]*/) &
                                                EHCI_PORTSCX_PORT_SUSPEND)
   {

      usb_dev_ptr->USB_DEV_STATE_B4_SUSPEND = usb_dev_ptr->USB_STATE;
      usb_dev_ptr->USB_STATE = MV_USB_STATE_SUSPEND;

      /* Inform the upper layers */
      MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_INFO,"USB suspend\n");
      _usb_device_call_service(usb_dev_ptr, MV_USB_SERVICE_SUSPEND, 0, 0, 0, 0, 0);
   } /* Endif */

   if (!(USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX/*[0]*/) & EHCI_PORTSCX_PORT_SUSPEND)
                    && (usb_dev_ptr->USB_STATE == MV_USB_STATE_SUSPEND))
   {
      MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_INFO,"USB resume\n");
      usb_dev_ptr->USB_STATE = usb_dev_ptr->USB_DEV_STATE_B4_SUSPEND;
      /* Inform the upper layers */
      _usb_device_call_service(usb_dev_ptr, MV_USB_SERVICE_RESUME, 0, 0, 0, 0, 0);

      MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_SUSPEND, "process_port_change, SUCCESSFUL, resumed\n");
      return;
   } /* Endif */

   usb_dev_ptr->USB_STATE = MV_USB_STATE_DEFAULT;

} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_process_error
*  Returned Value : None
*  Comments       :
*        Services error interrupt
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_process_error
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR          usb_dev_ptr;

   MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ERROR, "process_error\n");

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;

   /* Increment the error count */
   usb_dev_ptr->ERRORS++;

} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_set_speed_full
*  Returned Value : None
*  Comments       :
*        Force the controller port in full speed mode.
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_set_speed_full
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,

      /* The port number on the device */
      uint_8                     port_number
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   //uint_32                                      port_control;

   MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ANY, "FORCE set_speed_full\n");

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   //port_control = USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX/*[port_number]*/);
   //port_control |= EHCI_PORTSCX_FORCE_FULL_SPEED_CONNECT;
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX |= USB_32BIT_LE(EHCI_PORTSCX_FORCE_FULL_SPEED_CONNECT);

} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_suspend_phy
*  Returned Value : None
*  Comments       :
*        Suspends the PHY in low power mode
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_suspend_phy
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,

      /* The port number on the device */
      uint_8                     port_number
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   uint_32                                      port_control;

   MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_SUSPEND, "set_suspend_phy\n");

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   port_control = USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX/*[port_number]*/);
   port_control |= EHCI_PORTSCX_PHY_CLOCK_DISABLE;
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX/*[port_number]*/ = USB_32BIT_LE(port_control);

} /* EndBody */


/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_set_address
*  Returned Value : None
*  Comments       :
*        Sets the newly assigned device address
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_set_address
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,

      /* Address of the device assigned by the host */
      uint_8                     address
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR          usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR             dev_ptr;

   MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ADDR, "set_address: address=%d\n",address);

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

#ifdef SET_ADDRESS_HARDWARE_ASSISTANCE
   /***********************************************************
   Hardware Rev 4.0 onwards have special assistance built in
   for handling the set_address command. As per the USB specs
   a device should be able to receive the response on a new
   address, within 2 msecs after status phase of set_address is
   completed. Since 2 mili second may be a very small time window
   (on high interrupt latency systems) before software could
   come to the code below and write the device register,
   this routine will be called in advance when status phase of
   set_address is still not finished. The following line in the
   code will set the bit 24 to '1' and hardware will take
   the address and queue it in an internal buffer. From which
   it will use it to decode the next USB token. Please look
   at hardware rev details for the implementation of this
   assistance.

   Also note that writing bit 24 to 0x01 will not break
   any old hardware revs because it was an unused bit.
   ***********************************************************/
   /* The address bits are past bit 25-31. Set the address
   also set the bit 24 to 0x01 to start hardware assistance*/
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.DEVICE_ADDR =
      USB_32BIT_LE((uint_32)address << VUSBHS_ADDRESS_BIT_SHIFT) |
      (0x01 << (VUSBHS_ADDRESS_BIT_SHIFT -1));
#else
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.DEVICE_ADDR =
      USB_32BIT_LE((uint_32)address << VUSBHS_ADDRESS_BIT_SHIFT);
#endif /* SET_ADDRESS_HARDWARE_ASSISTANCE */

   usb_dev_ptr->USB_STATE = MV_USB_STATE_ADDRESS;

} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_get_setup_data
*  Returned Value : None
*  Comments       :
*        Reads the Setup data from the 8-byte setup buffer
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_get_setup_data
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,

      /* [IN] the Endpoint number */
      uint_8                     ep_num,

      /* [OUT] address of the buffer to read the setup data into */
      uint_8_ptr                  buffer_ptr
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   volatile VUSB20_REG_STRUCT_PTR               dev_ptr;
   volatile VUSB20_EP_QUEUE_HEAD_STRUCT_PTR     ep_queue_head_ptr;
   volatile boolean                             read_safe = FALSE;
   unsigned int tmp;

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   /* Get the endpoint queue head */
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)usb_dev_ptr->EP_QUEUE_HEAD_PTR +
                                                                2*ep_num + MV_USB_RECV;
#ifdef TRIP_WIRE
   /********************************************************************
   CR 1219. Hardware versions 2.3+ have a implementation of tripwire
   semaphore mechanism that requires that we read the contents of
   QH safely by using the semaphore. Read the USBHS document to under
   stand how the code uses the semaphore mechanism. The following are
   the steps in brief

   1. USBCMD Write 1 to Setup Tripwire in register.
   2. Duplicate contents of dQH.StatusBuffer into local software byte
      array.
   3  Read Setup TripWire in register. (if set - continue; if
      cleared goto 1.)
   4. Write '0' to clear Setup Tripwire in register.
   5. Process setup packet using local software byte array copy and
      execute status/handshake phases.


   ********************************************************************/
  while(!read_safe)
  {
      /*********************************************************
      start with setting the semaphores
      *********************************************************/

      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD |=
                                                USB_32BIT_LE(EHCI_CMD_SETUP_TRIPWIRE_SET);

      /* Copy the setup packet to private buffer */
      USB_memcopy((uint_8_ptr)ep_queue_head_ptr->SETUP_BUFFER, buffer_ptr, 8);

      /*********************************************************
      If setup tripwire semaphore is cleared by hardware it means
      that we have a danger and we need to restart.
      else we can exit out of loop safely.
      *********************************************************/
      if(USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD) &
                                                   EHCI_CMD_SETUP_TRIPWIRE_SET)
      {
          read_safe = TRUE; /* we can proceed exiting out of loop*/
      }
      MV_USB_DEBUG_CODE(MV_USB_DEBUG_FLAG_STATS, (mv_usb_read_setup_count++));
  }

   /*********************************************************
   Clear the semaphore bit now
   *********************************************************/
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD &=
                                    USB_32BIT_LE(EHCI_CMD_SETUP_TRIPWIRE_CLEAR);
#else
   /* Copy the setup packet to private buffer */
      USB_memcopy((uint_8_ptr)ep_queue_head_ptr->SETUP_BUFFER, buffer_ptr, 8);
#endif

   /* Clear the bit in the ENDPTSETUPSTAT */
   //mdb debug - try writting 0, see if it clears it
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPT_SETUP_STAT = USB_32BIT_LE(0);
   //mdb debug - read it, write it, read it back
   tmp = USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPT_SETUP_STAT);
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPT_SETUP_STAT = USB_32BIT_LE(1 << ep_num);
   tmp = USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPT_SETUP_STAT);

} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_init_endpoint
*  Returned Value : None
*  Comments       :
*        Initializes the specified endpoint and the endpoint queue head
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_vusb20_init_endpoint
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,

      /* [IN] the transaction descriptor address */
      XD_STRUCT_PTR              xd_ptr
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   VUSB20_EP_QUEUE_HEAD_STRUCT _PTR_            ep_queue_head_ptr;
   uint_32                                      bit_pos;

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   /* Get the endpoint queue head address */
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)usb_dev_ptr->EP_QUEUE_HEAD_PTR +
                                                    2*xd_ptr->EP_NUM + xd_ptr->BDIRECTION;

   bit_pos = (1 << (16 * xd_ptr->BDIRECTION + xd_ptr->EP_NUM));

   /* Check if the Endpoint is Primed */
   if ((!(USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME) & bit_pos)) &&
       (!(USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS) & bit_pos)))
   {
      /* Set the max packet length, interrupt on Setup and Mult fields */
      if (xd_ptr->EP_TYPE == MV_USB_ISOCHRONOUS_ENDPOINT)
      {
         /* Mult bit should be set for isochronous endpoints */
         ep_queue_head_ptr->MAX_PKT_LENGTH = USB_32BIT_LE((xd_ptr->WMAXPACKETSIZE << 16) |
            ((xd_ptr->MAX_PKTS_PER_UFRAME ?  xd_ptr->MAX_PKTS_PER_UFRAME : 1) <<
            VUSB_EP_QUEUE_HEAD_MULT_POS));
      }
      else
      {
         if (xd_ptr->EP_TYPE != MV_USB_CONTROL_ENDPOINT)
         {
             /* BULK or INTERRUPT */
            ep_queue_head_ptr->MAX_PKT_LENGTH = USB_32BIT_LE((xd_ptr->WMAXPACKETSIZE << 16) |
               (xd_ptr->DONT_ZERO_TERMINATE ? VUSB_EP_QUEUE_HEAD_ZERO_LEN_TER_SEL : 0));
         }
         else
         {
             /* CONTROL */
            ep_queue_head_ptr->MAX_PKT_LENGTH = USB_32BIT_LE((xd_ptr->WMAXPACKETSIZE << 16) |
                                                    VUSB_EP_QUEUE_HEAD_IOS);
         } /* Endif */
      } /* Endif */

      /* Enable the endpoint for Rx and Tx and set the endpoint type */
      if(xd_ptr->BDIRECTION  == MV_USB_SEND)
      {
        dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[xd_ptr->EP_NUM] |=
                         USB_32BIT_LE((EHCI_EPCTRL_TX_ENABLE | EHCI_EPCTRL_TX_DATA_TOGGLE_RST) |
                          (xd_ptr->EP_TYPE << EHCI_EPCTRL_TX_EP_TYPE_SHIFT));
      }
      else
      {
        dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[xd_ptr->EP_NUM] |=
                         USB_32BIT_LE((EHCI_EPCTRL_RX_ENABLE | EHCI_EPCTRL_RX_DATA_TOGGLE_RST) |
                          (xd_ptr->EP_TYPE << EHCI_EPCTRL_RX_EP_TYPE_SHIFT));
      }
      /* Note: Disabled endpoint must be changed from default control type */
      if( (USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[xd_ptr->EP_NUM]) &
            EHCI_EPCTRL_RX_ENABLE) == 0)
      {
          dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[xd_ptr->EP_NUM] |=
              USB_32BIT_LE(MV_USB_BULK_ENDPOINT << EHCI_EPCTRL_RX_EP_TYPE_SHIFT);
      }

      if( (USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[xd_ptr->EP_NUM]) &
            EHCI_EPCTRL_TX_ENABLE) == 0)
      {
          dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[xd_ptr->EP_NUM] |=
              USB_32BIT_LE(MV_USB_BULK_ENDPOINT << EHCI_EPCTRL_TX_EP_TYPE_SHIFT);
      }

      MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_INIT,
                    "init ep #%d %s: type=0x%x, max_pkt=%d, zero_term=%s, ENDPTCTRLX=0x%x\n",
                        xd_ptr->EP_NUM, xd_ptr->BDIRECTION ? "SEND" : "RECV", xd_ptr->EP_TYPE,
                        xd_ptr->WMAXPACKETSIZE, xd_ptr->DONT_ZERO_TERMINATE ? "NO" : "YES",
                        (unsigned)USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[xd_ptr->EP_NUM]));
   }
   else
   {
      MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"ep=%d %s: Init ERROR: ENDPTPRIME=0x%x, ENDPTSTATUS=0x%x, bit_pos=0x%x\n",
                (unsigned)xd_ptr->EP_NUM, xd_ptr->BDIRECTION ? "SEND" : "RECV",
                (unsigned)USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME),
                (unsigned)USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS),
                (unsigned)bit_pos);
        return USBERR_EP_INIT_FAILED;
   } /* Endif */

   return USB_OK;

} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_get_transfer_status
*  Returned Value : USB_OK or error code
*  Comments       :
*        Gets the status of a transfer
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_vusb20_get_transfer_status
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,

      /* [IN] the Endpoint number */
      uint_8                     ep_num,

      /* [IN] direction */
      uint_8                     direction
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_EP_TR_STRUCT_PTR                      dTD_ptr;
   XD_STRUCT_PTR                                xd_ptr;
   uint_8                                       status;

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;

   /* Unlink the dTD */
   dTD_ptr = usb_dev_ptr->EP_DTD_HEADS[2*ep_num + direction];

   if (dTD_ptr)
   {
      /* Get the transfer descriptor for the dTD */
      xd_ptr = (XD_STRUCT_PTR)dTD_ptr->SCRATCH_PTR->XD_FOR_THIS_DTD;
      status = xd_ptr->BSTATUS;
   }
   else
   {
      status = MV_USB_STATUS_IDLE;
   } /* Endif */

   return (status);

} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_get_transfer_details
*  Returned Value : pointer to structure that has details for transfer
*        Gets the status of a transfer
*
*END*-----------------------------------------------------------------*/
XD_STRUCT_PTR  _usb_dci_vusb20_get_transfer_details
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,

      /* [IN] the Endpoint number */
      uint_8                     ep_num,

      /* [IN] direction */
      uint_8                     direction
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   VUSB20_EP_TR_STRUCT_PTR                      dTD_ptr, temp_dTD_ptr;
   XD_STRUCT_PTR                                xd_ptr;
   uint_32                                      temp, remaining_bytes;
   VUSB20_EP_QUEUE_HEAD_STRUCT_PTR              ep_queue_head_ptr;


   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;
   temp = (2*ep_num + direction);

   /* get a pointer to QH for this endpoint */
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)usb_dev_ptr->EP_QUEUE_HEAD_PTR + temp;

   MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_TRACE, "get_transfer_details\n");

   /* Unlink the dTD */
   dTD_ptr = usb_dev_ptr->EP_DTD_HEADS[2*ep_num + direction];

   if (dTD_ptr)
   {
      /* Get the transfer descriptor for the dTD */
      xd_ptr = (XD_STRUCT_PTR)dTD_ptr->SCRATCH_PTR->XD_FOR_THIS_DTD;
      if(!xd_ptr) return NULL;

      /* Initialize the transfer length field */
      xd_ptr->WSOFAR =0;
      remaining_bytes =0;

      /*if length of this transfer is greater than 20K
      we have multiple DTDs to count */
      if(xd_ptr->WTOTALLENGTH > VUSB_EP_MAX_LENGTH_TRANSFER)
      {
         /* it is a valid DTD. We should parse all DTDs for this XD
         and find the total bytes used so far */
         temp_dTD_ptr = dTD_ptr;

         /*loop through the list of DTDS until an active DTD is found
         or list has finished */
         while(!(USB_32BIT_LE(dTD_ptr->NEXT_TR_ELEM_PTR) & VUSBHS_TD_NEXT_TERMINATE))
         {

            /**********************************************************
            If this DTD has been overlayed, we take the actual length
            from QH.
            **********************************************************/

            if (/*(uint_32)*/(USB_32BIT_LE(ep_queue_head_ptr->CURR_DTD_PTR) & VUSBHS_TD_ADDR_MASK) ==
                                     (uint_32)USB_VIRT_TO_PHYS(usb_dev_ptr, temp_dTD_ptr) )
            {
                remaining_bytes +=
                  ((USB_32BIT_LE(ep_queue_head_ptr->SIZE_IOC_INT_STS) & VUSB_EP_TR_PACKET_SIZE) >> 16);
            }
            else
            {
               /* take the length from DTD itself */
                remaining_bytes +=
                  ((USB_32BIT_LE(temp_dTD_ptr->SIZE_IOC_STS) & VUSB_EP_TR_PACKET_SIZE) >> 16);
            }

            dTD_ptr = temp_dTD_ptr;

            /* Get the address of the next dTD */
            temp_dTD_ptr = (VUSB20_EP_TR_STRUCT_PTR)USB_PHYS_TO_VIRT(usb_dev_ptr,
                                (uint_32)(USB_32BIT_LE(temp_dTD_ptr->NEXT_TR_ELEM_PTR) & VUSBHS_TD_ADDR_MASK) );
         }
         xd_ptr->WSOFAR = xd_ptr->WTOTALLENGTH - remaining_bytes;
      }
      else
      {
         /*look at actual length from QH*/
         xd_ptr->WSOFAR = xd_ptr->WTOTALLENGTH -
            ((USB_32BIT_LE(ep_queue_head_ptr->SIZE_IOC_INT_STS) & VUSB_EP_TR_PACKET_SIZE) >> 16);
      }
   }
   else
   {
      xd_ptr = NULL;
   } /* Endif */

   return (xd_ptr);

} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_deinit_endpoint
*  Returned Value : None
*  Comments       :
*        Disables the specified endpoint and the endpoint queue head
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_vusb20_deinit_endpoint
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,

      /* [IN] the Endpoint number */
      uint_8                     ep_num,

      /* [IN] direction */
      uint_8                     direction
)
{ /* Body */
    USB_DEV_STATE_STRUCT_PTR        usb_dev_ptr;
    VUSB20_REG_STRUCT_PTR           dev_ptr;
    VUSB20_EP_QUEUE_HEAD_STRUCT*    ep_queue_head_ptr;
    uint_32                         bit_pos;
    uint_8                          status = USB_OK;

    MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ANY, "deinit_endpoint ep=%d %s\n",
                    (unsigned)ep_num, direction ? "SEND" : "RECV");

    usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
    dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

    /* Get the endpoint queue head address */
    ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)usb_dev_ptr->EP_QUEUE_HEAD_PTR +
                                                                    (2*ep_num + direction);

    bit_pos = (1 << (16 * direction + ep_num));

    /* Check if the Endpoint is Primed */
    if( ((USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME) & bit_pos)) ||
        ((USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS) & bit_pos)) )
    {
        MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING,"ep=%d %s: Deinit ERROR: ENDPTPRIME=0x%x, ENDPTSTATUS=0x%x, bit_pos=0x%x\n",
                (unsigned)ep_num, direction ? "SEND" : "RECV",
                (unsigned)USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME),
                (unsigned)USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS),
                (unsigned)bit_pos);
        status = USBERR_EP_DEINIT_FAILED;
    }

    /* Reset the max packet length and the interrupt on Setup */
    ep_queue_head_ptr->MAX_PKT_LENGTH = 0;

    /* Disable the endpoint for Rx or Tx and reset the endpoint type */
    dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[ep_num] &=
         USB_32BIT_LE( ~((direction ? EHCI_EPCTRL_TX_ENABLE : EHCI_EPCTRL_RX_ENABLE) |
                        (direction ? EHCI_EPCTRL_TX_TYPE : EHCI_EPCTRL_RX_TYPE)));

   return status;
}


/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_shutdown
*  Returned Value : None
*  Comments       :
*        Shuts down the VUSB_HS Device
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_shutdown
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   /* Disable interrupts */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_INTR &=
      ~(USB_32BIT_LE(EHCI_INTR_INT_EN | EHCI_INTR_ERR_INT_EN |
      EHCI_INTR_PORT_CHANGE_DETECT_EN | EHCI_INTR_RESET_EN));

   USB_uncached_memfree(usb_dev_ptr->DRIVER_MEMORY_VIRT,
                        usb_dev_ptr->DRIVER_MEMORY_SIZE,
                        usb_dev_ptr->DRIVER_MEMORY_PHYS);

   /* Reset the Run the bit in the command register to stop VUSB */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD &= ~USB_32BIT_LE(EHCI_CMD_RUN_STOP);

   /* Reset the controller to get default values */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD = USB_32BIT_LE(EHCI_CMD_CTRL_RESET);

} /* EndBody */

/* EOF */


//----------------------------------------------------------------------------------------
//  _usb_dci_vusb20_set_xcvr_interface()
//
//
//----------------------------------------------------------------------------------------
void _usb_dci_vusb20_set_xcvr_interface( _usb_device_handle handle, UINT32 setting, UINT32 port )
{
    USB_DEV_STATE_STRUCT_PTR         usb_dev_ptr;
    VUSB20_REG_STRUCT_PTR            dev_ptr;
    UINT32                           temp;
    
    usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
    dev_ptr     = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;
    temp        = dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX/*[port]*/;
    
    temp &= ~EHCI_PORTSCX_PAR_XCVR_SELECT;
    temp |= setting;
    dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX/*[port]*/ = temp;
}

//----------------------------------------------------------------------------------------
//  _usb_dci_vusb20_set_xcvr_width()
//
//
//----------------------------------------------------------------------------------------
void _usb_dci_vusb20_set_xcvr_width( _usb_device_handle handle, UINT32 setting, UINT32 port )
{
    USB_DEV_STATE_STRUCT_PTR         usb_dev_ptr;
    VUSB20_REG_STRUCT_PTR            dev_ptr;
    UINT32                           temp;

    usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
    dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;
    temp = dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX/*[port]*/;

    temp &= ~EHCI_PORTSCX_PTW_SELECT_MASK;
    temp |= setting;
    dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX/*[port]*/ = temp;
}

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_set_speed_high
*  Returned Value : None
*  Comments       :
*        ENABLE (not guarantee) the controller port to connect
*           in HIGH speed mode.
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_set_speed_high
(
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,

      /* The port number on the device */
      uint_8                     port_number
)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   uint_32                                      port_control;

   MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ANY, "FORCE set_speed_full\n");

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   port_control = USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX/*[port_number]*/);
   port_control &= ~(EHCI_PORTSCX_FORCE_FULL_SPEED_CONNECT);
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX = USB_32BIT_LE(port_control);

} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_enable_endpoint
*  Returned Value : None
*  Comments       :
*        Enable the specified endpoint but does not prime the endpoint
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_vusb20_enable_endpoint
(
     /* [IN] the USB_USB_dev_initialize state structure */
      _usb_device_handle         handle,

      /* [IN] the Endpoint number */
      uint_8                     ep_num,

      /* [IN] MAX Packet size for this endpoint */
      uint_16                    max_pkt_size,

      /* [IN] Direction */
      uint_8                     direction,

      /* [IN] Type of Endpoint */
      uint_8                     type

)
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   uint_32                                      bit_pos;

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   bit_pos = (1 << (16 * direction + ep_num));

   /* Check if the Endpoint is Primed */
   if (((USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME) & bit_pos)) &&
       ((USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS) & bit_pos)))
   {
	 //Flush endpoint
	 dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTFLUSH	|= bit_pos;
	}
  
      /* Enable the endpoint for Rx and Tx and set the endpoint type */
      if(direction  == MV_USB_SEND)
      {
        dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[ep_num] |=
                         USB_32BIT_LE((EHCI_EPCTRL_TX_ENABLE | EHCI_EPCTRL_TX_DATA_TOGGLE_RST) |
                          (type << EHCI_EPCTRL_TX_EP_TYPE_SHIFT));
      }
      else
      {
        dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[ep_num] |=
                         USB_32BIT_LE((EHCI_EPCTRL_RX_ENABLE | EHCI_EPCTRL_RX_DATA_TOGGLE_RST) |
                          (type << EHCI_EPCTRL_RX_EP_TYPE_SHIFT));
      }
      /* Note: Disabled endpoint must be changed from default control type */
      if( (USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[ep_num]) &
            EHCI_EPCTRL_RX_ENABLE) == 0)
      {
          dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[ep_num] |=
              USB_32BIT_LE(MV_USB_BULK_ENDPOINT << EHCI_EPCTRL_RX_EP_TYPE_SHIFT);
      }

      if( (USB_32BIT_LE(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[ep_num]) &
            EHCI_EPCTRL_TX_ENABLE) == 0)
      {
          dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[ep_num] |=
              USB_32BIT_LE(MV_USB_BULK_ENDPOINT << EHCI_EPCTRL_TX_EP_TYPE_SHIFT);
      }

    return USB_OK;

} /* EndBody */
