/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2006 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:
**
**  PURPOSE:
**                      USB 2.0 Client Device Controller DMA transfer routines
**
**
**  LAST MODIFIED:
**
******************************************************************************/

#include "xllp_defs.h"
#include "misc.h"      // memcpy
#include "xllp_u2d.h"
#include "protocolmanager.h"
#include "interrupts.h"

extern void EnableIrqInterrupts(void);
extern void DisableIrqInterrupts(void);

XLLP_U2D_T U2DDeviceControl;
volatile P_XLLP_U2D_T                  U2DDeviceController = &U2DDeviceControl;
static   P_XLLP_U2D_T                  pU2DHandle;
static   P_XLLP_U2D_REGISTERS_T        pU2DRegs;
const    XLLP_U2D_EP_T             usbTransmitEp = U2D_ENDPOINT_A;
const    XLLP_U2D_EP_T             usbReceiveEp  = U2D_ENDPOINT_B;

//----------------------------------------------------------------------------
//  U2DTransmit
//
//
//
//----------------------------------------------------------------------------
XLLP_UINT32_T U2DTransmit( XLLP_UINT32_T TxDataLength,
                           XLLP_UINT8_T  *pTxBuff,
                           XLLP_UINT8_T  SyncTransfer      )
{
    UINT8_T Retval = NoError;
    XLLP_INT32_T actualRxCount = 0;
	static __attribute__((aligned(16)))XLLP_UINT8_T usbBuffer[256];
    P_XLLP_UINT8_T pLocal;

    if( TxDataLength > SYS_U2D_DMA_ENDPOINT_A_TRANSFER_MEM_SIZE )
       return 1;

    //-------------------------------------------------
    // NOTE: Although I attempted to speed up this code
    //       by by-passing the aligned local buffers
    //       the result was that the DMA transfer would
    //       fail.  I suspect bad DMA interaction with
    //       the NAND somehow...  I leave the code here
    //       for possible future improvement...
    //-------------------------------------------------

    // DMA transfers with the U2D must be with buffers
    // on minimum 8 byte alignment
    //------------------------------------------------
    //if( (XLLP_UINT32_T)pTxBuff & 0x07 )
    {
      pLocal = usbBuffer;
      memcpy( pLocal, pTxBuff, TxDataLength );
    }
    //else
    //  pLocal = pTxBuff;

    Retval = XllpU2DMAxfer( usbTransmitEp,          // Endpoint
                            pLocal,                 // Memory buffer
                            TxDataLength,           // Buffer Size
                            SyncTransfer,           // return after transmit complete?
                            &actualRxCount          // Not used on transmit...
                          );

    return (Retval);
}

//----------------------------------------------------------------------------
//  U2DReceive
//
//
//
//----------------------------------------------------------------------------
XLLP_UINT32_T U2DReceive( XLLP_UINT32_T RxDataLength,
                          XLLP_UINT8_T  *pRxBuff,
                          XLLP_INT32_T  *byteCount
						  )
{
    UINT8_T Retval = NoError, SyncRx = 1;
	static __attribute__((aligned(8)))XLLP_UINT8_T usbBuffer[4];
    P_XLLP_UINT8_T pLocal;
	XLLP_UINT8_T	copyBack = 0;

    if( RxDataLength > SYS_U2D_DMA_ENDPOINT_B_TRANSFER_MEM_SIZE  )
       return 1;

    //-------------------------------------------------
    // NOTE: Although I attempted to speed up this code
    //       by by-passing the aligned local buffers
    //       the result was that the DMA transfer would
    //       fail.  I suspect bad DMA interaction with
    //       the NAND somehow...  I leave the code here
    //       for possible future improvement...
    //-------------------------------------------------

    // DMA transfers with the U2D must be with buffers
    // on minimum 8 byte alignment
    //-------------------------------------------------------
    if( (XLLP_UINT32_T)pRxBuff & 0x07 )
	{
       pLocal = usbBuffer;
	   copyBack = 1;
	}
	else
       pLocal = pRxBuff;

    Retval = XllpU2DMAxfer( usbReceiveEp,       // Endpoint
                            pLocal,             // Memory buffer
                            RxDataLength,       // Buffer Size
                            SyncRx,             // synchronus Receive
                            byteCount           // # bytes transferred by U2D
                          );

    // If we used a local buffer for alignment
    // purposes, we need to copy back to the
    // callers buffer
    //----------------------------------------
    if( copyBack && (*byteCount > 0) )
    {
      RxDataLength = (*byteCount > RxDataLength) ? RxDataLength : *byteCount ;
      memcpy( pRxBuff, pLocal, RxDataLength );
    }

    return (Retval);
}

//------------------------------------------------------------------
//  U2DDownloadInit
//
//
//------------------------------------------------------------------
void U2DDownloadInit(void)
{


   // Initialize the USB context structure
   //-------------------------------------
   BootRomU2DSWInit();

   // Initialize the handles for this module
   //---------------------------------------
   pU2DHandle = U2DDeviceController;
   pU2DRegs   = (P_XLLP_U2D_REGISTERS_T)pU2DHandle->pRegsBase;

   // Allow interrupts from the U2D
   //-------------------------------------------
   EnablePeripheralIRQInterrupt(U2D_CLIENT_INT);

   // Enable IRQ interrupts
   //----------------------
   EnableIrqInterrupts();

   // Setup U2D Hardware
   //-------------------
   //BootRomU2DHWSetup();

}


//--------------------------------------------------------------------------
// U2DGetMsg -> Get bytes/packet etc. from U2D
//              This executes in Interrupt execution context chain
//
//--------------------------------------------------------------------------
void U2DGetMsg(void)
{
  pProtocolCmd pCmd;
  pProtocolISR pISR;
  XLLP_INT32_T byteCount = 0, PacketLength = 0;
//  static XLLP_INT32_T DebugBreakCount = 0;
  UINT8_T Buff[PREAMBLE_SIZE], *buffPtr;
	UINT8_T dummy[8];

  buffPtr = Buff;
  pISR    = getProtocolISR();
  pCmd    = getProtocolCmd();

  PacketLength = (pU2DRegs->U2DBCR_X[usbReceiveEp] & XLLP_U2DBCRX_BC_MASK);

  // Is it the Preamble ??
  //----------------------
  if( PacketLength == PREAMBLE_SIZE )
  {
      //Clear the stack buffer
      //----------------------
      memset( buffPtr, 0, sizeof(Buff));

      //Get the bytes from the U2D
      //--------------------------
      U2DReceive( PREAMBLE_SIZE, buffPtr, &byteCount);

      // See if it's the Preamble string
      //--------------------------------
      if( ProtocolVerifyPreamble( buffPtr ) != 0 )
      {
         #if !VERBOSE_DEBUG //leave this on for dumping messages
         ShutdownPort(FFUART_D);
		 #endif
         ShutdownPort(DIFF_USB_D);
         pISR->PreambleReceived = TRUE;
         pISR->InterruptPort    = U2D_USB_D;

		 ClearPortInterruptFlag();
         // Preamble checks out, so respond
         //--------------------------------
         SendResponse(Buff, PREAMBLE_SIZE);
         return;
      }
  }

  //-------------------------------------------------------------------
  // ok, process command stream
  //-------------------------------------------------------------------

  //------------------------------------------
  // Check for bogus transmission, if so then
  // zero the command structure, and let the
  // protocol layer handle it...
  //------------------------------------------
  if( PacketLength < 8 )
  {
		U2DReceive( PacketLength, (XLLP_UINT8_T *)&dummy, &byteCount);
     return;
  }

  // packet size seems ok ...
  // read constant part of command packet
  //-------------------------------------
  U2DReceive( 8, (XLLP_UINT8_T *)&(pCmd->Command), &byteCount);
  PacketLength -= byteCount;

  // update Interrupt Status Flags
  //-----------------------------
  pISR->CurrentCommand  = pCmd->Command;
  pISR->DataLeft        = pCmd->Length;
  pISR->CommandReceived = TRUE;

  switch( pISR->CurrentCommand )
  {
	case MessageCmd:
		//try and send a message
		HandleMessageCmd();
		break;

	case DownloadDataCmd:

         // Next we get the rest of the image that follows
         // the command stucture and put it in the proper
         // image area
         //
         // NOTE: this should not have to loop like this
         //       to complete the DATA transfer... this
         //       is a work-around until I can figure out
         //       if my DMA code is BAD or there's a
         //       Monahans B0 or "P" silicon bug...
         //-----------------------------------------------
         while( pISR->DataLeft > 0 )
         {
           U2DReceive( pISR->DataLeft, (XLLP_UINT8_T *)(pISR->CurrentDownloadAddress), &byteCount);
           pISR->CurrentDownloadAddress += byteCount;
           pISR->DataLeft -= byteCount;
         }

         break;

	case DisconnectCmd:
		pISR->PreambleReceived = TRUE;
    default:

         // rest of messages: should not have over 4 bytes of data
         // this will need to change if protocol changes
         //-------------------------------------------------------
         if(PacketLength > 4)
           PacketLength = 4;

         if(PacketLength)
           U2DReceive( PacketLength, (XLLP_UINT8_T *)(&(pCmd->Data[0])), &byteCount);

         break;
  }
}


//-----------------------------------------
//  ImageU2DInterruptHandler
//
//  Top Level Application interrupt handler
//  for U2D
//
//  We pass through the initial interrupt
//  to the lower-level handler first,
//  then take care of the application next
//
//-----------------------------------------
void ImageU2DInterruptHandler(void)
{
	pU2DHandle = U2DDeviceController;
	pU2DRegs   = (P_XLLP_U2D_REGISTERS_T)pU2DHandle->pRegsBase;
	// check the OTG/ULPI register for line changes (i.e. cable plugin)

  // Call the EPO, and USB config handler
  //-------------------------------------
  U2DInterruptHandler();

  //allow enumeration.  check to make sure downloads are allowed
  if( ( pU2DHandle->enumerateComplete) == TRUE )
  {
	  if( getProtocolISR()->DownloadsEnabled)
	  {
		  // did we get an interrupt on Endpoint "B" which is used
		  // for host to U2D transport layer? look for short packet
		  // and packet complete interrupt bits
		  //-----------------------------------------------------------
		  if( USBEndPointPacketInterruptReceived( usbReceiveEp ) )
		  {
			  // Call the application layer
			  // USB 2.0 interrupt handler
			  //---------------------------
			  U2DGetMsg();
		  }
		}
		else {

		  DisablePeripheralIRQInterrupt(U2D_CLIENT_INT);
		  // Disable the interrupt requests from endpoints 0 -> G,
		  // and all Event interrupts
		  //------------------------------------------------------
		  pU2DRegs->U2DICR = 0;
		  // Clear any spurious interrupts
		  //------------------------------
		  pU2DRegs->U2DISR  = 0xfeffffff;

		  ClearPortInterruptFlag();
	  }
  }
}
