/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************

  Function Name:
    XllpUdcHWShutdown

  Description:
    This function will shutdown the U2D Controller's hardware and clears the U2D
    handle's variables.

  Global Register Modified:
    None.

  Input Arguments:
    P_XLLP_U2D_T pU2DHandle - a pointer to the U2D's handle

  Output Arguments:
    None

  Return Value:
    XLLP_STATUS_T - error code if failed, zero if success

*******************************************************************************/

XLLP_STATUS_T XllpU2DHWShutdown (P_XLLP_U2D_T pU2DHandle)
{
  P_XLLP_U2D_REGISTERS_T pRegs = (P_XLLP_U2D_REGISTERS_T)pU2DHandle->pRegsBase;
  XLLP_UINT32_T tempReg;
  XLLP_STATUS_T error = 0;

  //-------------------------------------------------------
  // read-back reg values to ensure in-order execution path
  //-------------------------------------------------------

  // Disable the interrupt requests from endpoints 0 -> G,
  // and all Event interrupts
  //------------------------------------------------------
  pRegs->U2DICR = 0;
  tempReg = pRegs->U2DICR;

  // Disable first level U2D interrupt
  error = XllpU2DDisableFirstLevelInterrupt();

  // Disconnect the UTMI (transciever) from the U2D
  //-----------------------------------------------
  MonsU2DPlatformDefault();

  // This also resets endpoint info registers
  // and clears the config registers
  //-----------------------------------------
  pRegs->U2DCR = 0;
  tempReg = pRegs->U2DCR;
  for( tempReg=1;tempReg<8;tempReg++)
    pRegs->U2DCR_X[tempReg] = 0;

  // Clear the rest of the U2D handles variables
  //---------------------------------------------
  pU2DHandle->setupComplete =     XLLP_FALSE;
  pU2DHandle->enumerateComplete = XLLP_FALSE;
  pU2DHandle->cableAttached =     XLLP_FALSE;
  pU2DHandle->enableDma =         XLLP_FALSE;

  pU2DHandle->usbConfigNumActive         = 0;
  pU2DHandle->usbInterfaceNumActive      = 0;
  pU2DHandle->usbIntAltSettingsNumActive = 0;

  // Initialize Endpoint 0 state machine to the Idle state
  //------------------------------------------------------
  pU2DHandle->controlXfer.statusEp0 = EP0_IDLE_STATE;

  // Clear any spurious interrupts
  //------------------------------
  pRegs->U2DISR  = 0xfeffffff;
  tempReg = pRegs->U2DISR;

  return error;
}

/******************************************************************************

  Function Name:
    XllpU2DSWInit

  Description:
    This function initializes the handle's variables that will be used by the
    U2D driver and no hardware will be accessed it this point.

  Global Register Modified:
    None.

  Input Arguments:
    P_XLLP_U2D_T pU2DHandle - a pointer to the U2D's handle
	P_XLLP_U2D_REGISTERS_T pRegs - a pointer to the U2D's registers base

  Output Arguments:
    None

  Return Value:
    None

*******************************************************************************/

void XllpU2DSWInit (P_XLLP_U2D_T pU2DHandle, P_XLLP_U2D_REGISTERS_T pRegs)
{
    // Set the U2D handles pointer to the U2D Registers
    // with the base address of the U2D Registers.
    //---------------------------------------------------
    pU2DHandle->pRegsBase = (P_XLLP_U2D_REGISTERS_T)pRegs;

    // Sets the U2D handles pointer to the Endpoints Configuration Table
    // with the pointer to the default Endpoints Configuration Table.
    //--------------------------------------------------------------------
    pU2DHandle->pConfigTable = (P_XLLP_U2D_EP_CONFIG_TABLE_T)defaultU2DEndpointConfigTable;

#if 0
    // Clear the Interrupt Statistics structure
    memset (&pU2DHandle->interruptStat, 0, sizeof(XLLP_U2D_INT_STATISTICS_T));

    // Clear the Endpoints transfer structures
    memset (&pU2DHandle->EpXferTable[0], 0, (sizeof(XLLP_U2D_XFER_T) * XLLP_U2D_MAX_EP_NUM));

    // Clear the Endpoint 0 control structure
    memset (&pU2DHandle->controlXfer, 0, sizeof(XLLP_U2D_USB_CTRL_XFER_T));

    // Clear the Vendor requests control structure
    memset (&pU2DHandle->vendorReq, 0, sizeof(XLLP_U2D_VENDOR_REQ_DATA_T));

    // Clear the List of Active Endpoints
    memset (&pU2DHandle->listOfActiveEndpoints, 0,
                         (sizeof(XLLP_U2D_USB_BIND_ENDPOINTS_T) * XLLP_U2D_MAX_EP_NUM));
#endif

    // Clear the rest of the U2D handles variables
	pU2DHandle->setupComplete     = XLLP_FALSE;
    pU2DHandle->enumerateComplete = XLLP_FALSE;
    pU2DHandle->cableAttached     = XLLP_FALSE;
    pU2DHandle->enableDma         = XLLP_FALSE;

    pU2DHandle->usbConfigNumActive         = 0;
    pU2DHandle->usbInterfaceNumActive      = 0;
    pU2DHandle->usbIntAltSettingsNumActive = 0;

    // Initialize Endpoint 0 state machine to the Idle state
    //------------------------------------------------------
    pU2DHandle->controlXfer.statusEp0 = EP0_IDLE_STATE;

    // Get the U2D DMA initialized
    //----------------------------
    XllpU2DInit(pU2DHandle);
}