/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
** INTEL CONFIDENTIAL
** Copyright 2003-2004 Intel Corporation All Rights Reserved.
**
** The source code contained or described herein and all documents
** related to the source code (Material) are owned by Intel Corporation
** or its suppliers or licensors.  Title to the Material remains with
** Intel Corporation or its suppliers and licensors. The Material contains
** trade secrets and proprietary and confidential information of Intel
** or its suppliers and licensors. The Material is protected by worldwide
** copyright and trade secret laws and treaty provisions. No part of the
** Material may be used, copied, reproduced, modified, published, uploaded,
** posted, transmitted, distributed, or disclosed in any way without Intel's
** prior express written permission.
**
** No license under any patent, copyright, trade secret or other intellectual
** property right is granted to or conferred upon you by disclosure or
** delivery of the Materials, either expressly, by implication, inducement,
** estoppel or otherwise. Any license under such intellectual property rights
** must be express and approved by Intel in writing.
**
**  FILENAME: xllp_dfc.c
**
**  PURPOSE:  XLLP interface to the data flash controller (DFC).
**
******************************************************************************/

#include "xllp_dfc.h"
#include "platform_dfc_timing.h"
#include "platformconfig.h"

////////////
/////////////
//////////       S U P P O R T   R O U T I N E S
////////////
///////////

/** Workhorses **/
extern int  xdfc_write      (P_DFC_BUFFER buffer, unsigned int address, unsigned int bytes, P_FlashProperties_T pFlashDevices);
extern int  xdfc_read       (P_DFC_BUFFER buffer, unsigned int address, unsigned int bytes, unsigned int* SpareArea, P_FlashProperties_T pFlashDevices);
extern int  xdfc_readID     (P_DFC_BUFFER buffer, DEVICE_TYPE dev_type);
extern int  xdfc_reset      (void);
extern int  xdfc_readstatus (P_DFC_BUFFER buffer, P_FlashProperties_T pFlashDevices);
extern int  xdfc_erase      (unsigned int address, P_FlashProperties_T pFlashDevices);
extern void xdfc_setTiming    (FLASH_TIMING* flash);


/*
 *  NAND Small block device ID's
 */

char Samsung[] ={ 0x71, 0x78, 0x79, 0x72,
				  0x74, 0x36, 0x76, 0x46,
				  0x56, 0x35, 0x75, 0x45,
				  0x55, 0x33, 0x73, 0x43,
				  0x53, 0x39, 0xE6, 0x49,
				  0x59, 0x00};

char ST[] = {0x73, 0x35, 0x75, 0x45,
			 0x55, 0x76, 0x36, 0x46,
			 0x56, 0x79, 0x39, 0x49,
			 0x59, 0x00};

char Micron[] = {0x00};

char Hynix[] = {0x76, 0x56, 0x36, 0x46,
				0x75, 0x55, 0x35, 0x45,
				0x73, 0x53, 0x49, 0x00};

char Toshiba[] = {0x46, 0x79, 0x75, 0x73, 0x72, 0xE6, 0x00};

/*
 * Default settings:
 *   Large block:
 *       read -     CMD1 = 0x00                pg_program -     CMD1 = 0x80
 *                  CMD2 = 0x30                                 CMD2 = 0x10
 *                  ADDR_CYC = 5                                ADDR_CYC = 5
 *                  DBC = 1                                     DBC = 1
 *                  NC = 0                                      NC = 0
 *                  CMD_TYPE = 0x0                              CMD_TYPE = 0x1
 *                  CSEL = 0                                    CSEL = 0
 *                  AUTO_RS = 0                                 AUTO_RS = 0
 *        read_id - CMD1 = 0x90                blk_erase-       CMD1 = 0x60
 *                  CMD2 = 0x00                                 CMD2 = 0xD0
 *                  ADDR_CYC = 1                                ADDR_CYCLE = 0X3
 *                  DBC = 0                                     DBC = 1
 *                  NC = 0                                      NC = 0
 *                  CMD_TYPE = 0x3                              CMD_TPYE = 0X3
 *                  CSEL = 0                                    CSEL = 0
 *                  AUTO_RS = 0                                 AUTO_RS = 0
 *        read_status -     CMD1 = 0x70                         CMD2 = 0x00
 *                  ADDR_CYC = 0
 *                  DBC = 0
 *                  NC = 0
 *                  CMD_TYPE = 0x6
 *                  CSEL = 0
 *                  AUTO_RS = 0
 * Small Block
 *         read -   CMD1 = 0x00                 pg_program -     CMD1 = 0x80
 *                  CMD2 = 0x00                                  CMD2 = 0x10
 *                  ADDR_CYC = 4                                 ADDR_CYC = 5
 *                  DBC = 0                                      DBC = 1
 *                  NC = 0                                       NC = 0
 *                  CMD_TYPE = 0x0                               CMD_TYPE = 0x1
 *                  CSEL = 0                                     CSEL = 0
 *                  AUTO_RS = 0                                  AUTO_RS = 0
 *          read_id - CMD1 = 0x90                blk_erase-      CMD1 = 0x60
 *                    CMD2 = 0x00                                CMD2 = 0xD0
 *                    ADDR_CYC = 1                               ADDR_CYCLE = 0x3
 *                    DBC = 0                                    DBC = 1
 *                    NC = 0                                     NC = 0
 *                    CMD_TYPE = 0x3                             CMD_TPYE = 0x3
 *                    CSEL = 0                                   CSEL = 0
 *                    AUTO_RS = 0                                AUTO_RS = 0
 *          read_status -     CMD1 = 0x70
 *                    CMD2 = 0x00
 *                    ADDR_CYC = 0
 *                    DBC = 0
 *                    NC = 0
 *                    CMD_TYPE = 0x6
 *                    CSEL = 0
 *                    AUTO_RS = 0
 *
 *
 */

// Read		ReadID		ReadStat   Program	  Erase		Reset
static CMD_BLOCK CMD_ARRAY[DEVICE_TYPE_SIZE] =
{
#if (PV_CHIP)
//LARGE
{ 0x000D3000,0x00610090,0x00800070,0x002D1080,0x004BD060,0x00A000ff},
//SMALL
{ 0x00040000,0x00610090,0x00800070,0x002D1080,0x004BD060,0x00A000ff}
#else
//LARGE
{ 0x000D3000,0x00610090,0x00800070,0x002D1080,0x004BD060,0x00B000ff},
//SMALL
{ 0x00040000,0x00610090,0x00800070,0x002D1080,0x004BD060,0x00B000ff}
#endif
};


/***********************************************************************
////////////
/////////////
//////////       D F C   I N T E R F A C E   R O U T I N E S
////////////
///////////
***********************************************************************/
int XllpDfcInit(unsigned int width, P_FlashProperties_T pFlashDevices)
{
	NDCR_REG  		dfc_control_reg;
	NDTR0CS0_REG 	dfc_timing0_reg;
	NDTR1CS0_REG	dfc_timing1_reg;
    unsigned int  buf[2]; // Read ID data should not be more than 5 bytes.
    unsigned int Retval = 0;

	FLASH_TIMING* pFT;		  // For use in timing override

	/*
	 * Setup Multifunction pins for NAND
	 */

	ChipSelectDFC();

	/*
	 * Initial DFC state:
	 * See values in xllp_dfc_defs.h
	 * 	- Disable all interrupts
	 * 	- Read_ID count = 2
	 *  - SPARE and ECC enabled
	 *  - DMA disabled
	 *  - DWIDTH is x16
	 *	- Small block PAGE_SZ
	 *
	 *	Set timing to the worst (i.e. slowest) possible timing
	 *		This can be adjusted after we determine the Flash type
	 */
    pFlashDevices->FlashBusWidth = width;
	dfc_control_reg.value = DFC_INITIAL_CONTROL;
	dfc_control_reg.bits.DMWIDTH_M = (width == 8)? 0 : 1;
    dfc_control_reg.bits.DMWIDTH_C = (width == 8)? 0 : 1;
#if (PV_CHIP)
	// Use default values
	dfc_timing0_reg.value = *DFC_TIMING_0;
	dfc_timing1_reg.value = *DFC_TIMING_1;
	// Set wait Mode
	dfc_timing1_reg.bits.wait_mode = 1;
#else
	dfc_timing0_reg.value = DFC_INITIAL_TIMING0;
	dfc_timing1_reg.value = DFC_INITIAL_TIMING1;
#endif
	/*
     *  DFC Workaround code turn off Arbiter and NAND Clock before writing registers
	 */
#if DFC_WORKAROUND	// Turn OFF clock
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
#endif

    /*
     *  Setup the DFC controller for generic probing
     *
     */
    *DFC_CONTROL  = dfc_control_reg.value;
	*DFC_TIMING_0 = dfc_timing0_reg.value;
	*DFC_TIMING_1 = dfc_timing1_reg.value;
	/*
     *  DFC Workaround code turn on Arbiter and NAND Clock after writing registers
	 */
#if DFC_WORKAROUND		// Turn ON clock
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
#endif
	/*
	 * First, issue a reset command
	 */
	Retval = xdfc_reset();
	if ( Retval != 0 )
		return Retval;

	/*
	 *  Get the device ID first and check if it is small block
	 *	 or large block.  If large block go back and read all 4
	 *   bytes from the read id to get device parameters.
	 */
	Retval = xdfc_readID(&buf[0], SMALL);
  	if ( Retval == 0 )
  	{
  		 pFlashDevices->NAND_Prop.NAND_type = XllpCheckDeviceType(&buf[0], pFlashDevices);
   		 /*
		 *  Once we determine Large or Small block set up the flash
		 *   parameters structure and adjust the DFC Control register settings
		 */
		 if(pFlashDevices->NAND_Prop.NAND_type == LARGE)
		 {
			Retval = xdfc_readID(&buf[0], LARGE);
			if ( Retval != 0 )
				return Retval;
			if ((buf[0] == 0) && (buf[1] == 0))
				return DFCInitFailed;

			pFlashDevices->NAND_Prop.device_cmds = &CMD_ARRAY[LARGE];
			XllpLoadProperties(&buf[0], pFlashDevices);
			dfc_control_reg.value = *DFC_CONTROL;
			dfc_control_reg.bits.PAGE_SZ = 1;
			dfc_control_reg.bits.PG_PER_BLK = 1;
			dfc_control_reg.bits.RA_START = 1;
		 }
		 else	// Its Small Block
		 {
			pFlashDevices->NAND_Prop.device_cmds = &CMD_ARRAY[SMALL];
			pFlashDevices->NAND_Prop.PageSize = 512;
			pFlashDevices->BlockSize = 512 * 32;
			pFlashDevices->NAND_Prop.SpareAreaSize = 16;
			pFlashDevices->NAND_Prop.NumBBTSlots = 24;
			dfc_control_reg.value = *DFC_CONTROL;
			dfc_control_reg.bits.PAGE_SZ = 0;
			dfc_control_reg.bits.PG_PER_BLK = 0;
			dfc_control_reg.bits.RA_START = 0;
		 }

		/*
		*  DFC Workaround code turn OFF Arbiter and NAND Clock
		*/
		#if DFC_WORKAROUND	// Turn OFF clock
			*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
			*ADDR_CCU_D0CKEN_A;
		#endif
		*DFC_CONTROL = dfc_control_reg.value;

		 /*	Overwrite Some Device Specific Data
		 *	Timing is done by the DKB/OBM only since DKB/OBM needs to support more than
		 *  	what BootROM needs.
		 */

		 switch (pFlashDevices->ManufacturerCode)
		 {
			case SAMSUNG_CODE:
			{
					if (pFlashDevices->FlashID == 0xAA)		 // LARGE Block - Part K9K2G08R0A
					{
						pFlashDevices->NumOfBlocks = 2048;
						#if !(BOOTROM)
					   		pFT = &SAMSUNG_MFG_TIMING[3];				  // Override Timing
					   		xdfc_setTiming(pFT);
						#endif
					}
					else	// SMALL Block - All Supported
					{
						pFlashDevices->NumOfBlocks = 4096;
						#if !(BOOTROM)
					   		pFT = &SAMSUNG_MFG_TIMING[1];				  // Override Timing
					   		xdfc_setTiming(pFT);
						#endif
					}
					break;
			}
			case TOSHIBA_CODE:
					break;
			case HYNIX_CODE:
					break;
			case ST_CODE:
					break;
			case MICRON_CODE:
			{
					// Cover Parts #MT29F1GXXABA LB
					if ((pFlashDevices->FlashID == 0xA1) || (pFlashDevices->FlashID == 0xB1))
					{
						pFlashDevices->NumOfBlocks = 1024;
						#if !(BOOTROM)
					   		pFT = &MICRON_MFG_TIMING[1];				  // Override Timing
					   		xdfc_setTiming(pFT);
						#endif
					}
					break;
			}
			default:
				    pFlashDevices->NumOfBlocks = 1024;
					break;
		 }
		/*
		*  DFC Workaround code turn ON Arbiter and NAND Clock
		*/
	#if DFC_WORKAROUND		// Turn ON clock
		*ADDR_CCU_D0CKEN_A |= 0x10;
		*ADDR_CCU_D0CKEN_A;
	#endif
	}

	return Retval;
}

DEVICE_TYPE XllpCheckDeviceType( P_DFC_BUFFER buffer, P_FlashProperties_T pFlashDevices )
{
    char maker, device;
    DEVICE_TYPE type = LARGE;
	int match = 0;
	char * search_list;

    maker  = pFlashDevices->ManufacturerCode = (buffer[0]  & 0xFF);
    device = pFlashDevices->FlashID = ((buffer[0] >> 8) & 0xFF);

	/*
	 *  Check to see if this is a known small block device
	 */
    switch (maker){
		case SAMSUNG_CODE:
				search_list = &Samsung[0];
				break;
		case TOSHIBA_CODE:
				search_list = &Toshiba[0];
				break;
		case HYNIX_CODE:
				search_list = &Hynix[0];
				break;
		case ST_CODE:
				search_list = &ST[0];
				break;
		case MICRON_CODE:
				search_list = &Micron[0];
				break;
		default:
			return (type);
	}
	while (match == 0)
	{
		if(*search_list == device)
		{
			match = 1;
			type = SMALL;
		}
		if(*search_list == 0)
			match = 1;
		search_list++;
	}

	return (type);
}

void XllpLoadProperties(P_DFC_BUFFER buffer, P_FlashProperties_T pFlashDevices)
{
    char maker, properties;
	char pg_size, blk_size, spare_size, organization;

    maker  = buffer[0]  & 0xFF;
    properties = (buffer[0] >> 24) & 0xFF;

	pg_size = properties & 0x3;
	blk_size = (properties >> 4) & 0x3;
	organization = (properties >> 6) & 0x1;

	switch(pg_size){
		case 0:
			pFlashDevices->NAND_Prop.PageSize = 1024;
			break;
		case 1:
			pFlashDevices->NAND_Prop.PageSize = 2048;
			break;
		case 2:
			pFlashDevices->NAND_Prop.PageSize = 4096;
			break;
		case 3:
			pFlashDevices->NAND_Prop.PageSize = 8192;
			break;
		default:
			pFlashDevices->NAND_Prop.PageSize = 2048;

	}
	pFlashDevices->BlockSize =  pFlashDevices->NAND_Prop.PageSize * 64;
	pFlashDevices->NAND_Prop.NumBBTSlots = 63;
	switch(maker)
	{
		case MICRON_CODE:
				spare_size = (properties >> 2) & 0x3;
				pFlashDevices->NAND_Prop.SpareAreaSize = 64;  //only documented size
				break;
		case SAMSUNG_CODE:
		case TOSHIBA_CODE:
		case HYNIX_CODE:
		case ST_CODE:
		default:
				spare_size = (properties >> 2) & 0x1;
				if(spare_size == 0)
				{
					pFlashDevices->NAND_Prop.SpareAreaSize = (pFlashDevices->NAND_Prop.PageSize / 512) * 8;
				}
				else
				{
					pFlashDevices->NAND_Prop.SpareAreaSize = (pFlashDevices->NAND_Prop.PageSize / 512) * 16;
				}
	}

}

/*
 * Wrapper functions below this point
 */
int XllpDfcWrite(P_DFC_BUFFER buffer, unsigned int address, unsigned int bytes, P_FlashProperties_T pFlashDevices)
{
   return xdfc_write(buffer, address, bytes, pFlashDevices);
}

int XllpDfcRead(P_DFC_BUFFER buffer, unsigned int address, unsigned int bytes, unsigned int* SpareArea, P_FlashProperties_T pFlashDevices)
{
   return xdfc_read(buffer, address, bytes, SpareArea, pFlashDevices);
}


int XllpDfcErase(unsigned int address, P_FlashProperties_T pFlashDevices)
{
   return xdfc_erase(address, pFlashDevices);
}
