/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
** INTEL CONFIDENTIAL
** Copyright 2003-2004 Intel Corporation All Rights Reserved.
**
** The source code contained or described herein and all documents
** related to the source code (Material) are owned by Intel Corporation
** or its suppliers or licensors.  Title to the Material remains with
** Intel Corporation or its suppliers and licensors. The Material contains
** trade secrets and proprietary and confidential information of Intel
** or its suppliers and licensors. The Material is protected by worldwide
** copyright and trade secret laws and treaty provisions. No part of the
** Material may be used, copied, reproduced, modified, published, uploaded,
** posted, transmitted, distributed, or disclosed in any way without Intel's
** prior express written permission.
**
** No license under any patent, copyright, trade secret or other intellectual
** property right is granted to or conferred upon you by disclosure or
** delivery of the Materials, either expressly, by implication, inducement,
** estoppel or otherwise. Any license under such intellectual property rights
** must be express and approved by Intel in writing.
**
**  FILENAME: xllp_dfc.h
**
**  PURPOSE:  XLLP interface to the data flash controller (DFC).
**
******************************************************************************/

#ifndef __XLLP_DFC_H__
#define __XLLP_DFC_H__

#include "typedef.h"
#include "general.h"
#include "xllp_dfc_defs.h"
#include "xllp_dfc_config.h"

//
//	Structure to keep track of NAND device parameters
//		To be initialized during the XllpDfcInit routine
//		then used by the work functions in xllp_dfc_support.c
//

// Structures for NAND support
#define NAND_RELOC_MAX 			511			// We make this large enough to fit a max 2K page size
#define VALID_HEADER            0x0000524E  // = 'N' + ('R'*256)

typedef struct {
   USHORT  From;
   USHORT  To;
   } Rel_T;

typedef struct {
   USHORT  Header;						// 0x0
   USHORT  NumReloc;					// 0x2
   Rel_T   Relo[NAND_RELOC_MAX];		// 0x4
   } Reloc_T;							// Size = 0x4 + (511 * 0x4) = 0x800 (2KB)

typedef enum
{
	TYAX_FLASH   	= 1,
	SIBLEY_FLASH	= 2,
	NAND_FLASH  	= 3,
	OneNAND 		= 4,
    MSYS_DOC_FLASH  = 5
} FlashType_T;

// Indicates if there were any changes to the local BBT during this execution of the code
typedef enum
{
	BBT_NOTFOUND	= 0,
	BBT_UNCHANGED  	= 1,
	BBT_CHANGED		= 2
} RelocState_T;

typedef struct
{
    unsigned int              BaseAddr;
}XIP_Properties;

typedef struct
{
 	unsigned int             	PageSize;
 	unsigned int             	SpareAreaSize;
 	DEVICE_TYPE					NAND_type;
	CMD_BLOCK					*device_cmds;
 	Reloc_T        				*pLocalRelotable;           // Pointer to local relo table
 	Reloc_T           			*pRelotable;                  // Points (NAND address) to relo table
	RelocState_T				BBT_State;
	unsigned int				BBT_Slot;					// Indicates which slot/page in Block BBT located
	unsigned int				NumBBTSlots;
}NAND_Properties;

typedef struct
{
	FlashType_T      		FlashType;
    unsigned int           	ManufacturerCode;
    unsigned int           	FlashID;
    unsigned int           	NumOfBlocks;
    unsigned int           	BlockSize;
    unsigned int           	BufferSize;                    // Write Buffer
    unsigned int           	FlashBusWidth;
	unsigned int			NextSlotAvailable;				// Used for WTM Save State slot pointer
    XIP_Properties   		XIP_Prop;
    NAND_Properties  		NAND_Prop;
}FlashProperties_T, *P_FlashProperties_T;


///////////////////////////////////////////////////////////////////////////////
/////
///     D F C   I N T E R F A C E   R O U T I N E S
///
//////
int XllpDfcInit(unsigned int width, P_FlashProperties_T pFlashDevices);
DEVICE_TYPE XllpCheckDeviceType( P_DFC_BUFFER buffer, P_FlashProperties_T pFlashDevices );
void XllpLoadProperties(P_DFC_BUFFER buffer, P_FlashProperties_T pFlashDevices);
int  XllpDfcRead       (P_DFC_BUFFER buffer, unsigned int address, unsigned int bytes, unsigned int* SpareArea, P_FlashProperties_T pFlashDevices);
int  XllpDfcWrite      (P_DFC_BUFFER buffer, unsigned int address, unsigned int bytes, P_FlashProperties_T pFlashDevices);
int  XllpDfcErase      (unsigned int address, P_FlashProperties_T pFlashDevices);
int  XllpDfcReadID     (P_DFC_BUFFER buffer);


#endif
