/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
** INTEL CONFIDENTIAL
** Copyright 2003-2004 Intel Corporation All Rights Reserved.
**
** The source code contained or described herein and all documents
** related to the source code (Material) are owned by Intel Corporation
** or its suppliers or licensors.  Title to the Material remains with
** Intel Corporation or its suppliers and licensors. The Material contains
** trade secrets and proprietary and confidential information of Intel
** or its suppliers and licensors. The Material is protected by worldwide
** copyright and trade secret laws and treaty provisions. No part of the
** Material may be used, copied, reproduced, modified, published, uploaded,
** posted, transmitted, distributed, or disclosed in any way without Intel's
** prior express written permission.
**
** No license under any patent, copyright, trade secret or other intellectual
** property right is granted to or conferred upon you by disclosure or
** delivery of the Materials, either expressly, by implication, inducement,
** estoppel or otherwise. Any license under such intellectual property rights
** must be express and approved by Intel in writing.
**
**  FILENAME: xllp_dfc_support.c
**
**  PURPOSE:  XLLP DFC support routines.
**
******************************************************************************/

#include "xllp_dfc.h"
#include "platformconfig.h"
#include "flash.h"


/** Initializing stuff **/
void        xdfc_setTiming    (FLASH_TIMING* flash);
int         xdfc_getStatusPadding(P_FlashProperties_T pFlashDevices);

/** Workhorses **/
int  xdfc_write      (P_DFC_BUFFER buffer, unsigned int address, unsigned int bytes, P_FlashProperties_T pFlashDevices);
int  xdfc_read       (P_DFC_BUFFER buffer, unsigned int address, unsigned int bytes, unsigned int* SpareArea, P_FlashProperties_T pFlashDevices);
int  xdfc_readID     (P_DFC_BUFFER buffer, DEVICE_TYPE dev_type);
int  xdfc_readstatus (P_DFC_BUFFER buffer, P_FlashProperties_T pFlashDevices);
int  xdfc_erase      (unsigned int address, P_FlashProperties_T pFlashDevices);
int  xdfc_reset      (void);


/** Miscellaneous **/
void getAddr(unsigned int address, NDCB1_REG * addr1234, NDCB2_REG * addr5, P_FlashProperties_T pFlashDevices);
void getEraseAddr(unsigned int address, NDCB1_REG * addr1234, NDCB2_REG * addr5, P_FlashProperties_T pFlashDevices);
int   waitfor  (unsigned int srmask);
#if (PV_CHIP)
int _WaitForDFCOperationComplete( unsigned int statusMask, UINT32 TimeOutMillisec);
#endif

#define MIN(x, y)           \
   ({ typeof (x) x_ = (x);  \
      typeof (y) y_ = (y);  \
      (x_ < y_) ? x_ : y_;  \
    })


////////
/////////
//////       S U P P O R T   R O U T I N E S
////////
///////
int
xdfc_write(P_DFC_BUFFER buffer, unsigned int address, unsigned int bytes, P_FlashProperties_T pFlashDevices)
{
   CMD_BLOCK	*default_cmds;
   NDCB1_REG	addr1234;
   NDCB2_REG	addr5;
   unsigned int  i;
   unsigned int  pages, page_mask, page_first, page_last, page_size, page_extra;
   unsigned int  byte_count = 0, longwords;
   unsigned int  pre_bytes, post_bytes;
   unsigned int  status;
   unsigned int  page1[2048/4], pageN[2048/4];
   unsigned int  buf[2];
#if (PV_CHIP)
   UINT_T Retval;
#endif

   /*
   *   Preliminary stuff.
    */
   page_size  = pFlashDevices->NAND_Prop.PageSize;
   page_mask  = ~(page_size - 1);
   page_first =  address              & page_mask;
   page_last  = (address + bytes - 1) & page_mask;
   page_extra = ((address + bytes) > page_last) ? 1 : 0;

   pages = (bytes / page_size) + page_extra;


   getAddr(address, &addr1234, &addr5, pFlashDevices);


   //////////////////////////////////////////////////////////////////////////////////
   ///////////                                                           ////////////
   ///////////            P R E S E R V A T I O N   S T E P S            ////////////
   ///////////                                                           ////////////
   //////////////////////////////////////////////////////////////////////////////////
   /*
   *  Calculate the pre and post padding.
    */
   pre_bytes  = address - page_first;
   post_bytes = (page_last + page_size) - (address + bytes);

   if (pre_bytes)  xdfc_read(&page1[0], page_first, page_size, NULL, pFlashDevices);
   if (post_bytes) xdfc_read(&pageN[0], page_last,  page_size, NULL, pFlashDevices);


   //////////////////////////////////////////////////////////////////////////////////
   ///////////                                                           ////////////
   ///////////               S T A R T   W R I T I N G                   ////////////
   ///////////                                                           ////////////
   //////////////////////////////////////////////////////////////////////////////////

   /*
   *  SET UP THE COMMAND
    */

	default_cmds = pFlashDevices->NAND_Prop.device_cmds;
	getAddr(page_first, &addr1234, &addr5, pFlashDevices);
	// If read status command is not 0x70, do not use auto read status method.
 	if( default_cmds->read_status.bits.CMD1 == 0x70)
   		default_cmds->pg_program.bits.AUTO_RS = 1;

	#if DFC_WORKAROUND	// Turn OFF Clocks
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
	#endif

   // Clear write status bit and set DFC to expect a command.
#if (PV_CHIP)
   *DFC_STATUS  = *DFC_STATUS;
#else
   *DFC_STATUS  |= DFC_SR_WRCMDREQ;
#endif
   *DFC_CONTROL |= DFC_CTRL_ND_RUN;

	#if DFC_WORKAROUND	// Clock ON
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
	#endif

   //
   // Wait till DFC is ready for the next command to be issued.
#if (PV_CHIP)
   	Retval = _WaitForDFCOperationComplete(DFC_SR_WRCMDREQ, 1);
	CheckErrorReturn(Retval);
#else
   if (waitfor(DFC_SR_WRCMDREQ) < 0) return -1;
#endif

	#if DFC_WORKAROUND	// Turn OFF Clocks
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
	#endif

   *DFC_COMMAND0 = default_cmds->pg_program.value;
   *DFC_COMMAND0 = addr1234.value;
   *DFC_COMMAND0 = addr5.value;

	#if DFC_WORKAROUND	// Clock ON
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
	#endif

   //
   // Wait till the DFC is ready for us to write.
#if (PV_CHIP)
   	Retval = _WaitForDFCOperationComplete(DFC_SR_WRDREQ, 1);
	if (Retval != NoError)
		return -2;
#else
   if (waitfor(DFC_SR_WRDREQ) < 0) return -2;
#endif


   /*
   *  Write the pre padding. (No need to check for a page boundary.)
    */
   byte_count = 0;
   longwords  = 0;
   while (pre_bytes)
   {
      *DFC_DATA = page1[longwords++];
      byte_count += 4;
      pre_bytes  -= 4;
   }

   /*
   *  Write the buffer contents. (Check for page boundaries.)
    */
   longwords = 0;
   while (bytes)
   {
      *DFC_DATA = buffer[longwords++];
      byte_count += 4;
      bytes      -= 4;

      /*
      *  Check for page boundary. Reissue the write command all over again.
       */
      if (bytes && ((byte_count & ~page_mask) == 0))
      {
         /*
         *  Write the status area padding.
          */
         for (i = xdfc_getStatusPadding(pFlashDevices); i > 0; i--)
            *DFC_DATA = 0x00000000;

         /*
         *  Wait for write to be done.
          */
         if (default_cmds->pg_program.bits.CSEL == 0)
#if (PV_CHIP)
		   	Retval = _WaitForDFCOperationComplete(DFC_SR_CS0_CMDD, 10);
			if (Retval != NoError)
				return -1;
#else
		    if (waitfor(DFC_SR_CS0_CMDD) < 0) return -1;
#endif

         if (default_cmds->pg_program.bits.CSEL == 1)
#if (PV_CHIP)
		   	Retval = _WaitForDFCOperationComplete(DFC_SR_CS1_CMDD, 10);
			if (Retval != NoError)
				return -1;
#else
		    if (waitfor(DFC_SR_CS1_CMDD) < 0) return -1;
#endif

         // Check for double bit error.
         if (*DFC_STATUS & DFC_SR_DBERR)
         	return -6; // DFCDoubleBitError;

        // If using auto status read, read the status returned.
    	if (default_cmds->pg_program.bits.AUTO_RS)
    	{
      		// NOTE: Use the data returned by the NAND in some way...
      		status = *DFC_DATA;
      		status = *DFC_DATA;
      	}
      	else
      	{
			xdfc_readstatus( &buf[0], pFlashDevices);
	  	}

   		if ((default_cmds->pg_program.bits.CSEL == 0) && (*DFC_STATUS & DFC_SR_CS0_BBD))
         	return -4;
   		if ((default_cmds->pg_program.bits.CSEL == 1) && (*DFC_STATUS & DFC_SR_CS1_BBD))
         	return -5;


         // Increment page address.
         page_first += page_size;
         getAddr(page_first, &addr1234, &addr5, pFlashDevices);

         /*
         *  SET UP THE COMMAND
          */
		 #if DFC_WORKAROUND	// Turn OFF Clocks
		 *ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
		 *ADDR_CCU_D0CKEN_A;
		 #endif

         // Clear write status bit and set DFC to expect a command.
#if (PV_CHIP)
         *DFC_STATUS  = DFC_SR_WRCMDREQ;
#else
         *DFC_STATUS  |= DFC_SR_WRCMDREQ;
#endif
         *DFC_CONTROL |= DFC_CTRL_ND_RUN;

		 #if DFC_WORKAROUND	// Clock ON
		 *ADDR_CCU_D0CKEN_A |= 0x10;
		 *ADDR_CCU_D0CKEN_A;
		 #endif

         //
         // Wait till DFC is ready for the next command to be issued.
#if (PV_CHIP)
	   	Retval = _WaitForDFCOperationComplete(DFC_SR_WRCMDREQ, 1);
		if (Retval != NoError)
		     	return -1;
#else
         if (waitfor(DFC_SR_WRCMDREQ) < 0) return -1;
#endif

		 #if DFC_WORKAROUND	// Turn OFF Clocks
		 *ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
		 *ADDR_CCU_D0CKEN_A;
		 #endif

	   	 *DFC_COMMAND0 = default_cmds->pg_program.value;
   		 *DFC_COMMAND0 = addr1234.value;
   		 *DFC_COMMAND0 = addr5.value;

		 #if DFC_WORKAROUND	// Clock ON
		 *ADDR_CCU_D0CKEN_A |= 0x10;
		 *ADDR_CCU_D0CKEN_A;
		 #endif

         //
         // Wait till the DFC is ready for us to write.
#if (PV_CHIP)
	   	Retval = _WaitForDFCOperationComplete(DFC_SR_WRDREQ, 1);
			if (Retval != NoError)
			 	return -2;
#else
         if (waitfor(DFC_SR_WRDREQ) < 0) return -2;
#endif
         }
      }

   /*
   *  Write the post padding. (No need to check for a page boundary.)
    */
   longwords = ((page_last + page_size) - (address + bytes)) / 4;
   while (post_bytes)
   {
      *DFC_DATA = pageN[longwords++];
      byte_count += 4;
      post_bytes -= 4;
   }

   /*
   *  Write the status area padding.
    */
   for (i = xdfc_getStatusPadding(pFlashDevices); i > 0; i--)
      *DFC_DATA = 0x00000000;

   /*
   *  Wait for write to be done.
    */
   if (default_cmds->pg_program.bits.CSEL == 0)
#if (PV_CHIP)
   	Retval = _WaitForDFCOperationComplete(DFC_SR_CS0_CMDD, 10);
		if (Retval != NoError)
			return -1;
#else
      if (waitfor(DFC_SR_CS0_CMDD) < 0) return -1;
#endif

   if (default_cmds->pg_program.bits.CSEL == 1)
#if (PV_CHIP)
   	Retval = _WaitForDFCOperationComplete(DFC_SR_CS1_CMDD, 10);
  		if (Retval != NoError)
  			return -1;
#else
      if (waitfor(DFC_SR_CS1_CMDD) < 0) return -1;
#endif


   /*
   *  If we used auto status read, the bad block status is in the status register.
    */
   if (default_cmds->pg_program.bits.AUTO_RS)
   {
      // NOTE: Use the data returned by the NAND in some way...
      status = *DFC_DATA;
      status = *DFC_DATA;
   }
   else
   {
		xdfc_readstatus( &buf[0], pFlashDevices);
   }

   if ((default_cmds->pg_program.bits.CSEL == 0) && (*DFC_STATUS & DFC_SR_CS0_BBD))
         return -4;
   if ((default_cmds->pg_program.bits.CSEL == 1) && (*DFC_STATUS & DFC_SR_CS1_BBD))
         return -5;

   return 0;
}

// SpareArea for the last page is returned only. However it could be 1st page and or last page of an erased block for SLC.
// MLC - Bad Block table no longer maintained in spare area. Must be created on first scan of device and saved.
int xdfc_read(P_DFC_BUFFER buffer, unsigned int address, unsigned int bytes, unsigned int* SpareArea, P_FlashProperties_T pFlashDevices)
{
   CMD_BLOCK	*default_cmds;
   NDCB1_REG	addr1234;
   NDCB2_REG	addr5;
   unsigned int  page_first, page_last, page_mask;
   unsigned int  byte_count = 0, longwords;
   unsigned int  page_size, pre_bytes, post_bytes;
   unsigned int  i, status;
#if (PV_CHIP)
   UINT_T Retval;
#endif


   /*
   *  Some preliminary information.
    */
   page_size  = pFlashDevices->NAND_Prop.PageSize;
   page_mask  = ~(page_size - 1);
   page_first =  address              & page_mask;
   page_last  = (address + bytes - 1) & page_mask;

   pre_bytes  = address - page_first;
   post_bytes = (page_last + page_size) - (address + bytes);

   /*
   *  SET UP THE COMMAND
    */
	default_cmds = pFlashDevices->NAND_Prop.device_cmds;
	getAddr(page_first, &addr1234, &addr5, pFlashDevices);

	#if DFC_WORKAROUND	// Clock OFF
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
	#endif

   // Clear read status bit and set DFC to expect a command.
#if (PV_CHIP)
   *DFC_STATUS  = *DFC_STATUS;
#else
   *DFC_STATUS  |= DFC_SR_RDDREQ;
#endif
   *DFC_CONTROL |= DFC_CTRL_ND_RUN;

	#if DFC_WORKAROUND		// Clock ON
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
	#endif

   //
   // Wait till DFC is ready for next command to be issued.
#if (PV_CHIP)
   Retval = _WaitForDFCOperationComplete(DFC_SR_WRCMDREQ, 1);
   if (Retval != NoError)
   		return -7;
#else
   if (waitfor(DFC_SR_WRCMDREQ) < 0) return -7;
#endif

	#if DFC_WORKAROUND	// Clock OFF
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
	#endif

   *DFC_COMMAND0 = default_cmds->read.value;
   *DFC_COMMAND0 = addr1234.value;
   *DFC_COMMAND0 = addr5.value;

	#if DFC_WORKAROUND		// Clock ON
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
	#endif

   //
   // Wait for page to be ready to read.
#if (PV_CHIP)
   Retval = _WaitForDFCOperationComplete(DFC_SR_RDDREQ, 1);
   if (Retval != NoError)
   		return -8;
#else
   if (waitfor(DFC_SR_RDDREQ) < 0) return -8;
#endif

   /*
   *  Read and discard the pre padding. (No need to check for a page boundary.)
    */
   byte_count = 0;
   while (pre_bytes)
   {
      i = *DFC_DATA;
      byte_count += 4;
      pre_bytes  -= 4;
   }

   /*
   *  Read into the buffer. (Check for page boundaries.)
    */
   longwords = 0;
   while (bytes)
   {
      buffer[longwords++] = *DFC_DATA;
      byte_count += 4;
      bytes      -= 4;

      /*
      *  Check for page boundary. Reissue the read command all over again.
       */
      if (bytes && ((byte_count & ~page_mask) == 0))
      {
         /*
         *  Read the status area padding.
          */
         for (i = xdfc_getStatusPadding(pFlashDevices); i > 0; i--)
            status = *DFC_DATA;

         // Check for double bit error.
         if (*DFC_STATUS & DFC_SR_DBERR)
         	return -10;

         // Increment page address.
         page_first += page_size;
	     getAddr(page_first, &addr1234, &addr5, pFlashDevices);

		#if DFC_WORKAROUND	// Clock OFF
		*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
		*ADDR_CCU_D0CKEN_A;
		#endif

         /*
         *   SET UP THE COMMAND
          */
         // Clear read status bit and set DFC to expect a command.
#if (PV_CHIP)
         *DFC_STATUS = *DFC_STATUS;
#else
         *DFC_STATUS  |= DFC_SR_RDDREQ;
#endif
         *DFC_CONTROL |= DFC_CTRL_ND_RUN;

		#if DFC_WORKAROUND		// Clock ON
		*ADDR_CCU_D0CKEN_A |= 0x10;
		*ADDR_CCU_D0CKEN_A;
		#endif

         //
         // Wait till DFC is ready for next command to be issued.
#if (PV_CHIP)
	   	Retval = _WaitForDFCOperationComplete(DFC_SR_WRCMDREQ, 1);
	    if (Retval != NoError)
         	return -7;
#else
         if (waitfor(DFC_SR_WRCMDREQ) < 0) return -7;
#endif

		#if DFC_WORKAROUND	// Clock OFF
		*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
		*ADDR_CCU_D0CKEN_A;
		#endif

         *DFC_COMMAND0 = default_cmds->read.value;
         *DFC_COMMAND0 = addr1234.value;
         *DFC_COMMAND0 = addr5.value;

		#if DFC_WORKAROUND		// Clock ON
		*ADDR_CCU_D0CKEN_A |= 0x10;
		*ADDR_CCU_D0CKEN_A;
		#endif

         //
         // Wait for page to be ready to read.
#if (PV_CHIP)
	   	 Retval = _WaitForDFCOperationComplete(DFC_SR_RDDREQ, 1);
	    	if (Retval != NoError)
	         	return -8;
#else
         if (waitfor(DFC_SR_RDDREQ) < 0) return -8;
#endif
	  }
   }

   /*
   *  Read and discard the post padding.
    */
   while (post_bytes)
   {
      i = *DFC_DATA;
      post_bytes -= 4;
   }

   // Read the spare area.
   if (SpareArea != NULL)
   {
    for (i = 0; i < xdfc_getStatusPadding(pFlashDevices); i++)
    SpareArea[i] = *DFC_DATA;
   }
   else
   {
    for (i = xdfc_getStatusPadding(pFlashDevices); i > 0; i--)
    status = *DFC_DATA;
   }

   return 0;
}


int
xdfc_readID(P_DFC_BUFFER buffer, DEVICE_TYPE dev_type)
{
	NDCB0_REG cmd0;
	NDCB1_REG cmd1;
	NDCB2_REG cmd2;
	NDCR_REG control;
#if (PV_CHIP)
	UINT_T Retval;
#endif

	/*
	*  DFC Workaround code turn OFF Arbiter and NAND Clock
	*/
	#if DFC_WORKAROUND	// Clock OFF
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
	#endif

   /*
    *  Clear any previous Read Status
    *
    */
#if (PV_CHIP)
   *DFC_STATUS = *DFC_STATUS;
#else
   *DFC_STATUS  |= 0xFFF;
#endif
    /*
	 *	Kick off the command
	 *   	Note that you should be able to fill the command buffer
	 *           and then start the command but this does not seem
	 *           to work with the DFC controller as documented in
	 *           the IAS.  So we kick off the command and wait for
	 *           the DFC to request the next command like in DMA mode.
	 */

	control.value = *DFC_CONTROL;
	if(dev_type == LARGE)
		control.bits.RD_ID_CNT = 4;
	else
		control.bits.RD_ID_CNT = 2;

    control.bits.ND_RUN = 1;
	*DFC_CONTROL = control.value;
	/*
	*  DFC Workaround code turn ON Arbiter and NAND Clock
	*/
	#if DFC_WORKAROUND	// Clock ON
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
	#endif

	cmd0.value = 0; //take care of reserved bits, etc.
	cmd0.bits.CMD1 = 0x90;
	cmd0.bits.ADDR_CYC = 1;
	cmd0.bits.CMD_TYPE = DFC_CMDTYPE_READID;

	cmd1.value = 0x0;
	cmd2.value = 0x0;


   /*
    *  Wait for write command request bit.
    */
#if (PV_CHIP)
   	Retval = _WaitForDFCOperationComplete(DFC_SR_WRCMDREQ, 1);
	CheckErrorReturn(Retval);
#else
   if (waitfor(DFC_SR_WRCMDREQ) < 0) return -1;
#endif

	#if DFC_WORKAROUND	// Clock OFF
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
	#endif

   /*
    *  Write commands to command buffer 0.
    *
    */
   *DFC_COMMAND0 = cmd0.value;
   *DFC_COMMAND0 = cmd1.value;
   *DFC_COMMAND0 = cmd2.value;

	#if DFC_WORKAROUND	// Clock ON
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
	#endif

   /*
    *  Wait for data in the buffer and check command done in the control register.
    */
#if (PV_CHIP)
   	Retval = _WaitForDFCOperationComplete(DFC_SR_RDDREQ, 1);
	CheckErrorReturn(Retval);
#else
   if (waitfor(DFC_SR_RDDREQ) < 0) return -1;
#endif

   /*
    *  Read the read ID bytes.
    */
	buffer[0] = *DFC_DATA;
    buffer[1] = *DFC_DATA;

	#if DFC_WORKAROUND	// Clock OFF
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
	#endif

	/*
	 *  Clear the Status and make sure the command completed.
	 */
#if (PV_CHIP)
   *DFC_STATUS  = *DFC_STATUS;
#else
   *DFC_STATUS  |= DFC_SR_RDDREQ;
#endif
	control.value = *DFC_CONTROL;
	if (control.bits.ND_RUN == 1)
		*DFC_CONTROL &= ~DFC_CTRL_ND_RUN;

	#if DFC_WORKAROUND	// Clock ON
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
	#endif

   return 0;
}


int xdfc_reset(void)
{

   	NDCB0_REG cmd0;
	NDCB1_REG cmd1;
	NDCB2_REG cmd2;
#if (PV_CHIP)
	NDCR_REG control;
	UINT_T Retval;
#endif

   /*
   *  Setting ND_RUN bit will get the transfer going.
    *
   */
	#if DFC_WORKAROUND	// Clock OFF
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
	#endif

#if (PV_CHIP)
   	*DFC_STATUS  = *DFC_STATUS;
#else
   *DFC_STATUS  |= DFC_SR_RDDREQ;
#endif
   *DFC_CONTROL |= DFC_CTRL_ND_RUN;

	#if DFC_WORKAROUND	// Clock ON
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
	#endif

   /*
   *  Setup Commands.
    *
   */
	cmd0.value = 0; //take care of reserved bits, etc.
	cmd0.bits.CMD1 = 0xFF;
	cmd0.bits.CMD_TYPE = DFC_CMDTYPE_RESET;

   /*
    *  Write commands to command buffer 0.
    *
    */
	#if DFC_WORKAROUND	// Clock OFF
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
	#endif

   *DFC_COMMAND0 = cmd0.value;
   	*DFC_COMMAND0 = 0;
   	*DFC_COMMAND0 = 0;

	#if DFC_WORKAROUND	// Clock ON
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
	#endif

   /*
   *  Wait for ready.
   */
#if (PV_CHIP)
   	Retval = _WaitForDFCOperationComplete(DFC_SR_CS0_CMDD, 10);
	CheckErrorReturn(Retval);

   	/*
   	*  Make sure RDY is set at this point
   	*/
	if ((*DFC_STATUS & DFC_SR_RDY0) != DFC_SR_RDY0)
		return -1;
#else
   if (waitfor(DFC_SR_RDY) < 0) return -1;
#endif

   /*
   *  Read the status register.
   */
	#if DFC_WORKAROUND	// Clock OFF
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
	#endif

#if (PV_CHIP)
    /* Clear RDY */
   	*DFC_STATUS  = DFC_SR_RDY0 | DFC_SR_RDY1 | DFC_SR_CS0_CMDD;
	control.value = *DFC_CONTROL;

	// Clear ND_RUN bit
	if (control.bits.ND_RUN == 1)
		*DFC_CONTROL &= ~DFC_CTRL_ND_RUN;
#else
   *DFC_STATUS  |= DFC_SR_CS0_CMDD;
#endif

	#if DFC_WORKAROUND	// Clock ON
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
	#endif

   return 0;
}

int xdfc_readstatus(P_DFC_BUFFER buffer, P_FlashProperties_T pFlashDevices)
{
   	CMD_BLOCK	*default_cmds;
	NDCB1_REG cmd1;
	NDCB2_REG cmd2;
#if (PV_CHIP)
	UINT_T Retval;
#endif

   /*
   *  Setting ND_RUN bit will get the transfer going.
    *
   */
	#if DFC_WORKAROUND	// Clock OFF
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
	#endif

#if (PV_CHIP)
   *DFC_STATUS = *DFC_STATUS;
#else
   *DFC_STATUS  |= DFC_SR_RDDREQ;
#endif
   *DFC_CONTROL |= DFC_CTRL_ND_RUN;

	#if DFC_WORKAROUND	// Clock ON
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
	#endif

   /*
   *  Setup Commands.
    *
   */
	default_cmds = pFlashDevices->NAND_Prop.device_cmds;
	cmd1.value = 0;
	cmd2.value = 0;
   /*
   *  Wait for write command request bit.
    */
#if (PV_CHIP)
   	Retval = _WaitForDFCOperationComplete(DFC_SR_WRCMDREQ, 1);
	CheckErrorReturn(Retval);
#else
   if (waitfor(DFC_SR_WRCMDREQ) < 0) return -1;
#endif

   /*
   *  Write commands to command buffer 0.
    *
   */
	#if DFC_WORKAROUND	// Clock OFF
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
	#endif

   *DFC_COMMAND0 = default_cmds->read_status.value;
   *DFC_COMMAND0 = cmd1.value;
   *DFC_COMMAND0 = cmd2.value;

	#if DFC_WORKAROUND	// Clock ON
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
	#endif

   /*
   *  Wait for ready.
   */
#if (PV_CHIP)
   	Retval = _WaitForDFCOperationComplete(DFC_SR_RDDREQ, 1);
	CheckErrorReturn(Retval);
#else
   if (waitfor(DFC_SR_RDDREQ) < 0) return -1;
#endif

   /*
   *  Read the status register.
   */
	#if DFC_WORKAROUND	// Clock OFF
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
	#endif

#if (PV_CHIP)
   *DFC_STATUS = *DFC_STATUS;
#else
   *DFC_STATUS  |= DFC_SR_RDDREQ;
#endif

	#if DFC_WORKAROUND	// Clock ON
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
	#endif

   buffer[0] = *DFC_DATA;
   buffer[1] = *DFC_DATA;

   return 0;
}


/*
*  NOTE: Introduce a generic erase. One that lets you erase any random slice of memory.
 *       Do this by reading from the segments of the block(s) that are to be preserved
*        before issuing the erase command. (Of couse, with about 32 pages per block,
 *       this would be an expensive operation...)
*/

//
// NOTE: To delete a page, is the address the start of the page, or can it be
//       any address within the page? [02aug2004agapito]
//

int xdfc_erase(unsigned int address, P_FlashProperties_T pFlashDevices)
{
   CMD_BLOCK	*default_cmds;
   NDCB1_REG	addr1234;
   NDCB2_REG	addr5;
   unsigned int status;
   unsigned int buf[2];

	#if DFC_WORKAROUND	// Clock OFF
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
	#endif

   /*
   *  Setting ND_RUN bit will get the transfer going.
    *
   */
   *DFC_STATUS  |= 0xFFF;  //clear only *non* reserved bits
   *DFC_CONTROL |= DFC_CTRL_ND_RUN;

	#if DFC_WORKAROUND	// Clock ON
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
	#endif


   /*
   *  Generate command
    *
   */
 	default_cmds = pFlashDevices->NAND_Prop.device_cmds;
	getEraseAddr(address, &addr1234, &addr5, pFlashDevices);

	// If read status command is not 0x70, do not use auto read status method.
 	if( default_cmds->read_status.bits.CMD1 == 0x70)
   		default_cmds->pg_program.bits.AUTO_RS = 1;

   /*
   *  Wait for write command request bit.
    */
   if (waitfor(DFC_SR_WRCMDREQ) < 0)
   		return -1;

   /*
   *  Write commands to command buffer 0.
    *
   */
	#if DFC_WORKAROUND	// Clock OFF
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
	#endif

   *DFC_COMMAND0 = default_cmds->blk_erase.value;

   // Write out the address cycles
   *DFC_COMMAND0 = addr1234.value;
   *DFC_COMMAND0 = addr5.value;

	#if DFC_WORKAROUND	// Clock ON
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
	#endif


   /*
   *  Erasing the block... Wait for ready to check status.
    *
   */
   // Check Device ready.
#if (PV_CHIP)
   if (waitfor(DFC_SR_RDY0) < 0)
#else
   if (waitfor(DFC_SR_RDY) < 0)
#endif
   		return -1;
   if (default_cmds->pg_program.bits.CSEL == 0)
   		if (waitfor(DFC_SR_CS0_CMDD) < 0)
   			return -1;
   if (default_cmds->pg_program.bits.CSEL == 1)
   		if (waitfor(DFC_SR_CS1_CMDD) < 0)
   			return -1;

  /*
   *  If we used auto status read get the status else issue read status
   *    Then check the bad block status is in the status register.
    *
   */
    if (default_cmds->pg_program.bits.AUTO_RS)
    {
      status = *DFC_DATA;
      status = *DFC_DATA;
 	}
 	else
 	{
		xdfc_readstatus( &buf[0], pFlashDevices);
	}
    if ((default_cmds->pg_program.bits.CSEL == 0) && (*DFC_STATUS & DFC_SR_CS0_BBD))
         return -1;
    if ((default_cmds->pg_program.bits.CSEL == 1) && (*DFC_STATUS & DFC_SR_CS1_BBD))
         return -1;


   /*
   *  Check for double bit error.
    *
   */
   if (*DFC_STATUS & DFC_SR_DBERR)
      return DFCDoubleBitError;

   return 0;
}


void
xdfc_setTiming(FLASH_TIMING *flash)
{
   NDTR0CS0_REG	timing0;
   NDTR1CS0_REG	timing1;

   //
   // NOTE: Confirm these calculations make sense! [25jun2004agapito]
   // METHOD:
   //    - integer division will truncate the result, so add a 1 in all cases
   //    - subtract the extra 1 cycle added to all register timing values
   //
   timing0.bits.tCH = MIN(((int) (flash->tCH * NAND_CONTROLLER_CLOCK / 1000) + 1), TIMING_MAX_tCH);
   timing0.bits.tCS = MIN(((int) (flash->tCS * NAND_CONTROLLER_CLOCK / 1000) + 1), TIMING_MAX_tCS);
   timing0.bits.tWH = MIN(((int) (flash->tWH * NAND_CONTROLLER_CLOCK / 1000) + 1), TIMING_MAX_tWH);
   timing0.bits.tWP = MIN(((int) (flash->tWP * NAND_CONTROLLER_CLOCK / 1000) + 1), TIMING_MAX_tWP);
   timing0.bits.tRH = MIN(((int) (flash->tRH * NAND_CONTROLLER_CLOCK / 1000) + 1), TIMING_MAX_tRH);
   timing0.bits.tRP = MIN(((int) (flash->tRP * NAND_CONTROLLER_CLOCK / 1000) + 1), TIMING_MAX_tRP);

   timing1.bits.tR   = MIN(((int) (flash->tR   * NAND_CONTROLLER_CLOCK / 1000) + 1), TIMING_MAX_tR);
   timing1.bits.tWHR = MIN(((int) (flash->tWHR * NAND_CONTROLLER_CLOCK / 1000) + 1), TIMING_MAX_tWHR);
   timing1.bits.tAR  = MIN(((int) (flash->tAR  * NAND_CONTROLLER_CLOCK / 1000) + 1), TIMING_MAX_tAR);

   //
   // Write timing registers to corresponding memory mapped registers.
   //
	#if DFC_WORKAROUND	// Clock OFF
	*ADDR_CCU_D0CKEN_A &= ADDR_DOCKEN_A_NAND_MASK;
	*ADDR_CCU_D0CKEN_A;
	#endif

   *DFC_TIMING_0 = timing0.value;
   *DFC_TIMING_1 = timing1.value;

	#if DFC_WORKAROUND	// Clock ON
	*ADDR_CCU_D0CKEN_A |= 0x10;
	*ADDR_CCU_D0CKEN_A;
	#endif

   return;
}

int
waitfor(unsigned int srmask)
{
	int i = 0;
	while ((srmask & *DFC_STATUS) == 0)
		if (i++ > DFC_TIMEOUT) return -1;

	return 0;
}

void getAddr(unsigned int address, NDCB1_REG * addr1234, NDCB2_REG * addr5, P_FlashProperties_T pFlashDevices)
{
   addr1234->value = 0;
   addr5->value    = 0;

   switch (pFlashDevices->NAND_Prop.NAND_type)
   {
      case SMALL:
         addr1234->bits.ADDR1  = address & 0x00000FF;	// A0-A7
         addr1234->bits.ADDR2  = (address & 0x001FE00) >> 9; 	// A9-A16
         addr1234->bits.ADDR3  = (address & 0x1FE0000) >> 17;	// A17-A24
         addr1234->bits.ADDR4  = (address & 0x6000000) >> 25;	// A25-A26
         addr5->bits.ADDR5     = 0;
         break;

      case LARGE:
      default:
	  	// NOTE: The DFC should handle incrementing the column address
		// 		 we just need to worry about the row address to get
		//		 to the correct page.  Row address starts in addr cycle 3.
         addr1234->bits.ADDR1  = address & 0x000000FF;		// A0-A7
         addr1234->bits.ADDR2  = (address & 0x00000700) >> 8; 	// A8-A10
		 addr1234->bits.ADDR3  = (address & 0x0007F800) >> 11;	// A11-A18
         addr1234->bits.ADDR4  = (address & 0x07F80000) >> 19;	// A19-A26
         addr5->bits.ADDR5     = (address & 0x38000000) >> 27;	// A27-A29

         break;
      }
   return;
}

void getEraseAddr(unsigned int address, NDCB1_REG * addr1234, NDCB2_REG * addr5, P_FlashProperties_T pFlashDevices)
{
   addr1234->value = 0;
   addr5->value    = 0;

   switch (pFlashDevices->NAND_Prop.NAND_type)
   {
   case SMALL:
	   addr1234->bits.ADDR1  = (address & 0x001FE00) >> 9; 	// A9-A16
	   addr1234->bits.ADDR2  = (address & 0x1FE0000) >> 17;	// A17-A24
	   addr1234->bits.ADDR3  = (address & 0x2000000) >> 25;	// A25-A26
	   addr1234->bits.ADDR4  = 0;
	   addr5->bits.ADDR5     = 0;
	   break;

   default:
	   addr1234->bits.ADDR1  = (address & 0x0007F800) >> 11;	// A11-A18
	   addr1234->bits.ADDR2  = (address & 0x07F80000) >> 19;	// A19-A26
	   addr1234->bits.ADDR3  = (address & 0x08000000) >> 27;	// A27
	   addr1234->bits.ADDR4  = 0;
	   addr5->bits.ADDR5     = 0;
	   break;
   }
   return;
}

int
xdfc_getStatusPadding(P_FlashProperties_T pFlashDevices)
{
   if (pFlashDevices->NAND_Prop.PageSize > 512) return 10;
   else                               return  2;
}

#if (PV_CHIP)
//----------------------------------------------------------------------------
// _WaitForProtectionOperationComplete
//        Replaces method susceptible to "infinite loop"
//
//
//----------------------------------------------------------------------------
int _WaitForDFCOperationComplete( unsigned int statusMask, UINT32 TimeOutMillisec )
{
  UINT32 startTime, endTime;

  startTime = GetOSCR0();
  do
  {
    if( (statusMask & *DFC_STATUS) == 0 )
     break;
    endTime = GetOSCR0();
    if (endTime < startTime)
    {
//      endTime += (0x0 - startTime);
        endTime += (0xFFFFFFFF - startTime);
        startTime = 0;
    }
  }
  while( OSCR0IntervalInMicro(startTime, endTime) < 50 );

  startTime = GetOSCR0();
  do
  {
    if(statusMask & *DFC_STATUS)
     return 0;
    endTime = GetOSCR0();
    if (endTime < startTime)
    {
//      endTime += (0x0 - startTime);
        endTime += (0xFFFFFFFF - startTime);
        startTime = 0;
    }
  }
  while( OSCR0IntervalInMilli(startTime, endTime) < TimeOutMillisec );

  return -1;
}
#endif

