/******************************************************************************
**
**  COPYRIGHT (C) 2006 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	PlatformConfig.c
**
**  PURPOSE: 	Contain platform specific configuration routines
**
******************************************************************************/

#include "xllp_defs.h"
#include "PlatformConfig.h"
#include "general.h"
#include "misc.h"



typedef struct MONS_CHIP_SELECT_SETUP_S
{
  const P_XLLP_VUINT32_T   registerAddr;
  const XLLP_UINT32_T      regValue;
  XLLP_UINT32_T            defaultValue;
}CHIP_SELECT_SETUP_S, *P_CHIP_SELECT_SETUP_S;

#define CS0_ONLY_OFFSET 0
#define CS0_ONLY_ITEMS  5
#define CS2_ONLY_OFFSET 5
#define CS2_ONLY_ITEMS  3
#define CS3_ONLY_OFFSET 8
#define CS3_ONLY_ITEMS  3
#define CS_COMMON_OFFSET ( CS0_ONLY_ITEMS + CS2_ONLY_ITEMS + CS3_ONLY_ITEMS )

/*=============================================================================================*/
/* This table defines the register addresses and data values for chip select configuration

   The table was ordererd according to the original code execution for the existing chip select
   routines to preserve and sequence dependency.  Each chip select config routine reads the
   appropriate parts specific to its needs, and then sequences thru the common section needed
   for both...

   The default field is for the storage of the the register contents by the SaveDefault routine

   This method saves considerable space in the binary image ... over 2k.
===============================================================================================*/
CHIP_SELECT_SETUP_S chipSelectSetup[]=
{
   /*========================================================================================*/
   /*                  register addr            Signal/pad name          Setup       default */
   /*                                                                    value       storage */
   /*========================================================================================*/

   /* CS0_ONLY section */
   (P_XLLP_VUINT32_T) (SMEMC_BASE | 0x0008),    /* SMEMC_MSC0       */   0x07FF8,    0x0,
   (P_XLLP_VUINT32_T) (SMEMC_BASE | 0x0080),    /* SMEMC_CSADRCFG_0 */   0x0320919,  0x0,
   (P_XLLP_VUINT32_T) (SMEMC_BASE | 0x000C),    /* SMEMC_CSMSADRCFG */   0x1,        0x0,
   (P_XLLP_VUINT32_T) (SMEMC_BASE | 0x00A0),    /* MDCNFG           */   0x4,        0x0,
   (P_XLLP_VUINT32_T) (0x48100090),             /* EMPI             */   0x01000000, 0x0,

   /* CS2_ONLY  section, but shared by M-SYSTEMS as well...         */
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0130),    /* GPIO3:nCS        */   0x1,        0x0,
   (P_XLLP_VUINT32_T) (0x4A000000 | 0x0088),    /* SMEMC_CSADRCFG_2 */   0x0320919,  0x0,
   (P_XLLP_VUINT32_T) (0x4A000000 | 0x000C),    /* SMEMC_MSC1       */   0x07FF8,    0x0,

   /* CS3_ONLY section  (M-SYSTEMS H3 DEVICE) */
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0224),    /* DF_NCS0          */   0x1,        0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x022C),    /* DF_NWRE          */   0x1,        0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0230),    /* DF_NRE_NOE       */   0x1,        0x0,

   /* COMMON_SECTION */
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x012C),    /* GPIO2:RDY        */   0x1,        0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0134),    /* GPIO4:CS3        */   0x1,        0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0208),    /* DF_nADV1_ALE     */   0x1801,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0138),    /* nXCVREN          */   0x1900,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0214),    /* nBE0             */   0x1800,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0218),    /* nBE1             */   0x1800,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0234),    /* nLUA             */   0x1900,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0238),    /* nLLA             */   0x1900,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x023C),    /* DF_ADDR0         */   0x1800,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0240),    /* DF_ADDR1         */   0x1800,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0244),    /* DF_ADDR2         */   0x1800,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0248),    /* DF_ADDR3         */   0x1800,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0204),    /* ND_CLE           */   0x1800,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x024C),    /* ND_IO0           */   0x1401,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0254),    /* ND_IO1           */   0x1401,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x025C),    /* ND_IO2           */   0x1401,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0264),    /* ND_IO3           */   0x1401,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x026C),    /* ND_IO4           */   0x1401,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0274),    /* ND_IO5           */   0x1401,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x027C),    /* ND_IO6           */   0x1401,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0284),    /* ND_IO7           */   0x1401,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0250),    /* ND_IO8           */   0x1401,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0258),    /* ND_IO9           */   0x1401,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0260),    /* ND_IO10          */   0x1401,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0268),    /* ND_IO11          */   0x1401,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0270),    /* ND_IO12          */   0x1401,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0278),    /* ND_IO13          */   0x1401,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0280),    /* ND_IO14          */   0x1401,     0x0,
   (P_XLLP_VUINT32_T) (0x40E10000 | 0x0288),    /* ND_IO15          */   0x1401,     0x0
};

//-----------------------------------------------------------------------
// ConfigRegSetup
//
//
//-----------------------------------------------------------------------
void ConfigRegSetup( P_CHIP_SELECT_SETUP_S regPtr, const UINT32_T size )
{
    UINT32_T i,tmp;

    for(i=0; i<size; i++ )
    {
      *(regPtr->registerAddr) |= regPtr->regValue;
      tmp = *(regPtr->registerAddr);  // ensure write complete
      regPtr++;
    }
}

#if 0
//-----------------------------------------------------------------------
// ChipSelect2
//
//
//-----------------------------------------------------------------------
void ChipSelect2(void)
{
    UINT_T  AscrReg;
    UINT_T  ArscReg;

    AscrReg = *ASCR;
    ArscReg = *ARSR;

   if((AscrReg & BIT0) || (ArscReg & BIT3))
     return;

   ConfigRegSetup( chipSelectSetup + CS2_ONLY_OFFSET, CS2_ONLY_ITEMS );
   ConfigRegSetup( (chipSelectSetup + CS_COMMON_OFFSET),
                   ((sizeof(chipSelectSetup)/sizeof(CHIP_SELECT_SETUP_S))-CS_COMMON_OFFSET) );
}

//-----------------------------------------------------------------------
// ChipSelect0
//
//
//-----------------------------------------------------------------------
void ChipSelect0(void)
{
    UINT_T  AscrReg;
    UINT_T  ArscReg;

    AscrReg = *ASCR;
    ArscReg = *ARSR;

   if((AscrReg & BIT0) || (ArscReg & BIT3))
     return;

   ConfigRegSetup( (chipSelectSetup + CS0_ONLY_OFFSET), CS0_ONLY_ITEMS );
   ConfigRegSetup( (chipSelectSetup + CS_COMMON_OFFSET),
                   ((sizeof(chipSelectSetup)/sizeof(CHIP_SELECT_SETUP_S))-CS_COMMON_OFFSET) );
}
#endif

//-----------------------------------------------------------------------
// ChipSelectMsys
//
//
//-----------------------------------------------------------------------
void ChipSelectMsys( void )
{
    UINT_T  AscrReg;
    UINT_T  ArscReg;

    {
      // Mike B. ... I assume this needs to be here, since the other CS
      // routines do this ... please confirm ... DCB
      //---------------------------------------------------------------
      AscrReg = *ASCR;
      ArscReg = *ARSR;

     if((AscrReg & BIT0) || (ArscReg & BIT3))
       return;
   }

   // M-sys also uses CS2_ONLY items...
   //-----------------------------------
   ConfigRegSetup( chipSelectSetup + CS2_ONLY_OFFSET,
                   ((sizeof(chipSelectSetup)/sizeof(CHIP_SELECT_SETUP_S))-CS2_ONLY_OFFSET) );

}
