/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2002, 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	Flash.h
**
**  PURPOSE: 	Holds all flash related definitions
**
******************************************************************************/
#ifndef __Flash_H__
#define __Flash_H__

#include "xllp_dfc.h"
#include "xllp_dmac.h"
#include "protocolmanager.h"
#include "tim.h"

#if (TRUSTED || BOOTROM)
	#include "savestate.h"
#endif

// Bad Block Table Addresses in Block 0
#define RELOC_16KB   0x3E00
#define RELOC_32KB   0x7E00
#define RELOC_64KB   0xF800
#define RELOC_128KB	 0x1F800

#define FlashDefaultAddr			0x10000000
#define AllFlashBufferSize			0x20
#define K3_BufferSize				0x20
#define K18_BufferSize				0x20
#define L18_BufferSize				0x20
#define K3_BlockSize				0x10000
#define K18_BlockSize				0x10000
#define L18_MainBlockSize			0x10000
#define L18_ParamBlockSize			0x4000

// Program Block Sizes
#define SIBLEYBLOCKSIZE_x16 		0x400
#define SIBLEYBLOCKSIZE_x32 		0x800

//Size of the phsical block
#define SIBLEY_BLK_SZ	0x40000
#define TYAX_BLK_SZ		0x8000
#define ONENAND_BLK_SZ  (128*1024) /*128kB*/
#define ONENAND_PAGE_SZ	0x800		//2KB

// Intel x32 flash commands
#define ReadArrayCmd               	0x00FF00FF
#define ReadIdentifierCmd			0x00900090
#define ReadQueryCmd               	0x00980098
#define ReadStatusRegisterCmd      	0x00700070
#define ClearStatusRegisterCmd     	0x00500050
#define WordProgramCmd             	0x00400040
#define WordProgramCmd_Sibley      	0x00410041
#define ByteProgramCmd             	0x00100010
#define WriteToBufferCmd           	0x00E800E8
#define WriteToBufferCmd_Sibley  	0x00E900E9
#define WriteToBufferConfirmCmd    	0x00D000D0
#define WriteToBufferEFPCmd        	0x00800080 // Buffered Enhanced Factory Program
#define BlockEraseCmd              	0x00200020
#define BlockEraseConfirmCmd       	0x00D000D0
#define ProgramSuspendCmd          	0x00B000B0
#define BlockEraseSuspendCmd       	0x00B000B0
#define ProgramResumeCmd           	0x00D000D0
#define BlockEraseResumeCmd    		0x00D000D0
#define LockBlockCmd   	       		0x00600060
#define UnlockBlockCmd	    	   	0x00600060
#define LockDownBlockCmd       		0x00600060
#define LockBlockConfirmCmd 	  	0x00010001
#define UnlockBlockConfirmCmd 	  	0x00D000D0
#define LockDownBlockConfirmCmd   	0x002F002F

#define ProgramProtectionRegCmd		0x00C000C0
#define ProgramLockRegCmd			0x00C000C0
#define Lock2kOtpProtectionCmd		0x00C000C0
#define ProgramReadConfigRegCmd		0x00600060
#define STSPinConfigCmd     		0x00B800B8
#define ConfirmCmd                 	0x00D000D0

// Device identifier Codes
#define ManufacturerID	 			0x00890089
#define L18_64MbDeviceID 			0x880B880B
#define L18_128MbDeviceID 			0x880C880C
#define L18_256MbDeviceID 			0x880D880D
#define K3_64MbDeviceID 			0x88018801
#define K3_128MbDeviceID 			0x88028802
#define K3_256MbDeviceID 			0x88038803
#define K18_64MbDeviceID 			0x88058805
#define K18_128MbDeviceID 			0x88068806
#define K18_256MbDeviceID 			0x88078807

#define ReadArrayCmd_16             0x00FF
#define ReadIdentifierCmd_16		0x0090
#define ReadQueryCmd_16             0x0098
#define ReadStatusRegisterCmd_16    0x0070
#define ClearStatusRegisterCmd_16   0x0050
#define WordProgramCmd_16           0x0040
#define WordProgramCmd_16_Sibley    0x0041
#define ByteProgramCmd_16           0x0010
#define WriteToBufferCmd_16         0x00E8
#define WriteToBufferCmd_16_Sibley  0x00E9
#define WriteToBufferConfirmCmd_16  0x00D0
#define WriteToBufferEFPCmd_16      0x0080 // Buffered Enhanced Factory Program
#define BlockEraseCmd_16            0x0020
#define BlockEraseConfirmCmd_16     0x00D0
#define ProgramSuspendCmd_16        0x00B0
#define BlockEraseSuspendCmd_16     0x00B0
#define ProgramResumeCmd_16         0x00D0
#define BlockEraseResumeCmd_16    	0x00D0
#define LockBlockCmd_16   	       	0x0060
#define UnlockBlockCmd_16	    	0x0060
#define LockDownBlockCmd_16       	0x0060
#define LockBlockConfirmCmd_16 	  	0x00010001
#define UnlockBlockConfirmCmd_16 	0x00D0
#define LockDownBlockConfirmCmd_16  0x002F

#define ProgramProtectionRegCmd_16	0x00C0
#define ProgramLockRegCmd_16		0x00C0
#define Lock2kOtpProtectionCmd_16	0x00C0
#define ProgramReadConfigRegCmd_16	0x0060
#define STSPinConfigCmd_16     		0x00B8
#define ConfirmCmd_16               0x00D0

// Device identifier Codes
#define ManufacturerID_16	 		0x0089
#define L18_64MbDeviceID_T_16 		0x880B
#define L18_128MbDeviceID_T_16		0x880C
#define L18_256MbDeviceID_T_16		0x880D
#define L18_64MbDeviceID_B_16 		0x880E
#define L18_128MbDeviceID_B_16		0x880F
#define L18_256MbDeviceID_B_16		0x8810

#define K3_64MbDeviceID_16 			0x8801
#define K3_128MbDeviceID_16 		0x8802
#define K3_256MbDeviceID_16 		0x8803
#define K18_64MbDeviceID_16 		0x8805
#define K18_128MbDeviceID_16 		0x8806
#define K18_256MbDeviceID_16 		0x8807

#define DeviceReadyStatus_16        0x0080
#define EraseSuspendStatus_16       0x0040
#define EraseErrorStatus_16		 	0x0020
#define ProgramErrorStatus_16		0x0010
#define VppErrorStatus_16		    0x0008
#define ProgramSuspendStatus_16	    0x0004
#define BlockLockErrorStatus_16    	0x0002
#define BufferedEFPInProgressStatus_16 0x0001

// Actual OTP Lock Register Addresses.  This is due
// to the fact that the addresses are shifted by 2
// for 32-bit flash, and by 1 for 16-bit flash
#define OTPLockRegister0_32			(0x200)
#define OTPLockRegister1_32			(0x224)
#define OTPLockRegister0_16			(0x100)
#define OTPLockRegister1_16			(0x112)

#define PRLockReg0Addr_Factory_32	(0x204)
#define PRLockReg0Addr_Factory_16	(0x102)
#define PRLockReg0Addr_32			(0x214)
#define PRLockReg0Addr_16			(0x10A)
#define PRLockReg1Addr_32			(0x228) // 0x8A-0x91
#define PRLockReg1Addr_16			(0x114) // 0x8A-0x91
#define PRLockReg2Addr_32			(0x248) // 0x92-0x99
#define PRLockReg2Addr_16			(0x124) // 0x92-0x99
#define PRLockReg3Addr_32			(0x268) // 0x9A-0xA1
#define PRLockReg3Addr_16			(0x134) // 0x9A-0xA1
#define PRLockReg4Addr_32			(0x288) // 0xA2-0xA9
#define PRLockReg4Addr_16			(0x144) // 0xA2-0xA9
#define PRLockReg5Addr_32			(0x2A8) // 0xAA-0xB1
#define PRLockReg5Addr_16			(0x154) // 0xAA-0xB1
#define PRLockReg6Addr_32			(0x2C8) // 0xB2-0xB9
#define PRLockReg6Addr_16			(0x164) // 0xB2-0xB9
#define PRLockReg7Addr_32			(0x2E8) // 0xBA-0xC1
#define PRLockReg7Addr_16			(0x174) // 0xBA-0xC1
#define PRLockReg8Addr_32			(0x308) // 0xC2-0xC9
#define PRLockReg8Addr_16			(0x184) // 0xC2-0xC9
#define PRLockReg9Addr_32			(0x328) // 0xCA-0xD1
#define PRLockReg9Addr_16			(0x194) // 0xCA-0xD1
#define PRLockReg10Addr_32			(0x348) // 0xD2-0xD9
#define PRLockReg10Addr_16			(0x1A4) // 0xD2-0xD9
#define PRLockReg11Addr_32			(0x368) // 0xDA-0xE1
#define PRLockReg11Addr_16			(0x1B4) // 0xDA-0xE1
#define PRLockReg12Addr_32			(0x388) // 0xE2-0xE9
#define PRLockReg12Addr_16			(0x1C4) // 0xE2-0xE9
#define PRLockReg13Addr_32			(0x3A8) // 0xEA-0xF1
#define PRLockReg13Addr_16			(0x1D4) // 0xEA-0xF1
#define PRLockReg14Addr_32			(0x3C8) // 0xF2-0xF9
#define PRLockReg14Addr_16			(0x1E4) // 0xF2-0xF9
#define PRLockReg15Addr_32			(0x3E8) // 0xFA-0x101
#define PRLockReg15Addr_16			(0x1F4) // 0xFA-0x101
#define PRLockReg16Addr_32			(0x408) // 0x102-0x109
#define PRLockReg16Addr_16			(0x204) // 0x102-0x109

// Status Register Bit definitions
#define DeviceReadyStatus        	0x00800080
#define EraseSuspendStatus       	0x00400040
#define EraseErrorStatus		 	0x00200020
#define ProgramErrorStatus			0x00100010
#define VppErrorStatus		        0x00080008
#define ProgramSuspendStatus	    0x00040004
#define BlockLockErrorStatus    	0x00020002
#define BufferedEFPInProgressStatus 0x00010001

// Device geometry definition
#define SizeExponentOffset          0x2A // Buffer size offset
#define NumOfEraseBlksOffset        0x2D // Num of erase blocks offset
#define EraseBlockRegionSizeOffset  0x2F // Block region size offset

#define BlockLockOperationStatus	 0x1
#define BlockLockDownOperationStatus 0x2

#define FlashBusWidth32		  		32
#define FlashBusWidth16		  		16
#define FlashBusWidth8				8

typedef enum
{
	BlockLockOperation     = 0,
	BlockUnlockOperation   = 1,
	BlockLockDownOperation = 2
} BlockLockType_T;

typedef enum
{
	ProtectionRegister0  = 0,
	ProtectionRegister1,
	ProtectionRegister2,
	ProtectionRegister3,
	ProtectionRegister4,
	ProtectionRegister5,
	ProtectionRegister6,
	ProtectionRegister7,
	ProtectionRegister8,
	ProtectionRegister9,
	ProtectionRegister10,
	ProtectionRegister11,
	ProtectionRegister12,
	ProtectionRegister13,
	ProtectionRegister14,
	ProtectionRegister15,
	ProtectionRegister16
} ProtectionRegister_T;

typedef enum
{
  	FlashTypeCmd			= 1,
  	FlashLockDownStatusCmd,
  	FlashOTPStatusCmd,
  	EraseFlashCmd,
  	ProgramBootStatusCmd,
  	ReadFlashIDCmd,
  	ReadProtectionLockRegCmd,
  	ReadRNGSeedCmd,
  	ProgramRNGSeedCmd,
  	SetFlashBusWidthCmd,
  	ReadProtectionRegDataCmd,
  	ProgramInternalSRAMImageCmd,
  	GetFlashBusWidthCmd,
	ReadCS2ProtectionRegDataCmd
}FlashCommand_T;

// Function Prototypes
// This is the number of descriptors used for Memory to Memory moves
#define NumMem2MemDesc  600

    UINT_T Configure_Flashes (pFUSE_SET,  P_FlashProperties_T);
#if DKB
	void CopyOBMLocalToLoad(pTIM pTIM_h, P_FlashProperties_T pFlashP);
	UINT_T Finalize_Flashes (P_FlashProperties_T pFlashProp, pTIM pTim, UINT_T burnTIM_flag);
    UINT_T CreateBBMTable (UINT_T WTMFlashSign, UINT_T PlatformSettings, P_FlashProperties_T pFlashP);
#else
	UINT_T LoadTim( P_FlashProperties_T pFlashProp, UINT8_T *TIMArea, TIM *pTIM_h);
	UINT_T SetTIMPointers( UINT8_T * StartAddr, TIM *pTIM_h);
	UINT_T ExtractKeyHashFromOTP(UINT16_T *OTPKeyHashData, P_FlashProperties_T pFlashP, OTPHashType HashType);
	UINT_T FindNANDBBT (P_FlashProperties_T pFlashProp);
#endif

#if (TRUSTED || BOOTROM)
		UINT_T LoadSaveState(P_FlashProperties_T pFlashProp, pSaveState	State, TIM *pTIM_h);
		UINT_T Configure_SaveStateFlash(pFLASH_I pFLASH_I_h,  P_FlashProperties_T pFlashProp);
		UINT_T WriteWTMStateToFlash (pFLASH_I pFLASH_I_h, P_FlashProperties_T pFlashP, pSaveState pSaveState_h, UINT_T StartSlotNumber, UINT_T EndSlotNumber);
#endif

#if NAND_CODE
UINT_T ReadNANDWithBBM (UINT_T Address, UINT_T WriteAmount, P_DFC_BUFFER Buffer, P_FlashProperties_T pFlashP);
UINT_T WriteNANDWithBBM (UINT_T Address, UINT_T WriteAmount, P_DFC_BUFFER Buffer, P_FlashProperties_T pFlashP);
UINT_T RelocateBlock(P_FlashProperties_T pFlashP, UINT_T Block, UINT_T *BBEntry, USHORT *pReloEntry);
#endif

#if NOR_CODE
UINT_T ReadProtectionLockRegister(P_FlashProperties_T pFlashP, UINT16_T *LockRegister0, UINT16_T *LockRegister1);
void   RestoreFlashDefaultMode(P_FlashProperties_T pFlashP);
UINT_T VerifySingleWordProgram(P_FlashProperties_T pFlashP, VUINT_T WordVerifyAddr, UINT_T WordToVerify);
UINT_T LockProtectionRegister(P_FlashProperties_T pFlashP, UINT_T ProtectionRegLockNum);
UINT_T ProgramProtectionReg(P_FlashProperties_T pFlashP,
							UINT_T   ProtectionRegNum,
							UINT16_T *ProtectionRegData,
							UINT8_T  NumWordsToProgram);
UINT_T ProgramProtectionRegister(P_FlashProperties_T pFlashP, UINT_T ProtectionRegNum, UINT16_T *ProtectionRegData);
UINT_T ProgramProtectionRegisterWord(P_FlashProperties_T pFlashP,
									 UINT_T ProtectionRegNum,
									 UINT16_T  Data,
									 UINT8_T RegisterWord);
UINT_T ReadProtectionRegisterData(P_FlashProperties_T pFlashP,
    							  ProtectionRegister_T ProtectionRegToRead,
								  UINT16_T *ProtectionRegData);
UINT_T ProgramSingleWord(P_FlashProperties_T pFlashP, UINT_T WordProgramAddr, UINT_T WordToProgram);
UINT_T ProgramBlock(P_FlashProperties_T pFlashP, UINT_T BufferProgramAddr, UINT_T* WordToProgram);
UINT_T ProgramBlockSibley(P_FlashProperties_T pFlashP, UINT_T BufferProgramAddr, UINT_T* WordToProgram, UINT_T NumBlocks);
UINT_T DetermineProductID(P_FlashProperties_T pFlashP, UINT_T *ManufacturerCode, UINT_T *DeviceCode);
UINT_T  FlashMainProgram(P_FlashProperties_T pFlashP,
					  	 FlashCommand_T ReqdAction,
					  	 UINT_T   *InOut1,
					  	 UINT_T   *InOut2) ;
UINT_T Erase_XIP_Block (UINT_T Addr, P_FlashProperties_T pFlashProp);
UINT_T Erase_XIP_Flash (P_FlashProperties_T pFlashProp);
#endif //endif NOR_CODE


UINT_T CopyImageLocalToFlash(UINT_T ImageID, pTIM pTIM_h, P_FlashProperties_T pFlashP);
UINT_T CopyImageFlashToLocal(UINT_T, pTIM, P_FlashProperties_T, UINT_T *);
#if (!TRUSTED)
	UINT_T ProbeFlash(pFUSE_SET pFuses, pTIM pTIM_h, P_FlashProperties_T pFlashProperties, Reloc_T *pLocalRelo, UINT8_T * TIM_Area);
#endif


void Mem2Mem_ISR(void);
extern void EnableIrqInterrupts(void);
extern void DisableIrqInterrupts(void);

#endif


