/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. ?
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2002, 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	OBM.c
**
**  PURPOSE: 	OBM main routine
**
**	History:	Initial Creation 4/23/06
******************************************************************************/

#include "flash.h"
#include "protocolmanager.h"
#include "interrupts.h"
#include "general.h"
#include "obm_pxa930.h"
#include "obm_pxa-regs.h"
#include "misc.h"
#if TAVOR_BOERNE
#include "data_to_os.h"
#endif

#if TRUSTED
	#include "obm_security.h"
#endif


// These defines are for a backward compatible boot with version 2.xx boot ROM
// used for MH-P
#define VERSION_2_XX	0x322e
#define OS_LOAD_ADDRESS 0x5c020000
#define FLASH_ADDRESS	0x20000
#define IMAGE_SIZE		0x3A368

// Routines defined outside
extern void   TransferControl(unsigned int, int, unsigned int);
extern UINT_T PlatformOBMPrebootCode(pTIM, P_FlashProperties_T);
extern UINT_T PlatformOBMPostbootCode(pTIM, P_FlashProperties_T);
void OBMFinalize(UINT_T Code, P_FlashProperties_T pFlashProp);
extern UINT_T SetUpUSBDescriptors (pTIM, UINT_T, UINT_T);
#if VERBOSE_DEBUG
extern void ConvertIntToBuf8(unsigned char* StringPtr,unsigned int Value, int Width,unsigned int Count);
#endif

// Reference
void _mod0(void) {while (1);}
void InitDefaultPort(pFUSE_SET pFuses);
UINT_T InitTIMPortWithBackwardsCompatibleTIM( pFUSE_SET pFuses, UINT_T *Reserved);
UINT_T InitTIMPort( pFUSE_SET pFuses, pTIM pTIM_h);
void LoadVersion2XXImage(FUSE_SET PlatformSettings, P_FlashProperties_T pFlashProp);

#if (RUN_SW_UPGRADE)
extern void DownloadDKBToTarget(pTIM pTIM_h, P_FlashProperties_T pFlashProp);
#endif// RUN_SW_UPGRADE
extern UINT_T OBM_WaitForSwUpdateTrigger ( void );


/**
 * Boot Type is saved in r10 for OEM boot code usage
 **/
UINT_T SaveParamsForOsLoader(FUSE_SET  Fuses , UINT32_T   TlvAddress )
{
	__asm("MOV r10, r0");
	__asm("MOV r11, r1");
	return (NoError);
}

/**
 * Boot Type is saved in r10 for OEM boot code usage
 **/
UINT_T SaveParamsForEboot(FUSE_SET  Fuses)
{
    __asm("MOV r10, r0");
    return (NoError);
}


void OBMBootMain(UINT_T PlatformSettings)
{
	UINT_T   	Retval = NoError;
	FUNC_STATUS RetStatus;
	FlashProperties_T pFlashProp[2];	// 0 - Boot Flash, 1- Caddo Save State Flash
	Reloc_T  	LocalRelotable; 			// Local Copy of the Relocation Table
	FUSE_SET 	Fuses;					// Fuse information passed in from BootROM
	UINT8_T  	PlatformInitialized = TRUE;	// Keep track if the platform has been initialized or not
	UINT8_T		FlashProbed = FALSE;
	volatile    pIMAGE_INFO imageInfo;
	UINT_T ImageID;
	#if TRUSTED
  	FastSignatureVerificationImages FSVI_h;
	#endif
 	#if VERBOSE_DEBUG
	UINT8_T Msg[] = "-Fuse Config from boot ROM: 0xxxxxxxxx\0";
 	#endif
	UINT_T  *Version;

 	// TIM
	TIM TIM_h;							// Our struct to work our way around the TIM

    PMCR &= ~PMCR_BIE;

    pxa930_gpio_init();
#if SAAR_PV
    set_power();
#endif
#ifndef DDR_WORKAROUND
    power2saving();
    pxa_charger_ini();
#endif

#if NOFLASH_CODE
	PlatformSettings = 0x1820021;
#endif

	// Initialize any fuse information passed in by the BootROM
	Fuses.value = PlatformSettings;

	#if VERBOSE_DEBUG
	Fuses.bits.Download_Disable = 0;  //  allows ports to be enabled
	#endif

	#if OBMTEST
	 //Fuses.bits.PlatformState = 0x5; 	// Default to CS2 Sibley
	Fuses.bits.PlatformState = 0x4;	// Default to NAND x16
	Fuses.bits.BootPlatformState = 0x9;
	#endif

	// Read TBR version from 0x5c0127F0. The startup code copies the the version into this adddress
	// so the version info doesn't get overwritten once the MMU is enabled.
	Version = (UINT_T *) 0x5c0127F0;

	// If version is 2.xx, load w/o TIM header.	We will not return from this call.
	if ( (*Version >> 16) ==  VERSION_2_XX)
	{
 	   LoadVersion2XXImage(Fuses, &pFlashProp[0]);

	}

 	// Initialize the ports
	EnableIrqInterrupts();
	// Prepare to set up flashes, set BBT ptr's
	pFlashProp[0].NAND_Prop.pLocalRelotable = pFlashProp[1].NAND_Prop.pLocalRelotable = &LocalRelotable;

	// OPTION 1 - An old BootROM probed the flashes but did not pass us configuration data
	if ((Fuses.bits.PlatformState == 0)	&& (Fuses.bits.BootPlatformState == 0))
	{
		#if (!TRUSTED)
		// Probe the flashes
			Retval = ProbeFlash(&Fuses, &TIM_h, &pFlashProp[0], &LocalRelotable, (UINT8_T *)TIMLOADADDRESS);
			FlashProbed = TRUE;
		#else
			Retval = NotSupportedError;
		#endif
	}
	// OPTION 2,3 - BootROM Probed the flashes and passed us configuration data, or it is an initialized boot
	else
	{
	   	if (Fuses.bits.PlatformState == 0)
	   	{
	   	 Fuses.bits.PlatformState = Fuses.bits.BootPlatformState; // BootROM must have probed the flashes.
	   	 PlatformInitialized = FALSE;
	   	}
	   	// Configure flashes
        Retval = Configure_Flashes (&Fuses, &pFlashProp[0]);
	   	if (Retval == NoError)
			Retval = LoadTim(&pFlashProp[0], (UINT8_T *)TIMLOADADDRESS, &TIM_h);
	}

	// Any fatal errors to this point?
    if (Retval != NoError)
        OBMFinalize(Retval, &pFlashProp[0]);

	// Setup the ports from either TIM/NTIM or use default values
	RetStatus.ErrorCode = InitTIMPort(&Fuses, &TIM_h);
	if (RetStatus.ErrorCode == DownloadPortError)
		InitDefaultPort(&Fuses);

	// Print out who we are
#if TRUSTED
	AddMessage((UINT8_T*) ("*** OBM is running Trusted mode ***\0"));
#else
	AddMessage((UINT8_T*) ("*** OBM is running Non-Trusted mode ***\0"));
#endif
#if NOFLASH_CODE
	AddMessage((UINT8_T*) ("-Flash NOT configured.\0"));
#else
	AddMessage((UINT8_T*) ("-Flash Configured.\0"));
#endif

	//Jump to any specific platform code routines
	Retval = PlatformOBMPrebootCode(&TIM_h, &pFlashProp[0]);

#if VERBOSE_DEBUG
	// Print out fuse values from boot ROM
 	ConvertIntToBuf8(&Msg[30], PlatformSettings, 8,16);
 	AddMessage(Msg);
#endif
	// Any fatal errors to this point?
	if (Retval != NoError)
		OBMFinalize(Retval, &pFlashProp[0]);
	// Load the OSLoader or the image after the OBM
	// First see if there is an OSLoader image in the TIM
#if (!NOFLASH_CODE)
 	imageInfo = FindImageInTIM(&TIM_h, (UINT_T)OSLOADERID);
 	if(imageInfo == NULL)
 	{
		imageInfo = FindImageInTIM(&TIM_h, OBMIDENTIFIER);
		ImageID =  imageInfo->NextImageID;
		//update the imageInfo structure for the image being booted to
		imageInfo = FindImageInTIM(&TIM_h, ImageID);
		if(imageInfo == NULL)
		{
			//if we cannot find an image to boot to, error out
			OBMFinalize(ImageNotFound, &pFlashProp[0]);
		}
	}
	else
	{
  	   ImageID = OSLOADERID;
	}

	// Copy the image
	AddMessage((UINT8_T*) ("-Loading OS Loader Image\0"));
#if (!TAVOR_BOERNE)
#if VERBOSE_DEBUG
	Msg[0] = 'I';Msg[1] = 'D';Msg[2] = ':';Msg[3] = ' ';
	*(UINT_T*)&Msg[4] = ImageID;
	Msg[8] = NULL;
 	AddMessage(Msg);
#endif
#endif

#if (!TAVOR_BOERNE)
	AddMessageError(PlatformBusy);
#endif

	Retval = CopyImageFlashToLocal(ImageID, &TIM_h, pFlashProp, NULL);

#if (!TAVOR_BOERNE)
	AddMessageError(PlatformReady);
#endif
	// If any errors, print out message and die. BSP can customize this behaviour
	if (Retval != NoError)
		OBMFinalize(Retval, &pFlashProp[0]);


	#if !TRUSTED
		// Validate NTIM
	 	AddMessage((UINT8_T*) ("-Verifying Checksums...\0"));
		if (!FlashProbed)
			Retval = ValidateCheckSum(TIMIDENTIFIER, &TIM_h, &pFlashProp[0]);
		   	if (Retval != NoError)
				OBMFinalize(Retval, &pFlashProp[0]);
			Retval = ValidateCheckSum(ImageID, &TIM_h, &pFlashProp[0]);
		   	if (Retval != NoError)
				OBMFinalize(Retval, &pFlashProp[0]);

	#else   // TRUSTED
		if ( OBM_GetProductionModeState() == NON_PRODUCTION_MODE_STATE )		// This is done to save time in production
		{
			// Validate TIM
			AddMessage((UINT8_T*) ("-Validate TIM...\0"));
			Retval = ValidateTIM(&TIM_h);
	   		if (Retval != NoError)
				OBMFinalize(Retval, &pFlashProp[0]);
			// Validate the Image
			AddMessage((UINT8_T*) ("-Validate Image...\0"));
   			Retval = ValidateImage(ImageID, NULL, &TIM_h);
	   		if (Retval != NoError)
				OBMFinalize(Retval, &pFlashProp[0]);
			// Validation complete
			AddMessage((UINT8_T*) ("-Validation Complete...\0"));
		}
 	#endif   // !TRUSTED
#endif   //NOFLASH_CODE


#if 1
    {extern void enable_i_d_cache(void);
        enable_i_d_cache();
    }
#endif

 	Retval = PlatformOBMPostbootCode(&TIM_h, &pFlashProp[0]);
#if 1
    {extern void disable_i_d_cache(void);
        disable_i_d_cache();
    }
#endif
	if (Retval != NoError)
		OBMFinalize(Retval, &pFlashProp[0]);

#if TRUSTED
	if ( ( PlatformInitialized == FALSE ) &&
	     ( OBM_GetProductionModeState() == PRODUCTION_MODE_STATE ) )
	{
		AddMessage((UINT8_T*) ("***AutoBind Required...\0"));
#if defined (PERFORM_FUSE_PROGRAM_WA)
		OBM_ChangeMvtVoltageForBinding();
#endif
  		Retval = AutoBindPlatform ( &TIM_h , pFlashProp , Fuses.value );
		if (Retval == NoError)
			 AddMessage((UINT8_T*) ("Platform Bound, attempting boot.\0"));
		else
			OBMFinalize( Retval , &pFlashProp[0] );

		OBM_WaitForSwUpdateTrigger ();

#if (RUN_SW_UPGRADE)
		if ( CheckSwUpdateTriggeredFromUsb() == TRUE )
		   DownloadDKBToTarget( &TIM_h , &pFlashProp[0] );
#endif
	}
#endif  // TRUSTED


#if TRUSTED
	AddMessage((UINT8_T*) ("-Disabling Caddo.\0"));
	EnableDisableCaddo(DISABLE_CADDO); 	  		// Disable the Caddo Unit

	AddMessage((UINT8_T*) ("-Allow upper SRAM partition.\0"));
	WtmSramBoundry ( SRAM_BOUNDRY_LOCKED_UPPER_4K );
#endif
#ifndef DDR_WORKAROUND
    pxa_load_rtc(0);pxa_printf_rtc(0);
    #if 0
    {
        int i;
        for(i = 0;i < 10;i++)
        {
            pxa_load_rtc(0);
            pxa_printf_rtc(0);
            Delay_seconds(1);
        }
    }
    #endif
#endif

    load_prdcfg_element(pFlashProp);

 	AddMessage((UINT8_T*) ("-XFer Control To OS Loader...\0"));
 	AddMessage((UINT8_T*) (""));


 	//0HandleDisconnect();
#if (!NOFLASH_CODE)
 	ShutdownPort(FFUART_D);
#endif
 	ShutdownPort(ALTUART_D);
 	ShutdownPort(DIFF_USB_D);
 	ShutdownPort(U2D_USB_D);
 	DisableIrqInterrupts();

#if TAVOR_BOERNE
    SaveParamsForOsLoader(Fuses , OBM_OS_BUFFRE_BASE_ADDRESS);
#else
    SaveParamsForOsLoader(Fuses , NULL);
#endif

    // TRANSFER CONTROL TO OS
#if (NOFLASH_CODE)
 	TransferControl(0x80000000, 0, 0x80000000);
#else
 	TransferControl(imageInfo->LoadAddr, 0, imageInfo->LoadAddr);
#endif
	_mod0();
}

void LoadVersion2XXImage(FUSE_SET PlatformSettings, P_FlashProperties_T pFlashProp)
{

 	UINT_T Retval;

	switch((PlatformSettings.value)& 0x7){
		case 0x0:
		case 0x1:
		case 0x2:
			Retval = InvalidPlatformConfigError;
			break;
		case 0x3:
		case 0x5:
		case 0x7:
			// jump to XIP address
			Retval = NoError;
			break;
#if NAND_CODE
		case 0x4:
			//NAND x16
			Retval = XllpDfcInit( 16, pFlashProp);
			Retval = XllpDfcRead((P_DFC_BUFFER)OS_LOAD_ADDRESS, FLASH_ADDRESS, IMAGE_SIZE, NULL, pFlashProp);
			break;
		case 0x6:
			//NAND x8
			Retval = XllpDfcInit( 8, pFlashProp);
			Retval = XllpDfcRead((P_DFC_BUFFER)OS_LOAD_ADDRESS, FLASH_ADDRESS, IMAGE_SIZE, NULL, pFlashProp);
			break;
#endif //NAND_CODE
 		default:
			Retval = InvalidPlatformConfigError;
			break;

	}
	if (Retval == NoError)
	{
		AddMessage((UINT8_T*) ("*** XFer Control To OS...\0"));
		TransferControl(OS_LOAD_ADDRESS,0,OS_LOAD_ADDRESS);
	}
	else
	{
		OBMFinalize(Retval, &pFlashProp[0]);
	}
  	_mod0();
}



#if TRUSTED
/*****************************************************************************************/
/*****************************************************************************************/
void OBM_ReEnableJtag ( P_FlashProperties_T     pFlashProp )
{
	FUNC_STATUS RetStatus;


  	if ( GetPortInterruptFlag() )				// Any JTAG Re-enablement requests?
  	{
        AddMessage((UINT8_T*) ("-Start running Jtag Re-Enable process.\0"));
 		RetStatus = HandleRequest ( NULL , JTAGIDENTIFIER , pFlashProp );
		if ( RetStatus.ErrorCode == NoError )
        	AddMessage((UINT8_T*) ("-Jtag Re-Enable Ended.\0"));
		else
        	AddMessage((UINT8_T*) ("*** Error: Jtag Re-Enable Failed.\0"));
		ClearPortInterruptFlag();
  	}
}
/*****************************************************************************************/
/*****************************************************************************************/
#endif




// We write Pass/Fail code
// Write Block 0 for NAND flash
void OBMFinalize ( UINT_T    Code , P_FlashProperties_T   pFlashProp )
{
	unsigned char 	Msg[9];
	TIM 			TIM_h;

	// Print out why we failed
    if (Code != NoError)
	{
        AddMessage((UINT8_T*) ("Platform boot failed.\0"));
#if VERBOSE_DEBUG
		ConvertIntToBuf8(Msg, Code, 8,16);
		Msg[8] = NULL;
		AddMessage(Msg);
#else
 		AddMessageError(Code);
#endif
  	}

 	// Fatal error occured. Loop forever. BSP can optinally handle this differently.
 	do
 	{
		/* allow SW Upgrade when OBM is stuck */
#if (RUN_SW_UPGRADE)
		/* is SW Upgrade triggered via USB? */
		if ( CheckSwUpdateTriggeredFromUsb() == TRUE )
		{
			DownloadDKBToTarget( (UINT8_T *)&TIM_h, &pFlashProp[0]);
		}
#endif// RUN_SW_UPGRADE

#if TRUSTED
		OBM_ReEnableJtag( pFlashProp );
#endif

 	} while (Code != NoError);

 	return;
}

//**************************************************************************************
// Used to initialize default ports when no TIM is found or no port is specified in the TIM
void InitDefaultPort(pFUSE_SET pFuses)
{
	unsigned int port_selection;
	unsigned int usb_default_port;

	//read the GPIO's to determine port configuration
	port_selection= getPlatformPortSelection(&usb_default_port);
	switch(port_selection)
	{
	case 0:
			if (pFuses->bits.Download_Disable)		// By RIM request Disable USB if no TIM descriptor packet
				break;
			SetUpUSBDescriptors (NULL, 0, SE_USB_D);
			InitPort(SE_USB_D, pFuses);
			pFuses->bits.USBPort = 1;
			pFuses->bits.PortEnabled = 1;
			break;

	case 1:
			InitPort(ALTUART_D, pFuses);
			pFuses->bits.UARTPort = 1;
			pFuses->bits.PortEnabled = 1;
			break;

	case 2:
	case 3:
			InitPort(FFUART_D, pFuses);
			pFuses->bits.PortEnabled = 1;
			if (pFuses->bits.Download_Disable)		 // By RIM request Disable USB if no TIM descriptor packet
				break;
			SetUpUSBDescriptors (NULL, 0, usb_default_port);
			InitPort(usb_default_port, pFuses);
			pFuses->bits.USBPort = 0;
			pFuses->bits.UARTPort = 0;
	}
//	pFuses->bits.PortEnabled = 1;
}

// This routine can override Port settings based on the TIM
UINT_T InitTIMPortWithBackwardsCompatibleTIM( pFUSE_SET pFuses, UINT_T *Reserved)
{
	pOPT_SET pOptions;
	UINT_T Retval = NoError;

	pOptions = (pOPT_SET)Reserved;
	switch(pOptions->PortType)
	{
		case UARTID:
			if(pOptions->Port == FFIDENTIFIER)
			{
				InitPort(FFUART_D, pFuses);
				pFuses->bits.UARTPort = 0;
				pFuses->bits.PortEnabled = 1;
			}
			else if(pOptions->Port == ALTIDENTIFIER)
			{
				InitPort(ALTUART_D, pFuses);
				pFuses->bits.UARTPort = 1;
				pFuses->bits.PortEnabled = 1;
			}
			else Retval = DownloadPortError;
				break;

		case USBID:

			if(pOptions->Port == DIFFIDENTIFIER)
			{
				SetUpUSBDescriptors (NULL, 0, DIFF_USB_D);
				InitPort(DIFF_USB_D, pFuses);
				pFuses->bits.USBPort = 0;
				pFuses->bits.PortEnabled = 1;
			}else if(pOptions->Port == SEIDENTIFIER)
			{
				InitPort(SE_USB_D, pFuses);
				pFuses->bits.USBPort = 1;
				pFuses->bits.PortEnabled = 1;
				SetUpUSBDescriptors (NULL, 0, SE_USB_D);
			}else if(pOptions->Port == U2DIDENTIFIER)
			{
				InitPort(U2D_USB_D, pFuses);
				pFuses->bits.USBPort = 1;
				pFuses->bits.PortEnabled = 1;
				SetUpUSBDescriptors (NULL, 0, U2D_USB_D);
			}else Retval = DownloadPortError;
				break;

		default:
			Retval = DownloadPortError;
	}
	return Retval;
}

// This routine can override Port settings based on the TIM
UINT_T InitTIMPort( pFUSE_SET pFuses, pTIM pTIM_h)
{
	pOPT_PROTCOL_SET pOptions;				// Protocol Options
	pWTP_RESERVED_AREA_HEADER pWRAH = NULL;
	UINT_T Retval = NoError;
	UINT_T PortStatus = DownloadPortError;

#if (!DISABLE_DEFAULT_USB_ENUM)
	return PortStatus;
#endif

	// Only init ports if this is not a resume process
	if(!pFuses->bits.Resume)
	{
		pWRAH = FindPackageInReserved (&Retval, pTIM_h, UARTID);
		if (Retval != NoError)
			pWRAH = FindPackageInReserved (&Retval, pTIM_h, USBID);

		if (Retval == NoError)
		{
			// We found a USB or UART Package.
			pOptions = (pOPT_PROTCOL_SET) pWRAH;
			switch(pOptions->WRAH.Identifier)
			{
		 		case UARTID:
					if(pOptions->Port == FFIDENTIFIER)
					{
						InitPort(FFUART_D, pFuses);
						pFuses->bits.UARTPort = 0;
						pFuses->bits.PortEnabled = 1;
						PortStatus = NoError;
					}
					else if(pOptions->Port == ALTIDENTIFIER)
					{
						InitPort(ALTUART_D, pFuses);
						pFuses->bits.UARTPort = 1;
						pFuses->bits.PortEnabled = 1;
						PortStatus = NoError;
					}
					break;
				case USBID:
					if(pOptions->Port == DIFFIDENTIFIER)
					{
						SetUpUSBDescriptors (pTIM_h, 1, DIFF_USB_D);
						InitPort(DIFF_USB_D, pFuses);
						pFuses->bits.USBPort = 0;
						pFuses->bits.PortEnabled = 1;
						PortStatus = NoError;
					}
					else if(pOptions->Port == SEIDENTIFIER)
					{
						SetUpUSBDescriptors (pTIM_h, 1, SE_USB_D);
						InitPort(SE_USB_D, pFuses);
						pFuses->bits.USBPort = 1;
						pFuses->bits.PortEnabled = 1;
						PortStatus = NoError;
					}
					else if(pOptions->Port == U2DIDENTIFIER)
					{
						SetUpUSBDescriptors (pTIM_h, 1, U2D_USB_D);
						InitPort(U2D_USB_D, pFuses);
						pFuses->bits.USBPort = 1;
						pFuses->bits.PortEnabled = 1;
						PortStatus = NoError;
					}
					break;
			}
		}
	}
	return PortStatus;
}
