/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2003, 2004 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	Jtag.c
**
**  PURPOSE: 	Main Jtag ReEnabling program.  Handles all aspects of the jtag
** 				re-enabling process
**
******************************************************************************/

#include "errors.h"
#include "typedef.h"
#include "flash.h"
#include "general.h"
#include "trustedboot.h"
#include "dsvl.h"
#if BOOTROM
 #include "BootROM.h"
 #include "Test.h"
#endif

// external functions
extern UINT_T GetPlatformConfig(void);

//timer functions
extern UINT_T GetOSCR0(void);
extern UINT_T OSCR0IntervalInSec(UINT_T Before, UINT_T After);

#ifdef JTAG_RE_ENABLE_TEST
UINT_T OEMTestPassWd[] = {0x03487739, 0x936da3a8, };
#endif // JTAG_RE_ENABLE_TEST

#ifdef RTL_TEST
//UINT_T TmpBuf = 0;
UINT_T OEMPublicKey[] = {0x51aeec5f, 0x00000000, 0x00000000, 0x00000000,
						 0x00000000, 0x00000000, 0x00000000, 0x00000000,
						 0x00000000, 0x00000000, 0x00000000, 0x00000000,
						 0x00000000, 0x00000000, 0x00000000, 0x00000000,
						 0xd29a138d, 0xf4c3b5be, 0x5a1975de, 0xd20cc497,
						 0xb1f74960, 0x050d61f7, 0xbdb3a7d6, 0x6d3ac7ee,
						 0x17b66993, 0xfcf1d3c5, 0xc712d110, 0xb7477387,
						 0x9d022842, 0xe30d7893, 0x6ba00258, 0x8d248dbe,};
UINT_T OEMSignedPassWd[] = {0x4dd76be2, 0x7a830295, 0xc9cb779c, 0x873da997,
							0xbecb15f5, 0xe7178382, 0xa5ccc563, 0xd24348d,
							0x14357689, 0xc262c75a, 0x64fdff8c, 0x29c431d3,
							0xb34e92b6, 0x1eba1e56, 0x7c24c51e, 0x41dac463,};
// THIS IS FOR PASSWORD VALUE 0F 0xFFFFFFFF 0xFFFFFFFF, with 512 key length
//{0x1d86d3bc, 0x5752f887, 0x79bc17c2, 0x3a26f7fc,
// 							0x383a5dbb, 0xfeb35247, 0xfa99f7c0, 0xae6dc350,
// 							0x7cb2f625, 0xa74f464e, 0xe8cf7712, 0xea45275e,
//							0x6446add0, 0xfa615f03, 0xb2a584ee, 0x85193205,};
//UINT_T OEMTestPassWd[] = {0x03487739, 0x936da3a8, };
#endif


UINT_T ValidatePublicKey(JTAGKey *PublicKey, P_FlashProperties_T pFlashP){

	int i;
	UINT_T Retval;
	UINT_T DataLength;
 //	UINT8_T	*ImageDLLocation;
  //	TIM TIM_h;
  //	pVERSION_I pTIMVersion = (pVERSION_I)ImageDLLocation;

  //	ImageDLLocation = getTimAddr;
	if((PublicKey->KeyLength != 64) && (PublicKey->KeyLength != 32) && (PublicKey->KeyLength != 16))
		{
			return JtagReEnableOEMKeyLengthError;
		}

	//Compute the hash on the key we just got.
	DataLength     = (PublicKey->KeyLength * 8); // Length in bytes (public key and modulus)
	Retval = WtmSHA1MessageDigest((UINT8_T *)&PublicKey->RSAPubExp_Mod_SPW_Pad[0],
								 	 DataLength,	// Size of data to hash in bytes
							 	     &PublicKey->SHAOutput[0]);
 #if 0
	// If TIM has passed validation it will be in ISRAM at the proper location when this was a download or NAND
	// if it is an XIP we need to point to the XIP address.  Check the ISRAM first to cover the case where
	// TIM in XIP has failed validation.
	// Load up a structure with the pointers and verify the JTAG key hash with that in the TIM
	// Note that Set Trusted Keys will have already been called and verified the OTP if necessary.
	if(GetTIMValidationStatus() == PASSED){
		if( pTIMVersion->Identifier == TIMIDENTIFIER ){
			Retval = LoadTim(NULL, (UINT8_T*)ImageDLLocation, &TIM_h);
		}else if ((pFlashP->FlashType == SIBLEY_FLASH) || (pFlashP->FlashType == TYAX_FLASH)){
			Retval = LoadTim(pFlashP, (UINT8_T*)ImageDLLocation, &TIM_h);
		}else {
			Retval = TIMNotFound;
		}
		if(Retval == NoError)
			Retval = VerifyKeyHashes(JTAGIDENTIFIER, (UINT_T *)&PublicKey->SHAOutput[0], &TIM_h);
	}else{
		Retval = InvalidTIMImageError;
	}

 #endif
	return Retval;
}

UINT_T GetJTAGPassword(UINT8_T *buffer){
	UINT_T DummyBuff[20];
	UINT_T Retval = NoError;

	Retval = WtmJtagRequest((UINT_T *)buffer);

	return Retval;
}


UINT_T VerifySignedPassword(JTAGKey *PublicKey){

	UINT_T DataLength = 0;
	UINT_T KeyHashDataSrc[64];
	UINT_T Retval = 0;
	int i, Tmp;

	// Compute the required padding for the key length supplied
	// from host. (need to pad 16 words of 0's
	Tmp = (PublicKey->KeyLength*3);
	for (i=0; i<16; i++)
	{
		PublicKey->RSAPubExp_Mod_SPW_Pad[(Tmp+i)] = 0x0;
	}
	PublicKey->RSAPubExp_Mod_SPW_Pad[Tmp] = 0x80000000;
	// Store the size of the public key in bits. This
	// covers both the exponent and system modulus
	PublicKey->RSAPubExp_Mod_SPW_Pad[(Tmp+15)] = (PublicKey->KeyLength*64); // Size in bits

	DataLength = ((PublicKey->KeyLength*3) + 16) * 4;      // Size in bytes
	WtmSHAInit();
	Retval = WtmJtagResponse(&PublicKey->RSAPubExp_Mod_SPW_Pad[0], DataLength, PublicKey->KeyLength);

	return Retval;
}




