/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2002, 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	Security.c
**
**  PURPOSE: 	OBM specific security routines
**
**	History:	Initial Creation, 3/23/06
**
******************************************************************************/

#include "obm_security.h"
#include "flash.h"

#if MDOC_CODE
#include "msys.h"
#endif

#if ONENAND_CODE
//#include "onenand_obm_interface.h"
#endif

#if VERBOSE_DEBUG
#include "protocolmanager.h"
extern void ConvertIntToBuf8(unsigned char* StringPtr,unsigned int Value, int Width,unsigned int Count);
#endif

/********************	ValidateImage ******************************************************
*  Validate the Image based on a Hash passed in for the TIM or the hash for individual images
*  can be determined from IMAGE_INFO structure.
*
********************************************************************************************/
UINT_T ValidateImage(UINT_T ImageID, UINT_T* TIM_Hash, pTIM pTIM_h)
{
 UINT_T Retval = NoError;
 UINT_T StartAddr, EndAddr, HashSize, i, IVM_KeyLength;
 UINT_T CalculatedHash[WordLengthOf_SHA1];			// Contains 5 words
 UINT_T* HashInTIM;
 UINT8_T ImageOrder;

 // First Find the Image
 for( i = 0; i < pTIM_h->pConsTIM->NumImages; i++)
 {
	 if(pTIM_h->pImg[i].ImageID == ImageID)
	 {
		ImageOrder = i;
		break;
	 }
 }

 // Get Start Address to begin Hash
 switch (ImageID)
 {
  case TIMIDENTIFIER:
    StartAddr = (UINT_T) pTIM_h->pConsTIM;			  // Validate in ISRAM
	HashInTIM = TIM_Hash;
	break;
  default:
  	StartAddr = pTIM_h->pImg[ImageOrder].LoadAddr; 	  // Validate in ISRAM
	HashInTIM = pTIM_h->pImg[ImageOrder].Hash;
	break;
 }

 // Get the image size to hash
 HashSize  = pTIM_h->pImg[ImageOrder].ImageSizeToHash;

 // Do the SHA1 Hash
 Retval = WtmSHA1MessageDigest((unsigned char *)StartAddr, HashSize, (unsigned char *)CalculatedHash);
 CheckErrorReturn(Retval);

 // Compare the 2 hashes
 for (i = 0; i < WordLengthOf_SHA1; i++)
 {
	if (HashInTIM[i] != CalculatedHash[i])
		return (InvalidTIMImageError);
 }

 return NoError;
}

UINT_T ValidateTIM(pTIM pTIM_h)
{
	UINT_T Retval = NoError;

	// Digital Signature
	UINT_T *DigitalSignatureEncrypted;
	UINT_T DigitalSignatureDecrypted[KeyLength_64];		// Allow for maximum possible size
	UINT_T ModulusExponentPacked[MAXKEYSIZEWORDS * 2];
	UINT_T FastSignatureVerifyOutput[6], i;

	// Key Lengths
    UINT16_T KeyLength;					// Key Length in words
	FastSignatureVerify FSV_h;



	// Set KeyLength
	KeyLength = ((pTIM_h->pTBTIM_DS->ModulusSize) / 4);  // In number of words
	if (pTIM_h->pTBTIM_DS->DSAlgorithmID == Intel_DS)
	{
		// Decrypt Digital Signature inside TIM
		DigitalSignatureEncrypted = (UINT_T *) pTIM_h->pTBTIM_DS->RSADigS;

		Retval = WtmRSAEncrypt_Int(DigitalSignatureEncrypted,
							  pTIM_h->pTBTIM_DS->RSAModulus,
							  KeyLength,
							  &(pTIM_h->pTBTIM_DS->RSAPublicExponent[0]),
							  1,
							  DigitalSignatureDecrypted);
		CheckErrorReturn(Retval);

		Retval = ValidateImage(TIMIDENTIFIER, DigitalSignatureDecrypted, pTIM_h);
	}
	else   // Validate the TIM only using FastSignaturePI.
	{
	 // Key prep - Fill in the Exponent
	 for (i = 0; i < KeyLength; i++)
		ModulusExponentPacked[i] = pTIM_h->pTBTIM_DS->RSAPublicExponent[i];
	 // Fill in the modules
	 for (i = 0; i < KeyLength; i++)
		ModulusExponentPacked[(KeyLength+i)] = pTIM_h->pTBTIM_DS->RSAModulus[i];

	 FSV_h.Originator = SW_NoVerify_NoBindExport_NoPublicKeyVerify;
	 FSV_h.KeyLength = KeyLength;
	 FSV_h.ByteSwap = No_Swap;
	 FSV_h.PaddingType = PKCS_v15Pad_LittleEndian;
	 FSV_h.ImageAndSHAByteLength[0] = pTIM_h->pImg[0].ImageSizeToHash;  // Only deal with the covered part of the TIM
	 FSV_h.pImageAddress[0] = (UINT_T *)pTIM_h->pConsTIM;
	 FSV_h.pImageSHAAddress[0] = 0;
	 for (i=1; i<7; i++)
	 {
	 	FSV_h.ImageAndSHAByteLength[i] = 0;
		FSV_h.pImageSHAAddress[i] = NULL;
		FSV_h.pImageAddress[i] = NULL;
	 }
	 FSV_h.pDigitalSignature = pTIM_h->pTBTIM_DS->RSADigS;
	 FSV_h.pProvisionedKeyDigest = pTIM_h->pTBTIM_DS->Hash;
	 FSV_h.pRSAPublicKey = ModulusExponentPacked;
	 FSV_h.pOutput = FastSignatureVerifyOutput;
	 Retval = WTMFastSignatureVerify(&FSV_h);
	}

	return Retval;
}

#if(0)
UINT_T UpdateSaveState(UINT_T Code, pTIM pTIM_h, P_FlashProperties_T pFlashProp)
{
	UINT_T Retval = NoError;
	SaveState SaveState_h;
	UINT_T 	MinSlot, BBTSlot;

    // Configure and Load Save State
	Retval = Configure_SaveStateFlash(&(pTIM_h->pConsTIM->FlashInfo), pFlashProp);
	CheckErrorReturn(Retval);		   // If save state broken we can't save anything anyway

	// Load the save state to local area memory
	Retval = LoadSaveState(pFlashProp, &SaveState_h, pTIM_h);
	CheckErrorReturn(Retval);		   // If save state broken we can't save anything anyway

	// Load MicroCode Patch if there is one. This should be the last PI executed.
	//if (Code == NoError)
	  //	Code = MicroCode_Patching(pFlashProp, &SaveState_h, pTIM_h);

	// No need to update the save state if no error
	if (Code == NoError)
		return Retval;

	// Change the OBM error code
	SaveState_h.SlotN.DKBOBMStatus = Code;

	// Write out a new slot
	Retval = WriteWTMStateToFlash (&(pTIM_h->pConsTIM->FlashInfo), &pFlashProp[1], &SaveState_h, SaveState_h.NextSlotAvailable, SaveState_h.NextSlotAvailable);

 	// Has the Bad Block Table changed as a result of these writes? If so update a BBT to a new slot.
#if NAND_CODE
 	if ((pFlashProp[1].FlashType == NAND_FLASH) && (pFlashProp[1].NAND_Prop.BBT_State == BBT_CHANGED))
	{
		MinSlot = (pFlashProp[1].BlockSize / pFlashProp[1].NAND_Prop.PageSize) - pFlashProp[1].NAND_Prop.NumBBTSlots;	// Lowest slot we are allowed to write to.
		BBTSlot = pFlashProp[1].NAND_Prop.BBT_Slot - 1;				// We write to the next slot
		if (MinSlot != BBTSlot)										// Make sure there is one available
			Retval = XllpDfcWrite((P_DFC_BUFFER) (pFlashProp[1].NAND_Prop.pLocalRelotable), (pFlashProp[1].NAND_Prop.BBT_Slot * pFlashProp[1].NAND_Prop.PageSize), sizeof(Reloc_T), &pFlashProp[1]);
	}
#endif
	return Retval;
}
#endif


/****************** Program Platform Configuration Fuses *************
* This routine is called only if trusted boot is successful so far...
*********************************************************************/
UINT_T ProgramPlatformFuses(pFLASH_I pFLASH_I_h, UINT_T PlatformSettings)
{
 UINT_T Retval;
 FUSE_SET  Fuses;				// Fuse information passed in from BootROM
 UINT_T CaddoPConfigOutput[3] = {0,0,0};
#if VERBOSE_DEBUG
 unsigned char Msg[9];
 UINT_T MsgInt;
 UINT_T i;
#endif

 Fuses.value = PlatformSettings;
 Fuses.value &= 0x001E;			// Mask out all bits but the flash configuration bits


  // Write it out
 Retval = WtmPlatform_Config (Fuses.value, CaddoPConfigOutput, 0x2A8, 0x2A8, 0x1530);
#if VERBOSE_DEBUG
 AddMessage("Output from Platform Config:\0");
 for (i = 0; i < 3; i++){
		MsgInt = CaddoPConfigOutput[i];
		ConvertIntToBuf8(Msg, MsgInt, 8,16);
		Msg[8] = NULL;
		AddMessage(Msg);
 }
 if (Retval != NoError){
		AddMessage((UINT8_T*) ("***  Retval :\0"));
		MsgInt = Retval;
		ConvertIntToBuf8(Msg, MsgInt, 8,16);
		Msg[8] = NULL;
		AddMessage(Msg);
  }
#endif
// if (Retval != NoError)
//	return (FuseRWError);

 return NoError;
}

// This routine will program TIM verification keys to Caddo via Bind and program OTP registers for XIP.
// JTAG verification keys are also programmed to OTP.
UINT_T ProgramOBMImage(pTIM pTIM_h, P_FlashProperties_T pFlashP)
{
	UINT_T   	i, Regval, Retval;
	UINT_T   	ProtectionRegNum;
	UINT_T   	ProgramStatus;
	UINT16_T 	Lock0, Lock1;
	UINT16_T 	ProtectionDataRead[26];
	UINT_T 		WordData;
	USHORT		KeyIndex;
	static 		UINT_T  FlashID= 0;
	UINT_T		JtagKeyFound = FALSE;
#if VERBOSE_DEBUG
    unsigned char Msg[9];
    UINT_T MsgInt;
#endif

	// Used for OEM Verfication Key
	static __attribute__((aligned(32))) UINT_T OEMKeyData[BINDKEYSIZE]; 				// Contains key to hash	128 words maximum

	// Hashed KeyData
	UINT_T KeyDataC[WordLengthOf_SHA1 * 2];			// Contains 10 words, 5 for OEM, 5 for JTAG
	UINT_T *OEMKeyDataC, *JTAGKeyDataC;
	// Used for JTAG Verfication Key
	UINT_T 	JTAGKeyData[BINDKEYSIZE];		 		// Used for JTAG

	// Key Lengths
    UINT_T DS_KeyLength;			// Key Length in words
    UINT_T JTAG_KeyLength;		// Key Length in words



	/**************** Calculate Hash for the OEM Verification Keys ********/
	// Extract the KeyLength value from TIM Digital Signature in ISRAM in words
	DS_KeyLength    = ((pTIM_h->pTBTIM_DS->ModulusSize) / 4);

	// Fill in the Exponent
	for (i = 0; i < DS_KeyLength; i++)
		OEMKeyData[i] = pTIM_h->pTBTIM_DS->RSAPublicExponent[i];

	// Fill in the modulus
	for (i = 0; i < DS_KeyLength; i++)
		OEMKeyData[(DS_KeyLength+i)] = pTIM_h->pTBTIM_DS->RSAModulus[i];

	// Call Platform Bind, the output of which will be placed in OTP registers
	OEMKeyDataC = KeyDataC;
  	Retval = WtmPlatform_Bind (OEMKeyData, (pTIM_h->pTBTIM_DS->ModulusSize), OEMKeyDataC, 0x2A8, 0x2A8, 0x1530);
#if VERBOSE_DEBUG
	if( Retval == NoError)
	 	 AddMessage((UINT8_T*) ("*** Bind Succesful...\0"));
	else
	{
	 	AddMessage((UINT8_T*) ("*** Bind Failed...\0"));
		AddMessage((UINT8_T*) ("***  Retval :\0"));
		MsgInt = Retval;
		ConvertIntToBuf8(Msg, MsgInt, 8,16);
		Msg[8] = NULL;
		AddMessage(Msg);
	}
	AddMessage("Calculated Hash from Bind:\0");
	for (i = 0; i < 5; i++){
		MsgInt = KeyDataC[i];
		ConvertIntToBuf8(Msg, MsgInt, 8,16);
		Msg[8] = NULL;
		AddMessage(Msg);
	}
#endif
	/**************** Calculate Hash for the JTAG *********************/
	// Find the JTAG Key
	for (KeyIndex = 0; KeyIndex < pTIM_h->pConsTIM->NumKeys; KeyIndex++)
	{
		if (pTIM_h->pKey[KeyIndex].KeyID ==  JTAGIDENTIFIER)
		{
			JtagKeyFound = TRUE;
			break;
		}
  	}
    if (JtagKeyFound == TRUE)
    {
		// Extract the KeyLength value from TIM Digital Signature in ISRAM in words
		JTAG_KeyLength = ((pTIM_h->pKey[KeyIndex].ModulusSize) / 4);

		// Fill in the Exponent
		for (i = 0; i < JTAG_KeyLength; i++)
			JTAGKeyData[i] = pTIM_h->pKey[KeyIndex].RSAPublicExponent[i];

		// Fill in the modules
		for (i = 0; i < JTAG_KeyLength; i++)
			JTAGKeyData[(JTAG_KeyLength+i)] = pTIM_h->pKey[KeyIndex].RSAModulus[i];

		// Use WTM to do the Hash
		JTAGKeyDataC = (UINT_T *) &KeyDataC[WordLengthOf_SHA1];
		Retval = WtmSHA1MessageDigest((unsigned char *)JTAGKeyData, (4*2*JTAG_KeyLength), (unsigned char *)JTAGKeyDataC);
	 }
    /************ Program the OTP's *********************/
#if PERFORM_OTP_LOCKING
 	switch( pFlashP->FlashType )
 	{
     case TYAX_FLASH:
     case SIBLEY_FLASH:
	 {
	  // Check that protection register is locked
	  Retval = ReadProtectionLockRegister(pFlashP, &Lock0, &Lock1);
	  CheckErrorReturn(Retval);

	  // Read protection data
	  Retval = ReadProtectionRegisterData(pFlashP, ProtectionRegister0, &ProtectionDataRead[0]);
	  Retval = ReadProtectionRegisterData(pFlashP, ProtectionRegister1, &ProtectionDataRead[4]);
	  Retval = ReadProtectionRegisterData(pFlashP, ProtectionRegister2, &ProtectionDataRead[12]);
	  CheckErrorReturn(Retval);

	  // Program protection data
	  Retval = ProgramProtectionReg(pFlashP,  ProtectionRegister0, (UINT16_T *)&OEMKeyDataC[0], 4);
	  Retval = ProgramProtectionReg(pFlashP,  ProtectionRegister1, (UINT16_T *)&OEMKeyDataC[2], 6);
	  CheckErrorReturn(Retval);
    if (JtagKeyFound == TRUE)
    {
	  // Now program JTAG key hash into registers 1 and 2
	  WordData = (0x0000FFFF & (JTAGKeyDataC[0]));
	  Retval = ProgramProtectionRegisterWord(pFlashP,  ProtectionRegister1, WordData, 6 );
	  WordData = (0x0000FFFF & ((JTAGKeyDataC[0])>>16));
	  Retval = ProgramProtectionRegisterWord(pFlashP,  ProtectionRegister1, WordData, 7 );
	  Retval = ProgramProtectionReg(pFlashP,  ProtectionRegister2, (UINT16_T *)&JTAGKeyDataC[1], 8);
	  CheckErrorReturn(Retval);
	}
 	  #if PERFORM_OTP_LOCKING
		// Lock the protection registers 0 and 1
		Retval = LockProtectionRegister(pFlashP, ProtectionRegister0);
		Retval = LockProtectionRegister(pFlashP, ProtectionRegister1);
		Retval = LockProtectionRegister(pFlashP, ProtectionRegister2);

	   CheckErrorReturn(Retval);
	  #endif // PERFORM_OTP_LOCKING

	  // Check that protection register is locked
	  Retval = ReadProtectionLockRegister(pFlashP, &Lock0, &Lock1);
	  CheckErrorReturn(Retval);

	  // Read back data
	  Retval = ReadProtectionRegisterData(pFlashP, ProtectionRegister0, &ProtectionDataRead[0]);
	  Retval = ReadProtectionRegisterData(pFlashP, ProtectionRegister1, &ProtectionDataRead[4]);
	  Retval = ReadProtectionRegisterData(pFlashP, ProtectionRegister2, &ProtectionDataRead[12]);
	  return Retval;
	 }
#if ONENAND_CODE
	 case OneNAND:
     {
	  // Write the Platform Keys 160 * 2 bits
/*	  Retval = ONENAND_WriteOtpArea(0, (UINT8_T *) KeyDataC, WordLengthOf_SHA1*4*2);
	  CheckErrorReturn(Retval);
 	  #if PERFORM_OTP_LOCKING
	   Retval = ONENAND_PermanentProtectOtpArea();
 	  #endif
	  return Retval;*/
	 }
#endif

#if MDOC_CODE
	 case MSYS_DOC_FLASH:  // Write out to MSYS MDOC
	 {
	  // Write the Platform Keys 160 bits
	  Retval = mDOC_WriteOTPArea((void *) KeyDataC, WordLengthOf_SHA1*4*2);
	  return Retval;
	 }
#endif
	 //Device does not have an OTP
	 default:
	  break;
	}
#endif // PERFORM_OTP_LOCKING
  return NoError;
}

/* This function will Bind the TIM platform keys and flash settings */
UINT_T AutoBindPlatform (pTIM pTIM_h, P_FlashProperties_T pFlashP, UINT_T PlatformSettings)
{
	// Declerations
	UINT_T 		Retval = NoError;
#if defined (PERFORM_FUSE_PROGRAM_WA)
	UINT32_T 	PVCR_reg_val;
	volatile 	UINT32_T  *PVCR_addr = 0x40F50100;


	// Store the PVCR value
	PVCR_reg_val = *PVCR_addr;

	// Change the External device address to dummy
	*PVCR_addr = (PVCR_reg_val | 0x40);

#endif
	// Device Key
	#if DEVICE_KEY
 	 AddMessage((UINT8_T*) ("*** Device Key...\0"));
	 Retval = ProgramOBMImage(pTIM_h, &pFlashP[0]);
	#endif

	CheckErrorReturn(Retval);

	// Program Platform configuration Fuses
	#if PERFORM_FUSE_PROGRAM
 	 AddMessage((UINT8_T*) ("*** Program Fuses...\0"));
	 Retval = ProgramPlatformFuses(&(pTIM_h->pConsTIM->FlashInfo), PlatformSettings);
 	#endif

#if defined (PERFORM_FUSE_PROGRAM_WA)
	// Restore the PVCR value
	*PVCR_addr = PVCR_reg_val;
#endif
	return Retval;
}

