#ifndef PRDCFG_GENERAL_H
#define PRDCFG_GENERAL_H

/*******************************************************************************
Copyright (C), 200{x}, Borqs Tech. Co., Ltd.
File Name: 
    PRDCFG_General.h
Description: 
 
History:  
    Date          Author                         Comments
    -----------    ---------------------------    --------------------
    2008-08-13      Bai yu                                          Created
*******************************************************************************/

/*==================================================================================================
                                           CONSTANTS
 ==================================================================================================*/
#define NORMAL_CFG_ELEMENT_LENGTH_MAX    1024   


/*==================================================================================================
                                           TYPEDEF
==================================================================================================*/

typedef int PRDCFG_ELEMENT_ID;
typedef int PRDCFG_ELEMENT_ATTRIBUTES;
typedef int PRDCFG_ELEMENT_OFFSET;


typedef struct
{
    unsigned char * nvm_file_name;
}PRDCFG_NVM_FILE_NAME;

typedef struct
{
    


}PRDCFG_SETTING_INFO;

/*==================================================================================================
                                           ENUMS
==================================================================================================*/

/*****************************   Error type ***************************/
typedef enum
{
    PRDCFG_SUCCESS,
    PRDCFG_FAIL,
    PRDCFG_OPEN_FILE_FAIL,
    PRDCFG_VERSION_MISMATCH,
    PRDCFG_INVALID_IN_PARA,
    PRDCFG_INVALID_PREAMBLE_BIT,
    PRDCFG_INVALID_EPILOGUE_BIT,
    PRDCFG_INVALID_CHECKSUM,
    PRDCFG_UNKOWN_ERR, 
}PRDCFG_ERR_TYPE;

/*****************************   Customer level  ***************************/
typedef enum
{
    PRDCFG_NORMAL_CUSTOMER ,
    PRDCFG_SUPER_CUSTOMER,
    PRDCFG_DEFAULT_CUSTOMER = PRDCFG_NORMAL_CUSTOMER,
}PRDCFG_CUSTOMER_LEVEL;

/***************************** Element group ID   ***************************/
enum
{
    PRDCFG_VERSION_INFO_GROUP_ID = 0x0,
    PRDCFG_BIT_CFG_GROUP_ID = 0x01,
    PRDCFG_CFG_GROUP_ID = 0x1001,
    PRDCFG_PHASING_FILE_GROUP_ID = 0x20001,
    PRDCFG_OTP_DATA_GROUP_ID = 0x40001,
};

/***************************** Element ID   ***************************/
enum
{
#if 0
    PRDCFG_VERSION_INFO = PRDCFG_VERSION_INFO_GROUP_ID,

   /****************************** Put Bit config element here  ***************************************/
    PRDCFG_EEH_LIVETIME_REPORT_EN = PRDCFG_BIT_CFG_GROUP_ID,
    PRDCFG_EFEM_FEATURE_EN,
    PRDCFG_EFEM_FEATURE_EN_DEFAULT,
    PRDCFG_DIAG_LAUNCH_EN,
    PRDCFG_DIAG_LAUNCH_EN_DEGAULT,
    PRDCFG_BIT_CFG_ELEMENT_ID_MAX,

    /******************************  Put config element here *****************************************/
    PRDCFG_EEH_LIVETIME_PERIOD = PRDCFG_CFG_GROUP_ID,
    PRDCFG_DIAG_EXT_INTERFACE_SETTING,
    PRDCFG_CFG_ELEMENT_ID_MAX,

    /******************************  For phasing data ***********************************************/
#endif
    PRDCFG_PHASING_HEADER = 0, 
    PRDCFG_PHASING_SETTING_INFO,  // not used now, reserved
    PRDCFG_PHASING_RF_PARA,
    PRDCFG_PHASING_VOLTAGE,
    PRDCFG_PHASING_TEMPERATURE,
    PRDCFG_PHASING_BT_ADDR,
    PRDCFG_PHASING_FACTORY_ID,
    PRDCFG_PHASING_TEST_INFO,
    PRDCFG_PHASING_MAT_FLAG,
    PRDCFG_PHASING_ENG_MODE_FLAG,
    PRDCFG_PHASING_ELEMENT_ID_MAX,

#if 0
    /******************************  For OTP data  ***********************************************/    
    PRDCFG_OTP_IMEI = PRDCFG_OTP_DATA_GROUP_ID,
    PRDCFG_OTP_ELEMENT_ID_MAX,
#endif

};

/*==================================================================================================
                                           FUNCTION PROTOTYPES
==================================================================================================*/ 

#ifdef CACHE_PRDCFG_IMG
PRDCFG_ERR_TYPE PRDCFG_Init(const int ele_num, const PRDCFG_ELEMENT_ID* ele_listconst, PRDCFG_CUSTOMER_LEVEL customer_level);
#else
PRDCFG_ERR_TYPE PRDCFG_Init(const PRDCFG_CUSTOMER_LEVEL customer_level);
#endif

int PRDCFG_Get_element_length(const PRDCFG_ELEMENT_ID id);
int PRDCFG_Get_element_access_attribute(const PRDCFG_ELEMENT_ID id);

// App who call this API should ensure buf size enough to contain all element data, or may overflow
// Recommend get element length before call this API
PRDCFG_ERR_TYPE PRDCFG_Get_config_element_value(const PRDCFG_ELEMENT_ID id, unsigned char * buf);

// App who call this API should ensure buf size enough to contain all element data, or may overflow
// Recommend get element length before call this API
PRDCFG_ERR_TYPE PRDCFG_Set_config_element_value(const PRDCFG_ELEMENT_ID id,const unsigned char* buf);

PRDCFG_ERR_TYPE PRDCFG_Get_bit_config_element_value(const PRDCFG_ELEMENT_ID id, unsigned char* buf);
PRDCFG_ERR_TYPE PRDCFG_Set_bit_config_element_value(const PRDCFG_ELEMENT_ID id, const unsigned char* buf);

PRDCFG_ERR_TYPE PRDCFG_Get_whole_bit_config_group(unsigned char* buf);

PRDCFG_ERR_TYPE PRDCFG_Load_phasing_file(const PRDCFG_ELEMENT_ID id, unsigned char* buf);
PRDCFG_ERR_TYPE PRDCFG_Store_phasing_file(const PRDCFG_ELEMENT_ID id, const unsigned char* buf );

PRDCFG_ERR_TYPE PRDCFG_Store_RF_data(const unsigned char* path);
PRDCFG_ERR_TYPE PRDCFG_Load_RF_data(const unsigned char* path);

PRDCFG_ERR_TYPE PRDCFG_Master_Clear_Restore();
PRDCFG_ERR_TYPE PRDCFG_Master_Reset_Restore();

PRDCFG_ERR_TYPE PRDCFG_Get_Reliable_Data(unsigned char* buf);
PRDCFG_ERR_TYPE PRDCFG_Set_Reliable_Data(unsigned char* buf);

PRDCFG_ERR_TYPE PRDCFG_Get_otp_element_value(const PRDCFG_ELEMENT_ID id, unsigned char* buf);
PRDCFG_ERR_TYPE PRDCFG_Set_otp_element_value(const PRDCFG_ELEMENT_ID id,unsigned char* buf);
PRDCFG_ERR_TYPE PRDCFG_Lock_otp_element(const PRDCFG_ELEMENT_ID id);
#endif


