#include "obm_micco.h"
#include "obm_pxa930_linux.h"
#include "misc.h"

void nor_i2c_read( UINT8_T addr, UINT8_T* data )
{
	UINT16_T data_size, cmd_size;
	UINT8_T dataPtr[2], already_display = FALSE;
	I2C_ReturnCode rc;

	data_size = 1;
	cmd_size = 1;
	dataPtr[0] = addr;           // Register address in Micco to be update
	dataPtr[1] = 0x0;            // Data area to be read

	do
	{
		rc = I2CMasterReceiveDataDirect ( &dataPtr[0],
						cmd_size,
						PMC_I2C_WRITE_SLAVE_ADDRESS,
						&dataPtr[1],
						data_size,
						PMC_I2C_READ_SLAVE_ADDRESS );
        if(rc != I2C_RC_OK)
        {
            I2CConfigure();
            AddMessage("-i2c_reset");
            Delay_ms(200);
        }
	}
	while ( rc != I2C_RC_OK );

	*data = dataPtr[1];
}

void nor_i2c_write( UINT8_T addr, UINT8_T data )
{
	UINT16_T data_size , cmd_size;
	UINT8_T dataPtr[2] , already_display = FALSE;
	I2C_ReturnCode rc;

	dataPtr[0] = addr;
	dataPtr[1] = data;
	data_size = 2;
	already_display = FALSE;

	do
	{
		rc = I2CMasterSendDataDirect ( dataPtr,
						data_size,
						PMC_I2C_WRITE_SLAVE_ADDRESS );
	}
	while ( rc != I2C_RC_OK );
}

int micco_read(UINT8_T reg, UINT8_T *pval)
{
    nor_i2c_read(reg,pval);
    return 0;
}

int micco_write(UINT8_T reg, UINT8_T val)
{
    nor_i2c_write(reg,val);
    return 0;
}

#ifndef DDR_WORKAROUND
void start_calc_time(void) {}
void end_calc_time(void) {}
/* micco_power_module[] should be consistent with enum
 * in include/asm-arm/arch-pxa/pxa3xx_pmic.h */
static struct power_supply_module miccoB0_power_modules[] = {
	/* {command,		power_module}, */
	{VCC_CORE,		BUCK1},
	{VCC_SRAM,		LDO2},
	{VCC_MVT,		LDO1},
	{VCC_3V_APPS,		LDO3},
	{VCC_SDIO,		LDO13},
	{VCC_CAMERA_ANA,	LDO9},
	{VCC_USB,		LDO5},
	{VCC_LCD,		LDO3},
	{VCC_TSI,		0},
	{VCC_CAMERA_IO,		0},
	{VCC_1P8V,		LDO4},
	{VCC_MEM,		BUCK3},
	{HDMI_TX,		0},
	{TECH_3V,		0},
	{TECH_1P8V,		0},
	{VCC_BT,		LDO10},
};

/* micco_power_module[] should be consistent with enum
 *  * in include/asm-arm/arch-pxa/pxa3xx_pmic.h */
static struct power_supply_module miccoEA_power_modules[] = {
	/* {command,		power_module}, */
	{VCC_CORE,		BUCK1},
	{VCC_SRAM,		BUCK2},
	{VCC_MVT,		LDO1},
	{VCC_3V_APPS,		LDO3},
	{VCC_SDIO,		LDO13},
	{VCC_CAMERA_ANA,	LDO9},
	{VCC_USB,		LDO5},
	{VCC_LCD,		LDO3},
	{VCC_TSI,		0},
	{VCC_CAMERA_IO,		0},
	{VCC_1P8V,		LDO4},
	{VCC_MEM,		LDO2},
	{HDMI_TX,		0},
	{TECH_3V,		0},
	{TECH_1P8V,		0},
	{VCC_BT,		LDO10},
};

static struct power_chip micco_chips[] = {
	{MICCO_B0_ID,	"miccoB0",	miccoB0_power_modules},
	{MICCO_EA_ID,	"miccoEA",	miccoEA_power_modules},
	{MICCO_EB_ID,	"miccoEB",	miccoEA_power_modules},
	{0,	NULL,		NULL},
};
static struct power_supply_module *micco_power_module;

static int get_power_supply_module(int cmd)
{
	int command, power_module;

	if (cmd < VCC_CORE || cmd >= MAX_CMD) {
		printk(KERN_WARNING "input wrong command: %d\n", cmd);
		return -EINVAL;
	}
	command = micco_power_module[cmd - VCC_CORE].command;
	if (command != cmd) {
		printk(KERN_WARNING "micco_power_module[] is error: %d\n", cmd);
		return -EINVAL;
	}
	power_module = micco_power_module[cmd - VCC_CORE].power_module;
	if (power_module == 0) {
		printk(KERN_WARNING "the command not supported: %d\n", cmd);
		return -EINVAL;
	}

	return power_module;
}
static int set_micco_voltage(int cmd, int mv)
{
	int power_module, status = 0;
	u8 val;

	power_module = get_power_supply_module(cmd);

	start_calc_time();
	switch (power_module) {
	case BUCK1:
		if ((mv < MICCO_VBUCK1_BASE) || (mv > MICCO_VBUCK1_MAX))
			return -EINVAL;
		status = micco_read(MICCO_BUCK1_DVC2, &val);
		if (status)
			return status;

		val &= 0xe0;
		val = val | ((mv - MICCO_VBUCK1_BASE) / MICCO_VBUCK1_STEP);
		status = micco_write(MICCO_BUCK1_DVC2, val);
		/* Update the voltage output. Otherwise, voltage won't change */
		status = micco_write(MICCO_VCC1, 0x33);
		break;
	case BUCK2: /* TODO */
		return 0;
	case BUCK3: /* TODO */
		return 0;
	case LDO1:
		if ((mv < MICCO_VLDO1_BASE) || (mv > MICCO_VLDO1_MAX))
			return -EINVAL;
		status = micco_read(MICCO_LDO1_MDTV1, &val);
		if (status)
			return status;

		val &= 0xf0;
		val = val | ((mv - MICCO_VLDO1_BASE) / MICCO_VLDO1_STEP);
		status = micco_write(MICCO_LDO1_MDTV1, val);
		break;
	case LDO2:
		if ((mv < MICCO_VSRAM_BASE) || mv > (MICCO_VSRAM_MAX))
			return -EINVAL;
		status = micco_read(MICCO_SRAM_DVC2, &val);
		if (status)
			return status;

		val &= 0xe0;
		val = val | ((mv - MICCO_VSRAM_BASE) / MICCO_VSRAM_STEP);
		status = micco_write(MICCO_SRAM_DVC2, val);
		/* Update the voltage output. Otherwise, voltage won't change */
		status = micco_write(MICCO_VCC1, 0x33);
		break;
	case LDO3:
		if ((mv < MICCO_VLDO3_BASE) || (mv > MICCO_VLDO3_MAX))
			return -EINVAL;

		status = micco_read(MICCO_LDO643, &val);
		if (status)
			return status;

		val &= 0xf0;
		val = val | ((mv - MICCO_VLDO3_BASE) / MICCO_VLDO3_STEP);
		status = micco_write(MICCO_LDO643, val);
		break;
	case LDO4:
		if ((mv != 2900) && (mv != 1800))
			return -EINVAL;
		status = micco_read(MICCO_LDO643, &val);
		if (2900 == mv)
			val |= 0x10;
		else
			val &= ~0x10;
		status = micco_write(MICCO_LDO643, val);
		break;
	case LDO5: /* TODO */
		return 0;
	case LDO6:
		if ((mv < MICCO_VLDO6_BASE) || (mv > MICCO_VLDO6_MAX))
			return -EINVAL;

		status = micco_read(MICCO_LDO643, &val);
		if (status)
			return status;

		val &= 0x1f;
		val |= ((mv - MICCO_VLDO6_BASE) / MICCO_VLDO6_STEP) << 5;
		status = micco_write(MICCO_LDO643, val);
		break;
	case LDO7:
		printk(KERN_WARNING "power supply module TODO: %d\n", cmd);
		return -EINVAL;
	case LDO8:
		printk(KERN_WARNING "power supply module TODO: %d\n", cmd);
		return -EINVAL;
	case LDO9:
		if ( mv == 0 )
		{
			/* disable LDO9 */
			status = micco_read(MICCO_APP_OVER2, &val);
			if (status)
				return status;
			val &= ~MICCO_LDO9_ENABLE;
			micco_write(MICCO_APP_OVER2, val);
			break;
		}

		if ((mv < MICCO_VLDO9_BASE) || (mv > MICCO_VLDO9_MAX))
			return -EINVAL;
		status = micco_read(MICCO_LDO987, &val);
		if (status)
			return status;

		val &= 0x1f;
		val |= ((mv - MICCO_VLDO9_BASE) / MICCO_VLDO9_STEP) << 5;
		status = micco_write(MICCO_LDO987, val);

		/* enable LDO9 */
		status = micco_read(MICCO_APP_OVER2, &val);
		if (status)
			return status;
		val |= MICCO_LDO9_ENABLE;
		micco_write(MICCO_APP_OVER2, val);
		break;
	case LDO10:
		if ((mv < MICCO_VLDO10_BASE) || (mv > MICCO_VLDO10_MAX))
			return -EINVAL;

		status = micco_read(MICCO_LDO1110, &val);
		if (status)
			return status;

		val &= 0xf8;
		val = val | ((mv - MICCO_VLDO10_BASE) / MICCO_VLDO10_STEP);
		status = micco_write(MICCO_LDO1110, val);
		break;
	case LDO11:
		if ((mv < MICCO_VLDO11_BASE) || (mv > MICCO_VLDO11_MAX))
			return -EINVAL;

		status = micco_read(MICCO_LDO1110, &val);
		if (status)
			return status;

		val &= 0x0f;
		val |= ((mv - MICCO_VLDO11_BASE) / MICCO_VLDO11_STEP) << 4;
		status = micco_write(MICCO_LDO1110, val);
		break;
	case LDO12:
		if ((mv < MICCO_VLDO12_BASE) || (mv > MICCO_VLDO12_MAX))
			return -EINVAL;


		status = micco_read(MICCO_LDO12, &val);
		if (status)
			return status;

		val &= 0xf8;
		if (mv > MICCO_VLDO12_BASE1) {
			val = val | ((mv - MICCO_VLDO12_BASE1) /	\
				MICCO_VLDO12_STEP);
			val |= 0x08;
		} else {
			val = val | ((mv - MICCO_VLDO12_BASE) /		\
				MICCO_VLDO12_STEP);
			val &= ~0x08;
		}
		status = micco_write(MICCO_LDO12, val);
		break;
	case LDO13:
		if ((mv < MICCO_VLDO13_BASE) || (mv > MICCO_VLDO13_MAX))
			return -EINVAL;

		status = micco_read(MICCO_LDO13, &val);
		if (status)
			return status;

		val &= 0xf0;
		val |= ((mv - MICCO_VLDO13_BASE) / MICCO_VLDO13_STEP);
		status = micco_write(MICCO_LDO13, val);
		break;
	case LDO14:
	case LDO15:
	case USB_OTG: /* TODO */
	case LDO_GPADC: /* TODO */
	case LDO_AUDIO: /* TODO */
	case LDO_PMCORE: /* TODO */
	case LDO_BBAT: /* TODO */
		printk(KERN_WARNING "power supply module TODO: %d\n", cmd);
		return -EINVAL;
	default:
		printk(KERN_INFO "error command\n");
		return -EINVAL;
	}

	if (status != 0)
		return status;

	end_calc_time();
	return status;
}

static int enable_micco_voltage(int cmd, int enable)
{
	int power_module, status = 0;
	u8 val;
	power_module = get_power_supply_module(cmd);
	switch (power_module) {
	case BUCK1:
		status = micco_read(MICCO_OVER1, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_BUCK1_ENABLE;
		else
			val &= ~MICCO_BUCK1_ENABLE;
		micco_write(MICCO_OVER1, val);
		break;
	case BUCK2:
		status = micco_read(MICCO_OVER1, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_BUCK2_ENABLE;
		else
			val &= ~MICCO_BUCK2_ENABLE;
		micco_write(MICCO_OVER1, val);
		break;
	case BUCK3:
		status = micco_read(MICCO_APP_OVER3, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_BUCK3_ENABLE;
		else
			val &= ~MICCO_BUCK3_ENABLE;
		micco_write(MICCO_APP_OVER3, val);
		break;
	case LDO1:
		status = micco_read(MICCO_APP_OVER3, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO1_ENABLE;
		else
			val &= ~MICCO_LDO1_ENABLE;
		micco_write(MICCO_APP_OVER3, val);
		break;
	case LDO2:
		status = micco_read(MICCO_OVER1, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO2_ENABLE;
		else
			val &= ~MICCO_LDO2_ENABLE;
		micco_write(MICCO_OVER1, val);
		break;
	case LDO3:
		status = micco_read(MICCO_APP_OVER3, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO3_ENABLE;
		else
			val &= ~MICCO_LDO3_ENABLE;
		micco_write(MICCO_APP_OVER3, val);
		break;
	case LDO4:
		status = micco_read(MICCO_APP_OVER3, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO4_ENABLE;
		else
			val &= ~MICCO_LDO4_ENABLE;
		micco_write(MICCO_APP_OVER3, val);
		break;
	case LDO5:
		status = micco_read(MICCO_APP_OVER3, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO5_ENABLE;
		else
			val &= ~MICCO_LDO5_ENABLE;
		micco_write(MICCO_APP_OVER3, val);
		break;
	case LDO6:
		status = micco_read(MICCO_APP_OVER2, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO6_ENABLE;
		else
			val &= ~MICCO_LDO6_ENABLE;
		micco_write(MICCO_APP_OVER2, val);
		break;
	case LDO7:
		status = micco_read(MICCO_APP_OVER2, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO7_ENABLE;
		else
			val &= ~MICCO_LDO7_ENABLE;
		micco_write(MICCO_APP_OVER2, val);
		break;
	case LDO8:
		status = micco_read(MICCO_APP_OVER2, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO8_ENABLE;
		else
			val &= ~MICCO_LDO8_ENABLE;
		micco_write(MICCO_APP_OVER2, val);
		break;
	case LDO9:
		status = micco_read(MICCO_APP_OVER2, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO9_ENABLE;
		else
			val &= ~MICCO_LDO9_ENABLE;
		micco_write(MICCO_APP_OVER2, val);
		break;
	case LDO10:
		status = micco_read(MICCO_APP_OVER2, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO10_ENABLE;
		else
			val &= ~MICCO_LDO10_ENABLE;
		micco_write(MICCO_APP_OVER2, val);
		break;
	case LDO11:
		status = micco_read(MICCO_APP_OVER2, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO11_ENABLE;
		else
			val &= ~MICCO_LDO11_ENABLE;
		micco_write(MICCO_APP_OVER2, val);
		break;
	case LDO12:
		status = micco_read(MICCO_APP_OVER2, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO12_ENABLE;
		else
			val &= ~MICCO_LDO12_ENABLE;
		micco_write(MICCO_APP_OVER2, val);
		break;
	case LDO13:
		status = micco_read(MICCO_APP_OVER3, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO13_ENABLE;
		else
			val &= ~MICCO_LDO13_ENABLE;
		micco_write(MICCO_APP_OVER3, val);
		break;
	case LDO14:
	default:
		printk(KERN_INFO "error command\n");
		return -EINVAL;
	}

	return status;
}

int pxa3xx_pmic_enable_voltage(int cmd, int enable)
{
    //Leith
    //pick up from drivers/i2c/chips/micco.c->micco_pmic_ops
    return enable_micco_voltage(cmd, enable);
}

int pxa3xx_pmic_set_voltage(int cmd, int val)
{
    return set_micco_voltage(cmd, val);
}

int micco_probe(void)
{
    int ret;
	int chip_id = 0;
	int i;

	micco_read(MICCO_CHIP_ID, &chip_id);
    for (i = 0; micco_chips[i].chip_id != 0; i++) {
		if (micco_chips[i].chip_id == chip_id) {
			micco_power_module = \
				micco_chips[i].power_supply_modules;
			break;
		}
	}
	if (micco_chips[i].chip_id == 0)
	{/* if not found a match version, then use default */
        poweron_reason_print("-not find a match micco version,detected chip_id=", chip_id, 1);
		micco_power_module = micco_chips[0].power_supply_modules;
		//panic("This Micco chip is not supported");
	}else poweron_reason_print("-micco version = ", micco_chips[i].chip_id, 1);
}
#endif

