#include "obm_pxa_lcd.h"
#include "obm_pxa_ssp.h"
#include "platformconfig.h"
#include "misc.h"
#include "obm_micco.h"

#define CKENA		__REG(0x4134000C)	/* A Clock Enable Register */
#define CKENB		__REG(0x41340010)	/* B Clock Enable Register */
#define CKENB		__REG(0x41340010)	/* B Clock Enable Register */
#define CKENC		__REG(0x41340024)	/* C Clock Enable Register */


#define udelay(x) Delay_us(x)
#define mdelay(x) Delay_ms(x)


/* use bit 30, 31 as the indicator of command parameter number */
#define CMD0(x)		((0x00000000) | ((x)))
#define DAT0(x)		((0x00000100) | ((x)))
#define CMD1(x, x1)	((0x40000000) | ((x) << 9) | 0x100 | (x1))
#define CMD2(x, x1, x2)	((0x80000000) | ((x) << 18) | 0x20000 |\
			((x1) << 9) | 0x100 | (x2))
#define SPARK_LCD_DSS 	9 	//Data size select for LCD spi
static uint32_t power_on[] = {
	CMD0(0x11),
};


static uint32_t disp_on[]={
	CMD0(0x29),
};

static uint32_t disp_off[]={
	CMD0(0x28),
};

static uint32_t power_off[]={
	CMD0(0x10),
};


#ifdef SPARK
static uint32_t initialize[] = {
	CMD0(0xFC),
	DAT0(0x00),
	DAT0(0x00),
	DAT0(0x08),

	CMD0(0xD1),
	DAT0(0xA4),
	//DAT0(0x98),
	DAT0(0x9E),
	DAT0(0x86),

	CMD0(0x3A),
	DAT0(0x60),
};
#else 
#ifdef BLAZE
static uint32_t initialize[] = {
	CMD0(0xFC),
	DAT0(0x00),
	DAT0(0x00),
	DAT0(0x00),
	DAT0(0x40),

	CMD0(0xD1),
	DAT0(0x84),
	DAT0(0x98),
	DAT0(0x86),
	DAT0(0x86),

	CMD0(0x3A),
	DAT0(0x60),

	CMD0(0xF3),
	DAT0(0xB0),

	CMD0(0xF2),
	DAT0(0x00),
	DAT0(0x09),
};
#else
#ifdef BENZINA
static uint32_t initialize[] = {
	CMD0(0xFC),
	DAT0(0x00),
	DAT0(0x00),
	DAT0(0x00),
	DAT0(0x40),

	CMD0(0xD1),
	DAT0(0x84),
	DAT0(0x98),
	DAT0(0x86),
	DAT0(0x86),

	CMD0(0x3A),
	DAT0(0x60),

	CMD0(0xF3),
	DAT0(0xB0),

	CMD0(0xF2),
	DAT0(0x00),
	DAT0(0x09),
};
#endif
#endif
#endif



static void clk_pxa3xx_cken_enable(struct clk *clk)
{
	unsigned long mask = 1ul << (clk->cken & 0x1f);

	local_irq_disable();

	if (clk->cken < 32)
		CKENA |= mask;
    else if (clk->cken < 64)
		CKENB |= mask;
	else
		CKENC |= mask;
	local_irq_enable();
}

static void clk_pxa3xx_cken_disable(struct clk *clk)
{
	unsigned long mask = 1ul << (clk->cken & 0x1f);

	local_irq_disable();

	if (clk->cken < 32)
		CKENA &= ~mask;
    else if (clk->cken < 64)
		CKENB |= mask;
	else
		CKENC |= mask;

	local_irq_enable();
}

int clk_enable(struct clk *clk)
{
	unsigned long flags;

	spin_lock_irqsave(&clocks_lock, flags);
	if (clk->enabled++ == 0)
		clk->ops->enable(clk);
	spin_unlock_irqrestore(&clocks_lock, flags);

	if (clk->delay)
//		udelay(clk->delay);
        Delay_us(clk->delay);
	return 0;
}

void clk_disable(struct clk *clk)
{
	unsigned long flags;

	if (clk->enabled == 0) {
		WARN_ON(clk->enabled == 0);
		return;
	}

	spin_lock_irqsave(&clocks_lock, flags);
	if (--clk->enabled == 0)
		clk->ops->disable(clk);
	spin_unlock_irqrestore(&clocks_lock, flags);
}

static unsigned char hss_mult[4] = { 8, 12, 16, 0 };
static unsigned long clk_pxa3xx_hsio_getrate(struct clk *clk)
{
	unsigned long acsr;
	unsigned int hss, hsio_clk;

	acsr = ACSR;

	hss = (acsr >> 14) & 0x3;
	hsio_clk = (acsr & ACCR_D0CS) ? RO_CLK : hss_mult[hss] * BASE_CLK;

	return hsio_clk;
}

#ifdef CONFIG_FB_PXA
#if defined(CONFIG_FB_JADES_HVGA)
/* use bit 30, 31 as the indicator of command parameter number */
#define CMD0(x)         ((0x00000000) | ((x) << 8))
#define CMD1(x, x1)     ((0x40000000) | ((x) << 8) | (x1))
#define CMD2(x, x1, x2) ((0x80000000) | ((x) << 16) | ((x1) << 8) | (x2))

static uint32_t lcd_panel_on[] = {
};
static uint32_t lcd_panel_off[] = {
        CMD1(0x07, 0xD9),
};
static uint32_t lcd_panel_config[] = {
	CMD1(0x08, 0x01),
	CMD1(0x08, 0x00),
#ifndef DDR_WORKAROUND
	CMD1(0x02, 0x10), /*MARVELL version CMD1(0x02, 0x00)*/
	CMD1(0x03, 0x02), /*MARVELL version CMD1(0x03, 0x04)*/
	CMD1(0x04, 0x0F), /*MARVELL version CMD1(0x04, 0x0E)*/
#else
	CMD1(0x02, 0x00),
	CMD1(0x03, 0x04),
	CMD1(0x04, 0x0E),
#endif
	CMD1(0x09, 0x03),
	CMD1(0x0B, 0x08),
	CMD1(0x0C, 0x53),
	CMD1(0x0D, 0x01),
	CMD1(0x0E, 0xE0),
	CMD1(0x0F, 0x01),
	CMD1(0x10, 0x58),
	CMD1(0x20, 0x1E),
	CMD1(0x21, 0x0A),
	CMD1(0x22, 0x0A),
	CMD1(0x23, 0x1E),
	CMD1(0x25, 0x32),
	CMD1(0x26, 0x00),
	CMD1(0x27, 0xAC),
	CMD1(0x2A, 0xA8),
	CMD1(0x2D, 0x05),
	CMD1(0x2E, 0xAA),
	CMD1(0x2F, 0xFF),
	CMD1(0x30, 0x86),
	CMD1(0x31, 0xD8),
	CMD1(0x32, 0xB4),
	CMD1(0x33, 0xE6),
	CMD1(0x34, 0x1C),
	CMD1(0x35, 0x3A),
	CMD1(0x36, 0x62),
	CMD1(0x37, 0xDA),
	CMD1(0x38, 0x33),
	CMD1(0x39, 0x83),
	CMD1(0x3A, 0xB9),
	CMD1(0x3B, 0xFF),
	CMD1(0x07, 0xC9),
};

static void spark_backlight_power( int on, int intensity)
{
    if( on) {
        CKENB |= 1<<(CKEN_PWM0-32);
        PWM_CTRL0 = 0x8;
        PWM_PWDUTY0 = intensity;
        PWM_PERVAL0 = 0xfe;
    } else {
        PWM_CTRL0 = 0x0;
        PWM_PWDUTY0 = 0x0;
        PWM_PERVAL0 = 0x0;
        CKENB &= ~(1<<(CKEN_PWM0-32));
    }
}


#define PXAFB_RESET_BY_GPIO
void lcd_panel_reset(void)
{
//        printk("%s\n",__FUNCTION__);
#if defined(PXAFB_RESET_BY_GPIO)
        gpio_direction_output(mfp_to_gpio(MFP_PIN_GPIO21), 0);
//        udelay(100);
        Delay_us(100);
        gpio_direction_output(mfp_to_gpio(MFP_PIN_GPIO21), 1);
//        udelay(20);
#ifndef DDR_WORKARUOND
        Delay_ms(20);	/*MARVELL Delay_us(20);*/
#else
		Delay_us(20)
#endif
#else
        /* start chucking commands at the lcd */
	ssp_send_cmd(ARRAY_AND_SIZE(lcd_panel_reset));
#endif
}


static void spark_lcd_power(int on, struct fb_var_screeninfo *var)
{
	static int pin_requested = 0, mfpr_val;

	if( on)
	{
		gpio_direction_output(mfp_to_gpio(MFP_CFG_PIN(LCD_RESET_N)), 0);
        udelay(100);
		gpio_direction_output(mfp_to_gpio(MFP_CFG_PIN(LCD_RESET_N)), 1);
		mdelay(1);
		ssp_send_cmd(ARRAY_AND_SIZE(power_on));
		ssp_send_cmd(ARRAY_AND_SIZE(initialize));
		ssp_send_cmd(ARRAY_AND_SIZE(disp_on));
		mdelay(100);
	}
	else
	{
		ssp_send_cmd(ARRAY_AND_SIZE(disp_off));
		ssp_send_cmd(ARRAY_AND_SIZE(power_off));
		gpio_direction_output(MFP_PIN_GPIO169, 0);

	}
}
#endif
#endif

static const struct clkops clk_pxa3xx_hsio_ops = {
	.enable		= clk_pxa3xx_cken_enable,
	.disable	= clk_pxa3xx_cken_disable,
	.getrate	= clk_pxa3xx_hsio_getrate,
};

static struct clk pxa3xx_clks[] = {
	PXA3xx_CK("LCDCLK", LCD,    &clk_pxa3xx_hsio_ops, 0),//&pxa_device_fb.dev),
};

static struct pxafb_mode_info novatek_l1N304_modes[] = {
	[0] = {
		//.flags			= PXAFB_AUTO_REFRESH,
		.pixclock		= 62500,
		.xres			= 640,
		.yres			= 360,
		.bpp			= 32,
		.hsync_len		= 2,
		.left_margin		= 4,
		.right_margin		= 24,
		.vsync_len		= 2,
		.upper_margin		= 3,
		.lower_margin		= 5,
		//		.sync			= 0,
		//		.update_framedata	= spark_lcd_update,
	},
};

static struct pxafb_mach_info tpo_hvga_info = {
    .modes			= novatek_l1N304_modes,
	.num_modes      	    = 1,
	.fixed_modes		    = 1,
	.lccr0                  = LCCR0_Act,
	.lccr3			= LCCR3_PixRsEdg | LCCR3_HSP | LCCR3_VSP,    
    .pxafb_lcd_power	= spark_lcd_power,
	.pxafb_backlight_power	= spark_backlight_power,
};

static void (*pxafb_backlight_power)(int,int);
static void (*pxafb_lcd_power)(int, struct fb_var_screeninfo *);
struct pxafb_info pxafb;
static u32 pseudo_palette[16];
static void pxafb_setmode(struct fb_var_screeninfo *var, struct pxafb_mode_info *mode);

static void pxafb_init_fbinfo_fill(struct pxafb_info *fbi)
{
    struct pxafb_mach_info *inf = &tpo_hvga_info;
    struct pxafb_mode_info *mode = &novatek_l1N304_modes;
    int i, smemlen;

    fbi->clk = &pxa3xx_clks[0];
    /* ensure the LCD clock is off */
	clk_enable(fbi->clk);
	clk_disable(fbi->clk);

    memcpy(fbi->fb.fix.id, PXA_NAME, sizeof(PXA_NAME));
    //strcpy(fbi->fb.fix.id, PXA_NAME);
    fbi->fb.fix.type	= FB_TYPE_PACKED_PIXELS;
	fbi->fb.fix.type_aux	= 0;
	fbi->fb.fix.xpanstep	= 0;
	fbi->fb.fix.ypanstep	= 1;
	fbi->fb.fix.ywrapstep	= 0;
	fbi->fb.fix.accel	= FB_ACCEL_NONE;

	fbi->fb.var.nonstd	= 0;
	fbi->fb.var.activate	= FB_ACTIVATE_NOW;
	fbi->fb.var.height	= -1;
	fbi->fb.var.width	= -1;
	fbi->fb.var.accel_flags	= 0;
	fbi->fb.var.vmode	= FB_VMODE_NONINTERLACED;

	fbi->fb.fbops		= 0;//&pxafb_ops;
	fbi->fb.flags		= FBINFO_DEFAULT|FBINFO_HWACCEL_YPAN;
	fbi->fb.node		= -1;

    fbi->fb.pseudo_palette = pseudo_palette;

	fbi->buffer_num = PXAFB_BUFFER_NUM;	/* BUFFER NUMBER FOR FLIP CACHE , declared in pxafb.h */
	fbi->buffer_index = 0;			/* set initial buffer start index*/
	fbi->bl_brightness = MAX_BRIGHTNESS/2;
	pxafb_setmode(&fbi->fb.var, mode);
	fbi->flags = mode->flags;
	fbi->wr_setup_time = mode->wr_setup_time;
	fbi->wr_pulse_width = mode->wr_pulse_width;
	fbi->rd_setup_time = mode->rd_setup_time;
	fbi->rd_pulse_width = mode->rd_pulse_width;
	fbi->op_hold_time = mode->op_hold_time;
	fbi->cmd_inh_time = mode->cmd_inh_time;
	fbi->sync_cnt = mode->sync_cnt;
	fbi->update_framedata = mode->update_framedata;

	fbi->cmap_inverse		= inf->cmap_inverse;
	fbi->cmap_static		= inf->cmap_static;

	fbi->lccr0			= inf->lccr0;
	fbi->lccr3			= inf->lccr3;
	fbi->lccr4			= inf->lccr4;
	fbi->lccr6			= inf->lccr6;
	fbi->state			= C_STARTUP;
	fbi->task_state			= (u_char)-1;

    for (i = 0; i < inf->num_modes; i++) {
		/* Only 18 and 19 bpp (maybe 24,25) should be set separately */
		if( (mode[i].bpp == 18) || (mode[i].bpp == 19) ||
		    (mode[i].bpp == 24) || (mode[i].bpp == 25) || mode[i].bpp == 32 )
			smemlen = 4;
		else
			smemlen = mode[i].bpp/8;

		smemlen *= mode[i].xres * mode[i].yres ;
	
		if (smemlen > fbi->fb.fix.smem_len)
			fbi->fb.fix.smem_len = smemlen;
	}
	/* szq: set double buffer size */
	fbi->fb.fix.smem_len *= fbi->buffer_num;

//    fbi->set_overlay_ctrlr_state	= NULL;

//  init_waitqueue_head(&fbi->ctrlr_wait);
//	INIT_WORK(&fbi->task, pxafb_task);
//	init_MUTEX(&fbi->ctrlr_sem);

//	return fbi;
}

extern void pxa_logo(void);
static int __init pxafb_map_video_memory(struct pxafb_info *fbi, int mod)
{
    u_long palette_mem_size;

	/*
	 * We reserve one page for the palette, plus the size
	 * of the framebuffer.
	 */
#if 0
	fbi->map_size = PAGE_ALIGN(fbi->fb.fix.smem_len + PAGE_SIZE);
#else
	/* we reserve 3 pages after the framebuffer for occasionally out
	 * of boundary access
	 */
	fbi->map_size = PAGE_ALIGN(fbi->fb.fix.smem_len + 4 * PAGE_SIZE);
#endif
    switch(mod)
    {
        case ___video_boot_mod:   
        fbi->map_cpu = (u_char *)___Bvideo_memory_screen_dma_ddr_addr;//dma_alloc_writecombine(fbi->dev, fbi->map_size,
        fbi->map_dma = ___Bvideo_memory_screen_dma_ddr_addr;
        break;
        case ___video_flash_mod:
        fbi->map_cpu = (u_char *)___Fvideo_memory_screen_dma_ddr_addr;//dma_alloc_writecombine(fbi->dev, fbi->map_size, 
        fbi->map_dma = ___Fvideo_memory_screen_dma_ddr_addr;
        break;
    }

	if (fbi->map_cpu) 
    {
		/* prevent initial garbage on screen */
		memset(fbi->map_cpu, 0, fbi->map_size);
		fbi->fb.screen_base = fbi->map_cpu + PAGE_SIZE;
		fbi->screen_dma = fbi->map_dma + PAGE_SIZE;

        switch(mod)
        {
            case ___video_boot_mod:   
            //pxa_logo();
            break;
            case ___video_flash_mod:
            break;
        }

        /*
		 * FIXME: this is actually the wrong thing to place in
		 * smem_start.  But fbdev suffers from the problem that
		 * it needs an API which doesn't exist (in this case,
		 * dma_writecombine_mmap)
		 */
		fbi->fb.fix.smem_start = fbi->screen_dma;
		fbi->palette_size = fbi->fb.var.bits_per_pixel == 8 ? 256 : 16;

		if ((fbi->lccr4 & LCCR4_PAL_FOR_MASK) == LCCR4_PAL_FOR_0)
			palette_mem_size = fbi->palette_size * sizeof(u16);
		else
			palette_mem_size = fbi->palette_size * sizeof(u32);

		pr_debug("pxafb: palette_mem_size = 0x%08lx\n", palette_mem_size);

		fbi->palette_cpu = (u16 *)(fbi->map_cpu + PAGE_SIZE - palette_mem_size);
		fbi->palette_dma = fbi->map_dma + PAGE_SIZE - palette_mem_size;

		fbi->cmd_cpu = fbi->map_cpu;
		fbi->cmd_dma = fbi->map_dma;
	}

	return fbi->map_cpu ? 0 : -1;//-ENOMEM;
}

static struct pxafb_mode_info *pxafb_getmode(struct pxafb_mach_info *mach, struct fb_var_screeninfo *var)
{
	struct pxafb_mode_info *mode = NULL;
	struct pxafb_mode_info *modelist = mach->modes;
	unsigned int best_x = 0xffffffff, best_y = 0xffffffff;
	unsigned int i;

	for (i = 0 ; i < mach->num_modes ; i++) {
		if (modelist[i].xres >= var->xres && modelist[i].yres >= var->yres &&
				modelist[i].xres < best_x && modelist[i].yres < best_y &&
				modelist[i].bpp >= var->bits_per_pixel ) {
			best_x = modelist[i].xres;
			best_y = modelist[i].yres;
			mode = &modelist[i];
		}
	}

	return mode;
}

static void pxafb_setmode(struct fb_var_screeninfo *var, struct pxafb_mode_info *mode)
{
	var->xres		= mode->xres;
	var->yres		= mode->yres;
	var->bits_per_pixel	= mode->bpp;
	var->pixclock		= mode->pixclock;
	var->hsync_len		= mode->hsync_len;
	var->left_margin	= mode->left_margin;
	var->right_margin	= mode->right_margin;
	var->vsync_len		= mode->vsync_len;
	var->upper_margin	= mode->upper_margin;
	var->lower_margin	= mode->lower_margin;
	var->sync		= mode->sync;
	var->grayscale		= mode->cmap_greyscale;
	var->xres_virtual 	= var->xres;
	var->yres_virtual	= var->yres * PXAFB_BUFFER_NUM;
}

static int pxafb_check_var(struct fb_var_screeninfo *var, struct fb_info *info)
{
	struct pxafb_info *fbi = (struct pxafb_info *)info;
	struct pxafb_mach_info *inf = &tpo_hvga_info;//fbi->dev->platform_data;

	if (var->xres < MIN_XRES)
		var->xres = MIN_XRES;
	if (var->yres < MIN_YRES)
		var->yres = MIN_YRES;

	if (inf->fixed_modes) {
		struct pxafb_mode_info *mode;
		struct pxafb_mode_info mode2;

		mode = pxafb_getmode(inf, var);
		memcpy(&mode2, mode, sizeof(struct pxafb_mode_info));
		mode = &mode2;
		mode->bpp = var->bits_per_pixel;
		if (!mode)
			return -EINVAL;
		pxafb_setmode(var, mode);
		fbi->wr_setup_time = mode->wr_setup_time;
		fbi->wr_pulse_width = mode->wr_pulse_width;
		fbi->rd_setup_time = mode->rd_setup_time;
		fbi->rd_pulse_width = mode->rd_pulse_width;
		fbi->op_hold_time = mode->op_hold_time;
		fbi->cmd_inh_time = mode->cmd_inh_time;
		fbi->sync_cnt = mode->sync_cnt;
		fbi->update_framedata = mode->update_framedata;
	} else {
		if (var->xres > inf->modes->xres)
			return -EINVAL;
		if (var->yres > inf->modes->yres)
			return -EINVAL;
		if ((var->bits_per_pixel != 1) &&
			(var->bits_per_pixel != 2) &&
			(var->bits_per_pixel != 4) &&
			(var->bits_per_pixel != 8) &&
			(var->bits_per_pixel != 16) &&
			(var->bits_per_pixel != 18) &&
			(var->bits_per_pixel != 19) &&
			(var->bits_per_pixel != 24) &&
			(var->bits_per_pixel != 25) &&
			(var->bits_per_pixel != 32)) 
			return -EINVAL;
	}

	var->xres_virtual =
		max(var->xres_virtual, var->xres);
	var->yres_virtual =
		max(var->yres_virtual, var->yres);

        /*
	 * Setup the RGB parameters for this display.
	 *
	 * The pixel packing format is described on page 7-11 of the
	 * PXA2XX Developer's Manual.
         */
	if (var->bits_per_pixel == 16) {
		var->red.offset   = 11; var->red.length   = 5;
		var->green.offset = 5;  var->green.length = 6;
		var->blue.offset  = 0;  var->blue.length  = 5;
		var->transp.offset = var->transp.length = 0;
	}else if(var->bits_per_pixel == 18){
		var->red.offset   = 18; var->red.length   = 6;
		var->green.offset = 10; var->green.length = 6;
		var->blue.offset  = 2;  var->blue.length  = 6;
		var->transp.offset= 0;
		var->transp.length =0;
	}else if(var->bits_per_pixel == 19 ){
		var->red.offset   = 18; var->red.length   = 6;
		var->green.offset = 10; var->green.length = 6;
		var->blue.offset  = 2;  var->blue.length  = 6;
		var->transp.offset= 24;
		var->transp.length =1;
	}else if(var->bits_per_pixel == 24){
		var->red.offset   = 16; var->green.offset = 8; var->blue.offset = 0;
		var->red.length = var->green.length = var->blue.length = 8;
		var->transp.offset = var->transp.length = 0;
	}else if(var->bits_per_pixel == 25){
		var->red.offset   = 16; var->green.offset = 8; var->blue.offset = 0;
		var->red.length = var->green.length = var->blue.length = 8;
		var->transp.offset = 24; var->transp.length = 1;
	}else if( var->bits_per_pixel == 32){
		var->red.offset   = 16; var->green.offset = 8; var->blue.offset = 0;
		var->red.length = var->green.length = var->blue.length = var->transp.length = 8;
		var->transp.offset = 24;
	}else {
		var->red.offset = var->green.offset = var->blue.offset = var->transp.offset = 0;
		var->red.length   = 8;
		var->green.length = 8;
		var->blue.length  = 8;
		var->transp.length = 0;
	}

#ifdef CONFIG_CPU_FREQ
	pr_debug("pxafb: dma period = %d ps, clock = %d kHz\n",
		 pxafb_display_dma_period(var),
		 get_clk_frequency_khz(0));
#endif

	return 0;
}

static int pxafb_validate_var(struct fb_var_screeninfo *var)
{
        int ret = -EINVAL;

        var->xres_virtual =
            var->xres_virtual < var->xres ? var->xres : (var->xres_virtual
        /var->xres)*var->xres;
        var->yres_virtual =
            var->yres_virtual < var->yres ? var->yres : (var->yres_virtual
        /var->yres)*var->yres;

        var->xoffset = 0;
        var->yoffset = (var->yoffset/var->yres)*var->yres;
        if (var->yoffset >= var->yres_virtual) {
                var->yoffset = var->yres_virtual - var->yres;
        }
        ret = 0;
        return ret;
}

static inline void pxafb_set_truecolor(u_int is_true_color)
{
	pr_debug("pxafb: true_color = %d\n", is_true_color);
	// do your machine-specific setup if needed
}

void fb_dealloc_cmap(struct fb_cmap *cmap)
{
	kfree(cmap->red);
	kfree(cmap->green);
	kfree(cmap->blue);
	kfree(cmap->transp);

	cmap->red = cmap->green = cmap->blue = cmap->transp = NULL;
	cmap->len = 0;
}

unsigned long clk_get_rate(struct clk *clk)
{
	unsigned long rate;

	rate = clk->rate;
	if (clk->ops->getrate)
		rate = clk->ops->getrate(clk);

	return rate;
}

unsigned int __smart_panel_timing(struct pxafb_info *fbi, 
		unsigned long long lclk, int time) 
{
	unsigned long long value;
	unsigned long long remainder;

	value = lclk*time;
	remainder = do_div(value, 100000);
	if (remainder > 50000)
		value += 1;
	return (unsigned int) value;
}

unsigned int smart_panel_timing(struct pxafb_info *fbi, int time) 
{
	unsigned long long lclk;

	lclk = clk_get_rate(fbi->clk)/10000;/* in 10KHZ */
	return __smart_panel_timing(fbi, lclk, time);
}

static inline unsigned int __get_pcd(unsigned long long lclk, unsigned int pixclock)
{
	unsigned long long pcd;
	unsigned long long remainder;

	/* FIXME: Need to take into account Double Pixel Clock mode
         * (DPC) bit? or perhaps set it based on the various clock
         * speeds */

	pcd = (unsigned long long)lclk*pixclock;

	if (LCCR4 & LCCR4_PCDDIV) {
		remainder = do_div(pcd, 100000000);
		if (remainder < 50000000)
			pcd -= 1;
	}
	else {
		remainder = do_div(pcd, 100000000 * 2);
		if (remainder < 100000000)
			pcd -= 1;
	}

	/* FIXME: for lcd clock(10khz) equals 10400 or 5200, special
	   PCD value is used. If we use the formula to calculate the
	   PCD value, the LCD will flicker when DVFM.
	*/
	return (unsigned int)pcd;
}


static inline unsigned int get_pcd(struct pxafb_info *fbi, unsigned int pixclock)
{
	unsigned long long lclk = clk_get_rate(fbi->clk) / 10000; /* in 10KHZ */

	if (fbi->flags & PXAFB_STD_SMART_PANEL)
		return __smart_panel_timing(fbi, lclk, fbi->cmd_inh_time);
	else 
		return __get_pcd(lclk, pixclock);
}

static int pxafb_bpp_to_lccr3(struct fb_var_screeninfo *var, struct pxafb_info *fbi)
{
        int ret = 0;
        switch (var->bits_per_pixel) {
        case 1:  ret = LCCR3_1BPP; break;
        case 2:  ret = LCCR3_2BPP; break;
        case 4:  ret = LCCR3_4BPP; break;
        case 8:  ret = LCCR3_8BPP; break;
        case 16: ret = LCCR3_16BPP; break;
	case 18: ret = LCCR3_18BPP; break;
	case 19: ret = LCCR3_19BPP; break;
	
	case 24:
		if( fbi->flags & PXAFB_SMART_PANEL) 
			ret = LCCR3_24BPP; 
		else
			ret = LCCR3_18BPP;
		break;
	case 25:
	case 32:
		if( fbi->flags & PXAFB_SMART_PANEL)
			ret = LCCR3_25BPP; 
		else
			ret = LCCR3_19BPP;
		break;
	default:
		break;
        }
        return ret;
}

static int pxafb_create_dma(struct pxafb_info *info, struct fb_var_screeninfo *var, unsigned int buffer_index)
{
	struct pxafb_info *fbi=info;
	uint lines_per_panel = var->yres;

	/* setup dma descriptors */
	fbi->dmadesc_cmd_cpu    = (struct pxafb_dma_descriptor *)(fbi->palette_cpu) - (fbi->buffer_num - fbi->buffer_index) *4 + 0;
//	fbi->dmadesc_fblow_cpu  = (struct pxafb_dma_descriptor *)(fbi->palette_cpu) - (fbi->buffer_num - fbi->buffer_index) * 4 + 1;
	fbi->dmadesc_fbhigh_cpu = (struct pxafb_dma_descriptor *)(fbi->palette_cpu) - (fbi->buffer_num - fbi->buffer_index) * 4 + 2;
	fbi->dmadesc_palette_cpu= (struct pxafb_dma_descriptor *)(fbi->palette_cpu) - (fbi->buffer_num - fbi->buffer_index) * 4 + 3;

	fbi->dmadesc_cmd_dma     = fbi->palette_dma - ((fbi->buffer_num - fbi->buffer_index)*4 - 0) * sizeof(struct pxafb_dma_descriptor);
//	fbi->dmadesc_fblow_dma   = fbi->palette_dma - ((fbi->buffer_num - fbi->buffer_index)*4 - 1) * sizeof(struct pxafb_dma_descriptor);
	fbi->dmadesc_fbhigh_dma  = fbi->palette_dma - ((fbi->buffer_num - fbi->buffer_index)*4 - 2) * sizeof(struct pxafb_dma_descriptor);
	fbi->dmadesc_palette_dma = fbi->palette_dma - ((fbi->buffer_num - fbi->buffer_index)*4 - 3) * sizeof(struct pxafb_dma_descriptor);
	
#define BYTES_PER_PANEL (lines_per_panel * fbi->fb.fix.line_length)

	/* populate descriptors */
	fbi->dmadesc_cmd_cpu->fdadr = fbi->dmadesc_cmd_dma;
	fbi->dmadesc_cmd_cpu->fsadr = fbi->cmd_dma;
	fbi->dmadesc_cmd_cpu->fidr  = 0;
	fbi->dmadesc_cmd_cpu->ldcmd = 0;
#if 0
	fbi->dmadesc_fblow_cpu->fdadr = fbi->dmadesc_fblow_dma;
	fbi->dmadesc_fblow_cpu->fsadr = fbi->screen_dma + BYTES_PER_PANEL;
	fbi->dmadesc_fblow_cpu->fidr  = 0;
	fbi->dmadesc_fblow_cpu->ldcmd = BYTES_PER_PANEL;

	fbi->fdadr1 = fbi->dmadesc_fblow_dma; /* only used in dual-panel mode */
#endif
	fbi->dmadesc_fbhigh_cpu->fsadr = fbi->screen_dma + buffer_index * BYTES_PER_PANEL;
	fbi->dmadesc_fbhigh_cpu->fidr = 0;
	fbi->dmadesc_fbhigh_cpu->ldcmd = BYTES_PER_PANEL | LDCMD_EOFINT;

	fbi->dmadesc_palette_cpu->fsadr = fbi->palette_dma;
	fbi->dmadesc_palette_cpu->fidr  = 0;
	if ((fbi->lccr4 & LCCR4_PAL_FOR_MASK) == LCCR4_PAL_FOR_0)
		fbi->dmadesc_palette_cpu->ldcmd = fbi->palette_size *
							sizeof(u16);
	else
		fbi->dmadesc_palette_cpu->ldcmd = fbi->palette_size *
							sizeof(u32);
	fbi->dmadesc_palette_cpu->ldcmd |= LDCMD_PAL;

	if ( (var->bits_per_pixel == 16) || (var->bits_per_pixel == 18) ||
	     (var->bits_per_pixel == 19) || (var->bits_per_pixel == 24) ||
	     (var->bits_per_pixel == 25) || (var->bits_per_pixel == 32) )
	{
		/* palette shouldn't be loaded in true-color mode */
		fbi->dmadesc_fbhigh_cpu->fdadr = fbi->dmadesc_fbhigh_dma;
		fbi->fdadr0 = fbi->dmadesc_fbhigh_dma; /* no pal just fbhigh */
		/* init it to something, even though we won't be using it */
		fbi->dmadesc_palette_cpu->fdadr = fbi->dmadesc_palette_dma;
	} else {
		fbi->dmadesc_palette_cpu->fdadr = fbi->dmadesc_fbhigh_dma;
		fbi->dmadesc_fbhigh_cpu->fdadr = fbi->dmadesc_palette_dma;
		fbi->fdadr0 = fbi->dmadesc_palette_dma; /* flips back and forth between pal and fbhigh */
	}
	fbi->fdadr6 = fbi->dmadesc_cmd_dma;

#if 1
	pr_debug("fbi->dmadesc_cmd_cpu = 0x%p\n", fbi->dmadesc_cmd_cpu);
//	pr_debug("fbi->dmadesc_fblow_cpu = 0x%p\n", fbi->dmadesc_fblow_cpu);
	pr_debug("fbi->dmadesc_fbhigh_cpu = 0x%p\n", fbi->dmadesc_fbhigh_cpu);
	pr_debug("fbi->dmadesc_palette_cpu = 0x%p\n", fbi->dmadesc_palette_cpu);

	pr_debug("fbi->dmadesc_cmd_dma = 0x%x\n", fbi->dmadesc_cmd_dma);
//	pr_debug("fbi->dmadesc_fblow_dma = 0x%x\n", fbi->dmadesc_fblow_dma);
	pr_debug("fbi->dmadesc_fbhigh_dma = 0x%x\n", fbi->dmadesc_fbhigh_dma);
	pr_debug("fbi->dmadesc_palette_dma = 0x%x\n", fbi->dmadesc_palette_dma);

	pr_debug("fbi->dmadesc_cmd_cpu->fdadr = 0x%x\n", fbi->dmadesc_cmd_cpu->fdadr);
//	pr_debug("fbi->dmadesc_fblow_cpu->fdadr = 0x%x\n", fbi->dmadesc_fblow_cpu->fdadr);
	pr_debug("fbi->dmadesc_fbhigh_cpu->fdadr = 0x%x\n", fbi->dmadesc_fbhigh_cpu->fdadr);
	pr_debug("fbi->dmadesc_palette_cpu->fdadr = 0x%x\n", fbi->dmadesc_palette_cpu->fdadr);

	pr_debug("fbi->dmadesc_cmd_cpu->fsadr = 0x%x\n", fbi->dmadesc_cmd_cpu->fsadr);
//	pr_debug("fbi->dmadesc_fblow_cpu->fsadr = 0x%x\n", fbi->dmadesc_fblow_cpu->fsadr);
	pr_debug("fbi->dmadesc_fbhigh_cpu->fsadr = 0x%x\n", fbi->dmadesc_fbhigh_cpu->fsadr);
	pr_debug("fbi->dmadesc_palette_cpu->fsadr = 0x%x\n", fbi->dmadesc_palette_cpu->fsadr);

	pr_debug("fbi->dmadesc_cmd_cpu->ldcmd = 0x%x\n", fbi->dmadesc_cmd_cpu->ldcmd);
//	pr_debug("fbi->dmadesc_fblow_cpu->ldcmd = 0x%x\n", fbi->dmadesc_fblow_cpu->ldcmd);
	pr_debug("fbi->dmadesc_fbhigh_cpu->ldcmd = 0x%x\n", fbi->dmadesc_fbhigh_cpu->ldcmd);
	pr_debug("fbi->dmadesc_palette_cpu->ldcmd = 0x%x\n", fbi->dmadesc_palette_cpu->ldcmd);
#endif
	return 0;
}

static inline void set_hsync_time(struct pxafb_info *fbi, unsigned int pcd)
{
	unsigned long htime;

	if ((pcd == 0) || (fbi->fb.var.hsync_len == 0)) {
		fbi->hsync_time=0;
		return;
	}

	htime = clk_get_rate(fbi->clk) / (pcd * fbi->fb.var.hsync_len);

	fbi->hsync_time = htime;
}

static int pxafb_activate_var(struct fb_var_screeninfo *var, struct pxafb_info *fbi)
{
	struct pxafb_lcd_reg new_regs;
	u_long flags;
	u_int lines_per_panel, pcd = get_pcd(fbi, var->pixclock);

	pr_debug("pxafb: Configuring PXA LCD\n");

	pr_debug("var: xres=%d hslen=%d lm=%d rm=%d\n",
		 var->xres, var->hsync_len,
		 var->left_margin, var->right_margin);
	pr_debug("var: yres=%d vslen=%d um=%d bm=%d\n",
		 var->yres, var->vsync_len,
		 var->upper_margin, var->lower_margin);
	pr_debug("var: pixclock=%d pcd=%d\n", var->pixclock, pcd);

#if DEBUG_VAR
	if (var->xres < 16        || var->xres > 1024)
		printk(KERN_ERR "%s: invalid xres %d\n",
			fbi->fb.fix.id, var->xres);
	switch(var->bits_per_pixel) {
	case 1:
	case 2:
	case 4:
	case 8:
	case 16:
	case 18:
	case 19:
	case 24:
	case 25:
	case 32:
		break;
	default:
		printk(KERN_ERR "%s: invalid bit depth %d\n",
		       fbi->fb.fix.id, var->bits_per_pixel);
		break;
	}
	if (var->hsync_len < 1    || var->hsync_len > 64)
		printk(KERN_ERR "%s: invalid hsync_len %d\n",
			fbi->fb.fix.id, var->hsync_len);
	if (var->left_margin < 1  || var->left_margin > 255)
		printk(KERN_ERR "%s: invalid left_margin %d\n",
			fbi->fb.fix.id, var->left_margin);
	if (var->right_margin < 1 || var->right_margin > 255)
		printk(KERN_ERR "%s: invalid right_margin %d\n",
			fbi->fb.fix.id, var->right_margin);
	if (var->yres < 1         || var->yres > 1024)
		printk(KERN_ERR "%s: invalid yres %d\n",
			fbi->fb.fix.id, var->yres);
	if (var->vsync_len < 1    || var->vsync_len > 64)
		printk(KERN_ERR "%s: invalid vsync_len %d\n",
			fbi->fb.fix.id, var->vsync_len);
	if (var->upper_margin < 0 || var->upper_margin > 255)
		printk(KERN_ERR "%s: invalid upper_margin %d\n",
			fbi->fb.fix.id, var->upper_margin);
	if (var->lower_margin < 0 || var->lower_margin > 255)
		printk(KERN_ERR "%s: invalid lower_margin %d\n",
			fbi->fb.fix.id, var->lower_margin);
#endif

//	down(&fbi->ctrlr_sem);

	new_regs.lccr0 = fbi->lccr0 |
		(LCCR0_LDM | LCCR0_SFM | LCCR0_IUM | LCCR0_EFM |
#if defined(CONFIG_PXA27x) || defined(CONFIG_PXA3xx)
		LCCR0_OUC |
#endif
                 LCCR0_QDM | LCCR0_BM  | LCCR0_OUM);

	new_regs.lccr4 = LCCR4_REOFM0 | LCCR4_REOFM1 | LCCR4_REOFM2 |
		LCCR4_REOFM3 | LCCR4_REOFM4 | LCCR4_REOFM5 |
		LCCR4_REOFM6;

	new_regs.lccr5 = LCCR5_SOFM1 | LCCR5_SOFM2 | LCCR5_SOFM3 | 
		LCCR5_SOFM4 | LCCR5_SOFM5 | LCCR5_SOFM6 | 
		LCCR5_EOFM1 | LCCR5_EOFM2 | LCCR5_EOFM3 |
		LCCR5_EOFM4 | LCCR5_EOFM5 | LCCR5_EOFM6 | 
		LCCR5_BSM1 | LCCR5_BSM2 | LCCR5_BSM3 |
		LCCR5_BSM4 | LCCR5_BSM5 | LCCR5_BSM6 | 
		LCCR5_IUM1 | LCCR5_IUM2 | LCCR5_IUM3 |
		LCCR5_IUM4 | LCCR5_IUM5 | LCCR5_IUM6;

	if (fbi->flags & PXAFB_STD_SMART_PANEL)
		goto std_smart_panel;

	new_regs.lccr0 |= LCCR0_CMDIM | LCCR0_RDSTM;

	new_regs.lccr1 =
		LCCR1_DisWdth(var->xres) +
		LCCR1_HorSnchWdth(var->hsync_len) +
		LCCR1_BegLnDel(var->left_margin) +
		LCCR1_EndLnDel(var->right_margin);

	/*
	 * If we have a dual scan LCD, we need to halve
	 * the YRES parameter.
	 */
	lines_per_panel = var->yres;
	if ((fbi->lccr0 & LCCR0_SDS) == LCCR0_Dual)
		lines_per_panel /= 2;

	new_regs.lccr2 =
		LCCR2_DisHght(lines_per_panel) +
		LCCR2_VrtSnchWdth(var->vsync_len) +
		LCCR2_BegFrmDel(var->upper_margin) +
		LCCR2_EndFrmDel(var->lower_margin);

	new_regs.lccr3 = fbi->lccr3 |
		pxafb_bpp_to_lccr3(var,fbi) |
		(var->sync & FB_SYNC_HOR_HIGH_ACT ? LCCR3_HorSnchH : LCCR3_HorSnchL) |
		(var->sync & FB_SYNC_VERT_HIGH_ACT ? LCCR3_VrtSnchH : LCCR3_VrtSnchL);

	new_regs.lccr6 = LCCR6;

	new_regs.cmdcr = CMDCR;

	goto set_pcd;

std_smart_panel:
	lines_per_panel = var->yres;
	new_regs.lccr0 |= LCCR0_LCDT;
	new_regs.lccr1 = 
		LCCR1_DisWdth(var->xres) +
		LCCR1_HorSnchWdth(smart_panel_timing(fbi, fbi->op_hold_time)) +
		LCCR1_BegLnDel(smart_panel_timing(fbi, fbi->wr_pulse_width)) + 
		LCCR1_EndLnDel(smart_panel_timing(fbi, fbi->wr_setup_time));
	new_regs.lccr2 =
		LCCR2_DisHght(lines_per_panel) +
		LCCR2_BegFrmDel(smart_panel_timing(fbi, fbi->rd_pulse_width)) +
		LCCR2_EndFrmDel(smart_panel_timing(fbi, fbi->rd_setup_time));
	new_regs.lccr3 = fbi->lccr3 |
		pxafb_bpp_to_lccr3(var, fbi);
	new_regs.lccr6 = fbi->lccr6;
	new_regs.cmdcr = fbi->sync_cnt;

set_pcd:
	if (pcd)
		new_regs.lccr3 |= LCCR3_PixClkDiv(pcd);


	pxafb_create_dma(fbi, var, fbi->buffer_index);

	fbi->reg_lccr0 = new_regs.lccr0;
	fbi->reg_lccr1 = new_regs.lccr1;
	fbi->reg_lccr2 = new_regs.lccr2;
	fbi->reg_lccr3 = new_regs.lccr3;
	fbi->reg_lccr4 = new_regs.lccr4 & (~LCCR4_PAL_FOR_MASK);
	fbi->reg_lccr4 |= (fbi->lccr4 & LCCR4_PAL_FOR_MASK);
	fbi->reg_lccr5 = new_regs.lccr5;
	fbi->reg_lccr6 = new_regs.lccr6;
	fbi->reg_cmdcr = new_regs.cmdcr;
	set_hsync_time(fbi, pcd);

	pr_debug("nlccr0 = 0x%08x\n", fbi->reg_lccr0);
	pr_debug("nlccr1 = 0x%08x\n", fbi->reg_lccr1);
	pr_debug("nlccr2 = 0x%08x\n", fbi->reg_lccr2);
	pr_debug("nlccr3 = 0x%08x\n", fbi->reg_lccr3);
	pr_debug("nlccr4 = 0x%08x\n", fbi->reg_lccr4);
	pr_debug("nlccr5 = 0x%08x\n", fbi->reg_lccr5);
	pr_debug("nlccr6 = 0x%08x\n", fbi->reg_lccr6);
	pr_debug("ncmdcr = 0x%08x\n", fbi->reg_cmdcr);

	pr_debug("FDADR0 0x%08x\n", (unsigned int) FDADR0);
	pr_debug("FDADR1 0x%08x\n", (unsigned int) FDADR1);
	pr_debug("LCCR0 0x%08x\n", (unsigned int) LCCR0);
	pr_debug("LCCR1 0x%08x\n", (unsigned int) LCCR1);
	pr_debug("LCCR2 0x%08x\n", (unsigned int) LCCR2);
	pr_debug("LCCR3 0x%08x\n", (unsigned int) LCCR3);
	pr_debug("LCCR4 0x%08x\n", (unsigned int) LCCR4);
	pr_debug("LCCR5 0x%08x\n", (unsigned int) LCCR5);
	pr_debug("LCCR6 0x%08x\n", (unsigned int) LCCR6);
	pr_debug("CMDCR 0x%08x\n", (unsigned int) CMDCR);

//	up(&fbi->ctrlr_sem);

	/*
	 * Only update the registers if the controller is enabled
	 * and something has changed.
	 * szq: if fb_pan_display is used, FBR0 will be used instead of FDADR0,
	 *      so we don't compare FDADR0 and FDADR1 in this case.
	 */
/*
	if (( (LCCR0 & (~LCCR0_ENB)) != (fbi->reg_lccr0 & (~LCCR0_ENB))) || 
	    (LCCR1  != fbi->reg_lccr1) || (LCCR2  != fbi->reg_lccr2) || 
	    (LCCR3  != fbi->reg_lccr3) || (LCCR4  != fbi->reg_lccr4) || 
	    ( (fbi->flags & PXAFB_SMART_PANEL) && ((LCCR5  != fbi->reg_lccr5) || (LCCR6  != fbi->reg_lccr6) || (CMDCR  != fbi->reg_cmdcr))) ||
	    ( (!fbi->fb.fbops->fb_pan_display) && (FDADR0 != fbi->fdadr0) )  )
		pxafb_schedule_work(fbi, C_REENABLE);
*/
	return 0;
}

static int pxafb_set_par(struct fb_info *info)
{
	struct pxafb_info *fbi = (struct pxafb_info *)info;
	struct fb_var_screeninfo *var = &info->var;
	unsigned long palette_mem_size;

	pr_debug("pxafb: set_par\n");
	pxafb_validate_var(var);
	fbi->buffer_index = var->yoffset / var->yres;


	if ( (var->bits_per_pixel == 16) || (var->bits_per_pixel == 18) ||
	     (var->bits_per_pixel == 19) || (var->bits_per_pixel == 24) ||
	     (var->bits_per_pixel == 25) || (var->bits_per_pixel == 32) )
		fbi->fb.fix.visual = FB_VISUAL_TRUECOLOR;
	else if (!fbi->cmap_static)
		fbi->fb.fix.visual = FB_VISUAL_PSEUDOCOLOR;
	else {
		/*
		 * Some people have weird ideas about wanting static
		 * pseudocolor maps.  I suspect their user space
		 * applications are broken.
		 */
		fbi->fb.fix.visual = FB_VISUAL_STATIC_PSEUDOCOLOR;
	}
    switch (var->bits_per_pixel) {
	case 16:
		fbi->fb.fix.line_length = var->xres_virtual * 2;
		fbi->palette_size = 0;
		break;
	case 18:
	case 19:
	case 24:
	case 25:
	case 32:
		fbi->fb.fix.line_length = var->xres_virtual * 4;
		fbi->palette_size = 0;
		break;
	default:
		fbi->fb.fix.line_length = var->xres_virtual * var->bits_per_pixel / 8;
		fbi->palette_size = var->bits_per_pixel == 1 ? 4 : 1 << var->bits_per_pixel;
	}
	if ((fbi->lccr4 & LCCR4_PAL_FOR_MASK) == LCCR4_PAL_FOR_0)
		palette_mem_size = fbi->palette_size * sizeof(u16);
	else
		palette_mem_size = fbi->palette_size * sizeof(u32);

	pr_debug("pxafb: palette_mem_size = 0x%08lx\n", palette_mem_size);

	fbi->palette_cpu = (u16 *)(fbi->map_cpu + PAGE_SIZE - palette_mem_size);
	fbi->palette_dma = fbi->map_dma + PAGE_SIZE - palette_mem_size;

	/*
	 * Set (any) board control register to handle new color depth
	 */
	pxafb_set_truecolor(fbi->fb.fix.visual == FB_VISUAL_TRUECOLOR);

	if( (fbi->fb.var.bits_per_pixel == 16) || (fbi->fb.var.bits_per_pixel == 18) ||
	    (fbi->fb.var.bits_per_pixel == 19) || (fbi->fb.var.bits_per_pixel == 24) ||
	    (fbi->fb.var.bits_per_pixel == 25) || (fbi->fb.var.bits_per_pixel == 32 ) )
		fb_dealloc_cmap(&fbi->fb.cmap);
	//else
	//	fb_alloc_cmap(&fbi->fb.cmap, 1<<fbi->fb.var.bits_per_pixel, 0);

	pxafb_activate_var(var, fbi);

	return 0;
}

static void pxafb_enable_controller(struct pxafb_info *fbi)
{
	pr_debug("pxafb: Enabling LCD controller\n");
	pr_debug("fdadr0 0x%08x\n", (unsigned int) fbi->fdadr0);
	pr_debug("fdadr1 0x%08x\n", (unsigned int) fbi->fdadr1);
	pr_debug("reg_lccr0 0x%08x\n", (unsigned int) fbi->reg_lccr0);
	pr_debug("reg_lccr1 0x%08x\n", (unsigned int) fbi->reg_lccr1);
	pr_debug("reg_lccr2 0x%08x\n", (unsigned int) fbi->reg_lccr2);
	pr_debug("reg_lccr3 0x%08x\n", (unsigned int) fbi->reg_lccr3);
	pr_debug("reg_lccr4 0x%08x\n", (unsigned int) fbi->reg_lccr4);
	pr_debug("reg_lccr5 0x%08x\n", (unsigned int) fbi->reg_lccr5);
	pr_debug("reg_lccr6 0x%08x\n", (unsigned int) fbi->reg_lccr6);

	/* Sequence from 11.7.10 */
	if( fbi->flags & PXAFB_SMART_PANEL )
	{
		CMDCR = fbi->reg_cmdcr;
		LCCR6 = fbi->reg_lccr6;
		LCCR5 = fbi->reg_lccr5;
	}
	LCCR4 = fbi->reg_lccr4;
	LCCR3 = fbi->reg_lccr3;
	LCCR2 = fbi->reg_lccr2;
	LCCR1 = fbi->reg_lccr1;
	LCCR0 = fbi->reg_lccr0 & ~LCCR0_ENB;

	FDADR0 = fbi->fdadr0;
/*	FDADR1 = fbi->fdadr1; */

	if (!(fbi->flags & PXAFB_SMART_PANEL))
		LCCR0 |= LCCR0_ENB;

	/* enable LCD controller clock */
	clk_enable(fbi->clk);

	pr_debug("FDADR0 0x%08x\n", (unsigned int) FDADR0);
	pr_debug("FDADR1 0x%08x\n", (unsigned int) FDADR1);
	pr_debug("LCCR0 0x%08x\n", (unsigned int) LCCR0);
	pr_debug("LCCR1 0x%08x\n", (unsigned int) LCCR1);
	pr_debug("LCCR2 0x%08x\n", (unsigned int) LCCR2);
	pr_debug("LCCR3 0x%08x\n", (unsigned int) LCCR3);
	pr_debug("LCCR4 0x%08x\n", (unsigned int) LCCR4);
	pr_debug("LCCR5 0x%08x\n", (unsigned int) LCCR5);
	pr_debug("LCCR6 0x%08x\n", (unsigned int) LCCR6);
}

static void pxafb_disable_controller(struct pxafb_info *fbi)
{
	DECLARE_WAITQUEUE(wait, current);

	pr_debug("pxafb: disabling LCD controller\n");
	if(fbi->flags & PXAFB_SMART_PANEL)
		goto exit;

	set_current_state(TASK_UNINTERRUPTIBLE);
	add_wait_queue(&fbi->ctrlr_wait, &wait);

	LCSR = 0xffffffff;	/* Clear LCD Status Register */
	LCCR0 &= ~LCCR0_LDM;	/* Enable LCD Disable Done Interrupt */
	LCCR0 |= LCCR0_DIS;	/* Disable LCD Controller */

	schedule_timeout(200 * HZ / 1000);
    remove_wait_queue(&fbi->ctrlr_wait, &wait);

	LCCR0 &= ~LCCR0_ENB;
exit:
	/* disable LCD controller clock */
	clk_disable(fbi->clk);
}
static inline void __pxafb_backlight_power(struct pxafb_info *fbi, int on)
{
	pr_debug("pxafb: backlight o%s\n", on ? "n" : "ff");

 	if (pxafb_backlight_power)
 		pxafb_backlight_power(on, fbi->bl_brightness);
}

static inline void __pxafb_lcd_power(struct pxafb_info *fbi, int on)
{
	pr_debug("pxafb: LCD power o%s\n", on ? "n" : "ff");

	if (pxafb_lcd_power)
		pxafb_lcd_power(on, &fbi->fb.var);

}

#define GPIO21_GPIO		MFP_CFG(GPIO21, AF0)
/* SPARK MFP configurations */
static mfp_cfg_t spark_mfp_cfg[] __initdata = {
	/* LCD */
	LCD_VSYNC, 
	LCD_HSYNC, 
	LCD_PCLK, 
	LCD_DE, 
	LCD_R0, 
	LCD_R1, 
	LCD_R2, 
	LCD_R3, 
	LCD_R4, 
	LCD_R5, 
	LCD_G0, 
	LCD_G1, 
	LCD_G2, 
	LCD_G3, 
	LCD_G4, 
	LCD_G5, 
	LCD_B0, 
	LCD_B1, 
	LCD_B2, 
	LCD_B3, 
	LCD_B4, 
	LCD_B5, 
	LCD_LED_PWM, 
	LCD_RESET_N,


	/* LCD SPI */
	LCD_SPI_CLK, 
	LCD_SPI_CS, 
//	LCD_SPI_DOUT, 
	LCD_SPI_DIN, 
};


 void pxa_lcd_ini(int mod)
{
    struct pxafb_info *fbi;

    pxa3xx_mfp_config(ARRAY_AND_SIZE(spark_mfp_cfg));
#if 1
    {extern void enable_i_d_cache(void);
        enable_i_d_cache();
    }
#endif
    fbi = &pxafb;
    pxafb_init_fbinfo_fill(fbi);
    pxafb_map_video_memory(fbi, mod);
    pxafb_check_var(&fbi->fb.var, &fbi->fb);
    pxafb_set_par(&fbi->fb);
    pxafb_backlight_power = spark_backlight_power;
    pxafb_lcd_power = spark_lcd_power;
#if 1
    {extern void disable_i_d_cache(void);
        disable_i_d_cache();
    }
#endif
}

void pxa_lcd_enable(void)
{ 
    pxafb_enable_controller(&pxafb);
    AddMessage((UINT8_T*) ("pxafb_enable_controller finish\0"));
    __pxafb_lcd_power(&pxafb, 1);
    AddMessage((UINT8_T*) ("__pxafb_lcd_power finish\0"));
    __pxafb_backlight_power(&pxafb, 1);
    AddMessage((UINT8_T*) ("__pxafb_backlight_power finish\0"));
}

void pxa_lcd_off(void)
{
    struct pxafb_info *fbi;

    fbi = &pxafb;

    __pxafb_backlight_power(fbi, 0);
    __pxafb_lcd_power(fbi, 0);
    pxafb_disable_controller(fbi);
}

void get_lcd_pixel(uint32_t *data, uint32_t x, uint32_t y)
{
    uint32_t *fb_addr = (uint32_t *)(pxafb.screen_dma + 4*x + y*640*4);
    //uint32_t *fb_addr = (uint32_t *)(pxafb.screen_dma + 4*x + y*2*320*4);
    *data = *fb_addr;
}

void set_lcd_pixel(uint32_t data, uint32_t x, uint32_t y)
{
    uint32_t *fb_addr = (uint32_t *)(pxafb.screen_dma + 4*x + y*640*4);
    //uint32_t *fb_addr = (uint32_t *)(pxafb.screen_dma + 4*x + y*320*4*2);
    *fb_addr = data;
}

extern int gunzip2(void *out_buf, void *in_buf, unsigned long in_len);
void pxa_load_logo(pTIM pTIM_h, P_FlashProperties_T pFlashProp)
{
#if 0
    UINT_T logo_flash_addr, i;

    for (i = 0; i < pTIM_h->pConsTIM->NumImages; i++)
    {
        if (pTIM_h->pImg[i].ImageID == LOGOIDENTIFIER)    
            break;
    }

    if (i == pTIM_h->pConsTIM->NumImages)
    {
        AddMessage("No logo in the flash");
        return;
    }
    else
        logo_flash_addr = pTIM_h->pImg[i].FlashEntryAddr;

#endif
    AddMessage("Loading logo ...");
    Flex_ReadFlash(0x00A00000, 256*1024, (P_DFC_BUFFER)___Bvideo_memory_screen_dma_ddr_addr_logo, pFlashProp);
}

int check_panic(P_FlashProperties_T pFlashProp)
{
    UINT32_T *id, *panic_msg = NULL;

    /* load panic from flash to logo ram address. */
    Flex_ReadFlash(0x0FA00000, 256*1024, (P_DFC_BUFFER)___Bvideo_memory_screen_dma_ddr_addr_logo, pFlashProp);
    panic_msg = (UINT32_T *)___Bvideo_memory_screen_dma_ddr_addr_logo;

    poweron_reason_print("panic message = ", *panic_msg, 4); 
    /* judge the panic_msg at the head of panic space. if it equ 0xffffffff, it show that there has panic, else 
     * display logo.
     */
    if (0xffffffff != *panic_msg)
    {
        lcd_printf("kernel log",230,330,2,FONT_COLOR_RED,FONT_COLOR_BLACK,LCD_DRAWMODE_NORMAL);
#if 0
        LCD_RECT rect;
        rect.x0 = 0;
        rect.x1 = 320;
        rect.y0 = 0;
        rect.y1 = 480;
        lcd_fill_rect(&rect,FONT_COLOR_GREEN);

        rect.x0 = 60;
        rect.x1 = 320-52;
        rect.y0 = 53;
        rect.y1 = 480-100-3;
        lcd_printf("logo picture not found",rect.x0,200,1,FONT_COLOR_RED,FONT_COLOR_GREEN,LCD_DRAWMODE_NORMAL);
        AddMessage("logo picture not found.");

#endif
        return 1;
    }

    return 0;
}

void pxa_logo(void)
{
    UINT32_T *id;
    u32 *p = (u32 *)0x81000000, *q = NULL;


    id = (UINT32_T *)___Bvideo_memory_screen_dma_ddr_addr_logo;
    if(id[0] == LOGOIDENTIFIER)
    {
        char *in_buf,*out_buf;
        UINT32_T in_len, i;

        AddMessage("display logo ...");
        out_buf = pxafb.screen_dma;
        in_buf  = (char*)(___Bvideo_memory_screen_dma_ddr_addr_logo + ___Bvideo_memory_screen_dma_ddr_addr_logo_data_offset);
        in_len  = id[1];
        gunzip2(out_buf, in_buf, in_len);
        #if 0
        {
            int i;
            char *src,*dst;
            src = (char*)(___video_memory_screen_dma_ddr_addr_logo + ___video_memory_screen_dma_ddr_addr_logo_data_offset);
            dst = pxafb.screen_dma;
            for(i = 0;i < 320*480*4;i++)
            {
                *dst++ = *src++;
            }
        }
        #endif
        AddMessage("OK");
    }
}
