/******************************************************************************
**
**  COPYRIGHT (C) 2005 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	PinMux_defs.h
**
**  PURPOSE: 	This file provides basic type definitions for A0 chip model
**
******************************************************************************/


// Tavor pin MUX  definitions//
#ifndef _PINMUX_DEFS_H_
#define _PINMUX_DEFS_H_

#include "gpio_defs.h"


// pull_sel, pullup_en, pulldn_en - bits 15:13
#define PULL_SEL_AF         0x0000
#define N_AF_SLP_DN			0x2000 /* in normal mode the AF affect the pull, in sleep a pull down is conected*/
#define N_AF_SLP_UP			0x4000 /* in normal mode the AF affect the pull, in sleep a pull up is conected*/
#define NO_PULL				0x8000
#define PULL_SEL_DN         0xA000
#define PULL_SEL_UP         0xC000




// drive - bits 12:10  ==== For P only
#define FAST_1mA	 0x0000
#define FAST_2mA	 0x0400
#define FAST_3mA	 0x0800
#define FAST_4mA	 0x0C00
#define SLOW_6mA	 0x1000
#define FAST_6mA	 0x1400
#define SLOW_10mA	 0x1800
#define FAST_10mA	 0x1C00


// drive - bits 12:11  ==== For PV/p65 only
#define SLOW_0	 	0x0000
#define SLOW	 	0x0800
#define MEDIUM	 	0x1000
#define FAST	 	0x1800


// sleep_sel - bit 9
#define SLEEP_APPS    0x000    //BPMU firewall signal controls the sleep mode of the pin
//#define SLEEP_COMM    0x200    //CPMU firewall signal controls the sleep mode of the pin

/* elevy - in addition to bit 9 we force bit 6 to '1' since when the pad is comm wakeup event the
clear is done via another register (wrst) in the comm side. this bit must be 1 to allow proper clear via comm side*/
#define SLEEP_COMM        0x240    //CPMU firewall signal controls the sleep mode of the pin
#define SLEEP_COMM_RDH    0x248    //bit 3 set to "1" to enable RDH control.

// sleep_data - bit 8:7
#define SLEEP_DATA_OTPUT_LOW       0x000
#define SLEEP_DATA_OTPUT_HIGH      0x100
#define SLEEP_DATA_INPUT           0x080

// edge_clear, edge_fall_en, edge_rise_en - bits 6:4
#define EDGE_RISING_ENABLE         0x10
#define EDGE_FALLING_ENABLE        0x20
#define EDGE_BOTH_ENABLE           0x30
#define EDGE_DISABLE               0x40

// af_sel - bits 2:0
#define AF_0		0x0
#define AF_1		0x1
#define AF_2		0x2
#define AF_3		0x3
#define AF_4		0x4
#define AF_5		0x5
#define AF_6	    0x6
#define AF_7		0x7
#define AF_DEF    AF_0


#define BOERNE_GPIO_BASE      0x40E10000

#define NO_KEYPAD_WAS_PRESS     	0xFF
#define OBM_DEBUG_MODE				0x06


typedef enum
{
	LOW,
	HIGH
} OUTPUT_LEVEL;



int setTavorPinMux ( UINT8_T   user_keypad_press );
int setTavorI2CPinMux ( void );
int setI2CPinsToGpio ( void );
void sendClocksOnI2cClkPin ( void );
void setGpioToOutput ( GpioPins   gpio_num , OUTPUT_LEVEL   level );


#define MAX_MODE		5
#define MAX_GPIO_TP		198        // For Tavor P
#define MAX_GPIO_TPV	215        // For Tavor PV
#define MAX_GPIO_TP65	196        // For Tavor P65


#if (PV_CHIP)
#define MAX_GPIO          MAX_GPIO_TPV
#elif (P65)
#define MAX_GPIO          MAX_GPIO_TPV      //MAX_GPIO_TP65
#else
#define MAX_GPIO          MAX_GPIO_TP
#endif


#if (EVB2_PV)
#define  P65   1		// To allow pin-mux of PV and P65 to be in the same bin file
#endif

#if (P65)
#define  PV    1		// To allow pin-mux of PV and P65 to be in the same bin file
#endif


typedef enum
{
#if (EVB2)
	BOARD_EVB2 ,
#endif
#if (YARDEN)
	BOARD_YARDEN ,
#endif
#if (SAAR)
#if (!TRUSTED)
	BOARD_SAAR ,                                      // SAAR 2.5
#endif
	BOARD_SAAR_2_6 ,                                  // SAAR 2.6
#endif
#if (SAAR_PV)
	BOARD_SAAR_PV ,                                   // SAAR PV 3.1
	BOARD_SAAR_PV_3_2 ,                               // SAAR PV 3.2
#endif
#if (EVB2_PV)
	BOARD_PV ,
#endif
#if (P65)
	BOARD_P65 ,
#endif
	MAX_BOARD
} BOARD_TYPE;


typedef struct
{
	 unsigned int     padAddr;
	 unsigned int     ModeVals[MAX_MODE];
} PINMUX_ENTRYS ;


typedef struct
{
	 PINMUX_ENTRYS     gpio_pin[MAX_GPIO];
} GPIO_ENTRYS ;


typedef struct
{
	 GPIO_ENTRYS     board[MAX_BOARD];
} Tavor_PinmuxEntryS ;





#define PADENTRY(mfprAddr, af0, drive0, pullsel0, edge0, sleepsel0, sleepctrl0,  				  \
						   af1, drive1, pullsel1, edge1, sleepsel1, sleepctrl1,  				  \
						   af2, drive2, pullsel2, edge2, sleepsel2, sleepctrl2,  				  \
						   af3, drive3, pullsel3, edge3, sleepsel3, sleepctrl3,  				  \
						   af4, drive4, pullsel4, edge4, sleepsel4, sleepctrl4)  				  \
																								  \
						   			(unsigned int)(BOERNE_GPIO_BASE|mfprAddr), 						  \
									{(af0 | drive0 | pullsel0 | edge0 | sleepsel0 | sleepctrl0),  \
									 (af1 | drive1 | pullsel1 | edge1 | sleepsel1 | sleepctrl1),  \
									 (af2 | drive2 | pullsel2 | edge2 | sleepsel2 | sleepctrl2),  \
									 (af3 | drive3 | pullsel3 | edge3 | sleepsel3 | sleepctrl3),  \
									 (af4 | drive4 | pullsel4 | edge4 | sleepsel4 | sleepctrl4)}  \

#endif //_PINMUX_DEFS_H_


