/******************************************************************************

**	(C) Copyright 2007 Marvell International Ltd. ?

**	All Rights Reserved
******************************************************************************/

#include "protocolmanager.h"
#include "PRDCFG_Element_lookup_tbl.h"
#include "PRDCFG_General.h"
#include "obm_pxa_lcd.h"
#include "flash.h"
#include "tim.h"
#include "misc.h"
#include "platformconfig.h"
#include "obm_version.h"
#include "obm_micco.h"

#include "data_to_os.h"

#if TRUSTED
#include "obm_security.h"
#endif
#include "powerreason.h"
#if MDOC_CODE
#include "msys.h"
#endif

#if ONENAND_CODE
#include "FlexOBMInterface.h"
#endif

#if TAVOR_BOERNE
#include "PinMux_defs.h"
#include "obm_version.h"
#include "i2c.h"
#include "micco_def.h"
#include "product_id.h"
#include "reliable_data.h"
#if (USBCI)
#include "interrupts.h"
#endif

#define RELEASE_COMM_RESET		1

#define RUN_SW_UPGRADE			1
#define OBM_LCD_SUPPORT         1
#define MICCO_ADC_MUX_VBAT      0x00
#define MICCO_ADC_MUX_VERSION   0x06
#ifdef SAAR
#define VOLTAGE_3_4     0x135
#define VOLTAGE_3_5     0x150
#else
#endif
#define VOLTAGE_3_4     0x125
#define VOLTAGE_3_5     0x140
#define VOLTAGE_2_9     0x060
#define VOLTAGE_3_2     0x140
#define CHARGE_VOLTAGE_2_9     0x077
#define CHARGE_VOLTAGE_3_2     0x0F0
#endif


#define NVM_BLOCK_ADDR 0x009A0000 
#define NVM_BLOCK_SIZE (384 * 1024)
#define NVM_TMP_BUFFER 0x80500000
#define BATTERY_ELEMENT_RAM_ADDR 0x80400000
#define FACTORY_MODE_FLAG 0xA500


extern UINT32  ObmXsGetProcessorVersion (void);
extern UINT32  ObmXsGetDebuggerStatus (void);
extern void TransferControl(UINT_T, UINT_T, UINT_T);
extern UINT_T SaveParamsForOsLoader(FUSE_SET  Fuses , UINT32_T   TlvAddress );
extern UINT_T SaveParamsForEboot(FUSE_SET  Fuses);
extern unsigned short CheckBoardVersion(void);
extern int micco_adc_read(UINT8_T mux);
extern void poweron_reason_print( char* str, int data, int bytes);
extern void pxa_load_logo(pTIM pTIM_h, P_FlashProperties_T pFlashProp);

typedef enum
{
    LOW_CHARGE_MODE,   
    MID_CHARGE_MODE,   
    HIGH_CHARGE_MODE,   
}CHARGE_MODE;


#if TRUSTED
void OBM_ReEnableJtag ( P_FlashProperties_T     pFlashProp );
#endif


void load_prdcfg_element(P_FlashProperties_T  pFlashProp);
void fota_handle(P_FlashProperties_T pFlashProp);
#ifdef SAAR_PV
static void start_charging(void);
static void stop_charging(void);
#endif

static void change_charging_mode(CHARGE_MODE mode);
static UINT_T DownloadDKBImage( pTIM pTIM_h, P_FlashProperties_T pFlashProperties);
extern void determine_power_supply(P_FlashProperties_T pFlashProp);
extern int check_flashmode_status( void );
extern powerup_reason_t powerup_reason;


#if TAVOR_BOERNE

#define OSC_32K_BYPASS					1

#define SW_DOWNLOAD_KEY_PRESS				0x10
#define JTAG_REENABLE_KEY_PRESS 			0x20


///////   The defines below are for debug mode only
#define DEBUG_LOAD_ADDR_1_BASE			0x83000000
#define DEBUG_LOAD_ADDR_2_BASE			0x83900000
#define DEBUG_NUMBER_OF_IMAGES			0x2
#define DEBUG_IMAGE_1_SIZE_IN_BLOCK		0x1B
#define DEBUG_IMAGE_2_SIZE_IN_BLOCK		0x1

#if ((SAAR) && (ONENAND_CODE))

#define DEBUG_FLASH_ADDR_1_BASE			0x0ede0000
#define DEBUG_FLASH_ADDR_2_BASE			0x0f160000

#elif ((EVB2_PV) && (NOR_CODE))
//The reason is that  DEBUG_NOR_IMAGE_1_SIZE_IN_BYTES = DEBUG_FLASH_ADDR_2_BASE - DEBUG_FLASH_ADDR_1_BASE and this is equal to 0x00380000
#define DEBUG_NOR_IMAGE_1_SIZE_IN_BYTES 0x00380000
#define DEBUG_NOR_IMAGE_2_SIZE_IN_BYTES 0x00020000
#define DEBUG_FLASH_ADDR_1_BASE			0x02200000
#define DEBUG_FLASH_ADDR_2_BASE			0x02580000

#else   // SAAR

#define DEBUG_FLASH_ADDR_1_BASE			0x06b00000
#define DEBUG_FLASH_ADDR_2_BASE			0x06e80000

////////////////////////////// End of debug mode
#endif  // SAAR



static void OBM_PrintFlashType ( FlashType_T   FlashType );
void OBM_PrintWakeupReason ( void );
void OBM_ClearBIE ( void );
static void OBM_ReadImagesSize ( pTIM     pTIM_h , P_FlashProperties_T    pFlashP );
static void OBM_ConfigVccComm ( void );
static void OBM_ConfigSmemForNor ( void );
static void OBM_ConfigNorCsGpio ( void );
static UINT8_T OBM_CheckKeypadStatus ( void );
static void OBM_CopyDebugImagesToDDR ( P_FlashProperties_T    pFlashP );
static void releaseGilonReset ( void );
static UINT_T OBM_ReadFromFlash ( UINT_T   DDR_A , UINT_T   ImageSize , UINT_T   Flash_SA , P_FlashProperties_T    pFlashP );
static UINT8_T	OBM_CheckSignature ( void );
static void	OBM_IMEI_DeCode ( UINT_T *DDRaddr );
static void OBM_DebugModeHandle ( P_FlashProperties_T    pFlashProp );
OBM_BOARD_TYPE OBMGetBoardType ( void );
OBM_UsbType OBMGetUsbType ( void );
UINT_T OBM_CopyAllImagesByTim ( pTIM    pTIM_h , P_FlashProperties_T    pFlashProp );


/***************************  ONLY FOR PV2 Z0  ******************************/
#define PV2_DDR_A          0xBFE00000
#define PV2_IMAGE_SIZE     0x00100000
#define PV2_FLASH_ADDR     0x00720000
UINT_T OBM_CopyPv2Images( P_FlashProperties_T    pFlashP );
/****************************************************************************/


#if ((SAAR) || (SAAR_PV))
static void DisableIrda ( void );
static void AudioMonoConfig ( void );
static void AudioStereoConfig ( void );
static void GPS_ShutDown ( void );
#endif



typedef struct   arbelResetRoutineS
{
	VUINT32_T   Arbel_ResetRoutine[8];    /* LONGs - 32 bit */
} ARBEL_ResetRoutineS;

typedef struct   GbResetRoutineS
{
	VUINT16_T   Gb_ResetRoutine[16];      /* Shorts */
} GB_ResetRoutineS;


#define ADDRCONVERT_2COMM(aDDR)     ((aDDR & 0x00ffffff) | 0xD0000000)         // BFxx.xxxx -> D0xx.xxxx
#define ADDRCONVERT_2APPS(aDDR)	    ((aDDR & 0x00ffffff) | 0xBF000000)         // D0xx.xxxx -> BFxx.xxxx

#define APPS_MPMU_CSER   						0x40F5001C

#define COMM_IMAGE_ARBEL_MIN_LEGAL_ADDR_DST	   (ADDRCONVERT_2APPS(0xD0800000))	// we will page ARBEL image into DDR -- for 8M.
#define ARBEL_LOAD_TABLE_LEGAL_BASE_ADD	       (COMM_IMAGE_ARBEL_MIN_LEGAL_ADDR_DST + 0x00200) // This is for 8M DDR size - Use additional offset +0x200 since beginning used by JTAG scripts

#define RESET_ROUTINE_GB_ADDR		(ADDRCONVERT_2APPS(0xD0FFFFC0)) /* The address in which GB starts after COMM is released from reset */
#define RESET_ROUTINE_ARBEL_ADDR	(ADDRCONVERT_2APPS(0xD0FFFFE0)) /* The address in which Arbel  starts after COMM is released from reset */

static GB_ResetRoutineS     g_gbReset = { 0xE108, 0x8000 , 0xE148, 0xD0F8, 0x0050 };
static ARBEL_ResetRoutineS  g_arbelReset = {0xE51FF004, 0xD0A00000, 0xA1A2A3A4, 0xA1A2A3A4};



#define NOR_ADDR_BASE						(0x10000000)
#define NOR_READ_SIGNATURE_SIZE				(0xff)

#define ARBEL_LOAD_TABLE_OFFSET      		(0x1C0)
#define ARBEL_LOAD_ADDR_BASE      			(0x80100000)
#define LOAD_TABLE_SIGNATURE_STR  			"LOAD_TABLE_SIGN"  	// 15 + zeroEnd

#define RELAIBLE_DATA_FLASH_BLOCK_NUM		(8)          		// The data reside in block No. 7 in the flash

// !!!! This size MUST be align to the flash page size and smaller than 100K (sharedFreeEnd - sharedFreeBegin) !!!!!
#define RELAIBLE_DATA_SIZE				(0x16800)     		// 90KB - 10KB = 80KB bytes // Make sure the size is align to page size 2K - see bellow offset.
#define RELAIBLE_DATA_OFFSET_IN_DDR		(0x2800)     		// 10KB bytes - offset of 10KB for other use.


static char load_table_signature_str[] = LOAD_TABLE_SIGNATURE_STR;



#define ARSR_RESET_REASON           (ARSR_HARDWARE_RESET | ARSR_WATCHDOG_RESET | ARSR_LOW_PWR_MODE_RESET | ARSR_GPIO_RESET)


#define PSR_SS2S 					(1)
#define PSR_SS3S 					(2)
#define PSR_BFS 					(4)
#define PSR_PMU_STATUS      		(PSR_SS2S | PSR_SS3S | PSR_BFS)



#define PMCR_CLEARE_BIE_BIT			(0xfffffffe)


typedef struct
{
	unsigned long 	imageBegin;       	// image addresses are in HARBELL address space 0xD0??.????
	unsigned long 	imageEnd;         	//                 for BOERNE use conversion to 0xBF??.????
	char   			Signature[16];
	unsigned long 	sharedFreeBegin;
	unsigned long 	sharedFreeEnd;
	unsigned long 	ramRWbegin;
	unsigned long 	ramRWend;
	char   			spare[12];        	/* This area may be used for data transfer from loader to COMM */
	char			OBM_Version[8];
	unsigned long 	OBM_Data32bits;
}LoadTableType; 						/*total size 0x40=64bytes */

static LoadTableType *LoadTable = (LoadTableType *)(ARBEL_LOAD_ADDR_BASE + ARBEL_LOAD_TABLE_OFFSET);

#endif  // TAVOR_BOERNE



#ifdef OSC_32K_BYPASS
#define DELAY_AFTER_OSCC_WRITE 		0x500 		/*>100ms*/
#define OSCC_TCS_BIT_READ 			( *OSCC & 0x8000)
#define OSCC_CSSC_SET  				( *OSCC |= 0x2000);
#define OSCC_CSS_BIT_IS_SET			(( *OSCC & 0x4000 ) == 0x4000 )
#endif

#if (RUN_SW_UPGRADE)
#define BootROMDataArea  			0x5C008000
#endif

#if (RUN_SW_UPGRADE)       					// Remote SW upgrade for development
#define   RUN_SW_UPGRADE_DELAY		1       // This flag make a delay to the SW update trigger from USB
#endif
#if defined (RUN_SW_UPGRADE_DELAY)
UINT_T OBM_WaitForSwUpdateTrigger ( void );
#endif
#define SW_UPGRADE_DELAY  		5000        // Delay of up to 1000mSec


typedef enum
{
	USB_CABLE_DISCONNECTED,
	USB_CABLE_CONNECTED
} USB_CABLE_STATUS;

static MICCO_TYPE   			micco_type;
static UINT8					ADC_in6;
static OBM_ProModeState		    ProductionState;


/************************************************************************************/
/************************************************************************************/
void OBM_updtaeOsBuffre ( OBM_TO_OS_ID   data_entry , DATA_AMOUNT   data_amount , UINT32_T   *data_value )
{
	UINT8		i;
	OBM_OS_DATA_HEADER		    *data_header = (volatile UINT32_T  *)(OBM_OS_BUFFRE_BASE_ADDRESS);
	static  volatile UINT32_T	*buffre_ptr  = (volatile UINT32_T  *)(OBM_OS_DATA_FIRST_ENTRY_ADDRESS);


	*buffre_ptr++ = data_entry;

	if ( data_entry != DATA_TERMINATOR_ID )
	{
		*buffre_ptr++ = data_amount;

		for ( i = 0; i < data_amount; i++ )
		    *buffre_ptr++ = data_value[i];
	}
	else
	{
		data_header->signature = DATA_SIGNATURE_ID;
		data_header->version = OBM_OS_DATA_VERSION;
	}
}
/************************************************************************************/
/************************************************************************************/
void OBM_UpdateDdrSize ( void )
{
	UINT32_T				 ddr_size , mdcnfg_reg_value;
	volatile unsigned long  *DMEMREGS_MDCNFG_PHYSICAL_BASE = ((volatile unsigned long  *)0x48100000);


	mdcnfg_reg_value = *DMEMREGS_MDCNFG_PHYSICAL_BASE;
	if ( mdcnfg_reg_value & 0x8 )                           // Check bit 3 in the MDCNFG -- if 1 means 128M
		ddr_size = OBM_128M_BYTES_SIZE;
	else
#if ((SAAR) || (SAAR_PV))
		ddr_size = OBM_128M_BYTES_SIZE;
#else
		ddr_size = OBM_64M_BYTES_SIZE;
#endif

	OBM_updtaeOsBuffre ( DDR_SIZE_ENTRY , DATA_AMOUNT_1 , &ddr_size );

#if ((SAAR) || (SAAR_PV))
	ddr_size = OBM_DDR_BANK_2;           // We have two banks of 64M bytes.
#else
	ddr_size = OBM_DDR_BANK_1;           // We have one banks of 64M bytes in Tavor-P or one bank of 128M in Tavor-PV.
#endif

	OBM_updtaeOsBuffre ( DDR_BANK_ENTRY , DATA_AMOUNT_1 , &ddr_size );

}
/************************************************************************************/
/************************************************************************************/
void OBM_UpdateMvedExistence ( void )
{
	OBM_MvedExist          MvedExistence = MVED_NOT_EXIST;

	if ( OBMGetUsbType() == OBM_USBCI )
	   MvedExistence = MVED_EXIST;
	OBM_updtaeOsBuffre ( MVED_EXIST_ENTRY , DATA_AMOUNT_1 , &MvedExistence );

}
/************************************************************************************/
/************************************************************************************/
void OBM_UpdateUsbType ( void )
{
	UINT32_T              	 UsbType;
	UINT8_T Msg[] =          "-Running USB1 and 2\0";


#if (EVB2)
	UsbType = OBM_USB1_2;				// Assuming both usb1 and 2 (ULPI) are exists.
#else
	UsbType = OBMGetUsbType();
#endif

	OBM_updtaeOsBuffre ( USB_TYPE_ENTRY , DATA_AMOUNT_1 , &UsbType );

	switch ( UsbType )
	{
		case OBM_USB1 :
					   	  Msg[13] = '\0';
						  break;

		case OBM_USB2 :
						  Msg[12] = '2';                // ULPI
					   	  Msg[13] = '\0';
						  break;

		case OBM_USBCI :
						  Msg[12] = 'C';
					   	  Msg[13] = 'I';
					   	  Msg[14] = '\0';
						  break;

		case OBM_USB1_2 :
						  break;

		     default   :
					 	  break;
	}

 	AddMessage(Msg);

}
/************************************************************************************/
/************************************************************************************/
void OBM_UpdateMmcType ( void )
{
	UINT32_T	             MmcCap0 , MmcType = OLD_MMC;
	volatile unsigned long  *MMC_CAP0_REGISTER_ADDR = ((volatile unsigned long  *)0x55000040);


	MmcCap0 = (*MMC_CAP0_REGISTER_ADDR & 0x0fe33fff );
	if ( MmcCap0 == 0x09e330b0 )
		MmcType = NEW_MMC;

	OBM_updtaeOsBuffre ( MMC_TYPE_ENTRY , DATA_AMOUNT_1 , &MmcType );

}
/************************************************************************************/
/************************************************************************************/
OBM_ProModeState OBM_GetProductionModeState ( void )
{

	return ProductionState;

}
/************************************************************************************/
/************************************************************************************/
pOPT_PRODUCT_MODE OBM_FindTimProductionMode ( pTIM    pTIM_h , OBM_ProductionMode  *Mode )
{
    pWTP_RESERVED_AREA_HEADER 	pWRAH;
	UINT_T 						Retval = NoError;
	pOPT_PRODUCT_MODE		    ProductionData = NULL;


	pWRAH = FindPackageInReserved ( &Retval , pTIM_h , PRODUCTIONID );
	if ( Retval == NoError )
	{
		ProductionData = (pOPT_PRODUCT_MODE) pWRAH;
		*Mode = ProductionData->product_state;
	}

	return ProductionData;
}
/************************************************************************************/
/************************************************************************************/
OBM_ProductionMode OBM_ReadProductionMode ( pTIM    pTIM_h , pOPT_PRODUCT_MODE    *data_ptr )
{
	OBM_ProductionMode 			Mode = PRODUCTION_MODE_DISABLE;
	UINT_T 						Retval = NoError;
	pOPT_PRODUCT_MODE		    ProductionData;


	ProductionState = NON_PRODUCTION_MODE_STATE;
	ProductionData = OBM_FindTimProductionMode ( pTIM_h , &Mode );
	if ( ProductionData != NULL )
	{
		if ( ProductionData->product_state == PRODUCTION_MODE_ENABLE )
		{
		   *data_ptr = ProductionData;
	   	 	Mode = PRODUCTION_MODE_ENABLE;
			ProductionState = PRODUCTION_MODE_STATE;
		}
		else
		{
			if ( ProductionData->product_state == PRODUCTION_MODE_EXTERNAL )
			{
				if (( ADC_in6 > EVB2_PRODUCTION_LOW_LIMIT ) && ( ADC_in6 < EVB2_PRODUCTION_HIGH_LIMIT ))
				{
				   *data_ptr = ProductionData;
	   				Mode = PRODUCTION_MODE_ENABLE;
					ProductionState = PRODUCTION_MODE_STATE;
				}
			}
		}
	}

	return Mode;
}
/************************************************************************************/
/************************************************************************************/
void OBM_UpdateProductionMode ( pTIM     pTIM_h )
{
	UINT32_T	   	 	 Mode = PRODUCTION_MODE_DISABLE;
	OPT_PRODUCT_MODE	*ProductionData;


	Mode = OBM_ReadProductionMode( pTIM_h , &ProductionData );
	if ( Mode == PRODUCTION_MODE_ENABLE )
	{
		OBM_updtaeOsBuffre ( PRODUCT_MODE_TYPE_ENTRY,
							((ProductionData->WRAH.Size - sizeof(WTP_RESERVED_AREA_HEADER)) >> 2),
							((((UINT8_T *)ProductionData) + sizeof(WTP_RESERVED_AREA_HEADER))) );
		AddMessage((UINT8_T*) ("-Production mode was detected.\0"));
	}
	else
		OBM_updtaeOsBuffre ( PRODUCT_MODE_TYPE_ENTRY , DATA_AMOUNT_1 , &Mode );

}
/************************************************************************************/
/************************************************************************************/
void UpdateDataToOsLoader ( void )
{
	UINT32_T           boardTypeToOs;

	boardTypeToOs = OBMGetBoardType();
	OBM_updtaeOsBuffre ( BOARD_ENTRY , DATA_AMOUNT_1 , (UINT32_T *)&boardTypeToOs );

	OBM_UpdateDdrSize ();
	OBM_UpdateUsbType ();
	OBM_UpdateMmcType ();
	OBM_UpdateMvedExistence ();
	OBM_updtaeOsBuffre ( OBM_VERSION_ENTRY , DATA_AMOUNT_2 , OBM_VERSION );
	OBM_updtaeOsBuffre ( GPS_AVAILABLE_ENTRY , DATA_AMOUNT_1 , OBM_VERSION );     // Dummy parameter - just for display
	OBM_updtaeOsBuffre ( DATA_TERMINATOR_ID , DATA_AMOUNT_0 , NULL );
}
/************************************************************************************/
/************************************************************************************/
OBM_UsbType OBMGetUsbType ( void )
{
	UINT32_T	             UsbIdBits , UsbIdBits_N;
	unsigned int 			 usb_default_port;
	OBM_UsbType              UsbType = OBM_USB1;



	usb_default_port = GetDefaultUsbPort();

	switch ( usb_default_port )
	{
		case DIFF_USB_D :
							UsbType = OBM_USB1;
						  	break;

		case U2D_USB_D :
						  	UsbType = OBM_USB2;
						  	break;

		case CI2_USB_D :
						  	UsbType = OBM_USBCI;
						  	break;

		     default   :
					 	  	break;
	}

	return UsbType;
}
/************************************************************************************/
/************************************************************************************/
void OBM_Wait32kBeStabilize ( void )
{
#ifdef OSC_32K_BYPASS
		volatile unsigned long i = DELAY_AFTER_OSCC_WRITE;

//	if ( OSCC_CSS_BIT_IS_SET )    // This check was removed probably because of HW problem in the OSCC register
	{
		AddMessage((UINT8_T*) ("-Waiting for 32K oscillator to be stabilize...\0"));

		while ( OSCC_TCS_BIT_READ == 0 );
		OSCC_CSSC_SET;
		while ( i > 0 ) 				/*according to spec: delay for at least 2 cycle between consecutive write cycles*/
			i--;
	}
#endif
}
/************************************************************************************/
/************************************************************************************/
OBM_BOARD_TYPE OBMGetBoardType ( void )
{

#if (EVB2)
	return OBM_EVB_P_BOARD;
#elif (YARDEN)
	return OBM_YARDEN_BOARD;
#elif (SAAR)
	if ( micco_type == OBM_MICCO_EB )
		return OBM_SAAR_2_6_BOARD;
	else
		return OBM_SAAR_2_5_BOARD;
#elif (SAAR_PV)
	if (( ADC_in6 > SAAR_PV_3_2_LOW_LIMIT ) && ( ADC_in6 < SAAR_PV_3_2_HIGH_LIMIT ))
		return OBM_SAAR_PV_3_2_BOARD;
	else
		return OBM_SAAR_PV_3_1_BOARD;
#elif (EVB2_PV)
	return OBM_EVB_PV_BOARD;
#elif (P65)
	return OBM_P65_BOARD;
#else
	return MAX_BOARD_TYPE;
#endif
}
/************************************************************************************/
/************************************************************************************/
#if ((SAAR) || (SAAR_PV))
static void DisableIrda ( void )
{
	OBM_BOARD_TYPE 		board_type;


	board_type = OBMGetBoardType();

#if (SAAR_PV)
	if (( board_type == OBM_SAAR_PV_3_1_BOARD ) || ( board_type == OBM_SAAR_PV_3_2_BOARD ))
#endif
#if (SAAR)
	if ( board_type == OBM_SAAR_2_6_BOARD )
#endif
	{
		setGpioToOutput ( DISABLE_IRDA , HIGH );
	}
}
#endif
/************************************************************************************/
/************************************************************************************/
#if (SAAR)
static void GPS_ShutDown ( void )
{

   	setGpioToOutput ( GPS_SHUTDOWN , LOW );

}
#endif
/************************************************************************************/
/************************************************************************************/
#if ((SAAR) || (SAAR_PV))
static void AudioMonoConfig ( void )
{

   	setGpioToOutput ( AUDIO_MONO_CONFIG , LOW );

}
/************************************************************************************/
/************************************************************************************/
static void AudioStereoConfig ( void )
{

   	setGpioToOutput ( AUDIO_STEREO_CONFIG , LOW );

}
#endif
/************************************************************************************/
/************************************************************************************/
#if ((EVB2) || (EVB2_PV))
static void releaseSkyworksReset ( void )
{

	setGpioToOutput ( RELEASE_SKYWORKS_FROM_RESET , HIGH );

}
#endif
/************************************************************************************/
/************************************************************************************/
static void releaseGilonReset ( void )
{

#if (SAAR_PV)
	if ( OBMGetBoardType() == OBM_SAAR_PV_3_2_BOARD )
		setGpioToOutput ( RELEASE_GILON_FROM_RESET_SAAR_PV_3_2 , HIGH );
	else
#endif
		setGpioToOutput ( RELEASE_GILON_FROM_RESET , HIGH );

}
/************************************************************************************/
/************************************************************************************/
#if defined (RELEASE_COMM_RESET)
static void releaseComm_c(void)
{
	GB_ResetRoutineS      *GBReset    = (GB_ResetRoutineS *)RESET_ROUTINE_GB_ADDR;
	ARBEL_ResetRoutineS   *ArbelReset = (ARBEL_ResetRoutineS *)RESET_ROUTINE_ARBEL_ADDR;


	*GBReset = g_gbReset;       	// =RESET_ROUTINE_GB
	*ArbelReset = g_arbelReset; 	// =RESET_ROUTINE_ARBEL

#ifdef RELEASE_COMM_MPMU_CSER
	AddMessage((UINT8_T*) ("-Release Comm from reset.\0"));
	*((volatile unsigned long*)(APPS_MPMU_CSER)) = 0x11;
#endif

}
#endif
/************************************************************************************/
/************************************************************************************/
UINT32_T ObmGetCpuFamily ( void )
{
     UINT32_T  			product_id , product_family;
	 UINT32_T           cpu_family;


	 product_id = *(PRODUCT_ID_REG_ADD);
	 product_family = ( product_id & PRODUCT_WITHIN_FAMILY );
	 if ( product_family == TAVOR_PV_SUPERSET )
		cpu_family = TAVOR_PV;
	 else
	    if ( product_family == TAVOR_PV2_SUPERSET )
		   cpu_family = TAVOR_PV2;
	 else
		   cpu_family = TAVOR_P;

	 return cpu_family;
}
/************************************************************************************/
/************************************************************************************/
CPU_VERSION ObmGetCpuVersion ( void )
{
	 CPU_VERSION    	cpuVersion;
     UINT32_T  			product_id , product_family;


	 product_id = *(PRODUCT_ID_REG_ADD);

	 switch ( product_id )
	 {
		case TAVOR_P_A0 :
            			  		cpuVersion = CPU_TAVOR_A0;
		            	  		break;
		case TAVOR_P_B0 :
            			  		cpuVersion = CPU_TAVOR_B0;
		            	  		break;
		case TAVOR_P_B1 :
            			  		cpuVersion = CPU_TAVOR_B1;
		            	  		break;
		case TAVOR_P_B2 :
            			  		cpuVersion = CPU_TAVOR_B2;
		            	  		break;
		case TAVOR_PV_DIS_Z0 :
            			  		cpuVersion = CPU_TAVOR_PV_DIS_Z0;
		            	  		break;
		case TAVOR_PV_DIS_A0 :
            			  		cpuVersion = CPU_TAVOR_PV_DIS_A0;
		            	  		break;
		case TAVOR_PV_DIS_B0 :
            			  		cpuVersion = CPU_TAVOR_PV_DIS_B0;
		            	  		break;
		case TAVOR_PV_DIS_B1 :
            			  		cpuVersion = CPU_TAVOR_PV_DIS_B1;
		            	  		break;
		case TAVOR_PV_DIS_C0 :
            			  		cpuVersion = CPU_TAVOR_PV_DIS_C0;
		            	  		break;
		case TAVOR_PV2_DIS_Z0 :
            			  		cpuVersion = CPU_TAVOR_PV2_Z0;
		            	  		break;
		case TAVOR_PV2_DIS_A0 :
            			  		cpuVersion = CPU_TAVOR_PV2_A0;
		            	  		break;
		case TAVOR_PV2_DIS_B0 :
            			  		cpuVersion = CPU_TAVOR_PV2_B0;
		            	  		break;
		case TAVOR_PV2_DIS_C0 :
            			  		cpuVersion = CPU_TAVOR_PV2_C0;
		            	  		break;
        default :
            			  		cpuVersion = CPU_OTHER;
		            	  		break;
	 }

    return cpuVersion;
}
/**************************************************************************************/
/**************************************************************************************/
void OBM_DisplayCpuType ( void )
{
	UINT32_T 	    CpuVersion;
	UINT32_T		product_family;
	UINT8_T  		CpuVer[] = "-CPU type is Tavor-P   B0.\0";


	CpuVersion = ObmGetCpuVersion();
	OBM_updtaeOsBuffre ( CHIP_ENTRY , DATA_AMOUNT_1 , &CpuVersion );

	switch ( CpuVersion )
	{
		case CPU_TAVOR_PV2_Z0 :
							CpuVer[23] = 'Z';
							break;
		case CPU_TAVOR_A0:
		case CPU_TAVOR_PV_DIS_A0 :
		case CPU_TAVOR_PV2_A0 :
							CpuVer[23] = 'A';
							break;
		case CPU_TAVOR_B0:
		case CPU_TAVOR_PV_DIS_B0 :
		case CPU_TAVOR_PV2_B0 :
		    				break;
		case CPU_TAVOR_B1:
		case CPU_TAVOR_PV_DIS_B1:
							CpuVer[24] = '1';
							break;
		case CPU_TAVOR_B2:
							CpuVer[24] = '2';
							break;
		case CPU_TAVOR_PV_DIS_C0 :
							CpuVer[23] = 'C';
		    				break;
	    default:
							CpuVer[23] = '?';
							CpuVer[24] = '?';
			         		break;
	}

	product_family = ObmGetCpuFamily();
	switch ( product_family )
	{
		case TAVOR_P :
							break;
		case TAVOR_PV:
							CpuVer[20] = 'V';
							CpuVer[21] = 'D';
							break;
		case TAVOR_PV2:
							CpuVer[20] = 'V';
							CpuVer[21] = '2';
							break;
	    default:
							CpuVer[20] = '?';
							CpuVer[21] = '?';
			         		break;
	}

 	AddMessage(CpuVer);

}
/**************************************************************************************/
/**************************************************************************************/
USB_CABLE_STATUS OBM_GetUsbCableStatus ( void )
{
	UINT8_T 			dataPtr[2];
	USB_CABLE_STATUS    cable_status;


	dataPtr[0] = 0x06;           // Register address in Micco to be read -- checking if USB cable is connected
	dataPtr[1] = 0x0;            // Clear data area to be read
	I2CMiccoReceiveData ( dataPtr );

	if ( ( dataPtr[1] & 0x01 ) == 0x01 ) 	// Means USB cable is connected
		cable_status = USB_CABLE_CONNECTED;
	else
		cable_status = USB_CABLE_DISCONNECTED;

	return cable_status;
}
/**************************************************************************************/
/**************************************************************************************/
I2C_ReturnCode OBM_ReadMiccoVersion ( void )
{
	UINT8_T 			dataPtr[2];
	UINT8_T Msg[] =     "-Micco type is EA\0";
	I2C_ReturnCode		rc;


	// Checking Micco version
	dataPtr[0] = 0x00;           // Register address in Micco to be read -- checking Micco version
	dataPtr[1] = 0x0;            // Clear data area to be read
	rc = I2CMiccoReceiveData ( dataPtr );
	if ( rc != I2C_RC_OK )
		return rc;

	micco_type = dataPtr[1];
	OBM_updtaeOsBuffre ( MICCO_ENTRY , DATA_AMOUNT_1 , (UINT32_T *)&dataPtr[1] );

	switch ( micco_type )
	{
		case OBM_MICCO_A0 :
					   	  	Msg[15] = 'A';
					   	  	Msg[16] = '0';
						  	break;

		case OBM_MICCO_B0 :
					   	  	Msg[15] = 'B';
					   	  	Msg[16] = '0';
						  	break;

		case OBM_MICCO_C0 :
					   	  	Msg[15] = 'C';
					   	  	Msg[16] = '0';
						  	break;

		case OBM_MICCO_EA :
						  	break;

		case OBM_MICCO_EB :
					   	  	Msg[15] = 'E';
					   	  	Msg[16] = 'B';
						  	break;

        default :
					   	  	Msg[15] = 'X';
					   	  	Msg[16] = 'X';
						  	break;
	}

 	AddMessage(Msg);

}
/**************************************************************************************/
/**************************************************************************************/
#if defined (PERFORM_FUSE_PROGRAM_WA)
I2C_ReturnCode OBM_ChangeMvtVoltageForBinding ( void )
{
	UINT8_T 			dataPtr[2];
	I2C_ReturnCode		rc;


	if ( micco_type >= OBM_MICCO_EA )
	{
		AddMessage((UINT8_T*) ("-Changing the MVT voltage to 1.9V.\0"));
		dataPtr[0] = 0x33;           // Register address in Micco to be update -- Changing MDTV2 register
		dataPtr[1] = 0x18;           // Data to be writen --                      Changing MDTV2 voltage to 1.9V
		rc = I2CMiccoSendData ( dataPtr );
		if ( rc != I2C_RC_OK )
			return rc;

		dataPtr[0] = 0x20;           // Register address in Micco to be update -- Changing VCC1 register
		dataPtr[1] = 0xC0;           // Data to be writen --                      Select MDTV2
		rc = I2CMiccoSendData ( dataPtr );
		if ( rc != I2C_RC_OK )
			return rc;
	}
}
#endif
/**************************************************************************************/
/**************************************************************************************/
I2C_ReturnCode OBM_MiccoChangeAdc6State( OBM_MICCO_ADC6    adc6_state )
{
	UINT8_T 			dataPtr[2];
	I2C_ReturnCode		rc;


	dataPtr[0] = 0x50;           			// Register address in Micco to be read -- Read the ADC MAN CONTROL
	dataPtr[1] = 0x0;            			// Clear data area to be read
	rc = I2CMiccoReceiveData ( dataPtr );
	if ( rc != I2C_RC_OK )
		return rc;


	if ( adc6_state == MICCO_ADC6_ENABLE )
		dataPtr[1] |= MICCO_ADC_ENABLE_REF_VOLT_SUPPLY;  // Data to be writen -- set bit 4   == Enable reference voltage supply for ADC
	else
		dataPtr[1] &= ~MICCO_ADC_ENABLE_REF_VOLT_SUPPLY; // Data to be writen -- clear bit 4 == Disable reference voltage supply for ADC

	rc = I2CMiccoSendData ( dataPtr );
	if ( rc != I2C_RC_OK )
		return rc;


	dataPtr[0] = 0x52;           			// Register address in Micco to be read -- Read the ADC AUTO CONTROL
	dataPtr[1] = 0x0;            			// Clear data area to be read
	rc = I2CMiccoReceiveData ( dataPtr );
	if ( rc != I2C_RC_OK )
		return rc;


	if ( adc6_state == MICCO_ADC6_ENABLE )
		dataPtr[1] |= MICCO_ADC_ENABLE_AUTO_MEASUREMENT; // Data to be writen -- set bit 2   == Enable Auto ADC measurement for ADC in6
	else
		dataPtr[1] &= ~MICCO_ADC_ENABLE_AUTO_MEASUREMENT;// Data to be writen -- clear bit 2 == Disable Auto ADC measurement for ADC in6
	rc = I2CMiccoSendData ( dataPtr );
	if ( rc != I2C_RC_OK )
		return rc;

}
/**************************************************************************************/
/**************************************************************************************/
I2C_ReturnCode OBM_MiccoReadAdcIn6 ( void )
{
	UINT8_T 			dataPtr[2];
	I2C_ReturnCode		rc;


	ADC_in6 = 0;
	dataPtr[0] = 0x6B;           			// Register address in Micco to be read -- Read the ADC in6 voltage
	dataPtr[1] = 0x0;            			// Clear data area to be read
	rc = I2CMiccoReceiveData ( dataPtr );
	if ( rc != I2C_RC_OK )
		return rc;

	ADC_in6 = dataPtr[1];

	return rc;
}
/**************************************************************************************/
/**************************************************************************************/
void OBM_MiccoDisableWD ( void )
{
	UINT8_T 			dataPtr[2] , already_display = FALSE;
	I2C_ReturnCode		rc;


	AddMessage((UINT8_T*) ("-Disabling Micco WD.\0"));
	dataPtr[0] = 0x0B;           // Register address in Micco to be update
	dataPtr[1] = 0x0;            // Clear data area to be read

	do
	{
		rc = I2CMasterReceiveDataDirect ( &dataPtr[0] ,
			 						 	   MICCO_CMD_1_BYTE_SIZE,
					 					   PMC_I2C_WRITE_SLAVE_ADDRESS,
					                      &dataPtr[1],
					 					   MICCO_READ_1_BYTE,
					 					   PMC_I2C_READ_SLAVE_ADDRESS );

		if (( rc != I2C_RC_OK ) && ( already_display == FALSE ))
		{
			I2CErrorHandle ( I2C_READ_CYCLE );
			already_display = TRUE;
		}
	}
	while ( rc != I2C_RC_OK );

	dataPtr[1] &= 0xFB;          // Data to be writen -- clear bit 2 == Disable WD
	dataPtr[1] |= 0x80;          // Data to be writen -- set bit 7   == Reset WD
	already_display = FALSE;

	do
	{
	    rc = I2CMasterSendDataDirect ( dataPtr , MICCO_WRITE_2_BYTE , PMC_I2C_WRITE_SLAVE_ADDRESS );
		if (( rc != I2C_RC_OK ) && ( already_display == FALSE ))
		{
			I2CErrorHandle ( I2C_WRITE_CYCLE );
			already_display = TRUE;
		}
	}
	while ( rc != I2C_RC_OK );

}
/**************************************************************************************/
/**************************************************************************************/
I2C_ReturnCode OBM_MiccoSetup ( void )
{
	UINT8_T 			dataPtr[2];
	I2C_ReturnCode		rc;



	dataPtr[0] = 0x05;           // Register address in Micco to be read -- checking if charger is connected
	dataPtr[1] = 0x0;            // Clear data area to be read
	rc = I2CMiccoReceiveData ( dataPtr );
	if ( rc != I2C_RC_OK )
		return rc;

	if ( ( dataPtr[1] & 0x08 ) == 0x08 ) 	// Means charger is connected
	{
		AddMessage((UINT8_T*) ("-Enable charging.\0"));
		dataPtr[0] = 0x42;           // Register address in Micco to be update
		dataPtr[1] = 0x8A;           // Data to be writen -- set bit 7  == Enable charging,  the A is the current ( Do not change the current )
		rc = I2CMiccoSendData ( dataPtr );             // PAY ATTENTION -- Do not change the current
		if ( rc != I2C_RC_OK )                         //--------------------------------------------
			return rc;
	}


	if ( micco_type >= OBM_MICCO_EA )
	{
	    if ( micco_type == OBM_MICCO_EA )
		{
			AddMessage((UINT8_T*) ("-Turn Buck_3 to 1.8 Volt.\0"));
			dataPtr[0] = 0x2c;           // Register address in Micco to be update
			dataPtr[1] = 0x00;           // Data to be writen -- Write 0  == Change the voltage to be 1.8 volt
		    rc = I2CMiccoSendData ( dataPtr );
			if ( rc != I2C_RC_OK )
				return rc;
		}

		dataPtr[0] = 0x1f;           // Register address in Micco to be update
		dataPtr[1] = 0x01;           // Data to be writen -- Write 1  == Select voltage in register 3DTV2
		rc = I2CMiccoSendData ( dataPtr );
		if ( rc != I2C_RC_OK )
			return rc;


		// Enable RF_GP switch output - first read the regiser value
		dataPtr[0] = 0x12;           // Register address in Micco to be read
		dataPtr[1] = 0x0;            // Clear data area to be read
	    rc = I2CMiccoReceiveData ( dataPtr );
		if ( rc != I2C_RC_OK )
			return rc;

		AddMessage((UINT8_T*) ("-Enable RF_GP switch and Disable LDO4.\0"));
	    dataPtr[1] &= 0xBF;          // Data to be writen -- clear bit 6 == Disable LDO4
		dataPtr[1] |= 0x04;          // Data to be writen -- set bit 2   == Enable RF_GP switch output
		rc = I2CMiccoSendData ( dataPtr );
		if ( rc != I2C_RC_OK )
			return rc;

#if 0
		dataPtr[0] = 0x3D;           // Register address in Micco to be update
		dataPtr[1] = 0x00;           // Data to be writen -- Write 0  == Turn off LCD backlight
		rc = I2CMiccoSendData ( dataPtr );
		if ( rc != I2C_RC_OK )
			return rc;
#endif
	}



	// Set LDO13 to 2.8V for SD card  - first read the regiser value
	dataPtr[0] = 0x17;           // Register address in Micco to be read
	dataPtr[1] = 0x0;            // Clear data area to be read
	rc = I2CMiccoReceiveData ( dataPtr );
	if ( rc != I2C_RC_OK )
		return rc;


	AddMessage((UINT8_T*) ("-Set LDO13 to 2.8V for SD card.\0"));
	dataPtr[1] &= 0xF0;          // Data to be writen -- clear four lsb bits
	dataPtr[1] |= 0x0a;          // Data to be writen -- write to the four lsb bits   == Set LDO13 to 2.8V
	rc = I2CMiccoSendData ( dataPtr );
	if ( rc != I2C_RC_OK )
		return rc;



//	AddMessage((UINT8_T*) ("-Turn off the 32KHz oscillator.\0"));
//	dataPtr[0] = 0x46;           // Register address in Micco to be update
//	dataPtr[1] = 0x00;           // Data to be writen -- cleart bit 0  == Turn off the 32KHz oscillator
//	rc = I2CMiccoSendData ( dataPtr );
//	if ( rc != I2C_RC_OK )
//		return rc;



	AddMessage((UINT8_T*) ("-Changing Apps, Comm, Sram voltage ramp.\0"));
	dataPtr[0] = 0x25;           // Register address in Micco to be update -- Changing Apps voltage ramp
	dataPtr[1] = 0x18;           // Data to be writen -- cleart bits 0-2  == One step per every 1MicroSec
	                             //                         Set bits 3-4  == 12.5 mV ramps steps
	rc = I2CMiccoSendData ( dataPtr );
	if ( rc != I2C_RC_OK )
		return rc;


	dataPtr[0] = 0x28;           // Register address in Micco to be update -- Changing Comm voltage ramp
	dataPtr[1] = 0x18;           // Data to be writen -- cleart bits 0-2  == One step per every 1MicroSec
	                             //                         Set bits 3-4  == 12.5 mV ramps steps
	rc = I2CMiccoSendData ( dataPtr );
	if ( rc != I2C_RC_OK )
		return rc;


	dataPtr[0] = 0x2b;           // Register address in Micco to be update -- Changing Sram voltage ramp
	dataPtr[1] = 0x18;           // Data to be writen -- cleart bits 0-2  == One step per every 1MicroSec
	                             //                         Set bits 3-4  == 12.5 mV ramps steps
	rc = I2CMiccoSendData ( dataPtr );
	if ( rc != I2C_RC_OK )
		return rc;


#if (PV_CHIP)
	AddMessage((UINT8_T*) ("-Changing Sram voltage to 1.25V.\0"));
	dataPtr[0] = 0x23;           // Register address in Micco to be update --  ADTV1
	dataPtr[1] = 0x15;           // Data to be writen --  Changing the Sram voltage to 1.25v
	rc = I2CMiccoSendData ( dataPtr );
	if ( rc != I2C_RC_OK )
		return rc;


	dataPtr[0] = 0x20;           // Register address in Micco to be update --  VCC1
	dataPtr[1] = 0x01;           // Data to be writen --  Set bit 0 == Allowing change the Sram voltage
	rc = I2CMiccoSendData ( dataPtr );
	if ( rc != I2C_RC_OK )
		return rc;
#endif

	return rc;

}
/************************************************************************************/
/************************************************************************************/
#if (RUN_SW_UPGRADE)
void DownloadDKBToTarget(pTIM pTIM_h, P_FlashProperties_T pFlashProp)
{
	UINT_T 				JumpOffset = 0;
	FUSE_SET 			temp;
	UINT8_T Msg[] =    	"-USB cable disconnected...waiting to be connected\0";

#if (DISABLE_DEFAULT_USB_ENUM)
	unsigned int usb_default_port;

	usb_default_port = GetDefaultUsbPort();
	SetUpUSBDescriptors (NULL, 0, usb_default_port);
	PlatformUsbInit(usb_default_port);
	SetPortAsInitialize(usb_default_port);
#endif

	if ( OBM_GetUsbCableStatus() == USB_CABLE_DISCONNECTED )
	{
 	    AddMessage(Msg);
		while ( OBM_GetUsbCableStatus() == USB_CABLE_DISCONNECTED );
	}

	Msg[11] = 'i';
	Msg[12] = 's';
	Msg[13] = ' ';
	Msg[23] = '\0';
 	AddMessage(Msg);

	AddMessage((UINT8_T*) ("*** Loading DKB ***\0"));
	DownloadDKBImage( pTIM_h , &pFlashProp[0] );
 	AddMessage((UINT8_T*) ("-XFer Control to DKB...\0"));
	ShutdownPort(FFUART_D);
	ShutdownPort(ALTUART_D);
	ShutdownPort(DIFF_USB_D);
	ShutdownPort(U2D_USB_D);
	DisableIrqInterrupts();
	temp.value = 0;
	SaveParamsForOsLoader((FUSE_SET)temp , OBM_OS_BUFFRE_BASE_ADDRESS);
	JumpOffset = pTIM_h->pImg[1].LoadAddr;
	TransferControl( JumpOffset , pTIM_h->pImg[1].ImageSize, OBM_DDR_DOWNLOAD_ADDR);
}
#endif// RUN_SW_UPGRADE

/************************************************************************************/
/************************************************************************************/
void goto_rescue_flash_mode( pTIM   pTIM_h , P_FlashProperties_T    pFlashProp )
{

    /* Modified by b035 , if do not add extra string , flash will fail , will check the root cause */
    AddMessage((UINT8_T*) ("***Into the rescue flash mode***\0"));
    DownloadDKBImage( pTIM_h , &pFlashProp[0] );
    ShutdownPort(FFUART_D);
    ShutdownPort(ALTUART_D);
    ShutdownPort(CI2_USB_D);
    ShutdownPort(DIFF_USB_D);
#if SAAR
    ShutdownPort(U2D_USB_D);
#endif
    DisableIrqInterrupts();
    {
        UINT_T 		JumpOffset = 0;
        FUSE_SET	fuse_set_tmp;

        fuse_set_tmp.value = 0;
        SaveParamsForEboot((FUSE_SET)fuse_set_tmp);
        JumpOffset = pTIM_h->pImg[1].LoadAddr;
        TransferControl( JumpOffset , pTIM_h->pImg[1].ImageSize, OBM_DDR_DOWNLOAD_ADDR);
    }

}


/*
 * yu wang:load nvm data. pass the battery calibration data to blob, and decide factory mode via flag in nvm.
 */
void load_prdcfg_element(P_FlashProperties_T  pFlashProp)
{
    int i;
    UINT_T mode_element_buffer; 

    UINT_T battery_element_offset = prdcfg_phasing_element_table[PRDCFG_PHASING_VOLTAGE].offset + 1;
    UINT_T mode_element_offset = prdcfg_phasing_element_table[PRDCFG_PHASING_ENG_MODE_FLAG].offset + 1;

    UINT_T battery_element_size = prdcfg_phasing_element_table[PRDCFG_PHASING_VOLTAGE].length - 4;
    UINT_T mode_element_size = prdcfg_phasing_element_table[PRDCFG_PHASING_ENG_MODE_FLAG].length - 4;
    
/*for debug*/
#if 0
    poweron_reason_print("battery_element_offset = ", battery_element_offset, battery_element_size);
    poweron_reason_print("battery_element_size = ", battery_element_size, 4);
    poweron_reason_print("mode_element_offset = ", mode_element_offset, mode_element_size);
    poweron_reason_print("mode_element_size = ", mode_element_size, 4);
#endif
    

#if SAAR_PV
    Flex_ReadFlash(NVM_BLOCK_ADDR, NVM_BLOCK_SIZE, (P_DFC_BUFFER)NVM_TMP_BUFFER, pFlashProp);   
#else
    ReadNANDWithBBM(NVM_BLOCK_ADDR, NVM_BLOCK_SIZE, (P_DFC_BUFFER)NVM_TMP_BUFFER, pFlashProp);   
#endif 
    for (i = 0; i < battery_element_size; i++)
    {
        *(PUCHAR)(BATTERY_ELEMENT_RAM_ADDR + i) = *(PUCHAR)(NVM_TMP_BUFFER + battery_element_offset + i);
    }
    //because endian, we have to read twice.
    mode_element_buffer = *(PINT8)(NVM_TMP_BUFFER + mode_element_offset+1);
    mode_element_buffer <<= 8;
    mode_element_buffer |= *(PINT8)(NVM_TMP_BUFFER + mode_element_offset);

    
/*for debug*/
#if 0
    for (i = 0; i < (battery_element_size); i++)
    {
        poweron_reason_print("", *(unsigned char*)(BATTERY_ELEMENT_RAM_ADDR+i), battery_element_size);
    }

    poweron_reason_print("mode_element_buffer= ", mode_element_buffer, 4);
#endif
    if (mode_element_buffer != FACTORY_MODE_FLAG)
        return;

    powerup_reason = PU_REASON_USB_FACTORY;
    SavePowerUpReason( powerup_reason );
    AddMessage("factory mode!");
}


UINT_T PlatformOBMPrebootCode ( pTIM   pTIM_h , P_FlashProperties_T    pFlashProp )
{
#if (RUN_SW_UPGRADE)
	UINT8_T     updataSw;
#endif
	UINT8_T 	keypad_status = 0;
	int				pin_mux_rc;

    AddMessage("UART flash key");
    Delay_ms(100);
    if((ReadFFUartLineStatus() & 0x1) == 0x1)
    {
        AddMessage("uart received data...");
        if (ReadFFUartReceiveFIFO() == 'r')   /* key 'f' into the flash mode */
            goto_rescue_flash_mode( pTIM_h , &pFlashProp[0] );
    }

#ifdef SAAR_PV
	gpio_direction_output(MFP_PIN_GPIO167, 1); /* shutdown cap sensor, else the i2c will meet error on blaze */
    Delay(100000);
    AddMessage("GPIO167!!!");
#endif


 	AddMessage(ObmVersionMsg);

	OBM_DisplayCpuType();

	OBM_PrintFlashType ( pFlashProp[0].FlashType );

#ifdef OSC_32K_BYPASS
	OBM_Wait32kBeStabilize();
#endif

	I2CConfigure();

	pin_mux_rc = setTavorI2CPinMux();
	if ( pin_mux_rc == 0 )              		// 0 means An Error
	 	return GeneralError;

	AddMessage((UINT8_T*) ("-I2C module has been configured.\0"));

	I2CBusHungHandle();

#ifndef DDR_WORKAROUND
    micco_probe();
#endif
	OBM_MiccoDisableWD();

#ifndef DDR_WORKAROUND
    power2saving_micco();
#endif

#if (RUN_SW_UPGRADE)
#if (USBCI)
	if ( OBMGetUsbType() == OBM_USBCI )
	{
		if ( OBM_GetUsbCableStatus() == USB_CABLE_CONNECTED )
		{
  			if ( GetPortInterruptFlag() == FALSE )
			{
				CI2Enable();
			}
		}
	}
#endif

	OBM_MiccoChangeAdc6State( MICCO_ADC6_ENABLE );
	OBM_ReadMiccoVersion();

#if TRUSTED
	AddMessage((UINT8_T*) ("-Enabling Caddo unit.\0"));
	EnableDisableCaddo(ENABLE_CADDO);
	WtmReset(1);
#endif

#if 1
 if(updataSw != TRUE)
    {
        /*make lcd to be a stabilization status. we reset it.*/
        lcd_panel_reset();
        determine_power_supply(pFlashProp);
        updataSw = check_flashmode_status();

        if(updataSw != TRUE)
        {
            AddMessage("UART flash key");
            Delay_ms(100);
            if((ReadFFUartLineStatus() & 0x1) == 0x1)
            {
                AddMessage("uart received data...");
                if (ReadFFUartReceiveFIFO() == 'f')   /* key 'f' into the flash mode */
                    updataSw = TRUE;
                if (ReadFFUartReceiveFIFO() == 'u')
                {
                    fota_handle(pFlashProp);
                }
            }
        }
    }
#endif


    /* Support 'Software Upgrade' of SWDownloader feathers. */
    /* if usb connected, then delay 5s until usb receive upgrade protocol packet. else jump this delay operation. */
#if (RUN_SW_UPGRADE)
    if ( OBM_GetProductionModeState() == NON_PRODUCTION_MODE_STATE )
    {
        OBM_WaitForSwUpdateTrigger ();
        if (CheckSwUpdateTriggeredFromUsb() == TRUE) 
        {
            AddMessage((UINT8_T*)"SW upgrade Flash mode");
            updataSw = TRUE;
        }
    }
#endif

    if ( updataSw == TRUE )
	{	
        AddMessage((UINT8_T*) ("*** Flash Key pressed ...\0"));
#if defined(OBM_LCD_SUPPORT)
        LCD_RECT rect;
        pxa_charger_set_e1_e2(0x11);// turn off charger
        {
#if 1
            pxa_lcd_ini(___video_flash_mod);
            pxa_lcd_enable();
            int i,j;
#ifdef SAAR_PV
            {
                rect.x0 = 0;
                rect.y1 = 360 - 1;
                rect.y0 = 0;
                rect.x1 = 640-1;
                lcd_fill_rect(&rect,FONT_COLOR_BLACK);

                rect.x0 = 50;
                rect.y1 = 360-50;
                rect.y0 = 50;
                rect.x1 = 700-100;
                lcd_draw_rect(&rect,FONT_COLOR_GREEN);

                rect.x0 = 50+1;
                rect.y1 = 360-50-1;
                rect.y0 = 50+1;
                rect.x1 = 700-100-1;
                lcd_draw_rect(&rect,FONT_COLOR_BLUE);


                rect.x0 += 2;
                rect.y1 -= 1;
                rect.y0 += 2;
                rect.x1 -= 2;
                lcd_fill_rect(&rect,FONT_COLOR_GREEN);
                
                lcd_printf("flash mode!",180,100,3,FONT_COLOR_RED,FONT_COLOR_GREEN,LCD_DRAWMODE_NORMAL);

                lcd_printf("(C) Copyright 2008 Borqs  Ltd.",55,320,2,FONT_COLOR_WHITE,FONT_COLOR_BLACK,LCD_DRAWMODE_NORMAL);
            }
#else
            {
                rect.x0 = 0;
                rect.x1 = 320-1;
                rect.y0 = 0;
                rect.y1 = 480-1;
                lcd_fill_rect(&rect,FONT_COLOR_BLACK);

                rect.x0 = 50;
                rect.x1 = 320-50;
                rect.y0 = 50;
                rect.y1 = 480-100;
                lcd_draw_rect(&rect,FONT_COLOR_GREEN);
                rect.x0 = 50+1;
                rect.x1 = 320-50-1;
                rect.y0 = 50+1;
                rect.y1 = 480-100-1;
                lcd_draw_rect(&rect,FONT_COLOR_BLUE);


                rect.x0 += 2;
                rect.x1 -= 1;
                rect.y0 += 2;
                rect.y1 -= 2;
                lcd_fill_rect(&rect,FONT_COLOR_GREEN);
                
                lcd_printf("flash mode!",rect.x0+8,200,2,FONT_COLOR_RED,FONT_COLOR_GREEN,LCD_DRAWMODE_NORMAL);

                lcd_printf("(C) Copyright 2008 Borqs  Ltd.",8,480-30,1,FONT_COLOR_WHITE,FONT_COLOR_BLACK,LCD_DRAWMODE_NORMAL);
            }
#endif
#endif
        }
#endif
        /* Modified by b035 , if do not add extra string , flash will fail , will check the root cause */
        AddMessage((UINT8_T*) ("***Flash Key Pressed Loading DKB ***\0"));
		DownloadDKBImage( pTIM_h , &pFlashProp[0] );
		ShutdownPort(FFUART_D);
		ShutdownPort(ALTUART_D);
		ShutdownPort(CI2_USB_D);
		ShutdownPort(DIFF_USB_D);
#if SAAR
		ShutdownPort(U2D_USB_D);
#endif
		DisableIrqInterrupts();
#if defined(OBM_LCD_SUPPORT)
        {
            //pxa_lcd_off(); 
#ifdef SAAR_PV
            lcd_printf("Downloading",180,100,3,FONT_COLOR_RED,FONT_COLOR_GREEN,LCD_DRAWMODE_NORMAL);
#else
            lcd_printf("Downloading",rect.x0+8,200,2,FONT_COLOR_RED,FONT_COLOR_GREEN,LCD_DRAWMODE_NORMAL);
#endif
        }
#endif
        {
            UINT_T 		JumpOffset = 0;
            FUSE_SET	fuse_set_tmp;

            fuse_set_tmp.value = 0;
            SaveParamsForEboot((FUSE_SET)fuse_set_tmp);
            JumpOffset = pTIM_h->pImg[1].LoadAddr;
            TransferControl( JumpOffset , pTIM_h->pImg[1].ImageSize, OBM_DDR_DOWNLOAD_ADDR);
        }
	}
    AddMessage((UINT8_T*) ("*** No Flash Key pressed ...\0"));

    switch (check_fota_mode(pFlashProp))
    {
       case NORMAL_FOTA_MODE: 
            AddMessage("NORMAL_FOTA_MODE");
            fota_handle(pFlashProp);
            break;
       case RECOVERY_FOTA_MODE: 
            AddMessage("RECOVERY_FOTA_MODE");
			SavePowerUpReason( PU_REASON_REBOOT_RECOVERY );
            break;
       case NON_FOTA:
       default:
            AddMessage("this is no valid fota identifier");
            break;
    }

#if defined(OBM_LCD_SUPPORT)
    pxa_lcd_ini(___video_boot_mod);
#endif
#ifndef DDR_WORKAROUND
/* judge the version is P1 or P2, if P2, check voltage. 
 * if the voltage less than 3.5V, it will circle until the voltage more than 3.5V.
 */
    int version;
    version = CheckBoardVersion();      
    poweron_reason_print("the version value from micco = ", version, 4);

    if ((version >= 0x0) && (version < 0x10F))  
    {                                           
        AddMessage((UINT8_T*) ("board version is P2.\0"));  
        PrechargCheck();
    }                                     
#else
    PrechargCheck();
#endif

#if defined(OBM_LCD_SUPPORT)
    /* if there has panic, display panic picture, else load logo to display. */
    pxa_load_logo(pTIM_h, pFlashProp);                  /* load the logo to ram from flash */
    {
        extern void enable_i_d_cache(void);
        enable_i_d_cache();
    }
    pxa_logo();                                 /* fill into the frame buff */
    {
        extern void disable_i_d_cache(void);
        disable_i_d_cache();
    }

    check_panic(pFlashProp);
    check_assert_poweron_reason();

    pxa_lcd_enable();
#endif
        //########################tmp############################
#if 0
    pxa_lcd_off();
#endif
    //########################tmp############################




#if TRUSTED
		if ( keypad_status == JTAG_REENABLE_KEY_PRESS )
		{
			int    i , delaytime = (int)(TIMER_FREQ(900*1000*1000));      // Needed a delay of 30 Min -- it was split to avoid overlap -- the timer run 3.25M


			AddMessage((UINT8_T*) ("-Waiting for JTAG Re-enable command...\0"));

			for ( i = 0; i < 2; i++ )
			{
    			TIMER_CLEAR;
				while ( TIMER_GET < delaytime )
					OBM_ReEnableJtag ( pFlashProp );
			}
		}
#endif  // TRUSTED

#endif // RUN_SW_UPGRADE


	OBM_PrintWakeupReason();
	OBM_ClearBIE();

#if (!TRUSTED)
	OBM_ReadImagesSize ( pTIM_h , pFlashProp );
#endif


#if (SAAR)
	GPS_ShutDown();
#endif

	OBM_MiccoReadAdcIn6();
	OBM_MiccoChangeAdc6State( MICCO_ADC6_DISABLE );

	OBM_UpdateProductionMode ( pTIM_h );

	return NoError;
}
/************************************************************************************/
/************************************************************************************/
UINT_T PlatformOBMPostbootCode(pTIM pTIM_h, P_FlashProperties_T pFlashProp)
{
#if (TAVOR_BOERNE)
	UINT_T 			Retval = NoError;
	UINT8_T 		keypad_status;
	int				pin_mux_rc;
	I2C_ReturnCode	rc;
	CPU_VERSION		cpu_ver;



	// Handling with the I2C -- Micco
	rc = OBM_MiccoSetup();
	if ( rc != I2C_RC_OK )
		AddMessage((UINT8_T*) ("******* I2C Got error while accessing Micco *******\0"));

	keypad_status = OBM_CheckKeypadStatus();

	// Handling with Images copy from FLASH to DDR
	if ( keypad_status == OBM_DEBUG_MODE )
	{
		OBM_DebugModeHandle ( pFlashProp );
	}
	else
	{
		Retval = OBM_CopyAllImagesByTim ( pTIM_h , pFlashProp );
		if ( Retval != NoError )
		    return Retval;
	}

	cpu_ver = ObmGetCpuVersion();
	if ( cpu_ver == CPU_TAVOR_PV2_Z0 )
		 OBM_CopyPv2Images( pFlashProp );

	// Handling with the pin-mux setup
	pin_mux_rc = setTavorPinMux( keypad_status );
	if ( pin_mux_rc == 0 )              // 0 means an Error
	 	return GeneralError;

#if ((NAND_CODE) || (NOR_CODE))
  	OBM_ConfigNorCsGpio();

	OBM_ConfigSmemForNor();
#endif

	OBM_ConfigVccComm();


#endif     // TAVOR_BOERNE

#ifdef RELEASE_COMM_RESET
	releaseComm_c ();
#endif

#if ((SAAR) || (SAAR_PV))
	DisableIrda();
	AudioMonoConfig ();
	AudioStereoConfig ();
#endif

	releaseGilonReset();
#if ((EVB2) || (EVB2_PV))
	releaseSkyworksReset();
#endif

	UpdateDataToOsLoader();


#if 0
/* this feather is move to before determine poweron reason.*/
/* Because we can't make sure when the usb can reveive upgrade packet. */
#if (RUN_SW_UPGRADE)       // Remote SW upgrade for development
	if ( OBM_GetProductionModeState() == NON_PRODUCTION_MODE_STATE )
	{
		OBM_WaitForSwUpdateTrigger ();

		if ( CheckSwUpdateTriggeredFromUsb() == TRUE )
			DownloadDKBToTarget ( pTIM_h , &pFlashProp[0] );
	}
#endif // RUN_SW_UPGRADE
#endif

	return Retval;
}
/************************************************************************************/
/************************************************************************************/
UINT_T OBM_WaitForSwUpdateTrigger ( void )
{
	UINT16_T    num_of_loop = SW_UPGRADE_DELAY;

#if defined (RUN_SW_UPGRADE_DELAY)
	if (( OBM_GetUsbCableStatus() == USB_CABLE_CONNECTED ) &&
		( CheckSwUpdateTriggeredFromUsb() == FALSE ) )
	{

		AddMessage((UINT8_T*) ("-Waiting for SW update trigger..."));

		do
		{
		   Delay ( 1000 );   	      			// Delay in uSec ---> 1000uSec == 1mSec
		} while (( CheckSwUpdateTriggeredFromUsb() == FALSE ) && ( num_of_loop-- > 0 ));
	}
#endif // RUN_SW_UPGRADE_DELAY

	return num_of_loop;
}
/************************************************************************************/
/************************************************************************************/
void OBM_DebugModeHandle ( P_FlashProperties_T    pFlashProp )
{

	AddMessage((UINT8_T*) ("***********     RUNNING  DEBUG  MODE     ***********"));

	OBM_CopyDebugImagesToDDR ( pFlashProp );

	*((volatile UINT_T *)0x40f50114) = 0x00000082;
	*((volatile UINT_T *)0x83b00000) = 0x02116d0d;
	*((volatile UINT_T *)0x83b00004) = 0xffffffff;
	*((volatile UINT_T *)0x83b00008) = 0x00ff00fb;

}
/************************************************************************************/
/************************************************************************************/
UINT_T OBM_CopyAllImagesByTim ( pTIM    pTIM_h , P_FlashProperties_T    pFlashProp )
{
	UINT_T 			Retval = NoError;
	UINT8_T 		ImageOrder = 1;				// Keep Track of Image Order, First Image is TIM->OBM->AnyImage
	UINT8_T  		LoadMessage[] = "-Loading - xxxx Image to DDR...";

// For each image, copy (if needed), and validate
		for (ImageOrder = 0;  ((ImageOrder < pTIM_h->pConsTIM->NumImages) && (Retval == NoError)); ImageOrder++)
		{
			// Skip the OBM, TIM, OSLO
              if(
#if (SAAR_PV)
                (pTIM_h->pImg[ImageOrder].ImageID == ARBELID) ||  //  copy CP in OBM
                (pTIM_h->pImg[ImageOrder].ImageID == _BMQID) ||
                (pTIM_h->pImg[ImageOrder].ImageID == GBID) ||
#endif
				(pTIM_h->pImg[ImageOrder].ImageID == IMEIID)
                ) 
            {
    			// Copy the image
	    		LoadMessage[11] = *(((UINT8_T *)&pTIM_h->pImg[ImageOrder].ImageID)+3);
		    	LoadMessage[12] = *(((UINT8_T *)&pTIM_h->pImg[ImageOrder].ImageID)+2);
			    LoadMessage[13] = *(((UINT8_T *)&pTIM_h->pImg[ImageOrder].ImageID)+1);
    			LoadMessage[14] = *(((UINT8_T *)&pTIM_h->pImg[ImageOrder].ImageID)+0);
 	    		AddMessage(LoadMessage);
                
                /*  The size of IMEI and Tavor_M05 is stationary, so we only destine the Arbel file size. */
                if (pTIM_h->pImg[ImageOrder].ImageID == ARBELID) /*Arbel_NVM_SAC_NOCOMMRTC.bin*/
                {
	                AddMessage((UINT8_T*) ("Force copy CP size as a Large  size !!!"));
                    pTIM_h->pImg[ImageOrder].ImageSize = 0x500000;
                }

			    Retval = CopyImageFlashToLocal ( pTIM_h->pImg[ImageOrder].ImageID , pTIM_h , pFlashProp , NULL );
    			if ( Retval != NoError )
	    		{
		    		AddMessage((UINT8_T*) ("******* Error: Image loading failed"));
			        if ( pTIM_h->pImg[ImageOrder].ImageID != IMEIID )
				        return Retval;
    				else
	    				Retval = NoError;
                }

                if ( pTIM_h->pImg[ImageOrder].ImageID == IMEIID )
                {
                    //OBM_IMEI_DeCode((UINT *)0xBF407B10 );  //for debug
                    //AddMessage("Temp disable the function for fail reason on some boards\0");
                    poweron_reason_print("Decode imei load address = ", ADDRCONVERT_2APPS(LoadTable->sharedFreeBegin) + RELAIBLE_DATA_OFFSET_IN_DDR, 4);
                    OBM_IMEI_DeCode( ADDRCONVERT_2APPS(LoadTable->sharedFreeBegin) + RELAIBLE_DATA_OFFSET_IN_DDR);
                }
                // Validate the Image
#if TRUSTED
                if ( OBM_GetProductionModeState() == NON_PRODUCTION_MODE_STATE )     // This is done to save time in production
                {
                    AddMessage((UINT8_T*) ("-Validate the Image..."));
                    Retval = ValidateImage ( pTIM_h->pImg[ImageOrder].ImageID , NULL , pTIM_h );
                }
#else
                AddMessage((UINT8_T*) ("-Verifying Checksums..."));
                Retval = ValidateCheckSum ( pTIM_h->pImg[ImageOrder].ImageID , pTIM_h , &pFlashProp[0] );
#endif  // TRUSTED

                if (Retval != NoError)
                {
                    AddMessage((UINT8_T*) ("-Image Verifying failed"));
                    return Retval;
                }
            }//If it is the same cp
        }//For each image        

	return Retval;
}
/************************************************************************************/
/************************************************************************************/
#if (RUN_SW_UPGRADE)
UINT_T DownloadDKBImage(pTIM pTIM_h, P_FlashProperties_T pFlashProperties)
{
	UINT_T	ImageIdentifier;
	UINT32_T ImageDLLocation, saved_location;
	UINT8_T	DownloadingDone;
	FUNC_STATUS RetStatus;

	ImageIdentifier = TIMIDENTIFIER;
	ImageDLLocation = BootROMDataArea;
	DownloadingDone = FALSE;
	do{
		RetStatus = HandleRequest(ImageDLLocation, ImageIdentifier, pFlashProperties);
		ClearPortInterruptFlag();
		if(RetStatus.ErrorCode != NoError) break;
		if(RetStatus.StatusCode == DebugBootIssued) continue;
		if(RetStatus.StatusCode == DebugCmdIssued) continue;
		if(RetStatus.StatusCode == JtagREIssued) continue;
		if(ImageIdentifier == TIMIDENTIFIER)
		{
			RetStatus.ErrorCode = LoadTim(NULL, (UINT8_T*)ImageDLLocation, pTIM_h);
			if(RetStatus.ErrorCode != NoError) break;
		}
#if TRUSTED
		if(ImageIdentifier == TIMIDENTIFIER)
		{
			RetStatus.ErrorCode = ValidateTIM(pTIM_h);
		}
		else
		{
			RetStatus.ErrorCode = ValidateImage(ImageIdentifier, NULL, pTIM_h);
		}
#else
		RetStatus.ErrorCode = ValidateCheckSum(ImageIdentifier, pTIM_h, NULL);

#endif

		if(RetStatus.ErrorCode != NoError) break;
		if(ImageIdentifier == DKBIDENTIFIER)
			DownloadingDone = TRUE;
		else //we just finished validing TIM
		{	 //make next image to be downloaded the DKB
			ImageIdentifier = DKBIDENTIFIER;
			//download DKB to a predetermined address in ddr
			// overwrite 'load address' in TIM for DKB so DKB will
			// pass validation, then restore it after validation
			saved_location = pTIM_h->pImg[1].LoadAddr;
			pTIM_h->pImg[1].LoadAddr = (UINT_T) OBM_DDR_DOWNLOAD_ADDR;
			ImageDLLocation = (UINT_T) OBM_DDR_DOWNLOAD_ADDR;
		}

	}while(DownloadingDone == FALSE);
	if(DownloadingDone == TRUE)
	{
		HandleDisconnect();
		ClearPortInterruptFlag();
	}
	//restore load address
	pTIM_h->pImg[1].LoadAddr = saved_location;
	return RetStatus.ErrorCode;
}
#endif   // RUN_SW_UPGRADE
/************************************************************************************/
/************************************************************************************/
#if TAVOR_BOERNE
void OBM_PrintFlashType ( FlashType_T   FlashType )
{
	UINT32_T	FlashTypeToOs;

	switch ( FlashType )
	{
#if NOR_CODE
	case TYAX_FLASH :
		FlashTypeToOs = OBM_TYAX_FLASH;
		AddMessage((UINT8_T*) ("-Flash type is TYAX.\0"));
		break;

	case SIBLEY_FLASH :
		FlashTypeToOs = OBM_SIBLEY_FLASH;
		AddMessage((UINT8_T*) ("-Flash type is Sibley.\0"));
		break;
#endif

#if NAND_CODE
	case NAND_FLASH :
		FlashTypeToOs = OBM_NAND_FLASH;
		AddMessage((UINT8_T*) ("-Flash type is NAND.\0"));
		break;
#endif

#if ONENAND_CODE
	case OneNAND :
		FlashTypeToOs = OBM_ONENAND_FLASH;
		AddMessage((UINT8_T*) ("-Flash type is OneNAND.\0"));
		break;
#endif

#if MDOC_CODE
	case MSYS_DOC_FLASH :
		FlashTypeToOs = OBM_MSYS_DOC_FLASH;
		AddMessage((UINT8_T*) ("-Flash type is M-system DOC.\0"));
		break;
#endif

	default:
		FlashTypeToOs = OBM_UNKNOWN_FLASH;
		AddMessage((UINT8_T*) ("-Flash type is UnKnown.\0"));
		break;

	}

	OBM_updtaeOsBuffre ( FLASH_ENTRY , DATA_AMOUNT_1 , &FlashType);
}

/************************************************************************************/
/************************************************************************************/
void OBM_PrintWakeupReason ( void )
{
	UINT32_T         arsr_value , psr_value;
	UINT8_T Msg[]  = "-Last wakeup reason was POWER_ON.\0";
	UINT8_T Msg1[] = "-Last PMU status was - SS3S.\0";


	arsr_value = *ARSR & ARSR_RESET_REASON;
	psr_value = *PSR;

	switch ( arsr_value )
	{
		case 0 :
								break;

		case ARSR_HARDWARE_RESET :
								break;

		case ARSR_WATCHDOG_RESET :
								Msg[24] = 'W';
								Msg[25] = 'A';
								Msg[26] = 'T';
								Msg[27] = 'C';
								Msg[28] = 'H';
								Msg[29] = 'D';
								Msg[30] = 'O';
								Msg[31] = 'G';
								break;

		case ARSR_LOW_PWR_MODE_RESET :
								Msg[24] = 'L';
								Msg[25] = 'O';
								Msg[26] = 'W';
								Msg[27] = '_';
								Msg[28] = 'P';
								Msg[29] = 'W';
								Msg[30] = 'R';
								Msg[31] = ' ';
								break;

		case ARSR_GPIO_RESET :
								Msg[24] = 'G';
								Msg[25] = 'P';
								Msg[26] = 'I';
								Msg[27] = 'O';
								Msg[28] = '_';
								Msg[29] = 'R';
								Msg[30] = 'S';
								Msg[31] = 'T';
								break;

				       default :
								break;
	}

 	AddMessage(Msg);
	OBM_updtaeOsBuffre ( WAKEUP_REASON_ENTRY , DATA_AMOUNT_1 , &arsr_value);

	if ( psr_value & PSR_PMU_STATUS )
	{
		switch ( psr_value & PSR_PMU_STATUS )
		{
			case PSR_SS2S :
									Msg1[25] = '2';
							 break;

			case PSR_SS3S :
									Msg1[25] = '3';
							 break;

			case PSR_BFS  :
									Msg1[23] = 'B';
									Msg1[24] = 'F';
									Msg1[25] = 'S';
									Msg1[26] = ' ';
							 break;

				  default :
							 break;
		}

 		AddMessage(Msg1);
	}

}
/************************************************************************************/
/************************************************************************************/
void OBM_ClearBIE ( void )
{

	*PMCR &= PMCR_CLEARE_BIE_BIT;

}
/************************************************************************************/
/************************************************************************************/
static UINT_T OBM_ReadFromFlash ( UINT_T   DDR_A , UINT_T   ImageSize , UINT_T   Flash_SA , P_FlashProperties_T    pFlashP )
{

	UINT_T   	Retval = NoError;
#if MDOC_CODE
	UINT_T      sectorOffset,partition;
#endif
#if NOR_CODE
	UINT_T      *buffer = (UINT_T *)ARBEL_LOAD_ADDR_BASE , i;
#endif


	switch ( pFlashP->FlashType )
	{
#if NOR_CODE
	case TYAX_FLASH :
	case SIBLEY_FLASH :
						Flash_SA |= NOR_ADDR_BASE;
						for ( i = 0; i < NOR_READ_SIGNATURE_SIZE; i++ )
		                    buffer[i] = *(((UINT_T *)Flash_SA) + i);
		break;
#endif

#if NAND_CODE
	case NAND_FLASH :
		Retval = ReadNANDWithBBM (Flash_SA, ImageSize, (P_DFC_BUFFER) DDR_A, pFlashP);
		break;
#endif
#if ONENAND_CODE
	case OneNAND :
		Retval = Flex_ReadFlash(Flash_SA, ImageSize, DDR_A, pFlashP);
		break;
#endif
#if MDOC_CODE
	case MSYS_DOC_FLASH :
		partition    = (Flash_SA & MDOC_PARTITION_MASK) >> MDOC_PARTITION_SHIFT;
		sectorOffset = (Flash_SA & MDOC_SECTOR_MASK) >> MDOC_SECTOR_SHIFT;
		Retval = mDOC_GetImage( (unsigned char *)DDR_A, sectorOffset, ImageSize, partition );
		break;
#endif
	default:
		return GeneralError;;
		break;
	}

	return Retval;
}
/************************************************************************************/
/************************************************************************************/
UINT8_T	OBM_CheckSignature ( void )
{
	UINT8_T		signature_fail = FALSE , i;

	//signature checking
	for ( i = 0; ((i < sizeof(load_table_signature_str)) && (signature_fail != TRUE)); i++ )
	{
		if ( LoadTable->Signature[i] != load_table_signature_str[i] )
		{
			signature_fail = TRUE;
			AddMessage((UINT8_T*) (" *** Signature not found.\0"));
		}
	}

	return signature_fail;

}
/************************************************************************************/
/************************************************************************************/
void OBM_ReadImagesSize ( pTIM     pTIM_h , P_FlashProperties_T    pFlashP )
{

	UINT_T 		Flash_SA;			// Starting Address for flash
	UINT_T 		DDR_A;				// Address for ISRAM or DDR
	UINT_T		ImageSize; 			// Size of image to copy
	UINT_T		Retval;
	UINT8_T		sig_not_ok;
	pIMAGE_INFO imageInfo;


#if (NOFLASH_CODE)
	g_arbelReset.Arbel_ResetRoutine[1] = ADDRCONVERT_2COMM(0xbfa00000);		 // overwrite according to real start
#endif

	imageInfo = FindImageInTIM ( pTIM_h , ARBELID );
	if ( imageInfo == NULL )
		return;

	DDR_A = ARBEL_LOAD_ADDR_BASE;
	ImageSize = pFlashP->NAND_Prop.PageSize;
	Flash_SA = imageInfo->FlashEntryAddr;

#if (!NOFLASH_CODE)
	AddMessage((UINT8_T*) ("-Reading ARBEL first image block from flash....\0"));
#endif

 	Retval = OBM_ReadFromFlash ( DDR_A , ImageSize , Flash_SA , pFlashP );
	if ( Retval != NoError )
		return;

	sig_not_ok = OBM_CheckSignature();

	if ( sig_not_ok != TRUE )
	{                                 // Means the signature is ok
		AddMessage((UINT8_T*) ("-The size from image header was updated in the NTIM.\0"));
		imageInfo->ImageSize = ( LoadTable->imageEnd - LoadTable->imageBegin );
		imageInfo->LoadAddr = ADDRCONVERT_2APPS(LoadTable->imageBegin);
		g_arbelReset.Arbel_ResetRoutine[1] = ADDRCONVERT_2COMM(LoadTable->imageBegin);			// overwrite according to real start


		imageInfo = FindImageInTIM ( pTIM_h , IMEIID );
		if ( imageInfo == NULL )
			return;

		imageInfo->LoadAddr = ADDRCONVERT_2APPS(LoadTable->sharedFreeBegin) + RELAIBLE_DATA_OFFSET_IN_DDR;
	}

	return;

}
/************************************************************************************/
/************************************************************************************/
UINT_T OBM_CopyPv2Images( P_FlashProperties_T    pFlashP )
{
	UINT_T		Retval;


 	Retval = OBM_ReadFromFlash ( PV2_DDR_A , PV2_IMAGE_SIZE , PV2_FLASH_ADDR , pFlashP );

	return Retval;;

}
/***************************************************************************************/
/* Decode the IMEI infomation - user need to implement this according to his algorthim */
/***************************************************************************************/
static void	OBM_IMEI_DeCode ( UINT_T   *DDRaddr )
{
	UINT8_T 		i , Msg[] = "-IMEI part1: xxxxxxxx\0";
	const UINT8_T 	MarvellSignature[] = "Marvell"; 			          // defualt signature
	INT_T 			signature_not_valid = 0xFF;
	RELIABLE_DATA	*rd;
	UINT_T          *ReliableDataPtr;



	rd = (RELIABLE_DATA *)DDRaddr;

	if ( rd->header.validBufferStamp == VALID_BUF_STMP )                  // New reliable data format
	{
		if ( rd->entry.entryType == IMEI_TYPE )
		   signature_not_valid = 0x0;

		ReliableDataPtr = &rd->entry.pFileToWrite;
	}
	else                                                                  // Old reliable data format
	{
		/* Check signature - the signature is in 16 bytes offset*/
		signature_not_valid = memcmp( &MarvellSignature, ( (UINT8_T*)DDRaddr + 0x10) , (sizeof(MarvellSignature)-1));
		ReliableDataPtr = DDRaddr;
	}

	/* if signature is not valid set all IMEI to zero - IMEI area in the relabile area is 32 bytes */
	if ( signature_not_valid )
	{
	    memset ( ReliableDataPtr , 0 , 32 );
		AddMessage((UINT8_T*) ("******* Warning: IMEI signature not found *******\0"));
	}


	/* print the information after decoding*/
	/*Print the IMEI*/
	for ( i = 0; i < 2; i++ )
	{
		ConvertIntToBuf8 ( &Msg[13] , *(UINT_T*)ReliableDataPtr , 8 , 16 );
 		AddMessage(Msg);
		ReliableDataPtr++;
 		Msg[10] = 2 + '0';
	}

}
/************************************************************************************/
/************************************************************************************/
static void OBM_CopyDebugImagesToDDR ( P_FlashProperties_T    pFlashP )
{
	UINT_T   	Retval = NoError;
	UINT_T 		Flash_SA;			// Starting Address for flash
	UINT_T 		DDR_A;				// Address for ISRAM or DDR
	UINT_T		ImageSize; 			// Size of image to copy
	UINT8_T		i;
	UINT8_T  	DebugMode[] = "-Copy DEBUG image x to DDR....\0";
	UINT_T		DesAdd[DEBUG_NUMBER_OF_IMAGES] = { DEBUG_LOAD_ADDR_1_BASE , DEBUG_LOAD_ADDR_2_BASE };
	UINT_T		SrcAdd[DEBUG_NUMBER_OF_IMAGES] = { DEBUG_FLASH_ADDR_1_BASE , DEBUG_FLASH_ADDR_2_BASE };
	UINT_T		DebugImageSize[DEBUG_NUMBER_OF_IMAGES] = { 0 , 0  };
#if MDOC_CODE
	UINT_T      sectorOffset,partition;
#endif
#if NOR_CODE
	UINT_T   	j;
#endif



#if NOR_CODE
	DebugImageSize[0] = DEBUG_NOR_IMAGE_1_SIZE_IN_BYTES;
	DebugImageSize[1] = DEBUG_NOR_IMAGE_2_SIZE_IN_BYTES;
#else
	DebugImageSize[0] = (pFlashP->BlockSize * DEBUG_IMAGE_1_SIZE_IN_BLOCK);    // Calculate the size in bytes
	DebugImageSize[1] = (pFlashP->BlockSize * DEBUG_IMAGE_2_SIZE_IN_BLOCK);    // Calculate the size in bytes
#endif

	for ( i = 0; i < DEBUG_NUMBER_OF_IMAGES; i++ )
	{
		DDR_A = DesAdd[i];
		ImageSize = DebugImageSize[i];
		Flash_SA = SrcAdd[i];

		DebugMode[18] = ((i + 1) + '0');
 		AddMessage(DebugMode);

		switch ( pFlashP->FlashType )
		{
#if NOR_CODE
		case TYAX_FLASH :
			break;

		case SIBLEY_FLASH :
						Flash_SA |= NOR_ADDR_BASE;
						for ( j = 0; j < DebugImageSize[i]; j++ )
		                    *(((UINT_T *)DDR_A) + j) = *(((UINT_T *)Flash_SA) + j);
			break;
#endif

#if NAND_CODE
		case NAND_FLASH :
			Retval = ReadNANDWithBBM (Flash_SA, ImageSize, (P_DFC_BUFFER) DDR_A, pFlashP);
			break;
#endif
#if ONENAND_CODE
		case OneNAND :
			Retval = Flex_ReadFlash(Flash_SA, ImageSize, DDR_A, pFlashP);
			break;
#endif
#if MDOC_CODE
		case MSYS_DOC_FLASH :
			partition    = (Flash_SA & MDOC_PARTITION_MASK) >> MDOC_PARTITION_SHIFT;
			sectorOffset = (Flash_SA & MDOC_SECTOR_MASK) >> MDOC_SECTOR_SHIFT;
			Retval = mDOC_GetImage( (unsigned char *)DDR_A, sectorOffset, ImageSize, partition );
			break;
#endif
		default:
			return;
			break;

		}
	}
}
/************************************************************************************/
/************************************************************************************/
void OBM_ConfigNorCsGpio ( void )
{
	/* For NOR flash cs - for Tavor EVB2 only */
	*(unsigned long*)(0x40e10224) = 0x00001803; 	// setting ND_nADV2_ALE to NCS2
#if (PV_CHIP)
	*(unsigned long*)(0x40e00410) = 0x00010000; 	// setting GPIO_144 to output
	*(unsigned long*)(0x40e00128) = 0x00010000; 	// setting GPIO_144 to low
#else
	*(unsigned long*)(0x40e00404) = 0x00400000; 	// setting GPIO_54 to output
	*(unsigned long*)(0x40e00028) = 0x00400000; 	// setting GPIO_54 to low
#endif

}
/************************************************************************************/
/************************************************************************************/
void OBM_ConfigSmemForNor ( void )
{

	AddMessage((UINT8_T*) ("-Configure the Static Memory Controler\0"));

	/* The following config the Smem for NOR access. only for Tavor B0*/
	*(unsigned long *) 0x4a000008 = 0x72297229;
	*(unsigned long *) 0x4a00000c = 0x72297229;
	*(unsigned long *) 0x4a000014 = 0x00000002;
	*(unsigned long *) 0x4a00001c = 0x40044004;
	*(unsigned long *) 0x4a000028 = 0x00000100;
	*(unsigned long *) 0x4a000030 = 0x00000100;
	*(unsigned long *) 0x4a000038 = 0x00000100;
	*(unsigned long *) 0x4a000068 = 0x00020002;
	*(unsigned long *) 0x4a000080 = 0x00320919;
	*(unsigned long *) 0x4a000084 = 0x00320919;
	*(unsigned long *) 0x4a000088 = 0x00320919;

}
/************************************************************************************/
/************************************************************************************/
#define VCC_COMM_REG_ADDR   0x40f5009c

void OBM_ConfigVccComm ( void )
{
	volatile unsigned long     CVCR_reg;   // VCC COMM voltage control register


	AddMessage((UINT8_T*) ("-Setting COMM VCC to 1.4V..\0"));

	CVCR_reg = *(unsigned long *)(VCC_COMM_REG_ADDR);
//	CVCR_reg = (CVCR_reg & ~0x1F000000);
//	CVCR_reg = (CVCR_reg | 0x1C000000);  //0x1c = 1.425v  -- Level 3
	CVCR_reg = (CVCR_reg & ~0x001F0000);
	CVCR_reg = (CVCR_reg |  0x001C0000);  //0x1c = 1.4v   -- Level 2

	*(unsigned long *)(VCC_COMM_REG_ADDR) = CVCR_reg;

}
/************************************************************************************/
/************************************************************************************/
#define GPIO_TIME_DELAY	  		400
#define PULLUP_TIME_DELAY	  	1400
UINT8_T OBM_CheckKeypadStatus ( void )
{

	volatile unsigned int *GPLR0 = (volatile unsigned int *)0x40e00000;
	volatile unsigned int *GPDR0 = (volatile unsigned int *)0x40e0000c;
	volatile unsigned int *GPSR0 = (volatile unsigned int *)0x40e00018;
//	volatile unsigned int *GSDR0 = (volatile unsigned int *)0x40e00400;
//	volatile unsigned int *GCDR0 = (volatile unsigned int *)0x40e00420;
	volatile unsigned int *GPCR0 = (volatile unsigned int *)0x40e00024;
	volatile unsigned int *MFPR_gpio0 = (volatile unsigned int *)0x40e102e0;
	volatile unsigned int *MFPR_gpio2 = (volatile unsigned int *)0x40e102e8;
	volatile unsigned int *MFPR_gpio4 = (volatile unsigned int *)0x40e102e4;
	volatile unsigned int *MFPR_gpio6 = (volatile unsigned int *)0x40e102f8;
	unsigned int 		   mfpr_gpio0_val;
	unsigned int 		   GPDR0_val;
	UINT8_T 		 	   keypad_press = NO_KEYPAD_WAS_PRESS;


	// Save the value of GPDR0 and MFPR-GPIO0
	GPDR0_val = *GPDR0;
	mfpr_gpio0_val = *MFPR_gpio0;

	// Set direction of GPIO 1, 3 5, and 7 to be -- output
   *GPDR0 = 0xaa;

	// GPIO 0 - set to pull-down
   *MFPR_gpio0 &= ~(0xe000);
   *MFPR_gpio0 |= 0xa000;

	// GPIO 2 - set to pull-down
   *MFPR_gpio2 &= ~(0xe000);
   *MFPR_gpio2 |= 0xa000;

	// GPIO 4 - set to pull-down
   *MFPR_gpio4 &= ~(0xe000);
   *MFPR_gpio4 |= 0xa000;

	// GPIO 6 - set to pull-down
   *MFPR_gpio6 &= ~(0xe000);
   *MFPR_gpio6 |= 0xa000;
    Delay_us(PULLUP_TIME_DELAY);

#if ((EVB2) || (EVB2_PV))

   *GPCR0 = 0xaa;
   *GPSR0 = 0x02;                       // Set output of GPIO 1
    Delay_us(GPIO_TIME_DELAY);
	switch ( *GPLR0 & 0x15 )
	{
		case 0x01 :
					keypad_press = SW_DOWNLOAD_KEY_PRESS;
					break;

		case 0x04 :
					keypad_press = 5;   // keypad 5
					break;

		case 0x10 :
					keypad_press = 1;   // keypad 1
					break;
	}

	if ( keypad_press == NO_KEYPAD_WAS_PRESS )
	{
	   *GPCR0 = 0x02;
	   *GPSR0 = 0x08;                       // Set output of GPIO 3
    	Delay_us(GPIO_TIME_DELAY);
		switch ( *GPLR0 & 0x15 )
		{
			case 0x01 :
					    keypad_press = JTAG_REENABLE_KEY_PRESS;     // keypad '#'
						break;

			case 0x04 :
						keypad_press = 6;   // keypad 6
						break;

			case 0x10 :
						keypad_press = 2;   // keypad 2
						break;
		}

		if ( keypad_press == NO_KEYPAD_WAS_PRESS )
		{
		   *GPCR0 = 0x08;
		   *GPSR0 = 0x20;                       // Set output of GPIO 5
    		Delay_us(GPIO_TIME_DELAY);
			switch ( *GPLR0 & 0x15 )
			{
				case 0x01 :
							keypad_press = 9;   // keypad 9
							break;

				case 0x04 :
							keypad_press = 7;   // keypad 7
							break;

				case 0x10 :
							keypad_press = 3;   // keypad 3
							break;
			}

			if ( keypad_press == NO_KEYPAD_WAS_PRESS )
			{
			   *GPCR0 = 0x20;
			   *GPSR0 = 0x80;                       // Set output of GPIO 7
    			Delay_us(GPIO_TIME_DELAY);
				switch ( *GPLR0 & 0x15 )
				{
					case 0x01 :
								keypad_press = 0;   // keypad 0
								break;

					case 0x04 :
								keypad_press = 8;   // keypad 8
								break;

					case 0x10 :
								keypad_press = 4;   // keypad 4
								break;
				}
			}
		}
	}

	*GPCR0 = 0x80;

#endif    // EVB2

#if ((SAAR) || (SAAR_PV))

   *GPCR0 = 0xaa;
   *GPSR0 = 0x02;                       // Set output of GPIO 1
    Delay_us(GPIO_TIME_DELAY);
	switch ( *GPLR0 & 0x15 )
	{
		case 0x01 :
					keypad_press = 1;   // keypad 1
					break;

		case 0x04 :
					keypad_press = 2;   // keypad 2
					break;

		case 0x10 :
					keypad_press = 3;   // keypad 3
					break;
	}

	if ( keypad_press == NO_KEYPAD_WAS_PRESS )
	{
	   *GPCR0 = 0x02;
	   *GPSR0 = 0x08;                       // Set output of GPIO 3
    	Delay_us(GPIO_TIME_DELAY);
		switch ( *GPLR0 & 0x15 )
		{
			case 0x01 :
						keypad_press = 4;   // keypad 4
						break;

			case 0x04 :
						keypad_press = 5;   // keypad 5
						break;

			case 0x10 :
						keypad_press = 6;   // keypad 6
						break;
		}

		if ( keypad_press == NO_KEYPAD_WAS_PRESS )
		{
		   *GPCR0 = 0x08;
		   *GPSR0 = 0x20;                       // Set output of GPIO 5
    		Delay_us(GPIO_TIME_DELAY);
			switch ( *GPLR0 & 0x15 )
			{
				case 0x01 :
							keypad_press = 7;   // keypad 7
							break;

				case 0x04 :
							keypad_press = 8;   // keypad 8
							break;

				case 0x10 :
							keypad_press = 9;   // keypad 9
							break;
			}

			if ( keypad_press == NO_KEYPAD_WAS_PRESS )
			{
			   *GPCR0 = 0x20;
			   *GPSR0 = 0x80;                       // Set output of GPIO 7
    			Delay_us(GPIO_TIME_DELAY);
				switch ( *GPLR0 & 0x15 )
				{
					case 0x01 :                     // keypad *
								keypad_press = SW_DOWNLOAD_KEY_PRESS;     // keypad '*'
								break;

					case 0x04 :
								keypad_press = 0;   // keypad 0
								break;

					case 0x10 :                     // keypad #
					    		keypad_press = JTAG_REENABLE_KEY_PRESS;     // keypad '#'
								break;
				}
			}
		}
	}

	*GPCR0 = 0x80;

#endif    // SAAR SAAR_PV


	// restore reset values of registers
	*MFPR_gpio0 = mfpr_gpio0_val;
	*GPDR0 = GPDR0_val;

#if defined (MODE6)
	return OBM_DEBUG_MODE;
#else
	return keypad_press;
#endif

}
/************************************************************************************/
/************************************************************************************/

#endif   // TAVOR_BOERNE
#define READ_ADC_NUM	10
int get_adc_average(u8 mux)
{
	int i, sum = 0;

	micco_adc_read(mux);       /* Because the first value is too bigger than other value when restart. we give up it. */
	for (i = 0; i < READ_ADC_NUM; i++)
		sum += micco_adc_read(mux);
	return (sum / READ_ADC_NUM);
}


/* return the version of board(P2 or P2) */
unsigned short CheckBoardVersion(void)
{
    unsigned short val;

    val = get_adc_average(MICCO_ADC_MUX_VERSION);
    return  val;
}

extern unsigned int reboot_reason;
extern void poweron_reason_print( char* str, int data, int bytes);
/* The function is to check the battery voltage is if to need precharging. 
 * 1) If factory cable is attached just return.
 * 2) If voltage is > 3.5V, just power up whenever charger is attached.
 * 3) If voltage is 3.4V – 3.5V, if charger is attached, do pre-charge. If no charger, power up. 
 * 4) If voltage is <3.4V, if charger is attached, do pre-charge. If no charger, power down.
 * ps:when pre-charging, display a message on lcd.
 */
void PrechargCheck(void)
{
    unsigned short data;
    int i = 0, sum = 0, count = 0;
    u16 adc = 0;
    int mode;
    LCD_RECT rect;

	AddMessage((UINT8_T*)("Check battery voltage\0"));

/* for Debug, get the micco value of voltage */
#if 0
    while (1)
    {
        data = get_adc_average(MICCO_ADC_MUX_VBAT);

        //poweron_reason_print("battery voltage =", data, 4);
        Delay( 1000000 );
    }
#endif

/* get a average value of voltage */
    for (i = 0; i < 5; i++)
    {
        sum += micco_adc_read(MICCO_ADC_MUX_VBAT);
    }
    data = sum / 5;

#if 0
    if (data == 0x0)
    {
        AddMessage("there is no battery");
        return;
    }
#endif

    
/* if factory cable or 3.4 < voltage < 3.5(and no usb connect) or voltage > 3.5 just return. */
    if ( (powerup_reason == PU_REASON_USB_FACTORY) || (data > (unsigned short)VOLTAGE_3_5) 
        || ( (((unsigned short)VOLTAGE_3_4 <= data) && (data <= (unsigned short)VOLTAGE_3_5)) 
        && ((powerup_reason != PU_REASON_USB_CHARGER) && (reboot_reason != REBOOT_DKB) && (powerup_reason != PU_REASON_USB_CABLE)))
        )
    {
#ifdef SAAR_PV
        stop_charging();
#endif
        poweron_reason_print( "value of voltage = ", data, 2 );
	    AddMessage((UINT8_T*)("not enough condition of pre-charging to pre-charging\0"));
        return;
    }
/* if voltage < 3.5V(usb connect), pre-charging, and display message */
    else if ( ( (((unsigned short)VOLTAGE_3_4 <= data) && (data <= (unsigned short)VOLTAGE_3_5)) 
        && ((powerup_reason == PU_REASON_USB_CHARGER) || (reboot_reason == REBOOT_DKB) || (powerup_reason == PU_REASON_USB_CABLE)))
        || ( (data <= (unsigned short)VOLTAGE_3_4) 
        && ((powerup_reason == PU_REASON_USB_CHARGER) || (reboot_reason == REBOOT_DKB) || (powerup_reason == PU_REASON_USB_CABLE)) ) )
    {
        poweron_reason_print( "value of voltage = ", data, 2 );
#ifdef SAAR_PV
        start_charging();
        if (data < VOLTAGE_2_9)
        {
            change_charging_mode(LOW_CHARGE_MODE);
            mode = LOW_CHARGE_MODE;
        }
        else if (data < VOLTAGE_3_2)
        {
            change_charging_mode(MID_CHARGE_MODE);
            mode = MID_CHARGE_MODE;
        }
        else if (data >= VOLTAGE_3_2)
        {
            change_charging_mode(HIGH_CHARGE_MODE);
            mode = HIGH_CHARGE_MODE;
        }
#endif
#if defined(OBM_LCD_SUPPORT)
	    AddMessage((UINT8_T*)("enough condition of pre-charging, doing pre-charging now\0"));
        pxa_lcd_enable();
#ifdef SAAR_PV
        {
            rect.x0 = 0;
            rect.y1 = 360 - 1;
            rect.y0 = 0;
            rect.x1 = 640-1;
            lcd_fill_rect(&rect,FONT_COLOR_BLACK);

            rect.x0 = 50;
            rect.y1 = 360-50;
            rect.y0 = 50;
            rect.x1 = 700-100;
            lcd_draw_rect(&rect,FONT_COLOR_GREEN);

            rect.x0 = 50+1;
            rect.y1 = 360-50-1;
            rect.y0 = 50+1;
            rect.x1 = 700-100-1;
            lcd_draw_rect(&rect,FONT_COLOR_BLUE);


            rect.x0 += 2;
            rect.y1 -= 1;
            rect.y0 += 2;
            rect.x1 -= 2;
            lcd_fill_rect(&rect,FONT_COLOR_GREEN);

            lcd_printf("Pre-charging...",140,100,3,FONT_COLOR_RED,FONT_COLOR_GREEN,LCD_DRAWMODE_NORMAL);
        }
#else
        {
            rect.x0 = 0;
            rect.x1 = 320-1;
            rect.y0 = 0;
            rect.y1 = 480-1;
            lcd_fill_rect(&rect,FONT_COLOR_BLACK);

            rect.x0 = 50;
            rect.x1 = 320-50;
            rect.y0 = 50;
            rect.y1 = 480-100;
            lcd_draw_rect(&rect,FONT_COLOR_GREEN);
            rect.x0 = 50+1;
            rect.x1 = 320-50-1;
            rect.y0 = 50+1;
            rect.y1 = 480-100-1;
            lcd_draw_rect(&rect,FONT_COLOR_BLUE);


            rect.x0 += 2;
            rect.x1 -= 1;
            rect.y0 += 2;
            rect.y1 -= 2;
            lcd_fill_rect(&rect,FONT_COLOR_GREEN);

            lcd_printf("Battery Low",rect.x0+8,200,2,FONT_COLOR_RED,FONT_COLOR_GREEN,LCD_DRAWMODE_NORMAL);
            lcd_printf("Pre",rect.x0+7,225,2,FONT_COLOR_RED,FONT_COLOR_GREEN,LCD_DRAWMODE_NORMAL);
            lcd_printf("-charging..",rect.x0+7,250,2,FONT_COLOR_RED,FONT_COLOR_GREEN,LCD_DRAWMODE_NORMAL);
        }
#endif
#endif
    }
/* if voltage < 3.4(no usb), do powerdown */
    else if (( (data < (unsigned short)VOLTAGE_3_4) 
        && ((powerup_reason != PU_REASON_USB_CHARGER) && (reboot_reason != REBOOT_DKB) && (powerup_reason != PU_REASON_USB_CABLE))))
    {
        poweron_reason_print( "value of voltage = ", data, 2 );
        AddMessage((UINT8_T*)("voltage less than 3.4V, and have no usb, do powerdown\0"));
      
        do_powerdown();    
    }
/* the circle will stop when the voltage more than 3.5V */
    AddMessage((UINT8_T*)("3.4V = 0x135, 3.5V = 0x150.\0"));
    do
    {
        Delay( 500000 );
        data = micco_adc_read(MICCO_ADC_MUX_VBAT);
        poweron_reason_print( "value of voltage = ", data, 2 );

#ifdef SAAR_PV
        if ((data > CHARGE_VOLTAGE_2_9) && (mode == LOW_CHARGE_MODE))
        {
            AddMessage("pre-chargind mode change to mid_charge_mode\n");
            change_charging_mode(MID_CHARGE_MODE);
            mode = MID_CHARGE_MODE;
        }
        else if ((data > CHARGE_VOLTAGE_3_2) && (mode == MID_CHARGE_MODE))
        {
            AddMessage("pre-chargind mode change to high_charge_mode\n");
            change_charging_mode(HIGH_CHARGE_MODE);
            mode = HIGH_CHARGE_MODE;
        }
#endif
    }while(data < 0x150);

#ifdef SAAR_PV
    if (data > 0x200)
    {
        lcd_printf("No battery...",140,100,3,FONT_COLOR_RED,FONT_COLOR_GREEN,LCD_DRAWMODE_NORMAL);
        AddMessage("There is no battery, please insert batter and reboot the phone.");
        while (1);
    }
    else
        stop_charging();
#endif
#if defined(OBM_LCD_SUPPORT)
    pxa_lcd_off();          /*turn off the lcd, because after return, will call enable_lcd another time. if lcd enable twice, it will be take mistake. so we turn off it here. */
#endif

    return;
}


#define FOTA_ddr_addr  OBM_DDR_DOWNLOAD_ADDR
void fota_handle(P_FlashProperties_T pFlashProp)
{
    pxa_lcd_ini(___video_flash_mod);
    pxa_lcd_enable();
#if SAAR_PV
    //Flex_ReadFlash(0x0F400000, 0xDC00, (P_DFC_BUFFER)FOTA_ddr_addr, pFlashProp);
    //Flex_ReadFlash(0x0F400000, 0xF000, (P_DFC_BUFFER)FOTA_ddr_addr, pFlashProp);
    Flex_ReadFlash(0x0F900000, 0x40000, (P_DFC_BUFFER)FOTA_ddr_addr, pFlashProp);
#else
    //ReadNANDWithBBM(0x0F400000, 0xDC00, (P_DFC_BUFFER)FOTA_ddr_addr, pFlashProp);   
    //ReadNANDWithBBM(0x0F400000, 0xF000, (P_DFC_BUFFER)FOTA_ddr_addr, pFlashProp);   
    ReadNANDWithBBM(0x0F900000, 0x40000, (P_DFC_BUFFER)FOTA_ddr_addr, pFlashProp);   
#endif
    AddMessage("Load FOTA success\0");
 	//TransferControl(0x80000000, 0xDC00, FOTA_ddr_addr);
 	//TransferControl(0x80000000, 0xF000, FOTA_ddr_addr);
 	TransferControl(0x80000000, 0x40000, FOTA_ddr_addr);
 	//TransferControl(0x5c013000, 0xDC00, FOTA_ddr_addr);
}
#ifdef SAAR_PV
static void start_charging(void)
{
	u8 val;

    gpio_direction_output(MFP_PIN_GPIO174, 0);
	//micco_read(MICCO_CHARGE_CONTROL, &val);
	//val |= 0x80;
    val = 0x80;
	micco_write(MICCO_CHARGE_CONTROL, val);
	return;
}


static void stop_charging(void)
{
	u8 val;

    gpio_direction_output(MFP_PIN_GPIO174, 1);
	micco_read(MICCO_CHARGE_CONTROL, &val);
	val &= ~0x80;
	micco_write(MICCO_CHARGE_CONTROL, val);
	return;
}
#endif

static void change_charging_mode(CHARGE_MODE mode)
{
    switch (mode)
    {
        case LOW_CHARGE_MODE:    
	             micco_write(MICCO_CHARGE_CONTROL, 0x80);
                 break;
        case MID_CHARGE_MODE:    
	             micco_write(MICCO_CHARGE_CONTROL, 0x88);
                 break;
        case HIGH_CHARGE_MODE:    
	             micco_write(MICCO_CHARGE_CONTROL, 0x90);
                 break;
    }
    
}
