/**************************************************************************
**
** (C) Copyright December 2006 Marvell International Ltd.
**
** All Rights Reserved.
**
**
** THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE   
** IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR  
** PURPOSE ARE EXPRESSLY DISCLAIMED.   
**
**  FILENAME:	TIM.h
**
**  PURPOSE: 	Defines the Trusted/Non-Trusted Image Module
**
**                  
******************************************************************************/

#ifndef __TIM_H__
#define __TIM_H__


/******************* TIM Layout ************************************/
/* Below Presents the Maxiumum Size TIM layout in memory

VERSION_I      	VersionBind;         			// 0                                             
FLASH_I        	FlashInfo;           			// 0x10                                                 
UINT_T         	NumImages;           			// 0x20                
UINT_T         	NumKeys;						// 0x24
UINT_T         	SizeOfReserved;					// 0x28
IMAGE_INFO 		IMG[MAX_IMAGES];                // 0x2C                    
KEY_MOD        	Key[MAX_KEYS];                  // 0x284
UINT_T         	Reserved[0x4E8];       			// 0x908  (0x4E8 bytes)                                     
PLAT_DS        	TBTIM_DS;                       // 0xDF0                                  
												// 0x1000 = 4 KB
Below Presents the Maxiumum Size NTIM layout in memory

VERSION_I   	VersionBind;         			// 0                                             
FLASH_I     	FlashInfo;           			// 0x10                                                 
UINT_T      	NumImages;           			// 0x20                
UINT_T         	NumKeys;						// 0x24
UINT_T      	SizeOfReserved;					// 0x28
IMAGE_INFO 		IMG[MAX_IMAGES];                // 0x2C                      
UINT_T      	Reserved[0xD80];       			// 0x284                                       
NTIM, *pNTIM;									// 0x1000 = 4KB
/******************* TIM Layout ************************************/

// B1 TIM
#include "typedef.h"
#define MAX_IMAGES			10
#define MAX_KEYS	 		3
#define MAX_TIM_SIZE 		0x1000
#define TIMBUFFER 			4096    		//4k for a Tim structure size
#define MAXKEYSIZEWORDS		64				// 2048 bits
#define TIMIDENTIFIER		0x54494D48		// "TIMH"
#define OBMIDENTIFIER		0x4F424D49		// "OBMI"
#define DKBIDENTIFIER		0x444B4249		// "DKBI"
#define JTAGIDENTIFIER		0x4A544147		// "JTAG"
#define PATCHIDENTIFIER		0x50415443		// "PATC"
#define TCAIDENTIFIER		0x5443414B		// "TCAK"
#define OSLOADERID			0x4F534C4F		// "OSLO"

//Option Identifiers
#define OPTIONALHEADER		0x4F505448      // "OPTH"
#define FFIDENTIFIER		0x00004646		// "FF"
#define ALTIDENTIFIER		0x00414C54		// "ALT"
#define DIFFIDENTIFIER		0x44696666		// "Diff"
#define SEIDENTIFIER		0x00005345		// "SE"
#define UARTID				0x55415254		// "UART"
#define USBID				0x00555342		// "USB"
#define PINSIDENTIFIER		0x50696E73		// "Pins"
#define TERMINATOR			0x5465726D		// "Term"

#define BINDKEYSIZE			128				// (MAXKEYSIZEWORDS * 2) 
#define FLASHSIGMASK		0x0000000F		// mask off the 4 bits of platform state

typedef enum
{
	NetworkOperatorCAKey, 
	SoftwareUpdateCAKey, 
	DRMDeviceCAKey, 
	OEMReservedKey1, 
	OEMReservedKey2, 
	NUMKEYMODULES
} 
KEYMODULES_T;

typedef enum
{
	Intel_DS, 
	PKCS1_v1_5, 
	PKCS1_v2_1
} 
DSALGORITHMID_T;

typedef struct
{
 volatile int *Addr;
 UINT_T Value;
}GPIO_DEF, *pGPIO_DEF;

typedef struct
{
 UINT_T Identifier;
 UINT_T PortType;
 UINT_T Port;
 UINT_T GPIOPresent;
 UINT_T NumGpios;
 pGPIO_DEF	GPIO;
}OPT_SET, *pOPT_SET;

typedef struct 
{
 UINT_T Version;
 UINT_T	Identifier;					// "TIMH" 
 UINT_T Trusted;					// 1- Trusted, 0 Non                          
 UINT_T IssueDate;                       
 UINT_T OEMUniqueID;                     
} VERSION_I, *pVERSION_I;			// 0x10 bytes		

typedef struct 
{
 UINT_T WTMFlashSign;                    
 UINT_T WTMEntryAddr;                    
 UINT_T WTMEntryAddrBack;
 UINT_T WTMPatchSign;
 UINT_T WTMPatchAddr;
 UINT_T BootFlashSign;                   
} FLASH_I, *pFLASH_I;				// 0x10 bytes

typedef struct 
{
 UINT_T	KeyID;						// Associate an ID with this key
 UINT_T HashAlgorithmID;            // 160 - SHA1, 256 - SHA2
 UINT_T ModulusSize;				// 1024 bit or 2048 bit keys
 UINT_T PublicKeySize;		
 UINT_T RSAPublicExponent[MAXKEYSIZEWORDS];		// Contents depend on PublicKeySize	
 UINT_T RSAModulus[MAXKEYSIZEWORDS];// Up to 2K bits	
 UINT_T KeyHash[8]; 				// Reserve 256 bits for the hash
} KEY_MOD, *pKEY_MOD;				// 0x22C bytes	

typedef struct
{
 UINT_T ImageID;					// Indicate which Image
 UINT_T NextImageID;				// Indicate next image in the chain
 UINT_T FlashEntryAddr;			 	// Block numbers for NAND
 UINT_T LoadAddr;
 UINT_T ImageSize;
 UINT_T ImageSizeToHash;
 UINT_T HashAlgorithmID;            // 160 - SHA1, 256 - SHA2
 UINT_T Hash[8];					// Reserve 256 bits for the hash
} IMAGE_INFO, *pIMAGE_INFO;			// 0x3C bytes

typedef struct
{
 UINT_T DSAlgorithmID; 				// See DSALGORITHMID_T
 UINT_T HashAlgorithmID;            // 160 - SHA1, 256 - SHA2
 UINT_T ModulusSize;               	// 1024 bit or 2048 bit keys
 UINT_T Hash[8];					// Reserve 256 bits for the hash
 UINT_T RSAPublicExponent[MAXKEYSIZEWORDS];        
 UINT_T RSAModulus[MAXKEYSIZEWORDS];           	// Up to 2K bits	
 UINT_T RSADigS[MAXKEYSIZEWORDS];				// Contains TIM Hash   
} PLAT_DS, *pPLAT_DS;				// 0x780 bytes

// Constant part of the TIMs
typedef struct
{
 VERSION_I      VersionBind;         			// 0                                             
 FLASH_I        FlashInfo;           			// 0x10                                                 
 UINT_T         NumImages;           			// 0x20                
 UINT_T         NumKeys;						// 0x24
 UINT_T         SizeOfReserved;					// 0x28
} CTIM, *pCTIM;									// 0x2C

// TIM structure for use by DKB/OBM/BootROM
typedef struct
{
 pCTIM 			pConsTIM;			// Constant part
 pIMAGE_INFO	pImg;				// Pointer to Images
 pKEY_MOD		pKey;				// Pointer to Keys
 PUINT			pReserved;			// Pointer to Reserved Area
 pPLAT_DS       pTBTIM_DS;			// Pointer to Digital Signature
} TIM, *pTIM;

// NTIM structure for use by DKB/OBM/BootROM
typedef struct
{
 pCTIM 			pConsTIM;			// Constant part
 pIMAGE_INFO	pImg;				// Pointer to Images
 PUINT			pReserved;			// Pointer to Reserved Area
} NTIM, *pNTIM;
                      
#endif