/**************************************************************************
**
** (C) Copyright December 2006 Marvell International Ltd.
**
** All Rights Reserved.
**
** This software file (the "File") is distributed by Marvell International  
** Ltd. under the terms of the GNU General Public License Version 2, June  
** 1991 (the "License").  You may use, redistribute and/or modify this File   
** in accordance with the terms and conditions of the License, a copy of  
** which is available along with the File in the license.txt file or by   
** writing to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   
** Boston, MA 02111-1307 or on the worldwide web at 
** http://www.gnu.org/licenses/gpl.txt.
**
** THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE   
** IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR  
** PURPOSE ARE EXPRESSLY DISCLAIMED.  The License provides additional
** details about this warranty disclaimer. 
**
**  FILENAME:	WTPUSBLIB.c
**
**  PURPOSE: 	Windows Dll routines 
**                  
******************************************************************************/

#pragma warning ( disable : 4244 4715 4047 4024) // Disable warning messages


#include <time.h>
#include <stdio.h>
#include <math.h>
#include "../Include/Typedef.h"
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/signal.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/errno.h>


#include "WTPUSBLIB.h"

#define debug_response_CMD  0

int usb_fd = 0;
char * DeviceName = "/dev/ttyUSB0";

//#define USBDEBUG

#ifdef USBDEBUG
int	dbg_level = 1;
#define usb_dbg(args...)	\
	do {			\
		printf(args);	\	
	}while(0)
#else
int	dbg_level = 0;
#define usb_dbg(args...)	\
	do {			\
	}while(0)
#endif

void dbg_usbStatus(WTPSTATUS *WtpStatus)
{
#ifdef USBDEBUG
    int i;

    if (WtpStatus->DLEN > 0)
    {
	for (i=0; i< WtpStatus->DLEN; i++)
	    printf("data 0x%2x-----", WtpStatus->Data[i]);
    }
    printf ("\n");
    printf ("CMD 0x%2x-----", WtpStatus->CMD);
    printf ("SEQ 0x%2x-----", WtpStatus->SEQ);
    printf ("CID 0x%2x-----", WtpStatus->CID);
    printf ("Status 0x%2x----- ", WtpStatus->Status);
    printf ("Flags 0x%2x-----", WtpStatus->Flags);
    printf ("DLEN 0x%2x-----", WtpStatus->DLEN);
    printf ("\n");
#endif
}

BOOL OpenUsbPort (long int DelayTime)
{
	char Message[100];
	unsigned long int Seconds, count=100;
	BOOL DeviceFound = FALSE,MessageEnable = TRUE;
	struct termios options, old_options;
	int file;

	file = open(DeviceName, O_RDWR | O_NOCTTY | O_NDELAY);

	if (file < 0) 
	{
		printf("ERROR: Open driver failed:%d \n", errno);
		return FALSE;
	}

	fcntl(file, F_SETFL, 0);
	tcgetattr(file, &options); /* Fetch & saver attributes */
	memcpy( &old_options, &options, sizeof(options) );
	cfsetispeed(&options, B115200);
	cfsetospeed(&options, B115200);
	options.c_cflag     |= (CLOCAL|CREAD); /* Ignore modem control lines and enable receiver */
	options.c_lflag     &= ~(ICANON|ECHO|ECHOE|ISIG);
	options.c_oflag     &= ~(OPOST); /* Enable implementation-defined output processing */
	options.c_cc[VMIN]   = 0;
	options.c_cc[VTIME]  = 10; /* Time out in deciseconds for non-canonical read */
	options.c_cflag &= ~CRTSCTS;
	tcsetattr(file, TCSANOW, &options); /* Apply changes now */

	usb_fd = file;
	usb_dbg("%s end, usb_fd %d\n", __func__, usb_fd);
	return TRUE;
}

BOOL WriteUsb (WTPCOMMAND WtpCmd,DWORD dwCmdSize)
{
	char Message[100];
	int result;
	DWORD IOCTL_CMD, dwActualBytes = 0;
	BOOL IsOk;

	if (usb_fd == 0) 
	{
		sprintf (Message,"  Error: OpenUsbPort must be called prior to calling WriteUsb!");
		usb_dbg("%s %d USB COMMUNICATIONS ERROR\n", __func__, __LINE__);
		return FALSE;
	}

	result = write( usb_fd, &WtpCmd, dwCmdSize);
	if (result < 0)
	{
	    sprintf (Message,"  Error: WriteUsb failed with error %d", result);
	    usb_dbg("%s %d USB COMMUNICATIONS ERROR\n", __func__, __LINE__);
	}

	if (dbg_level)
	{
		sprintf (Message,"IOCTL code: %X",IOCTL_CMD);
		usb_dbg("%s %d USB write %d bytes\n", __func__, __LINE__, result);
	}

	return TRUE;
}

BOOL ReadUsb(WTPSTATUS *WtpStatus, DWORD dwSize, PDWORD pBytesRead)
{
	int result;
	
	usb_dbg("%s %d USB read %d bytes\n", __func__, __LINE__, dwSize);

	result = read(usb_fd, WtpStatus, dwSize);
	if (result <=  0)
	{
	    printf("%s return %d\n", __func__, result);
	    return FALSE;
	}
	
	dbg_usbStatus(WtpStatus);
	    
	*pBytesRead = result;
	return TRUE;
		
}

BOOL ReadUsbCharBuffer (char *WtpMessage,DWORD dwSize,PDWORD pBytesRead)
{
	int result;

	if (usb_fd < 0) 
	{
		printf ("  Error: OpenUsbPort must be called prior to calling ReadUsb!");
		return FALSE;
	}
	result = read(usb_fd, WtpMessage, dwSize);
	if (result < 0 )
	{
		printf ("  Error: ReadFile failed with error\n");
		return FALSE;
	}
	
	*pBytesRead = result;
	if (*pBytesRead != dwSize) return FALSE;
	
	return TRUE;
}

void CloseUsbPort ()
{
	if (usb_fd != 0) close (usb_fd);
}
